#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# acme.tcl: auxillary modules for ACME modeling of COSY multiplets.
#           (Amplitude-Constrained Multiplet Evaluation)

proc ACME args \
{
   global objInfo stdfont

   set obj acme

   if {[flagLoc $args -help]} \
      {
       showACMEArgHelp
       return
      }

   if {[flagLoc $args -quit]} \
      {
       return
      }

   if {![info exists stdfont]} {setFonts}

   set parent [lindex $args 0]
   set sx1    [lindex $args 1]
   set sxn    [lindex $args 2]
   set sy1    [lindex $args 3]
   set syn    [lindex $args 4]

   showVerb "ACME: $parent"

   resetACMESignals

   set objInfo($obj,specX1)  $sx1
   set objInfo($obj,specXN)  $sxn
   set objInfo($obj,specY1)  $sy1
   set objInfo($obj,specYN)  $syn

   if {![info exists objInfo($obj,initFlag)]} \
      {
       initACMEVars
       initACMEArgs

       initACMESpecWindow $parent
       initACMEParmWindow $parent
      }

   unhiliteACMEView acme_$objInfo($obj,regID)_propose

   incr objInfo($obj,regID)

   hiliteACMEView   acme_$objInfo($obj,regID)_propose $objInfo(color,propose)

   readView $obj $objInfo($obj,roiID) 

   set  objInfo($obj,rms)     None
   set  objInfo($obj,sum)     None
   set  objInfo($obj,absSum)  None
   set  objInfo($obj,avgAmp)  None
   set  objInfo($obj,fitFlag) 0

   set w .acmeP

   wm title $w "ACME Region: $objInfo($obj,regID) RMS/N: $objInfo($obj,rms)"

   writeACMEView
   readACMEView
   showACMEView
   showACMECount
}

proc initACMEVars {} \
{
   global objInfo

   set obj acme

   set objInfo($obj,initFlag)     1
   set objInfo($obj,maxSig)       5
   set objInfo($obj,maxPassive)   3
   set objInfo($obj,sigCount)     0 
   set objInfo($obj,maxIter)      75 

   set objInfo($obj,resizeProc) resizeACMEView
   set objInfo($obj,redrawProc) showACMEView
   set objInfo($obj,tabName)    acme.tab
   set objInfo($obj,specName)   acme.exp.dat
   set objInfo($obj,simName)    acme.sim.dat
   set objInfo($obj,difName)    acme.dif.dat
   set objInfo($obj,acmeDir)    acmeDir

   set objInfo($obj,origName)   None
   set objInfo($obj,noise)      0.0
   set objInfo($obj,jHi)        100.0
   set objInfo($obj,N)          0
   set objInfo($obj,rms)        None
   set objInfo($obj,sum)        None
   set objInfo($obj,sumAbs)     None
   set objInfo($obj,avgAmp)     None
   set objInfo($obj,regID)      1

   set objInfo($obj,xDelay)     0
   set objInfo($obj,yDelay)     0 

   set objInfo($obj,xP0)        -90
   set objInfo($obj,xP1)        0
   set objInfo($obj,yP0)        -90
   set objInfo($obj,yP1)        0
   set objInfo($obj,diagFlag)   0
   set objInfo($obj,xActive)    1
   set objInfo($obj,yActive)    1

   set objInfo($obj,roiID)      [getNextID]
   set objInfo($obj,simID)      [getNextID]
   set objInfo($obj,difID)      [getNextID]

   set objInfo($obj,cellExp)    $objInfo($obj,roiID) 
   set objInfo($obj,cellSim)    $objInfo($obj,simID) 
   set objInfo($obj,cellDif)    $objInfo($obj,difID) 

   set objInfo($obj,idList) \
    "$objInfo($obj,roiID) $objInfo($obj,simID) $objInfo($obj,difID)"

   set objInfo($obj,defXSize)   400
   set objInfo($obj,defYSize)   850 

   set objInfo($obj,noteWidth)  420 
   set objInfo($obj,noteHeight) 190 
}

proc showACMEArgHelp {} \
{
   puts "ACME (Amplitude-Constrained Multiplet Evaluation) Arguments:"
   puts " -hi       First Contour Height for Display."
   puts " -noise    Estimated RMS of Spectral Noise."
   puts " -amp      Initial Amplitude of Model Signals."
   puts " -xDelay   X-Axis Acquisition Delay in X-Axis Signal, msec."
   puts " -yDelay   Y-Axis Acquisition Delay in X-Axis Signal, msec."
   puts " -xP0      X-Axis Zero-Order Phase, degrees."
   puts " -xP1      X-Axis First-Order Phase, degrees."
   puts " -yP0      Y-Axis Zero-Order Phase, degrees."
   puts " -yP1      Y-Axis First-Order Phase, degrees."
   puts " -parent   Parent Spectrum, No Extraction."
   puts " -acmeDir  Directory for Saving Result Tables." 
   puts " -id       First ID for Saving Result Tables." 
   puts " -iter     Max Number of Iterations for Fit."
   puts " -xPassive X-Axis has Passive-Only Signals."
   puts " -yPassive Y-Axis has Passive-Only Signals."
   puts " -diag     Operate in Diagonal Fitting Mode."
}

proc initACMEArgs {} \
{
   global objInfo ARGC ARGV

   set obj acme

   if {[flagLoc $ARGV -diag]} \
      {
       set objInfo($obj,xP0)      0.0
       set objInfo($obj,yP0)      0.0
       set objInfo($obj,diagFlag) 1
       set objInfo($obj,xActive)  0
       set objInfo($obj,yActive)  0
       set objInfo($obj,acmeDir)  acmeDiagDir
      }

   if {[flagLoc $ARGV -xPassive]} {set objInfo($obj,xActive) 0}
   if {[flagLoc $ARGV -yPassive]} {set objInfo($obj,yActive) 0}

   getArgD $ARGV -xDelay  objInfo($obj,xDelay)
   getArgD $ARGV -yDelay  objInfo($obj,yDelay)

   getArgD $ARGV -xP0     objInfo($obj,xP0)
   getArgD $ARGV -xP1     objInfo($obj,xP1)
   getArgD $ARGV -yP0     objInfo($obj,yP0)
   getArgD $ARGV -yP1     objInfo($obj,yP1)

   getArgD $ARGV -parent  objInfo($obj,origName)
   getArgD $ARGV -acmeDir objInfo($obj,acmeDir)
   getArgD $ARGV -noise   objInfo($obj,noise)
   getArgD $ARGV -iter    objInfo($obj,maxIter)
   getArgD $ARGV -id      objInfo($obj,regID)
   getArgD $ARGV -amp     objInfo($obj,jHi)

   incr objInfo($obj,regID) -1

   if {![file isdirectory $objInfo($obj,acmeDir)]} \
      {
       set error [catch {file mkdir $objInfo($obj,acmeDir)}]

       if {$error} \
          {
           showError ACME \
            "Error creating output directory $objInfo($obj,acmeDir)"
          }
      }
}

proc restoreACMEView {} \
{
   global objInfo

   set obj acme

   set types \
      {
       { {Table}      {.tab} }
       { {PDB}        {.pdb} }
       { {dat}        {.dat .DAT} }
       { {seq}        {.seq} }
       { {Text}       {.txt} }
       { {TCL}        {.tcl} }
       { {C-Shell}    {.com .csh} }
       { {K-Shell}    {.ksh} }
       { {Shell}      {.sh}  }
       { {All Files}  *      }
      }

   set inName [tk_getOpenFile -initialdir $objInfo($obj,acmeDir) \
                              -title "ACME Input" -filetypes $types]

   if {"$inName" == ""} return

   readACMEFit $inName
}

proc unhiliteACMEView { id } \
{
   delLabelItem $id
}

proc hiliteACMEView { id color } \
{
   global objInfo

   set obj acme

   set sx1 $objInfo($obj,specX1)
   set sxn $objInfo($obj,specXN)
   set sy1 $objInfo($obj,specY1)
   set syn $objInfo($obj,specYN)

   addLabelItem "box $id $sx1 $sxn $sy1 $syn 1 $color"
}

proc acceptACMEView {} \
{
   global objInfo

   set obj acme

   hiliteACMEView acme_$objInfo($obj,regID)_accept $objInfo(color,accept)
   saveACMEFit
}

proc saveACMEFit {} \
{
   global objInfo jInfo

   set obj acme

   set outName [format $objInfo($obj,acmeDir)/acme%04d.tab $objInfo($obj,regID)]
   set error   [catch {set outUnit [open $outName w]}]

   if {$error} \
      {
       showError ACME "Error opening output table $outName"
       return 1
      }

   puts $outUnit "DATA REGION   $objInfo($obj,regID)"
   puts $outUnit "DATA SIGNALS  $objInfo($obj,sigCount)"
   puts $outUnit ""

   for {set i 1} {$i <= $objInfo($obj,sigCount)} {incr i} \
       {
        puts $outUnit "DATA ASS $i $objInfo($obj,ass,$i)"
       }

   set fmt "DATA %s_AXIS % 8.3fppm % 8.3fppm"
 
   puts $outUnit ""
   puts $outUnit [format $fmt X $objInfo($obj,specX1) $objInfo($obj,specXN)]
   puts $outUnit [format $fmt Y $objInfo($obj,specY1) $objInfo($obj,specYN)]
   puts $outUnit ""
   puts $outUnit "DATA PARENT $objInfo($obj,parent)"
   puts $outUnit "DATA NOISE  $objInfo($obj,noise)"
   puts $outUnit "DATA RMS    $objInfo($obj,rms)"
   puts $outUnit "DATA N      $objInfo($obj,N)"
   puts $outUnit "DATA SUM    $objInfo($obj,sum)"
   puts $outUnit "DATA SUMABS $objInfo($obj,sumAbs)"
   puts $outUnit "DATA AVGAMP $objInfo($obj,avgAmp)"
   puts $outUnit ""

   puts $outUnit "VARS   SIGNAL VARNAME ADJUSTABLE MULT VALUE"
   puts $outUnit "FORMAT %2d    %10s    %d         %.1f %f"
   puts $outUnit ""
 
   for {set i 1} {$i <= $objInfo($obj,sigCount)} {incr i} \
       {
        foreach p $jInfo \
           {
            set pName [lindex $p 1]
            set fmt "%2d %-12s %d %.1f % 10.3f"

            set v1 $objInfo($obj,$pName,$i,active)
            set v2 $objInfo($obj,$pName,$i,mult)
            set v3 $objInfo($obj,$pName,$i)

            puts $outUnit [format $fmt $i $pName $v1 $v2 $v3]
          }
      }

   close $outUnit
}

proc resetACMEView {} \
{
   resetACMESignals
   showACMECount
   showACMEView
}

proc simACMEView {} \
{
   global objInfo

   set obj acme

   execACMEScript 0
}

proc fitACMEView {} \
{
   global objInfo

   set obj acme

   execACMEScript $objInfo($obj,maxIter)
}

proc execACMEScript { iter } \
{
   global objInfo jInfo

   set obj acme

   if {$objInfo($obj,sigCount) < 1} \
      {
       showError ACME "No signals to simulate."
       return
      }

   set scriptName acme.com
   set cmnd       acme2D.com

   set error [catch {set outUnit [open $scriptName w 0744]}]

   if {$error} \
      {
       showError ACME "Error opening output script $scriptName"
       return 1
      }

   puts $outUnit "#!/bin/csh -f"
   puts $outUnit ""

   set g       62x12+15+15
   set pInfo   ""
   set pCount  0

   puts $outUnit "$cmnd \\"

   for {set i 1} {$i <= $objInfo($obj,sigCount)} {incr i} \
       {
        set vInfo   ""

        foreach p $jInfo \
           {
            set pName [lindex $p 1]

            if {$objInfo($obj,$pName,$i,active)} \
               {
                lappend pInfo $objInfo($obj,$pName,$i)
                incr pCount
               }

            lappend vInfo $objInfo($obj,$pName,$i)
            lappend vInfo $objInfo($obj,$pName,$i,active)
            lappend vInfo $objInfo($obj,$pName,$i,mult)
           }

        puts $outUnit "   $vInfo \\"
       }
 
   puts $outUnit "   -sigCount     $objInfo($obj,sigCount)     \\"
   puts $outUnit "   -maxSig       $objInfo($obj,maxSig)       \\"
   puts $outUnit "   -maxPassive   $objInfo($obj,maxPassive)   \\"
   puts $outUnit "   -noise        $objInfo($obj,noise)        \\"
   puts $outUnit "   -parent       $objInfo($obj,origName)     \\"
   puts $outUnit "   -xDelay       $objInfo($obj,xDelay)       \\"
   puts $outUnit "   -yDelay       $objInfo($obj,yDelay)       \\"
   puts $outUnit "   -xP0          $objInfo($obj,xP0)          \\"
   puts $outUnit "   -xP1          $objInfo($obj,xP1)          \\"
   puts $outUnit "   -yP0          $objInfo($obj,yP0)          \\"
   puts $outUnit "   -yP1          $objInfo($obj,yP1)          \\"
   puts $outUnit "   -diag         $objInfo($obj,diagFlag)     \\"
   puts $outUnit "   -xActive      $objInfo($obj,xActive)      \\"
   puts $outUnit "   -yActive      $objInfo($obj,yActive)      \\"
   puts $outUnit "   -iter         $iter \\"
   puts $outUnit "   -pCount       $pCount $pInfo" 
 
   close $outUnit
 
   catch {[exec xterm -bg cyan -fg black -geom $g -e $scriptName]}

   readACMEView
   readACMEFit $objInfo($obj,tabName)
   showACMEView

   return 0
}

proc readACMEView {} \
{
   global objInfo

   set obj acme

   readROI -in $objInfo($obj,simName) -roi $objInfo($obj,simID) -ndim 2
   readROI -in $objInfo($obj,difName) -roi $objInfo($obj,difID) -ndim 2
}

proc readACMEFit { inName } \
{
   global objInfo 

   set obj acme

   set acmeTab [gdbRead table -in $inName]
   set entry   [gdbFirst $acmeTab]

   set objInfo($obj,fitFlag) 0

   while {$entry} \
      {
       set vName [gdbGet $entry VARNAME]
       set vSig  [gdbGet $entry SIGNAL]
       set vAdj  [gdbGet $entry ADJUSTABLE]
       set val   [gdbGet $entry VALUE]

       if {$vSig} \
          {
           set objInfo($obj,$vName,$vSig)        [format %.3f $val]
           set objInfo($obj,$vName,$vSig,active) $vAdj
          } \
       else \
          {
           set objInfo($obj,$vName) [format %.3f $val]
          }
 
       set entry [gdbNext $entry]
      }

   gdbDestroy $acmeTab

   set w   .acmeP

   wm title $w "ACME Region: $objInfo($obj,regID) RMS/N: $objInfo($obj,rms)"

   update
}

proc writeACMEView {} \
{
   global objInfo

   set obj acme

   writeROI -roi $objInfo($obj,roiID) -out $objInfo($obj,specName) -ov
   writeROI -roi $objInfo($obj,roiID) -out $objInfo($obj,simName)  -ov
   writeROI -roi $objInfo($obj,roiID) -out $objInfo($obj,difName)  -ov

   catch {exec zeroNMR $objInfo($obj,simName)}
}

proc resizeACMEView args \
{
}

proc showACMEView {} \
{
   global objInfo

   set obj acme

   showVerb "ACMEView: Showing ROI $objInfo($obj,roiID)."

   busyView hold $objInfo($obj,window)

   clearACMELabels

   newScreen -win $objInfo($obj,pixmap)

   drawROI -win $objInfo($obj,pixmap) \
           -cell $objInfo($obj,cellExp) -roi $objInfo($obj,roiID)      \
           -hi $objInfo($obj,hi) -bg $objInfo(color,bg)                \
           -color1 $objInfo(color,posColor1)                           \
           -color2 $objInfo(color,negColor1)                           \
           -plev $objInfo(prog,plev) -nlev $objInfo(prog,nlev)         \
           -$objInfo(prog,drawType) -xGrid -yGrid

   drawROI -win $objInfo($obj,pixmap) \
           -cell $objInfo($obj,cellSim) -roi $objInfo($obj,simID)      \
           -hi $objInfo($obj,hi) -bg $objInfo(color,bg)                \
           -color1 $objInfo(color,posColor1)                           \
           -color2 $objInfo(color,negColor1)                           \
           -plev $objInfo(prog,plev) -nlev $objInfo(prog,nlev)         \
           -$objInfo(prog,drawType) -xGrid -yGrid

   drawROI -win $objInfo($obj,pixmap) \
           -cell $objInfo($obj,cellDif) -roi $objInfo($obj,difID)      \
           -hi $objInfo($obj,hi) -bg $objInfo(color,bg)                \
           -color1 $objInfo(color,posColor1)                           \
           -color2 $objInfo(color,negColor1)                           \
           -plev $objInfo(prog,plev) -nlev $objInfo(prog,nlev)         \
           -$objInfo(prog,drawType) -xGrid -yGrid

   showACMELabels

   busyView release $objInfo($obj,window)
}

proc initACMEParmWindow { parent } \
{
    global objInfo jInfo

    set obj acme
    set w   .acmeP 

    if {[winfo exists $w]} return

    showVerb "Initializing ACME parameter window."

    toplevel $w

    wm title $w "ACME Region: None RMS/N: None"

    set objInfo($obj,notebook) $w.note
    set pageList ""

    for {set i 1} {$i <= $objInfo($obj,maxSig)} {incr i} \
       {
        lappend pageList "Signal $i"
       }

    notebook_create $objInfo($obj,notebook) -pages $pageList \
                    -width  $objInfo($obj,noteWidth)  \
                    -height $objInfo($obj,noteHeight)

    pack $objInfo($obj,notebook) -in $w -fill both -expand 1 -side top

    if {!$objInfo($obj,xActive) && !$objInfo($obj,yActive)} \
       {
        set jInfo {
                   {"J Active:"       jActive     0.0 0 left  0.0   label}
                   {"X Loc:"          xPos        0.0 1 left  None  label}
                   {"X Width:"        xWidth      5.0 1 left  0.0   label}
                   {"X Passive A^1:"  xPassive1  15.0 1 left  0.0   button}
                   {"X Passive B^1:"  xPassive2  10.0 0 left  0.0   button}
                   {"X Passive C^1:"  xPassive3   5.0 0 left  0.0   button}
                   {"Hi*1:"           jHi       100.0 1 right None  button}
                   {"Y Loc:"          yPos        0.0 1 right None  label}
                   {"Y Width:"        yWidth      5.0 1 right 0.0   label}
                   {"Y Passive A^1:"  yPassive1  15.0 1 right 0.0   button}
                   {"Y Passive B^1:"  yPassive2  10.0 0 right 0.0   button}
                   {"Y Passive C^1:"  yPassive3   5.0 0 right 0.0   button}
                  }
       } \
    else \
       {
        set jInfo {
                   {"J Active:"       jActive     8.0 1 left  0.0   label}
                   {"X Loc:"          xPos        0.0 1 left  None  label}
                   {"X Width:"        xWidth      5.0 1 left  0.0   label}
                   {"X Passive A^1:"  xPassive1  15.0 1 left  0.0   button}
                   {"X Passive B^1:"  xPassive2  10.0 0 left  0.0   button}
                   {"X Passive C^1:"  xPassive3   5.0 0 left  0.0   button}
                   {"Hi*1:"           jHi       100.0 0 right None  button}
                   {"Y Loc:"          yPos        0.0 1 right None  label}
                   {"Y Width:"        yWidth      5.0 1 right 0.0   label}
                   {"Y Passive A^1:"  yPassive1  15.0 1 right 0.0   button}
                   {"Y Passive B^1:"  yPassive2  10.0 0 right 0.0   button}
                   {"Y Passive C^1:"  yPassive3   5.0 0 right 0.0   button}
                  }
       }

    for {set i 1} {$i <= $objInfo($obj,maxSig)} {incr i} \
       {
        set pageName "Signal $i"
        set ww       [notebook_frame $objInfo($obj,notebook) $pageName]

        frame $ww.f${i}left
        frame $ww.f${i}mid
        frame $ww.f${i}right
        frame $ww.f${i}bot

        foreach p $jInfo \
           {
            set pStr  [lindex $p 0]
            set pName [lindex $p 1]
            set pVal  [lindex $p 2]
            set aVal  [lindex $p 3]
            set fLoc  [lindex $p 4]
            set dVal  [lindex $p 5]
            set pType [lindex $p 6]

            frame $ww.fp$i$pName

            set objInfo($obj,$pName,$i)        $pVal
            set objInfo($obj,$pName,$i,active) $aVal 
            set objInfo($obj,$pName,$i,mult)   1

#           label $ww.lab$pName$i -text $pStr -width 13 -anchor e  \
#                 -padx 4 -pady 4

            button $ww.lab$pName$i -text $pStr -width 13 -anchor e \
                   -relief flat -padx 1 -pady 1 -command           \
                   "setACMEMult $pName $i $ww.lab$pName$i 1"

            bind $ww.lab$pName$i <ButtonRelease-3> \
                 "+ setACMEMult $pName $i $ww.lab$pName$i -1"

            entry $ww.ent$pName$i -textvariable objInfo($obj,$pName,$i) \
                  -width 10 

            checkbutton $ww.but$pName$i -selectcolor $objInfo(color,on) \
                        -variable objInfo($obj,$pName,$i,active)

            if {[strcasecmp $dVal None]} \
               {
                if {!$aVal} {set objInfo($obj,$pName,$i) $dVal}

                bind $ww.ent$pName$i <ButtonRelease-3> \
                     "+ setACMEDef $obj $pName $i $pVal $dVal $ww.lab$pName$i"
               }

            pack $ww.lab$pName$i $ww.ent$pName$i $ww.but$pName$i \
                 -in $ww.fp$i$pName -side left

            pack $ww.fp$i$pName -in $ww.f$i$fLoc -side top
           }

        label $ww.labAss$i -text "Ass:" -anchor e
        entry $ww.entAss$i -textvariable objInfo($obj,ass,$i) -width 16

        bind  $ww.entAss$i <Return> "+ setACMEAss $i"

        pack  $ww.labAss$i $ww.entAss$i -in $ww.f${i}bot \
              -side left -expand 1 -fill x

        pack $ww.f${i}left $ww.f${i}right \
             -in $ww.f${i}mid -side left -expand 1 -fill both

        pack $ww.f${i}mid -in $ww -side top -expand 1 -fill both
        pack $ww.f${i}bot -in $ww -side top
       }

    for {set i 1} {$i <= $objInfo($obj,maxSig)} {incr i} \
       {
        set objInfo($obj,jHi,$i) $objInfo($obj,jHi)
       }

   showACMECount
}

proc setACMEMult { pName i w step } \
{
   global objInfo

   switch $pName \
      {
       jHi       { setACMEWMult $pName $i $w $step }
       xPassive1 { setACMEWMult $pName $i $w $step }
       xPassive2 { setACMEWMult $pName $i $w $step }
       xPassive3 { setACMEWMult $pName $i $w $step }
       yPassive1 { setACMEWMult $pName $i $w $step }
       yPassive2 { setACMEWMult $pName $i $w $step }
       yPassive3 { setACMEWMult $pName $i $w $step }
      }
}

proc setACMEWMult { pName i w step } \
{
   global objInfo

   set obj acme

   set txt  [string trim [$w cget -text] ": "]
   set n    [string length $txt]
   set str  [string range $txt 0 [expr $n - 2]]
   set mult [string index $txt   [expr $n - 1]]
   
   incr mult $step

   if {$mult < 1} {set mult 9}
   if {$mult > 9} {set mult 1}
 
   set objInfo($obj,$pName,$i,mult) $mult 

   $w configure -text "$str$mult:"
  
   update
}

proc setACMEAss { i } \
{
   global objInfo

   clearACMELabels
   showACMELabels
}

proc setACMEDef { obj pName i pVal dVal w } \
{
   global objInfo

   if {$objInfo($obj,$pName,$i,active)} \
      {   
       set objInfo($obj,$pName,$i)        $dVal
       set objInfo($obj,$pName,$i,active) 0
       set objInfo($obj,$pName,$i,mult)   l
      } \
   else \
      {
       set objInfo($obj,$pName,$i)        $pVal
       set objInfo($obj,$pName,$i,active) 1
       set objInfo($obj,$pName,$i,mult)   l
      }

   setACMEMult $pName $i $w 0
}

proc showACMECount {} \
{
   global objInfo

   set obj acme

   update

   for {set i 0} {$i < $objInfo($obj,maxSig)} {incr i} \
      {
       if {$i < $objInfo($obj,sigCount)} \
          {
           notebook_pageconfig $objInfo($obj,notebook) $i -state enabled
          } \
       else \
          {
           notebook_pageconfig $objInfo($obj,notebook) $i -state disabled
          }
      }

   update
}

proc insertACMESignal { obj sxPix syPix } \
{
   global objInfo

   if {$objInfo(acme,sigCount) < $objInfo(acme,maxSig)} \
      {
       incr objInfo(acme,sigCount)
      }

   set xPix [$objInfo($obj,canvas) canvasx $sxPix]
   set yPix [$objInfo($obj,canvas) canvasy $syPix]
   set id   [findACMECell $obj $xPix $yPix]

   if {!$id} return

   set winID $objInfo($obj,pixmap)

   set xPPM \
    [format %.3f [pix2spec -cell $id -win $winID -roi $id -x $xPix -units pts]]

   set yPPM \
    [format %.3f [pix2spec -cell $id -win $winID -roi $id -y $yPix -units pts]]

   set xPts \
    [format %.3f [pix2spec -cell $id -win $winID -roi $id -x $xPix -units pts]]

   set yPts \
    [format %.3f [pix2spec -cell $id -win $winID -roi $id -y $yPix -units pts]]

   set i   $objInfo(acme,sigCount)
   set ass None 

   set objInfo($obj,xPos,$i) $xPts
   set objInfo($obj,yPos,$i) $yPts
   set objInfo($obj,ass,$i)  $ass

   set sigInfo "$xPts $yPts $xPPM $yPPM $ass"

   lappend objInfo($obj,sigList) $sigInfo

   showACMECount
   showACMELabels
}

proc raiseACMESignal { obj tID sigID } \
{
   global objInfo
}

proc resetACMESignals {} \
{
   global objInfo

   set obj acme

   set objInfo($obj,sigCount) 0
   set objInfo($obj,sigList)  ""
}

proc clearACMELabels {} \
{
   global objInfo

   set obj acme

   $objInfo($obj,canvas) delete signal label
}

proc showACMELabels {} \
{
   global objInfo stdfont

   set obj acme

   clearACMELabels

   foreach id $objInfo($obj,idList) \
      { 
       set i 0

       foreach sigInfo $objInfo($obj,sigList) \
          { 
           incr i

           if {![strcasecmp $objInfo($obj,ass,$i) None]} \
              {
               set str $i
              } \
           else \
              {
               set str "$i=$objInfo($obj,ass,$i)"
              }
 
           set xPts [lindex $sigInfo 0]
           set yPts [lindex $sigInfo 1]

           set xPix [spec2pix -cell $id -win $objInfo($obj,pixmap) \
                              -roi $id -x ${xPts}pts]

           set yPix [spec2pix -cell $id -win $objInfo($obj,pixmap) \
                              -roi $id -y ${yPts}pts]

           set tID [$objInfo($obj,canvas) create text $xPix $yPix \
                       -fill $objInfo(color,label) -tags signal   \
                       -font $stdfont -anchor center -text $str]

           $objInfo($obj,canvas) bind $tID <ButtonRelease-3> \
              "+ raiseACMESignal $obj $tID $objInfo(acme,sigCount)"
          }
     }

   update
}

proc findACMECell { obj xPix yPix } \
{
   global objInfo

   foreach id $objInfo($obj,idList) \
      {
       set cInfo [getCellInfo -cell $id -win $objInfo($obj,pixmap)]
       set xInfo [lindex $cInfo 1]
       set yInfo [lindex $cInfo 2]

       set ix1   [lindex $xInfo 0]
       set ixn   [lindex $xInfo 1]
       set iy1   [lindex $yInfo 0]
       set iyn   [lindex $yInfo 1]

       if {$xPix < $ix1 || $xPix > $ixn} continue
       if {$yPix < $iy1 || $yPix > $iyn} continue

       return $id
      }

   return 0
}

proc initACMESpecWindow { parent } \
{
    global objInfo

    set obj acme
    set w   .$obj

    if {[winfo exists $w]} return

    showVerb "Initializing ACME spectrum window."

    toplevel $w

    wm title $w "ACME Spectrum/Simulation/Residual"

    set objInfo($obj,winXSize)      $objInfo($obj,defXSize)
    set objInfo($obj,winYSize)      $objInfo($obj,defYSize) 

    set objInfo($obj,leftMargin)    10 
    set objInfo($obj,rightMargin)   10 
    set objInfo($obj,topMargin)     10
    set objInfo($obj,bottomMargin)  10

    set objInfo($obj,winXAdj)        5 
    set objInfo($obj,winYAdj)       20 

    set objInfo($obj,cellXSize) \
        [expr $objInfo($obj,winXSize)     - \
              ($objInfo($obj,leftMargin)  + \
               $objInfo($obj,rightMargin) + \
               $objInfo($obj,winXAdj))]

    set objInfo($obj,cellYSize) \
        [expr $objInfo($obj,winYSize)      - \
              ($objInfo($obj,topMargin)    + \
               $objInfo($obj,bottomMargin) + \
               $objInfo($obj,winYAdj))]
 
    set objInfo($obj,xSize) \
        [expr $objInfo($obj,winXSize) - $objInfo($obj,winXAdj)]

    set objInfo($obj,ySize) \
        [expr $objInfo($obj,winYSize) - $objInfo($obj,winYAdj)]

    set id  [getNextID]

    set objInfo($obj,window)       $w
    set objInfo($obj,canvas)       $w.c
    set objInfo($obj,scrollH)      $w.sh
    set objInfo($obj,scrollV)      $w.sv
    set objInfo($obj,parent)       $parent
    set objInfo($obj,pixmap)       $id 
    set objInfo($obj,cell)         $id 
    set objInfo($obj,inName1)      $objInfo($parent,inName1)
    set objInfo($obj,inName2)      None

    set objInfo($obj,inList) "$objInfo($obj,inName1) $objInfo($obj,inName2)"

    if {![strcasecmp $objInfo($obj,origName) None]} \
       {
        set objInfo($obj,origName) $objInfo($parent,inName1)
       }

    if {$objInfo($obj,noise) == 0.0} \
       {
        set objInfo($obj,noise) $objInfo($parent,noise1)
       }

    set objInfo($obj,resizeProc)   resizeACMEView
    set objInfo($obj,redrawProc)   showACMEView

    set objInfo($obj,x1) $objInfo($obj,leftMargin)
    set objInfo($obj,xn) [expr $objInfo($obj,x1) + $objInfo($obj,cellXSize)]
    set objInfo($obj,y1) $objInfo($obj,topMargin)
    set objInfo($obj,yn) [expr $objInfo($obj,y1) + $objInfo($obj,cellYSize)]

    set dy [expr $objInfo($obj,winYAdj)/3]
    set cy [expr $objInfo($obj,cellYSize)/3 - $dy]

    set objInfo($obj,x1Exp) $objInfo($obj,x1)
    set objInfo($obj,xnExp) $objInfo($obj,xn)
    set objInfo($obj,y1Exp) $objInfo($obj,y1)
    set objInfo($obj,ynExp) [expr $objInfo($obj,y1) + $cy]

    set objInfo($obj,x1Sim) $objInfo($obj,x1)
    set objInfo($obj,xnSim) $objInfo($obj,xn)
    set objInfo($obj,y1Sim) [expr $objInfo($obj,ynExp) + $dy]
    set objInfo($obj,ynSim) [expr $objInfo($obj,y1Sim) + $cy]

    set objInfo($obj,x1Dif) $objInfo($obj,x1)
    set objInfo($obj,xnDif) $objInfo($obj,xn)
    set objInfo($obj,y1Dif) [expr $objInfo($obj,ynSim) + $dy]
    set objInfo($obj,ynDif) [expr $objInfo($obj,y1Dif) + $cy]

    set xPix [expr ($objInfo($obj,x1Exp) + $objInfo($obj,xnExp))/2]
    set yPix [expr ($objInfo($obj,y1Exp) + $objInfo($obj,ynExp))/2]

    set objInfo($obj,x1Pix)        $xPix
    set objInfo($obj,y1Pix)        $yPix
    set objInfo($obj,xnPix)        $xPix
    set objInfo($obj,ynPix)        $yPix
    set objInfo($obj,x0Pix)        -1 
    set objInfo($obj,y0Pix)        -1

    set objInfo($obj,roiList)      "0 0"
    set objInfo($obj,roi1)         0
    set objInfo($obj,roi2)         0
    set objInfo($obj,hi)           $objInfo($parent,hi1)
    set objInfo($obj,hi1)          $objInfo($parent,hi2)
    set objInfo($obj,hi2)          $objInfo($parent,hi2)

    set objInfo($obj,factor)       $objInfo($parent,factor)
    set objInfo($obj,xAxis)        $objInfo($parent,xAxis)
    set objInfo($obj,yAxis)        $objInfo($parent,yAxis)

    frame $w.top
    frame $w.bottom
   
    button $w.fit   -text "Fit"      -padx 2 -pady 2 -command  fitACMEView
    button $w.sim   -text "Simulate" -padx 2 -pady 2 -command  simACMEView
    button $w.reset -text "Reset"    -padx 2 -pady 2 -command  resetACMEView
    button $w.read  -text "Read"     -padx 2 -pady 2 -command  restoreACMEView
    button $w.save  -text "Accept"   -padx 2 -pady 2 -command  acceptACMEView
    button $w.draw  -text "Draw"     -padx 2 -pady 2 -command  showACMEView
 
    entry  $w.first1 -textvariable objInfo($obj,hi) -width 12 
    button $w.incr1  -text "+" -padx 1 -pady 1 -command "incrHi $obj 0"
    button $w.decr1  -text "-" -padx 1 -pady 1 -command "decrHi $obj 0"
    bind   $w.first1 <Return> "+ setHi $obj"

    pack $w.fit $w.sim $w.reset $w.draw $w.read $w.save -in $w.top -side left 
    pack $w.decr1 $w.incr1 $w.first1 -in $w.top -side right

    set dx 10
    set dy 10

    canvas $objInfo($obj,canvas) -border 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize)      \
       -closeenough 10

    wm geom  $objInfo($obj,window) \
       =$objInfo($obj,winXSize)x$objInfo($obj,winYSize)-$dx+$dy

    update

    pack $objInfo($obj,canvas)  -in $w.bottom -side left -expand 1 -fill both

    pack $w.top    -side top -expand 0 -fill x
    pack $w.bottom -side top -expand 1 -fill both

    update

    $objInfo($obj,canvas) create pixmap 0 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
       -win $objInfo($obj,pixmap) -tags $obj

    defineCell -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
               -x $objInfo($obj,x1) $objInfo($obj,xn) \
               -y $objInfo($obj,y1) $objInfo($obj,yn)

    defineCell -cell $objInfo($obj,cellExp) -win $objInfo($obj,pixmap) \
               -x $objInfo($obj,x1Exp) $objInfo($obj,xnExp) \
               -y $objInfo($obj,y1Exp) $objInfo($obj,ynExp)

    defineCell -cell $objInfo($obj,cellSim) -win $objInfo($obj,pixmap) \
               -x $objInfo($obj,x1Sim) $objInfo($obj,xnSim) \
               -y $objInfo($obj,y1Sim) $objInfo($obj,ynSim)

    defineCell -cell $objInfo($obj,cellDif) -win $objInfo($obj,pixmap) \
               -x $objInfo($obj,x1Dif) $objInfo($obj,xnDif) \
               -y $objInfo($obj,y1Dif) $objInfo($obj,ynDif)

    bind $objInfo($obj,canvas) <ButtonRelease-1> "insertACMESignal $obj %x %y"
}
