#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Adjust Data in Selected Table Lines, with Optional Sorting:"
    puts " -in   inName  \[test.tab\] Input Table or PDB File." 
    puts " -out  outName \[None\]     Output Table." 
    puts "Options to Set Values in Table (Use Either -val or -expr, not both):"
    puts " -set  vName   \[None\]     Table Variable to Adjust."
    puts " -cond cond    \[1\]        Selection Condition."
    puts " -val  tclVal  \[0\]        TCL Expression for New Value;"
    puts "                          Should be the Last Argument."
    puts " -expr tclExpr \[None\]     TCL Numerical Expression for New Value;"
    puts "                          Should be the Last Argument."
    puts " -keep                    Keep Table Data Unchanged." 
    puts "Options for Changing the Variable:"
    puts " -rename newName \[None\]   New Name for Variable."
    puts " -refmt  newFmt  \[None\]   New Format for Variable."
    puts "Options for Keeping First or Last Entries (Use Only One):"
    puts " -head   nH      \[All\]    Number of Rows at Start of Table to Keep." 
    puts " -tail   nT      \[All\]    Number of Rows at End of Table to Keep." 
    puts "Options for Sorting Table Output:"
    puts " -sort   sVar    \[None\]   Table Variable for Sort." 
    puts " -up                      Sort by Ascending Order (Default)."
    puts " -down                    Sort by Descending Order (Default)."
    puts " -before                  Sort Before Setting Values." 
    puts " -after                   Sort After Setting Values (Default)." 
    puts " -numSort                 Sort Text Data Numerically." 
    puts " -abs                     Sort Numbers by Absolute Value."
    puts "PDB Options:"
    puts " -pdb                     Input is PDB Data (Default for .pdb)."
    puts " -nopdb                   Input is NMRPipe-Format Table."
    puts " -hetatm                  Interpret HETATMs as ATOMs."
    puts " -renumber                Renumber ATOMID Before Writing Result."
    puts " -seg2chain               Reset CHAINID from SEGID."
    puts " -chain2seg               Reset SEGID from CHAINID."
    puts "Other Options:"
    puts " -tcl  sName   \[None\]     Name of TCL Initialize File to source."
    puts " -del                     Delete All Non-Selected Entries from Table."
    puts " -notext                  Delete Comment Text from Table."
    puts " -verb                    Verbose Mode ON (Default)."
    puts " -noverb                  Verbose Mode OFF."
    puts "Notes:"
    puts " 1. Use table variable names as TCL variables for the -expr argument;"
    puts "    use ' (single quote) characters or \\ (backslash) characters"
    puts "    to preserve \$ (dollar-signs) in variable names."
    puts "    characters in the command line."
    puts " 2. Addition variables are defined for use in expressions:"
    puts "      \$n   Number of entries in the table."
    puts "      \$m   Number of entries in the selection."
    puts "      \$i   Selection number of the entry, 0 to \$m - 1."
    puts "      \$loc Entry number, in the range of  0 to \$n - 1."
    puts " 3. Use the -tcl option to define special functions for expressions,"
    puts "    set random number seeds, etc."
    puts " 4. PDB data will only be saved in PDB format if the output"
    puts "    name ends in the .pdb extension." 
    puts "Examples:"
    puts "Set table variable SIGMA to 100.0:"
    puts " -set SIGMA -val 100.0"
    puts "Set table variable SIGMA to have the value of table variable DD:" 
    puts " -set SIGMA -val '\$DD'"
    puts "Set table variable RESNAME to text ALA:"
    puts " -set RESNAME -val ALA"
    puts "Set table variable CHAINID to blanks:"
    puts " -set CHAINID -val \"(NULL)\""
    puts "Set table variable SHIFT to sum of table variables RC_SHIFT and BASE:"
    puts " -set SHIFT -expr '\$RC_SHIFT + \$BASE'"
    puts "Renumber table variable INDEX:"
    puts " -set INDEX -expr '\$loc + 1'"
    puts "Keep the 10 Rows with the largest HEIGHT:"
    puts " -sort HEIGHT -down -abs -head 10"
    exit 1 
   }

set inName      test.tab
set outName     None 
set tclName     None
set vName       None 
set sName       None 
set sProc       None
set cond        1
set valStr      0 
set exprStr     None 
set afterFlag   1
set delSelFlag  0
set delTextFlag 0
set renumFlag   0
set verbose     1

set sortMode    up
set pdbMode     nopdb
set hetMode     nohetatm
set atomMode    atom
set remMode     text
set absMode     noabs

set newName     None
set newFmt      None
set headCount   All
set tailCount   All
 
getArgD $ARGV -in   inName
getArgD $ARGV -out  outName
getArgD $ARGV -set  vName
getArgD $ARGV -tcl  tclName
getArgD $ARGV -sort sName

getArgD $ARGV -head headCount
getArgD $ARGV -tail tailCount

getArgD $ARGV -rename newName
getArgD $ARGV -refmt  newFmt

getListArgD $ARGV -val  valStr
getListArgD $ARGV -cond cond

set loc [flagLoc $ARGV -expr]

if {$loc} \
   {
    set exprStr ""

    foreach i [lrange $ARGV [expr $loc + 1] end] \
       {
        set p "{}"
        set exprStr "$exprStr [string trim $i $p]"
       }

    set exprStr [string trim $exprStr]
   }

if {![strcasecmp [file extension $inName] .pdb]} {set pdbMode pdb}

if {[flagLoc $ARGV -pdb]}        {set pdbMode pdb}
if {[flagLoc $ARGV -nopdb]}      {set pdbMode nopdb}

if {[flagLoc $ARGV -renumber]}   {set renumFlag 1}
if {[flagLoc $ARGV -norenumber]} {set renumFlag 0}

if {[flagLoc $ARGV -rem]}        {set remMode text}
if {[flagLoc $ARGV -norem]}      {set remMode notext}

if {[flagLoc $ARGV -atom]}       {set atomMode atom}
if {[flagLoc $ARGV -noatom]}     {set atomMode noatom}

if {[flagLoc $ARGV -hetatm]}     {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}   {set hetMode nohetatm}

if {[flagLoc $ARGV -up]}      {set sortMode up}
if {[flagLoc $ARGV -down]}    {set sortMode down}

if {[flagLoc $ARGV -del]}     {set delSelFlag 1}
if {[flagLoc $ARGV -nodel]}   {set delSelFlag 0}

if {[flagLoc $ARGV -text]}    {set delTextFlag 0}
if {[flagLoc $ARGV -notext]}  {set delTextFlag 1}

if {[flagLoc $ARGV -abs]}     {set absMode abs}
if {[flagLoc $ARGV -noabs]}   {set absMode noabs}

if {[flagLoc $ARGV -numSort]}     {set sProc numStrSort}
if {[flagLoc $ARGV -nonumSort]}   {set sProc None}

if {[flagLoc $ARGV -before]}  {set afterFlag 0}
if {[flagLoc $ARGV -after]}   {set afterFlag 1}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

if {[flagLoc $ARGV -keep]} \
   {
    set valStr  None
    set exprStr None
   }

set cond [string trim $cond "{}"]

proc numStrSort { a b } \
{
   global sortMode absMode

   set errorA [catch {set nA [scan $a %d aVal]}]
   set errorB [catch {set nB [scan $b %d bVal]}]

   if {$errorA || $errorB} \
      {
       set nA   0
       set nB   0
       set aVal 0
       set bVal 0
      }

   if {![strcasecmp $absMode abs]} \
      {
       set aVal [expr $abs($aVal)]
       set bVal [expr $abs($bVal)]

#      set a [string trimleft $a -]
#      set b [string trimleft $b -]
      }

   if {$nA && $nB} \
      {
       if {![strcasecmp $sortMode up]} \
          {
           if {$aVal > $bVal} {return  1}
           if {$aVal < $bVal} {return -1}
          } \
       else \
          {
           if {$aVal > $bVal} {return -1}
           if {$aVal < $bVal} {return  1}
          }
       }

   if {![strcasecmp $sortMode up]} \
      {
       return [strcmp $a $b]
      } \
   else \
      {
       return [strcmp $b $a]
      }

   return 0
}

#
#----------------------------------------------------------------------

if {[strcasecmp $tclName None]} \
   {
    if {![file exists $tclName]} \
       {
        puts stderr "Error Finding TCL Initialization File $tclName"
        exit 1
       }

    source $tclName
   }

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input Table $inName"
    exit 1
   }

set t      [gdbRead table -in $inName -$pdbMode -$hetMode]
set sList  [gdbSelect -from $t -cond [string trim $cond "{}"]]
set eList  [gdbGetInfo $t -parms entries]
set vList  [gdbGetInfo $t -parms varNames]
set n      [llength $eList]
set m      [llength $sList]

if {$verbose} \
   {
    puts stderr ""
    puts stderr "Input:            $inName"
    puts stderr "Entries In:       $n"
    puts stderr "Adjust Variable:  $vName"

    if {[strcasecmp $newName None]} \
       {
        puts stderr "New Variable:     $newName"
       }

    if {[strcasecmp $newFmt None]} \
       {
        puts stderr "New Format:       $newFmt"
       }

    if {[strcasecmp $exprStr None]} \
       {
        puts stderr "Expression:       $exprStr"
       } \
    elseif {[strcasecmp $valStr None]} \
       {
        puts stderr "New Value:        $valStr"
       } \
    else \
       {
        puts stderr "New Value:        No Change"
       }

    puts stderr "Sort Variable:    $sName"

    if {[strcasecmp $sName None]} \
       {
        if {![strcasecmp $absMode abs]} \
           {
            set sStr "$sortMode, absolute value"
           } \
        else \
           {
            set sStr $sortMode
           }

        if {[strcasecmp $sProc None]} \
           {
            set nStr " Sort text numerically,"
           } \
        else \
           {
            set nStr ""
           }

        if {$afterFlag} \
           {
            puts stderr "Sort Mode:        Sort $sStr,$nStr Sort after setting values."
           } \
        else \
           {
            puts stderr "Sort Mode:        Sort $sStr,$nStr Sort before setting values."
           }
       }

    puts stderr "Condtion:         $cond"
    puts stderr "Selected:         $m"
    puts stderr "Retain from Head: $headCount"
    puts stderr "Retain from Tail: $tailCount"
   }

if {!$afterFlag} \
   {
    if {[strcasecmp $sName None]} \
       {
        gdbSortTab $t $sName -$sortMode -$absMode -proc $sProc
        set eList [gdbGetInfo $t -parms entries]
       }
   }

if {[strcasecmp $vName None]} \
   { 
    if {[lsearch -exact $vList $vName] >= 0} \
       {
        set i 0

        foreach entry $sList \
           {
            foreach v $vList {set $v [gdbGet $entry $v]}

            set loc [lsearch -exact $eList $entry]

            if {[strcasecmp $exprStr None]} \
               {
                gdbSet $entry $vName [eval "expr $exprStr"]
               } \
            elseif {[strcasecmp $valStr None]} \
               {
                gdbSet $entry $vName [string trim [subst $valStr] "{}"]
               }

            incr i
           }
       }
   }

if {$afterFlag} \
   {
    if {[strcasecmp $sName None]} \
       {
        gdbSortTab $t $sName -$sortMode -$absMode -proc $sProc
        set eList [gdbGetInfo $t -parms entries]
       }
   }

if {$delSelFlag && [strcmp $cond 1]} \
   {
    set dList "" 

    foreach e $eList \
       {
        if {[lsearch -exact $sList $e] < 0} {lappend dList $e}
       }

    gdbDestroy $dList

    set eList [gdbGetInfo $t -parms entries]
   }

set entryCount [llength $eList]

if {[strcasecmp $headCount All]} \
   {
    if {$headCount < $entryCount} {gdbDestroy [lrange $eList $headCount end]}
    set eList  [gdbGetInfo $t -parms entries]
   }

if {[strcasecmp $tailCount All]} \
   {
    if {$tailCount < $entryCount} {gdbDestroy [lrange $eList 0 [expr $entryCount - $tailCount - 1]]}
    set eList  [gdbGetInfo $t -parms entries]
   }

if {$delTextFlag} {gdbDelNote $t}

if {[strcasecmp $vName None]} \
   {
    if {[lsearch -exact $vList $vName] >= 0} \
       {
        set vPtr [gdbFindByName $t $vName]

        if {[strcasecmp $newName None]} {gdbSetInfo $vPtr -name   $newName}
        if {[strcasecmp $newFmt  None]} {gdbSetInfo $vPtr -varFmt $newFmt}
       }
   }

if {[strcasecmp [file extension $outName] .pdb]} {set pdbMode nopdb}

if {![strcmp $pdbMode pdb]} \
   {
    if {[flagLoc $ARGV -chain2seg]} \
       {
        foreach e $eList \
           {
            set chain [gdbGet $e CHAINID]
            gdbSet $e SEGID   $chain
            gdbSet $e CHAINID "(Null)"
           }
       }

    if {[flagLoc $ARGV -seg2chain]} \
       {
        foreach e $eList \
           {
            set seg [gdbGet $e SEGID]
            gdbSet $e CHAINID $seg  
            gdbSet $e SEGID   "(Null)" 
           }
       }

    if {$renumFlag} \
       {
        set eList [gdbGetInfo $t -parms entries]
        set id    0

        foreach e $eList \
           {
            incr id
            gdbSet $e ATOMID $id
           } 

        if {$verbose} {puts stderr "Renumbered:       $id"}
       }

    gdbWrite $t $outName -pdb -$hetMode -$atomMode -$remMode
   } \
else \
   {
    gdbWrite $t $outName
   }

if {$verbose} \
   {
    puts stderr "Output:           $outName"
    puts stderr "Entries Out:      [gdbGetInfo $t -parms entryCount]"
   }

exit
