
#
# Event handler for building conversion from Amgen Protocol Bruker parameters.

proc readParams_BRUKER_AMGEN {} \
{
   global inName specialArgs DECIM DSPFVS
   global bigfont

   toplevel .brukAmgenTop

   wm       title .brukAmgenTop "Bruker (Amgen Protocol)"

   message  .brukAmgenTop.brukMsg -width 30c -font $bigfont -relief raised \
            -bg red -fg white -text "Please Wait . . ."

   pack     .brukAmgenTop.brukMsg -expand 1

   clearScript

   getAmgenBruker3D $inName

   set specialArgs " -noswap"

   if {$DECIM > 1} \
      {
       if {$dmxMode} \
          {
           append specialArgs " -DMX -decim $DECIM -dspfvs $DSVFVS"
          } \
       else \
          {
           append specialArgs " -AMX -decim $DECIM -dspfvs $DSVFVS"
          }
      }

   destroy .brukAmgenTop
}

#
# getAmgenBruker3D: get parameters from an Amgen-protocol Bruker pulse sequence.

proc getAmgenBruker3D { nmrPath } \
{
   global Nx Tx MODEx SWx OBSx CARx FTx LABx \
          Ny Ty MODEy SWy OBSy CARy FTy LABy \
          Nz Tz MODEz SWz OBSz CARz FTz LABz \
          Na Ta MODEa SWa OBSa CARa FTa LABa

   global DECIM EXPCODES EXPNAME outName dataSource

   global tcl_precision old_tcl_precision

#
# Set the precision for the PPM expressions, enable restore of precision later:

   if {[info exists tcl_precision]} {set old_tcl_precision $tcl_precision}

   set shutdown \
     {
      if {[info exists old_tcl_precision]} \
         {
          set tcl_precision $old_tcl_precision
         }
     }

   set tcl_precision 17
#
# Test for the existence of the parameter and data files.
# Read their contents into arrays.

   set fileList {acqus acqu2s ser}

   foreach i $fileList \
      {
       if {![tkFileTest exists $nmrPath/1/$i]} \
          {
           puts stderr "Error: Unable to find file $nmrPath/1/$i"
           eval $shutdown
           return 1
          }
      }

   rdJCAMP $nmrPath/1/acqus  acqus
   rdJCAMP $nmrPath/1/acqu2s acqu2s

#
# Test for the existence of all needed parameters:

   set parmList1 \
    {INSTRUM PULPROG TE SFO1 SFO2 SFO3 TD SW_h DSLIST IN AQ_mod DECIM}

   set parmList2 {TD NUCLEUS}

   foreach parm $parmList1 \
      {
       if {![parmExists acqus $parm]} \
          {
           eval $shutdown
           return 1
          }
      }
   
   foreach parm $parmList2 \
      {
       if {![parmExists acqu2s $parm]} \
          {
           eval $shutdown
           return 1
          }
      }

#
# Reset default values:

   set FTx Time
   set FTy Time
   set FTz Time
   set FTa Time

#
# Find the instrument type, and set up the PPM calibration curve variables:

   if {-1 != [string first "600" $acqus(INSTRUM)]} \
      {
       set HORIGIN 600.1279745
       set MHZDEG  0.0000061157143
      } \
   elseif {-1 != [string first "500" $acqus(INSTRUM)]} \
      {
       set HORIGIN 500.1283120
       set MHZDEG  0.0000050966494
      } \
   else \
      {
       puts stderr "Error: unknown instrument name $acqus(INSTRUM)"
       eval $shutdown
       return 1
      }

#
# Set the direct acqusition mode:


   set DECIM $acqus(DECIM)

   switch $acqus(AQ_mod) \
      {
       0 {set MODEx Real}
       1 {set MODEx Complex}
       2 {set MODEx Sequential}
       3 {set MODEx DQD}
       default \
          {
           puts "Error: unknown acqusition mode AQ_mod = $$acqus(AQ_mod)"
           eval $shutdown
           return 1
          }
      }

#
# Find the experiment type from these codes in pulse program name:
#  di dipsy
#  pr presat
#  ct contant-time
#  se sensitivity-enhanced
#  gs gradient-enhanced 
#  ea each-antiecho
 
   set expCodeList { di pr ct se gs ea t2 t1 }
   set EXPCODES    [list nmr]

   set EXPNAME $acqus(PULPROG)

   foreach i $expCodeList \
      {
       set loc [string last $i $EXPNAME]

       if {$loc != -1} \
          {
           lappend EXPCODES $i

           set s1 [string range $EXPNAME 0 [expr $loc - 1]]
           set s2 [string range $EXPNAME [expr $loc + 2] end]
           set EXPNAME $s1$s2
          }
      }

   set    outName fid/$EXPNAME
   append outName %03d.fid

# Dimension sizes:
# X-Axis size: check for multiple of 256
# Y-Axis size: assumes States-mode acquisition.
# Z-Axis size: check for even plane count

   set serCount   [llength [glob -nocomplain $nmrPath/*/ser]]
   set dataSource CATSER

   if {!$serCount} \
      {
       puts stderr "Error: no ser files found."
       eval $shutdown
       return 1
      }

   set pad [expr $acqus(TD) % 256]

   if {$pad} \
      {
       set Nx [expr $acqus(TD) + 256 - $pad]
      } \
   else \
      {
       set Nx $acqus(TD)
      }

   if {$MODEx == "Sequential" || $MODEx == "Real"} \
      {
       set Tx $acqus(TD) 
      } \
   else \
      {
       set Tx [expr $acqus(TD) / 2]
      }

   set Ny $acqu2s(TD)
   set Ty [expr $Ny / 2]

   if {[expr $serCount % 2]} \
      {
       incr serCount -1
      }

   set Nz $serCount
   set Tz [expr $Nz / 2]

#
# Set spectral widths in Hz:

   set IN0 [lindex $acqus(IN) 0]
   set IN6 [lindex $acqus(IN) 6]

   set SWx $acqus(SW_h)
   set SWy [expr 1.0 / (2.0 * $IN0)]
   set SWz [expr 1.0 / (2.0 * $IN6)]

#
# Extract labels.
# Fix degenerate ones.

   set LABx F3
   set LABy F1
   set LABz F2

   if {[parmExists acqus  NUCLEUS]} {set LABx $acqus(NUCLEUS)}
   if {[parmExists acqu2s NUCLEUS]} {set LABy $acqu2s(NUCLEUS)}
   if {[parmExists acqus  DSLIST]}  {set LABz $acqus(DSLIST)}

#
# Set observe frequencies:

   set OBSx 999.0
   set OBSy 999.0
   set OBSz 999.0

   set CARx 0.000
   set CARy 0.000
   set CARz 0.000

   set HZERO [expr $HORIGIN + (double($acqus(TE)) * $MHZDEG)]
   set NZERO [expr $HZERO * 0.101329118]
   set CZERO [expr $HZERO * 0.2514495923]

   set HCAR  [expr (1.0e6 * ($acqus(SFO1) - $HZERO)) / $HZERO]
   set NCAR  [expr (1.0e6 * ($acqus(SFO3) - $NZERO)) / $NZERO]
   set CCAR  [expr (1.0e6 * ($acqus(SFO2) - $CZERO)) / $CZERO]

   set OBSx $HZERO
   set CARx $HCAR

   if {$LABy == "1H"} \
      {
       set OBSy $HZERO
       set CARy $HCAR
      } \
   elseif {$LABy == "15N"} \
      {
       set OBSy $NZERO
       set CARy $NCAR
      } \
   elseif {$LABy == "13C"} \
      {
       set OBSy $CZERO
       set CARy $CCAR
      }

   if {$LABz == "1H"} \
      {
       set OBSz $HZERO
       set CARz $HCAR
      } \
   elseif {$LABz == "15N"} \
      {
       set OBSz $NZERO
       set CARz $NCAR
      } \
   elseif {$LABz == "13C"} \
      {
       set OBSz $CZERO
       set CARz $CCAR
      }

#
# Adjust redundant labels:

   if {$LABx == "1H"} \
      {
       if {$LABy == "15N"} \
          {
           set LABx HN
           set LABy N
          } \
       elseif {$LABz == "15N"} \
          {
           set LABx HN
           set LABy N
          }
      }

   if {$LABy == $LABz} \
      {
       append LABx "x" 
       append LABy "y" 
      }

#
# Adjust precision of calculated parameters:

   set SWx  [format "%.3f" $SWx]
   set SWy  [format "%.3f" $SWy]
   set SWz  [format "%.3f" $SWz]

   set CARx [format "%.3f" $CARx]
   set CARy [format "%.3f" $CARy]
   set CARz [format "%.3f" $CARz]

   set OBSx [format "%.3f" $OBSx]
   set OBSy [format "%.3f" $OBSy]
   set OBSz [format "%.3f" $OBSz]

   eval $shutdown

   return 0
}

#
# Bottom.
