#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Y-Axis Baseline Correction of Selected X-Axis Regions:"
   echo " -in     inName   [test.ft2]   Input to Correct."
   echo " -out    outName  [base.ft2]   Corrected Output."
   echo " -dif    difName  [None]       Baseline Difference Output, or Keyword None."
   echo " -mask   maskName [None]       Baseline Mask Output, or Keyword None."
   echo " -x1     x1List   [0% ...]     X-Axis Region Start Coords."
   echo " -xn     xnList   [100% ...]   X-Axis Region End Coords"
   echo " -dx     dxList   [0.05ppm]    Baseline Roll-off Width, Pts, %, PPM, Hz."
   echo " -prefix prefix   [base"'$''$'"_tmp] Prefix for Temporary Files ($$ is Process ID)."
   echo "Correction by Polynomial Subtraction (Keyword None for No Correction):"
   echo " -ord    ordList  [None ...]   Polynomial Order."
   echo "Correction by Time-Domain Replacement (0 for No Correction):"
   echo " -tdN    tdNList  [5 ...]      Number of Time-Domain Points to Replace."
   echo " -tdArgs tdArgs   [None]       Extra Arguments for tdBaseline.com script."
   echo "Notes:"
   echo " 1. Option -tdArgs should be last on command line."
   echo " 2. The amount of correction applied to each region is in the shape of a plateau"
   echo "    with cosine-square roll-off. So, for a given correction, with coords in points:"
   echo "    no correction will be applied from point 1 to the point x1 - dx, the amount"
   echo "    of correction will increase until point x1, full correction will be applied"
   echo "    until point xn, and the amount of correction will decrease until it is zero"
   echo "    again at point xn + dx."
   echo " 2. If a roll-off region passes the limits of the spectrum, no roll-off will be"
   echo "    applied."
   exit 0
endif

@ cleanFlag = 1
@ vFlag     = 1

set defPrefix   = base$$_tmp
set tdArgs      = None

set prefix      = (`getArgD $argv -prefix $defPrefix`)
set inName      = (`getArgD $argv -in     test.ft2`)
set outName     = (`getArgD $argv -out    base.ft2`)

set maskNameOut = (`getArgD $argv -mask   None`)
set difNameOut  = (`getArgD $argv -dif    None`)

set x1List      = (`getListArgD $argv -x1  -- 0%`)
set xnList      = (`getListArgD $argv -xn  -- 100%`)

set ordListOrig = (`getListArgD $argv -ord -- None`)
set tdNListOrig = (`getListArgD $argv -tdN -- 0`)
set dxListOrig  = (`getListArgD $argv -dx  -- 0.05ppm`)

if (`flagLoc $argv -clean`) then
   @ cleanFlag = 1
endif

if (`flagLoc $argv -noclean`) then
   @ cleanFlag = 0
endif

if (`flagLoc $argv -verb`) then
   @ vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   @ vFlag = 0
endif

set loc = (`flagLoc $argv -tdArgs`)

if ($loc) then
   @ loc++
   set tdArgs = $argv[$loc-]
endif

if (`string csame $tdArgs None`) then
   set tdArgs = ""
endif

set regCount = $#x1List

if ($regCount != $#xnList) then
   echo "baseRegion Error: -x1 -xn Mismatch."
   exit 0
endif

#
#------------------------------------------------------------------------------------

set ordList = (`extendList.com -n $regCount -list $ordListOrig`)
set tdNList = (`extendList.com -n $regCount -list $tdNListOrig`)
set dxList  = (`extendList.com -n $regCount -list $dxListOrig`)

@ corNeeded = 0
@ i = 1

while ($i <= $regCount)
   if (`string csame $ordList[$i] None` == 0) then
      @ corNeeded = 1
      break
   endif

   if ($tdNList[$i] != 0) then
      @ corNeeded = 1
      break
   endif
 
   @ i++
end

if ($vFlag) then
   echo ""
   echo "baseRegions. Input:  $inName"
   echo "baseRegions. Output: $outName"
   echo ""
endif

if ($corNeeded == 0) then
   if ($vFlag) then
      echo "baseRegions. Status: No Corrections Needed."
      echo ""
   endif

   /bin/cp $inName $outName
   exit 0
endif

set maskName  = ${prefix}_mask%03d.dat
set corName   = ${prefix}_cor%03d.dat
set tmpName   = ${prefix}_tmp.dat
set tpName    = ${prefix}_tp.dat

set cleanList  = ($tpName $tmpName)
set maskList   = ""

set corNamePrev = None
set ordPrev     = NULL
set tdNPrev     = NULL

/bin/cp $inName $outName

nmrPipe -in $inName -fn TP -out $tpName  -ov

set xSize = (`getParm -in $inName -parm NDSIZE -dim CUR_XDIM -fmt %.0f`)

while ($i <= $regCount)
   set thisMaskName  = (`nmrPrintf $maskName  $i`)
   set thisCorName   = (`nmrPrintf $corName   $i`)
   set maskList      = ($maskList $thisMaskName)

   set x1Pts = (`spec2pnt  $inName CUR_XDIM $x1List[$i]`)
   set xnPts = (`spec2pnt  $inName CUR_XDIM $xnList[$i]`)
   set dxPts = (`spec2pntW $inName CUR_XDIM $dxList[$i]`)

   if (`IMATH "$x1Pts > $xnPts"`) then
      set tmp   = $x1Pts
      set x1Pts = $xnPts
      set xnPts = $tmp
   endif

   set x1Pts = (`IMATH "$x1Pts + 0.5"`)
   set xnPts = (`IMATH "$xnPts + 0.5"`)
   set dxPts = (`IMATH "$dxPts + 0.5"`)

   set xfPts = (`IMATH "$x1Pts - $dxPts"`)
   set xlPts = (`IMATH "$xnPts + $dxPts"`)

   if ($vFlag) then
      echo "baseRegions. Region $i X-Axis Region:  $x1List[$i] $xnList[$i]"
      echo "baseRegions. Region $i Roll-Off Width: $dxList[$i]"
      echo "baseRegions. Region $i Profile Coord:  $xfPts $x1Pts $xnPts $xlPts (Size $xSize)"
      echo ""
      echo "baseRegions. Region $i Time-Domain Points to Replace: $tdNList[$i]"
      echo "baseRegions. Region $i Polynomial Correction Order:   $ordList[$i]"
   endif

   if (`IMATH "$xfPts < 1"`) then
      set rollUp   = "NULL"
      set leftZero = "NULL"
      set x1Pts    = 1
   else
      set rollUp   = "SP -off 0.0 -end 0.5 -pow 2 -size $dxPts -start $xfPts -one"
      set leftZero = "SET -c 0.0 -xn $xfPts"
   endif

   if (`IMATH "$xlPts > $xSize"`) then
      set rollDown  = "NULL"
      set rightZero = "NULL"
      set xnPts     = $xSize 
   else
      set rollDown  = "SP -off 0.5 -end 1.0 -pow 2 -size $dxPts -start $xnPts -one"
      set rightZero = "SET -c 0.0 -x1 $xlPts"
   endif

   nmrPipe -in $inName \
   | nmrPipe -fn SET -c 1.0 \
   | nmrPipe -fn $rollUp    \
   | nmrPipe -fn $rollDown  \
   | nmrPipe -fn $leftZero  \
   | nmrPipe -fn $rightZero \
     -out $thisMaskName -ov

   if (`string csame $ordList[$i] $ordPrev` && `string csame $tdNList[$i] $tdNPrev`) then
      echo "baseRegions. Region $i Using Corrected Data from Previous Region."
      /bin/cp $corNamePrev $thisCorName
   else
      if (`string csame $ordList[$i] None`) then
         tdBaseline.com -in $tpName  -out $thisCorName -n $tdNList[1] $tdArgs
      else
         if ($tdNList[$i] == 0) then
            echo ""
            nmrPipe -in $tpName -fn POLY -auto -ord $ordList[$i] -out $thisCorName -verb -ov
         else
            echo ""
            nmrPipe -in $tpName -fn POLY -auto -ord $ordList[$i] -out $tmpName -verb -ov
            tdBaseline.com -in $tmpName -out $thisCorName -n $tdNList[1] $tdArgs
         endif
      endif

      nmrPipe -in $thisCorName -fn TP -out $thisCorName -inPlace
   endif

   addNMR -in1 $inName  -in2 $thisCorName  -out $tmpName     -sub
   addNMR -in1 $tmpName -in2 $thisMaskName -out $thisCorName -mult
   addNMR -in1 $outName -in2 $thisCorName  -out $outName     -sub

   if ($cleanFlag) then
      set cleanList = ($cleanList $thisMaskName $thisCorName)
   endif

   set corNamePrev = $thisCorName
   set ordPrev     = $ordList[$i]
   set tdNPrev     = $tdNList[$i]

   @ i++
end

if (`string csame $maskNameOut None` == 0) then
   sumNMR.com -in $maskList -out $maskNameOut
endif

if (`string csame $difNameOut None` == 0) then
   addNMR -in1 $inName -in2 $outName -out $difNameOut -sub 
endif

/bin/rm -f $cleanList

exit 0
