#!/bin/csh

set thisVersion = "2024.061.14.35"
set progName    = "basicFT1.com"

if (`flagLoc $argv -help`) then
   echo "basicFT1.com $thisVersion"
   echo "Perform Basic Processing for 1D Data:"
   echo " -in      inName  [test.fid]  NMRPipe-Format Time-Domain Input."
   echo " -out     outName [test.ft1]  Spectrum Output."
   echo " -script  sName   [nmr.com]   Processing Script Output."
   echo " -tmp     tName   [tmp.dat]   Intermediate Result if Needed."
   echo " -clean                       Delete Intermediate Results (Default)."
   echo " -noclean                     Retain Intermediate Results."
   echo "Other Options:"
   echo " -parent  pName   [None]      Aux Time-Domain Input for Parameter Extraction."
   echo " -title   title   [None]      Optional Title for Output."
   echo " -scaleTo maxVal  [None]      Scale to Given Max Value, or None for No Scaling."
   echo " -noexec                      Create Processing Script, Do Not Process Data."
   echo " -list                        Print Script Without Saving or Processing."
   echo " -append                      Append to Existing Script."
   echo " -noappend                    Create New Script (Default)."
   echo " -ssNMR                       Set Defaults for Solid-State NMR Data."
   echo " -ioargs                      Add -in/-out Arguments to the Processing Script."
   echo "Processing Parameters (Use -x...):"
   echo " -xSOL    sName   [NULL]      Solvent Function Name and Options."
   echo " -xTDP1   tdP1    [0.0]       Time-Domain First Order Phase Correction."
   echo " -xAPOD   aName   [SP]        Window Function Name."
   echo " -xQ1     q1      [0.5]       Window Parameter Q1."
   echo " -xQ2     q2      [0.5]       Window Parameter Q2."
   echo " -xQ3     q3      [1]         Window Parameter Q3."
   echo " -xELB    elb     [1.0]       Exponential Line Broaden, Hz."
   echo " -xGLB    glb     [0.0]       Gaussian Line Broaden, Hz."
   echo " -xGOFF   goff    [0.0]       Gaussian Center, 0.0 to 1.0."
   echo " -xC1     c       [Auto]      First-Point Time-Domain Scale (0.5 1.0 Auto)."
   echo " -xP0     p0      [0.0]       Zero-Order Phase Correction."
   echo " -xP1     p1      [0.0]       First Order Phase Correction."
   echo " -xZFARG  zfArgs  [zf=1,auto] ZF (Zero Fill) Option."
   echo " -xFTARG  ftArg   [None]      FT (Fourier Transform) Options (None neg alt)."
   echo " -xEXTX1  x1      [0%]        Extract Range Start (% Hz ppm pts)."
   echo " -xEXTXN  xn      [100%]      Extract Range End (% Hz ppm pts)."
   echo " -xPREFT  fnArgs  [NULL]      Function to Execute Before Any Other Processing."
   echo " -xPOSTFT fnArgs  [NULL]      Function to Execute After FT and Phase Correction."
   echo " -xMC                         Use Magnitude Calculation."
   echo " -apodDF                      Adjust Window Function for Digital Oversampling."
   echo " -nodi                        Do Not Delete Imaginary Data After Fourier Transform."
   echo "Baseline Correction Function Name and Options:"
   echo " -xBASEARG        [NULL]      Spectrum Baseline Correction."
   echo " -xBASEARG1       [NULL]      Same as -xBASEARG"
   echo " -xBASEARG2       [NULL]      Secondary Baseline Correction."
   echo "Auto Phasing Options (Currently Only For X-Axis Zero Order Phase, -xP0 Auto):"
   echo " -apOrd    ord    [0]         Auto-Phase Baseline Order (-1 for No Correction)."
   echo " -apWindow w      [2%]        Auto-Phase Baseline Window (Pts Hz ppm %)."
   echo " -apxELB   elb    [xELB]      Additional Line Broadening, Hz."
   echo " -apArgs   apArgs [None]      Other Options for basicAutoPhase.com Utility."
   echo "Notes:"
   echo " 1. Options -xSOL and -xBASEARG are expanded into NMRPipe processing function"
   echo "    names and command line arguments. Options should contain no spaces. Flags"
   echo "    are separated by commas, and options with values are given using = signs."
   echo "    For example:"
   echo ""
   echo "      -xSOL     NULL              Becomes: nmrPipe -fn NULL (No Solvent Filter)"
   echo "      -xSOL     SOL               Becomes: nmrPipe -fn SOL"
   echo "      -xSOL     POLY,time         Becomes: nmrPipe -fn POLY -time"
   echo "      -xBASEARG POLY,auto,ord=0   Becomes: nmrPipe -fn POLY -auto -ord 0"
   echo ""
   echo " 2. Options -xZFARG -xFTARG are expanded to command line arguments for ZF"
   echo "    and FT, for example:"
   echo ""
   echo "      -xZFARG size=2048,auto  Becomes: nmrPipe -fn ZF -size 2048 -auto"
   echo "      -xFTARG neg             Becomes: nmrPipe -fn FT -neg"
   echo ""
   echo " 3. When first-point time-domain scaling is set to Auto (-xC1 Auto etc),"
   echo "    the scaling value will be set to 0.5 if the first-order phase correction"
   echo "    for that dimension is 0+/-10 degrees, or to 1.0 otherwise."
   echo ""
   echo " 4. If the X-Axis label of the input spectrum starts with 'HN', default"
   echo "    parameters will be adjusted as follows:"
   echo ""
   echo "       -xSOL SOL -xBASEARG POLY,auto -xEXTX1 3% -xEXTXN 47%"
   echo ""
   echo " 5. If the -ssNMR flag is used, default parameters will be:"
   echo ""
   echo "      -xSOL NULL -xBASEARG NULL"
   echo ""
   echo " 6. Auto phasing is available only for zero-order phase correction in the"
   echo "    direct-detected dimension (-xP0 Auto). The auto-phasing, which is performed"
   echo "    via the utility basicAutoPhase.com, might not be ideal for all cases. Auto"
   echo "    phasing parameters for this utility can be adjusted via the -apArgs option."
   echo "    For example, to use the X-Axis region 9.0ppm to 7.0ppm for evaluating"
   echo "    phase correction:"
   echo ""
   echo "      $progName \"
   echo "       -xP0 Auto -xP1 0 -xEXTX1 10.6ppm -xEXTXN 5.5ppm \"
   echo "       -apArgs apx1=9.0ppm,apxn=7.0ppm"
   exit 0
endif

set inName     = (`getArgD $argv -in      test.fid`)
set parentName = (`getArgD $argv -parent  None`)
set outName    = (`getArgD $argv -out     test.ft1`)
set scriptName = (`getArgD $argv -script  nmr.com`)
set tmpName    = (`getArgD $argv -tmp     tmp.dat`)
set titStr     = (`getArgD $argv -title   None`)
set scaleToVal = (`getArgD $argv -scaleTo None`)

set origName   = $inName
set vFlag      = verb
set apodDF     = ""
set xMCFlag    = 0
set ioArgsFlag = 0

set vFlag      = (`getLogArgD $argv -verb     -noverb   verb noverb`)
set listFlag   = (`getLogArgD $argv -nolist   -list     0    1`)
set cleanFlag  = (`getLogArgD $argv -clean    -noclean  1    0`)
set ssNMRFlag  = (`getLogArgD $argv -nossNMR  -ssNMR    0    1`)
set appendFlag = (`getLogArgD $argv -noappend -append   0    1`)
set execFlag   = (`getLogArgD $argv -exec     -noexec   1    0`)

if (`flagLoc $argv -apodDF`) then
   set apodDF = "-df"
endif

if (`flagLoc $argv -noapodDF`) then
   set apodDF = ""
endif

if (`flagLoc $argv -xMC`) then
   set xMCFlag = 1
endif

if (`flagLoc $argv -ioargs`) then
   set ioArgsFlag = 1
endif

if (`flagLoc $argv -noioargs`) then
   set ioArgsFlag = 0
endif

if (`string csame $parentName None`) then
   set parentName = $origName
   set thisName   = $origName
else
   set thisName   = $parentName
endif

if (!(-e $thisName)) then
   echo "basicFT1.com Notice: Missing Input $inName (parent $parentName)."
   exit 1
endif

set thisDir = (`string path $outName`)

if (!(-d $thisDir)) then
   if (`string same $vFlag verb`) then
      echo "$progName New Directory: $thisDir"
      echo ""
   endif

   mkdir $thisDir
endif

if (`string same $vFlag verb`) then
   if ($listFlag == 0) then
      echo "$progName Version:       $thisVersion"
      echo "$progName Input:         $inName"
      echo "$progName FT Output:     $outName"
      echo "$progName Script Output: $scriptName"
      echo ""
   endif
endif

set xLAB = (`getParm -in $thisName -txt -fmt %s -parm NDLABEL -dim CUR_XDIM`)

if (`string match "HN*" $xLAB`) then
   set uxSOL     = "SOL"
   set uxBASEARG = "POLY,auto" 
   set uxEXTX1   = "3%"
   set uxEXTXN   = "47%"
else
   set uxSOL     = "NULL"
   set uxBASEARG = "NULL"
   set uxEXTX1   = "0%"
   set uxEXTXN   = "100%"
endif

if ($ssNMRFlag) then
   set uxSOL     = "NULL"
   set uxBASEARG = "NULL"
endif

set xSOL    = (`getArgD $argv -xSOL    $uxSOL`)
set xAPOD   = (`getArgD $argv -xAPOD   SP`)
set xQ1     = (`getArgD $argv -xQ1     0.5`)
set xQ2     = (`getArgD $argv -xQ2     0.5`)
set xQ3     = (`getArgD $argv -xQ3     1`)
set xELB    = (`getArgD $argv -xELB    1.0`)
set xGLB    = (`getArgD $argv -xGLB    0.0`)
set xGOFF   = (`getArgD $argv -xGOFF   0.0`)
set xC1     = (`getArgD $argv -xC1     Auto`)
set xZFARG  = (`getArgD $argv -xZFARG  zf=1,auto`)
set xFTARG  = (`getArgD $argv -xFTARG  None`)
set xP0     = (`getArgD $argv -xP0     0.0`)
set xP1     = (`getArgD $argv -xP1     0.0`)
set xTDP1   = (`getArgD $argv -xTDP1   0.0`)
set xEXTX1  = (`getArgD $argv -xEXTX1  $uxEXTX1`)
set xEXTXN  = (`getArgD $argv -xEXTXN  $uxEXTXN`)
set xPREFT  = (`getArgD $argv -xPREFT  NULL`)
set xPOSTFT = (`getArgD $argv -xPOSTFT NULL`)

set apArgs   = (`getArgD $argv -apArgs   None`)
set apOrd    = (`getArgD $argv -apOrd    0`)
set apWindow = (`getArgD $argv -apWindow 2%`)
set apxELB   = (`getArgD $argv -apxELB   $xELB`)

set xBASEARG_PHASE = (`getArgD $argv -xBASEARG_PHASE NULL`) 

set xBASEARG1 = (`getArgD $argv -xBASEARG  $uxBASEARG`)
set uxBASEARG = $xBASEARG1

set xBASEARG1 = (`getArgD $argv -xBASEARG1 $uxBASEARG`)
set xBASEARG2 = (`getArgD $argv -xBASEARG2 NULL`)

if (`string csame $xZFARG None`) then
   set xZFARG = "zf=0"
endif

if (`string csame $xC1 Auto`) then
   set v = (`nmrPrintf "%f" $xP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set xC1 = 1.0
   else
      set xC1 = 0.5
   endif
endif

set cleanList = ""

set inNameStr  = $inName
set outNameStr = $outName

if ($listFlag) then
   if ($appendFlag == 0) then
      echo "#\!/bin/csh"
   endif

   echo ""
   echo "#"
   echo "# Generated by $progName Version $thisVersion"

   echo "#"
   prettyPrintCmnd.tcl $0 `stripArgs -list -noexec -append -noappend -parent= -- $argv`
   echo ""

  if ($ioArgsFlag) then
      echo 'set inName  = (`getArgD $argv -in  '$inName'`)'
      echo 'set outName = (`getArgD $argv -out '$outName'`)'
      echo ""

      set inNameStr  = '$inName'
      set outNameStr = '$outName'
   endif

   if (`string csame $xP0 Auto`) then
      if (`string csame $apArgs None`) then
         set apArgInfo = ""
      else
         set apArgInfo = (`expandArg $apArgs`)
      endif

      echo "#"
      echo "# Auto-phase determination; arguments duplicated later on the"
      echo "# basicAutoPhase.com command line ignored:"

      if (`string same $vFlag verb`) then
         echo ""
         nmrPrintf 'nmrPrintf "Auto-Phase ...\\n\\n"'
         echo ""
      endif

      if (`string csame $apOrd None`) then
         set apOrd = -1
      endif

      echo ""
      echo "set xP0 = ("\`basicAutoPhase.com -in $inNameStr $apArgInfo -apxELB $apxELB -apxP1 $xP1 -apOrd $apOrd -apWindow $apWindow\`")"
      echo ""

      if (`string same $vFlag verb`) then
         nmrPrintf 'nmrPrintf "Auto-Phase xP0: %%.1f\\n\\n" $xP0'
         echo ""
         echo ""
      endif

      set xP0 = '$xP0'
   endif

   echo "nmrPipe -in $inNameStr \"

   if (`string notcsame $xPREFT NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $xPREFT` "\"
   endif

   if (`string notcsame $xTDP1 0.0`) then
      echo "| nmrPipe -fn" PS -p1 $xTDP1 -noup "\"
   endif

   if (`string notcsame $xSOL NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $xSOL` "\"
   endif

   if (`string csame $xAPOD SP`) then
      if (`string same $xGOFF 0.0`) then
         echo "| nmrPipe -fn" SP -off $xQ1 -end $xQ2 -pow $xQ3 -elb $xELB -glb $xGLB -c $xC1 $apodDF "\"
      else
         echo "| nmrPipe -fn" SP -off $xQ1 -end $xQ2 -pow $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 $apodDF "\"
      endif
   else
      echo "| nmrPipe -fn" APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 $apodDF "\"
   endif

   echo "| nmrPipe -fn" ZF `expandArg $xZFARG` "\"
   echo "| nmrPipe -fn" FT `expandArg $xFTARG` "\"

   if ($xMCFlag) then
      echo "| nmrPipe -fn MC \"
   else
      if (`flagLoc $argv -nodi`) then
         echo "| nmrPipe -fn" PS -p0 $xP0 -p1 $xP1 "\"
      else
         echo "| nmrPipe -fn" PS -p0 $xP0 -p1 $xP1 -di "\"
      endif
   endif

   if (`string notcsame $xEXTX1 0%` || `string notcsame $xEXTXN 100%`) then
      echo "| nmrPipe -fn" EXT -x1 $xEXTX1 -xn $xEXTXN -sw "\"
   endif

   if (`string notcsame $xBASEARG1 NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $xBASEARG1` "\"
   endif

   if (`string notcsame $xBASEARG2 NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $xBASEARG2` "\"
   endif

   if (`string notcsame $xPOSTFT NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $xPOSTFT` "\"
   endif

   echo "  -out $outNameStr -ov"

   if (`string notcsame $scaleToVal None`) then
      echo ""
      echo rescale.com -in $outNameStr -out $outNameStr -max $scaleToVal
   endif


   if (`string casecmp $titStr None`) then
      echo ""
      echo "sethdr.com $outNameStr -title $titStr"
   endif

   set cleanCount = (`string length $cleanList`)

   if ($cleanFlag && $cleanCount) then
      echo ""
      echo "foreach i ("$cleanList")"
      echo '   if (-e $i) then'
      echo '      /bin/rm -f $i'
      echo '   endif'
      echo "end"
   endif
else
   if ($appendFlag) then
      if (!(-e $scriptName)) then
         echo "#!/bin/csh" >  $scriptName
         echo ""           >> $scriptName
      endif

      basicFT1.com $* -list >> $scriptName
   else
      basicFT1.com $* -list >  $scriptName
   endif

   chmod a+rx $scriptName

   if ($execFlag) then
      $scriptName

      if (`string same $vFlag verb`) then
         echo ""
      endif
   endif
endif

exit 0
