#!/bin/csh

set thisVersion = "2024.061.14.35"

set progName    = (`getArgD    $argv -progName basicFT2.com`)
set nusFlag     = (`getLogArgD $argv -us       -nus      0    1`)
set retainFlag  = (`getLogArgD $argv -noretain -retain   0    1`)
set appendFlag  = (`getLogArgD $argv -noappend -append   0    1`)

set nusZFFlag = 0

if (`flagLoc $argv -nusZF`) then
   set nusZFFlag = 1
endif

if ($nusZFFlag) then
   set reconMode = "ist"
else
   set reconMode = "ft"
endif

foreach i (ist rist smile nesta hmsIST maxent)
   if (`flagLoc $argv -$i`) then
      set reconMode = $i
      set nusZFFlag = 1
   endif
end

if (`string csame $reconMode maxent`) then
   set reconMode  = "ft"
   set nusZFFlag  = 0
endif

if (`string csame $reconMode rist`) then
   set reconMode  = "ist"
   set retainFlag = 1
endif

if (`flagLoc $argv -nonusZF`) then
   set nusZFFlag = 0
   
endif

if ($nusFlag == 0 && $nusZFFlag == 0) then
   set reconMode = "ft"
endif

if ($nusZFFlag) then
   set yNUSZFDef    = "zf=1"
else
   set yNUSZFDef    = "zf=0"
endif

set smileProg   = nusPipe
set nestaProg   = nesta_wrapper.com
set hmsISTProg  = hmsIST

set extProg     = ext_xy.com
set srcProg     = nmrPipe
set destProg    = nmrPipe
set smileProg   = nusPipe

@ skipCount = 0

if (`string csame $reconMode smile`) then
   if (`cmndExists.com $smileProg` == 0) then
      echo "$progName Error Finding SMILE Program $smileProg"
      exit 1
   endif
else if (`string csame $reconMode hmsIST`) then
   if (`cmndExists.com $hmsISTProg` == 0) then
      echo "$progName Error Finding hmsIST Program $hmsISTProg"
      exit 1
   endif
else if (`string csame $reconMode nesta`) then
   if (`cmndExists.com $nestaProg` == 0) then
      echo "$progName Error Finding NESTA Program $nestaProg"
      exit 1
   endif
endif

if (`flagLoc $argv -help`) then
   if (`string csame $reconMode smile`) then
      echo "$progName (Using basicFT2.com) $thisVersion"
      echo "Perform SMILE Processing for 2D NUS Data (Requires SMILE):"
      echo " https://spin.niddk.nih.gov/bax/software/smile"
      echo "Options:"
   else if (`string csame $reconMode hmsIST`) then
      echo "$progName (Using basicFT2.com) $thisVersion"
      echo "Perform hmsIST Processing for 2D NUS Data (Requires hmsIST):"
      echo " http://gwagner.med.harvard.edu/intranet/istHMS"
      echo "Options:"
   else if (`string csame $reconMode nesta`) then
      echo "$progName (Using basicFT2.com) $thisVersion"
      echo "Perform NESTA Processing for 2D NUS Data (Requires nestanmr):"
      echo " http://nestanmr.com"
      echo "Options:"
   else
      echo "$progName $thisVersion"
      echo "Perform Basic Processing for 2D Phase-Sensitive Data:"
   endif

   echo " -in        inName  [Auto]    Phase-Sensitive Time-Domain Input."
   echo " -out       outName [Auto]    Spectrum Output."
   echo " -ft1       ft1Name [None]    Optional Output for Interferogram Result."
   echo " -script    sName   [nmr.com] Processing Script Output."

   if (`string csame $reconMode smile`) then
      echo "SMILE Options (See Also: $smileProg -help -fn SMILE):"
      echo " -sample    sName   [Auto]    Sample Schedule Input Name."
      echo " -nusZF                       NUS Zero-Fill Extrapolation ON (Default)."
      echo " -nonusZF                     NUS Zero-Fill Extrapolation OFF."
      echo " -yNUSZF    zfArgs  [zf=1]    Amount of Y-Axis NUS Zero-Fill Extrapolation."
      echo " -smileArgs sArgs   [None]    Additional SMILE Argument Info, or Keyword None."
      echo "                              https://spin.niddk.nih.gov/bax/software/smile"
      echo "                              See: $smileProg -fn SMILE -help for More."
   else if (`string csame $reconMode hmsIST`) then
      echo "hmsIST Options (See Also: $hmsISTProg -help):"
      echo " -nusZF                       NUS Zero-Fill Extrapolation ON (Default)."
      echo " -nonusZF                     NUS Zero-Fill Extrapolation OFF."
      echo " -yNUSZF    zfArgs  [zf=1]    Amount of Y-Axis NUS Zero-Fill Extrapolation."
      echo " -hmsIter   iter    [-1]      Number of hmsIST Iterations, -1 for Auto."
      echo " -hmsArgs   hArgs   [...]     Additional hmsIST Argument Info, or Keyword None."
      echo "                              http://gwagner.med.harvard.edu/intranet/istHMS"
      echo "                              See: $hmsISTProg -help for More."
   else if (`string csame $reconMode NESTA`) then
      echo "NESTA Options (See Also: $nestaProg -help):"
      echo " -sample      sName   [Auto]  Sample Schedule Input Name."
      echo " -nusZF                       NUS Zero Fill Mode ON."
      echo " -yNUSZF      zfArgs  [zf=1]  Amount of Y-Axis NUS Zero-Fill Extrapolation."
      echo " -nThread     n       [Auto]  Number of Parallel Threads (NESTA --threads)."
      echo " -L1                          L1 Regularization (NESTA --method 1, Default)."
      echo " -IRL1                        IRL1 Regularization (NESTA --rwiter > 1)."
      echo " -GAUSS                       Gaussian-SL0 Regularization (NESTA --method 0)."
      echo " -nestaIter   iter    [Auto]  Number of Iterations (NESTA --iter)."
      echo " -nestaRWIter rwiter  [Auto]  Number of Reweighting Iterations (NESTA --rwiter)."
      echo " -nestaCutoff cuttoff [0.10]  Cutoff for Gaussian-SL0 (NESTA --cutoff)."
      echo " -nestaScale  scale   [0.98]  Scaling for Gaussian-SL0 (NESTA --scaling)."
      echo " -nestaArgs   sArgs   [...]   Additional NESTA Arguments, or Keyword None."
   endif

   echo "Other Options:"
   echo " -parent    pName  [None]     Aux Time-Domain Input for Parameter Extraction."
   echo " -title     title  [None]     Optional Title for Output."
   echo " -scaleTo   maxVal [None]     Scale to Given Max Value, or None for No Scaling."
   echo " -noexec                      Create Processing Script, Do Not Process Data."
   echo " -list                        Print Script Without Saving or Processing."
   echo " -append                      Append to Existing Script."
   echo " -noappend                    Create New Script (Default)."
   echo " -ssNMR                       Set Defaults for Solid-State NMR Data."
   echo "Intermediate Results:"
   echo " -tmpPrefix prefix [Auto]     Prefix for Temporary Files."
   echo " -clean                       Delete Intermediate Results (Default)."
   echo " -noclean                     Retain Intermediate Results."
   echo "Processing Parameters (Use -x... -y...):"
   echo " -xSOL    sName   [NULL]      Solvent Function Name and Options (X-Axis Only)."
   echo " -xAPOD   aName   [SP]        Window Function Name."
   echo " -xQ1     q1      [0.50]      Window Parameter Q1 (SP -off)."
   echo " -xQ2     q2      [0.95]      Window Parameter Q2 (SP -end)."
   echo " -xQ3     q3      [2]         Window Parameter Q3 (SP -pow, Default -yQ3 is 1)."
   echo " -xELB    elb     [0.0]       Exponential Line Broaden, Hz."
   echo " -xGLB    glb     [0.0]       Gaussian Line Broaden, Hz."
   echo " -xGOFF   goff    [0.0]       Gaussian Center Position, 0.0 to 1.0."
   echo " -xC1     c       [Auto]      First-Point Time-Domain Scale (0.5 1.0 Auto)."
   echo " -xP0     p0      [0.0]       Zero-Order Phase Correction (Or -xP0 Auto)."
   echo " -xP1     p1      [0.0]       First Order Phase Correction."
   echo " -xZFARG  zfArgs  [zf=1,auto] Zero Fill (ZF) Options."
   echo " -xFTARG  ftArg   [None]      Fourier Transform (FT) Options (None neg alt)."
   echo " -xEXTX1  x1      [0%]        Extract Range Start (X-Axis Only) (% Hz ppm pts)."
   echo " -xEXTXN  xn      [100%]      Extract Range End   (X-Axis Only) (% Hz ppm pts)."
   echo " -xPREFT  fnArgs  [NULL]      Function to Execute Before Any Other Processing."
   echo " -xPOSTFT fnArgs  [NULL]      Function to Execute After FT and Phase Correction."
   echo " -apodDF                      Adjust Window Function for Digital Oversampling."
   echo "Auto Phasing Options (Currently Only For X-Axis Zero Order Phase, -xP0 Auto):"
   echo " -apOrd   ord     [0]         Auto-Phase Baseline Order (-1 for No Correction)."
   echo " -apArgs  apArgs  [None]      Other Options for basicAutoPhase.com Utility."
   echo "Magntude-Mode Options:"     
   echo " -noxDI                       Do Not Delete X-Axis Imaginary Data."
   echo " -noyDI                       Do Not Delete Y-Axis Imaginary Data."
   echo " -xMC                         Use X-Axis Magnitude Calculation."
   echo " -yMC                         Use Y-Axis Magnitude Calculation."
   echo " -mag                         Same As: -noxDI -yMC xQ1 0.1 -yQ1 0.1 -yQ3 2."
   echo "Baseline Correction Function Name and Options (Use -x... -y...):"
   echo " -xBASEARG_FT1  bArgs [NULL]  Interferogram Baseline Correction (X-Axis Only)."
   echo " -xBASEARG      bArgs [NULL]  Spectrum Baseline Correction."
   echo "Processing Parameters for X-Axis Only:"
   echo " -xBASEARG_INIT bArgs [NULL]  Baseline Correction Before Region Extraction."
   echo " -xP0EXT        p0Ext [0.0]   Zero-Order Phase After Region Extraction."
   echo " -xP1EXT        p1Ext [0.0]   First Order Phase After Region Extraction."

   if ($nusFlag == 0) then
      echo "Linear Prediction Extrapolation (Indirect Dimension Only, Not Used for -nusZF):"
      echo " -yLP    lpArgs   [NULL]      Linear Prediction Arguments."
      echo "NUS Zero-Fill (NUS-style Extrapolation of Conventional Uniformly Sampled Data):"
      echo " -nusZF                       NUS Zero Fill Extrapolation ON."
      echo " -nonusZF                     NUS Zero Fill Extrapolation OFF (Default)."
      echo " -yNUSZF zfArgs   [zf=1]      Amount of Y-Axis NUS Zero-Fill Extrapolation."
      echo "Extrapolation by NMRPipe IST (See Also: ist2D.com -help):"
      echo " -ist                         Use NMRPipe IST Reconstruction (Default)."
      echo " -retain                      Retain Original Time-Domain Data Points."
      echo " -rist                        Same as -ist -retain Option."
      echo "Extrapolation by SMILE (See Also: $smileProg -fn SMILE -help):"
      echo " -smile                       Use SMILE Reconstruction; Requires SMILE:"
      echo "                              https://spin.niddk.nih.gov/bax/software/smile"
      echo " -smileArgs sArgs [None]      SMILE Argument Info, or Keyword None;"
      echo "Extrapolation by hmsIST (See Also: $hmsISTProg -help):"
      echo " -hmsIST                      Use hmsIST Reconstruction; Requires hmsIST:"
      echo "                              http://gwagner.med.harvard.edu/intranet/istHMS" 
      echo " -hmsIter  iter       [-1]    Number of hmsIST Iterations, -1 for Auto."
      echo " -hmsArgs  hArgs      [None]  hmsIST Argument Info, or Keyword None;"
      echo "Extrapolation by NESTA (See Also: $nestaProg -help):"
      echo " -sample      sName   [Auto]  Sample Schedule Input Name."
      echo " -nThread     n       [Auto]  Number of Parallel Threads (NESTA --threads)."
      echo " -L1                          L1 Regularization (NESTA --method 1, Default)."
      echo " -IRL1                        IRL1 Regularization (NESTA --rwiter > 1)."
      echo " -GAUSS                       Gaussian-SL0 Regularization (NESTA --method 0)."
      echo " -nestaIter   iter    [Auto]  Number of Iterations (NESTA --iter)."
      echo " -nestaRWIter rwiter  [Auto]  Number of Reweighting Iterations (NESTA --rwiter)."
      echo " -nestaCutoff cuttoff [0.10]  Cutoff for Gaussian-SL0 (NESTA --cutoff)."
      echo " -nestaScale  scale   [0.98]  Scaling for Gaussian-SL0 (NESTA --scaling)."
      echo " -nestaArgs   sArgs   [...]   Additional NESTA Arguments, or Keyword None."
   endif

   echo "Options for 2D Planes from 3D and 4D Data:"
   echo " -yPlane yPlane   [1]         Y-Coord of Plane to Extract (3D-4D, -xz or -xa)."
   echo " -zPlane zPlane   [1]         Z-Coord of Plane to Extract (3D-4D, -xy or -xa)."
   echo " -aPlane aPlane   [1]         A-Coord of Plane to Extract (4D Only, -xy or -xz)."
   echo " -xy                          Extract and Process XY Plane (Default for 3D-4D)."
   echo " -xz                          Extract and Process XZ Plane."
   echo " -xa                          Extract and Process XA Plane (4D Only)."
   echo " -pseudo3D                    Process Data as Pseudo-3D 2D Series."

   echo ""
   echo "Notes:"
   echo " 1. Options -xSOL -xBASEARG_INIT -xBASEARG_FT1 and -xBASEARG are expanded into"
   echo "    NMRPipe processing function names and command line arguments. Options"
   echo "    should contain no spaces. Flags are separated by commas, and options"
   echo "    with values are given using = signs. For example, solvent filter:"
   echo ""
   echo "      -xSOL     NULL            Becomes: nmrPipe -fn NULL (No Solvent Filter)"
   echo "      -xSOL     SOL             Becomes: nmrPipe -fn SOL"
   echo "      -xSOL     POLY,time       Becomes: nmrPipe -fn POLY -time"
   echo ""
   echo "    For baseline correction:"
   echo ""
   echo "      -xBASEARG NULL            Becomes: nmrPipe -fn NULL (No Correction)"
   echo "      -xBASEARG POLY,auto,ord=0 Becomes: nmrPipe -fn POLY -auto -ord 0"
   echo ""
   echo " 2. Options -xZFARG -xFTARG are expanded to command line arguments for ZF"
   echo "    and FT, for example:"
   echo ""
   echo "      -xZFARG zf=2,auto         Becomes: nmrPipe -fn ZF -zf 2 -auto"
   echo "      -xFTARG neg               Becomes: nmrPipe -fn FT -neg"
   echo "      -xFTARG neg,alt           Becomes: nmrPipe -fn FT -neg -alt"
   echo ""
   echo " 3. When first-point time-domain scaling is set to Auto (-xC1 Auto etc),"
   echo "    the scaling value will be set to 0.5 if the first-order phase correction"
   echo "    for that dimension is 0+/-10 degrees, or to 1.0 otherwise." 
   echo ""
   echo " 4. Option -yLP is expanded to command line arguments for Linear Prediction:"
   echo ""
   echo "      -yLP NULL                 No Linear Prediction (Default)."
   echo "      -yLP fb,ord=12            Becomes: nmrPipe -fn LP -fb -ord 12"
   echo ""
   echo " 5. If the X-Axis label of the input spectrum starts with 'HN', the default"
   echo "    parameters will be adjusted as follows:"
   echo ""
   echo "      -xSOL SOL -xBASEARG POLY,auto -xEXTX1 3% -xEXTXN 47%"
   echo ""
   echo " 6. If the -ssNMR flag is used, default parameters will be:"
   echo ""
   echo "      -xSOL NULL -xBASEARG NULL"
   echo ""
   echo " 7. If Y-Axis P1 is 360 +/- 10 degrees, default Y-Axis baseline will be:"
   echo ""
   echo "      -yBASEARG POLY,auto,ord=0"
   echo ""
   echo " 8. When -pseudo3D mode if used, template output names will be used by"
   echo "    default, such as: ft/test%03d.ft2"
   echo ""
   echo " 9. Auto phasing is available only for zero-order phase correction in the"
   echo "    direct-detected dimension (-xP0 Auto). The auto-phasing, which is performed"
   echo "    via the utility basicAutoPhase.com, might not be ideal for all cases. Auto"
   echo "    phasing parameters for this utility can be adjusted via the -apArgs option."
   echo "    For example, to use the Y-Axis region 120.0 ppm to 114.0ppm for evaluating"
   echo "    phase correction:"
   echo ""
   echo "      $progName \"
   echo "       -xP0 Auto -xP1 0 -xEXTX1 10.6ppm -xEXTXN 5.5ppm \"
   echo "       -apArgs apy1=120.0ppm,apyn=114.0ppm"
   echo ""
   echo "    See 'basicAutoPhase.com -help' for more."
   echo ""

   if (`string csame $reconMode smile`) then
      echo "10. Commonly, only region of interest, FT, and phase correction options"
      echo "    need to be provided along with the sampling schedule name. Time-domain"
      echo "    extrapolation (-nusZF) is included by default. Typical example:"
      echo ""
      echo "      $progName -sample nuslist \"
      echo "       -xP0 -16 -xP1   0 -xEXTX1 2.6ppm -xEXTXN -0.8ppm \"
      echo "       -yP0 -90 -yP1 180 -yZFARG zf=2,auto -yFTARG alt"
      echo ""
      echo "11. Option -smileArgs is expanded to command line arguments for SMILE:"
      echo ""
      echo "      -smileArgs maxMem=2.0,thresh=0.9"
      echo ""
      echo "    Becomes: $smileProg -fn SMILE -maxMem 2.0 -thresh 0.9 ..."
      echo ""
      echo "    See '$smileProg -fn SMILE -help' for more."
      echo ""
   else if (`string csame $reconMode hmsIST`) then
      echo "10. Commonly, only region of interest, FT, and phase correction options"
      echo "    need to be provided; hmsIST deduces the NUS sampling schedule by"
      echo "    finding parts of the input data that are set to zero, so that no"
      echo "    sampling schedule input (-sample ...) or mask (-mask ...) is needed."
      echo "    Time-domain extrapolation (-nusZF) is included by default. Typical"
      echo "    example:"
      echo ""
      echo "      $progName \"
      echo "       -xP0 -16 -xP1   0 -xEXTX1 2.6ppm -xEXTXN -0.8ppm \"
      echo "       -yP0 -90 -yP1 180 -yZFARG zf=2,auto -yFTARG alt"
      echo ""
      echo "11. Option -hmsArgs is expanded to command line arguments for hmsIST:"
      echo ""
      echo "      -hmsArgs IST=D,thresh=0.95"
      echo ""
      echo "    Becomes: $hmsISTProg -IST D -thresh 0.95 ..."
      echo ""
      echo "    See '$hmsISTProg -help' for more."
      echo ""
   else if (`string csame $reconMode nesta`) then
      echo "10. Commonly, only region of interest, FT, and phase correction options"
      echo "    need to be provided along with the sampling schedule name. Time-domain" 
      echo "    extrapolation (-nusZF) is included by default. Typical example:"
      echo ""
      echo "      $progName -sample nuslist \"
      echo "       -xP0 -16 -xP1   0 -xEXTX1 2.6ppm -xEXTXN -0.8ppm \"
      echo "       -yP0 -90 -yP1 180 -yZFARG zf=2,auto -yFTARG alt"
      echo ""
      echo "11. Option -nestaArgs is expanded to -- command line arguments for NESTA:"
      echo ""
      echo "      -nestaArgs --method=1,--rwiter=5"
      echo ""
      echo "    Becomes: nestanmr --method 1 --rwiter 5"
      echo ""
      echo "    See http://nestanmr.com for more."
      echo ""
      echo "    The defaults are:"
      echo ""
      echo "      --method 1 --iter 50 --rwiter 10 --cutoff 0.1 --scaling 0.98"
      echo ""
   else
      echo "10. Commonly, only region of interest, FT, and phase correction options"
      echo "    need to be provided. Typical example:"
      echo ""
      echo "      $progName \"
      echo "       -xP0 -16 -xP1   0 -xEXTX1 2.6ppm -xEXTXN -0.8ppm \"
      echo "       -yP0 -90 -yP1 180 -yZFARG zf=2,auto -yFTARG alt"
      echo ""
   endif

   if ($nusFlag == 0) then
      echo "11. Turn on NUS Zero Fill (NUS-style extrapolation of time-domain data)"
      echo "    with the -nusZF flag, and select a reconstruction method with one of:"
      echo "    -ist -rist -smile -hmsIST -nesta. The default is NMRPipe IST (-ist)."
      echo ""
      echo "12. Option -smileArgs is expanded to command line arguments for SMILE:"
      echo ""
      echo "      -smileArgs maxMem=2.0,thresh=0.9"
      echo ""
      echo "    Becomes: $smileProg -fn SMILE -maxMem 2.0 -thresh 0.9 ..."
      echo ""
      echo "    See '$smileProg -fn SMILE -help' for more."
      echo ""
      echo "13. Option -hmsArgs is expanded to command line arguments for hmsIST:"
      echo ""
      echo "      -hmsArgs IST=D,thresh=0.95"
      echo ""
      echo "    Becomes: $hmsISTProg -IST D -thresh 0.95 ..."
      echo ""
      echo "    See '$hmsISTProg -help' for more."
      echo ""
      echo "14. Option -nestaArgs is expanded to -- command line arguments for NESTA:"
      echo ""
      echo "      -nestaArgs --method=1,--rwiter=5"
      echo ""
      echo "    Becomes: nestanmr --method 1 --rwiter 5"
      echo ""
      echo "    See http://nestanmr.com for more."
      echo ""
      echo "    The defaults are:"
      echo ""
      echo "      --method 1 --iter 50 --rwiter 10 --cutoff 0.1 --scaling 0.98"
      echo ""
      echo "15. When using -nusZF -ist (or -rist), all options on the basicFT2.com command"
      echo "    line will be passed to the ist2D.com IST reconstruction script. For example,"
      echo "    to set the IST option for -istTMult:"
      echo ""
      echo "      basicFT2.com -nusZF -ist -istTMult 0.9 ..."
   endif

   exit 0
endif

set vFlag        = (`getLogArgD $argv -verb       -noverb   verb noverb`)
set listFlag     = (`getLogArgD $argv -nolist     -list     0    1`)
set ssNMRFlag    = (`getLogArgD $argv -nossNMR    -ssNMR    0    1`)
set cleanFlag    = (`getLogArgD $argv -clean      -noclean  1    0`)
set execFlag     = (`getLogArgD $argv -exec       -noexec   1    0`)
set pseudo3DFlag = (`getLogArgD $argv -nopseudo3D -pseudo3D 0    1`)

if (`string same $vFlag verb`) then
   set vVal = 1
else
   set vVal = 0
endif

set inName       = (`getArgD $argv -in        Auto`)
set parentName   = (`getArgD $argv -parent    None`)
set outName      = (`getArgD $argv -out       Auto`)
set sampleName   = (`getArgD $argv -sample    None`)
set scriptName   = (`getArgD $argv -script    nmr.com`)
set tmpPrefix    = (`getArgD $argv -tmpPrefix Auto`)
set ft1Name      = (`getArgD $argv -ft1       None`)
set titStr       = (`getArgD $argv -title     None`)
set yPlane       = (`getArgD $argv -yPlane    1`)
set zPlane       = (`getArgD $argv -zPlane    1`)
set aPlane       = (`getArgD $argv -aPlane    1`)
set istMaxRes    = (`getArgD $argv -istMaxRes Auto`)
set hmsIter      = (`getArgD $argv -hmsIter   -1`)
set scaleToVal   = (`getArgD $argv -scaleTo   None`)
set nThreads     = (`getArgD $argv -nThread   Auto`)

set istName      = (`getArgDNF $argv -ist None`)

set xDIArg       = "-di"
set yDIArg       = "-di"

@ xzFlag     = 0
@ extFlag    = 0
@ ndFlag     = 0
@ ioArgsFlag = 0

@ xMCFlag = 0
@ yMCFlag = 0

set xQ1Def = 0.5
set xQ2Def = 0.95
set xQ3Def = 2

set yQ1Def = 0.5
set yQ2Def = 0.95
set yQ3Def = 1

if (`string csame $tmpPrefix Auto`) then
   set tmpPrefix = tmp$$
endif

set tmpFIDName   = ${tmpPrefix}.fid
set tmpPreName   = ${tmpPrefix}_pre.ft1
set tmpFT1Name   = ${tmpPrefix}.ft1
set tmpFT2Name   = ${tmpPrefix}.ft2
set cleanList    = ""

set uSMILEARGS  = "None"
set uNESTAARGS  = "None"
set uHMSARGS    = "None"

set nestaArgs   = "-nThread $nThreads"
set smileArgs   = ""
set smileFTArgs = ""
set hmsArgs     = ""
set apodDF      = ""

if (`string csame $sampleName None`) then
   set hmsSchedName = "/dev/null"
else
   set hmsSchedName = $sampleName
endif

if (`string csame $reconMode smile`) then
   set uSMILEARGS = (`getArgD $argv -smileArgs  None`)

   if (`string csame $uSMILEARGS None`) then
      set smileArgs   = "-nDim 2 -maxIter 2048 -nSigma 2.5"
      set smileFTArgs = ""
   else
      set smileArgs   = "`expandArg $uSMILEARGS` -nocheckArgs -nDim 2 -maxIter 2048 -nSigma 2.5"
      set smileFTArgs = ""
   endif
endif

if (`string csame $reconMode hmsIST`) then
   set uHMSARGS   = (`getArgD  $argv -hmsArgs   None`)
endif

if (`string csame $reconMode nesta`) then
   set uNESTAARGS = (`getArgD  $argv -nestaArgs None`)

   if (`string notcsame $uNESTAARGS None`) then
      set nestaArgs = "$nestaArgs -nestaArgs `expandArg $uNESTAARGS`"
   endif
endif

if (`flagLoc $argv -apodDF`) then
   set apodDF = "-df"
endif

if (`flagLoc $argv -noapodDF`) then
   set apodDF = ""
endif

if (`flagLoc $argv -xy`) then
   set extProg = ext_xy.com
   @ xzFlag    = 0
   @ ndFlag    = 1
endif

if (`flagLoc $argv -xz`) then
   set extProg = ext_xz.com
   @ xzFlag    = 1
   @ ndFlag    = 1
endif

if (`flagLoc $argv -xa`) then
   set extProg = ext_xa.com
   @ xzFlag    = 2
   @ ndFlag    = 1
endif

if ($pseudo3DFlag) then
   set extProg    = ext_xy.com
   @ xzFlag       = 0
   @ ndFlag       = 1

   set srcProg  = xyz2pipe
   set destProg = pipe2xyz
endif

if (`flagLoc $argv -mag`) then
   @ yMCFlag  = 1
   set xDIArg = ""
   set xQ1Def = 0.1
   set yQ1Def = 0.1
   set yQ3Def = 2
endif

if (`flagLoc $argv -xMC`) then
   @ xMCFlag = 1
endif

if (`flagLoc $argv -yMC`) then
   @ yMCFlag = 1
endif

if (`flagLoc $argv -noxDI`) then
   set xDIArg = ""
endif

if (`flagLoc $argv -noyDI`) then
   set yDIArg = ""
endif

if (`flagLoc $argv -ioargs`) then
   @ ioArgsFlag = 1
endif

if (`flagLoc $argv -noioargs`) then
   @ ioArgsFlag = 0
endif

if ($listFlag) then
   if ($appendFlag == 0) then
      echo "#\!/bin/csh"
   endif

   echo ""
   echo "#"

   if ($nusFlag) then
      echo "# Generated by $progName (via basicFT2.com) Version $thisVersion"
   else
      echo "# Generated by $progName Version $thisVersion"
   endif

   echo "#"
   prettyPrintCmnd.tcl $0 `stripArgs -list -noexec -append -noappend -parent= -- $argv`
   echo ""
else
   if (`string same $vFlag verb`) then
      echo ""

      if ($nusFlag) then
         echo "$progName Version:       $thisVersion (via basicFT2.com)"
      else
         echo "$progName Version:       $thisVersion"
      endif
   endif
endif

if (`string csame $inName Auto`) then
   set inName = test.fid

   if ($ndFlag || $pseudo3DFlag) then
      set fList = (fid/test%03d.fid data/test%03d.fid fid/spec%03d.fid data/spec%03d.fid fid/test%04d.fid data/test%04d.fid test.fid spec.fid)
   else
      set fList = (test.fid spec.fid fid/test%03d.fid data/test%03d.fid fid/spec%03d.fid data/spec%03d.fid fid/test%04d.fid data/test%04d.fid)
   endif

   foreach i ($fList)
      set thisName = (`nmrPrintf $i 1`)
   
      if (-e $thisName) then
         set inName = $i
         break
      endif
   end
endif

if (`string csame $outName Auto`) then
   if ($pseudo3DFlag) then
      if (`string csame $reconMode ist`) then
         if ($retainFlag) then
            set outName     = rist/test%03d.ft2
         else
            set outName     = ist/test%03d.ft2
         endif
      else if (`string csame $reconMode smile`) then
         set outName     = smile/test%03d.ft2
      else if (`string csame $reconMode hmsIST`) then
         set outName     = hms_ist/test%03d.ft2
      else if (`string csame $reconMode nesta`) then
         set outName     = nesta/test%03d.ft2
      else
         set outName     = ft/test%03d.ft2
      endif
   else
      if (`string csame $reconMode ist`) then
         if ($retainFlag) then
            set outName     = rist.ft2
         else
            set outName     = ist.ft2
         endif
      else if (`string csame $reconMode smile`) then
         set outName     = smile.ft2
      else if (`string csame $reconMode hmsIST`) then
         set outName     = hms_ist.ft2
      else if (`string csame $reconMode nesta`) then
         set outName     = nesta.ft2
      else
         set outName     = test.ft2
      endif
   endif
endif

set thisDir = (`string path $outName`)

if (!(-d $thisDir)) then
   if (`string same $vFlag verb`) then
      echo "$progName New Directory: $thisDir"
      echo ""
   endif

   mkdir $thisDir
endif

set origInName    = $inName
set origInNameStr = $inName
set inNameStr     = $inName
set outNameStr    = $outName

if ($listFlag) then
  if ($ioArgsFlag) then
      echo 'set inName  = (`getArgD $argv -in  '$inName'`)'
      echo 'set outName = (`getArgD $argv -out '$outName'`)'
      echo ""

      set origInNameStr = '$inName'
      set inNameStr     = '$inName'
      set outNameStr    = '$outName'
   endif
endif

if (`string csame $reconMode ist`) then
   if (`string csame $istName None`) then
      set istName = $outName
   else
      set outName = $istName
   endif

   if (`string notcsame $ft1Name None`) then
      set ft1Name   = $tmpFT1Name
      set cleanList = ($cleanList $tmpFT1Name)
   endif

   set cleanList = ($cleanList $tmpFT2Name)
endif

if (`string csame $reconMode nesta`) then
   set cleanList = ($cleanList $tmpPreName $tmpFT1Name)
endif

if (`string csame $reconMode smile` || `string csame $reconMode nesta`) then
   if ($nusFlag) then
      if (`string csame $sampleName None`) then
         nmrPrintf -stderr "%s\n" "$progName Error: No Sample Schedule Input Specified."
         exit 1
      endif

      if (!(-e $sampleName)) then
         nmrPrintf -stderr "%s\n" "$progName Error Finding Sample Schedule $sampleName"
         exit 1
      endif
   endif
endif

if (`string same $vFlag verb`) then
   if ($listFlag == 0) then
      echo "$progName Input:         $inName"

      if (`string csame $reconMode smile`) then
         if ($nusFlag) then
            echo "$progName Schedule:      $sampleName"
         endif

         echo "$progName SMILE Output:  $outName"
      else if (`string csame $reconMode ist`) then
         echo "$progName FT Output:     $tmpFT2Name"
         echo "$progName IST Output:    $outName"
      else if (`string csame $reconMode hmsIST`) then
         echo "$progName hmsIST Output: $outName"
      else
         echo "$progName FT Output:     $outName"
      endif

      echo "$progName Script Output: $scriptName"
      echo ""
   endif
endif

if (`string csame $parentName None`) then
   @ fmtCount     = (`echo $inName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)
   set thisName   = (`nmrPrintf $origInName 1 1`)
   set parentName = $origInName

   if ($fmtCount == 0) then
      if (!(-e $inName)) then
         nmrPrintf -stderr "%s\n" "$progName Notice: Missing Input $inName"
         exit 1
      endif
   endif
else
   @ fmtCount   = (`echo $parentName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)
   set thisName = (`nmrPrintf $parentName 1 1`)

   if ($fmtCount == 0) then
      if (!(-e $parentName)) then
         nmrPrintf -stderr "%s\n" "$progName Notice: Missing Parent Input $parentName"
         exit 1
      endif
   endif
endif

if (!(-e $thisName)) then
   nmrPrintf -stderr "$progName Notice: Missing Test Input %s\n" $thisName
   exit 1
endif

set dimCount = (`getParm -in $thisName -parm FDDIMCOUNT -fmt %.0f`)
set pipeFlag = (`getParm -in $thisName -parm FDPIPEFLAG -fmt %.0f`)
set cubeFlag = (`getParm -in $thisName -parm FDCUBEFLAG -fmt %.0f`)

if ($dimCount == 2) then
   set extProg = ext_xy.com
   @ xzFlag    = 0
else if ($pipeFlag || $cubeFlag) then
   @ extFlag = 1
else if ($pseudo3DFlag) then
   @ extFlag = 0
else if ($xzFlag == 0) then
   if ($dimCount == 3 && $fmtCount == 0) then
      if ($pipeFlag) then
         @ extFlag = 1
      else
         @ extFlag = 0
      endif
   else if ($dimCount == 3 && $fmtCount == 1) then
      set inName    = (`nmrPrintf $origInName $zPlane`)
      set inNameStr = $inName
      @ extFlag = 1
   else if ($dimCount == 4 && $fmtCount == 2) then
      set inName    = (`nmrPrintf $origInName $aPlane $zPlane`)
      set inNameStr = $inName
      @ extFlag = 1
   else
      @ extFlag = 1
   endif
else
   @ extFlag = 1
endif

set apName = $origInNameStr

if ($extFlag) then
   if ($listFlag) then
      echo "if (-e $tmpFIDName) then"
      echo "   /bin/rm -f $tmpFIDName"
      echo "endif"
      echo ""

      if ($xzFlag == 1) then
         echo "$extProg $origInNameStr $tmpFIDName $yPlane $aPlane"
      else if ($xzFlag == 2) then
         echo "$extProg $origInNameStr $tmpFIDName $yPlane $zPlane"
      else
         echo "$extProg $origInNameStr $tmpFIDName $zPlane $aPlane"
      endif

      echo ""
   endif

   set inName    = $tmpFIDName
   set inNameStr = $tmpFIDName
   set apName    = $tmpFIDName
   set cleanList = ($cleanList $tmpFIDName)
endif

if ($pseudo3DFlag) then
   set inName = $origInName
endif

set xLAB = (`getParm -in $thisName -txt -fmt %s -parm NDLABEL -dim CUR_XDIM`)

if (`string match "HN*" $xLAB`) then
   set uxSOL     = "SOL"
   set uxBASEARG = "POLY,auto" 
   set uxEXTX1   = "3%"
   set uxEXTXN   = "47%"
else
   set uxSOL     = "NULL"
   set uxBASEARG = "NULL"
   set uxEXTX1   = "0%"
   set uxEXTXN   = "100%"
endif

if ($ssNMRFlag) then
   set uxSOL     = "NULL"
   set uxBASEARG = "NULL" 
endif

set apArgs  = (`getArgD $argv -apArgs  None`)
set apOrd   = (`getArgD $argv -apOrd   0`)
set xSOL    = (`getArgD $argv -xSOL    $uxSOL`)
set xAPOD   = (`getArgD $argv -xAPOD   SP`)
set xQ1     = (`getArgD $argv -xQ1     $xQ1Def`)
set xQ2     = (`getArgD $argv -xQ2     $xQ2Def`)
set xQ3     = (`getArgD $argv -xQ3     $xQ3Def`)
set xELB    = (`getArgD $argv -xELB    0.0`)
set xGLB    = (`getArgD $argv -xGLB    0.0`)
set xGOFF   = (`getArgD $argv -xGOFF   0.0`)
set xC1     = (`getArgD $argv -xC1     Auto`)
set xZFARG  = (`getArgD $argv -xZFARG  zf=1,auto`)
set xFTARG  = (`getArgD $argv -xFTARG  None`)
set xP0     = (`getArgD $argv -xP0     0.0`)
set xP1     = (`getArgD $argv -xP1     0.0`)
set xP0EXT  = (`getArgD $argv -xP0EXT  0.0`)
set xP1EXT  = (`getArgD $argv -xP1EXT  0.0`)
set xEXTX1  = (`getArgD $argv -xEXTX1  $uxEXTX1`)
set xEXTXN  = (`getArgD $argv -xEXTXN  $uxEXTXN`)
set xPREFT  = (`getArgD $argv -xPREFT  NULL`)
set xPOSTFT = (`getArgD $argv -xPOSTFT NULL`)

if (`string csame $xZFARG None`) then
   set xZFARG = "zf=0"
endif

if (`string same $xP0EXT 0.0` && `string same $xP1EXT 0.0`) then
   @ usePSEXTFlag = 0
else
   @ usePSEXTFlag = 1
endif

set xBASEARG_INIT = (`getArgD $argv -xBASEARG_INIT  NULL`)
set xBASEARG_FT1  = (`getArgD $argv -xBASEARG_FT1   NULL`)
set xBASEARG      = (`getArgD $argv -xBASEARG       $uxBASEARG`)

if (`string csame $xC1 Auto`) then
   set v = (`nmrPrintf "%f" $xP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set xC1 = 1.0
   else
      set xC1 = 0.5
   endif
endif

set yAPOD   = (`getArgD $argv -yAPOD   SP`)
set yQ1     = (`getArgD $argv -yQ1     $yQ1Def`)
set yQ2     = (`getArgD $argv -yQ2     $yQ2Def`)
set yQ3     = (`getArgD $argv -yQ3     $yQ3Def`)
set yELB    = (`getArgD $argv -yELB    0.0`)
set yGLB    = (`getArgD $argv -yGLB    0.0`)
set yGOFF   = (`getArgD $argv -yGOFF   0.0`)
set yC1     = (`getArgD $argv -yC1     Auto`)
set yZFARG  = (`getArgD $argv -yZFARG  zf=1,auto`)
set yNUSZF  = (`getArgD $argv -yNUSZF  $yNUSZFDef`)
set yFTARG  = (`getArgD $argv -yFTARG  None`)
set yP0     = (`getArgD $argv -yP0     0.0`)
set yP1     = (`getArgD $argv -yP1     0.0`)
set yPREFT  = (`getArgD $argv -yPREFT  NULL`)
set yPOSTFT = (`getArgD $argv -yPOSTFT NULL`)

if (`string csame $yZFARG None`) then
   set yZFARG = "zf=0"
endif

if (`string csame $yNUSZF None`) then
   set yNUSZF = "zf=0"
endif

if (`IMATH "near( $yP1, 360, 10 )"`) then
   set uyBASEARG = "POLY,auto,ord=0"
else
   set uyBASEARG = "NULL"
endif

set yBASEARG = (`getArgD $argv -yBASEARG $uyBASEARG`)
set yLP      = (`getArgD $argv -yLP      NULL`)

if (`string csame $yC1 Auto`) then
   set v = (`nmrPrintf "%f" $yP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set yC1 = 1.0
   else
      set yC1 = 0.5
   endif
endif

if (!(`string csame $yLP NULL`)) then
   set yLP = "LP $yLP"
endif

if ($nusZFFlag) then
   set nusZFArgs = "-nusZF -yNUSZF $yNUSZF"
else
   set nusZFArgs = ""
endif

set yNUSZFArgs = "`expandArg $yNUSZF`"

if (`string csame $reconMode smile`) then
   if (`string same $vFlag verb`) then
      if (`flagLoc SMILE $smileArgs -report` == 0) then
         set smileArgs = "$smileArgs -report 1"
      endif
   endif

   if ($nusFlag) then
      set smileArgs = "$smileArgs -sample $sampleName"
   else
      set smileArgs = "$smileArgs -sample None"
   endif

   set thisARGV = "FT `expandArg $yFTARG`"

   if (`flagLoc $thisARGV -alt`) then
      set smileFTArgs = "$smileFTArgs -xAlt"
   endif

   if (`flagLoc $thisARGV -neg`) then
      set smileFTArgs = "$smileFTArgs -xNeg"
   endif
endif

if ($listFlag) then
   if (`string csame $xP0 Auto`) then
      if (`string csame $apArgs None`) then
         set apArgInfo = ""
      else
         set apArgInfo = (`expandArg $apArgs`)
      endif

      if (`string notcsame $zPlane 1`) then
         set apArgInfo = "$apArgInfo -apzPlane $zPlane"
      endif

      if (`string notcsame $aPlane 1`) then
         set apArgInfo = "$apArgInfo -apaPlane $aPlane"
      endif

      echo "#"
      echo "# Auto-phase determination; arguments duplicated later on the"
      echo "# basicAutoPhase.com command line ignored:"

      if (`string same $vFlag verb`) then
         echo ""
         nmrPrintf 'nmrPrintf "Auto-Phase ...\\n\\n"'
         echo ""
      endif

      if (`string csame $apOrd None`) then
         set apOrd = -1
      endif

      if ($pseudo3DFlag) then
         echo ""
         echo "set xP0 = ("\`basicAutoPhase.com -pseudo3D -in $apName $apArgInfo -apxP1 $xP1 -apyP0 $yP0 -apyP1 $yP1 -apyFTARG $yFTARG -apOrd $apOrd\`")"
         echo ""
      else
         echo ""
         echo "set xP0 = ("\`basicAutoPhase.com -in $apName $apArgInfo -apxP1 $xP1 -apyP0 $yP0 -apyP1 $yP1 -apyFTARG $yFTARG -apOrd $apOrd\`")"
         echo ""
      endif

      if (`string same $vFlag verb`) then
         nmrPrintf 'nmrPrintf "Auto-Phase xP0: %%.1f\\n\\n" $xP0'
         echo ""
         echo ""
      endif

      set xP0 = '$xP0'
   endif

   if (`string csame $reconMode smile`) then
      echo ""
      echo "#"
      echo "# Arguments duplicated later on the SMILE command line are ignored:"
      echo ""
   endif

   echo "$srcProg -in $inNameStr \"

   if (`string notcsame $xPREFT NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $xPREFT` "\"
   endif

   if (`string notcsame $xSOL NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $xSOL` "\"
   endif

   if (`string csame $xAPOD SP`) then
      if (`string same $xGOFF 0.0`) then
         echo "| nmrPipe -fn" SP -off $xQ1 -end $xQ2 -pow $xQ3 -elb $xELB -glb $xGLB -c $xC1 $apodDF "\"
      else
         echo "| nmrPipe -fn" SP -off $xQ1 -end $xQ2 -pow $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 $apodDF "\"
      endif
   else
      echo "| nmrPipe -fn" APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 $apodDF "\"
   endif

   echo "| nmrPipe -fn" ZF `expandArg $xZFARG` "\"

   if (`string csame $reconMode smile`) then
      echo "| nmrPipe -fn" FT `expandArg $xFTARG` "\"
   else
      echo "| nmrPipe -fn" FT `expandArg $xFTARG` -$vFlag "\"
   endif

   if ($usePSEXTFlag) then
      echo "| nmrPipe -fn" PS -p0 $xP0 -p1 $xP1 "\"
   else
      echo "| nmrPipe -fn" PS -p0 $xP0 -p1 $xP1 $xDIArg "\"
   endif

   if (`string notcsame $xBASEARG_INIT NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $xBASEARG_INIT` "\"
   endif

   echo "| nmrPipe -fn" EXT -x1 $xEXTX1 -xn $xEXTXN -sw "\"

   if ($usePSEXTFlag) then
      echo "| nmrPipe -fn" PS -p0 $xP0EXT -p1 $xP1EXT $xDIArg "\"
   endif

   if (`string notcsame $xBASEARG_FT1 NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $xBASEARG_FT1` "\"
   endif

   if (`string notcsame $xPOSTFT NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $xPOSTFT` "\"
   endif

   set tpNeeded = 0

   if (`string csame $reconMode smile`) then
      echo "| nmrPipe -fn" TP "\"
      echo "| $smileProg -fn SMILE" $smileArgs "\"

      if (`string same $yGOFF 0.0`) then
         echo "                    -xApod" $yAPOD -xQ1 $yQ1 -xQ2 $yQ2 -xQ3 $yQ3 -xELB $yELB -xGLB $yGLB "\"
      else
         echo "                    -xApod" $yAPOD -xQ1 $yQ1 -xQ2 $yQ2 -xQ3 $yQ3 -xELB $yELB -xGLB $yGLB -xGOFF $yGOFF "\"
      endif

      echo "                    -xT" `getZFSize.com $inName CUR_YDIM $yNUSZFArgs` -xP0 $yP0 -xP1 $yP1 "\"
   else if (`string csame $reconMode hmsIST`) then
      echo "| nmrPipe -fn" TP "\"
      echo "| $hmsISTProg" `expandArg $uHMSARGS` "-format padded_pipe -sched $hmsSchedName -dim 1 -itr $hmsIter -xN" `getZFSize.com $inName CUR_YDIM $yNUSZFArgs` "-xc $yC1 -verb $vVal" "\"
   else if (`string csame $reconMode nesta`) then
      @ fmtCount = (`echo $tmpPreName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

      if ($fmtCount == 0) then
         echo "  -out $tmpPreName -ov"
         echo ""
      else
         echo "| pipe2xyz -out $tmpPreName -ov"
         echo ""
      endif

      echo "$nestaProg -in $tmpPreName -out $tmpFT1Name $nusZFArgs -sample $sampleName $nestaArgs"
      echo ""

      @ fmtCount = (`echo $tmpFT1Name | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

      if ($fmtCount == 0) then
         echo "nmrPipe -in $tmpFT1Name \"
      else
         echo "xyz2pipe -in $tmpFT1Name \"
      endif

      set tpNeeded = 1
   else
      set tpNeeded = 1
   endif

   if (`string csame $ft1Name None`) then
      if ($tpNeeded) then
         echo "| nmrPipe -fn" TP "\"
      endif
   else
      if ($tpNeeded) then
         echo "| nmrPipe -fn SAVE -name $ft1Name \"
         echo "| nmrPipe -fn" TP "\"
      else
         echo "| nmrPipe -fn" TP "\"
         echo "| nmrPipe -fn SAVE -name $ft1Name \"
         echo "| nmrPipe -fn" TP "\"
      endif
   endif

   if (`string notcsame $yPREFT NULL`) then
       echo "| nmrPipe -fn" `expandArg0 $yPREFT` "\"
   endif

   if (`string notcsame $yLP NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $yLP` "\"
   endif

   if (`string notcsame $reconMode smile`) then
      if (`string csame $yAPOD SP`) then
         if (`string same $yGOFF 0.0`) then
            echo "| nmrPipe -fn SP -off $yQ1 -end $yQ2 -pow $yQ3 -elb $yELB -glb $yGLB -c $yC1 \"
         else
            echo "| nmrPipe -fn SP -off $yQ1 -end $yQ2 -pow $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1 \"
         endif
      else
         echo "| nmrPipe -fn APOD -qName $yAPOD -q1 $yQ1 -q2 $yQ2 -q3 $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1 \"
      endif
   endif

   echo "| nmrPipe -fn" ZF `expandArg $yZFARG` "\"
   echo "| nmrPipe -fn" FT `expandArg $yFTARG` -$vFlag "\"

   if ($yMCFlag) then
      echo "| nmrPipe -fn" MC "\"
   else
      echo "| nmrPipe -fn" PS -p0 $yP0 -p1 $yP1 $yDIArg "\"
   endif

   if (`string notcsame $yBASEARG NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $yBASEARG` "\"
   endif

   if (`string notcsame $yPOSTFT NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $yPOSTFT` "\"
   endif

   echo "| nmrPipe -fn" TP "\"

   if ($xMCFlag) then
      echo "| nmrPipe -fn" MC "\"
   endif

   if (`string notcsame $xBASEARG NULL`) then
      echo "| nmrPipe -fn" `expandArg0 $xBASEARG` -$vFlag "\"
   endif

   set skipArgs = "-in= -out= -istName= -noexec -list -nonusZF -nusZF -ist"
   set istArgs  = ""

   if (`string csame $reconMode ist`) then
      echo "  -out $tmpFT2Name -ov"

      if (`string csame $istMaxRes Auto`) then
         echo ""
         echo "set istMaxRes = ("\`specStat.com -in $tmpFT2Name -brief -stat istMaxRes\`")"
         set istArgs  = '-istMaxRes $istMaxRes'
         set skipArgs = "$skipArgs -istMaxRes="
      endif
   else
      if ($pseudo3DFlag) then
         echo "| $destProg -out $outNameStr -ov"
      else
         echo "  -out $outNameStr -ov"
      endif
   endif

   if (`string csame $reconMode ist`) then
      echo ""
      echo "#"
      echo "# Arguments duplicated later on the command line are ignored:"
      echo ""

      prettyPrintCmnd.tcl -nocomment \
       ist2D.com -us $istArgs -nusZF -in $inNameStr -istName $istName -mask Auto -origArgs `stripArgs $skipArgs -- $*`
   else
      if (`string notcsame $scaleToVal None`) then
         echo ""
         echo rescale.com -in $outNameStr -out $outNameStr -max $scaleToVal
      endif
   endif

   if ($pseudo3DFlag) then
      echo ""
      echo copyTau.com $inNameStr $outNameStr -noverb -all
      echo ""

      if ($nusZFFlag) then
         if (`string notcsame $outName $istName`) then
            echo ""
            echo copyTau.com $inNameStr $istName -noverb -all
         endif
      endif
   endif

   if (`string casecmp $titStr None`) then
      if (`string notcsame $reconMode ist`) then
         echo ""
         echo "sethdr.com $outNameStr -title $titStr"
      endif
   endif

   set cleanCount = (`string length $cleanList`)

   if ($cleanFlag && $cleanCount) then
      echo ""
      echo "foreach i ("$cleanList")"
      echo '   if (-e $i) then'
      echo '      /bin/rm -f $i'
      echo '   endif'
      echo "end"
   endif
endif

if ($listFlag == 0) then
   if ($appendFlag) then
      if (!(-e $scriptName)) then
         echo "#!/bin/csh" >  $scriptName
         echo ""           >> $scriptName
      endif

      basicFT2.com $* -list >> $scriptName
   else
      basicFT2.com $* -list >  $scriptName
   endif

   chmod a+rx $scriptName

   if ($execFlag) then
      $scriptName

      if (`string same $vFlag verb`) then
         echo ""
      endif
   endif
endif

exit 0
