#!/bin/csh

set thisVersion = "2018.334.10.06"

if (`flagLoc $argv -help`) then
   echo "basicFT2Mag.com $thisVersion"
   echo "Perform Basic Processing for 2D Magnitude-Mode Data:"
   echo " -in   inName    [test.fid]   2D Magntiude-Mode Time-Domain Input."
   echo " -out  outName   [test.ft2]   2D Spectrum Output."
   echo " -list                        List Script Contents Without Processing."
   echo "Processing Parameters (Use -x... -y...):"
   echo " -xSOL   sName   [NULL]       Solvent Function Name and Options (X-Axis Only)."
   echo " -xAPOD  aName   [SP]         Window Function Name."
   echo " -xQ1    q1      [0.05]       Window Parameter Q1."
   echo " -xQ2    q2      [0.95]       Window Parameter Q2."
   echo " -xQ3    q3      [1]          Window Parameter Q3."
   echo " -xELB   elb     [0.0]        Exponential Line Broaden, Hz."
   echo " -xGLB   glb     [0.0]        Gaussian Line Broaden, Hz."
   echo " -xGOFF  goff    [0.0]        Gaussian Center, 0.0 to 1.0."
   echo " -xC1    c       [0.5]        First-Point Time-Domain Scale (0.5 1.0)."
   echo " -xZFARG zfArgs  [zf=1,auto]  ZF (Zero Fill) Option."
   echo " -xFTARG ftArg   [None]       FT (Fourier Transform) Options (None neg alt)."
   echo " -xEXTX1 x1      [0%]         Extract Range Start (X-Axis Only) (% Hz ppm pts)."
   echo " -xEXTXN xn      [100%]       Extract Range End   (X-Axis Only) (% Hz ppm pts)."
   echo "Baseline Correction Function Name and Options:"
   echo " -xBASEARG_FT1   [NULL]       X-Axis Interferogram Baseline Correction."
   echo " -xBASEARG       [NULL]       X-Axis Spectrum Baseline Correction."
   echo " -yBASEARG       [NULL]       Y-Axis Spectrum Baseline Correction."
   echo "Linear Prediction (Indirect Dimension Only):"
   echo " -yLP    lpArgs  [NULL]       Linear Prediction Arguments."
   echo "Notes:"
   echo " 1. Options -xSOL -xBASEARG_FT1 -xBASEARG etc are expanded into NMRPipe"
   echo "    processing function names and command line arguments. Options should"
   echo "    contain no spaces. Flags are separated by commas, and options with"
   echo "    values are given using = signs. For example:"
   echo ""
   echo "      -xSOL     NULL              Becomes: nmrPipe -fn NULL (No Solvent Filter)"
   echo "      -xSOL     SOL               Becomes: nmrPipe -fn SOL"
   echo "      -xSOL     POLY,time         Becomes: nmrPipe -fn POLY -time"
   echo "      -xBASEARG POLY,auto,ord=0   Becomes: nmrPipe -fn POLY -auto -ord 0"
   echo ""
   echo " 2. Options -xZFARG -xFTARG are expanded to command line arguments for ZF"
   echo "    and FT, for example:"
   echo ""
   echo "      -xZFARG size=2048,auto  Becomes: nmrPipe -fn ZF -size 2048 -auto"
   echo "      -xFTARG neg             Becomes: nmrPipe -fn FT -neg"
   echo ""
   echo " 3. Options -yLP are expanded to command line arguments for Linear Prediction:"
   echo ""
   echo "       -yLP NULL              No Linear Prediction is performed (Default)."
   echo "       -yLP fb,ord=12         Becomes: nmrPipe -fn LP -fb -ord 12"
   echo ""
   echo " 4. If the X-Axis label of the input spectrum starts with 'HN', the default"
   echo "    parameters will be adjusted as follows:"
   echo ""
   echo "       -xSOL SOL -xBASEARG POLY,auto -xEXTX1 3% -xEXTXN 47%"
   echo ""
   exit 0
endif

set inName    = (`getArgD $argv -in     test.fid`)
set outName   = (`getArgD $argv -out    test.ft2`)
set origName  = $inName
set vFlag     = verb

@ listFlag    = 0

if (`flagLoc $argv -verb`) then
   set vFlag = verb
endif

if (`flagLoc $argv -noverb`) then
   set vFlag = noverb
endif

if (`flagLoc $argv -list`) then
   @ listFlag = 1
endif

if (`flagLoc $argv -nolist`) then
   @ listFlag = 0
endif

if ($listFlag) then
   echo "#\!/bin/csh"
   echo ""
   echo "#"
   echo "# Generated by basicFT2Mag.com Version $thisVersion"
   echo ""
else
   echo ""
   echo "basicFT2Mag.com Version $thisVersion"
   echo ""
endif

if (!(-e $inName)) then
   nmrPrintf -stderr "%s\n" "basicFT2Mag.com Notice: Missing Input $inName"
   exit 1
endif

set xLAB = (`getParm -in $inName -txt -fmt %s -parm NDLABEL -dim CUR_XDIM`)

if (`string match "HN*" $xLAB`) then
   set uxSOL     = "SOL"
   set uxBASEARG = "POLY,auto" 
   set uxEXTX1   = "3%"
   set uxEXTXN   = "47%"
else
   set uxSOL     = "NULL"
   set uxBASEARG = "NULL"
   set uxEXTX1   = "0%"
   set uxEXTXN   = "100%"
endif

set xSOL   = (`getArgD $argv -xSOL   $uxSOL`)
set xAPOD  = (`getArgD $argv -xAPOD  SP`)
set xQ1    = (`getArgD $argv -xQ1    0.05`)
set xQ2    = (`getArgD $argv -xQ2    0.95`)
set xQ3    = (`getArgD $argv -xQ3    1`)
set xELB   = (`getArgD $argv -xELB   0.0`)
set xGLB   = (`getArgD $argv -xGLB   0.0`)
set xGOFF  = (`getArgD $argv -xGOFF  0.0`)
set xC1    = (`getArgD $argv -xC1    0.5`)
set xZFARG = (`getArgD $argv -xZFARG zf=1,auto`)
set xFTARG = (`getArgD $argv -xFTARG None`)
set xP0    = 0.0
set xP1    = 0.0

set xEXTX1 = (`getArgD $argv -xEXTX1 $uxEXTX1`)
set xEXTXN = (`getArgD $argv -xEXTXN $uxEXTXN`)

set xBASEARG_FT1   = (`getArgD $argv -xBASEARG_FT1   NULL`)
set xBASEARG       = (`getArgD $argv -xBASEARG       $uxBASEARG`)

set yAPOD  = (`getArgD $argv -yAPOD  SP`)
set yQ1    = (`getArgD $argv -yQ1    0.05`)
set yQ2    = (`getArgD $argv -yQ2    0.95`)
set yQ3    = (`getArgD $argv -yQ3    1`)
set yELB   = (`getArgD $argv -yELB   0.0`)
set yGLB   = (`getArgD $argv -yGLB   0.0`)
set yGOFF  = (`getArgD $argv -yGOFF  0.0`)
set yC1    = (`getArgD $argv -yC1    0.5`)
set yZFARG = (`getArgD $argv -yZFARG zf=1,auto`)
set yFTARG = (`getArgD $argv -yFTARG None`)
set yP0    = 0.0
set yP1    = 0.0

set yBASEARG       = (`getArgD $argv -yBASEARG       NULL`)
set yLP            = (`getArgD $argv -yLP            NULL`)

if (!(`string csame $yLP NULL`)) then
   set yLP = "LP $yLP"
endif

if ($listFlag) then
   echo "nmrPipe -in $inName \"
   echo "| nmrPipe -fn" `expandArg0 $xSOL` "\"

   if (`string csame $xAPOD SP`) then
      if (`string same $xGOFF 0.0`) then
         echo "| nmrPipe -fn SP -off $xQ1 -end $xQ2 -pow $xQ3 -elb $xELB -glb $xGLB -c $xC1 \"
      else
         echo "| nmrPipe -fn SP -off $xQ1 -end $xQ2 -pow $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 \"
      endif
   else
      echo "| nmrPipe -fn APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 \"
   endif

   echo "| nmrPipe -fn" ZF `expandArg $xZFARG` "\"
   echo "| nmrPipe -fn" FT `expandArg $xFTARG` -$vFlag "\"
   echo "| nmrPipe -fn" EXT -x1 $xEXTX1 -xn $xEXTXN -sw "\"
   echo "| nmrPipe -fn" `expandArg0 $xBASEARG_FT1` "\"
   echo "| nmrPipe -fn" TP "\"
   echo "| nmrPipe -fn" `expandArg0 $yLP` "\"

   if (`string csame $yAPOD SP`) then
      if (`string same $xGOFF 0.0`) then
         echo "| nmrPipe -fn SP -off $yQ1 -end $yQ2 -pow $yQ3 -elb $yELB -glb $yGLB -c $yC1 \"
      else
         echo "| nmrPipe -fn SP -off $yQ1 -end $yQ2 -pow $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1 \"
      endif
   else
      echo "| nmrPipe -fn APOD -qName $yAPOD -q1 $yQ1 -q2 $yQ2 -q3 $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1 \"
   endif

   echo "| nmrPipe -fn" ZF `expandArg $yZFARG` "\"
   echo "| nmrPipe -fn" FT `expandArg $yFTARG` -$vFlag "\"
   echo "| nmrPipe -fn" MC "\"
   echo "| nmrPipe -fn" `expandArg0 $yBASEARG` "\"
   echo "| nmrPipe -fn" TP "\"
   echo "| nmrPipe -fn" `expandArg0 $xBASEARG` -$vFlag "\"
   echo "  -out $outName -ov"
else
   nmrPipe -in $inName \
   | nmrPipe -fn `expandArg0 $xSOL` \
   | nmrPipe -fn APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 \
   | nmrPipe -fn ZF `expandArg $xZFARG` \
   | nmrPipe -fn FT `expandArg $xFTARG` -$vFlag \
   | nmrPipe -fn EXT -x1 $xEXTX1 -xn $xEXTXN -sw \
   | nmrPipe -fn `expandArg0 $xBASEARG_FT1` \
   | nmrPipe -fn TP \
   | nmrPipe -fn `expandArg0 $yLP` \
   | nmrPipe -fn APOD -qName $yAPOD -q1 $yQ1 -q2 $yQ2 -q3 $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1 \
   | nmrPipe -fn ZF `expandArg $yZFARG` \
   | nmrPipe -fn FT `expandArg $yFTARG` -$vFlag \
   | nmrPipe -fn MC \
   | nmrPipe -fn `expandArg0 $yBASEARG` \
   | nmrPipe -fn TP \
   | nmrPipe -fn `expandArg0 $xBASEARG` -$vFlag \
     -out $outName -ov
endif

if (`string same $vFlag verb`) then
   echo ""
endif

exit 0
