#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
# cont.tcl: post-process NMRWish PostScript files to condense contour lines.
#---------------------------------------------------------------------------

proc contArgs {} \
{
   global inUnit outUnit inName outName verbose ARGV

   set inName   stdin 
   set outName  stdout 
   set inUnit   stdin
   set outUnit  stdout

   set recLimit 8000 
   set verbose  1

   if {[flagLoc $ARGV -help]} \
      {
       puts "Condense NMRWish PostScript Output; improves the appearance of contours:"
       puts "Options:"
       puts " -in     inName  \[stdin\]   NMRWish PostScript Input."
       puts " -out    outName \[stdout\]  Condensed PostScript Output."
       puts " -rec    rLimit  \[8000\]    Recursion Limit (Increase if Needed)." 
       puts " -verb                     Verbose mode ON (Default)."
       puts " -noverb                   Verbose mode OFF."
       exit
      }

   getArgD $ARGV -in  inName
   getArgD $ARGV -out outName
   getArgD $ARGV -rec recLimit

   if {[flagLoc $ARGV -verb]}   {set verbose 1}
   if {[flagLoc $ARGV -noverb]} {set verbose 0}
  
   if {$recLimit} {setRecursion $recLimit}
 
   if {![strcmp $inName $outName]} \
      {
       puts "Error: in-place processing is not allowed."
       exit
      }

   if {[strcasecmp $inName stdin]} \
      {
       set error [catch {set inUnit [open $inName r]}]

       if {$error} \
          {
           puts "Error opening input file $inName"
           exit
          }
      }

   if {[strcasecmp $outName stdout]} \
      {
       set error [catch {set outUnit [open $outName w]}]
   
       if {$error} \
          {
           puts "Error opening output file $outName"
           exit
          }
      }
}
   
#
# Fix degenerate vectors. These are vectors that start and end in the same
# place, and are the endpoints of some other vector.

proc fixCont {} \
{
   global dInfo vInfo vCount

   if {![info exists dInfo]} return

   set id [array startsearch dInfo]

   while {[array anymore dInfo $id]} \
      {
       set loc  [array nextelement dInfo $id]
       set vals [split $loc ,]

       set ix1   [lindex $vals 0] 
       set iy1   [lindex $vals 1] 
       set iz    [lindex $vals 2] 
       set iw    [lindex $vals 3] 
       set it    [lindex $vals 4] 

       if {[info exists cInfo($loc)]} \
          {
           foreach j $dInfo($loc) \
              {
               set vInfo($j) ""
              }
          } \
       else \
          {
           set vInfo($vCount) "$ix1 $iy1 $ix1 $iy1 $iz $iw $it"

           lappend cInfo($loc) $vCount
           incr    vCount
          }
      }

   array donesearch dInfo $id
}

#
# Print the PostScript condensed vector list.

proc showCont {} \
{
   global vList qList sCount outUnit

   if {[llength $vList] > 2} \
      {
       set cCount [llength $vList]
       set pCount [expr $cCount/2]
       set j      0

       set  x0 [lindex $vList $j]
       incr j

       set  y0 [lindex $vList $j]
       incr j

       puts $outUnit "stroke"
       puts $outUnit "newpath"

       puts $outUnit "$x0 $y0 M"

       for {set i 1} {$i < $pCount} {incr i} \
          {
           set  x1 [lindex $vList $j]
           incr j

           set  y1 [lindex $vList $j]
           incr j

           puts $outUnit "$x1 $y1 L"
          }

        if {$x0 == $x1 && $y0 == $y1} {puts $outUnit "closepath"}

        incr sCount
       }

   puts $outUnit "stroke"
   puts $outUnit ""

   if {[llength $qList] > 2} \
      {
       set cCount [llength $qList]
       set qCount [expr $cCount/4]
       set j      0

       for {set i 0} {$i < $qCount} {incr i} \
          {
           set  x1 [lindex $qList $j]
           incr j

           set  y1 [lindex $qList $j]
           incr j

           set  xn [lindex $qList $j]
           incr j

           set  yn [lindex $qList $j]
           incr j

           puts $outUnit "$x1 $y1 $xn $yn V"
          }

       puts $outUnit "stroke"
       puts $outUnit ""
      }
}

#
# Recursive function to find connected vectors.

proc findVec { i ixn iyn iz iw it } \
{
   global vList qList cInfo vInfo uCount rCount rCountMax

   set loc $ixn,$iyn,$iz,$iw,$it

   incr rCount

   if {$rCount > $rCountMax} {set rCountMax $rCount}

   if {![info exists cInfo($loc)]} \
      {
       incr uCount
       incr rCount -1
       return
      }

   set nC [llength $cInfo($loc)]

   if {$nC > 2} \
      {
       foreach j $cInfo($loc) \
           {
            if {$i != $j} \
               {
                set line      $vInfo($j)
                set n         [llength $line]
                set vInfo($j) ""

                if {$n == 7} \
                   {
                    set jx1 [lindex $line 0]
                    set jy1 [lindex $line 1]
                    set jxn [lindex $line 2]
                    set jyn [lindex $line 3]

                    lappend qList $jx1 $jy1 $jxn $jyn
                   }
               }
           }

       return
      }

   foreach j $cInfo($loc) \
      {
       if {$i != $j} \
          {
           set line      $vInfo($j)
           set n         [llength $line]
           set vInfo($j) ""

           if {$n == 7} \
              {
               set jx1 [lindex $line 0]
               set jy1 [lindex $line 1]
               set jxn [lindex $line 2]
               set jyn [lindex $line 3]
               set jz  [lindex $line 4]
               set jw  [lindex $line 5]
               set jt  [lindex $line 6]

               if {$ixn == $jx1 && $iyn == $jy1 && $iz == $jz && $iw == $jw && $it == $jt} \
                  {
                   lappend vList $jxn $jyn
                   set error [catch "findVec $j $jxn $jyn $jz $jw $jt"]
                  } \
               else \
                  {
                   lappend vList $jx1 $jy1
                   set error [catch "findVec $j $jx1 $jy1 $jz $jw $jt"]
                  }

               if {$error} \
                  {
                   puts stderr ""
                   puts stderr "Error: Maximum Recursion Count Exceeded. Increase -rec value."
                   puts stderr ""
                   exit 1
                  }
              }
          }
      } 

   incr rCount -1
}

#
# Read and store the individual contour vectors and their endpoints.

proc readCont {} \
{
   global vList qList iCount vCount sCount dCount uCount rCount rCountMax cInfo dInfo eInfo vInfo
   global inUnit outUnit outName verbose

   set plotCount 0
   set contFlag  0
   set dCount    0
   set uCount    0
   set sCount    0
   set vCount    0
   set rCount    0
   set lCount    0
   set iCount    0
   set rCountMax 0

   set iz        0
   set iw        0
   set it        0

   while {[gets $inUnit line] >= 0} \
      {
       incr lCount

       if {"$line" == "% End contour plot."} \
          {
           if {$verbose} {puts stderr "Contour Plot $plotCount End Line:   $lCount"}

           set rCountMax 0
           set rCount    0

           fixCont
           joinCont

           if {$verbose} \
              {
               puts stderr " Total Vectors:       $vCount"
               puts stderr " Degenerate Vertices: $dCount"
               puts stderr " Unpaired Vertices:   $uCount"
               puts stderr " Identical Vectors:   $iCount"
               puts stderr " Final Paths:         $sCount"
               puts stderr " Max Recursions:      $rCountMax"
               puts stderr ""
              }

           set  contFlag 0
          }

       if {"$line" == "% Begin contour plot."} \
          {
           incr plotCount

           if {$verbose} {puts stderr "Contour Plot $plotCount Start Line: $lCount"}

           if {[info exists cInfo]} {unset cInfo}
           if {[info exists dInfo]} {unset dInfo}
           if {[info exists eInfo]} {unset eInfo}
           if {[info exists vInfo]} {unset vInfo}

           set   contFlag 1
           set   dCount   0
           set   uCount   0
           set   sCount   0
           set   vCount   0
           set   iCount   0

           set   iz       0
           set   iw       0
           set   it       0

           puts $outUnit $line
          }

       if {$contFlag == 0} \
          {
           if {![strcasecmp [string range $line 0 7] "%%Title:"]} {set line "%%Title: $outName"}
           puts $outUnit $line
          } \
       else \
          {
           set n  [llength $line]

           if {$n == 5} \
              {
               set ix1 [lindex $line 0]
               set iy1 [lindex $line 1]
               set ixn [lindex $line 2]
               set iyn [lindex $line 3]

               if {[info exists eInfo($ix1,$iy1,$ixn,$iyn,$iz,$iw,$it)] || [info exists eInfo($ixn,$iyn,$ix1,$iy1,$iz,$iw,$it)]} \
                  {
                   incr iCount
                  } \
               elseif {$ix1 == $ixn && $iy1 == $iyn} \
                  {
                   lappend dInfo($ix1,$iy1,$iz,$iw,$it) $vCount
                   incr    dCount
                  } \
               else \
                  {
                   set vInfo($vCount) "$ix1 $iy1 $ixn $iyn $iz $iw $it"

                   set eInfo($ix1,$iy1,$ixn,$iyn,$iz,$iw,$it) 1

                   lappend cInfo($ix1,$iy1,$iz,$iw,$it) $vCount
                   lappend cInfo($ixn,$iyn,$iz,$iw,$it) $vCount
                   incr    vCount
                  }
              } \
           elseif {$n == 1} \
              {
               set key [string index $line 0]

               switch $key \
                  {
                   C       {set iz  [string range $line 1 end]}
                   D       {set it  [string range $line 1 end]}
                   W       {set iw  [string range $line 1 end]}
                   default {set key ""}
                  }
              }
          }
      }
}

#
# Join the contour vectors.

proc joinCont {} \
{
   global vList qList vCount sCount vInfo cInfo outUnit

   set izPrev -1
   set iwPrev -1
   set itPrev -1

   for {set i 0} {$i < $vCount} {incr i} \
      {
       set line      $vInfo($i)
       set n         [llength $line]
       set vInfo($i) ""
       set qList     ""
       set vList     ""

       if {$n == 7} \
          {
           set ix1 [lindex $line 0]
           set iy1 [lindex $line 1]
           set ixn [lindex $line 2]
           set iyn [lindex $line 3]
           set iz  [lindex $line 4]
           set iw  [lindex $line 5]
           set it  [lindex $line 6]

           if {$iz != $izPrev} \
              {
               puts $outUnit "C$iz"
               set  izPrev $iz
              }

           if {$iw != $iwPrev} \
              {
               puts $outUnit "W$iw"
               set  iwPrev $iw
              }

           if {$it != $itPrev} \
              {
               puts $outUnit "D$it"
               set  itPrev $it
              }

           lappend vList $ix1 $iy1 $ixn $iyn

           findVec $i $ixn $iyn $iz $iw $it

           showCont
          }
      }
}

#
# Main application:

contArgs
readCont

exit
