#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set progTitle    "NMRPipe Conversion Utility"
set progVersion  "2024.061.14.29"

#
# conv.tcl: Graphical interface for creating and running NMRPipe
#           spectrometer format conversion and processing scripts. 
#           (C) 1995-2022 Frank Delaglio
#---------------------------------------------------------------------

proc main {} \
{
   global autoFlag infoFlag readFlag

   setOSType

   getParms

   setColors
   setFonts
   setTauInfo
   setParms
   setGamma
   setMisc

   createFrames

   addProtocols
   addNames
   addSpecial
   addLabels

   addDim x
   addDim y
   addDim z
   addDim a

   createAcqMenu x
   createIndMenu y
   createIndMenu z
   createIndMenu a

   createGenMenu x
   createGenMenu y
   createGenMenu z
   createGenMenu a

   addTextArea
   addAuxArea
   packFrames
   setBindings

   initConv

   if {$autoFlag} \
      {
       if {$infoFlag} \
          {
           readParams
           showInfoText
           saveInfoText
           quitProc
          } \
       else \
          {
           readParams
           saveScript
           quitProc
          }
      }

   showReadNeeded 1

   if {$readFlag} readParams

   set palInfo {tk_setPalette activeBackground gray10 \
                 foreground gray20 \
                 selectColor gray30 \
                 activeForeground gray40 \
                 highlightBackground blue \
                 selectBackground blue \
                 background blue \
                 highlightColor blue \
                 selectForeground blue \
                 disabledForeground blue \
                 insertBackground blue \
                 troughColor yellow}
}

#
# getParms: set some general defaults and extract command line arguments.

proc getParms {} \
{
   global spectralType convMode sleepFlag autoFlag tempFlag readFlag infoFlag evenFlag scriptName verbose
   global dimCount dimCountName progVersion progTitle temperature vj2pipeMode swhFlag o1Flag h2oFlag topspinFlag showPseudoFlag showObsMidFlag

   global nusFlag nusAvgFlag maskFlag nmrDrawFlag nmrDrawCmnd nusInfo nusDimCount nusName nusOffset nusRevFlag nusSampleCount nusSampleCountMax
   global inProtocol outProtocol inProtocolName outProtocolName scriptInfo labUpdateMode nmrboxFlag rnmrtkFlag ioArgsFlag pseudo3DFlag pseudoLoopMode
   global userAQSeq userScale userScans dataScale vendorScale vendorScans vendorMsg scaleByScansFlag scaleByUserFlag scaleByVendorFlag scaleByMultFlag
   global interleaveInfo interleaveFlag tauFileName tauList tauScale tauFmt expectedTauCount userTauCount
   global initTextArea hiddenList readCount dmxMode infoRequest infoResult dateText
   global vdlistName vdlistCount

   global inName inPath inRoot interName infoName outName outPath outRoot userOutDir maskName pow2Flag
   global titleStr extraArgs finalArgs ARGC ARGV env autoUpdateFlag
   global charX charY wideMode

   global INMULTx INMULTy INMULTz INMULTa NUCx NUCy NUCz NUCa NUSZFx NUSZFy NUSZFz NUSZFa
   global CARdx CARdy CARdz CARda carXSrc
   global wordSize wordType

   set hiddenList      {}
   set initTextArea    1

   set inProtocol      BRUKER_NIH
   set outProtocol     NMRPIPE
   set convMode        SPECTRUM 
   set spectralType    NMR
   set dimCount        3
   set sleepFlag       1
   set dmxMode         0
   set evenFlag        1
   set pseudoLoopMode  0
   set pseudo3DFlag    0
   set showPseudoFlag  0
   set showObsMidFlag  0
   set nmrDrawFlag     0
   set nmrDrawCmnd     ""
   set dateText        None

   set interleaveFlag   0
   set pseudo3DFlag     0
   set interleaveInfo   None
   set tauFileName      None
   set tauList          None
   set tauScale         1.0
   set tauFmt           %s
   set expectedTauCount 0
   set userTauCount     0

   set userScale         1000.0
   set dataScale         1.0
   set vendorScale       1.0
   set vendorScans       1
   set vendorMsg         None
   set userScans         Auto
   set userAQSeq         None

   set scaleByScansFlag  1
   set scaleByVendorFlag 1
   set scaleByUserFlag   1
   set scaleByMultFlag   1

   set h2oFlag        1
   set swhFlag        1
   set o1Flag         0
   set topspinFlag    0
   set pow2Flag       1
   set nmrboxFlag     0
   set rnmrtkFlag     0
   set ioArgsFlag     0
   set autoUpdateFlag 0
   set wordSize       Auto
   set wordType       Auto

   set vj2pipeMode  0

   if {[info exists env(NMRBASE)]} \
      {
       if {[file isdirectory $env(NMRBASE)/vj2pipe]} \
          {
           set vj2pipeMode  1
          }
      }

   if {[info exists env(NMR_NMRBOX)]} \
      {
       if {![strcasecmp $env(NMR_NMRBOX) 1]} {set nmrboxFlag 1}
       if {![strcasecmp $env(NMR_NMRBOX) 0]} {set nmrboxFlag 0}
      }
   
   set carXSrc      None 

   set autoFlag        0
   set tempFlag        0
   set readFlag        0
   set infoFlag        0
   set readCount       0

   foreach i {x y z a} {set labUpdateMode($i) 1}

   set nusFlag           0
   set nusAvgFlag        1
   set maskFlag          0
   set nusDimCount       0
   set nusRevFlag        0
   set nusName           None
   set nusInfo           None
   set nusOffset         Auto 
   set nusSampleCount    Auto 
   set nusSampleCountMax 0

   set vdlistName        None
   set vdlistCount       Auto 

   set inPath       .
   set inRoot       None
   set inName       None
   set interName    test.fid

   set outPath      .
   set outRoot      None
   set outName      None
   set maskName     None
   set userOutDir   None

   set scriptName   None
   set temperature  None
   set infoRequest  None
   set infoResult   None
   set infoName     None
   set titleStr     None
   set extraArgs    None 
   set finalArgs    None 
   set verbose      1

   set cdDir        .
   set scriptInfo   ""

   set INMULTx      Auto
   set INMULTy      Auto
   set INMULTz      Auto
   set INMULTa      Auto

   set NUCx         None
   set NUCy         None
   set NUCz         None
   set NUCa         None

   set CARdx        0.000
   set CARdy        0.000
   set CARdz        0.000
   set CARda        0.000

   set NUSZFx       0
   set NUSZFy       0
   set NUSZFz       0
   set NUSZFa       0

   set charX        10
   set charY        16
   set wideMode     0

   if {[flagLoc $ARGV -help]} \
      {
       puts "NMRPipe Conversion Script Creation Utility Version $progVersion."
       puts " -in        binaryInputName"
       puts " -out       NMRPipeOutName"
       puts " -outDir    Base Output Directory."
       puts " -cd        cdDir"
       puts " -script    scriptName"
       puts " -ndim      dimCount"
       puts " -spec      spectralType"
       puts " -mode      convMode"
       puts " -inType    inProtocol"
       puts " -outType   outProtocol"
       puts " -t         temperature"
       puts " -dataScale scale \[Default: 1000.0\]"
       puts " -scans     scans \[Integer or Keyword Auto (Default)\]"
       puts "Options for Calculating Data Intensity Scaling:"
       puts " -scaleByScans    Include Scan Count in Scaling Value (Default)."
       puts " -noscaleByScans  Do Not Include Scan Count in Scaling Value."
       puts " -scaleByVendor   Include Vendor Scaling Value (Default)."
       puts " -noscaleByVendor Do Not Include Vendor Scaling Value."
       puts " -scaleByUser     Include User-Specified -dataScale Value (Default)."
       puts " -noscaleByUser   Do Not Include User-Specified -dataScale Value."
       puts " -noscale         Disable All Scaling."
       puts "General Options:"
       puts " -autoDim     Automatically Determine Dimension Count (Default)."
       puts " -noautoDim   Manually Set Dimension Count."
       puts " -auto        Automatically Read Parameters, Save Script, and Exit."
       puts " -noauto      Interactive Operation (Default)."
       puts " -AUTO        Same as: -auto -nosleep -notk (Bruker, Varian, Delta)."
       puts " -read        Automatically Read Parameters When Started."
       puts " -noread      Do Not Automatically Read Parameters (Default)."
       puts " -textWin     Show the Script Text Window (Default)."
       puts " -notextWin   Hide the Script Text Window."
       puts " -sleep       Insert sleep Command in Conversion Script (Default)."
       puts " -nosleep     Do Not Insert sleep Command in Conversion Script."
       puts " -uniqLab     Update Axis Labels to Be Unique, X-Axis HN (Default)."
       puts " -nouniqLab   Do Not Update Axis Labels."
       puts " -even        Force Last Complex Dimension to Have Even Size (Default)."
       puts " -noeven      Don't Change Odd Size of Last Complex Dimension."
       puts " -verb        Verbose Mode ON (Default)."
       puts " -noverb      Verbose Mode OFF."
       puts " -notk        Suppress Graphics."
       puts " -ioargs      Conversion Takes -in/-out Arguments from Command Line."
       puts "Interleaved Experiments:"
       puts " -pseudo3D    Start Interface in Pseudo-3D Mode."
       puts " -nopseudo3D  Suppress Pseudo-3D Mode."
       puts " -tauScale ts Scale Factor for Tau Values."
       puts "Specification of Tau Values for Interleaved Experiments (Use Only One):"
       puts " -tauFile  tf Text File Input with Tau Values."
       puts " -tauList  tl List of Tau Values." 
       puts " -tauCount tc Number of Tau Values."
       puts "Listing Parameters for Informational Use:"
       puts " -list        List Parameters as Values and Exit (Auto Mode Only)."
       puts " -listArgs    List Parameters as Arguments and Exit (Auto Mode Only)."
       puts " -info     pL Additional Parameters to List (From acqus propar etc)."
       puts " -infoName n  Output Name for Parameter List, or Keyword None."
       puts "Special/Vendor-Specific Arguments:"
       puts " -dmx         Do Oversampling Correction (Bruker/JEOL Delta) (Default)."
       puts " -amx         Oversampling Correction During Processing (also -nodmx)."
       puts " -autoUpdate  Update During Conversion for Data Type and Size (Bruker)."
       puts " -df          Same as -dmx (for JEOL Delta)."
       puts " -nodf        Same as -amx (for JEOL Delta)."
       puts " -xNUC        Suggestion for Measurement Nucleus (-x -y -z -a)."
       puts " -dxCAR       Adjustment Added to Carrier PPM (-dx -dy dz da)."
       puts " -yINMULT     Increment Scaling Factor for Spectral Width (-z -a)."
       puts " -nouseH2O    Do Not Use H2O Chemical Shift to Calibrate PPM."
       puts " -nmrdraw     Run NMRDraw in Process Mode After Conversion."
       puts " -nonmrdraw   Do Not Run NMRDraw After Conversion."
       puts " -rnmrtk      Save Additional Result in RNMRTK Format."
       puts " -nornmttk    Do Not Create an RNMRTK Format Result."

       if {$vj2pipeMode} \
          {
           puts " -vj2pipe     Use VNMRJ vj2pipe Scheme to Extract Parameters (Default)."
           puts " -novj2pipe   Do Not Use vj2pipe Scheme to Extract Parameters."
           puts " -vjppm       Use VNMRJ vj2pipe PPM Scheme (For -vj2pipe Only)."
          } \
       else \
          {
           puts " -vj2pipe     Use VNMRJ vj2pipe Scheme to Extract Parameters."
           puts " -novj2pipe   Do Not Use vj2pipe Scheme to Extract Parameters (Default)."
           puts " -vjppm       Use VNMRJ vj2pipe PPM Scheme (For -vj2pipe Only)."
          }

       puts " -useSW_h     Use Bruker SW_h Value for Spectral Width (Default)."
       puts " -nouseSW_h   Do Not Use Bruker SW_h Value."
       puts " -useO1       Use Bruker O1/BF1 Value for Default Carrier PPM."
       puts " -topspin     Sets -nouseH2O -useSW_h -useO1 options."
       puts " -aqseq   val Overrides aqseq Value in pulse program File."
       puts "Non-Uniform Sampling (NUS) Flags:"
       puts " -nus         Use Non-Uniform Sampling Options."
       puts " -rev         Reverse Items in NUS Schedule Lines."
       puts " -useMask     Create NUS Schedule Mask Output (Default for -nus)."
       puts " -nouseMask   Do Not Create NUS Schedule Mask Output (Also -nomask)."
       puts " -pow2        Round Size After NUS Zero Fill to a Power of 2 (Default)."
       puts " -nopow2      Do Not Round Size After NUS Zero Fill."
       puts "Non-Uniform Sampling (NUS) Handling of Redundant Increments:"
       puts " -avg         Average Redundant Increments (Default)."
       puts " -sum         Sum Redundant Increments."
       puts " -sqrt        Use Sqrt of Redundant Increment Count."
       puts "Non-Uniform Sampling Parameters:"
       puts " -sample      nusScheduleName  Table of Increment Numbers."
       puts " -offset      nusOffset        Usually 0 1 or Auto."
       puts " -sampleCount nusSampleCount   Sample Count or Auto."
       puts " -mask        maskName         Default Mask Output Name."
       puts " -yNUSZF      yZF              Y-Axis NUS Zero Fill Count."
       puts " -zNUSZF      zZF              Z-Axis NUS Zero Fill Count."
       puts " -aNUSZF      aZF              A-Axis NUS Zero Fill Count."
       puts "Special Arguments for Bruker Legacy vdlist-format NUS;"
       puts "These Parameters Can Only Be Set from the Command Line:"
       puts " -vdlist      vdName   Required Argument for vdlist-format."
       puts " -vdCount     vdCount  Vector Pair Count in vdlist, or Keyword Auto."
       puts "Adding arguments to the conversion command:"
       puts " -extra extraInitArgs  -end"
       puts " -final extraFinalArgs -end"
       puts "Resetting Parameters to User-Defined Values:"
       puts " -user -xN ..."
       puts "See Also:"
       puts " bruk2pipe  -help"
       puts " var2pipe   -help"
       puts " delta2pipe -help"
       exit 1
      }

   set loc1 [flagLoc $ARGV -extra]

   if {$loc1 > 0} \
      {
       set locN [flagLoc $ARGV -end]

       if {$locN > 0} \
          {
           set locN [expr $locN - 1]
          } \
       else \
          {
           set locN [expr $ARGC - 1]
          }

       set extraArgs [lrange   $ARGV [expr $loc1 + 1] $locN]
       set ARGV      [lreplace $ARGV $loc1 [expr $locN + 1]]
       set ARGC      [llength  $ARGV]
      }

   set loc1 [flagLoc $ARGV -final]
 
   if {$loc1 > 0} \
      {
       set locN [flagLoc $ARGV -end]

       if {$locN > 0} \
          {
           set locN [expr $locN - 1]
          } \
       else \
          {
           set locN [expr $ARGC - 1]
          }

       set finalArgs [lrange   $ARGV [expr $loc1 + 1] $locN]
       set ARGV      [lreplace $ARGV $loc1 [expr $locN + 1]]
       set ARGC      [llength  $ARGV]
      }

   getArgD $ARGV -in        inName
   getArgD $ARGV -inter     interName
   getArgD $ARGV -out       outName
   getArgD $ARGV -outDir    userOutDir
   getArgD $ARGV -infoName  infoName
   getArgD $ARGV -script    scriptName
   getArgD $ARGV -ndim      dimCount
   getArgD $ARGV -spec      spectralType
   getArgD $ARGV -mode      convMode
   getArgD $ARGV -inType    inProtocol
   getArgD $ARGV -outType   outProtocol
   getArgD $ARGV -cd        cdDir
   getArgD $ARGV -t         temperature

   getArgD $ARGV -scans     userScans
   getArgD $ARGV -dataScale userScale
   getArgD $ARGV -aqseq     userAQSeq

   getArgD $ARGV -mask        maskName
   getArgD $ARGV -sample      nusName
   getArgD $ARGV -offset      nusOffset
   getArgD $ARGV -sampleCount nusSampleCount

   getArgD $ARGV -vdlist      vdlistName
   getArgD $ARGV -vdCount     vdlistCount

   getListArgD $ARGV -tauList tauList

   getArgD $ARGV -interleaveInfo interleaveInfo
   getArgD $ARGV -tauCount       userTauCount
   getArgD $ARGV -tauFile        tauFileName
   getArgD $ARGV -tauScale       tauScale
   getArgD $ARGV -tauFmt         %s

   getArgD $ARGV -xINMULT  INMULTx
   getArgD $ARGV -yINMULT  INMULTy
   getArgD $ARGV -zINMULT  INMULTz
   getArgD $ARGV -aINMULT  INMULTa

   getArgD $ARGV -xNUC     NUCx
   getArgD $ARGV -yNUC     NUCy
   getArgD $ARGV -zNUC     NUCz
   getArgD $ARGV -aNUC     NUCa

   getArgD $ARGV -dxCAR    CARdx
   getArgD $ARGV -dyCAR    CARdy
   getArgD $ARGV -dzCAR    CARdz
   getArgD $ARGV -daCAR    CARda

   getArgD $ARGV -xNUSZF   NUSZFx
   getArgD $ARGV -yNUSZF   NUSZFy
   getArgD $ARGV -zNUSZF   NUSZFz
   getArgD $ARGV -aNUSZF   NUSZFa

   getArgD $ARGV -charX    charX
   getArgD $ARGV -charY    charY

   if {[flagLoc $ARGV -scaleByScans]}    {set scaleByScansFlag  1}
   if {[flagLoc $ARGV -noscaleByScans]}  {set scaleByScansFlag  0}

   if {[flagLoc $ARGV -scaleByUser]}     {set scaleByUserFlag   1}
   if {[flagLoc $ARGV -noscaleByUser]}   {set scaleByUserFlag   0}
 
   if {[flagLoc $ARGV -scaleByVendor]}   {set scaleByVendorFlag 1}
   if {[flagLoc $ARGV -noscaleByVendor]} {set scaleByVendorFlag 0}

   if {[flagLoc $ARGV -scaleByMult]}     {set scaleByMultFlag   1}
   if {[flagLoc $ARGV -scaleByDiv]}      {set scaleByMultFlag   0}

   if {[flagLoc $ARGV -noscale]} \
      {
       set scaleByScansFlag  0
       set scaleByUserFlag   0
       set scaleByVendorFlag 0
      }

   if {[strcasecmp $vdlistName None] && ($NUSZFy > 0 || $NUSZFz > 0 || $NUSZFa > 0)} \
      {
       puts "Warning: NUS Zero Fill is Not Allowed with -vdlist Mode."

       set NUSZFx 0
       set NUSZFy 0
       set NUSZFz 0
       set NUSZFa 0
      }

   set nusName     [fixLocalName $nusName]
   set interName   [fixLocalName $interName]
   set tauFileName [fixLocalName $tauFileName]

   getListArgD $ARGV -info infoRequest

   if {[strcmp $cdDir .]} {catch "cd $cdDir"}

   if {[file exists acqu3s]} {set swhFlag  1}

   if {[flagLoc $ARGV -user] && [flagLoc $ARGV -xCAR]} {set carXSrc User}

   set dimCountName $dimCount

   if {[flagLoc $ARGV -AUTO]} \
      {
       set autoFlag  1
       set sleepFlag 0
      }

   if {[flagLoc $ARGV -pseudo3D]}   {set pseudo3DFlag 1}
   if {[flagLoc $ARGV -nopseudo3D]} {set pseudo3DFlag 0}

   if {$pseudo3DFlag} \
      {
       set pseudoLoopMode 1
       set showPseudoFlag 1
      }

   if {[flagLoc $ARGV -pseudoInner]}  {set pseudoLoopMode 1}
   if {[flagLoc $ARGV -pseudoOuter]}  {set pseudoLoopMode 2}

   if {[flagLoc $ARGV -showPseudo]}   {set showPseudoFlag 1}
   if {[flagLoc $ARGV -noshowPseudo]} {set showPseudoFlag 1}

   if {[flagLoc $ARGV -showOBS]}      {set showObsMidFlag 1}
   if {[flagLoc $ARGV -noshowOBS]}    {set showObsMidFlag 0}

   if {[flagLoc $ARGV -autoDim]}   {set dimCountName N}
   if {[flagLoc $ARGV -noautoDim]} {set dimCountName $dimCount}

   if {[flagLoc $ARGV -pow2]}      {set pow2Flag  1}
   if {[flagLoc $ARGV -nopow2]}    {set pow2Flag  0}

   if {[flagLoc $ARGV -auto]}      {set autoFlag  1}
   if {[flagLoc $ARGV -noauto]}    {set autoFlag  0}

   if {[flagLoc $ARGV -useTemp]}   {set tempFlag  1}
   if {[flagLoc $ARGV -nouseTemp]} {set tempFlag  0}

   if {[flagLoc $ARGV -read]}      {set readFlag  1}
   if {[flagLoc $ARGV -noread]}    {set readFlag  0}

   if {[flagLoc $ARGV -list]}       {set infoFlag  1}
   if {[flagLoc $ARGV -listArgs]}   {set infoFlag  2}
   if {[flagLoc $ARGV -nolist]}     {set infoFlag  0}
   if {[flagLoc $ARGV -nolistArgs]} {set infoFlag  0}

   if {[flagLoc $ARGV -autoUpdate]}   {set autoUpdateFlag 1}
   if {[flagLoc $ARGV -noautoUpdate]} {set autoUpdateFlag 0}

   if {[flagLoc $ARGV -verb]}      {set verbose   1}
   if {[flagLoc $ARGV -noverb]}    {set verbose   0}

   if {[flagLoc $ARGV -topspin]}   {set topspinFlag 1}
   if {[flagLoc $ARGV -notopspin]} {set topspinFlag 0}

   if {[flagLoc $ARGV -nmrbox]}    {set nmrboxFlag 1}
   if {[flagLoc $ARGV -nonmrbox]}  {set nmrboxFlag 0}

   if {[flagLoc $ARGV -rnmrtk]}    {set rnmrtkFlag 1}
   if {[flagLoc $ARGV -nornmrtk]}  {set rnmrtkFlag 0}

   if {[flagLoc $ARGV -even]}      {set evenFlag   1}
   if {[flagLoc $ARGV -noeven]}    {set evenFlag   0}

   if {[flagLoc $ARGV -ioargs]}    {set ioArgsFlag 1}
   if {[flagLoc $ARGV -noioargs]}  {set ioArgsFlag 0}

   if {$topspinFlag} \
      {
       set swhFlag     1
       set o1Flag      1
       set h2oFlag     0
      }

   if {[flagLoc $ARGV -vj2pipe]}   {set vj2pipeMode 1}
   if {[flagLoc $ARGV -novj2pipe]} {set vj2pipeMode 0}

   if {$vj2pipeMode} \
      {
       if {[flagLoc $ARGV -vjppm]}   {set vj2pipeMode 2}
       if {[flagLoc $ARGV -novjppm]} {set vj2pipeMode 1}
      }
   
   if {[flagLoc $ARGV -useSW_h]}   {set swhFlag   1}
   if {[flagLoc $ARGV -nouseSW_h]} {set swhFlag   0}

   if {[flagLoc $ARGV -useO1]}     {set o1Flag    1}
   if {[flagLoc $ARGV -nouseO1]}   {set o1Flag    0}

   if {[flagLoc $ARGV -useH2O]}    {set h2oFlag   1}
   if {[flagLoc $ARGV -nouseH2O]}  {set h2oFlag   0}

   if {[flagLoc $ARGV -nus]}       {set nusFlag   1}
   if {[flagLoc $ARGV -nonus]}     {set nusFlag   0}

   if {[flagLoc $ARGV -sqrt]}    {set nusAvgFlag 2}
   if {[flagLoc $ARGV -avg]}     {set nusAvgFlag 1}
   if {[flagLoc $ARGV -sum]}     {set nusAvgFlag 0}

   if {[flagLoc $ARGV -interleave]}     {set interleaveFlag 1}
   if {[flagLoc $ARGV -nointerleave]}   {set interleaveFlag 0}

   if {[flagLoc $ARGV -interleaved]}    {set interleaveFlag 1}
   if {[flagLoc $ARGV -nointerleaved]}  {set interleaveFlag 0}

   if {$nusFlag} \
      {
       set maskFlag 1
       set dmxMode  0
      }

   if {[flagLoc $ARGV -nmrdraw]}   {set nmrDrawFlag 1}
   if {[flagLoc $ARGV -nonmrdraw]} {set nmrDrawFlag 0} 

   if {[flagLoc $ARGV -dmx]}       {set dmxMode    1}
   if {[flagLoc $ARGV -df]}        {set dmxMode    1}

   if {[flagLoc $ARGV -nodmx]}     {set dmxMode    0}
   if {[flagLoc $ARGV -nodf]}      {set dmxMode    0}
   if {[flagLoc $ARGV -amx]}       {set dmxMode    0}


   if {[flagLoc $ARGV -useMask]}   {set maskFlag   1}
   if {[flagLoc $ARGV -nouseMask]} {set maskFlag   0}
   if {[flagLoc $ARGV -nomask]}    {set maskFlag   0}

   if {[flagLoc $ARGV -rev]}       {set nusRevFlag 1}
   if {[flagLoc $ARGV -norev]}     {set nusRevFlag 0}

   if {[flagLoc $ARGV -sleep]}     {set sleepFlag  1}
   if {[flagLoc $ARGV -nosleep]}   {set sleepFlag  0}

   if {[flagLoc $ARGV -uniqLab]} \
      {
       foreach i {x y z a} {set labUpdateMode($i) 1}
      }

   if {[flagLoc $ARGV -nouniqLab]} \
      {
       foreach i {x y z a} {set labUpdateMode($i) 0}
      }

   if {$autoFlag} {convIconify}
}

#
# Miscellaneous settings:

proc convIconify {} \
{
   global wishFlag

   if {!$wishFlag} return
   wm iconify .
}

proc convRaise {} \
{
   global wishFlag

   if {!$wishFlag} return
   raise .
   update
}

proc setMisc {} \
{
   global winTitle wishFlag

   if {!$wishFlag} return
   wm title . $winTitle
}

#
# First-time initialization of the conversion system.

proc initConv {} \
{
   global dimCount dimCountName initTextArea
   
   setDimCount $dimCountName
   setTextToggle

   if {!$initTextArea} \
      {
       hideTextArea
      }

   setUserParams
   convRaise
}

#
# Determines whether a graphics object should be hidden for this application.

proc hideThis { obj } \
{ 
   global hiddenList

   if {[lsearch -exact $hiddenList $obj] < 0} \
      {
       return 0
      }

   return 1
}

#
# Define primary frame and subframes for the application.

proc createFrames {} \
{
   global convMode pcFlag autoFlag wishFlag charX charX

   if {!$wishFlag} return

   toplevel .scriptFrame

   if {$autoFlag} {wm iconify .scriptFrame}

   if {$pcFlag} \
      {
       wm minsize  . 1 1
       wm geometry . =+0+0
       wm geometry .scriptFrame  =+50-50
      } \
   else \
      {
       set wide [winfo screenwidth .]
       set n    [expr $wide/$charX]

       wm minsize  . 1 1
       wm geometry . =+120+60

       if {$n > 180} \
          {
           wm geometry .scriptFrame  =-100+100
          } \
       else \
          {
           wm geometry .scriptFrame  =+175-100
          }
      }

   frame .hiddenFrame
   frame .nameFrame
   frame .protocolFrame
   frame .mainFrame
   frame .auxFrame
}

#
# Establish layout of primary frames for the application.

proc packFrames {} \
{
   global wishFlag

   if {!$wishFlag} return

   pack .nameFrame -in . -pady 2m -expand 1 -fill x 
   pack .mainFrame -in . -pady 2m
   pack .auxFrame  -in . -pady 1m -side bottom -expand 1 -fill x
}

#
# Establish special bindings for help messages tied to buttons, etc.

proc setBindings {} \
{
   global wishFlag

   if {!$wishFlag} return

   bind Button     <Any-Enter> "+ enterButton %W"
   bind Button     <Any-Leave> "+ leaveButton %W"

   bind Menubutton <Any-Enter> "+ enterMenuButton %W"
   bind Menubutton <Any-Leave> "+ leaveMenuButton %W"

   bind Entry      <ButtonRelease-2> "+ pasteToEntry %W"
}

#
# Event handler for pointer entering a button.

proc enterButton { bName } \
{
   global helpArray

#   tk_butEnter $bName

   if {[info exists helpArray($bName)]} \
      {
       wm title . $helpArray($bName)
       update
      }
}

#
# Event handler for pointer leaving a button.

proc leaveButton { bName } \
{
   global winTitle

#   tk_butLeave $bName

   wm title . $winTitle
   update
}

#
# Event handler for pointer entering a menubutton.

proc enterMenuButton { bName } \
{
   global helpArray

   if {[info exists helpArray($bName)]} \
      {
       wm title . $helpArray($bName)
       update
      }
}

#
# Event handler for pointer leaving a menubutton.

proc leaveMenuButton { bName } \
{
   global winTitle

   wm title . $winTitle
   update
}

#
# Event handler for paste to entry.

proc pasteToEntry { eName } \
{
   set error [catch "set str [selection get]"]

   if {!$error} \
      {
       $eName insert insert [selection get]
      }
}

#
# Adds a graphical item to the help text list.

proc addHelp {item str} \
{
   global helpArray
   set helpArray($item) $str
}

#
# Event handler that sets a given conversion variable.

proc setConvVar args \
{
   global carrierInfo labUpdateMode deferGradMsg planeCountMsg

   set deferGradMsg  0
   set planeCountMsg 0

   foreach i {x y z a} {set origMode($i) $labUpdateMode($i)}

   set varName [lindex $args 0]
   set dimName [lindex $args 1]
   set value   [lindex $args 2]
   set mode    [lindex $args end]

   global $varName$dimName CAR$dimName LAB$dimName POST$dimName
   global SW$dimName OBS$dimName MODE$dimName N$dimName T$dimName

   if {![strcasecmp $varName LAB] && ![strcasecmp $dimName x]} {set labUpdateMode(x) 0} 

   set       $varName$dimName $value
   updateVar .ent$varName$dimName set

#
# Special case for Observe Freq with extra info:

   if {![strcasecmp $varName LAB]} \
      {
       if {![strcasecmp $value TAU] || ![strcasecmp $value ID]} \
          {
           set T$dimName    [subst \$N$dimName]
           set SW$dimName   [expr 1.0*[subst \$N$dimName]]
           set MODE$dimName Real
           set OBS$dimName  1.0
           set CAR$dimName  0.0

           updateVar .entT$dimName    set
           updateVar .entSW$dimName   set
           updateVar .entMODE$dimName set
           updateVar .entOBS$dimName  set
           updateVar .entCAR$dimName  set
          }
      }
 
   if {![strcasecmp $varName OBS]} \
      {
       if {[llength $args] < 6} return

       set obsName [string trim [lindex $args 4] "()"]

       if {[info exists carrierInfo($obsName)]} \
          {
           set         CAR$dimName $carrierInfo($obsName)
           updateVar  .entCAR$dimName set
          }

       set nucName [lindex $args 5]

       set         LAB$dimName  $nucName
       set         POST$dimName ""

       updateVar   .entLAB$dimName set
      }

   foreach i {x y z a} {set labUpdateModeMode($i) $origMode($i)}
}

#
# Creates a column of Axis-specific parameters;
# called for the given dimension name (e.g x, y, z, a, d1, d2...)

proc addDim { dimName } \
{
   global parmNames wishFlag env

   if {!$wishFlag} return

   frame .frame$dimName
   label .dimLabel$dimName -width 14 -text "$dimName-axis" -anchor w
   pack  .dimLabel$dimName -in .frame$dimName -side top 

   foreach i $parmNames \
      {
       global $i$dimName
       frame .frame$i$dimName

       menubutton .menu$i$dimName -menu .menu$i$dimName.menu \
          -relief raised -bitmap @$env(NMRTXT)/caret.map

       entry .ent$i$dimName -width 14 -relief sunken \
          -textvariable $i$dimName

       bind  .ent$i$dimName <Return> "+ updateVar %W return"
   
       if {![hideThis $i]} \
          {
           pack  .ent$i$dimName   -in .frame$i$dimName  -side left
           pack  .menu$i$dimName  -in .frame$i$dimName  -side left
           pack  .frame$i$dimName -in .frame$dimName    -side top
          }
      }

   pack .frame$dimName -in .mainFrame -side left -padx 2m
}

#
# Disable a dimension-specific parameter panel;
# used to change the number of dimensions being converted.

proc disableDim { dimName } \
{
   global parmNames wishFlag TCLTK8

   if {!$wishFlag} return

   set bgColor [option get . background Background]

   .dimLabel$dimName  configure -foreground $bgColor

   foreach i $parmNames \
      {
       if {$TCLTK8} \
          {
           .ent$i$dimName  configure -state disabled \
              -background $bgColor -disabledbackground $bgColor \
              -foreground $bgColor -relief flat
          } \
       else \
          {
           .ent$i$dimName  configure -state disabled \
              -background $bgColor -foreground $bgColor -relief flat
          }

       .menu$i$dimName configure -state disabled            \
          -disabledforeground $bgColor -foreground $bgColor \
          -background $bgColor -relief flat
      }
}

#
# Enable a dimension-specific parameter panel;
# used to change the number of dimensions being converted.

proc enableDim { dimName } \
{
   global parmNames wishFlag

   if {!$wishFlag} return

   set bgColor [option get . background         Background]
   set fgColor [option get . foreground         Foreground]
   set daColor [option get . disabledForeground DisabledForeground]

   .dimLabel$dimName configure -foreground $fgColor

   foreach i $parmNames \
      {
       .ent$i$dimName  configure -state normal \
          -background $bgColor -foreground $fgColor -relief sunken 

       .menu$i$dimName configure -state normal              \
          -disabledforeground $daColor -foreground $fgColor \
          -background $bgColor -relief raised 
      }
}

#
# Creates labels for axis-specific parameter entries.

proc addLabels {} \
{
   global parmLabels parmNames parmHelp wishFlag

   if {!$wishFlag} return

   frame .labelFrame

   label  .labNULL -width 21 -anchor w -relief flat
   pack   .labNULL -in .labelFrame -side top

   foreach i $parmNames \
      {
       button .lab$i  -width 21 -anchor w -relief flat \
          -text $parmLabels($i) -command "showHelp $i" -borderwidth 0 -pady 2 

       addHelp .lab$i $parmHelp($i)

       if {![hideThis $i]} \
          {
           pack .lab$i -in .labelFrame -side top
          }
      }

   pack .labelFrame -in .mainFrame -side left -padx 2m
}

#
# Read the parameters for the selected data file.

proc readParams {} \
{
   global inProtocol bigfont autoFlag readCount wishFlag
   global CARx CARy CARz CARa CARdx CARdy CARdz CARda
   global nusDimCount nusInfo 

   set nusDimCount  0
   set nusInfo      None

   readTitle
   readParams_$inProtocol

   if {[string is double $CARx]} {set CARx [expr $CARx + $CARdx]}
   if {[string is double $CARy]} {set CARy [expr $CARy + $CARdy]}
   if {[string is double $CARz]} {set CARz [expr $CARz + $CARdz]}
   if {[string is double $CARa]} {set CARa [expr $CARa + $CARda]}

   incr readCount

#  updateOutName
   adjComplexDims
   adjNUSZF
   setUserParams

   clearScript
   insertMasterScript

   showReadNeeded 0
   showSaveNeeded 1
}

proc adjComplexDims {} \
{
   global Ny Ty MODEy Nz Tz MODEz Na Ta MODEa dimCount evenFlag verbose

   if {!$evenFlag} return

   if {$dimCount == 2} \
      {
       if {![strcasecmp $MODEy TPPI] || ![strcasecmp $MODEy Real]} return
       if {$Ny < 3 || !($Ny % 2)} return

       if {$verbose} {puts -nonewline "Updating Odd Data Size. Old yN: $Ny yT: $Ty"}

       incr Ny -1
       set  Ty [expr $Ny/2]

       if {$verbose} {puts " New yN: $Ny yT: $Ty"}
      } \
   elseif {$dimCount == 3} \
      {
       if {![strcasecmp $MODEz TPPI] || ![strcasecmp $MODEz Real]} return
       if {$Nz < 3 || !($Nz % 2)} return

       if {$verbose} {puts -nonewline "Updating Odd Data Size. Old zN: $Nz zT: $Tz"}

       incr Nz -1
       set  Tz [expr $Nz/2]

       if {$verbose} {puts " New zN: $Nz zT: $Tz"}
      } \
   elseif {$dimCount == 4} \
      {
       if {![strcasecmp $MODEa TPPI] || ![strcasecmp $MODEa Real]} return
       if {$Na < 3 || !($Na % 2)} return

       if {$verbose} {puts -nonewline "Updating Odd Data Size. Old aN: $Na aT: $Ta"}

       incr Na -1
       set  Ta [expr $Na/2]

       if {$verbose} {puts " New aN: $Na aT: $Ta"}
      }
}

proc adjNUSZF {} \
{
   global dimCount Ny Ty Nz Tz Na Ta NUSZFy NUSZFz NUSZFa pow2Flag verbose

   if {$dimCount >= 2 && $NUSZFy > 0} \
      {
       if {$verbose} {puts -nonewline "Updating for NUS Zero Fill. Old yN: $Ny yT: $Ty"}

       set  Ty [getZFSize $Ty $NUSZFy]
       if {$pow2Flag} {set Ty [nextPower2 $Ty]}

       set Ny [expr 2*$Ty]

       if {$verbose} {puts " New yN: $Ny yT: $Ty"}
      }

   if {$dimCount >= 3 && $NUSZFz > 0} \
      {
       if {$verbose} {puts -nonewline "Updating for NUS Zero Fill. Old zN: $Nz zT: $Tz"}

       set  Tz [getZFSize $Tz $NUSZFz]
       if {$pow2Flag} {set Tz [nextPower2 $Tz]}

       set Nz [expr 2*$Tz]

       if {$verbose} {puts " New zN: $Nz zT: $Tz"}
      }

   if {$dimCount >= 4 && $NUSZFa > 0} \
      {
       if {$verbose} {puts -nonewline "Updating for NUS Zero Fill. Old aN: $Na aT: $Ta"}

       set  Ta [getZFSize $Ta $NUSZFa]
       if {$pow2Flag} {set Ta [nextPower2 $Ta]}

       set Na [expr 2*$Ta]

       if {$verbose} {puts " New aN: $Na aT: $Ta"}
      }
}

proc setUserParams {} \
{
   global Nx Tx MODEx SWx ORIGx FIRSTx LASTx UNITSx OBSx CARx FTx LABx POSTx \
          Ny Ty MODEy SWy ORIGy FIRSTy LASTy UNITSy OBSy CARy FTy LABy POSTy \
          Nz Tz MODEz SWz ORIGz FIRSTz LASTz UNITSz OBSz CARz FTz LABz POSTz \
          Na Ta MODEa SWa ORIGa FIRSTa LASTa UNITSa OBSa CARa FTa LABa POSTa

   global commentStr titleStr aq2D dimCount readCount labUpdateMode verbose

   global ARGC ARGV

   if {$readCount > 1} return
   if {![flagLoc $ARGV -user]} return

   if {[flagLoc $ARGV -xLAB]} {set labUpdateMode(x) 0}
   if {[flagLoc $ARGV -yLAB]} {set labUpdateMode(y) 0}
   if {[flagLoc $ARGV -zLAB]} {set labUpdateMode(z) 0}
   if {[flagLoc $ARGV -aLAB]} {set labUpdateMode(a) 0}

   set origCar $CARx

   foreach dim {x y z a} \
      {
       foreach p {N T MODE SW ORIG FIRST LAST UNITS OBS CAR FT LAB POST} \
          {
           set loc [flagLoc $ARGV -${dim}$p]

           if {$loc} \
              {
               getArgD $ARGV -${dim}$p $p$dim

               if {![strcasecmp $p$dim CARx]} \
                  {
                   if {$readCount && ![strcasecmp $CARx H2O]} \
                      {
                       if {$verbose} {puts "Skipping User Value: -${dim}$p [subst \$$p$dim] Retained: $origCar"}
                       set CARx $origCar
                      } \
                   else \
                      {
                       if {$verbose} {puts "Updating User Value: -${dim}$p [subst \$$p$dim]"}
                      }
                  } \
               else \
                  {
                   if {$verbose} {puts "Updating User Value: -${dim}$p [subst \$$p$dim]"}
                  }
              }
          }
      }

   set loc [getListArgD $ARGV -comment commentStr]

   if {$loc} \
      {
       set commentStr [string trim $commentStr "{}"]
       if {$verbose} {puts "Updating User Value: -comment $commentStr"}
      }

   set loc [getListArgD $ARGV -title   titleStr]

   if {$loc} \
      {
       set titleStr [string trim $titleStr "{}"]
       if {$verbose} {puts "Updating User Value: -title $titleStr"}
      }

   set loc [getArgD $ARGV -ndim    dimCount]
   if {$loc && $verbose} {puts "Updating User Value: -ndim $dimCount"}

   set loc [getArgD $ARGV -aq2D    aq2D]
   if {$loc && $verbose} {puts "Updating User Value: -aq2D $aq2D"}
}

proc adjParams {} \
{
   global Nx Tx MODEx SWx ORIGx FIRSTx LASTx UNITSx OBSx CARx FTx LABx POSTx \
          Ny Ty MODEy SWy ORIGy FIRSTy LASTy UNITSy OBSy CARy FTy LABy POSTy \
          Nz Tz MODEz SWz ORIGz FIRSTz LASTz UNITSz OBSz CARz FTz LABz POSTz \
          Na Ta MODEa SWa ORIGa FIRSTa LASTa UNITSa OBSa CARa FTa LABa POSTa

   if {[string first "." $Nx] >= 0} {set Nx [expr int($Nx)]}
   if {[string first "." $Ny] >= 0} {set Ny [expr int($Ny)]}
   if {[string first "." $Nz] >= 0} {set Nz [expr int($Nx)]}
   if {[string first "." $Na] >= 0} {set Na [expr int($Na)]}

   if {[string first "." $Tx] >= 0} {set Tx [expr int($Tx)]}
   if {[string first "." $Ty] >= 0} {set Ty [expr int($Ty)]}
   if {[string first "." $Tz] >= 0} {set Tz [expr int($Tz)]}
   if {[string first "." $Ta] >= 0} {set Ta [expr int($Ta)]}
}

#
# Read title text file if needed.

proc readTitle {} \
{
   global titleStr inName

   return
   if {[strcasecmp $titleStr None]} return

   set d [file dirname $inName]
   if {![file exists $d/title]} return

   set titleStr [exec cat $d/title] 
}

#
# Save the contents of the text editing area.

proc saveScript {} \
{
   global infoName scriptName scriptInfo autoFlag wishFlag

   saveInfoText

   set fp stdout

   set lineCount 0

   if {[strcmp $scriptName stdout]} \
      {
       if {!$autoFlag} \
          {
           if {$wishFlag} \
              {
               if {![okWrite $scriptName]} {return 0}
              }
          }

       set error [catch {set fp [open $scriptName w 0744]}]

       if {$error} \
          {
           return 1
          }
      }

   if {$wishFlag} \
      {
       scan [.scriptFrame.scriptText index end] %d lineCount

       for {set i 1} {$i < $lineCount} {incr i} \
          {
           .scriptFrame.scriptText mark set first $i.0
           set  str [.scriptFrame.scriptText get  first "first lineend"]
           puts $fp $str
          }
      } \
   else \
      {
       puts $fp $scriptInfo
      }

   if {[strcasecmp $scriptName stdout]} {close $fp}

   incr lineCount -1

   if {!$autoFlag} \
      {
       if {$wishFlag} \
          {
           if {![strcasecmp $infoName None]} \
              {
               set txt   "Saved $lineCount lines in file \"$scriptName\"."
              } \
           else \
              {
               set txt   "Saved $lineCount lines in file \"$scriptName\".\nParameters saved in file \"$infoName\"."
              }

           tk_dialog .saveDialog Save $txt info 0 Continue
          }
      }

   showSaveNeeded 0
   showExecNeeded 1

   return 0
}

proc swapDim { mode } \
{
   global Ny Ty MODEy SWy OBSy CARy LABy Nz Tz POSTy MODEz SWz OBSz CARz LABz POSTz Na Ta MODEa SWa OBSa CARa LABa POSTa
   global deferGradMsg planeCountMsg

   set deferGradMsg  0
   set planeCountMsg 0

   if {![strcasecmp $mode YZ]} \
      {
       rSwap Ny    Nz
       rSwap Ty    Tz
       rSwap MODEy MODEz
       rSwap SWy   SWz
       rSwap OBSy  OBSz
       rSwap CARy  CARz
       rSwap LABy  LABz

       set POSTy ""
       set POSTz ""
       set POSTa ""
      } \
   elseif {![strcasecmp $mode YA]} \
      {
       rSwap Ny    Na
       rSwap Ty    Ta
       rSwap MODEy MODEa
       rSwap SWy   SWa
       rSwap OBSy  OBSa
       rSwap CARy  CARa
       rSwap LABy  LABa

       set POSTy ""
       set POSTz ""
       set POSTa ""
      }


   updateAxisLabels

   clearScript
   insertMasterScript

   showSaveNeeded 1
}

#
# Run the current script.

proc runScript {} \
{
   global appTerm scriptName pcFlag nmrDrawFlag nmrDrawCmnd ioArgsFlag nusFlag verbose

   if {$pcFlag} \
      {
       system sh $scriptName
      } \
   else \
      {
       if {$ioArgsFlag && $nusFlag} \
          {
           if {$verbose} \
              {
               puts "Running Conversion Script:"
               puts " $appTerm -e $scriptName -nusExact"
              }

           exec $appTerm -e $scriptName -nusExact
          } \
       else \
          {
           if {$verbose} \
              {
               puts "Running Conversion Script:"
               puts " $appTerm -e $scriptName"
              }

           exec $appTerm -e $scriptName
          }

       if {$nmrDrawFlag && ![strcmp [lindex $nmrDrawCmnd 0] nmrDraw]} \
          {
           if {$verbose} \
              {
               puts "Running NMRDraw:"
               puts " $nmrDrawCmnd"
              }

           catch {eval "exec $nmrDrawCmnd"}
          }
      }

   showExecNeeded 0
}

#
# Event handler to erase and re-do conversion script in response to var entry.

proc updateVar { eName mode } \
{
   global wishFlag labUpdateMode deferGradMsg planeCountMsg

   set deferGradMsg  0
   set planeCountMsg 0

   foreach i {x y z a} {set origMode($i) $labUpdateMode($i)}

   if {[string first LAB $eName] >= 0} \
      {
       clearAxisLabelPostfix
      }

   if {![strcasecmp $mode return] && [string first LABx $eName] >= 0} {set labUpdateMode(x) 0}
   if {![strcasecmp $mode return] && [string first LABy $eName] >= 0} {set labUpdateMode(y) 0}
   if {![strcasecmp $mode return] && [string first LABz $eName] >= 0} {set labUpdateMode(z) 0}
   if {![strcasecmp $mode return] && [string first LABa $eName] >= 0} {set labUpdateMode(a) 0}

   if {[string first CARx $eName] >= 0} {set carXSrc User}

   if {$wishFlag} \
      {
       set bgColor [option get . background Background]
       set fgColor [option get . foreground Foreground]

       $eName configure -foreground $fgColor -background $bgColor
      }

   clearScript
   insertMasterScript

   foreach i {x y z a} {set labUpdateMode($i) $origMode($i)}

   showSaveNeeded 1
}

#
# Build a complete filename path by catenating a directory and file name.

proc makePath { path file } \
{
   if {![string compare $file None]} {return None}

   set c [string index $path [expr [string length $path] - 1]]

   if {![string compare $c "/"]} \
      {
       return $path$file
      }

   return $path/$file
}

#
# Utility for adjusting script:

proc adjScript { mode loc txt } \
{
   global wishFlag scriptInfo 

   if {$wishFlag} \
      {
       .scriptFrame.scriptText $mode $loc $txt
      } 

   append scriptInfo $txt
}

#
# Insert acomplete script into the text area:

proc insertMasterScript {} \
{
   global dataSource convMode spectralType pcFlag sleepFlag
   global inName inPath outName maskName auxName outAuxName scriptName

   adjParams

   if {![string compare $convMode ISIS]} \
      {
       clearScript

       if {!$pcFlag} {adjScript insert end "#!/bin/csh\n\n"}

       set dataSource [getDataSource $inName]

       insertConvScript $inName $inPath $outName
       insertFTScript $outName

       if {[string compare $auxName None]} \
          {
           set dataSource [getDataSource $auxName]

           insertAuxScript
           insertFTScript $outAuxName
           insertAbsScript $outName $outAuxName
          }
      }

   if {![string compare $convMode SPECTRUM]} \
      {
       if {[strcasecmp $scriptName stdout]} \
          { 
           if {!$pcFlag} {adjScript insert end "#!/bin/csh\n\n"}
          }

       insertConvScript $inName $inPath $outName
      }

#
# Insert pause:

   if {$sleepFlag} \
      {
       adjScript insert end "\n"
       adjScript insert end "sleep 5"
      }
}

#
# Insert an auxillary data conversion script into the text area:

proc insertAuxScript {} \
{
   global auxName auxPath outAuxName

   adjScript insert end "\n"

   insertConvScript $auxName $auxPath $outAuxName
}

#
# Insert a transform script into the text area:

proc insertFTScript { thisInName } \
{
   global shExt

   set thisOutPath [tkFileFn dirname  $thisInName]_ft
   set thisOutRoot [tkFileFn tail     $thisInName]
   set thisOutName [makePath $thisOutPath $thisOutRoot]

   adjScript insert end "\n"
   adjScript insert end "procIR$shExt $thisInName $thisOutName"
   adjScript insert end "\n"
}

#
# Insert an absorbance correction script into the text area:

proc insertAbsScript { specName bgName } \
{
   global shExt

   set thisOutPath [tkFileFn dirname  $specName]_ft
   set thisOutRoot [tkFileFn tail     $specName]
   set specFTName  [makePath $thisOutPath $thisOutRoot]

   set thisOutPath [tkFileFn dirname  $bgName]_ft
   set thisOutRoot [tkFileFn tail     $bgName]
   set bgFTName    [makePath $thisOutPath $thisOutRoot]

   set thisOutPath [tkFileFn dirname  $specName]_abs
   set thisOutRoot [tkFileFn tail     $specName]
   set thisOutPath [string   trimright $thisOutPath "/"]

   adjScript insert end "\n"

   adjScript insert end \
      "absIR$shExt $specFTName $bgFTName $thisOutPath $thisOutRoot 1.0"

   adjScript insert end "\n"
}

#
# Creates the text window for displaying and editing the conversion script.

proc addTextArea {} \
{
   global pcFlag autoFlag nusFlag ioArgsFlag pseudo3DFlag nmrDrawFlag initTextArea maincolor color2 scriptParms nusFlag wishFlag

   if {!$wishFlag} return

   if {$pcFlag} \
      {
       set width 80
      } \
   else \
      {
       set width 100
      }

   wm title .scriptFrame "Conversion Script Text"

   set nn [expr [llength $scriptParms] + 12]

   if {$nusFlag}      {incr nn 7}
   if {$nmrDrawFlag}  {incr nn 3}
   if {$ioArgsFlag}   {incr nn 7}
   if {$pseudo3DFlag} {incr nn 6}

   text .scriptFrame.scriptText -yscrollcommand ".scriptFrame.scrollY set" \
      -height $nn -width $width -relief sunken -borderwidth 3   \
      -background $color2

   scrollbar .scriptFrame.scrollY -command \
      ".scriptFrame.scriptText yview" -relief sunken

   pack .scriptFrame.scrollY    -side right -fill y
   pack .scriptFrame.scriptText -side left -padx 1m -pady 1m -fill y

   if {!$autoFlag} {raise .scriptFrame}
}

#
# Show the text edit port:

proc showTextArea {} \
{
   global wishFlag

   if {$wishFlag} \
      {
       wm deiconify .scriptFrame
       raise .scriptFrame
      }

   setTextToggle
}

#
# Hide the text editting port:

proc hideTextArea {} \
{
   global wishFlag

   if {$wishFlag} \
      {
       wm iconify .scriptFrame
      }

   setTextToggle
}

#
# Event handler to establish input protocol.

proc setInProtocol { val } \
{
   global inProtocol inProtocolName inTypes

   set inProtocolName [getNameVal $inTypes $val]
   set inProtocol     $val
   setProgName

   clearScript
   insertMasterScript
}

#
# Event handler to establish 2D Mode.

proc setAq2DMode { val } \
{
   global aq2DTypes aq2D ySIZE Ny Ty MODEy inProtocol

   set aq2D [getNameVal $aq2DTypes $val]
    
   switch -exact $val \
      {
       REAL      -
       TPPI      - 
       IMAGE     -
       MAGNITUDE {
                  set MODEy Real

                  if {![strcasecmp $inProtocol varian]} \
                     {
                      if {$Ny != $Ty} {set Ny $Ty}
                     } \
                  else \
                     {
                      if {$Ny != $Ty} {set Ty $Ny}
                     }
                 } 

       default   {
                  if {![strcasecmp $MODEy Real]} {set MODEy Complex}
                  if {$Ny == $Ty} {set Ty [expr $Ny/2]}
                 }
      } 

   clearScript
   insertMasterScript

   showSaveNeeded 1
}

#
# Event handler to establish output protocol.

proc setOutProtocol { val } \
{
   global outProtocol outProtocolName outTypes outName

   set outProtocolName [getNameVal $outTypes $val]
   set outProtocol     $val

   if {![strcasecmp $outName None]} \
      {
       if {![strcmp $outProtocol ASCII]}     {set outName fid.txt} 
       if {![strcmp $outProtocol NMRVIEW]}   {set outName fid.nv} 
       if {![strcmp $outProtocol FTNMR_NEW]} {set outName fid.dat} 
      }

   clearScript
   insertMasterScript

   showSaveNeeded 1
}

#
# Event handlers for NUS parameters.

proc setSampleCount { n } \
{
    global nusSampleCount

    if {[strcasecmp $n current]} \
       {
        set nusSampleCount $n
       }

    showGoodEntries nusSampleCount
    showPoorEntries {Ny Ty Nz Tz Na Ta}
    showReadNeeded  1
}

proc setRevFlag { } \
{
   global nusRevFlag

   clearScript
   insertMasterScript

   showReadNeeded  1
}

proc setNUSZF args \
{
    showPoorEntries {Ny Ty Nz Tz Na Ta}
    showReadNeeded  1
}

proc setNUSOffset args \
{
   global nusOffset

   if {[strcasecmp [lindex $args 0] current]} \
      {
       set nusOffset ""
       foreach i $args {lappend nusOffset $i}
      }

   clearScript
   insertMasterScript

   showReadNeeded  1
}

#
# Event handler for running nmrDraw

proc setNMRDrawFlag {} \
{
   global nmrDrawFlag

   clearScript
   insertMasterScript

   showSaveNeeded  1
}

#
# Event handler for toggling use of Bruker's SW_h spectral width value.

proc setSWHFlag {} \
{
   global swhFlag

   clearScript
   insertMasterScript

   showSaveNeeded  1
}

#
# Event handler for saving in RNMRTK format

proc setRNMRTKFlag {} \
{
   global rnmrtkFlag

   clearScript
   insertMasterScript

   showSaveNeeded  1
}

#
# Convert NMRPipe output name to an RNMRTK .sec name.

proc getRNMRTKName { thisName } \
{
   set fmtCount [getFmtCount $thisName]

   if {$fmtCount == 0} \
      {
       set newName [file root $thisName].sec
      } \
   else \
      {
       set tName   [file tail $thisName]
       set loc     [string first "%" $tName]
       set newName [string range $tName 0 [expr $loc - 1]].sec
      }

   return $newName
}

#
# Handle updates to data scale, derived from user values and scan counts.

proc updateDataScale {} \
{
   global userScale userScans dataScale vendorScale vendorScans vendorMsg scaleByScansFlag scaleByUserFlag scaleByVendorFlag scaleByMultFlag

   set dataScale 1.0

   set scanCount $userScans

   if {![strcasecmp $scanCount Auto] || ![strcasecmp $scanCount None] || ![string is integer $scanCount]} {set scanCount $vendorScans}

   if {$scaleByUserFlag   && [strcasecmp $userScale   None] && [string is double $userScale]}   {set dataScale [expr $dataScale*$userScale]}
   if {$scaleByVendorFlag && [strcasecmp $vendorScale None] && [string is double $vendorScale]} {set dataScale [expr $dataScale*$vendorScale]}

   if {$scaleByScansFlag && [strcasecmp $scanCount   None] && [string is integer $scanCount]} \
      {
       if {$scanCount > 0} {set dataScale [expr $dataScale/$scanCount]}
      }

   set dataScale [format %.5e $dataScale]

   clearScript
   insertMasterScript

   showSaveNeeded 1
}

proc setScale args \
{
   global userScale userScans dataScale vendorScale vendorScans vendorMsg scaleByScansFlag scaleByUserFlag scaleByVendorFlag scaleByMultFlag

   set cmnd [lindex $argv 0]

   if {![strcasecmp $cmnd number]} {set dataScale [lindex $args 1]}

   set scaleByScansFlag  0
   set scaleByUserFlag   0
   set scaleByVendorFlag 0
 
   clearScript
   insertMasterScript

   showSaveNeeded 1
}

proc toggleScale args \
{
   global userScale userScans dataScale vendorScale vendorScans vendorMsg scaleByScansFlag scaleByUserFlag scaleByVendorFlag scaleByMultFlag

   updateDataScale

   clearScript
   insertMasterScript

   showSaveNeeded 1
}

#
# Event handler to establish temperature.

proc setTemperature { t } \
{
    global temperature

    switch $t \
       {
        "From File" {set temperature $t}
        "None"      {set temperature $t}
        "current"   {set temperature [format %.3f $temperature]}
        default     {set temperature [format %.3f $t]}
       }

    showPoorEntries {CARx CARy CARz CARa}
    showReadNeeded  1
}

#
# Check for redundant axis labels.

proc clearAxisLabelPostfix {} \
{
   global POSTx POSTy POSTz POSTa LABx LABy LABz LABa

   if {[string length $POSTx]} {set LABx [string trim $LABx $POSTx]}
   if {[string length $POSTy]} {set LABy [string trim $LABy $POSTy]}
   if {[string length $POSTz]} {set LABz [string trim $LABz $POSTz]}
   if {[string length $POSTa]} {set LABa [string trim $LABa $POSTa]}

   set POSTx ""
   set POSTy ""
   set POSTz ""
   set POSTa ""
}

proc normalizeNucName { nucOrig } \
{
   set nuc [string toupper $nucOrig]
   set n   [string length  $nuc]

   set isotope "" 
   set element ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $nuc $i]

       if {[string is digit $c]} \
          {
           append isotope $c
          } \
       else \
          {
           append element $c
          }
      }

   return $isotope$element
}

proc nucIsMatch { nuc1 nuc2 } \
{
   if {![strcasecmp [normalizeNucName $nuc1] [normalizeNucName $nuc2]]} {return 1}
   return 0
}

proc labIs1H { lab } \
{
   set s [string toupper $lab]

   if {[string first H  $s] == 0} {return 1}  
   if {[string first 1H $s] == 0} {return 1}         

   return 0
}

proc labIsHN { lab } \
{
   set s [string toupper $lab]

   if {[string first HN $s] == 0} {return 1}         

   return 0
}

proc labIs15N { lab } \
{
   set s [string toupper $lab]

   if {[string first 15N $s] == 0} {return 1}         
   if {[string first N   $s] == 0} {return 1}  

   return 0
}

proc someLabIs15N {} \
{
   global LABx LABy LABz LABa dimCount

   set sList "$LABx $LABy $LABz $LABa"

   for {set i 1} {$i < $dimCount} {incr i} \
      {
       set sI [lindex $sList $i]
       if {[labIs15N $sI]} {return 1}
      }

   return 0
}

proc updateAxisLabels {} \
{
   global POSTx POSTy POSTz POSTa LABx LABy LABz LABa 
   global labUpdateMode dimCount

   clearAxisLabelPostfix

   if {$dimCount == 1} return

   if {[someLabIs15N]} \
      {
       if {[labIs1H $LABx] && $labUpdateMode(x)} {set LABx HN}
      } \
   else \
      {
       if {[labIs1H $LABx] && $labUpdateMode(x)} {set LABx 1H}
      }

   if {$dimCount == 2} \
      {
       if {![strcasecmp $LABx $LABy]} \
          {
           if {$labUpdateMode(x)} {set POSTx x}
           if {$labUpdateMode(y)} {set POSTy y}
    
           if {$labUpdateMode(x)} {set LABx $LABx$POSTx}
           if {$labUpdateMode(y)} {set LABy $LABy$POSTy}
          }
      } \
   elseif {$dimCount == 3} \
      {
       if {![strcasecmp $LABx $LABy] && $labUpdateMode(y)} {set POSTy y}
       if {![strcasecmp $LABx $LABz] && $labUpdateMode(z)} {set POSTz z}

       if {![strcasecmp $LABy $LABz]} \
          {
           if {$labUpdateMode(y)} {set POSTy y}
           if {$labUpdateMode(z)} {set POSTz z}
          }
      
       if {$labUpdateMode(y)} {set LABy  $LABy$POSTy}
       if {$labUpdateMode(z)} {set LABz  $LABz$POSTz}
      } \
   elseif {$dimCount == 4} \
      {
       if {![strcasecmp $LABx $LABy] && $labUpdateMode(y)} {set POSTy y}
       if {![strcasecmp $LABx $LABz] && $labUpdateMode(z)} {set POSTz z}
       if {![strcasecmp $LABx $LABa] && $labUpdateMode(a)} {set POSTa a}

       if {![strcasecmp $LABy $LABz]} \
          {
           if {$labUpdateMode(y)} {set POSTy y}
           if {$labUpdateMode(z)} {set POSTz z}
          }

       if {![strcasecmp $LABy $LABa]} \
          {
           if {$labUpdateMode(y)} {set POSTy y}
           if {$labUpdateMode(a)} {set POSTa a}
          }

       if {![strcasecmp $LABz $LABa]} \
          {
           if {$labUpdateMode(z)} {set POSTz z}
           if {$labUpdateMode(a)} {set POSTa a}
          }

       if {$labUpdateMode(y)} {set LABy  $LABy$POSTy}
       if {$labUpdateMode(z)} {set LABz  $LABz$POSTz}
       if {$labUpdateMode(a)} {set LABa  $LABa$POSTa}
      }
}

#
# Event handler to establish current dimension count.

proc setDimCount { ndim } \
{
   global dimCount dimCountName axisNames

   clearScript

   set dimCountName $ndim
 
   if {[string compare $ndim N]} \
      {
       set dimCount $ndim
      }

   for {set i 1} {$i <= 4} {incr i} \
      {
       disableDim $axisNames($i)
      }

   for {set i 1} {$i <= $dimCount} {incr i} \
      {
       enableDim $axisNames($i)
      }

   updateOutName
}

#
# Event handler for help button.

proc showHelp { topic } \
{
   set hProc [string trimleft $topic "."]_help

#   set txt   "Help Requested for topic $topic.\nExpecting procedure $hProc."
#   tk_dialog .helpDialog Help $txt info 0 Continue
}

#
# Event handler for quit button.

proc quitProc {} \
{
   exit
}

#
# Event handler to toggle text editting port.

proc toggleText {} \
{
   global wishFlag

   if {$wishFlag} \
      {
       update

       if {[winfo ismapped .scriptFrame]} \
          {
           hideTextArea
          } \
       else \
          {
           showTextArea
          }
      }

   setTextToggle
}

#
# Display string according to whether text port is visible.

proc setTextToggle {} \
{
   global wishFlag

   if {$wishFlag} \
      {
       update
       .txtTogButton configure -text [getTextToggle]
      }
}

#
# Returns string according to whether text port is visible.

proc getTextToggle {} \
{
   global wishFlag

   if {!$wishFlag} {return "Hide Script"}

   update

   if {[winfo ismapped .scriptFrame.scriptText]} \
      {
       return "Hide Script"
      } \
   else \
      {
       return "Show Script"
      }
}

#
# Event handler to erase current script and reset variables.

proc clearAll {} \
{
   global dimCount

   setDimCount $dimCount
   clearScript
}

#
# Event handler to erase current script.

proc clearScript {} \
{
   global wishFlag scriptInfo

   if {$wishFlag} \
      {
       .scriptFrame.scriptText delete 1.0 end
       update
      }

   set scriptInfo ""
}

#
# Return part of a name/value pair:

proc getNameVal { nvList val } \
{
   foreach i $nvList \
      {
       set name    [lindex $i 0]
       set thisVal [lindex $i 1]

       if {![string compare $val $thisVal]} \
          {
           return $name
          }
      }

   return NONE
}

#
# Hilite entries with questionable values.

proc showPoorEntries { pList } \
{
   global slowBGColor slowFGColor wishFlag

   if {!$wishFlag} return

   foreach i $pList \
      {
       if {![winfo exists .ent$i]} continue

       set status [lindex [.ent$i configure -state] 4]

       if {[string compare $status disabled]} \
          {
           .ent$i configure -background $slowBGColor -foreground $slowFGColor
          }
      }
}

proc showGoodEntries { pList } \
{
   global wishFlag

   if {!$wishFlag} return

   set bgColor [option get . background Background]
   set fgColor [option get . foreground Foreground]

   foreach i $pList \
      {
       if {![winfo exists .ent$i]} continue

       set status [lindex [.ent$i configure -state] 4]

       if {[string compare $status disabled]} \
          {
           .ent$i configure -background $bgColor -foreground $fgColor
          }
      }
}

proc showReadNeeded { readNeededFlag } \
{
   global goBGColor goFGColor wishFlag

   if {!$wishFlag} return

   if {$readNeededFlag} \
      {
       .readButton configure -background $goBGColor -foreground $goFGColor
       showSaveNeeded 0
       showExecNeeded 0
      } \
   else \
      {
       set bgColor [option get . background Background]
       set fgColor [option get . foreground Foreground]

       .readButton configure -foreground $fgColor -background $bgColor
      }
}

proc showSaveNeeded { saveNeededFlag } \
{
   global goBGColor goFGColor readCount wishFlag

   if {!$wishFlag} return

   if {$readCount && $saveNeededFlag} \
      {
       .saveButton configure -background $goBGColor -foreground $goFGColor
       showReadNeeded 0
       showExecNeeded 0
      } \
   else \
      {
       set bgColor [option get . background Background]
       set fgColor [option get . foreground Foreground]

       .saveButton configure -foreground $fgColor -background $bgColor
      }
}

proc showExecNeeded { execNeededFlag } \
{
   global goBGColor goFGColor readCount wishFlag

   if {!$wishFlag} return

   if {$readCount && $execNeededFlag} \
      {
       .runButton configure -background $goBGColor -foreground $goFGColor
       showReadNeeded 0
       showSaveNeeded 0
      } \
   else \
      {
       set bgColor [option get . background Background]
       set fgColor [option get . foreground Foreground]

       .runButton configure -foreground $fgColor -background $bgColor
      }
}

proc setNUSName {} \
{
   global nusName nusInfo inRoot inPath wishFlag

   clearScript

   set nusInfo None 

   if {!$wishFlag} return

   set types \
      {
       { {All Files}  *      }
       { {Text}       {.txt} }
       { {sch}        {.sch} }
       { {hdr_3}      {.hdr_3} }
      }

   set oldNUSName $nusName

   set nusName [tk_getOpenFile -title "NUS Schedule Input" -filetypes $types]
   if {![llength $nusName]} {set nusName $oldNUSName}

   set nusName [fixLocalName $nusName]

   clearScript
   insertMasterScript

   showReadNeeded  1
}

proc setTauName {} \
{
   global tauFileName inRoot inPath wishFlag readCount

   clearScript

   if {!$wishFlag} return

   set types \
      {
       { {All Files}  *      }
       { {Text}       {.txt} }
       { {sch}        {.sch} }
       { {hdr_3}      {.hdr_3} }
      }

   set oldTauName $tauFileName

   set tauFileName [tk_getOpenFile -title "Tau Value Input" -filetypes $types]
   if {![llength $tauFileName]} {set nusName $oldTauName}

   set tauFileName [fixLocalName $tauFileName]

   setTauInfo
   clearScript

   if {$readCount} \
      {
       clearScript
       insertMasterScript
      }

#  showReadNeeded  1
}

#
# File Selection procedures:

proc setSpecInName {} \
{
   global inName inRoot inPath wishFlag

   clearScript

   if {!$wishFlag} return

   set types \
      {
       { {All Files}  *      }
       { {ser}        {ser*} }
       { {fid}        {fid*} }
       { {dat}        {.dat .DAT} }
       { {jdf}        {.jdf .JDF} }
       { {seq}        {.seq} }
       { {Text}       {.txt} }
       { {TCL}        {.tcl} }
       { {C-Shell}    {.com .csh} }
       { {K-Shell}    {.ksh} }
       { {Shell}      {.sh}  }
      }

   set oldInName $inName

   set inName [tk_getOpenFile -title "Spectral Data Input" -filetypes $types]

   if {![llength $inName]} \
      {
       set inName $oldInName
       return
      }

   set inName [fixLocalName     $inName]
   set inPath [tkFileFn dirname $inName]
   set inRoot [tkFileFn tail    $inName]

   showReadNeeded  1
}

proc setISISInName {} \
{
   global inName inRoot inPath wishFlag

   clearScript

   if {!$wishFlag} return

   set types \
      {
       { {All Files}  *      }
       { {seq}        {.seq} }
       { {spc}        {.spc} }
       { {dat}        {.dat .DAT} }
       { {ser}        {ser*} }
       { {Text}       {.txt} }
       { {TCL}        {.tcl} }
       { {C-Shell}    {.com} }
       { {K-Shell}    {.ksh} }
       { {Shell}      {.sh}  }
      }

   set inName [tk_getOpenFile -title "Image Data Input" -filetypes $types]
   set inPath [tkFileFn dirname $inName]
   set inRoot [tkFileFn tail    $inName]
}

proc setISISAuxName {} \
{
   global auxName auxRoot auxPath
   global outAuxRoot outAuxPath outAuxName wishFlag

   clearScript

   if {!$wishFlag} return

   set types \
      {
       { {All Files}  *      }
       { {seq}        {.seq} }
       { {spc}        {.spc} }
       { {ser}        {ser*} }
       { {fid}        {fid*} }
       { {dat}        {.dat .DAT} }
       { {Text}       {.txt} }
       { {TCL}        {.tcl} }
       { {C-Shell}    {.com .csh} }
       { {K-Shell}    {.ksh} }
       { {Shell}      {.sh}  }
      }

   set auxName [tk_getOpenFile -title "Image Background Input" -filetypes $types]
   set auxPath [tkFileFn dirname $auxName]
   set auxRoot [tkFileFn tail $auxName]

   set outAuxPath [tkFileFn rootname [tkFileFn tail $auxName]]
   set outAuxPath [tkFileFn rootname $outAuxPath]
   set outAuxName [makePath $outAuxPath $outAuxRoot]
}

#
# Update the output names:

proc updateOutName {} \
{
   global convMode

   if {![string compare $convMode ISIS]} \
      {
       updateISISOutName
      } \
   else \
      {
       updateSpecOutName
      }
}

proc updateSpecOutName {} \
{
   global inProtocol outProtocol dimCount outPath outRoot outName userOutDir maskName inPath pseudo3DFlag
   global ARGV

   if {[flagLoc $ARGV -out] && [strcasecmp $outName None]} \
      {
       set outPath [tkFileFn dirname $outName]
       set outRoot [tkFileFn tail    $outName]
       return
      }

   if {![string compare $inProtocol VARIAN]} \
      {
       if {[file exists fid]} \
          {
           if {[file isdir fid]} \
              {
               set outDir fid 
              } \
           else \
              {
               set outDir data 
              }
          } \
       else \
          {
           set outDir fid 
          }
      } \
   else \
      {
       set outDir fid 
      }

   set basePath $inPath

   if {[strcasecmp $userOutDir None]} {set basePath $userOutDir}

   switch $dimCount \
      {
       1 {
          set outPath  $basePath
          set outRoot  test.fid
          set maskName ./mask.fid 
         }
       2 {
          set outPath  $basePath

          if {$pseudo3DFlag} \
             {
              set outPath  [makePath $basePath $outDir]
              set outRoot  test%03d.fid
              set maskName ./mask/test%03d.fid 
             } \
          else \
             {
              set outRoot  test.fid
              set maskName ./mask.fid 
             }
         }
       3 {
          set outPath  [makePath $basePath $outDir]
          set outRoot  test%03d.fid
          set maskName ./mask/test%03d.fid
         }
       4 {
          set outPath  [makePath $basePath $outDir]

          if {[string first bruker [string tolower $inProtocol]] >= 0 || [string first varian [string tolower $inProtocol]] >= 0} \
             {
              set outRoot  test%04d.fid
              set maskName ./mask/test%04d.fid
             } \
          else \
             {
              set outRoot  test%02d%03d.fid
              set maskName ./mask/test%02d%03d.fid
             }
         }
      }

   if {![string compare $outProtocol ASCII]} \
      {
       set outDir  "" 
       set outPath [makePath $basePath $outDir]
       set outRoot test.txt
      }

   set outName [makePath $outPath $outRoot]
}

proc updateISISOutName {} \
{
   global dimCount outPath outRoot outName inName spectralType

   set outPath "[tkFileFn rootname [tkFileFn tail $inName]]"
   set outPath "[tkFileFn rootname $outPath]"
   set outRoot "[string tolower $spectralType]%03d.dat"
   set outName "[makePath $outPath $outRoot]"
}

#
# Analize a file name to determine the data source.

proc getDataSource { fName } \
{
   switch [tkFileFn extension $fName] \
      {
       .Z      {return ZCAT}
       .gz     {return GZCAT}
       default {return FILE}
      }

   return FILE
}

#
# Adds dimension count, input and output protocol menus.

proc addProtocols {} \
{
   global inProtocol outProtocol dimCount aq2D inTypes outTypes aq2DTypes
   global nusFlag nusRevFlag nusName nusSampleCount dataScale scaleByScansFlag scaleByUserFlag scaleByVendorFlag userScale
   global inProtocolName outProtocolName dimCountName convMode
   global fixedfont env wishFlag

   if {!$wishFlag} return

   frame .protoInFrame 
   frame .protoOutFrame

   frame .dimFrame
   frame .aq2DFrame
   frame .scaleFrame
   frame .temperatureFrame

   set outProtocolName [getNameVal $outTypes $outProtocol]
   set inProtocolName  [getNameVal $inTypes  $inProtocol]

   menubutton .menuInProtocol -menu .menuInProtocol.menu \
      -relief raised -bitmap @$env(NMRTXT)/caret.map

   menubutton .menuOutProtocol -menu .menuOutProtocol.menu \
      -relief raised -bitmap @$env(NMRTXT)/caret.map

   menubutton .menuAq2DMode -menu .menuAq2DMode.menu \
      -relief raised -bitmap @$env(NMRTXT)/caret.map

   menubutton .menuDimCount -menu .menuDimCount.menu \
      -relief raised -bitmap @$env(NMRTXT)/caret.map

   menubutton .menuTemperature -menu .menuTemperature.menu \
      -relief raised -bitmap @$env(NMRTXT)/caret.map

   menubutton .menuScale -menu .menuScale.menu \
      -relief raised -bitmap @$env(NMRTXT)/caret.map

   menu .menuInProtocol.menu  -tearoff False
   menu .menuOutProtocol.menu -tearoff False
   menu .menuDimCount.menu    -tearoff False
   menu .menuAq2DMode.menu    -tearoff False
   menu .menuTemperature.menu -tearoff False
   menu .menuScale.menu       -tearoff False

   foreach i $inTypes \
      {
       set name [lindex $i 0]
       set code [lindex $i 1]

       .menuInProtocol.menu add command \
          -label $name -font $fixedfont -command "setInProtocol $code"
      }

   foreach i $outTypes \
      {
       set name [lindex $i 0]
       set code [lindex $i 1]

       .menuOutProtocol.menu add command \
           -label $name -font $fixedfont -command "setOutProtocol $code"
      }

   foreach i {1 2 3 4 N} \
      {
       .menuDimCount.menu add command -label [format "%sD" $i] \
          -font $fixedfont -command "setDimCount $i"
      }

   foreach i $aq2DTypes \
      {
       set name [lindex $i 0]
       set code [lindex $i 1]

       .menuAq2DMode.menu add command \
           -label $name -font $fixedfont -command "setAq2DMode $code"
      }

  if {[string first BRUKER [string toupper $inProtocol]] >= 0} \
      {
       set sList {1.0 "By Scans" "By NC" "By Scans and NC"}

       .menuScale.menu add command  -label "Set Scale to 1.0" -command "setScale number 1.0\""

       .menuScale.menu add checkbutton -onvalue 1 -offvalue 0 -selectcolor blue -label "Scale by 1/(Scan Count)"         -variable scaleByScansFlag   -command "toggleScale scans"
       .menuScale.menu add checkbutton -onvalue 1 -offvalue 0 -selectcolor blue -label "Scale by 2^acqus(NC)"            -variable scaleByVendorFlag  -command "toggleScale vendor"
       .menuScale.menu add checkbutton -onvalue 1 -offvalue 0 -selectcolor blue -label "Scale by User Value $userScale"  -variable scaleByUserFlag    -command "toggleScale user"
      } \
   else \
      {
       .menuScale.menu add command  -label "Set Scale to 1.0" -command "setScale number 1.0\""

       .menuScale.menu add checkbutton -onvalue 1 -offvalue 0 -selectcolor blue -label "Scale by 1/(Scan Count)"         -variable scaleByScansFlag   -command "toggleScale scans"
       .menuScale.menu add checkbutton -onvalue 1 -offvalue 0 -selectcolor blue -label "Scale by User Value $userScale"  -variable scaleByUserFlag    -command "toggleScale user"
      }

   set tList { \
              "From File" 273 280 290 300 301 302 303 304 305 \
              306 307 308 309 310 311 312 313 \
             }

   foreach i $tList \
      {
       .menuTemperature.menu add command -label [format "%s" $i] \
          -font $fixedfont -command "setTemperature \"$i\""
      }

   button .labInProto  -text "Input Protocol:"  -width 16 -relief flat \
      -anchor w  -command "showHelp inProtocol" -pady 2

   button .labOutProto -text "Output Protocol:" -width 16 -relief flat \
      -anchor w -command "showHelp outProtocol" -pady 2

   button .labDimCount -text "Dimension Count:" -width 16 -relief flat \
      -anchor w -command "showHelp dimCount" -pady 2

   button .labAq2DMode -text "2D Mode:" -width 16 -relief flat \
      -anchor w -command "showHelp aq2DMode" -pady 2

   button .labScale -text "Intensity Scale:" -width 16 -relief flat \
      -anchor w -command "showHelp dataScale" -pady 2

   button .labTemperature -text "Temperature (K):" -width 16 -relief flat \
      -anchor w -command "showHelp temperature" -pady 2

   addHelp .labInProto     "Format of the input data and parameters."
   addHelp .labOutProto    "Desired format of the output data."
   addHelp .labDimCount    "Number of dimensions in the input (N for auto-mode)."
   addHelp .labAq2DMode    "2D Acquisition Mode."
   addHelp .labScale       "Multiply intensities by this scale factor."
   addHelp .labTemperature "Sample Temperature (Used for ppm calculations)."

   entry .inProtocol  -width 15 -relief sunken -textvariable inProtocolName
   entry .outProtocol -width 15 -relief sunken -textvariable outProtocolName
   entry .dimCount    -width 15 -relief sunken -textvariable dimCountName
   entry .aq2DMode    -width 15 -relief sunken -textvariable aq2D 
   entry .dataScale   -width 15 -relief sunken -textvariable dataScale
   entry .temperature -width 15 -relief sunken -textvariable temperature 

   pack .labInProto .inProtocol .menuInProtocol   \
        -in .protoInFrame -side left

   pack .labOutProto .outProtocol .menuOutProtocol \
        -in .protoOutFrame -side left

   pack .labDimCount .dimCount .menuDimCount \
        -in .dimFrame -side left

   pack .labAq2DMode .aq2DMode .menuAq2DMode \
        -in .aq2DFrame -side left

   pack .labScale .dataScale .menuScale \
        -in .scaleFrame -side left 

   pack .labTemperature .temperature .menuTemperature \
        -in .temperatureFrame -side left 

   pack .protoInFrame .protoOutFrame .dimFrame .aq2DFrame -in .protocolFrame  \
        -side top -pady 0

   if {[strcasecmp $convMode ISIS]} \
      {
       pack .scaleFrame .temperatureFrame -in .protocolFrame -side top -pady 0
      }

   bind .temperature <Return> "+ setTemperature current"
   bind .dataScale   <Return> "+ setScale       current"

   if {$nusFlag} \
      {
       frame .sampleFrame
       frame .offsetFrame
       frame .revFrame

       menubutton .menuSampleCount -menu .menuSampleCount.menu \
         -relief raised -bitmap @$env(NMRTXT)/caret.map

       menubutton .menuNUSOffset -menu .menuNUSOffset.menu \
         -relief raised -bitmap @$env(NMRTXT)/caret.map

       menu .menuSampleCount.menu -tearoff False
       menu .menuNUSOffset.menu   -tearoff False

       foreach i {Auto 128 512 1024 2048 4096 8192} \
          {
           .menuSampleCount.menu add command -label $i -font $fixedfont -command "setSampleCount $i"
          }

       .menuNUSOffset.menu add command -label "Auto  (2D-4D)"                  -font $fixedfont -command "setNUSOffset Auto"
       .menuNUSOffset.menu add command -label " 0    (2D-4D)"                  -font $fixedfont -command "setNUSOffset 0"
       .menuNUSOffset.menu add command -label " 1    (2D-4D)"                  -font $fixedfont -command "setNUSOffset 1"
       .menuNUSOffset.menu add command -label "-1    (Usually for 2D P1=360)"  -font $fixedfont -command "setNUSOffset -1"

       foreach i {" 0  0" " 1  1" " 0 -1" "-1 -1"} \
          {
           .menuNUSOffset.menu add command -label "$i (3D Only)" -font $fixedfont -command "setNUSOffset $i"
          }

       foreach i {" 0  0  0" " 1  1  1" "-1  0  0" " 0 -1  0" " 0  0 -1" " 0 -1 -1" "-1 -1 -1"} \
          {
           .menuNUSOffset.menu add command -label "$i (4D Only)" -font $fixedfont -command "setNUSOffset $i"
          }

       button .labSampleCount -text "NUS Samples:" -width 16 -relief flat \
          -anchor w -command "showHelp sampleCount" -pady 2

       button .labNUSOffset -text "NUS Index Offsets:" -width 16 -relief flat \
          -anchor w -command "showHelp nusOffset" -pady 2

       addHelp .labSampleCount "Number of NUS Samples Acquired."
       addHelp .labNUSOffset   "Offsets Subtracted from Schedule Values, Usually -1, 0, 1 or Auto."

       entry .entnusSampleCount -width 15 -relief sunken -textvariable nusSampleCount
       entry .nusOffset         -width 15 -relief sunken -textvariable nusOffset

       checkbutton .revFlag -text "Reverse NUS Column Order" -variable nusRevFlag -selectcolor blue -command setRevFlag

       pack .labSampleCount .entnusSampleCount .menuSampleCount -in .sampleFrame -side left
       pack .labNUSOffset   .nusOffset   .menuNUSOffset   -in .offsetFrame -side left

       pack .revFlag -in .revFrame -side left

       pack .sampleFrame .offsetFrame .revFrame -in .protocolFrame -side top -pady 0

       bind .entnusSampleCount <Return> "+ setSampleCount current"
       bind .nusOffset         <Return> "+ setNUSOffset   current"
      }
}

#--------------------------------------------------#
# -- Vendor and Application Specific Procedures -- # 
#--------------------------------------------------#

#
# Creates file name panel for all applications.

proc addNames {} \
{
   global convMode

   if {![string compare $convMode ISIS]} \
      {
       addISISNames
      } \
   else \
      {
       addSpecNames
      }
}

#
# Creates special input panel as needed.

proc addSpecial {} \
{
   global inProtocol

   addPseudoNDFrame

   if {[string first BRUKER [string toupper $inProtocol]] >= 0} \
      {
       addDMXFrame
       addBrukerCSFrame
      } \
   elseif {[string first JEOL_DELTA [string toupper $inProtocol]] >= 0} \
      {
       addDMXFrame
      }

   addNMRExtraFrame
}

#
# Creates options for Pseudo-ND data.

proc addPseudoNDFrame {} \
{
   global showPseudoFlag pseudoLoopMode wishFlag

   if {!$wishFlag || !$showPseudoFlag} return

   frame .pseudoFrame

   label .pseudoFrame.lab -text "Pseudo-ND Loop:"

   radiobutton .pseudoFrame.inner -variable pseudoLoopMode -value 1 -text "Inner" -selectcolor green -command updatePseudoMode
   radiobutton .pseudoFrame.outer -variable pseudoLoopMode -value 2 -text "Outer" -selectcolor green -command updatePseudoMode
   radiobutton .pseudoFrame.none  -variable pseudoLoopMode -value 0 -text "None"  -selectcolor green -command updatePseudoMode

   pack .pseudoFrame.lab .pseudoFrame.inner .pseudoFrame.outer .pseudoFrame.outer .pseudoFrame.none -in .pseudoFrame -anchor c -side left
   pack .pseudoFrame -in .mainFrame -side top
}

#
# Creates options for DMX digital filter adjustment

proc addDMXFrame {} \
{
   global dmxMode wishFlag

   if {!$wishFlag} return

   frame .dmxFrame

   label .dmxFrame.lab -text "Digital Oversampling Correction:"

   radiobutton \
    .dmxFrame.dmx -variable dmxMode -value 1 -text "During Conversion (Normal FID)" \
    -selectcolor green -command updateDMX

   radiobutton \
    .dmxFrame.amx -variable dmxMode -value 0 -text "During Processing (Better Baseline)" \
    -selectcolor green -command updateDMX

   pack .dmxFrame.lab .dmxFrame.dmx .dmxFrame.amx \
        -in .dmxFrame -anchor c -side left

   pack .dmxFrame -in .mainFrame -side top
}

proc addBrukerCSFrame {} \
{
   global wishFlag o1Flag swhFlag h2oFlag topspinFlag

   if {!$wishFlag} return

   frame .brcsFrame

   label .brcsFrame.lab -text "Chemical Shift Calibration: "

   radiobutton \
    .brcsFrame.h2o -variable topspinFlag -value 0 -text "From H2O and Temperature" \
    -selectcolor green -command updateTopspinFlag

   radiobutton \
    .brcsFrame.o1  -variable topspinFlag -value 1 -text "From O1/BF1" \
    -selectcolor green -command updateTopspinFlag

   label .brcsFrame.space1 -text "   "

   checkbutton .brcsFrame.swh -text "Use SW_h" -variable swhFlag -selectcolor blue -command setSWHFlag

   pack .brcsFrame.lab .brcsFrame.h2o .brcsFrame.o1 .brcsFrame.space1 .brcsFrame.swh \
        -in .brcsFrame -anchor c -side left

   pack .brcsFrame -in .mainFrame -side top
}

proc addNMRExtraFrame {} \
{
   global wishFlag nmrDrawFlag nmrboxFlag rnmrtkFlag interleaveFlag interleaveInfo tauFileName inProtocol fixedfont env

   if {!$wishFlag} return

   frame .nmrDrawFrame
   frame .extraSpaceFrame -height 0.5c

   checkbutton .nmrDrawFrame.proc -text "Run NMRDraw in Process Mode After Conversion" -variable nmrDrawFlag -selectcolor blue -command setNMRDrawFlag

   if {$nmrboxFlag} \
      {
       checkbutton .nmrDrawFrame.box   -text "Also Save as RNMRTK Format" -variable rnmrtkFlag -selectcolor blue -command setRNMRTKFlag
       label       .nmrDrawFrame.space -text "  "

       pack .nmrDrawFrame.proc .nmrDrawFrame.space .nmrDrawFrame.box -in .nmrDrawFrame -anchor c -side left
      } \
   else \
      {
       pack .nmrDrawFrame.proc -in .nmrDrawFrame -anchor c -side left
      }

   pack .nmrDrawFrame -in .mainFrame -side top

   if {$interleaveFlag} \
      {
       set thisInfo { None 
                      "yTime yQuadrature   tau" 
                      "yTime yEchoAntiEcho tau"
                      ""
                      "yTime tau yQuadrature"
                      "yTime tau yEchoAntiEcho"
                      ""
                      "tau yTime yQuadrature"
                      "tau yTime yEchoAntiEcho" }
    
       frame .interleaveModeFrame
       frame .interleaveSpaceFrame -height 0.25c

       label .interleaveModeLab -text "Interleave Order:"
       entry .interleaveMode -width 26 -textvariable interleaveInfo

       button .interleaveModeButton -relief raised -bitmap @$env(NMRTXT)/caret.map

       menu  .interleaveModeButton.menu -tearoff False

       foreach i $thisInfo \
          {
           .interleaveModeButton.menu add command -label $i -font $fixedfont -command "setInterleaveMode $i menu"
          }

       pack .interleaveModeLab .interleaveMode .interleaveModeButton -in .interleaveModeFrame -anchor c -side left
       pack .interleaveSpaceFrame .interleaveModeFrame -in .mainFrame -side top
      }

   pack .extraSpaceFrame -in .mainFrame -side top
}

#
# Creates file name panel for ISIS application.

proc addISISNames {} \
{
   global inName auxName outName outAuxName scriptName env
   global goFGColor fixedfont specialArgs wishFlag

   if {!$wishFlag} return

   frame .inNameFrame
   frame .auxNameFrame
   frame .spaceFrame1 -height 0.5c
   frame .outNameFrame
   frame .outAuxNameFrame
   frame .spaceFrame2 -height 0.5c
   frame .scriptNameFrame
   frame .otherOptionsFrame
   frame .nameFrameLeft
   frame .nameFrameRight

   catch {image delete pipeImage}

   image create photo pipeImage -file $env(NMRTXT)/frank.gif

   button .labInName      -width 27 -anchor w -relief flat \
      -text " Spectral Image Input:" -command "showHelp inPath"

   button .labAuxName     -width 27 -anchor w -relief flat \
      -text " Spectral Background Input:" -command "showHelp auxPath"

   button .labOutName     -width 27 -anchor w -relief flat \
      -text " Spectral Image Output:"   -command "showHelp outTemplate"

   button .labOutAuxName  -width 27 -anchor w -relief flat \
      -text " Spectral Background Output:"  -command "showHelp outAuxTemplate"

   button .labScriptName  -width 27 -anchor w -relief flat \
      -text " Output Script:"     -command "showHelp outScript"

   button .labOtherOptions -width 27 -anchor w -relief flat \
      -text " Other Options:"   -command "showHelp otherOptions"

   button .convLogo -relief sunken -image pipeImage -command "showHelp logo"

   addHelp .labInName       "The name of the raw spectral data file."
   addHelp .labAuxName      "The name of the raw background (air) data file."
   addHelp .labOutName      "The processed spectral image name."
   addHelp .labOutAuxName   "The processed spectral background name."
   addHelp .labScriptName   "The UNIX command script name."
   addHelp .labOtherOptions "Miscellaneous Conversion Options."
   addHelp .convLogo        "Display information about this application."

   button .menuInName  -command setISISInName \
      -relief raised -bitmap @$env(NMRTXT)/caret.map -fg $goFGColor

   button .menuAuxName -command setISISAuxName \
      -relief raised -bitmap @$env(NMRTXT)/caret.map -fg $goFGColor

   addHelp .menuInName   "File Browser for selection of spectral input."
   addHelp .menuAuxName  "File Browser for selection of spectral background."

   entry .inName       -relief sunken -width 32 -textvariable inName
   entry .auxName      -relief sunken -width 32 -textvariable auxName
   entry .outName      -relief sunken -width 32 -textvariable outName
   entry .outAuxName   -relief sunken -width 32 -textvariable outAuxName
   entry .scriptName   -relief sunken -width 32 -textvariable scriptName
   entry .otherOptions -relief sunken -width 32 -textvariable specialArgs

   bind  .inName       <Return> "+ updateVar %W return"
   bind  .auxName      <Return> "+ updateVar %W return"
   bind  .outName      <Return> "+ updateVar %W return"
   bind  .outAuxName   <Return> "+ updateVar %W return"
   bind  .otherOptions <Return> "+ updateVar %W return"
   
   pack .labInName  .inName  .menuInName  -in .inNameFrame  -side left
   pack .labAuxName .auxName .menuAuxName -in .auxNameFrame -side left

   pack .labOutName     .outName       -in .outNameFrame     -side left
   pack .labOutAuxName  .outAuxName    -in .outAuxNameFrame  -side left

   pack .labScriptName   .scriptName   -in .scriptNameFrame   -side left
   pack .labOtherOptions .otherOptions -in .otherOptionsFrame -side left

   pack .inNameFrame .auxNameFrame .spaceFrame1 .outNameFrame .outAuxNameFrame \
        .spaceFrame2 .scriptNameFrame .otherOptionsFrame \
        -in .nameFrameLeft -side top -anchor w


   pack .nameFrameLeft  -side left -in .nameFrame 
   pack .nameFrameRight -side left -in .nameFrame
   pack .protocolFrame  -side left -in .nameFrame -padx 3m
   pack .convLogo       -side left -in .nameFrame -padx 3m
}

#
# Creates file name panel for Spectral applications.

proc addSpecNames {} \
{
   global inName outName nusName maskName tauFileName nusFlag scriptName env
   global goFGColor fixedfont specialArgs wishFlag showPseudoFlag
   global NUSZFy NUSZFz NUSZFa

   if {!$wishFlag} return

   frame .inNameFrame
   frame .nusNameFrame
   frame .nusMaskFrame
   frame .nusZFFrame
   frame .tauFileFrame
   frame .spaceFrame -height 0.5c
   frame .outNameFrame
   frame .interNameFrame
   frame .scriptNameFrame
   frame .otherOptionsFrame
   frame .nameFrameLeft
   frame .nameFrameRight

   catch {image delete pipeImage}

#  image create photo pipeImage -file $env(NMRTXT)/frank.gif
   image create photo pipeImage -file $env(NMRTXT)/nmrpipe_logo_blackbg.gif

   button .labInName      -width 20 -anchor w -relief flat \
      -text " Spectrometer Input:" -command "showHelp inPath"      -pady 2

   if {$nusFlag} \
      {
       button .labNUSName      -width 20 -anchor w -relief flat \
        -text " NUS Schedule:"    -command "showHelp nusSchedule"     -pady 2

       button .labMaskName     -width 20 -anchor w -relief flat \
        -text " NUS Mask Output:" -command "showHelp nusMask"  -pady 2
      }

   if {$showPseudoFlag} \
      {
       button .labTauName      -width 20 -anchor w -relief flat \
        -text " Tau Value File:" -command "showHelp tauFile"     -pady 2

       button .labInterName    -width 20 -anchor w -relief flat \
        -text " Intermediate Output:" -command "showHelp interOutput"  -pady 2
      }

   button .labOutName     -width 20 -anchor w -relief flat \
      -text " Output Template:"   -command "showHelp outTemplate"  -pady 2

   button .labScriptName  -width 20 -anchor w -relief flat \
      -text " Output Script:"     -command "showHelp outScript"    -pady 2

   button .labOtherOptions  -width 20 -anchor w -relief flat \
      -text " Other Options:"     -command "showHelp otherOptions" -pady 2

   button .convLogo -relief ridge -image pipeImage -command "showHelp logo"

   addHelp .labInName       "The name of the spectrometer's binary input data."
   addHelp .labNUSName      "The NON-Uniform Sampling schedule text file input."
   addHelp .labOutName      "The name for the NMRPipe format output."
   addHelp .labScriptName   "The name for the NMRPipe conversion script."
   addHelp .labOtherOptions "Miscellaneous Options."
   addHelp .convLogo        "Display information about this application."

   if {$showPseudoFlag} \
      {
       addHelp .labTauName      "The text file input of tau values for interleaved data."
       addHelp .labInterName    "The name for an intermediate NMRPipe format output."
      }

   button .menuInName -command setSpecInName \
      -relief raised -bitmap @$env(NMRTXT)/caret.map -fg $goFGColor

   addHelp .menuInName "File Browser for selection of spectrometer input data."

   if {$nusFlag} \
      {
       button .labNUSZF -width 22 -anchor w -relief flat \
          -text " NUS Zero Fill:"   -command "showHelp NUSZF"

       button .menuNUSName -command setNUSName \
          -relief raised -bitmap @$env(NMRTXT)/caret.map -fg $goFGColor

       addHelp .menuNUSName "File Browser for selection of NUS schedule input."
       addHelp .labNUSZF    "Extend Data by NUS Zero Fill (for NMRPipe IST Reconstruction Only)."
      }

   if {$showPseudoFlag} \
      {
       button .menuTauName -command setTauName \
          -relief raised -bitmap @$env(NMRTXT)/caret.map -fg $goFGColor

       addHelp .menuTauName "File Browser for selection of Tau value input."
      }

   entry .inName       -relief sunken -width 32 -textvariable inName
   entry .outName      -relief sunken -width 32 -textvariable outName
   entry .scriptName   -relief sunken -width 32 -textvariable scriptName
   entry .otherOptions -relief sunken -width 32 -textvariable specialArgs

   if {$nusFlag} \
      {
       entry .nusName   -relief sunken -width 32 -textvariable nusName
       entry .maskName  -relief sunken -width 32 -textvariable maskName

       label .labNUSZFy -text "Y:" -width 3
       label .labNUSZFz -text "Z:" -width 4
       label .labNUSZFa -text "A:" -width 4

       spinbox .nusZFy -width 2 -textvariable NUSZFy -increment 1 -from 0 -to 16 -format %.0f -command "setNUSZF Y"
       spinbox .nusZFz -width 2 -textvariable NUSZFz -increment 1 -from 0 -to 16 -format %.0f -command "setNUSZF Z"
       spinbox .nusZFa -width 2 -textvariable NUSZFa -increment 1 -from 0 -to 16 -format %.0f -command "setNUSZF A"
      }

   if {$showPseudoFlag} \
      {
       entry .tauName   -relief sunken -width 32 -textvariable tauFileName
       entry .interName -relief sunken -width 32 -textvariable interName
      }

   bind  .inName       <Return> "+ updateVar %W return"
   bind  .outName      <Return> "+ updateVar %W return"
   bind  .otherOptions <Return> "+ updateVar %W return"

   if {$nusFlag} \
      {
       bind .nusName   <Return> "+ updateVar %W return"
      }

   pack .labInName .inName .menuInName -in .inNameFrame -side left

   if {$nusFlag} \
      {
       pack .labNUSName  .nusName  .menuNUSName -in .nusNameFrame -side left
       pack .labMaskName .maskName              -in .nusMaskFrame -side left

       pack .labNUSZF .labNUSZFy .nusZFy .labNUSZFz .nusZFz .labNUSZFa .nusZFa -in .nusZFFrame -side left
      }

   if {$showPseudoFlag} \
      {
       pack .labTauName   .tauName   .menuTauName -in .tauFileFrame   -side left
       pack .labInterName .interName              -in .interNameFrame -side left
      }

   pack .labOutName    .outName    -in .outNameFrame    -side left
   pack .labScriptName .scriptName -in .scriptNameFrame -side left

   pack .labOtherOptions .otherOptions -in .otherOptionsFrame -side left

   if {$nusFlag} \
      {
       if {$showPseudoFlag} \
          {
           pack .inNameFrame .nusNameFrame .tauFileFrame .interNameFrame .outNameFrame .nusMaskFrame \
                .scriptNameFrame .otherOptionsFrame -in .nameFrameLeft -side top -anchor w
          } \
       else \
          {
           pack .inNameFrame .nusNameFrame .outNameFrame .nusMaskFrame .scriptNameFrame \
                .otherOptionsFrame -in .nameFrameLeft -side top -anchor w
          }
      } \
   else \
      {
       if {$showPseudoFlag} \
          {
           pack .inNameFrame .tauFileFrame .interNameFrame .outNameFrame .scriptNameFrame \
                .otherOptionsFrame -in .nameFrameLeft -side top -anchor w
          } \
       else \
          {
           pack .inNameFrame .outNameFrame .scriptNameFrame \
                .otherOptionsFrame -in .nameFrameLeft -side top -anchor w
          }
      }

   pack .nameFrameLeft  -side left -in .nameFrame 
   pack .nameFrameRight -side left -in .nameFrame
   pack .protocolFrame  -side left -in .nameFrame -padx 3m
   pack .convLogo       -side left -in .nameFrame -padx 3m
}

#
# Creates the auxilary command panel.

proc addAuxArea {} \
{
   global wishFlag

   if {!$wishFlag} return

   button .readButton   -text "Read Parameters" -command {readParams}
   button .saveButton   -text "Save Script"     -command {saveScript}
   button .runButton    -text "Execute Script"  -command {runScript}
   button .swapYZButton -text "Swap YZ"         -command {swapDim yz}
   button .swapYAButton -text "Swap YA"         -command {swapDim ya}
   button .txtTogButton -text "Show Script"     -command {toggleText}
   button .clearButton  -text "Clear Script"    -command {clearAll}
   button .updateButton -text "Update Script"   -command {insertMasterScript}
   button .quitButton   -text "Quit"            -command {quitProc}
   button .helpButton   -text "Help"            -command {showHelp all}

   addHelp .runButton    "Execute the current conversion script."

   addHelp .txtTogButton "Show or Hide the conversion script text window."
   addHelp .clearButton  "Clear the conversion script text window."
   addHelp .swapYZButton "Swap parameters for the Y and Z Axis."
   addHelp .swapYAButton "Swap parameters for the Y and A Axis."
   addHelp .updateButton "Update the contents of the script text window."
   addHelp .saveButton   "Save the contents of the script text window."
   addHelp .quitButton   "Exit this conversion application." 
   addHelp .helpButton   "Display help text about this application."

   pack .readButton   .saveButton .runButton .swapYZButton .swapYAButton .clearButton  \
        .updateButton .quitButton -in .auxFrame -side left -expand 1 -fill x
}

#
# Creates menu of options for acquisition mode of directly-detected axis.

proc createAcqMenu { dimName } \
{
   global fixedfont wishFlag

   if {!$wishFlag} return

   menu .menuMODE$dimName.menu -tearoff False

   set modeNames \
     {Complex Real Sequential DQD Complex-N}

   foreach i $modeNames \
      {
       .menuMODE$dimName.menu add command -label $i -font $fixedfont \
          -command "setConvVar MODE $dimName \"$i\" menu"
      }
}

#
# Creates menu of options for acquisition mode of indirectly-detected axis.

proc createIndMenu { dimName } \
{
   global fixedfont wishFlag

   if {!$wishFlag} return

   menu .menuMODE$dimName.menu -tearoff False

   set modeNames \
     {
      Complex   States   States-TPPI   Rance-Kay Echo-AntiEcho \
      Complex-N States-N States-TPPI-N TPPI      Real
     }

   set modeNames \
     {
      Complex Real Rance-Kay Echo-AntiEcho TPPI
     }

   foreach i $modeNames \
      {
       .menuMODE$dimName.menu add command -label $i -font $fixedfont \
          -command "setConvVar MODE $dimName \"$i\" menu"
      }
}

#
# Creates menus of options for general axis-specific parameters.
 
proc createGenMenu { dimName } \
{
   global convMode wishFlag
   global fixedfont

   if {!$wishFlag} return

   set listList {N T SW ORIG FIRST LAST UNITS OBS CAR FT LAB}

   if {![string compare $convMode ISIS]} \
      { 
       set listN     { 64 124 125 128 256 512 1024 }
       set listT     { 64 124 125 128 256 512 1024 }
       set listSW    { 1975.0 3950.0 5925.0 7900.0 }
       set listFIRST { 0.0 0.500 1975.0 3950.0 5925.0 7900.0 }
       set listLAST  { 0.0 0.500 1975.0 3950.0 5925.0 7900.0} 
       set listUNITS { pts u mm cm m sec hz wn % }
       set listOBS   { 1.0 10.0 100.0 }
       set listORIG  { 0.0 }
       set listCAR   { 2954.79 }
       set listFT    { Time Freq }
       set listLAB   { X Y IR }
      } \
   else \
      { 
       set listN     {2048 1024 512 256 128 64 32 2 1}
       set listT     {1024  512 256 128 64 32  16 2 1}
       set listSW    {8000.0 5000.0 2000.0 1250.0}
       set listFIRST {0.0 1.0}
       set listLAST  {0.0 100.0 1000.0} 
       set listUNITS {sec pts hz ppm %}
       set listOBS   {750.0 600.0 500.0 150.0 60.0 1.0}
       set listORIG  { 0.0 }
       set listCAR   {H2O 4.70 4.71 4.72 4.73 4.74 4.75 4.76 0.0}
       set listFT    {Time Freq}
       set listLAB   {HN H H-acq H-ind HA HAHB N CO CA CACB C F1 F2 F3 F4 1H 13C 15N TAU ID}
      }

   foreach i $listList \
      {
       menu .menu$i$dimName.menu -tearoff False

       foreach j [eval concat \$list$i] \
          {
           .menu$i$dimName.menu add command -label $j -font $fixedfont \
             -command "setConvVar $i $dimName \"$j\" menu"
          }
      }
}

#
# Updates script based on pseudoND mode setting.

proc updatePseudoMode args \
{
   global pseudo3DFlag pseudoLoopMode inProtocol readCount verbose

   if {$pseudoLoopMode}  \
      {
       if {$verbose && !$pseudo3DFlag} {puts "Setting Pseudo-ND Mode ON."}
       set pseudo3DFlag 1
      } \
   else \
      {
       if {$verbose && $pseudo3DFlag} {puts "Setting Pseudo-ND Mode Off."}
       set pseudo3DFlag 0
      }

   clearScript

   showReadNeeded 1
   showSaveNeeded 0
}

#
# Updates script based on DMX mode setting.

proc updateDMX args \
{
   global dmxMode inProtocol readCount

   clearScript

   catch {updateSpecial_$inProtocol}

   if {$readCount} insertMasterScript

   showSaveNeeded 1
}

proc updateTopspinFlag args \
{
   global topspinFlag o1Flag h2oFlag swhFlag readCount carXSrc CARx

   if {$topspinFlag} \
      {
       set o1Flag  1
       set swhFlag 1
       set h2oFlag 0
       set carXSrc None
       if {![strcasecmp $CARx H2O]} {set CARx 4.70}
      } \
   else \
      {
       set o1Flag  0
       set swhFlag 1
       set h2oFlag 1
       set carXSrc H2O
       set CARx    H2O
      }

   clearScript

   showReadNeeded 1
}

#
# Updates menu for general axis-specific parameters.

proc updateGenMenu { parmName dimName nameValPairs } \
{
   global fixedfont wishFlag

   if {!$wishFlag} return

   .menu$parmName$dimName.menu delete 0 last

   foreach i $nameValPairs \
      {
       set str [lindex $i 0]
       set val [lindex $i 1]

       .menu$parmName$dimName.menu add command -label $str -font $fixedfont \
          -command "setConvVar $parmName $dimName $val $str menu"
      }
}

#
# set modes for interleave experiments

proc setInterleaveMode args \
{
   puts "setInterleaveMode $args"
}

#
# setGamma: establish NMR gamma values for PPM calculation.

proc setGamma {} \
{
    global gammaList

    set gammaList(H)    1.0
    set gammaList(1H)   1.0
    set gammaList(H1)   1.0

    set gammaList(2H)   0.153506088
    set gammaList(H2)   0.153506088

    set gammaList(C)    0.251449530
    set gammaList(13C)  0.251449530
    set gammaList(C13)  0.251449530

    set gammaList(N)    0.101329118
    set gammaList(15N)  0.101329118
    set gammaList(N15)  0.101329118

    set gammaList(P)    0.40480864 
    set gammaList(31P)  0.40480864
    set gammaList(P31)  0.40480864 

    set gammaList(off)  1.000000000
    set gammaList(OFF)  1.000000000
    set gammaList(NONE) 1.000000000
}

proc getGamma { nuc } \
{
   global gammaList

   set nuc [string toupper $nuc]

   if {[info exists gammaList($nuc)]} {return $gammaList($nuc)}

   return 1.0
}

#
# Test if nuc name is proton.

proc nucIsProton { nuc } \
{
   if {[string first H [string toupper $nuc]] < 0} {return 0}
   return 1 
}

#
# Sets input program name.

proc setProgName {} \
{
   global inProtocol progName

   switch $inProtocol \
      {
       NMRPIPE       { set progName "bin2pipe -bo 2048" }
       BRUKER_NIH    { set progName "bruk2pipe" }
       BRUKER_AMGEN  { set progName "bruk2pipe" }
       VARIAN        { set progName "var2pipe" }
       SPIFF         { set progName "bin2pipe" }
       CHEMAGNETICS  { set progName "bin2pipe -chem" }
       FTNMR_OLD     { set progName "bin2pipe -ftOLD" }
       FTNMR_NEW     { set progName "bin2pipe -ftNEW" }
       JEOL_GSX      { set progName "bin2pipe -gsx" }
       JEOL_ALPHA    { set progName "bin2pipe -alpha" }
       JEOL_LAMBDA   { set progName "bin2pipe -lambda" }
       JEOL_DELTA    { set progName "delta2pipe" }
       GE_EXPORT     { set progName "bin2pipe -ge" }
       GENERIC       { set progName "bin2pipe" }
       default       { set progName "bin2pipe" }
      }
}

#
# Extracts tau values, if any.

proc setTauInfo {} \
{
   global pseudo3DFlag tauFileName tauList tauScale tauFmt userTauCount expectedTauCount verbose

   if {[strcasecmp $tauList None]} {return 0}

   if {![strcasecmp $tauFileName None]} \
      {
       if {$userTauCount > 0} \
          {
           set tauList [getIota 1.0 1.0 $userTauCount]
          } \
       elseif {$expectedTauCount > 0 && (![strcasecmp $tauList None] || ![llength $tauList])} \
          {
           set tauList [getIota 1.0 1.0 $expectedTauCount]
          }

       return 0
      }

   set tauList None

   if {![file exists $tauFileName]} \
      {
       puts "Error Finding Tau File $tauFileName"
       return 1
      }

   if {$verbose} {puts "\nExtracting Tau Values from $tauFileName, Scale Factor $tauScale:"}
 
   set origList [getTextList $tauFileName]
   set tauList  ""

   set i 1

   foreach tau $origList \
      {
       set c      [string index $tau end]
       set val    [string range $tau 0 end-1]
       set tauAdj $tau

       if {[string is integer $c] || ![strcmp $c "."]} \
          {
           set tauAdj [expr $tauScale*$tau]
          } \
       elseif {[string is double $val] && ![strcasecmp $c "m"]} \
          {
           set tauAdj [expr $tauScale*$val]
          } \
       elseif {[string is double $val] && ![strcasecmp $c "u"]} \
          {
           set tauAdj [expr $tauScale*0.001*$val]
          } \
       elseif {[string is double $val] && ![strcasecmp $c "p"]} \
          {
           set tauAdj [expr $tauScale*1.0e-9*$val]
          } \
       elseif {[string is double $val] && ![strcasecmp $c "s"]} \
          {
           set tauAdj [expr $tauScale*1000.0*$val]
          } 

       lappend tauList $tauAdj
      }

   if {$verbose} \
      {
       set nO [getMaxStrlen $origList]
       set nN [getMaxStrlen $tauList]

       set i 0

       foreach tau $origList \
          {
           set tauAdj [lindex $tauList $i]

           puts [format " Tau Value %2d. Original: %${nO}s Adjusted: %${nN}s" [expr $i + 1] $tau $tauAdj]
           incr i
          }

       puts ""
      }
}

#
# Establishes starting conversion parameters for all data.

proc setParms {} \
{
   global axisNames parmNames parmFmts scriptParms convMode progName
   global dataSource inTypes outTypes aq2DTypes inProtocol outProtocol
   global defaultDir winTitle progTitle progVersion 

   set parmNames {N    T    MODE SW     ORIG   FIRST  LAST   UNITS OBS   CAR    FT LAB}

   set inTypes \
      {
       {"NMRPipe"          NMRPIPE}
       {"Bruker (NIH)"     BRUKER_NIH}
       {"Bruker (Amgen)"   BRUKER_AMGEN}
       {"Varian"           VARIAN}
       {"Chemagnetics"     CHEMAGNETICS}
       {"FTNMR (Old)"      FTNMR_OLD}
       {"FTNMR (New)"      FTNMR_NEW}
       {"JEOL EX/GSX"      JEOL_GSX}
       {"JEOL Alpha"       JEOL_ALPHA}
       {"JEOL Lambda"      JEOL_LAMBDA}
       {"JEOL Delta"       JEOL_DELTA}
       {"GE Export"        GE_EXPORT}
       {"SPIFF"            SPIFF}
       {"Other"            GENERIC}
      }

   set outTypes \
      {
       {"NMRPipe"     NMRPIPE}
       {"NMRView"     NMRVIEW}
       {"FTNMR (New)" FTNMR_NEW}
       {"ASCII"       ASCII}
      }

  set aq2DTypes \
      {
       {"States"    STATES}
       {"TPPI"      TPPI}
       {"Magnitude" MAGNITUDE}
       {"Real"      REAL}
       {"Complex"   COMPLEX}
       {"Image"     IMAGE}
      }

   set axisNames(1)     x
   set axisNames(2)     y
   set axisNames(3)     z
   set axisNames(4)     a

   set dataSource FILE

   setProgName

   if {![string compare $convMode ISIS]} \
      {
       set scriptParms {N T MODE SW OBS CAR FT LAB}

       set parmFmts  {%14d %14d %14s %14.5e %14.3f %14.3f %14.3f %14s %14.3f %14.5e %14s %14s}

       setISISParms
       setISISAQDefaults
      } \
   else \
      {
       if {[string first JEOL_DELTA [string toupper $inProtocol]] >= 0} \
          {
           set scriptParms {N T MODE SW ORIG OBS CAR FT LAB}
          } \
       else \
          {
           set scriptParms {N T MODE SW OBS CAR LAB}
          }

       set parmFmts  {%14d %14d %14s %14.3f %14.3f %14.3f %14.3f %14s %14.3f %14.3f %14s %14s}

       setSpecParms
       setSpecAQDefaults
      }

   set winTitle "$progTitle      Version $progVersion"
}

#
# Establishes default conversion parameters for ISIS image data.

proc setISISParms {} \
{
   global inPattern parmNames parmFmts parmLabels parmHelp axisNames 
   global progTitle initTextArea appTerm scriptName
   global specialArgs commentStr hiddenList spectralType
   global pcFlag shExt

   lappend hiddenList T MODE OBS FT SW ORIG CAR LAB

   set progTitle  "ISIS - the Interface for Spectral Imaging and Spectroscopy"

   getISISNames

   if {$pcFlag} \
      {
       set appTerm          start.exe
       set scriptName       conv$shExt
      } \
   else \
      {
       set appTerm          isisTerm
       set scriptName       conv$shExt
      }

   set inPattern         {*.seq *.SEQ *.spc *.dat *.ft* *.gz *.Z}
   set specialArgs       "  -swap -bo 1024"
   set commentStr        ""
   set initTextArea      0

   set parmLabels(N)     "Total Points:" 
   set parmLabels(T)     "Valid Points:"     
   set parmLabels(MODE)  "Acquisition Mode:"
   set parmLabels(SW)    "Axis Width:"
   set parmLabels(FIRST) "Axis Origin:"
   set parmLabels(LAST)  "Axis Endpoint:"
   set parmLabels(UNITS) "Axis Units:"
   set parmLabels(OBS)   "Observe Freq:"
   set parmLabels(ORIG)  "Last Point Hz:"
   set parmLabels(CAR)   "Center Position:"
   set parmLabels(FT)    "Domain:"
   set parmLabels(LAB)   "Axis Label:"

   set parmHelp(N)     "Total points in this dimension of the input data."
   set parmHelp(T)     "Valid points in this dimension of the input data."
   set parmHelp(MODE)  "Whether the dimension is real-only, complex, etc."
   set parmHelp(SW)    "Axis Width in Hertz."
   set parmHelp(FIRST) "Axis location of the first point in the given units."
   set parmHelp(LAST)  "Axis location of the last point in the given units."
   set parmHelp(UNITS) "Axis units label."
   set parmHelp(OBS)   "Observe Frequency in Megahertz."
   set parmHelp(ORIG)  "Location of Last Point in Hz."
   set parmHelp(CAR)   "Center (Carrier) Position in PPM."
   set parmHelp(FT)    "Whether the dimenion needs to be Fourier transformed."
   set parmHelp(LAB)   "Axis Label, 8 characters or less, no spaces, unique."
}

proc setISISAQDefaults {} \
{
   global Nx Tx MODEx SWx ORIGx FIRSTx LASTx UNITSx OBSx CARx FTx LABx POSTx \
          Ny Ty MODEy SWy ORIGy FIRSTy LASTy UNITSy OBSy CARy FTy LABy POSTy \
          Nz Tz MODEz SWz ORIGz FIRSTz LASTz UNITSz OBSz CARz FTz LABz POSTz \
          Na Ta MODEa SWa ORIGa FIRSTa LASTa UNITSa OBSa CARa FTa LABa POSTa

   global EXPCODES EXPNAME DECIM DSPFVS GRPDLY dimCount aq2D aqORD spectralType
   global temperature

   set EXPCODES        $spectralType
   set EXPNAME         $spectralType
   set DECIM           0
   set DSPFVS          0
   set GRPDLY          0

   set aq2D            Image
   set aqORD           0

   set temperature     273

   set Nx      128 
   set Tx      128 
   set MODEx   Real 
   set FIRSTx  0.000
   set LASTx   1.000
   set UNITSx  mm
   set SWx     0.500
   set ORIGx   0.000
   set OBSx    1.000
   set CARx    0.000
   set FTx     Freq 
   set LABx    X 
   set POSTx   ""

   set Ny      128
   set Ty      128
   set MODEy   Real
   set SWy     1.000
   set ORIGy   0.000
   set FIRSTy  0.000
   set LASTy   1.000
   set UNITSy  mm
   set OBSy    1.000
   set CARy    0.000
   set FTy     Freq
   set LABy    Y
   set POSTy   ""

   set Nz      256 
   set Tz      256 
   set MODEz   Real
   set SWz     1975.0
   set ORIGz   0.000
   set FIRSTz  3950.0
   set LASTz   1975.0
   set UNITSz  wn
   set OBSz    1.000
   set CARz    2954.79 
   set FTz     Time
   set LABz    IR 
   set POSTz   ""

   set Na      256
   set Ta      256
   set MODEa   Real
   set SWa     1975.0
   set FIRSTa  3950.0
   set LASTa   1975.0
   set UNITSa  wn
   set OBSa    1.000
   set ORIGa   0.000
   set CARa    2954.79
   set FTa     Time
   set LABa    IR2
   set POSTa   ""
}

#
# Get default file names based on current directory listing.

proc getISISNames {} \
{
   global inName inPath inRoot outName outRoot outPath
   global auxName auxRoot auxPath outAuxName outAuxRoot outAuxPath

   set inPath  .
   set auxPath .

   set seqList [glob -nocomplain $inPath/*.seq*]

   if {[llength $seqList] > 0} \
      { 
       set inRoot  [tkFileFn tail [lindex $seqList 0]]
       set inName  [makePath $inPath $inRoot]
       set outRoot ir%03d.dat
       set outPath [tkFileFn rootname [tkFileFn tail $inName]]
       set outPath [tkFileFn rootname $outPath]
       set outName [makePath $outPath $outRoot]
      } \
   else \
      {
       set inRoot  None
       set inPath  None
       set inName  None
       set outRoot None
       set outPath None
       set outName None
      }

   if {[llength $seqList] > 1} \
      {
       set auxRoot    [tkFileFn tail [lindex $seqList 1]]
       set auxName    [makePath $inPath $auxRoot]
       set outAuxRoot ir%03d.dat
       set outAuxPath [tkFileFn rootname [tkFileFn tail $auxName]]
       set outAuxPath [tkFtileFn rootname $outAuxPath]
       set outAuxName [makePath $outAuxPath $outAuxRoot]
      } \
   else \
      {
       set auxRoot    None
       set auxPath    None
       set auxName    None
       set outAuxRoot None
       set outAuxPath None
       set outAuxName None
      }

   set inName     [makePath $inPath     $inRoot]
   set auxName    [makePath $auxPath    $auxRoot]
   set outName    [makePath $outPath    $outRoot]
   set outAuxName [makePath $outAuxPath $outAuxRoot]
}

proc getSpecNames {} \
{
   global inProtocol inName inPath inRoot inPattern nusFlag nusName

   if {[strcasecmp $inName None]} \
      {
       set inPath [tkFileFn dirname $inName]
       set inRoot [tkFileFn tail    $inName]
       return
      }

   switch $inProtocol \
     {
      BRUKER_NIH    {
                     foreach i {ser ser.Z ser.gz fid fid.z fid.gz} \
                        {
                         if {[tkFileTest exists $i] && ![tkFileTest isdir $i]} \
                            {
                            set inRoot $i
                            set inPath .
                            break
                           }
                        }
                    }
      BRUKER_AMGEN  {
                     foreach i {ser ser.Z ser.gz fid fid.z fid.gz} \
                        {
                         if {[tkFileTest exists $i] && ![tkFileTest isdir $i]} \
                            {
                             set inRoot $i
                             set inPath .
                             break
                            }
                        }
                    }
      VARIAN        {
                     foreach i {fid fid.Z fid.gz hnco_fid_for_frankie} \
                        {
                         if {[tkFileTest exists $i] && ![tkFileTest isdir $i]} \
                            {
                             set inRoot $i
                             set inPath .
                             break
                            }
                        }
                    }
      JEOL_DELTA    {
                     foreach i [glob -nocomplain *.jdf *.JDF] \
                        {
                         if {[tkFileTest exists $i] && ![tkFileTest isdir $i]} \
                            {
                             set inRoot $i
                             set inPath .
                             break
                            }
                        }
                    }
     default        {
                     foreach j $inPattern \
                        {
                         set fList [lsort -dictionary [glob -nocomplain $j]]

                         foreach i $fList \
                            {
                             if {[tkFileTest exists $i] && ![tkFileTest isdir $i]} \
                                {
                                 set inRoot $i
                                 set inPath .
                                 break
                                }
                            }
                        }
                    }
     }

  if {$nusFlag && ![strcasecmp $nusName None]} \
     {
      set fList [findNUSInput .]
      if {[llength $fList]} {set nusName [makePath . [lindex $fList 0]]}
     }

  set inName [makePath $inPath $inRoot]
}

#
# Establishes default conversion parameters for spectral data.

proc setSpecParms {} \
{
   global inName inPath inRoot inPattern inProtocol outName outPath outRoot
   global auxName auxRoot auxPath outAuxName outAuxPath outAuxRoot
   global parmNames parmFmts parmLabels parmHelp axisNames 
   global appTerm scriptName pcFlag shExt
   global specialArgs commentStr hiddenList

   if {[string first JEOL_DELTA [string toupper $inProtocol]] >= 0} \
      {
       lappend hiddenList FIRST LAST UNITS .spiffButton
      } \
   else \
      {
       lappend hiddenList ORIG FT FIRST LAST UNITS .spiffButton
      }

   set inPattern  {ser fid *.fid *.ft* *.dat *.DAT *.nmf* *.gxd *.gz *.Z}

   set auxPath    None 
   set auxRoot    None
   set auxName    None

   set outAuxPath None
   set outAuxRoot None
   set outAuxName None

    if {$pcFlag} \
       {
        set appTerm    start.exe
        if {![strcasecmp $scriptName None]} {set scriptName fid$shExt}
       } \
    else \
       {
        set appTerm    nmrTerm
        if {![strcasecmp $scriptName None]} {set scriptName fid$shExt}
       }

   getSpecNames
   
   set specialArgs      " "
   set commentStr       ""

   set parmLabels(N)     "Total Points R+I:" 
   set parmLabels(T)     "Valid Points:"     
   set parmLabels(MODE)  "Acquisition Mode:"
   set parmLabels(SW)    "Spectral Width Hz:"
   set parmLabels(FIRST) "Axis Origin:"
   set parmLabels(LAST)  "Axis Endpoint:"
   set parmLabels(UNITS) "Axis Units:"
   set parmLabels(ORIG)  "Last Point Hz:"
   set parmLabels(OBS)   "Observe Freq MHz:"
   set parmLabels(CAR)   "Center Position PPM:"
   set parmLabels(FT)    "Domain:"
   set parmLabels(LAB)   "Axis Label:"

   set parmHelp(N)       "Total points in this dimension of the input file."
   set parmHelp(T)       "Valid points in this dimension of the input data."
   set parmHelp(MODE)    "Whether the dimension is real-only, complex, etc."
   set parmHelp(SW)      "Axis Width in Hertz."
   set parmHelp(ORIG)    "Location of Last Point in Hz."
   set parmHelp(FIRST)   "Axis location of the first point in the given units."
   set parmHelp(LAST)    "Axis location of the last point in the given units."
   set parmHelp(UNITS)   "Axis units label."
   set parmHelp(OBS)     "Observe Frequency in Megahertz."
   set parmHelp(CAR)     "Center Position (Carrier, point N/2 + 1 of 1 ... N) in PPM."
   set parmHelp(FT)      "Whether the dimension is time-domain or frequency-domain."
   set parmHelp(LAB)     "Axis Label, 8 characters or less, no spaces, unique."
}

#
# Set or reset spectral acquistion parameter values.

proc setSpecAQDefaults {} \
{
   global Nx Tx MODEx SWx ORIGx FIRSTx LASTx UNITSx OBSx CARx FTx LABx POSTx \
          Ny Ty MODEy SWy ORIGy FIRSTy LASTy UNITSy OBSy CARy FTy LABy POSTy \
          Nz Tz MODEz SWz ORIGz FIRSTz LASTz UNITSz OBSz CARz FTz LABz POSTz \
          Na Ta MODEa SWa ORIGa FIRSTa LASTa UNITSa OBSa CARa FTa LABa POSTa

   global aq2D aqORD dimCount EXPCODES EXPNAME DECIM DSPFVS GRPDLY inProtocol
   global BYTORDA temperature h2oFlag

   set EXPCODES         nmr
   set EXPNAME          nmr
   set DECIM            1
   set DSPFVS           1
   set GRPDLY           1
   set BYTORDA          0

   set aq2D             Complex
   set aqORD            0

   set Nx      1024
   set Tx      512
   set MODEx   Complex
   set SWx     10000.00
   set ORIGx   0.00
   set FIRSTx  1.0 
   set LASTx   512.0 
   set UNITSx  pts 
   set OBSx    500.000
   set CARx    4.700
   set FTx     Time
   set LABx    X 
   set POSTx   ""

   set Ny      128 
   set Ty      64 
   set MODEy   Complex
   set SWy     10000.00
   set ORIGy   0.00
   set FIRSTy  1.0 
   set LASTy   64.0 
   set UNITSy  pts 
   set OBSy    500.000 
   set CARy    4.700 
   set FTy     Time
   set LABy    Y 
   set POSTy   ""

   set Nz      64 
   set Tz      32 
   set MODEz   Complex
   set SWz     2000.000 
   set ORIGz   0.00
   set FIRSTz  1.0 
   set LASTz   32.0 
   set UNITSz  pts 
   set OBSz    100.00 
   set CARz    50.00
   set FTz     Time
   set LABz    Z 
   set POSTz   ""
 
   set Na      32 
   set Ta      16 
   set MODEa   Complex
   set SWa     1000.00 
   set ORIGa   0.00
   set FIRSTa  1.0 
   set LASTa   16.0 
   set UNITSa  pts 
   set OBSa    50.00 
   set CARa    100.00 
   set FTa     Time
   set LABa    A
   set POSTa   ""

   switch $inProtocol \
      {
       JEOL_DELTA    {
                      if {![strcasecmp $temperature None]} \
                         {
                          set temperature "From File"
                         }
                     }

       BRUKER_NIH   -
       BRUKER_AMGEN -
       VARIAN        {
                      if {![strcasecmp $temperature None]} \
                         {
                          set temperature "From File"
                         }

                      if {$h2oFlag} {set CARx H2O}
                     }
       default       {
                      if {![strcasecmp $temperature None]} \
                         {
                          set temperature 273 
                         }
                     }
      }
}

proc updateScriptVars {} \
{
   global convMode SWx SWy SWz CARz FIRSTx LASTx FIRSTy LASTy FIRSTz LASTz Nz

   if {![string compare $convMode ISIS]} \
      {
       set SWx  [expr ($LASTx - $FIRSTx)]
       set SWy  [expr ($LASTy - $FIRSTy)]

       set Tz   $Nz
       set size [expr $Nz/2]
       set mid  [expr 1.0 + $size/2]
       set SWz  [expr ($FIRSTz - $LASTz)*2.99792e+10]
       set CARz [expr $LASTz*2.99792e+10 + $SWz*($size - $mid)/(1.0*$size)]
      }

   updateAxisLabels
}

#
# Return H2O position based on degrees K.

proc getH2Oppm { t } \
{
   if {![strcasecmp $t "From File"]} {return 0.0}

   set m        -0.009552
   set b         5.011718

   set h2oPPM  [expr $m*($t - 273.0) + $b]

   return $h2oPPM
}
 
proc showInfoText { } \
{
   showInfoTextU stdout
}

proc saveInfoText { } \
{
   global infoName

   if {![strcasecmp $infoName None]} return

   set error [catch {set outUnit [open $infoName w]}]

   showInfoTextU $outUnit

   catch "close $outUnit"
}

proc showInfoTextU { outUnit } \
{
   global parmNames parmFmts axisNames scriptParms extraArgs specialArgs infoText infoFlag dateText
   global dimCount nusName maskName nusDimCount nusSampleCount aq2D aqORD temperature infoResult dataSource Nz Na Ty
   global BYTORDA DECIM DSPFVS GRPDLY EXPNAME wordSize wordType 
   global tauList commentStr titleStr

   updateScriptVars

   set planeCount $Nz
   set ordArgs    ""

   set thisDimCount    $dimCount
   set thisSampleCount $nusSampleCount

   if {$thisDimCount == 2 && $Ty == 1} {set thisDimCount 1}

   if {![strcasecmp $nusSampleCount Auto] || ![strcasecmp $nusSampleCount None]} \
      {
       set thisSampleCount 0
      }

   if {![strcasecmp $nusName Auto]} \
      {
       set thisNUSName None
      } \
   else \
      {
       set thisNUSName $nusName
      }

   if {[strcmp $thisSampleCount 0] || ![strcasecmp $thisNUSName None]} {set thisMaskName None}

   if {$infoFlag == 2} \
      {
       puts -nonewline $outUnit "-ndim $thisDimCount"
       puts -nonewline $outUnit " -nusDim $nusDimCount"
       puts -nonewline $outUnit " -sampleName  $thisNUSName"
       puts -nonewline $outUnit " -mask        $thisMaskName"
       puts -nonewline $outUnit " -sampleCount $thisSampleCount"
       puts -nonewline $outUnit " -aq2D $aq2D"
       puts -nonewline $outUnit " -aqORD $aqORD"
       puts -nonewline $outUnit " -dataSource $dataSource"
       puts -nonewline $outUnit " -infoResult $infoResult"
       puts -nonewline $outUnit " -comment $commentStr"
       puts -nonewline $outUnit " -title $titleStr"
       puts -nonewline $outUnit " -temperature $temperature"
       puts -nonewline $outUnit " -byteOrder $BYTORDA"
       puts -nonewline $outUnit " -decim $DECIM"
       puts -nonewline $outUnit " -dspfvs $DSPFVS"
       puts -nonewline $outUnit " -grpdly $GRPDLY"
       puts -nonewline $outUnit " -expName $EXPNAME"
       puts -nonewline $outUnit " -wordType $wordType"
       puts -nonewline $outUnit " -wordSize $wordSize"
       puts -nonewline $outUnit " -tau      $tauList"
       puts -nonewline $outUnit " -date     $dateText"
      } \
   else \
      {
       puts $outUnit "set dimCount       \"$thisDimCount\""
       puts $outUnit "set nusDimCount    \"$nusDimCount\""
       puts $outUnit "set nusSampleName  \"$thisNUSName\""
       puts $outUnit "set nusMaskName    \"$thisMaskName\""
       puts $outUnit "set nusSampleCount \"$thisSampleCount\""
       puts $outUnit "set aq2D           \"$aq2D\""
       puts $outUnit "set aqORD          \"$aqORD\""
       puts $outUnit "set dataSource     \"$dataSource\""
       puts $outUnit "set infoResult     \"$infoResult\""
       puts $outUnit "set comment        \"$commentStr\""
       puts $outUnit "set title          \"$titleStr\""
       puts $outUnit "set temperature    \"$temperature\""
       puts $outUnit "set byteOrder      \"$BYTORDA\""
       puts $outUnit "set decim          \"$DECIM\""
       puts $outUnit "set dspfvs         \"$DSPFVS\""
       puts $outUnit "set grpdly         \"$GRPDLY\""
       puts $outUnit "set expName        \"$EXPNAME\""
       puts $outUnit "set wordType       \"$wordType\""
       puts $outUnit "set wordSize       \"$wordSize\""
       puts $outUnit "set tauList        \"$tauList\""
       puts $outUnit "set date           \"$dateText\""
       puts $outUnit ""
      }
#
# Axis-specific conversion parameters:

   set k 0

   foreach i "$parmNames NUC" \
      {
       if {![strcasecmp $i NUC] || [lsearch -exact $scriptParms $i] >= 0} \
          {
           for {set j 1} {$j <= $dimCount} {incr j 1} \
              {
               global $i$axisNames($j)

               set value [eval concat \$$i$axisNames($j)]

               if {![strcasecmp $value "From File"]} {set value 0.0}

               if {![strcasecmp $value H2O]} \
                  {
                   set value [getH2Oppm $temperature]
                  }

               if {![strcasecmp $i NUC]} \
                  {
                   set fmt %s
                  } \
               else \
                  {
                   set fmt [lindex $parmFmts $k]
                  }

               set v   [string trim [format $fmt $value]]

               if {$infoFlag == 2} \
                  {
                   puts -nonewline $outUnit " -$axisNames($j)$i $v"
                  } \
               else \
                  {
                   puts $outUnit [format "set %-5s %16s" $axisNames($j)$i \"$v\"]
                  }
              }
          }

       incr k
      }

   if {$infoFlag == 2} {puts $outUnit ""} 
}

#
# Insert a conversion script from the current entry settings.
 
proc insertConvScript { thisInName  thisInPath  thisOutName } \
{
   global progName parmNames parmFmts scriptParms axisNames scriptName
   global EXPCODES dataSource Nx Ny Nz Na Tx Ty Tz Ta MODEy MODEz MODEa NUSZFy NUSZFz NUSZFa inProtocol outProtocol
   global pseudo3DFlag pseudoLoopMode tauList tauFileName tauScale tauFmt expectedTauCount
   global titleStr specialArgs extraArgs finalArgs commentStr verbose

   global tempFlag nusFlag nusAvgFlag maskFlag nusName maskName interName nusOffset nusRevFlag nusSampleCount nusDimCount
   global vdlistName vdlistCount nmrDrawFlag nmrDrawCmnd rnmrtkFlag ioArgsFlag autoUpdateFlag
   global dimCount aq2D aqORD temperature userScans vendorScans userScale dataScale vendorScale infoResult

   global deferGradMsg planeCountMsg

   updateScriptVars

   if {$nusRevFlag} \
      {
       set revStr " -rev"
      } \
   else \
      {
       set revStr ""
      }

   if {$nusAvgFlag == 1} \
      {
       set nusAvgStr "-avg"
      } \
   elseif {$nusAvgFlag == 2} \
      {
       set nusAvgStr "-sqrt"
      } \
   else \
      {
       set nusAvgStr "-sum"
      }

   set nusQuadStr ""
   set nusZFStr   ""

   if {![info exists deferGradMsg]}  {set deferGradMsg  0}
   if {![info exists planeCountMsg]} {set planeCountMsg 0}
 
   if {$dimCount >= 2 && $NUSZFy > 0} {append nusZFStr " -yT $Ty"}
   if {$dimCount >= 3 && $NUSZFz > 0} {append nusZFStr " -zT $Tz"}
   if {$dimCount >= 4 && $NUSZFa > 0} {append nusZFStr " -aT $Ta"}

   if {$dimCount >= 3 && ![strcasecmp $MODEz Real]} {append nusQuadStr " -qzSize 1"}
   if {$dimCount >= 4 && ![strcasecmp $MODEa Real]} {append nusQuadStr " -qaSize 1"}

   set planeCount $Nz
   set ordArgs    ""

   if {$aqORD} {set ordArgs "-aqORD $aqORD"}

   if {[string first BRUKER [string toupper $inProtocol]] >= 0} \
      {
       set nusMode bruker
      } \
   elseif {[string first VARIAN [string toupper $inProtocol]] >= 0} \
      {
       set nusMode varian 
      } \
   else \
      {
       set nusMode Unknown
      }

   set useDataScale 1

   if {![strcasecmp $dataScale None] || ![string is double $dataScale]} \
      {
       set useDataScale 0
      } \
   else \
      {
       if {$dataScale == 1.0} {set useDataScale 0}
      } 

#
# Insert leading comment:

   if {![strcasecmp $scriptName stdout]} \
      {
       set eol ""
      } \
   else \
      {
       set eol "\\\n"

       if {[string length $commentStr]} \
          {
           adjScript insert end $commentStr
          }
      }

#
# Start handling for Pseudo-3D and Tau list, if any:

   set pseudoOutName     $thisOutName
   set pseudo3DNeedsGrad 0

   if {$pseudo3DFlag} \
      {
       if {$expectedTauCount > 0 && (![strcasecmp $tauList None] || ![llength $tauList])} \
          {
           set tauList [getIota 1.0 1.0 $expectedTauCount]
          }

       adjScript insert end "set tauList = ($tauList)\n"
       adjScript insert end "\n"
      }

#
# Special handling to automatically update conversion parameters on the fly during acquisition:

   set sampleCountStr $nusSampleCount

   set NyStr $Ny
   set TyStr $Ty

   set NzStr $Nz
   set TzStr $Tz

   set NaStr $Na
   set TaStr $Ta

   if {$autoUpdateFlag && $nusFlag} \
      {
       set sampleCountStr "\$sampleCount"

       set NyStr \$Ny
       set TyStr \$Ty

       set NzStr \$Nz
       set TzStr \$Tz

       set NaStr \$Na
       set TaStr \$Ta

       adjScript insert end "set nusInfo     = (`getNUSInfo.tcl -sample $nusName -parmDir .`)\n"
       adjScript insert end "set sampleCount = (`getNamedItemD sampleCount \$nusInfo $nusSampleCount`)\n"
       adjScript insert end "\n"

       if {$nusDimCount >= 2} \
          {
           adjScript insert end "set yT = (`getNamedItemD yTNUS \$nusInfo $Ty`)\n"
           adjScript insert end "set yN = (`IMATH \"2*\$yT\"`)\n"
           adjScript insert end "\n"
          }

       if {$nusDimCount >= 3} \
          {
           adjScript insert end "set zT = (`getNamedItemD zTNUS \$nusInfo $Tz`)\n"
           adjScript insert end "set zN = (`IMATH \"2*\$zT\"`)\n"
           adjScript insert end "\n"
          }

       if {$nusDimCount >= 3} \
          {
           adjScript insert end "set zT = (`getNamedItemD zTNUS \$nusInfo $Tz`)\n"
           adjScript insert end "set zN = (`IMATH \"2*\$zT\"`)\n"
           adjScript insert end "\n"
          }

       adjScript insert end "echo \"\"\n"
       adjScript insert end "echo \"Conversion Auto-Update. NUS Sample Count: \$sampleCount yN: \$yN yT: \$yT"

       if {$nusDimCount >= 3} {adjScript insert end " zN: \$zN zT: \$zT"}
       if {$nusDimCount >= 4} {adjScript insert end " aN: \$aN aT: \$aT"}

       adjScript insert end "\"\n\n"
      }

#
# Insert handling for input/output values from command line:

   set thisRootName     [file root $thisInName]
   set thisRNMRTKName   [getRNMRTKName $thisOutName]
   set thisExpandedName ${thisRootName}_full

   set workingInName       $thisInName
   set workingInterName    $interName
   set workingOutName      $thisOutName
   set workingMaskName     $maskName
   set workingNUSName      $nusName
   set workingRNMRTKName   $thisRNMRTKName
   set workingExpandedName $thisExpandedName
   set workingSampleCount  $sampleCountStr
   set workingNUSOffset    $nusOffset

#  if {$pseudo3DFlag} {set workingOutName $interName}

   if {$ioArgsFlag} \
      {
       set workingInName     "\$inName"
       set workingInterName  "\$interName"
       set workingOutName    "\$outName"
       set workingRNMRTKName "\$rnmrtkName"

       if {$pseudo3DFlag} \
          {
           set pseudoOutName  $workingOutName
#          set workingOutName $workingInterName
          }

       if {$nusFlag} \
          {
           set workingMaskName     "\$maskName"
           set workingNUSName      "\$sampleName"
           set workingExpandedName "\$expandName"
           set workingSampleCount  "\$sampleCount"
           set workingNUSOffset    "\$nusOffset"

           adjScript insert end "set inName      = (`getArgD \$argv -in          $thisInName`)\n"

           if {$pseudo3DFlag} \
              {
               adjScript insert end "set interName   = (`getArgD \$argv -inter       $interName`)\n"
              }

           adjScript insert end "set outName     = (`getArgD \$argv -out         $thisOutName`)\n"
           adjScript insert end "set maskName    = (`getArgD \$argv -mask        $maskName`)\n"
           adjScript insert end "set expandName  = (`getArgD \$argv -expand      $thisExpandedName`)\n"
           adjScript insert end "set sampleName  = (`getArgD \$argv -sample      $nusName`)\n"
           adjScript insert end "\n"
           adjScript insert end "if (`flagLoc \$argv -nusExact`) then\n"
           adjScript insert end "   set sampleCount = (`getArgD \$argv -sampleCount $nusSampleCount`)\n"
           adjScript insert end "   set nusOffset   = (`getArgD \$argv -offset      $nusOffset`)\n"
           adjScript insert end "else\n"
           adjScript insert end "   set sampleCount = (`getArgD \$argv -sampleCount Auto`)\n"
           adjScript insert end "   set nusOffset   = (`getArgD \$argv -offset      Auto`)\n"
           adjScript insert end "endif\n"
          } \
       else \
          {
           adjScript insert end "set inName    = (`getArgD \$argv -in    $thisInName`)\n"

           if {$pseudo3DFlag} \
              {
               adjScript insert end "set interName = (`getArgD \$argv -inter $interName`)\n"
              }

           adjScript insert end "set outName   = (`getArgD \$argv -out   $thisOutName`)\n"
          }

       if {$rnmrtkFlag} \
          {
           adjScript insert end "\n"
           adjScript insert end "set rnmrtkName = (`getArgD \$argv -rnmrtk $thisRNMRTKName`)\n"
          }

       adjScript insert end "\n"
      }

#
# Options for preprocessing input:

   switch $dataSource \
      {
       CATSER \
         {
          if {$nusFlag} \
             {
              if {[strcasecmp $vdlistName None]} \
                 {
                  adjScript insert end "(catser$shExt $thisInPath 1 $planeCount) $eol"
                  adjScript insert end "| vdExpand.tcl -vCount $vdlistCount -in stdin -out $workingExpandedName -sample $vdlistName\n"
                 } \
              else \
                 {
                  adjScript insert end "(catser$shExt $thisInPath 1 $planeCount) $eol"
                  adjScript insert end "| nusExpand.tcl$nusQuadStr -mode $nusMode -sampleCount $workingSampleCount $nusAvgStr -off $workingNUSOffset $eol"
                  adjScript insert end "   -in stdin -out $workingExpandedName -sample $workingNUSName$revStr$nusZFStr\n"
                 }

              adjScript insert end "\n"
              adjScript insert end "$progName -in $workingExpandedName $eol"
              adjScript insert end " $specialArgs $ordArgs $eol"
             } \
          else \
             {
              adjScript insert end "(catser$shExt $thisInPath 1 $planeCount) $eol"
              adjScript insert end "| $progName $specialArgs $ordArgs $eol"
             }
         }

       ZCAT \
         {
          if {$nusFlag} \
             {
              if {[strcasecmp $vdlistName None]} \
                 {
                  adjScript insert end \
                   "cat $workingInName | zcat | vdExpand.tcl -vCount $vdlistCount -in stdin -out $workingExpandedName -sample $vdlistName\n"
                 } \
              else \
                 {
                  adjScript insert end "cat $workingInName | zcat | nusExpand.tcl$nusQuadStr -mode $nusMode -sampleCount $workingSampleCount $nusAvgStr -off $workingNUSOffset $eol"
                  adjScript insert end " -in stdin -out $workingExpandedName -sample $workingNUSName$revStr$nusZFStr\n"
                 }

              adjScript insert end "\n"
              adjScript insert end "$progName -in $workingExpandedName $eol"
              adjScript insert end " $specialArgs $ordArgs $eol"
             } \
          else \
             {
              adjScript insert end "cat $workingInName | zcat | $progName $eol"
              adjScript insert end " $specialArgs $ordArgs $eol"
             }
         }

       GZCAT \
         {
          if {$nusFlag} \
             {
             if {[strcasecmp $vdlistName None]} \
                 {
                  adjScript insert end \
                   "cat $workingInName | zcat | vdExpand.tcl -vCount $vdlistCount -in stdin -out $workingExpandedName -sample $vdlistName\n"
                 } \
              else \
                 {
                  adjScript insert end "cat $workingInName | zcat | nusExpand.tcl$nusQuadStr -mode $nusMode -sampleCount $workingSampleCount $nusAvgStr -off $workingNUSOffset $eol"
                  adjScript insert end " -in stdin -out $workingExpandedName -sample $workingNUSName$revStr$nusZFStr\n"
                 }

              adjScript insert end "\n"
              adjScript insert end "$progName -in $workingExpandedName $eol"
              adjScript insert end " $specialArgs $ordArgs $eol"
             } \
          else \
             {
              adjScript insert end "cat $workingInName | zcat | $progName $eol"
              adjScript insert end " $specialArgs $ordArgs $eol"
             }
         }

       default \
         {
          if {$nusFlag} \
             {
              if {[strcasecmp $vdlistName None]} \
                 {
                  adjScript insert end \
                   "vdExpand.tcl -vCount $vdlistCount -in $workingInName -out $workingExpandedName -sample $vdlistName\n"
                 } \
              else \
                 {
                  adjScript insert end "nusExpand.tcl$nusQuadStr -mode $nusMode -sampleCount $workingSampleCount $nusAvgStr -off $workingNUSOffset $eol"
                  adjScript insert end " -in $workingInName -out $workingExpandedName -sample $workingNUSName$revStr$nusZFStr\n"
                 }

              adjScript insert end "\n"
              adjScript insert end "$progName -verb -in $workingExpandedName $eol"
              adjScript insert end " $specialArgs $ordArgs $eol"
             } \
          else \
             {
              adjScript insert end "$progName -verb -in $workingInName $eol"
              adjScript insert end " $specialArgs $ordArgs $eol"
             }
         }
      }

#
# Insert special info, if any:

    if {[strcasecmp $infoResult None]} \
       {
        adjScript insert end " -comment $infoResult $eol"
       }

    if {[strcasecmp $titleStr None]} \
       {
        adjScript insert end " -title $titleStr $eol"
       }

    if {$tempFlag} \
       {
        adjScript insert end " -temp $temperature $eol"
       }

    if {[strcasecmp $extraArgs None]} \
       {
        adjScript insert end " $extraArgs $eol"
       }

#
# Insert axis-specific conversion parameters:

   set k 0

   foreach i $parmNames \
      {
       if {[lsearch -exact $scriptParms $i] >= 0} \
          {
           for {set j 1} {$j <= $dimCount} {incr j 1} \
              {
               global $i$axisNames($j)

               set value [eval concat \$$i$axisNames($j)]

               if {![strcasecmp $value "From File"]} {set value 0.0}

               if {![strcasecmp $value H2O]} \
                  {
                   set value [getH2Oppm $temperature]
                  }

               set fmt [lindex $parmFmts $k]

               if {$pseudo3DFlag && $j == 2} \
                  {
                   if {![strcmp $i MODE] && (![strcasecmp $value Echo-AntiEcho] || ![strcasecmp $value Rance-Kay])} \
                      {
#                      if {$verbose && !$deferGradMsg} {puts "Pseudo-3D Data: Deferring Gradient Shuffling."}
#
#                      set pseudo3DNeedsGrad 1
#                      set value Complex
#                      set deferGradMsg 1
                      }

                   if {![strcasecmp $i N] && $expectedTauCount > 0} \
                      {
#                      if {$verbose} {puts "Pseudo-3D Data: Adjusting Sizes yN $Ny zN $Nz for $expectedTauCount Planes."}
#                      set value [expr $expectedTauCount*$Ny]
#                      set planeCountMsg 1
                      }
                  }

               if {$j > 1 && $autoUpdateFlag && $nusFlag && ![strcmp $i N]} \
                  { 
                   set value "\$$axisNames($j)N"
                   set fmt   "%14s"
                  } \
               elseif {$j > 1 && $autoUpdateFlag && $nusFlag && ![strcmp $i T]} \
                  {
                   set value "\$$axisNames($j)T"
                   set fmt   "%14s"
                  }

               adjScript insert end [format "  -%-5s $fmt" $axisNames($j)$i $value]
              }

           adjScript insert end "  $eol"
          }

       incr k
      }

#
# Insert handling for output protocol:

   set newOutName  [format $thisOutName 1 1]
   set fmtCount    [getFmtCount $thisOutName]
   set outNameType $dimCount
   set usePipe     0

   if {$useDataScale} {set usePipe 1}

   if {$dimCount >= 3} \
      {
       if {$fmtCount == 0} \
          {
           set outNameType 2
          } \
       elseif {$fmtCount == 1} \
          {
           set outNameType 3
          } \
       elseif {$fmtCount == 2} \
          {
           set outNameType 4
          }
      }       
 
   switch $outNameType \
      {
       1 { 
          adjScript insert end [format "  -ndim  %14s  $eol" $dimCount]
  
          if {[strcasecmp $finalArgs None]} \
             {
              adjScript insert end " $finalArgs $eol"
             }

          if {$useDataScale} {adjScript insert end "| nmrPipe -fn MULT -c $dataScale $eol"}

          if {$pseudo3DFlag && $dimCount >= 3 && ![strcasecmp $MODEy Real]} \
             {
              adjScript insert end "| nmrPipe -fn TP  -exch -noord -nohdr $eol"
              adjScript insert end "| nmrPipe -fn ZTP -exch -noord $eol"
              adjScript insert end "| nmrPipe -fn TP  -exch -noord -nohdr $eol"
             }

          switch $outProtocol \
             { 
              NMRPIPE    {
                          adjScript insert end "  -out $workingOutName -ov\n"
                         }
              NMRVIEW    {
                          adjScript insert end "| pipe2xyz -nv -out $workingOutName -ov\n"
                         }
              SPIFF      {
                          adjScript insert end "| pipe2spiff -out $workingOutName -ov\n"
                         }
              FTNMR_NEW  {
                          adjScript insert end "| pipe2ft -out $workingOutName -ov\n"
                         }
              ASCII      {
                          set fArgs "-verb -tty -noWr -outHdr -all"

                          adjScript insert end "| nmrPipe -fn MAC -macro \$NMRTXT/showASCII.M $eol"
                          adjScript insert end "  $fArgs -str outName $workingOutName\n"
                         }
              default    {
                          adjScript insert end "  -out $workingOutName -ov\n"
                         }
             }
         }

       2 { 
          adjScript insert end [format "  -ndim  %14s  -aq2D  %14s  $eol" $dimCount $aq2D]

          if {$useDataScale} {adjScript insert end "| nmrPipe -fn MULT -c $dataScale $eol"}

          if {$pseudo3DFlag && $dimCount >= 3 && ![strcasecmp $MODEy Real]} \
             {
              adjScript insert end "| nmrPipe -fn TP  -exch -noord -nohdr $eol"
              adjScript insert end "| nmrPipe -fn ZTP -exch -noord $eol"
              adjScript insert end "| nmrPipe -fn TP  -exch -noord -nohdr $eol"
             }

          if {![strcasecmp $inProtocol CHEMAGNETICS]} \
             {
              adjScript insert end "| nmrPipe -fn MAC -macro \$NMRTXT/rrii.M -noRd -noWr  $eol"
             }

          if {[lsearch -exact $EXPCODES ea] >= 0} \
             {
              adjScript insert end "| nmrPipe -fn MAC -macro \$NMRTXT/ranceY2.M -noRd -noWr $eol"
             } \
          elseif {[lsearch -exact $EXPCODES se] >= 0} \
             {
              adjScript insert end "| nmrPipe -fn MAC -macro \$NMRTXT/seY.M -noRd -noWr -all $eol"
             }

          if {[strcasecmp $finalArgs None]} \
             {
              adjScript insert end " $finalArgs $eol"
             }

          switch $outProtocol \
             {
              NMRPIPE    {
                          adjScript insert end "  -out $workingOutName -ov\n"
                         }
              NMRVIEW    {
                          adjScript insert end "| pipe2xyz -nv -out $workingOutName -ov\n"
                         }
              SPIFF      {
                          adjScript insert end "| pipe2spiff -out $workingOutName -ov\n"
                         }
              FTNMR_NEW  {
                          adjScript insert end "| pipe2ft -out $workingOutName -ov\n"
                         }
              ASCII      {
                          set fArgs "-verb -tty -noWr -outHdr -all"

                          adjScript insert end "| nmrPipe -fn MAC -macro \$NMRTXT/showASCII.M $eol"
                          adjScript insert end "  $fArgs -str outName $workingOutName\n"
                         }
              default    {
                          adjScript insert end "  -out $workingOutName -ov\n"
                         }
             }
         }

       3 { 
          adjScript insert end [format "  -ndim  %14s  -aq2D  %14s         %14s  $eol" $dimCount $aq2D " "]

          if {$useDataScale} {adjScript insert end "| nmrPipe -fn MULT -c $dataScale $eol"}

          if {$pseudo3DFlag && $dimCount >= 3 && ![strcasecmp $MODEy Real]} \
             {
              adjScript insert end "| nmrPipe -fn TP  -exch -noord -nohdr $eol"
              adjScript insert end "| nmrPipe -fn ZTP -exch -noord $eol"
              adjScript insert end "| nmrPipe -fn TP  -exch -noord -nohdr $eol"
             }

          if {![strcasecmp $inProtocol CHEMAGNETICS]} \
             {
              adjScript insert end "| nmrPipe -fn MAC -macro \$NMRTXT/rrii.M -noRd -noWr  $eol"
             }

          if {[lsearch -exact $EXPCODES ea] >= 0} \
             {
              set usePipe 1

              adjScript insert end "| nmrPipe -fn MAC -macro \$NMRTXT/ranceY2.M -noRd -noWr $eol"
             } \
          elseif {[lsearch -exact $EXPCODES se] >= 0} \
             {
              set usePipe 1

              adjScript insert end "| nmrPipe -fn MAC -macro \$NMRTXT/seY.M -noRd -noWr -all $eol"
             }

          if {[strcasecmp $finalArgs None]} \
             {
              adjScript insert end " $finalArgs $eol"
             }

          switch $outProtocol \
             {
              NMRPIPE    {
                          if {$usePipe} \
                             {
                              adjScript insert end "| pipe2xyz -x -out $workingOutName -ov\n"
                             } \
                          else \
                             {
                              adjScript insert end "  -out $workingOutName -ov\n"
                             }
                         }
              NMRVIEW    {
                          adjScript insert end "| pipe2xyz -nv -out $newOutName -ov\n"
                         }
              SPIFF      {
                          adjScript insert end "| pipe2spiff -out $newOutName -ov\n"
                         }
              FTNMR_NEW  {
                          adjScript insert end "| pipe2ft -out $newOutName -ov\n"
                         }
              ASCII      {
                          set fArgs "-verb -tty -noWr -outHdr -all"

                          adjScript insert end "| nmrPipe -fn MAC -macro \$NMRTXT/showASCII.M $eol"
                          adjScript insert end "  $fArgs -str outName $workingOutName\n"
                         }
              default    {
                          adjScript insert end "  -out $workingOutName -ov\n"
                         }
             }
         }

       4 { 
          adjScript insert end [format "  -ndim  %14s  -aq2D  %14s         %37s  $eol" $dimCount $aq2D " "]

          if {$useDataScale} {adjScript insert end "| nmrPipe -fn MULT -c $dataScale $eol"}

          if {$pseudo3DFlag && $dimCount >= 3 && ![strcasecmp $MODEy Real]} \
             {
              adjScript insert end "| nmrPipe -fn TP  -exch -noord -nohdr $eol"
              adjScript insert end "| nmrPipe -fn ZTP -exch -noord $eol"
              adjScript insert end "| nmrPipe -fn TP  -exch -noord -nohdr $eol"
             }

          if {![strcasecmp $inProtocol CHEMAGNETICS]} \
             {
              adjScript insert end "| nmrPipe -fn MAC -macro \$NMRTXT/rrii.M -noRd -noWr  $eol"
             }

          if {[strcasecmp $finalArgs None]} \
             {
              adjScript insert end " $finalArgs $eol"
             }

          switch $outProtocol \
             {
              NMRPIPE    {
                          adjScript insert end "| pipe2xyz -x -out $workingOutName -ov\n"
                         }
              NMRVIEW    {
                          adjScript insert end "| pipe2xyz -nv -out $newOutName -ov\n"
                         }
              SPIFF      {
                          adjScript insert end "| pipe2spiff -out $newOutName -ov\n"
                         }
              FTNMR_NEW  {
                          adjScript insert end "| pipe2ft -out $newOutName -ov\n"
                         }
              ASCII      {
                          set fArgs "-verb -tty -noWr -outHdr -all"

                          adjScript insert end "| nmrPipe -fn MAC -macro \$NMRTXT/showASCII.M $eol"
                          adjScript insert end "  $fArgs -str outName $workingOutName\n"
                         }
              default    {
                          adjScript insert end "  -out $workingOutName -ov\n"
                         }
             }
         }
      }

   if {$pseudo3DFlag} \
      {
       adjScript insert end "\n"

       if {$pseudoLoopMode == 2} \
          {
           set pseudoArg "-outer"
          } \
       else \
          {
           set pseudoArg "-inner"
          }

       if {$pseudo3DNeedsGrad} {set pseudoArg "$pseudoArg -grad"}

#      set pseudoArg "-outer"
#
#      adjScript insert end "pseudo3D.com -in $workingInterName -out $pseudoOutName -tau \$tauList -title $pseudoArg\n\n"
#      adjScript insert end "/bin/rm -f $workingInterName\n"

       adjScript insert end "sortPlanes.com -in $pseudoOutName -out $pseudoOutName -tau \$tauList -title\n"

       set workingOutName $pseudoOutName
      }

   if {$nusFlag && $maskFlag && ![strcasecmp $outProtocol NMRPIPE] && [strcasecmp $maskName None]} \
      { 
       if {$fmtCount > 0} \
          {
           adjScript insert end "\n"
           adjScript insert end "xyz2pipe -in $workingOutName -noWr $eol"
           adjScript insert end "| nusExpand.tcl$nusQuadStr -mask -noexpand -mode pipe -sampleCount $workingSampleCount $nusAvgStr -off $workingNUSOffset $eol"
           adjScript insert end "  -in stdin -out $workingMaskName -sample $workingNUSName$revStr\n"
          } \
       else \
          {
           adjScript insert end "\n"
           adjScript insert end "nusExpand.tcl$nusQuadStr -mask -noexpand -mode pipe -sampleCount $workingSampleCount $nusAvgStr -off $workingNUSOffset $eol"
           adjScript insert end " -in $workingOutName -out $workingMaskName -sample $workingNUSName$revStr\n"
          }
      }

   if {$rnmrtkFlag} \
      {
       adjScript insert end "\n"
       adjScript insert end "spectrum-translator -st nmrpipe -sf $workingOutName -dt rowland -df $workingRNMRTKName --force"
       adjScript insert end "\n"
      }

   if {$nmrDrawFlag} \
      {
       adjScript insert end "\n"
       adjScript insert end "#\n"

       set nmrDrawCmnd "nmrDraw -process -in [stripDotSlash $thisOutName] -fid [stripDotSlash $thisOutName]"

       if {$nusFlag} \
          {
           append nmrDrawCmnd " -mask [stripDotSlash $maskName] -sample [stripDotSlash $nusName] -nus"
          } 

       if {$pseudo3DFlag} \
          {
           append nmrDrawCmnd " -pseudo3D"
           set pseudoArg " -pseudo3D"
          } \
       else \
          {
           set pseudoArg ""
          }

       if {$verbose} \
          {
           puts "NMRDraw Command Definition:"
           puts " $nmrDrawCmnd"
          }

       if {$ioArgsFlag} \
          {
           if {$nusFlag} \
              {
               if {$dimCount >= 3} \
                  {
                   adjScript insert end "# nmrDraw -process -in [stripDotSlash $workingOutName] -fid [stripDotSlash $workingOutName] $eol"
                   adjScript insert end "#         -mask [stripDotSlash $workingMaskName] -sample [stripDotSlash $workingNUSName] -nus$pseudoArg\n"
                  } \
               else \
                  {
                   adjScript insert end "# nmrDraw -process -in [stripDotSlash $workingOutName] -fid [stripDotSlash $workingOutName]"
                   adjScript insert end " -mask [stripDotSlash $workingMaskName] -sample [stripDotSlash $workingNUSName] -nus\n"
                  }
              } \
           else \
              {
               adjScript insert end "# nmrDraw -process -in [stripDotSlash $workingOutName] -fid [stripDotSlash $workingOutName]\n"
              }

           adjScript insert end "#\n"
          }

       if {$nusFlag} \
          {
           if {$dimCount >= 3} \
              {
               adjScript insert end "# nmrDraw -process -in [stripDotSlash $thisOutName] -fid [stripDotSlash $thisOutName] $eol"
               adjScript insert end "#         -mask [stripDotSlash $maskName] -sample [stripDotSlash $nusName] -nus$pseudoArg\n"
              } \
           else \
              {
               adjScript insert end "# nmrDraw -process -in [stripDotSlash $thisOutName] -fid [stripDotSlash $thisOutName]"
               adjScript insert end " -mask [stripDotSlash $maskName] -sample [stripDotSlash $nusName] -nus\n"
              }
          } \
       else \
          {
           adjScript insert end "# nmrDraw -process -in [stripDotSlash $thisOutName] -fid [stripDotSlash $thisOutName]$pseudoArg\n"
          }
      }
       
   update
}

#
# Execute Main application:

main

#
# Bottom.
