#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Convert Comma-Separated Values (CSV) Text to NMRPipe Table."
    puts stderr " -in  inName  \[stdin\]    CSV Text Input."
    puts stderr " -out outName \[stdout\]   NMRPipe Table Output."
    puts stderr " -var varList \[V1 ...\]   Variable Names for VARS Line of Output." 
    puts stderr " -fmt fmtList \[Auto ...\] Value Formats for FORMAT Line of Output." 
    puts stderr "Other Flags:"
    puts stderr " -useFmt                 Use -fmtList Values to Format Output Text."
    puts stderr " -csvVar                 Use First Line of CSV as Variable Names."
    puts stderr " -csvFmt                 Use Second Line of CSV as Variable Formats."
    puts stderr " -allTxt                 Treat All Values as Text."
    puts stderr " -silent                 Silent Mode (No Error Messages)."
    puts stderr "Default Output Formats:"
    puts stderr " -intFmt  iFmt  \[%7d\]    Default Integer Format."
    puts stderr " -fltFmt  fFmt  \[%12.4f\] Default Float Format."
    puts stderr " -txtFmt  tFmt  \[Auto\]   Default Text Format."
    puts stderr "Values Missing in Input:"
    puts stderr " -intNull iVal  \[0\]      Substitute for Missing Integer Values." 
    puts stderr " -fltNull fVal  \[0.0\]    Substitute for Missing Floating-Point Values." 
    puts stderr " -txtNull tVal  \[None\]   Substitute for Missing Text Values." 
    puts stderr "Handling Input Values Which Contain Spaces:"
    puts stderr " -join    jChar \[_\]      Replace Spaces in Output with Join Character (Default)."
    puts stderr " -sq                     Use Single Quotes Around Values with Spaces."
    puts stderr " -dq                     Use Double Quotes Around Values with Spaces."
    exit 0
   }

set inName     stdin
set outName    stdout 

set hdrFlag    0
set fmtFlag    0
set useFmtFlag 0
set varList    ""
set fmtList    Auto 

set txtFmt     %16s
set intFmt     %7d
set fltFmt     %12.4f

set txtNull    None
set fltNull    0.0
set intNull    0

set joinChar   _ 
set quoteChar  None

set silentFlag 0

getArgD $ARGV -in  inName
getArgD $ARGV -out outName

getListArgD $ARGV -var varList
getListArgD $ARGV -fmt fmtList

if {[flagLoc $ARGV -useFmt]}   {set useFmtFlag 1}
if {[flagLoc $ARGV -nouseFmt]} {set useFmtFlag 0}

if {[flagLoc $ARGV -csvVar]}   {set hdrFlag 1}
if {[flagLoc $ARGV -nocsvVar]} {set hdrFlag 0}

if {[flagLoc $ARGV -csvFmt]}   {set fmtFlag 1}
if {[flagLoc $ARGV -nocsvFmt]} {set fmtFlag 0}

if {[flagLoc $ARGV -silent]}   {set silentFlag 1}
if {[flagLoc $ARGV -nosilent]} {set silentFlag 0}

if {[flagLoc $ARGV -allTxt]} \
   {
    set intNull None
    set fltNull None
    set txtNull None
    set intFmt  %6s
    set fltFmt  %12s
    set txtFmt  Auto
   }

getArgD $ARGV -intNull intNull
getArgD $ARGV -fltNull fltNull
getArgD $ARGV -txtNull txtNull

getArgD $ARGV -intFmt  intFmt
getArgD $ARGV -fltFmt  fltFmt
getArgD $ARGV -txtFmt  txtFmt

getArgD $ARGV -join    joinChar

if {[flagLoc $argv -join]} {set quoteChar None}
if {[flagLoc $argv -sq]}   {set quoteChar '}
if {[flagLoc $argv -dq]}   {set quoteChar \"}

if {[strcasecmp $quoteChar None]} {set joinChar None}

#
#------------------------------------------------------------

proc showThisWarning { str } \
{
   global silentFlag

   if {$silentFlag} return

   puts stderr $str
}

proc csv2tabExit { status } \
{
   global varCount entryCount lineCount errCount outUnit silentFlag

   if {$silentFlag} \
      {
       if {[strcasecmp $outUnit stdout]} {puts "entryCount $entryCount varCount $varCount errCount $errCount"}
       exit 0
      } \
   else \
      {
       puts stderr "Input Lines: $lineCount Output Entries: $entryCount Variables: $varCount Errors: $errCount"
       exit $status 
      }
}

set inUnit     stdin
set outUnit    stdout 

set uVarCount  0
set varCount   0
set entryCount 0
set lineCount  0
set dataCount  0
set errCount   0

if {![strcasecmp $inName stdin]} \
   {
    set inUnit stdin
   } \
else \
   {
    if {![file exists $inName]} \
       {
        showThisWarning "CSV2Tab Error Finding CSV Input File $inName"
        csv2tabExit 1
       }

    set error [catch {set inUnit [open $inName r]}]

    if {$error} \
       {
        showThisWarnning "CSV2Tab Error Opening CSV Input File $inName"
        csv2tabExit 1
       }
   }

if {![strcasecmp $outName stdout]} \
   {
    set outUnit stdout
   } \
elseif {![strcasecmp $outName stderr]} \
   {
    set outUnit stderr
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        showThisWarning "CSV2Tab Error Opening Output File $outName"
        csv2tabExit 1
       }
   }

set uVarCount [llength $varList]
if {$uVarCount} {set varCount $uVarCount}

set commentCount 0

while {[gets $inUnit origLine] >= 0} \
   {
    incr lineCount

    set line [string trim $origLine]
    set key  [string index $line 0]
    set n    [cntToken $line ,]

    if {!$n || ![strcmp $key "#"]} \
       {
        incr commentCount
        set commentInfo($commentCount) $line
        continue
       }

    if {!$varCount} \
       {
        for {set j 1} {$j <= n} {incr j} \
           {
            set val [string trim [getToken $j $line ,]]
            if {![llength $val]} break
            incr varCount
           }

        if {!$varCount} continue
       }

    incr dataCount

    if {$n < $varCount} \
       {
#       showThisWarning "CSV2Tab Warning: Too few values on line $lineCount. Expected: $varCount Actual: $n"
#       incr errCount
       } \
    elseif {$n > $varCount} \
       {
#       showThisWarning "CSV2Tab Warning: Too many values on line $lineCount. Expected: $varCount Actual: $n"
#       incr errCount
       }

    if {$dataCount == 1} \
       {
        for {set j 1} {$j <= $varCount} {incr j} \
           {
            set typeInfo($j) ""
            set lenInfo($j)  1 
            set defVal($j)   0
            set defFmt($j)   $txtNull
            set outFmt($j)   %s
           }

        if {$hdrFlag} \
           {
            set varList ""

            for {set j 1} {$j <= $varCount} {incr j} \
               {
                set val     [string trim [getToken $j $line ,]]
                set varName [join [string toupper $val] "_"]
                set strLen  [string length $varName]

                if {!$strLen || [string is integer $val] || [string is double $val]} \
                   {
                    showThisWarning "CSV2Tab Warning: Bad Variable Name in CSV Header Item $j Line $lineCount: $val"
                    incr errCount

                    set varName V$varName
                   }

                if {[lsearch -exact $varList $varName] >= 0} \
                   {
                    showThisWarning "CSV2Tab Warning: Duplicate Variable Name in CSV Header Item $j Line $lineCount: $val"
                    incr errCount
                   }

                lappend varList $varName
               }

            continue
           }
       }

    if {$dataCount == 2} \
       {
        if {$fmtFlag} \
           {
            set fmtList ""

            for {set j 1} {$j <= $varCount} {incr j} \
               {
                set fmt [string trim [getToken $j $line ,]]

                set strLen [string length $varName]
                if {!$strLen} {set fmt Auto}

                if {[strcasecmp $fmt Auto]} \
                   {
                    set wordCount [llength $fmt]
                    set c         [string index $fmt 0]

                    if {$wordCount != 1 || [strcmp $c %]} \
                       {
                        showThisWarning "CSV2Tab Warning: Bad Format in CSV Format Item $j Line $lineCount: $fmt"
                        incr errCount

                        set fmt Auto
                       }
                   }

                lappend fmtList $fmt
               }

            continue
           }
       }

    incr entryCount

    set i $entryCount

    for {set j 1} {$j <= $varCount} {incr j} \
       {
        set val [string trim [getToken $j $line ,]]

        if {[string is integer $val]} \
           {
            if {[lsearch -exact $typeInfo($j) int] < 0}   {lappend typeInfo($j) int}
           } \
        elseif {[string is double $val]} \
           {
            if {[lsearch -exact $typeInfo($j) float] < 0} {lappend typeInfo($j) float}
           } \
        else \
           {
            if {[lsearch -exact $typeInfo($j) text] < 0}  {lappend typeInfo($j) text}
           }

        set csvInfo($i,$j) $val
       }
   }

set j 1

while {[llength $varList] < $varCount} \
   {
    if {[lsearch -exact $varList V$j] < 0} {lappend varList V$j}
    incr j
   }

while {[llength $fmtList] < $varCount} {lappend fmtList Auto} 

set newList ""

for {set j 1} {$j <= $varCount} {incr j} \
   {
    set maxWordCount($j) 1

    set thisFmt [lindex $fmtList [expr $j - 1]]

    if {[lsearch -exact $typeInfo($j) text] >= 0} \
       {
        set defFmt($j) $txtFmt
        set defVal($j) $txtNull
       } \
    elseif {[lsearch -exact $typeInfo($j) float] >= 0} \
       {
        set defFmt($j) $fltFmt
        set defVal($j) $fltNull

        if {![strcasecmp $thisFmt Auto]} {set thisFmt $defFmt($j)}
       } \
    elseif {[lsearch -exact $typeInfo($j) int] >= 0} \
       {
        set defFmt($j) $intFmt
        set defVal($j) $intNull

        if {![strcasecmp $thisFmt Auto]} {set thisFmt $defFmt($j)}
       } \
    else \
       {
        set defFmt($j) $txtFmt
        set defVal($j) $txtNull

        if {![strcasecmp $thisFmt Auto]} {set thisFmt $defFmt($j)}
       }


    lappend newList $thisFmt
   }

set fmtList $newList

for {set i 1} {$i <= $entryCount} {incr i} \
   {
    for {set j 1} {$j <= $varCount} {incr j} \
       {
        set val $csvInfo($i,$j)
        if {![llength $val]} {set val $defVal($j)}

        set wordCount [llength $val]

        if {$wordCount > $maxWordCount($j)} {set maxWordCount($j) $wordCount}
 
        if {$wordCount > 1} \
           {
            if {![strcasecmp $joinChar None]} \
               {
                set val [format %s%s%s $quoteChar $val $quoteChar]
               } \
            else \
               {
                set val [join $val $joinChar]
               }
           }

        set strLen [string length $val]

        if {$strLen > $lenInfo($j)} {set lenInfo($j) $strLen}

        set csvInfo($i,$j) $val
       }
   }

puts $outUnit ""

for {set j 1} {$j < $commentCount} {incr j} \
   {
    if {$j > 1} \
       {
        set jPrev [expr $j - 1]
        if {![llength $commentInfo($jPrev)] && ![llength $commentInfo($j)]} continue
       }

    puts $outUnit $commentInfo($j)
   }

if {$commentCount} {puts $outUnit ""}

set newList ""

for {set j 1} {$j <= $varCount} {incr j} \
    {
     if {![strcasecmp $joinChar None] && $maxWordCount($j) > 1} {incr lenInfo($j) 2}

     set thisFmt    [lindex $fmtList [expr $j - 1]]
     set outFmt($j) [format %%%ds $lenInfo($j)]

     if {![strcasecmp $thisFmt Auto]} {set thisFmt [format %%%ds [expr $lenInfo($j) + 2]]}

     lappend newList $thisFmt
    }

set fmtList $newList

puts -nonewline $outUnit "VARS  "

for {set j 1} {$j <= $varCount} {incr j} \
    {
     set nV [string length [lindex $varList [expr $j - 1]]]
     set nF [string length [lindex $fmtList [expr $j - 1]]]

     set thisFmt [format "  %%-%ds" [getMax "$nV $nF"]]

     puts -nonewline $outUnit [format $thisFmt [lindex $varList [expr $j - 1]]]
    }

puts $outUnit ""

puts -nonewline $outUnit "FORMAT"

for {set j 1} {$j <= $varCount} {incr j} \
    {
     set nV [string length [lindex $varList [expr $j - 1]]]
     set nF [string length [lindex $fmtList [expr $j - 1]]]

     set thisFmt [format "  %%-%ds" [getMax "$nV $nF"]]

     puts -nonewline $outUnit [format $thisFmt [lindex $fmtList [expr $j - 1]]]
    }

puts $outUnit ""
puts $outUnit ""

for {set i 1} {$i <= $entryCount} {incr i} \
   {
    for {set j 1} {$j <= $varCount} {incr j} \
       {
        if {$useFmtFlag} \
           {
            set thisFmt [lindex $fmtList [expr $j - 1]]
            puts -nonewline $outUnit [format " $outFmt($j)" [format $thisFmt $csvInfo($i,$j)]]
           } \
        else \
           {
            puts -nonewline $outUnit [format "  $outFmt($j)" $csvInfo($i,$j)]
           }
       }

    puts $outUnit ""
   }

csv2tabExit 0
