#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "SVD Fit of Dipolar Couplings for an Ensemble of PDB Files:"
    puts " -dc     dcName   \[dObsA.tab\]      Input DC Table."
    puts " -pdb    pdbList  \[s\[0-9\]*.pdb\]    Input PDB List."
    puts " -w      wList    \[1.0 ...\]        Weighting Factors for PDBs."
    puts " -out    outName  \[dEnsemble.tab\]  Output DC Name for Ensemble Result."
    puts " -outDir outDir   \[dCalc\]          Output Directory for Individual Results."
    puts " -ignore                           Ignore Chain/Seg Info (Default)."
    puts " -noignore                         Keep Chain and Seg Info."
    puts " -verb                             Verbose Mode ON (Default)."
    puts " -noverb                           Verbose Mode OFF."
    puts "Notes:"
    puts " PDB files must include all atoms used in the DC input tables." 
    exit
   }

set DC_METHYL_AVG -0.309107

set dcName     dObsA.tab
set outName    dEnsemble.tab
set outDir     dCalc
set pdbList    [lsort -dictionary [glob -nocomplain s\[0-9\]*.pdb]]
set wwList     ""

set tensor     A
set verbose    1
set ignoreFlag 1

getArgD     $ARGV -in     dcName
getArgD     $ARGV -dc     dcName
getArgD     $ARGV -out    outName
getArgD     $ARGV -outDir outDir

getListArgD $ARGV -pdb    pdbList

set wwList [getRepeat 1.0 [llength $pdbList]]

getListArgD $ARGV -w   wwList

if {[flagLoc $ARGV -ignore]}   {set ignoreFlag 1}
if {[flagLoc $ARGV -noignore]} {set ignoreFlag 0}

if {[llength $wwList] != [llength $pdbList]} \
   {
    puts stderr "Error: PDB List/Weight List mismatch."
    exit 1
   }

foreach thisName $pdbList \
   {
    if {![file exists $thisName]} \
       {
        puts stderr "Error Finding PDB Input $thisName"
        exit 1
       }
   }

if {![file exists $dcName]} \
   {
    puts stderr "Error Finding DC Input $dcName"
    exit 1
   }

if {![file isdirectory $outDir]} {catch "file mkdir $outDir"}

#
#-------------------------------------------------------------------------

proc getCoords { entry item pdbTab xPtr yPtr zPtr } \
{
    upvar $xPtr x
    upvar $yPtr y
    upvar $zPtr z

    set resID    [gdbGet $entry RESID_$item]
    set atomName [gdbGet $entry ATOMNAME_$item]

    set atomList [getAtomEntries $pdbTab $resID $atomName]
    set nAtoms   [llength $atomList]

    if {$nAtoms} \
       {
        set x ""
        set y ""
        set z ""

        foreach atom $atomList \
           {
            lappend x [gdbGet $atom X]
            lappend y [gdbGet $atom Y]
            lappend z [gdbGet $atom Z]
           }
       } \
    else \
       {
        set x 0.0
        set y 0.0
        set z 0.0
       }

   return $nAtoms
}

proc getSaupe { xII yII zII xJJ yJJ zJJ dVal di dd w ww tensor } \
{
   global s1 s2 s3 s4 s5 tcl_precision DC_METHYL_AVG

   set tcl_precision 17

   set nI [llength $xII]
   set nJ [llength $xJJ]

   set d      [expr $dVal/abs($di)]
   set sigma  [expr $dd/abs($di)]
   set q      [expr 1.0/$sigma]

   set s1($tensor) 0.0
   set s2($tensor) 0.0
   set s3($tensor) 0.0
   set s4($tensor) 0.0
   set s5($tensor) 0.0

   if {$nI*$nJ == 3} {set ww [expr 3.0*$ww*abs($DC_METHYL_AVG)]}

   for {set i 0} {$i < $nI} {incr i} \
      {
       set xI [lindex $xII $i]
       set yI [lindex $yII $i]
       set zI [lindex $zII $i]

       for {set j 0} {$j < $nJ} {incr j} \
          {
           set xJ [lindex $xJJ $j]
           set yJ [lindex $yJJ $j]
           set zJ [lindex $zJJ $j]

           set xIJ [expr $xJ - $xI]
           set yIJ [expr $yJ - $yI]
           set zIJ [expr $zJ - $zI]

           set rIJ  [expr sqrt($xIJ*$xIJ + $yIJ*$yIJ + $zIJ*$zIJ)] 
           set r    [expr 1.0/$rIJ]

           set xIJ [expr $xIJ*$r]
           set yIJ [expr $yIJ*$r]
           set zIJ [expr $zIJ*$r]

           set s1($tensor) [expr $s1($tensor) + $ww*$q*0.5*(3.0*$zIJ*$zIJ - 1.0)]
           set s2($tensor) [expr $s2($tensor) + $ww*$q*0.5*($xIJ*$xIJ - $yIJ*$yIJ)]
           set s3($tensor) [expr $s3($tensor) + $ww*$q*2.0*$xIJ*$yIJ]
           set s4($tensor) [expr $s4($tensor) + $ww*$q*2.0*$xIJ*$zIJ]
           set s5($tensor) [expr $s5($tensor) + $ww*$q*2.0*$yIJ*$zIJ]
          }
      }

   set qq [expr $q*$d]

   return $qq
}

proc getD { xII yII zII xJJ yJJ zJJ di dd w ww k } \
{
   global s1 s2 s3 s4 s5 tcl_precision DC_METHYL_AVG

   set tcl_precision 17

   set nI [llength $xII]
   set nJ [llength $xJJ]

   set dVal 0.0

   for {set i 0} {$i < $nI} {incr i} \
      {
       set xI [lindex $xII $i]
       set yI [lindex $yII $i]
       set zI [lindex $zII $i]

       for {set j 0} {$j < $nJ} {incr j} \
          {
           set xJ [lindex $xJJ $j]
           set yJ [lindex $yJJ $j]
           set zJ [lindex $zJJ $j]

           set xIJ [expr $xJ - $xI]
           set yIJ [expr $yJ - $yI]
           set zIJ [expr $zJ - $zI]

           set rIJ  [expr sqrt($xIJ*$xIJ + $yIJ*$yIJ + $zIJ*$zIJ)]
           set r    [expr 1.0/$rIJ]

           set xIJ [expr $xIJ*$r]
           set yIJ [expr $yIJ*$r]
           set zIJ [expr $zIJ*$r]

           set cs1 [expr $ww*0.5*(3.0*$zIJ*$zIJ - 1.0)]
           set cs2 [expr $ww*0.5*($xIJ*$xIJ - $yIJ*$yIJ)]
           set cs3 [expr $ww*2.0*$xIJ*$yIJ]
           set cs4 [expr $ww*2.0*$xIJ*$zIJ]
           set cs5 [expr $ww*2.0*$yIJ*$zIJ]

           set dVal [expr $dVal + $cs1*$s1($k)]
           set dVal [expr $dVal + $cs2*$s2($k)]
           set dVal [expr $dVal + $cs3*$s3($k)]
           set dVal [expr $dVal + $cs4*$s4($k)]
           set dVal [expr $dVal + $cs5*$s5($k)]
          }
      }
 
   set dVal [expr $dVal*abs($di)]

   if {$nI*$nJ == 3} {set dVal [expr 3.0*$dVal*abs($DC_METHYL_AVG)]}

   return $dVal
}

#
# DC Tables and PDB Files must correspond exactly.
# Use dcPrep.tcl to prepare dCalc tables that contain DI values.

set tcl_precision 17

dynInit

if {$verbose} {puts "Reading First PDB File ..."}
puts "dynRead -pdb -in [lindex $pdbList 0]"
set pdbSrc [dynRead -pdb -in [lindex $pdbList 0]]

if {$verbose} {puts "Reading First DC Input File ..."}
set dcSrc  [dynRead -dc -in $dcName]

if {$verbose} {puts "Initializing DC Input Values ..."}

if {$ignoreFlag} \
   {
    set dcList [eval exec dcPrep.tcl -dc $dcName -outDir $outDir -pdb $pdbList -ignore -noverb]
   } \
else \
   {
    set dcList [eval exec dcPrep.tcl -dc $dcName -outDir $outDir -pdb $pdbList -noignore -noverb]
   }

set dcCount [llength $dcList]

if {!$dcCount} \
   {
    puts stderr "Error Initializing DC Input Values."
    exit 1
   }

for {set i 0} {$i < $dcCount} {incr i} \
   {
    set pdbName   [lindex $pdbList $i]
    set thisName  [lindex $dcList  $i]

    set cond "D_OBS < 999.0 && D < 999.0 && DD < 999.0"

    if {$verbose} {puts " Reading Data for Structure [expr $i + 1] ..."}

    set pdbTab($i) [gdbRead table -in $pdbName -pdb]
    set dTab($i)   [gdbRead table -in $thisName]
    set eList($i)  [gdbSelect -from $dTab($i) -cond $cond]
   }

set n [llength $eList(0)]

set wwSum [getSum $wwList]
if {$wwSum == 0.0} {set wwSum 1.0}

set wSum 1.0

set xList  ""
set yList  ""

if {$verbose} {puts "Building Order Matrix Data ..."}

for {set j 0} {$j < $n} {incr j} \
   {
    for {set i 0} {$i < $dcCount} {incr i} \
       {
        set entry($i) [lindex $eList($i) $j]

        set yVal  [gdbGet $entry($i) D_OBS]
        set di    [gdbGet $entry($i) DI]
        set dd    [gdbGet $entry($i) DD]
        set w     [gdbGet $entry($i) W]

#       set cs1   [gdbGet $entry($i) S1]
#       set cs2   [gdbGet $entry($i) S2]
#       set cs3   [gdbGet $entry($i) S3]
#       set cs4   [gdbGet $entry($i) S4]
#       set cs5   [gdbGet $entry($i) S5]
#       set nVec  [gdbGet $entry($i) NVEC]

        set ww    [expr [lindex $wwList $i]/$wwSum]

        set nI [getCoords $entry($i) I $pdbTab($i) xI yI zI]
        set nJ [getCoords $entry($i) J $pdbTab($i) xJ yJ zJ]

        set q  [getSaupe $xI $yI $zI $xJ $yJ $zJ $yVal $di $dd $w $ww $i]

        lappend xList $s1($i) $s2($i) $s3($i) $s4($i) $s5($i)

        if {!$i} {lappend yList $q}
       }
   }


if {$verbose} {puts "Determining Order Matrix from [llength $yList] Couplings ..."}
set cList [svdLinSol -x $xList -y $yList]

if {$verbose} {puts "Calculating Couplings ..."}

gdbDelNote $dTab(0)

set j 0

for {set i 0} {$i < $dcCount} {incr i} \
   {
    set pdbName [lindex $pdbList $i]

    set s1($i) [lindex $cList $j]
    incr j
    set s2($i) [lindex $cList $j]
    incr j
    set s3($i) [lindex $cList $j]
    incr j
    set s4($i) [lindex $cList $j]
    incr j
    set s5($i) [lindex $cList $j]
    incr j
}

gdbAddNote $dTab(0) "REMARK PDB Weights [getFormat $wwList %.3f]"
gdbAddNote $dTab(0) ""

for {set i 0} {$i < $dcCount} {incr i} \
   {
    set pdbName [lindex $pdbList $i]

    set fmt "Saupe %d: %s % .4e % .4e % .4e % .4e % .4e"

    set str \
     [format $fmt [expr $i+1] $pdbName $s1($i) $s2($i) $s3($i) $s4($i) $s5($i)]

    gdbAddNote $dTab(0) "REMARK $str"
    puts $str
   }

gdbAddNote $dTab(0) ""

puts ""

for {set i 0} {$i < $dcCount} {incr i} \
   {
    set pdbName [lindex $pdbList $i]

    set fmt "Tensor %d: %s Da: % .4e Dr: % .4e Generalized Mag: %.4e"

    set dInfo [dynSimulate -dc $dcSrc -src $pdbSrc \
                           -saupe $s1($i) $s2($i) $s3($i) $s4($i) $s5($i)]

    set Da  [lindex $dInfo 3]
    set Dr  [lindex $dInfo 4]
    set Gm  [expr sqrt(8.0*$Da*$Da + 6.0*$Dr*$Dr)]
    set str [format $fmt [expr $i+1] $pdbName $Da $Dr $Gm] 

    gdbAddNote $dTab(0) "REMARK $str"
    puts $str
   }

gdbAddNote $dTab(0) ""
puts ""

set sa(0) $s1(0)
set sa(1) $s2(0)
set sa(2) $s3(0)
set sa(3) $s4(0)
set sa(4) $s5(0)

for {set i 1} {$i < $dcCount} {incr i} \
   {
    set sb(0) $s1($i)
    set sb(1) $s2($i)
    set sb(2) $s3($i)
    set sb(3) $s4($i)
    set sb(4) $s5($i)
  
    set a [getSaupeAngle sa sb]

    set fmt "Generalized Angle Between Tensor 1 and %d: % 7.3f" 
    set str [format $fmt [expr $i+1] $a]

    gdbAddNote $dTab(0) "REMARK $str"
    puts $str
   }

gdbAddNote $dTab(0) ""
puts ""

set dList(SVD) ""

for {set j 0} {$j < $n} {incr j} \
   {
    set dSum 0.0

    for {set i 0} {$i < $dcCount} {incr i } \
       {
        set entry($i) [lindex $eList($i) $j]

        set yVal  [gdbGet $entry($i) D_OBS]
        set di    [gdbGet $entry($i) DI]
        set dd    [gdbGet $entry($i) DD]
        set w     [gdbGet $entry($i) W]

        set ww    [expr [lindex $wwList $i]/$wwSum]

        set nI [getCoords $entry($i) I $pdbTab($i) xI yI zI]
        set nJ [getCoords $entry($i) J $pdbTab($i) xJ yJ zJ]

        set dCalc [getD $xI $yI $zI $xJ $yJ $zJ $di $dd $w $ww $i]
        set dSum  [expr $dSum + $dCalc]
       }

    set dDiff [expr $yVal - $dSum]
    set dVal  [expr $w*$dDiff]

    gdbSet $entry(0) D      $dSum
    gdbSet $entry(0) D_DIFF $dDiff

    lappend dList(SVD) [expr $w*($yVal - $dSum)] 
   }

set  str "RMS: [format %.3f [getRMS $dList(SVD)]]"
puts $str

gdbAddNote $dTab(0) "REMARK $str"
gdbAddNote $dTab(0) ""

if {$verbose} {puts "Saving Results ..."}
gdbWrite $dTab(0) $outName

exit
