#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]


if {[flagLoc $ARGV -help]} \
   {
    puts "Delete Entries from NUS Schedule According to Bruker Overflow Log."
    puts "Input:"
    puts " -sample      sName    \[nuslist\]       Non-Uniform Sampling (NUS) Schedule Input."
    puts " -sampleCount sCount   \[Auto\]          Number of Valid Samples in Schedule."
    puts " -log         logName  \[overflow*.txt\] Overflow Log Input."
    puts " -parmDir     pDir     \[.\]             Parameter Directory (For acqus acqu2s)."
    puts " -out         newName  \[None\]          Adjusted NUS Schedule Output, or Keyword None."
    puts " -ndim        dimCount \[Auto\]          Dimensions Count of Corresponding Spectrum."
    puts " -list                                 List Bad Schedule Entry Locations."
    puts " -verb                                 Verbose Mode ON (Default)."
    puts " -noverb                               Verbose Mode OFF."
    puts "Notes:"
    puts " 1. The overflow log is expected to have 3 columns of integers,"
    puts "    with the second column containing the experiment number,"
    puts "    with the first experiment starting at 1."
    puts " 2. 2D spectra are expected to have 2 experiments for each entry"
    puts "    in the NUS schedule file, 3D spectra have 4 experiments, etc." 
    puts " 3. To use this tool with conventional data, create a sampling schedule"
    puts "    for fully sampled data: fakeNUS.com -nus 1.0 -in ..."
    puts " 4. Verbose messages are printed to standard error."
    puts " 5. Values from -list mode are printed to standard output."
    puts " 6. Using -sampleCount Auto will read parameter files acqus and acqu2s"
    puts "    to determine the number of valid samples."
    exit 0
   }

set schedName   nuslist
set sampleCount Auto
set parmDir     .

set logName     [lindex [glob -nocomplain overflow*.txt] 0]
set outName     None
set extraFiles  None
set dimCount    Auto
set verbose     1
set listFlag    0

getArgD $ARGV -sample      schedName
getArgD $ARGV -sampleCount sampleCount
getArgD $ARGV -log         logName
getArgD $ARGV -out         outName
getArgD $ARGV -parmDir     parmDir
getArgD $ARGV -ndim        dimCount

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -list]}   {set listFlag 1}
if {[flagLoc $ARGV -nolist]} {set listFlag 0}

#
#----------------------------------------------------------------------------

if {![file exists $schedName]} \
   {
    puts stderr "Error Finding NUS Schedule $schedName"
    exit 1
   }

if {![file exists $logName]} \
   {
    puts stderr "Error Finding Overflow Log $logName"
    exit 1
   }

set schedInfo(lineCount) 0
set logInfo(lineCount)   0

set status [getTextLines $schedName schedInfo]
set status [getTextLines $logName   logInfo]

if {![strcasecmp $dimCount Auto]} \
   {
    for {set i 0} {$i < $schedInfo(lineCount)} {incr i} \
       {
        set n [llength $schedInfo($i)]
        if {!$n} continue

        if {![string is integer [lindex $schedInfo($i) 0]]} continue

        set dimCount [expr $n + 1]
        break
       }
   }

set expPerIncr 1

for {set i 1} {$i < $dimCount} {incr i} {set expPerIncr [expr 2*$expPerIncr]}

set nCol [expr $dimCount - 1]

set thisSampleCount 0
set ovCount         0

for {set i 0} {$i < $schedInfo(lineCount)} {incr i} \
   {
    set n [llength $schedInfo($i)]
    if {$n != $nCol} continue

    set item [lindex $schedInfo($i) 0] 
    set key  [string index $item 0]

    if {![strcmp $key "#"] || ![string is integer $item]} continue

    set nusInfo($thisSampleCount) $schedInfo($i)
    incr thisSampleCount
   }

if {![strcasecmp $sampleCount Auto]} \
   {
    if {[file exists $schedName] && [file exists $parmDir/acqus] && [file exists $parmDir/acqu2s]} \
       {
        set sInfo [exec getNUSInfo.tcl -sample $schedName -parmDir $parmDir]
        if {![info exists sInfo]} {set sInfo ""}

        set thisCount [getNamedItem sampleCount $sInfo]
       
        if {[string is integer $thisCount]} \
           {
            set extraFiles  "acqus acqu2s"
            set sampleCount $thisCount
           }
       }
   }

if {![strcasecmp $sampleCount Auto]} {set sampleCount $thisSampleCount}

set nCol 3

for {set i 0} {$i < $logInfo(lineCount)} {incr i} \
   {
    set n [llength $logInfo($i)]
    if {$n != $nCol} continue

    set item [lindex $logInfo($i) 0] 
    set key  [string index $item 0]

    if {![strcmp $key "#"] || ![string is integer $item]} continue

    set ovInfo($ovCount) $logInfo($i)

    incr ovCount
   }

set expList ""

for {set i 0} {$i < $sampleCount} {incr i} \
   {
    for {set j 0} {$j < $expPerIncr} {incr j} \
       {
        lappend expList $i
       }
   }

set badList  ""
set delCount 0

for {set i 0} {$i < $ovCount} {incr i} \
   {
    if {$verbose && !$i} {puts stderr ""}

    set badExp [lindex $ovInfo($i) 1]

    if {![string is integer $badExp]} \
       {
        puts stderr "Experiment ID Error in Overflow Log Entry [expr $i + 1]: $ovInfo($i)"
        continue
       }

    set loc [lindex $expList [expr $badExp - 1]]

    if {![info exists nusInfo($loc)]} \
       {
        puts stderr "Unknown Experiment ID. Loc: $loc Overflow Log Entry: [expr $i + 1] Contents: $ovInfo($i)"
        continue
       }

    if {[llength $nusInfo($loc)]} \
       {
        if {$verbose} {puts stderr [format "NUS Delete. Overflow Experiment: %4d Increment: %4d Value: %s" $badExp $loc $nusInfo($loc)]}

        if {![info exists badInfo($loc)]} {set badInfo($loc) $nusInfo($loc)}

        lappend badList $loc

        set nusInfo($loc) ""
        incr delCount
       } \
    else \
       {
        lappend badList $loc

        if {![info exists badInfo($loc)]} {set badInfo($loc) None}
        if {$verbose} {puts stderr [format "NUS Delete. Overflow Experiment: %4d Increment: %4d Value: %s (Already Deleted)" $badExp $loc $badInfo($loc)]}
       }
   }

if {[strcasecmp $outName None]} \
   {
    if {![strcasecmp $outName stdout]} \
       {
        set outUnit stdout
       } \
    else \
       {
        set error [catch {set outUnit [open $outName w]}]

        if {$error} \
           {
            puts stderr "deleteNUS Error Opening Output $outName"
            exit 1
           }
       }
   }

set validCount 0

for {set i 0} {$i < $sampleCount} {incr i} \
   {
    if {![llength $nusInfo($i)]} continue

    if {[strcasecmp $outName None]} {puts $outUnit $nusInfo($i)} 
    incr validCount
   }

if {[strcasecmp $outName None] && [strcasecmp $outName stdout]} {catch "close $outUnit"}

if {$verbose} \
   {
    puts stderr ""
    puts stderr [format "NUS Delete. Schedule Name:              %s" $schedName]
    puts stderr [format "NUS Delete. Auxiliary Files:            %s" $extraFiles]
    puts stderr [format "NUS Delete. Schedule File Lines:        %s" $schedInfo(lineCount)]
    puts stderr [format "NUS Delete. Schedule Increments Used:   %s" $sampleCount]
    puts stderr [format "NUS Delete. Schedule Increments Output: %s" $validCount]
    puts stderr [format "NUS Delete. Overflow Log Name:          %s" $logName]
    puts stderr [format "NUS Delete. Overflow Log Lines:         %s" $logInfo(lineCount)]
    puts stderr [format "NUS Delete. Overflow Count:             %s" $ovCount]
    puts stderr [format "NUS Delete. Increments Deleted:         %s" $delCount]
    puts stderr [format "NUS Delete. Dimension Count:            %s" $dimCount]
    puts stderr [format "NUS Delete. Experiments Per Increment:  %s" $expPerIncr]

    if {[llength $badList] > 10} \
       {
        puts stderr [format "NUS Delete. Bad Schedule Entries:       %s ..." [lrange $badList 0 9 ]]
       } \
    else \
       {
        puts stderr [format "NUS Delete. Bad Schedule Entries:       %s" $badList]
       }
   }

if {$listFlag} {puts -nonewline $badList}

exit 0
