#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

#
# dosyView.tcl: view 1D vectors for 2D DOSY, export to fitting script.
#------------------------------------------------------------------------------

proc export1D {} \
{
   global noise specLocPPM tauList dx fitID inName scriptName 
   global inList dName auxInfo skip

   incr fitID

   set inCount [llength $inList]
   set j       [expr 1 + $inCount]
 
   set roiInfo [getROIInfo -roi $j]
   set rdata   [lindex $roiInfo 0]
   set n       [lindex [lindex $roiInfo 2] 0]
   set nTau    [llength $tauList]

   set xList ""
   set yList ""
   set nList ""
   set tList ""

   for {set i $skip} {$i < $n} {incr i} \
      {
       lappend xList $i
       lappend yList [getPtrVal $rdata $i]
       lappend nList [lindex $noise   $i]
       lappend tList [lindex $tauList $i]
      }

   if {$nTau} \
      {
       if {$n != $nTau} \
          {
           puts stderr "Error: wrong number of points in tauList."
           exit
          }

       set xList $tList
      }

   set str  [format "%s %.2fppm +/- %d pts" $inName $specLocPPM $dx]
   set cmnd "$scriptName $auxInfo -anchor NE"

   append cmnd \
    " -x $xList -y $yList -noise $noise -id $fitID -title $str"

   append cmnd " -xLab Tau -yLab DOSY"

   set error [catch {set sUnit [open $dName w 0744]}]

   if {$error} \
      {
       puts stderr "Error Opening Fitting Script $dName"
       return 1
      }

   puts $sUnit "#!/bin/csh -f"
   puts $sUnit ""
   puts $sUnit $cmnd

   close $sUnit

   catch "exec xterm -iconic -e $dName &"
}

#
# Procedure to move the vertical cursor.

proc moveVCursor { ix iy } \
{
   global cx1 cxn cy1 cyn ixPrev

   if {$ix < $cx1 || $ix > $cxn} return
   if {$iy < $cy1 || $iy > $cyn} return

   .c coords cursor $ix $cy1 $ix $cyn

   set ixPrev $ix

   select1DX 1 $ix
}

proc updateVCursor {} \
{
   global cx1 cxn cy1 cyn ixPrev specLocPPM

   set ix     [spec2pix -cell 1 -roi 1 -x ${specLocPPM}ppm]
   set ixPrev $ix

   .c coords cursor $ix $cy1 $ix $cyn
}

#
# Procedure to change the spectral position for the evolution.

proc select1DX { id ix } \
{
   global inList specLoc specLocPPM

   if {$id == 1} \
      {
       set j 1
       set specLocPPM [pix2spec -cell $id -roi $j -x $ix -units ppm]
       updateSpecLoc ${specLocPPM}ppm
      } \
   else \
      {
       set inCount [llength $inList]
       set j       [expr 1 + 2*$inCount]

       set specLocPPM [pix2spec -cell $id -roi $j -x $ix -units ppm]
       updateSpecLoc ${specLocPPM}ppm
       updateVCursor
      }

   adjXLoc 0
}

proc updateSliceLoc { sPPM } \
{
   global sliceLoc

   set fdata      [lindex [getROIInfo -roi 1] 1]
   set y0         [spec2pnt $fdata CUR_YDIM $sPPM]
   set sliceLoc   [expr int(0.5 + $y0)]
}

proc updateSpecLoc { sPPM } \
{
   global specLoc

   set fdata      [lindex [getROIInfo -roi 1] 1]
   set x0         [spec2pnt $fdata CUR_XDIM $sPPM]
   set specLoc    [expr int(0.5 + $x0)]
}

proc updateSpecPPM { sLoc } \
{
   global specLocPPM

   set fdata      [lindex [getROIInfo -roi 1] 1]
   set specLocPPM [pnt2spec $fdata CUR_XDIM $sLoc ppm]
}

proc updateDX {} \
{
   global dx

   if {![strcasecmp $dx None]} {set dx 0}

   set fdata [lindex [getROIInfo -roi 1] 1]
   set x0    [spec2pntW $fdata CUR_XDIM $dx]
   set dx    [expr int(0.5 + $x0)]
}

#
# Procedure to change vertical scale of 1D vector.
 
proc scale1D { id iy } \
{
   global minVal maxVal iyPrev

   if {$iy > $iyPrev($id)} \
      {
       set minVal($id) [expr $minVal($id)/0.95]
       set maxVal($id) [expr $maxVal($id)/0.95]
      } \
   else \
      {
       set minVal($id) [expr $minVal($id)*0.95]
       set maxVal($id) [expr $maxVal($id)*0.95]
      }

   set iyPrev($id) $iy

   draw1D $id
}

#
# Procedure to change vertical offset of 1D vector.

proc move1D { id iy } \
{
   global maxVal minVal iyPrev

   set range [expr $maxVal($id) - $minVal($id)]

   if {$iy > $iyPrev($id)} \
      {
       set minVal($id) [expr $minVal($id) + $range*0.025]
       set maxVal($id) [expr $maxVal($id) + $range*0.025]
      } \
   else \
      {
       set minVal($id) [expr $minVal($id) - $range*0.025]
       set maxVal($id) [expr $maxVal($id) - $range*0.025]
      }

   set iyPrev($id) $iy

   draw1D $id
}

proc read1DDOSY {} \
{
   global inList specLoc xName yName dx specXSize

   if {$specLoc < 1} {set specLoc $specXSize}
   if {$specLoc > $specXSize} {set specLoc 1}

   updateSpecPPM $specLoc
   updateDX

   if {$dx < 0} {set dx 0}

   set inCount  [llength $inList]
   set j        $inCount

   for {set i 1} {$i <= $inCount} {incr i} \
      {
       incr j  
       readROI -roi $j -ndim 1 -inROI $i -x Y_AXIS -dy X_AXIS $specLoc $dx
      }
}

proc read1DSpec {} \
{
   global xName yName sliceLoc specLocPPM specLoc specYSize
   global zoomW inList firstTime

   updateSpecPPM $specLoc

   if {$sliceLoc < 1} {set sliceLoc $specYSize}
   if {$sliceLoc > $specYSize} {set sliceLoc 1}

   set inCount [llength $inList]
   set j       [expr 2*$inCount]
 

   for {set i 1} {$i <= $inCount} {incr i} \
      {
       incr j 

       if {![strcasecmp $zoomW None]} \
          {
           readROI -roi $j -ndim 1 -inROI $i -x X_AXIS -dy Y_AXIS $sliceLoc 0
          } \
       else \
          {
           readROI -roi $j -ndim 1 -inROI $i \
                   -dx X_AXIS ${specLocPPM}ppm $zoomW  -dy Y_AXIS $sliceLoc 0 
          }
      }
}

proc read2DDOSY {} \
{
   global xName yName specXSize specYSize inList inName scale 
   global sliceLoc specLoc specLocPPM dx tauList tauMin tauMax

   set inCount [llength $inList]

   for {set i 1} {$i <= $inCount} {incr i} \
      {
       set inName [lindex $inList [expr $i - 1]]

       if {![file exists $inName]} \
          {
           puts stderr "Error Finding 2D Input File $inName"
           exit 1
          }

       readROI -ndim 2 -roi $i -in $inName -x $xName -y $yName 
      }

   set roiInfo   [getROIInfo -roi 1]
   set rdata     [lindex $roiInfo 0]
   set sList     [lindex $roiInfo 2]

   set specXSize [lindex $sList 0]
   set specYSize [lindex $sList 1]

   set n         [expr $specXSize*$specYSize]

   vsMult $rdata $scale $n

   set thisMin [vMin $rdata $n]
   set thisMax [vMax $rdata $n]

   if {[llength $tauList] != $specYSize} \
      {
       set tauList ""

       for {set i 0} {$i < $specYSize} {incr i} {lappend tauList $i}
      }

   set tauMin [getMin $tauList]
   set tauMax [getMax $tauList]

#  if {$tauMin > 0.0} {set tauMin 0}

   if {![strcasecmp $specLocPPM None]} {set specLocPPM 50%} 
   if {![strcasecmp $sliceLoc   None]} {set sliceLoc 1}
   if {![strcasecmp $dx         None]} {set dx 0}

   updateSpecLoc  $specLocPPM
   updateSpecPPM  $specLoc
   updateSliceLoc $sliceLoc

   updateDX
}

proc full1D { id } \
{
   global drawCount

   set drawCount($id) 0

   draw1D $id
}

proc draw1D { id } \
{
   global minVal maxVal firstTime drawCount specLoc specLocPPM dx 
   global inList colorList tauMin tauMax tauList

   if {$firstTime} return

   set inCount [llength $inList]

   if {!$drawCount($id)} \
      {
       if {$id == 2} \
          { 
           set roiInfo [getROIInfo -roi [expr 1 + $inCount]]
           set rdata   [lindex $roiInfo 0]
           set n       [lindex [lindex $roiInfo 2] 0]
          } \
       else \
          {
           set roiInfo [getROIInfo -roi 1]
           set rdata   [lindex $roiInfo 0]
           set xSize   [lindex [lindex $roiInfo 2] 0]
           set ySize   [lindex [lindex $roiInfo 2] 1]
           set n       [expr $xSize*$ySize]
          }

       set thisMin [vMin $rdata $n]
       set thisMax [vMax $rdata $n]

       if {$thisMin != $thisMax} \
          {
           set range  [expr $thisMax - $thisMin]
           set minVal($id) [expr $thisMin - 0.1*$range]
           set maxVal($id) [expr $thisMax + 0.1*$range]
          }
      }

   if {$id == 2} \
      {
       .t.c configure -title \
        [format "Evolution at %.2fppm +/- %s pts" $specLocPPM $dx]

       set j  [expr 1 + $inCount]
       set lw 2 

       for {set i 1} {$i <= $inCount} {incr i} \
          {
           set color [lindex $colorList [expr ($i - 1) % $inCount]]
           set yList [getROIVals $j]

           catch ".t.c element delete D$i"

           .t.c element create D$i -xdata $tauList -ydata $yList   \
                -color $color -fill white -pixels 3 -linewidth $lw

           .t.c axis configure x -min $tauMin
           .t.c axis configure y -min $minVal($id) -max $maxVal($id)

           incr j
          }
      } \
   else \
      {
       drawRec -win $id -x 0% 100% -y 0% 100% -fg gray30 -noupdate
       drawRec -fg black -cell $id -x 0% 100% -y 0% 100% -noupdate

       set j     [expr 1 + 2*$inCount]
       set lw    1 

       set fdata [lindex [getROIInfo -roi $j] 1]
       set x0    [spec2pnt $fdata CUR_XDIM ${specLocPPM}ppm]
       set regX1 [expr int(0.5 + $x0 - $dx)]
       set regXN [expr int(0.5 + $x0 + $dx)]

       drawRec -fg dimGray -cell $id -roi $j \
               -x ${regX1}pts ${regXN}pts -y 0% 100% -noupdate

       for {set i 1} {$i <= $inCount} {incr i} \
          {
           set color [lindex $colorList [expr ($i - 1) % $inCount]]

           if {$i == 1} \
              {
               drawROI -roi $j -cell $id -zero -bg None -fg $color \
                       -axis Bottom -fhi $minVal($id) -lhi $maxVal($id) \
                       -lw1d $lw -update
              } \
           else \
              {
               drawROI -roi $j -cell $id -bg None -fg $color \
                       -fhi $minVal($id) -lhi $maxVal($id) -update -lw1d $lw
              }
    
           incr j
          }
      }

   incr drawCount($id)

   update
}

proc resize1DCanvas { w id xSize ySize } \
{
   global qx1 qxn qy1 qyn xSize1D ySize1D firstTime

   if {$id == 2} \
      {
       bind $w.c <Configure> ""
       $w.c configure -width $xSize -height $ySize
       bind $w.c <Configure> "resize1DCanvas $w $id %w %h"
       return
      }

   bind $w.c <Configure> ""

   set xSize1D $xSize
   set ySize1D $ySize

   if {!$firstTime} {$w.c delete pixmap -win $id -tag nmr$id}

   $w.c create pixmap 0 0 -width $xSize1D -height $ySize1D -win $id -tag nmr$id

   set qx1 10
   set qy1 10
   set qxn [expr $xSize - 10]
   set qyn [expr $ySize - 70]

   defineCell -x $qx1 $qxn -y $qy1 $qyn -cell $id -verb -win $id 

   bind $w.c <B1-Motion> "select1DX $id %x"
   bind $w.c <B2-Motion> "scale1D   $id %y"
   bind $w.c <B3-Motion> "move1D    $id %y"

   bind $w.c <Configure> "resize1DCanvas $w $id %w %h"

   draw1D $id
}

proc resize2DCanvas { w id xSize ySize } \
{
   global cx1 cxn cy1 cyn xSize2D ySize2D firstTime

   bind $w <Configure> ""

   set xSize2D $xSize
   set ySize2D $ySize

   if {!$firstTime} {$w delete pixmap -win 1 -tag nmr$id}

   $w create pixmap 0 0 -width $xSize2D -height $ySize2D -win 1 -tag nmr$id

   set cx1 10
   set cy1 10
   set cxn [expr $xSize - 80]
   set cyn [expr $ySize - 70]

   set ix  [expr 0.5*($cx1 + $cxn)]
   set iy  [expr 0.5*($cy1 + $cyn)]

   defineCell -x $cx1 $cxn -y $cy1 $cyn -cell $id -verb -win $id 

   $w raise cursor

   draw2D

   if {!$firstTime} {moveVCursor $ix $iy}

   set firstTime 0

   bind $w <Configure> "resize2DCanvas $w $id %w %h"
}

proc draw2D {} \
{
   global hi

   newScreen

   drawRec -win 1 -x 0% 100% -y 0% 100% -fg gray30

   drawROI -roi 1 -cell 1 -hi $hi -pix \
           -bg black -color1 multi -color2 multi -axis Bottom Right
}

proc initVCursor {} \
{
   global cx1 cxn cy1 cyn qyn ixPrev specLocPPM

   set ixPrev [expr int(0.5*($cx1 + $cxn))]
   set ix     [spec2pix -cell 1 -roi 1 -x ${specLocPPM}ppm]

   .c create line $ix $cy1 $ix $cyn -fill yellow -tags cursor
   .c bind cursor <B1-Motion> "moveVCursor %x %y"

   select1DX 1 $ix
}

proc initHCursor {} \
{
   global cx1 cxn cy1 cyn iyPrev sliceLoc

   set iy [spec2pix -cell 1 -roi 1 -y $sliceLoc]

   set iyPrev(2) $[expr int(0.5*($cy1 + $cyn))]
   set iyPrev(3) $[expr int(0.5*($cy1 + $cyn))]

   draw1D 3
}

proc adjXLoc { i } \
{
   global specLoc specXSize

   updateSpecLoc $specLoc

   set specLoc [expr $specLoc + $i]

   read1DSpec 
   read1DDOSY 

   draw1D 2
   draw1D 3
}

proc adjYLoc { i } \
{
   global sliceLoc specYSize

   updateSliceLoc $sliceLoc

   set sliceLoc [expr $sliceLoc + $i]

   read1DSpec 

   draw1D 2
   draw1D 3
}

proc adjDX { i } \
{
   global dx ixPrev

   updateDX

   set dx [expr $dx + $i]

   if {$dx < 0} {set dx 0}

   read1DDOSY 

   draw1D 2
   draw1D 3
}

proc adjHi { f } \
{
   global hi

   set hi [expr $hi*$f]
}

proc estHi {} \
{
   global hi minVal maxVal noise 

   set roiInfo [getROIInfo -roi 1]

   set rdata   [lindex $roiInfo 0]
   set xSize   [lindex [lindex $roiInfo 2] 0]
   set ySize   [lindex [lindex $roiInfo 2] 1]

   set nn      [expr $xSize*$ySize]
   set n       [expr $xSize*$ySize/5]

   set minVal(1) [vMin $rdata $nn]
   set maxVal(1) [vMax $rdata $nn]

   if {![strcasecmp $noise Auto]} {set noise [vEstNoise $rdata $n]}
   if {![strcasecmp $hi    Auto]} {set hi    [expr 6.0*$noise]}

   puts stderr "Min: $minVal(1) Max: $maxVal(1) Noise: $noise Hi: $hi"
}

#
# Creation of Tk windows:

proc initWindows {} \
{
   global xSize2D ySize2D xSize1D ySize1D sliceLoc hi noise specLoc dx
   global axisfont

   toplevel .t
   toplevel .s

   set xSizeEv $xSize1D
   set ySizeEv [expr int(0.8*$xSizeEv)] 

   frame    .cmnd
   canvas   .c   -width $xSize2D -height $ySize2D -border 0 -closeenough 1024 
   canvas   .s.c -width $xSize1D -height $ySize1D -border 0

   graph    .t.c -width $xSizeEv -height $ySizeEv -border 0 \
                 -plotbackground black -title Evolution

   .t.c axis configure x -tickfont $axisfont -ticklength 0.05i -title Tau
   .t.c axis configure y -tickfont $axisfont -ticklength 0.05i -title Amplitude 

   .t.c marker create line -coords "-Inf 0.0 Inf 0.0" -name zero \
                           -outline gray -under yes

   button   .draw   -text Draw     -command draw2D 
   button   .export -text "Fit 1D" -command export1D
   button   .quit   -text Quit     -command exit
   label    .sLab   -text Noise:
   entry    .sig    -textvariable noise -width 12

   label    .hiLab  -text Height:
   entry    .hi     -textvariable hi -width 12
   button   .incr   -text "+" -padx 1 -pady 1 -command "adjHi 1.3"
   button   .decr   -text "-" -padx 1 -pady 1 -command "adjHi [expr 1/1.3]"
   bind     .hi     <Return> "+ draw2D"

   label    .dxLab  -text DX:
   entry    .dx     -textvariable dx -width 3
   button   .idx    -text "+" -padx 1 -pady 1 -command "adjDX  1"
   button   .ddx    -text "-" -padx 1 -pady 1 -command "adjDX -1"
   bind     .dx     <Return> "+ adjDX 0"

   label    .lxLab  -text XLoc:
   entry    .lx     -textvariable specLoc -width 6
   button   .ixloc  -text "+" -padx 1 -pady 1 -command "adjXLoc  1"
   button   .dxloc  -text "-" -padx 1 -pady 1 -command "adjXLoc -1"
   bind     .lx     <Return> "+ adjXLoc 0"

   label    .lyLab  -text YLoc:
   entry    .ly     -textvariable sliceLoc -width 4 
   button   .iyloc  -text "+" -padx 1 -pady 1 -command "adjYLoc  1"
   button   .dyloc  -text "-" -padx 1 -pady 1 -command "adjYLoc -1"
   bind     .ly     <Return> "+ adjYLoc 0"

   pack     .draw .export .quit -side left -in .cmnd
   pack     .dyloc .iyloc .ly .lyLab -in .cmnd -side right
   pack     .dxloc .ixloc .lx .lxLab -in .cmnd -side right
   pack     .ddx   .idx   .dx .dxLab -in .cmnd -side right
   pack     .decr  .incr  .hi .hiLab -in .cmnd -side right
   pack     .sig .sLab -in .cmnd -side right
 
   pack   .cmnd -side top -fill x -expand 0
   pack   .c -side top -fill both -expand 1

   button .t.full -text "1D Full" -command "full1D 2"
   pack   .t.c -in .t -side top -fill both -expand 1
   pack   .t.full -in .t -side top -fill x -expand 1

   button .s.full -text "1D Full" -command "full1D 3"
   pack   .s.c -in .s -side top -fill both -expand 1
   pack   .s.full -in .s -side top -fill x -expand 1

   wm title .  \
    "DOSY Viewer 2D   \[Left: Select 1D\]"

   wm title .t \
    "1D Evolution \[Middle: Scale\] \[Right: Offset\]"

   wm title .s \
    "1D Spectrum  \[Left: Select 1D\] \[Middle: Scale\] \[Right: Offset\]"

   update

   wm geom  . +10+20
   wm geom .t +40-60
   wm geom .s -40-60

#  raise .

   bind .t.c <B2-Motion> "scale1D  2 %y"
   bind .t.c <B3-Motion> "move1D   2 %y"
}

proc initResize {} \
{
   bind .c   <Configure> "resize2DCanvas .c 1 %w %h"
   bind .t.c <Configure> "resize1DCanvas .t 2 %w %h"
   bind .s.c <Configure> "resize1DCanvas .s 3 %w %h"
}

proc initArgs {} \
{
   global ARGC ARGV

   global xName yName scriptName dName noise hi fitID firstTime
   global specLoc specLocPPM sliceLoc zoomW inName inList colorList 
   global dx drawCount scale skip auxInfo
   global xSize1D ySize1D xSize2D ySize2D 
   global tauList tauMin tauMax

   set firstTime 1 

   foreach id {1 2 3} \
      {
       set drawCount($id) 0
      }

   set specLoc    None 
   set specLocPPM None
   set sliceLoc   None 
   set dx         None 

   set xSize1D  630
   set ySize1D  330

   set xSize2D  700
   set ySize2D  390

   set xName      X_AXIS 
   set yName      Y_AXIS 
   set inList     test.ft1
   set scriptName fitXY.tcl
   set dName      dosyFit.com

   set colorList  "yellow cyan red white"
   set auxInfo    "" 

   set zoomW   None
   set noise   Auto
   set hi      Auto 
   set scale   1.0
   set skip    0

   set tauList ""
   set tauMin  0.0
   set tauMax  0.0

   set fitID   0

   if {[flagLoc $ARGV -help]} \
      {
       puts "View and Fit 1D Extracts from 2D DOSY Spectrum:"
       puts " -in    inName  \[test.ft1\]    2D Input Name."
       puts " -dosy  dName   \[dosyFit.com\] Name of Temporary Fitting Script."
       puts " -fit   fitName \[fitXY.tcl\]   Name of 1D XY Fitting Application."
       puts " -hi    hi      \[Auto\]        2D Contour Height."
       puts " -noise sigma   \[Auto\]        Noise Estimate, Contour and Fit."
       puts " -xName xAxis   \[X_AXIS\]      Spectrum Spectral Axis Name."
       puts " -yName yAxis   \[Y_AXIS\]      Spectrum DOSY Axis Name."
       puts " -x     xLoc    \[50%\]         Initial Evolution Location,"
       puts "                                in Pts, Hz, PPM, %."
       puts " -y     yLoc    \[1\]           Initial Slice Number, in Pts."
       puts " -dx    dx      \[0\]           Strip Width, Pts, Hz, PPM, %."
       puts " -tau   tList   \[1 2 3 ...\]   Tau List."
       puts " -scale sVal    \[1.0\]         Scale Factor for Intensities."
       puts " -skip  nSkip   \[0\]           Number of Points to Skip in Fit."
       puts " -zoom  zoomW   \[None\]        +/- Spectral Width for Auto 1D"
       puts "                                Zoom, in Pts, Hz, PPM, %."
       puts " -aux   auxInfo \[None\]        Aux Data for XY Fitting Script;"
       puts "                                Must be last argument." 
       exit 1
      }

   getListArgD $ARGV -in    inList
   getListArgD $ARGV -tau   tauList
   getListArgD $ARGV -color colorList

   getArgD $ARGV -xName xName
   getArgD $ARGV -yName yName

   getArgD $ARGV -hi    hi 
   getArgD $ARGV -noise noise
   getArgD $ARGV -dx    dx
   getArgD $ARGV -skip  skip
   getArgD $ARGV -scale scale
   getArgD $ARGV -zoom  zoomW

   getArgD $ARGV -x     specLocPPM
   getArgD $ARGV -y     sliceLoc

   set inName [lindex $inList 0] 
   set loc    [flagLoc $ARGV -aux]

   if {$loc} {set auxInfo [lrange $ARGV [expr $loc + 1] end]}

   if {![file exists $inName]} \
      {
       puts stderr "Error Finding Input Spectrum $inName"
       exit 1
      }
}

initArgs
setFonts
initWindows
read2DDOSY
estHi

read1DDOSY
read1DSpec

resize1DCanvas .s 3 $xSize1D $ySize1D
resize2DCanvas .c 1 $xSize2D $ySize2D

initVCursor
initHCursor
initResize

