#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set auto_path [linsert $auto_path 0 $env(NMRBASE)/dynamo/tcl]

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set tabName     csObs.tab
set pdbName     None 
set turnName    $env(SURF_DIR)/turnProb.tab
set outName     turnEval.tab
set maxRank     None 
set graphFlag   0

set colorList {yellow cyan green darkOrchid red blue magenta gray white}

if {[flagLoc $ARGV -help]} \
   {
    puts "Secondary Structure and Turn Type Probabilities from Sequence;"
    puts "Use One of the Following to Specify Sequence:"
    puts " -in   tabName \[csObs.tab\]    Table with DATA SEQUENCE Lines."
    puts " -pdb  pdbName \[None\]         PDB File for Sequence."
    puts "Other Arguments:"
    puts " -out  outName \[turnEval.tab\] Turn Type Probability Output." 
    puts " -maxRank rank \[None\]         Maximum Rank to Report."
    puts " -graph                         Graph Mode ON."
    puts " -nograph                       Graph Mode OFF (Default)."
    puts " -color  cList                  Color List."
    exit 1
   }

getArgD $ARGV -in      tabName
getArgD $ARGV -tab     tabName
getArgD $ARGV -out     outName
getArgD $ARGV -pdb     pdbName
getArgD $ARGV -turn    turnName
getArgD $ARGV -maxRank rank

getListArgD $ARGV -color colorList

if {[flagLoc $ARGV -graph]}   {set graphFlag 1}
if {[flagLoc $ARGV -nograph]} {set graphFlag 0}

setFonts

#
#----------------------------------------------------------------

proc drawTurn {} \
{
   global turnList turnInfo colorList r1 rN

   foreach turn $turnList \
      {
       set str [string tolower $turn]

       if {$turnInfo($turn,show)} \
          {
           .g element configure $str -hide 0
          } \
       else \
          {
           .g element configure $str -hide 1
          }
      }

   update
}

proc graphTurnInfo {} \
{
   global turnList turnInfo colorList r1 rN maxProb graphFlag axisfont

   if {!$graphFlag} return

   set xwSize [winfo screenwidth .] 
   set ywSize [winfo screenheight .]

   set xSize [expr 4*$xwSize/5]
   set ySize [expr $ywSize/2]

   wm geom . +10+10

   set resCount [expr 1 + $rN - $r1]

   if {$resCount <= 30} \
      {
       set nS 1
      } \
   elseif {$resCount <= 250} \
      {
       set nS 10
      } \
   else \
      {
       set nS 20
      }

   frame .cmnd -width [expr 14*20] 

   set n [llength $colorList]
   set i 0

   set turnInfo(AlphaHelix,show) 1
   set turnInfo(BetaSheet,show)  1

   foreach turn $turnList \
      {
       set color [lindex $colorList [expr $i % $n]]
       set str   [string tolower $turn]

       checkbutton .$str \
        -text $turn -anchor w -width 14 -selectcolor $color \
        -onvalue 1 -offvalue 0 -command drawTurn -variable turnInfo($turn,show)

       pack .$str -in .cmnd -side top 

       incr i
      }

   button .quit -text Quit -command exit 

   pack .quit -in .cmnd -side bottom -expand 1 -fill x
   pack .cmnd -in . -side right -expand 1 -fill y

   update

   graph .g -plotbackground black -width $xSize -height $ySize \
    -title "Secondary Structure and Turn Type Probabilities"

   pack .g -in . -side left -expand 1 -fill both

   set resCount [expr 1 + $rN - $r1]

   .g legend configure -hide 1

   .g axis configure y -hide 1 -min 0 -max $maxProb
   .g axis configure x -step $nS -subdivisions 1
   .g axis configure x -tickfont $axisfont -ticklength 0.05i

   set i 0

   foreach turn $turnList \
      {
       set color [lindex $colorList [expr $i % $n]]
       set str   [string tolower $turn]

       .g element create $str \
          -xdata $turnInfo($turn,x) -ydata $turnInfo($turn,y) \
          -color $color -pixels 0 -linewidth 2

       if {!$turnInfo($turn,show)} {.g element configure $str -hide 1}

       incr i
      }

   update
}

proc sortTurnScore { s1 s2 } \
{
   set v1 [lindex $s1 1]
   set v2 [lindex $s2 1]

   if {$v1 > $v2} {return -1}
   if {$v1 < $v2} {return  1}

   return 0
}

proc getTurnScore { seq turn n } \
{
   global p np

   set pVal 1.0
   set nVal 1 

   for {set i 0} {$i < $n} {incr i} \
      {
       set resName [string index $seq $i]

       set pVal [expr $pVal*$p($resName,$turn,$i)]
       set nVal [expr $nVal*$np($resName,$turn,$i)]
      }

   set pVal [expr pow( $pVal, 1.0/$n )]
   set nVal [expr pow( $nVal, 1.0/$n )]

   return "$turn $pVal $nVal" 
}

#
#------------------------------------------------------------------

initResNames

set maxSegLength 5
set maxProb      0.0

set outUnit stdout
set inTab   0
set pdbTab  0
set r1      0
set rN      0

#
# Get Sequence:

if {![strcasecmp $pdbName None]} \
   {
    if {![file exists $tabName]} \
       {
        puts "Error Finding Input Table $tabName"
        exit 1
       }

    set inTab   [gdbRead table -in $tabName -verb]
    set seqList [string toupper [getSeqList $inTab r1 rN]]
   } \
else \
   {
    if {![file exists $pdbName]} \
       {
        puts "Error Finding PDB Input $pdbName"
        exit 1
       }

    set pdbTab  [gdbRead table -in $pdbName -pdb -verb]
    set seqList [string toupper [getPDBSeqList $pdbTab r1 rN]]
   }

#
# Get turn type data:

if {![file exists $turnName]} \
   {
    puts "Error Finding Turn Probability Table $turnName"
    exit 1
   }

set tTab  [gdbRead table -in $turnName -verb]
set eList [gdbGetInfo $tTab -parms entries]

set turnList ""
set resList  ""

foreach entry $eList \
   {
    set turn    [gdbGet $entry TURN_CLASS]
    set resName [gdbGet $entry RESNAME]
    set n       [gdbGet $entry TURN_SIZE]

    if {[lsearch -exact $turnList $turn] < 0} \
       {
        lappend turnList $turn
        set turnSize($turn) $n
       }

    if {[lsearch -exact $resList $resName] < 0} \
       {
        lappend resList $resName
       }
   }

#
# Establish default (null) probabilities:

foreach turn $turnList \
   {
    set turnInfo($turn,x)    ""
    set turnInfo($turn,y)    ""
    set turnInfo($turn,show) 0 

    foreach resName $resList \
       {
        for {set i 0} {$i < $maxSegLength} {incr i} \
           {
            set p($resName,$turn,$i)  0.0
            set np($resName,$turn,$i) 0.0 
           }
       }

    foreach resName $seqList \
       {
        for {set i 0} {$i < $maxSegLength} {incr i} \
           {
            set p($resName,$turn,$i)  0.0
            set np($resName,$turn,$i) 0.0 
           }
       }
   }

#
# Extract actual probability data:

foreach entry $eList \
   {
    set turn    [gdbGet $entry TURN_CLASS]
    set resName [gdbGet $entry RESNAME]

    for {set i 0} {$i < $maxSegLength} {incr i} \
       {
        set p($resName,$turn,$i)  [expr 0.01*[gdbGet $entry P_I$i]]
        set np($resName,$turn,$i) [expr 0.01*[gdbGet $entry N_I$i]]
       }
   }

#
# Report results:

if {[strcasecmp $outName None]} \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error Opening Output file $outName"
        return 1
       }
   }

if {![strcasecmp $maxRank None]} {set maxRank [llength $turnList]}

puts $outUnit "VARS   RESID RESNAME TURN_CLASS RANK SCORE  SCORE_N"
puts $outUnit "FORMAT %3d   %s      %-14s      %2d  %e     %e"

for {set res $r1} {$res <= $rN} {incr res} \
   {
    set scoreList ""

    foreach turn $turnList \
       {
        set a1 $res
        set aN [expr $a1 + $turnSize($turn) - 1]   

        if {$aN > $rN} continue

        set seq ""

        for {set a0 $a1} {$a0 <= $aN} {incr a0} \
           {
            append seq [lindex $seqList [expr $a0 - $r1]] 
           }

        set thisScore [getTurnScore $seq $turn $turnSize($turn)]
        set scoreP    [lindex $thisScore 1]

        if {$scoreP > $maxProb} {set maxProb $scoreP}

        lappend scoreList $thisScore

        lappend turnInfo($turn,x) $res
        lappend turnInfo($turn,y) [lindex $thisScore 1] 
       }

    set scoreInfo($res) [lsort -command sortTurnScore $scoreList]
   }

for {set res $r1} {$res <= $rN} {incr res} \
   {
    set resName [lindex $seqList [expr $res - $r1]]
    set rank 1

    foreach score [lrange $scoreInfo($res) 0 [expr $maxRank - 1]] \
       {
        set turn [lindex $score 0]
        set f    [lindex $score 1]
        set n    [lindex $score 2]

        set fmt " %3d %s %-12s %2d %e %e"

        puts $outUnit [format $fmt $res $resName $turn $rank $f $n]

        incr rank
       }
   }

graphTurnInfo

if {!$graphFlag} exit
