#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Rough estimate of accessable surface of a protein."
    puts " -in     inList   \[ref.pdb\] Input PDB File List."
    puts " -r      rad      \[1.45\]    Radius of Surface Probe, A."
    puts " -tol    probeTol \[0.10\]    Tolerance for Overlap Test, A."
    puts " -step   stepSize \[0.10\]    Step Size for Probe, A."
    puts " -verb                      Verbose Mode ON (Default)."
    puts " -noverb                    Verbose Mode OFF."
    puts "Output Options (Use Only One):"
    puts " -out    outList  \[Auto\]    Output PDB File List (or Keyword None)."
    puts " -outDir outDir   \[None\]    Output Directory."
    puts "Notes:"
    puts " 1. Creates new PDB files where TEMPFACTOR column"
    puts "    is non-zero for atoms at the surface."
    puts " 2. When -out Auto is used, the output name will have a surf_ prefix."
    puts " 3. When -outDir is used, the output name will match the input name."
    exit 1
   }

set inList    ref.pdb
set outList   Auto 
set outDir    None
set rad       1.45
set tol       0.10
set step      0.10
set briefFlag 0
set verbose   2

getListArgD $ARGV -in  inList
getListArgD $ARGV -out outList

getArgD     $ARGV -outDir outDir

getArgD     $ARGV -r    rad 
getArgD     $ARGV -tol  tol 
getArgD     $ARGV -step step

set loc [flagLoc $ARGV -verb]

if {$loc} \
   {
    set nextFlag [nextFlag $ARGV $loc]
    set verbose  1

    if {$nextFlag == $loc + 2} \
       {
        getArgD $ARGV -verb verbose
       }
   }

if {[flagLoc $ARGV -noverb]}   {set verbose 0}

if {[flagLoc $ARGV -brief]}    {set briefFlag 1}
if {[flagLoc $ARGV -nobrief]}  {set briefFlag 0}

#
#----------------------------------------------------------------------

set   nameCount [llength $inList]
set n 0

for {set i 0} {$i < $nameCount} {incr i} \
   {
    set inName [lindex $inList $i]

    if {[file exists $inName]} {incr n} 
   }

if {!$n} {exit 0}

if {[strcasecmp $outList None] && [strcasecmp $outList Auto]} \
   {
    if {$nameCount != [llength $outList]} \
       {
        puts stderr "Error: mismatched name count for inList/outList"
        exit 1
       }
   }

if {[strcasecmp $outDir None]} \
   {
    if {![file exists $outDir]} \
       {
        catch "file mkdir $outDir"
       }
   }

dynInit -fast -uniqAtom -nowarn

for {set i 0} {$i < $nameCount} {incr i} \
   {
    set inName [lindex $inList $i]

    if {![file exists $inName]} \
       {
        puts stderr "Skipping file $inName"
       }

    set rootName [file tail $inName]

    if {[strcasecmp $outDir None]} \
       {
        set outName $outDir/$rootName
       } \
    else \
       {
        if {![strcasecmp $outList None]} \
           {
            set outName None
           } \
        elseif {![strcasecmp $outList Auto]} \
           {
            set outName surf_$inName
           } \
        else \
           {
            set outName [lindex $outList $i]
           }
       }

    set src    [dynRead -pdb -in $inName -hetatm]
    set pInfo  [dynGetInfo -src $src]
    set nAtoms [lindex $pInfo 2]

    if {$verbose} \
       {
        set sInfo [dynSurface -src $src -verb $verbose -r $rad -tol $tol -step $step]
       } \
    else \
       {
        set sInfo [dynSurface -src $src -r $rad -tol $tol -step $step]
       }

    set mass [lindex $sInfo 0]
    set area [lindex $sInfo 1]
    set vol  [lindex $sInfo 2]
    set nEx  [lindex $sInfo 3]

    if {$nAtoms > 0} \
       {
        set pct [expr 100.0*$nEx/$nAtoms]
       } \
    else \
       {
        set pct 0.0
       }

    if {$briefFlag} \
       {
        puts "$sInfo $nAtoms $nEx"
       } \
    else \
       {
        if {$nameCount > 1} \
           {
            set fmt "ATOMS: %5d EXPOSED: %5.1f%% MASS: %5d AREA: %6.1f A^2  VOLUME: %6.1f A^3 NAME: %s"
            set str [format $fmt $nAtoms $pct $nEx $mass $area $vol $rootName]
           } \
        else \
           {
            set fmt "ATOMS: %d EXPOSED: %.1f%% MASS: %d AREA: %.1f A^2 VOLUME: %.1f A^3"
            set str [format $fmt $nAtoms $pct $mass $area $vol]
           }

        if {$verbose} {puts ""}
        puts $str
       }

    set fmt "ATOMS: %d EXPOSED: %.1f%% MASS: %d AREA: %.1f A^2 VOLUME: %.1f A^3"
    set str [format $fmt $nAtoms $pct $mass $area $vol]

    if {![strcasecmp $outName None]} continue

    set fmt " Mass: %d Area: %.1f A^2 Volume: %.1f A^3"
    set str [format $fmt $mass $area $vol]
    set rem "Non-Zero Temperature Factor shows solvent-accessible atoms.\nREMARK DATA $str"

    dynWrite -src $src -pdb -out $outName -rem $rem -hetatm
    dynFree  -src $src
   }

exit 0
