#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Fit Data from Columns of a Table:"
    puts " -in  inName \[test.tab\]    Table Input."
    puts "Table Variable Expressions:"
    puts " -x   xExpr  \[1.0\]         Expression for X-Values."
    puts " -y   yExpr  \[1.0\]         Expression for Y-Values."
    puts " -n   nExpr  \[1.0\]         Expression for Noise Values."
    puts " -f   fExpr  \[1.0\]         Expression for Model Function." 
    puts "Model Parameter Values:"
    puts " -p   pList  \[1.0\]         Initial Parameter Values."
    puts " -pHi hList  \[+1e+16 ...\]  Upper Bound for Parameter Values."
    puts " -pLo lList  \[-1e+16 ...\]  Lower Bound for Parameter Values."
    puts " -v   vNames \[a b c ...\]   Corresponding Parameter Variable Names."
    puts "PDB Options:"
    puts " -pdb                        Input is PDB Data (Default for .pdb)."
    puts " -nopdb                      Input is NMRPipe-Format Table."
    puts " -hetatm                     Interpret HETATMs as ATOMs."
    puts "Other Arguments:"
    puts " -tol tol    \[1.0e-6\]      Convergence Tolerance."
    puts " -gc  gCount \[1\]           Monte Carlo Trial Count."
    puts " -res                      Model Returns Residual."
    puts " -verb                     Verbose Mode ON."
    exit 1
   }

set inName  test.tab

set pNames  "a b c d e f g h i j k l m n o p q r s t u v w x y z" 
set pInit   1.0 
set pLo     ""
set pHi     ""

set xExpr   1.0 
set yExpr   1.0 
set nExpr   1.0 
set fExpr   1.0
set pdbMode nopdb
set hetFlag nohetatm 

set tol     1.0e-6
set gc      1
set resFlag 0
set verbose 0

getArgD $ARGV -in inName
getArgD $ARGV -gc gc

getListArgD $ARGV -x xExpr
getListArgD $ARGV -y yExpr
getListArgD $ARGV -n nExpr
getListArgD $ARGV -f fExpr

getListArgD $ARGV -p   pInit
getListArgD $ARGV -pLo pLo
getListArgD $ARGV -pHi pHi
getListArgD $ARGV -v   pNames


if {![strcasecmp [file extension $inName] .pdb]} {set pdbFlag 1}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -pdb]}    {set pdbMode pdb}
if {[flagLoc $ARGV -nopdb]}  {set pdbMode nopdb}

if {[flagLoc $ARGV -res]}    {set resFlag 1}
if {[flagLoc $ARGV -nores]}  {set resFlag 0}

if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}

#
#-----------------------------------------------------------------

proc modelProc {} \
{
   global pNames vNames entries fExpr verbose
   global xList pList qList yModel xyCount

   foreach v $vNames {global $v}

   set i 0 

   foreach pVal $pList \
      {
       set p  [lindex $pNames $i]
       set $p $pVal
       incr i
      }

   set i 0

   set qExpr [string trim $fExpr "{}"]
   set sum   0.0

   foreach entry $entries \
      {
       set x [lindex $xList $i]
       set y [lindex $qList $i]
       set v 0.0

       extractEntryVals $entry 

       set error [catch {set v [eval "expr $qExpr"]}]

       if {$error} \
          {
           puts stderr "Error Evaluating Model Expression:"
           puts stderr " $fExpr"
           exit
          }

       lappend yModel $v

       set d   [expr $y - $v]
       set sum [expr $sum + $d*$d]

       incr i
      }

   if {$verbose} {puts "$pList [expr sqrt( $sum/$xyCount )]"}
}

proc getExprList { qExpr } \
{
   global entries vNames

   foreach v $vNames {global $v}
  
   set qList ""

   foreach entry $entries \
      {
       set v 0.0

       extractEntryVals $entry 

       set error [catch {set v [eval "expr $qExpr"]}]

       if {$error} \
          {
           puts stderr "Error Evaluating Expression:"
           puts stderr " $qExpr"
           exit 1
          }

       lappend qList $v
      }

   return $qList
}

proc extractEntryVals { entry } \
{
   global vNames

   foreach v $vNames \
      {
       global $v
       set $v [gdbGet $entry $v]
      }
}

set entries ""
set vNames  ""

set t 0

if {[llength $pInit] > [llength $pNames]} \
   {
    puts stderr "Mismatched Initial Value/Parameter Name Lists."
    exit 1
   }

if {![llength $pHi]} \
   {
    foreach p $pInit {lappend pHi  1.0e+16}
   }

if {![llength $pLo]} \
   {
    foreach p $pInit {lappend pLo -1.0e+16}
   }

if {[llength $pInit] != [llength $pHi] || [llength $pInit] != [llength $pLo]} \
   {
    puts stderr "Mismatched Boundry Value/Parameter Name Lists."
    exit 1
   }

if {![file exists $inName]} \
   {
    puts stderr "Error finding input table $inName"
    exit 1
   }

set t [gdbRead table -in $inName -$pdbMode -$hetMode]

set entries [gdbGetInfo $t -parms entries]
set vNames  [gdbGetInfo $t -parms varNames]

if {$verbose}  {set verbArg "-verb"}
if {!$verbose} {set verbArg "-noverb"}

if {$resFlag}  {set resArg "-res"}
if {!$resFlag} {set resArg "-nores"}

set xValList [getExprList $xExpr]
set yValList [getExprList $yExpr] 
set nValList [getExprList $nExpr]

set result \
 [modelXY -p $pInit -pHi $pHi -pLo $pLo \
          -x $xValList -y $yValList -noise $nValList \
          -gc $gc -tol $tol -macro modelProc $verbArg]

puts $result

exit
