#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

#
# fitXY.tcl: fit a given set of XY pairs.
#---------------------------------------------------------------------

proc modelProc {} \
{
   global xList pList yModel

   set yModel [evalProc $xList $pList]
}

proc evalProc { xList pList } \
{
   global f amp alpha beta gamma phi iSign iPow C mode varInfo

   set yModel ""

   set PI [expr 2.0*acos(0.0)]
   set i  0

   foreach v $varInfo \
      {
       if {[lsearch -exact $f $v] >= 0} continue
       set $v [lindex $pList $i]
       incr i
      }

   switch $mode \
      {
       quad {
             foreach x $xList \
                {
                 set y [expr $alpha*$x*$x + $beta*$x]
                 lappend yModel [expr $y + $C]
                }
            }

       exp {
            foreach x $xList \
               {
                set y [expr $amp*exp( $iSign*abs($alpha)*$x )]
                lappend yModel [expr $y + $C]
               }
           }

       cexp {
             foreach x $xList \
                {
                 set y [expr $amp*($C - exp( $iSign*abs($alpha)*$x ))]
                 lappend yModel $y
                }
            }

       gauss {
              foreach x $xList \
                 {
                  set y [expr $amp*exp( $iSign*abs($alpha)*pow($x,$iPow) )]
                  lappend yModel [expr $y + $C]
                 }
             }

       cgauss {
               foreach x $xList \
                  {
                   set y [expr $amp*($C - exp( $iSign*abs($alpha)*pow($x,$iPow) ))]
                   lappend yModel $y
                  }
              }

       egauss {
               foreach x $xList \
                  {
                   set y [expr $amp*exp( $iSign*abs($alpha)*$x )*exp( $iSign*abs($beta)*pow($x,$iPow) )]
                   lappend yModel [expr $y + $C]
                  }
              }

       sin {
            foreach x $xList \
               {
                set y [expr $amp*sin( $PI*($alpha*$x + $phi) )]
                lappend yModel [expr $y + $C]
               }
           }

       sinp {
             foreach x $xList \
                {
                 set y [expr $amp*pow(sin( $PI*($alpha*$x + $phi) ), $iPow )]
                 lappend yModel [expr $y + $C]
                }
           }

       dsin {
             foreach x $xList \
                {
                 set y [expr $amp*exp($iSign*abs($alpha)*$x)*sin($PI*($beta*$x+$phi))]
                 lappend yModel [expr $y + $C]
                }
            }

      gsin {
             foreach x $xList \
                {
                 set y [expr $amp*exp($iSign*pow($alpha*$x,$iPow))*sin($PI*($beta*$x+$phi))]
                 lappend yModel [expr $y + $C]
                }
            }

       karplus {
                foreach x $xList \
                   {
                    set y \
                     [expr $alpha*cos($PI*($x + $phi))*cos($PI*($x + $phi)) + \
                           $beta*cos($PI*($x + $phi)) + $gamma]
   
                    lappend yModel [expr $y + $C]
                   }
               }
      }

   return $yModel
}

proc exitFitXY {} \
{
   exit
}

proc getFitXYInfo {} \
{
   global modelInfo fitInfo varInfo initInfo title
   global amp alpha beta gamma phi iPow
   global pList eList f mode 

   set i 0

   set fitInfo $mode

   foreach v $varInfo \
      {
       if {[lsearch -exact $f $v] >= 0} \
          {
           if {![strcmp $v iPow] && [string first iPow $title] < 0} continue

           set modelInfo($v) [format "$v: %s Fixed" $initInfo($v)]
           continue
          }

       set val [lindex $pList $i]
       set dev [lindex $eList $i]

       if {![strcasecmp $v amp]} \
          {
           if {$val == 0.0} \
              {
               set pct 0.0
              } \
           else \
              {
               set pct [expr 100.0*$dev/$val]
              }

           set modelInfo($v) [format "$v: %.5e +/- %.2f%%" $val $pct]
          } \
       else \
          {
           if {abs($val) < 0.001 || abs($dev) < 0.0001} \
              {
               set modelInfo($v) [format "$v: %.5e +/- %.5e" $val $dev]
              } \
           else \
              {
               set modelInfo($v) [format "$v: %.5f +/- %.5f" $val $dev]
              }
          }

       append fitInfo " $v $val $dev"

       incr i
      }
}

proc saveFitXY {} \
{
   global x y s yModel xyCount modelInfo fitInfo title srcName
   global xLab yLab srcName outName graphFlag saveFlag fitID varList

   if {![strcasecmp $outName None]} return

   if {$graphFlag && !$saveFlag} \
      {
       if {![okWrite $outName]} return
      }

   set dirName [file dirname $outName]

   if {![file isdir $dirName]} {catch "file mkdir $dirName"}

   set error [catch {set outUnit [open $outName w]}]

   if {$error} \
      {
       puts stderr "Error Opening Output File $outName"
       exit 1
      }

   puts $outUnit "#"
   puts $outUnit "# $title"

   foreach v $varList \
      {
       if {[info exists modelInfo($v)]} {puts $outUnit "#  $modelInfo($v)"}
      }

   puts $outUnit ""
   puts $outUnit "DATA MODEL $fitInfo"
   puts $outUnit "DATA TITLE $srcName"
   puts $outUnit "DATA X_LAB $xLab"
   puts $outUnit "DATA Y_LAB $yLab"
   puts $outUnit "DATA ID    $fitID"
   puts $outUnit ""

   puts $outUnit "VARS   INDEX X    Y    Y_MODEL NOISE"
   puts $outUnit "FORMAT %4d   %.5e %.5e %.5e    %.5e"
   puts $outUnit ""

   for {set i 0} {$i < $xyCount} {incr i} \
      {
       set thisX [lindex $x $i]
       set thisY [lindex $y $i]
       set thisM [lindex $yModel $i]
       set thisS [lindex $s $i]

       set fmt " %4d % .5e % .5e % .5e % .5e"

       puts $outUnit [format $fmt [expr $i + 1] $thisX $thisY $thisM $thisS]
      }

   close $outUnit

   if {$graphFlag} {wm title . "Saved in $outName"}

   update
}

proc radConvert { vPtr } \
{
   upvar $vPtr v

   set loc [string first deg [string tolower $v]]
   if {$loc < 0} return

   set PI [expr 2.0*acos(0.0)]
   set vv [string range $v 0 [expr $loc - 1]]
   set v  [expr $PI*$vv/180.0]
}
 
proc initFitXY {} \
{
   global x y s p f fitID mode outName srcName title xLab yLab
   global varList varInfo valInfo initInfo smoothMode orient
   global amp alpha beta gamma phi iSign iPow iseed gc cf C

   global wishFlag graphFlag saveFlag silentFlag exitFlag verbose
   global ARGC ARGV
  
if {[flagLoc $ARGV -help]} \
   {
    puts "Fit XY Data Pairs to a Simple Model."
    puts "Input Data:"
    puts " -x     xList   List of X Points (Optional)."
    puts " -y     yList   List of Corresponding Y Points."
    puts " -noise nList   List of Corresponding Noise Values (Or One Value)."
    puts "Output:"
    puts " -out     outName \[txt/fit%03d.txt\]  Output Name for Save Command."
    puts " -id      fitID   \[0\]                ID Number for Fit."
    puts " -xLab    xLab    \[X\]                X-Axis Label of Data."
    puts " -yLab    yLab    \[Y\]                Y-Axis Label of Data."
    puts " -title   tStr    \[Data\]             Title of Data."
    puts " -graph                         Graphics Display ON (Default)."
    puts " -nograph                       Graphics Display OFF."
    puts " -verb                          Verbose Mode ON (Default)."
    puts " -noverb                        Verbose Mode OFF."
    puts " -save                          Save Fit Results Automatically."
    puts " -exit                          Exit After Saving Fit Results."
    puts " -nosave                        Save Fit Results Manually (Default)."
    puts " -silent                        Silent Mode ON."
    puts "Model Variables:"
    puts " -amp   amp     \[Auto\]   Initial Amplitude Value."
    puts " -alpha alpha   \[Auto\]   Initial Alpha Value."    
    puts " -beta  beta    \[Auto\]   Initial Beta Value."
    puts " -gamma gamma   \[Auto\]   Initial Gamma Value."
    puts " -phi   phi     \[0.0\]    Phi Value."
    puts " -pow   iPow    \[2\]      Exponential Value."
    puts " -C     C       \[0.0\]    Constant Added to Model (Except Karplus)."
    puts " -sign  sign    \[-1.0\]   Sign Value."
    puts "Variables Whose Value is Fixed:"
    puts " -fixed fList   \[phi iPow C\]"
    puts "Available Models, Variables amp, alpha, beta, gamma, phi, iPow, C:"
    puts " -exp       Exponential Decay:"
    puts "             amp*exp( sign*|alpha|*x )"
    puts " -cexp      Complement of Exponential Decay (C = 1.0 by default):"
    puts "             amp*(C - exp( sign*|alpha|*x ))"
    puts " -gauss     Gaussian (iPow = 2 by Default):"
    puts "             amp*exp( sign*|alpha|*(x^iPow) )"
    puts " -cgauss    Complement of Gaussian (iPow = 2, C = 1.0 by default):"
    puts "             amp*(C - exp( sign*|alpha|*(x^iPow) ))"
    puts " -egauss    Exponential Times a Gaussian:"
    puts "             amp*exp( sign*|alpha|*x )*exp( sign*|beta|*x^iPow )"
    puts " -sin       Sinusoid:"
    puts "             amp*sin( PI*(alpha*x + phi) )"
    puts " -sinp      Sinusoid to a Power:"
    puts "             amp*sin^iPow( PI*(alpha*x + phi) )"
    puts " -dsin      Exponentially Damped Sinusoid:"
    puts "             amp*exp( sign*|alpha|*x )*sin( PI*(beta*x + phi) )"
    puts " -gsin      Gauss Damped Sinusoid (iPow = 2 by Default):"
    puts "             amp*exp( sign*(alpha*x)^iPow )*sin( PI*(beta*x + phi) )"
    puts " -quad      Quadratic:"
    puts "             alpha*x^2 + beta*x + C"
    puts " -karplus   Karplus Expression, t = x + phi:"
    puts "             alpha*cos^2( PI*t ) + beta*cos( PI*t ) + gamma"
    puts "XY-Plot Drawing:"
    puts " -smooth                  Smooth-Interpolated Model Curve (Default)." 
    puts " -linear                  Linear-Interpolated  Curve." 
    puts " -anchor  aName   \[NW\]    Window Loction: NW NE SE SW." 
    puts "Monte Carlo Error Analysis:"
    puts " -iseed   seed    \[54321\] Random Number Seed (Zero for Auto)."
    puts " -gc      nTrials \[20\]    Total Number of Trials."
    puts " -cf      confid  \[0.9\]   Confidence Interval."
    exit 1
   }

   if {$wishFlag} \
      {
       set graphFlag  1
       setFonts
      } \
   else \
      {
       set graphFlag  0
      }

   set varList "amp alpha beta gamma phi iPow C"

   set PI [expr 2.0*acos(0.0)]
 
   set x     ""
   set y     ""
   set s     ""
   set p     ""
   set gc    20
   set cf    0.90
   set iseed 54321

   set f     "phi iPow C"
   set amp   Auto 
   set alpha Auto
   set beta  Auto
   set gamma Auto

   set phi    0.0
   set C      0.0
   set iSign -1.0
   set iPow   2

   set xLab   X
   set yLab   Y
   set orient NW

   set fitID 0
   set mode  exp
  
   set outTemplate txt/fit%03d.txt
   set smoothMode  natural
   set srcName     Data 
   set saveFlag    0
   set exitFlag    0
   set silentFlag  0
   set verbose     1

   getArgD $ARGV -out outTemplate
   getArgD $ARGV -id  fitID

   set outName [format $outTemplate $fitID]

   if {[flagLoc $ARGV -cexp]} \
      {
       set mode cexp
       set C    1.0
      }

   if {[flagLoc $ARGV -cgauss]}  \
      {
       set mode cgauss
       set C    1.0
      }

   if {[flagLoc $ARGV -exp]}     {set mode exp}
   if {[flagLoc $ARGV -gauss]}   {set mode gauss}
   if {[flagLoc $ARGV -egauss]}  {set mode egauss}
   if {[flagLoc $ARGV -sin]}     {set mode sin}
   if {[flagLoc $ARGV -sinp]}    {set mode sinp}
   if {[flagLoc $ARGV -dsin]}    {set mode dsin}
   if {[flagLoc $ARGV -gsin]}    {set mode gsin}
   if {[flagLoc $ARGV -quad]}    {set mode quad}
   if {[flagLoc $ARGV -karplus]} {set mode karplus}
   
   if {[flagLoc $ARGV -verb]}     {set verbose 1}
   if {[flagLoc $ARGV -noverb]}   {set verbose 0}

   if {[flagLoc $ARGV -silent]}   {set silentFlag 1}
   if {[flagLoc $ARGV -nosilent]} {set silentFlag 0}

   if {[flagLoc $ARGV -exit]}     {set exitFlag 1}
   if {[flagLoc $ARGV -noexit]}   {set exitFlag 0}

   if {[flagLoc $ARGV -save]}     {set saveFlag 1}
   if {[flagLoc $ARGV -nosave]}   {set saveFlag 0}

   if {[flagLoc $ARGV -smooth]}  {set smoothMode natural}
   if {[flagLoc $ARGV -linear]}  {set smoothMode linear}

   if {[flagLoc $ARGV -graph]}   {set graphFlag 1}
   if {[flagLoc $ARGV -nograph]} {set graphFlag 0}

   if {[flagLoc $ARGV -notk]}    {set graphFlag 0}

   getListArgD $ARGV -x     x
   getListArgD $ARGV -y     y
   getListArgD $ARGV -fixed f
   getListArgD $ARGV -noise s
   getListArgD $ARGV -title srcName 
   getListArgD $ARGV -xLab  xLab
   getListArgD $ARGV -yLab  yLab

   getArgD $ARGV -amp    amp
   getArgD $ARGV -alpha  alpha
   getArgD $ARGV -beta   beta
   getArgD $ARGV -gamma  gamma
   getArgD $ARGV -phi    phi
   getArgD $ARGV -sign   iSign
   getArgD $ARGV -pow    iPow 
   getArgD $ARGV -C      C
   getArgD $ARGV -anchor orient
   getArgD $ARGV -iseed  iseed
   getArgD $ARGV -gc     gc
   getArgD $ARGV -cf     cf

   if {![llength $s]} {lappend s 1.0}

   if {![llength $x]} \
      {
       for {set i 0} {$i < [llength $y]} {incr i} {lappend x $i}
      }

   while {[llength $s] < [llength $y]} {lappend s [lindex $s 0]}

   if {![llength $x] || ![llength $y] || [llength $x] != [llength $y]} \
      {
       puts stderr "Error: Bad or Mismatched -x or -y Lists."
       exit 1
      }
 
   if {![strcasecmp $amp Auto]} \
      {
       if {![strcasecmp $mode cexp] || ![strcasecmp $mode cgauss]} \
          {
           if {$iSign < 0.0} \
              {
               set amp [getSMaxAbs $y]
              } \
           else \
              {
               set amp [getSMaxAbs $y]
              }
          } \
       else \
          {
           set amp [expr [getSMaxAbs $y] - $C]
          }
      }

   if {![strcasecmp $C Auto] && [strcasecmp $mode quad]} {set C 0.0}

   set amp  [expr double($amp)]
   set xMin [getMin $x]
   set xMax [getMax $x]

   set xRange [expr $xMax - $xMin]
   set xAvg   [getAvg $x]

   if {$xRange == 0.0} {set xRange 1.0}

   set n    [llength $x]
   set qq   ""
   set q    ""
   set r    ""
   set freq 0

   switch $mode \
      {
       exp {  
            set varInfo {amp alpha C}

            if {![strcasecmp $alpha Auto]} \
               {
                for {set i 0} {$i < $n} {incr i} \
                   {
                    set xVal [lindex $x $i]
                    set yVal [expr [lindex $y $i] - $C]

                    if {$xVal == 0.0 || $yVal == 0.0 || $amp == 0.0} continue

                    set yVal [expr double($yVal)/$amp]

                    catch {lappend q [expr abs(log(abs($yVal))/double($xVal))]}
                   } 

                set alpha [getMedian $q]
               }

            set title "amp*exp( $iSign*|alpha|*x ) + C"
           }

      egauss {
              set varInfo {amp alpha beta C}

              for {set i 0} {$i < $n} {incr i} \
                 {
                  set xVal [lindex $x $i]
                  set yVal [expr [lindex $y $i] - $C]

                  if {$xVal == 0.0 || $yVal == 0.0 || $amp == 0.0} continue

                  set yVal [expr double($yVal)/$amp]

                  catch {lappend q  [expr abs(log(abs($yVal))/double($xVal))]}
                  catch {lappend qq [expr abs(log(abs($yVal))/pow($xVal,$iPow)]}
                 }

              if {![strcasecmp $alpha Auto]} {set alpha [getMedian $q]}
              if {![strcasecmp $beta  Auto]} {set beta  [getMedian $qq]}

              if {[lsearch -exact $f iPow] >= 0} \
                 {
                  set title "amp*exp( $iSign*|alpha|*x )*exp( $iSign*|beta|*(x^$iPow) ) + C"
                 } \
              else \
                 {
                  set title "amp*exp( $iSign*|alpha|*x )*exp( $iSign*|beta|*(x^iPow) ) + C"
                 }
             }

       cexp {
            set varInfo {amp alpha C}

            if {![strcasecmp $alpha Auto]} \
               {
                for {set i 0} {$i < $n} {incr i} \
                   {
                    set xVal [lindex $x $i]
                    set yVal [expr 1.0 - [lindex $y $i] - $C]

                    if {$xVal == 0.0 || $yVal == 0.0 || $amp == 0.0} continue

                    set yVal [expr double($yVal)/$amp]

                    catch {lappend q [expr abs(log(abs($yVal))/double($xVal))]}
                   }

                set alpha [getMedian $q]
               }

            set title "amp*(C - exp( $iSign*|alpha|*x ))"
           }

       gauss {
              set varInfo {amp alpha iPow C}

               if {![strcasecmp $alpha Auto]} \
                 {
                  for {set i 0} {$i < $n} {incr i} \
                     {
                      set xVal [lindex $x $i]
                      set yVal [expr [lindex $y $i] - $C]

                      if {$xVal == 0.0 || $yVal == 0.0 || $amp == 0.0} continue

                      set yVal [expr $yVal/$amp]

                      catch {lappend q [expr abs(log(abs($yVal))/pow($xVal,$iPow)]}
                     }

                  set alpha [getMedian $q]
                 }

              if {[lsearch -exact $f iPow] >= 0} \
                 {
                  set title "amp*exp( $iSign*|alpha|*(x^$iPow) ) + C"
                 } \
              else \
                 {
                  set title "amp*exp( $iSign*|alpha|*(x^iPow) ) + C"
                 }
             }

       cgauss {
               set varInfo {amp alpha iPow C}

                if {![strcasecmp $alpha Auto]} \
                  {
                   for {set i 0} {$i < $n} {incr i} \
                      {
                       set xVal [lindex $x $i]
                       set yVal [expr 1.0 - [lindex $y $i] - $C]

                       if {$xVal == 0.0 || $yVal == 0.0 || $amp == 0.0} continue

                       set yVal [expr $yVal/$amp]

                       catch {lappend q \
                        [expr abs(log(abs($yVal))/pow($xVal,$iPow)]}
                      }

                   set alpha [getMedian $q]
                  }

              if {[lsearch -exact $f iPow] >= 0} \
                 {
                  set title "amp*(1 - exp( $iSign*|alpha|*(x^$iPow) )) + C"
                 } \
              else \
                 {
                  set title "amp*(1 - exp( $iSign*|alpha|*(x^iPow) )) + C"
                 }
              }

       sin   {
              set varInfo {amp alpha phi C} 

              if {![strcasecmp $alpha Auto]} \
                 {
                  for {set i 0} {$i < $n - 1} {incr i} \
                     {
                      set j [expr $i + 1]

                      set yVal1 [expr [lindex $y $i] - $C]
                      set yVal2 [expr [lindex $y $j] - $C]

                      if {$yVal1*$yVal2 < 0} {incr freq}
                     }

                  set alpha [expr $PI*(0.5 + $freq)/$xRange]
                 }

              set title "amp*sin( PI*(alpha*x + phi) ) + C"
             }

       sinp  {
              set varInfo {amp alpha phi iPow C}

              if {![strcasecmp $alpha Auto]} \
                 {
                  set freq 1
                  set alpha [expr $PI*(0.5 + $freq)/$xRange]
                 }

              set title "amp*sin^iPow( PI*(alpha*x + phi) ) + C"
             }

       dsin  {
              set varInfo {amp alpha beta phi C} 

               if {![strcasecmp $alpha Auto]} \
                  {
                   for {set i 0} {$i < $n} {incr i} \
                      {
                       set xVal [lindex $x $i]
                       set yVal [expr [lindex $y $i] - $C]

                       if {$amp != 0.0} {set yVal [expr double($yVal)/$amp]}

                       if {$xVal <= $xAvg} \
                          {
                           lappend q $yVal
                          } \
                       else \
                          {
                           lappend r $yVal
                          }
                      }

                   set q [expr sqrt([getSumSq $q])]
                   set r [expr sqrt([getSumSq $r])]

                   if {$r == 0.0} {set r 1.0}

                   catch {set alpha [expr abs(log(abs($r/$q))/double($xAvg))]}
                  }

               if {![strcasecmp $beta Auto]} \
                  {
                   for {set i 0} {$i < $n - 1} {incr i} \
                      {
                       set j [expr $i + 1]

                       set yVal1 [expr [lindex $y $i] - $C]
                       set yVal2 [expr [lindex $y $j] - $C]

                       if {$yVal1*$yVal2 < 0} {incr freq}
                      }

                   set beta [expr $PI*(0.5 + $freq)/$xRange]
                  }

              set title "amp*exp( $iSign*|alpha|*x )*sin( PI*(beta*x + phi) ) + C"
             } 

       gsin  {
              set varInfo {amp alpha beta phi iPow C} 

               if {![strcasecmp $alpha Auto]} \
                  {
                   for {set i 0} {$i < $n} {incr i} \
                      {
                       set xVal [lindex $x $i]
                       set yVal [expr [lindex $y $i] - $C]

                       if {$amp != 0.0} {set yVal [expr double($yVal)/$amp]}

                       if {$xVal <= $xAvg} \
                          {
                           lappend q $yVal
                          } \
                       else \
                          {
                           lappend r $yVal
                          }
                      }

                   set q [expr sqrt([getSumSq $q])]
                   set r [expr sqrt([getSumSq $r])]

                   if {$r == 0.0} {set r 1.0}

                   catch {set alpha [expr abs(log(abs($r/$q))/$xAvg)]}
                   catch {set alpha [expr pow( $alpha, $iPow )]}
                  }

               if {![strcasecmp $beta Auto]} \
                  {
                   for {set i 0} {$i < $n - 1} {incr i} \
                      {
                       set j [expr $i + 1]

                       set yVal1 [expr [lindex $y $i] - $C]
                       set yVal2 [expr [lindex $y $j] - $C]

                       if {$yVal1*$yVal2 < 0} {incr freq}
                      }

                   set beta [expr $PI*(0.5 + $freq)/$xRange]
                  }

               if {[lsearch -exact $f iPow] >= 0} \
                  {
                   set title "amp*exp( $iSign*((alpha*x)^$iPow) )*sin( PI*(beta*x + phi) ) + C"
                  } \
              else \
                  {
                   set title "amp*exp( $iSign*((alpha*x)^iPow) )*sin( PI*(beta*x + phi) ) + C"
                  }
             }

       quad  {
              set varInfo {alpha beta C}

              set quadC [polySolve $x $y 2]

              if {![strcasecmp $alpha Auto]} {set alpha [lindex $quadC 0]}
              if {![strcasecmp $beta  Auto]} {set beta  [lindex $quadC 1]}
              if {![strcasecmp $C     Auto]} {set C     [lindex $quadC 2]}

              set title "alpha*x*x + beta*x + C"
             }

       karplus {
                set varInfo {alpha beta gamma phi}

                if {![strcasecmp $alpha Auto]} {set alpha [expr getSMaxAbs $y]}
                if {![strcasecmp $beta  Auto]} {set beta  [expr $alpha/2.0]}
                if {![strcasecmp $gamma Auto]} {set gamma [getAvg $y]}

                set title "alpha*cos^2( PI*(x+phi) ) + beta*cos( PI*(x+phi) ) + gamma"
               }
      }

   if {![strcasecmp $amp   Auto]} {set amp   1.0} 
   if {![strcasecmp $alpha Auto]} {set alpha 1.0} 
   if {![strcasecmp $beta  Auto]} {set beta  1.0} 
   if {![strcasecmp $gamma Auto]} {set gamma 1.0} 
   if {![strcasecmp $phi   Auto]} {set phi   0.0} 
   if {![strcasecmp $C     Auto]} {set C     0.0} 

   foreach v $varInfo \
      {
       if {[lsearch -exact $f $v] < 0} {eval lappend p \$$v}
      }

   set initInfo(amp)   $amp
   set initInfo(alpha) $alpha
   set initInfo(beta)  $beta
   set initInfo(gamma) $gamma
   set initInfo(phi)   $phi
   set initInfo(C)     $C
   set initInfo(iPow)  $iPow
}

proc showFitXY {} \
{
   global varList fitInfo modelInfo title smoothMode graphFlag orient verbose
   global pList eList x y s yModel lsRes srcName outName xLab yLab xyCount
   global exitFlag saveFlag silentFlag axisfont

   getFitXYInfo

   if {!$graphFlag || !$verbose} \
      {
       if {!$silentFlag} {puts $fitInfo}
      }

   if {$saveFlag} \
      {
       saveFitXY 
      }

   if {$exitFlag} exit

   if {!$graphFlag} exit 

   switch [string tolower $orient] \
      {
       nw {wm geom . +20+20}
       ne {wm geom . -20+20}
       se {wm geom . -20-20}
       sw {wm geom . +20-20}
      }

   frame  .cmnd
   graph  .g -plotbackground black -title "y = $title" -height 380 -width 500

   set yDiff ""

   for {set i 0} {$i < $xyCount} {incr i} \
      {
       set oVal  [lindex $y      $i]
       set cVal  [lindex $yModel $i]

       lappend yDiff [expr $oVal - $cVal]
      }

   .g axis configure x -tickfont $axisfont -ticklength 0.05i -title "x=$xLab"
   .g axis configure y -tickfont $axisfont -ticklength 0.05i -title "y=$yLab"

#  .g legend configure -hide 1

   .g element create Diff \
      -xdata $x -ydata $yDiff -color red -fill white -pixels 0 -linewidth 2

   .g element create Calc -smooth $smoothMode \
      -xdata $x -ydata $yModel -color blue -fill white -pixels 0 -linewidth 2

   .g element create Obs \
      -xdata $x -ydata $y -color yellow -fill yellow -pixels 2 -linewidth 0

   for {set i 0} {$i < $xyCount} {incr i} \
      {
       set xVal [lindex $x $i]
       set yVal [lindex $y $i]
       set sVal [lindex $s $i]

       set y1   [expr $yVal - $sVal]
       set y2   [expr $yVal + $sVal]

       .g marker create line -coords "$xVal $y1 $xVal $y2" \
          -outline gray -under no 
      }

   .g marker create line -coords "-Inf 0.0 Inf 0.0" -outline gray -under yes

   pack .g -in . -side top -expand 1 -fill both

   set fmt "Source: %s chi2/N: %.3f"

   button .save -text Save -command saveFitXY
   button .quit -text Quit -command exitFitXY
   label  .src  -text [format $fmt $srcName [expr $lsRes/$xyCount]]
   label  .lab  -text "Output:"
   entry  .name -width 16 -textvariable outName

   pack .src -in . -side top -expand 0 -fill x 

   set i 0

   foreach v $varList \
      {
       if {![info exists modelInfo($v)]} continue

       label .rep$i -text $modelInfo($v)
       pack  .rep$i -in . -side top -expand 0 -fill x 

       incr i
      }

   label .bl -text ""
   pack  .bl   -in . -side top -expand 0 -fill x 
   pack  .cmnd -in . -side top -expand 0 -fill x

   pack .lab .name .save -in .cmnd -side left
   pack .quit -in .cmnd -side right

   update
}

proc fitXY {} \
{
   global x y s p result iseed gc cf verbose

   if {$verbose} \
      {
       puts -nonewline stderr "REMARK Initial Values:"

       foreach pVal $p {puts -nonewline stderr [format " %.5e" $pVal]}

       puts stderr ""

       set result \
        [modelXY -x $x -y $y -noise $s -p $p -macro modelProc -gc $gc \
                 -cf $cf -iseed $iseed -silent -verb]
      } \
   else \
      {
       set result \
        [modelXY -x $x -y $y -noise $s -p $p -macro modelProc -gc $gc \
                 -cf $cf -iseed $iseed -silent -noverb]
      }
}

initFitXY
fitXY
showFitXY

