#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Adjust a Chemical Shift Table for use in a TALOS/DYNAMO database."
    puts " -in   inName   \[csObs.tab\]   CS Input Table."
    puts " -out  outName  \[csAdj.tab\]   CS Output, Adjusted."
    puts " -pdb  pdbName  \[ref.pdb\]     Reference PDB File."
    puts " -sim  simName  \[csCalc.tab\]  CS Simulation Output."
    puts " -keep frac     \[0.9\]         Fraction for Outlier Filtering."
    puts " -disu dist     \[3.0\]         Disulfide Bond Distance."
    puts " -cacb                          Allow Separate CA and CB Adjustments."
    exit
   }

set verbose 0

set inName   csObs.tab
set outName  csAdj.tab
set simName  csCalc.tab
set pdbName  ref.pdb 
set keepFrac 0.90
set disuDist 3.00
set cacbFlag 0

set minCor(HA)   0.1
set minCor(CA)   0.3
set minCor(CB)   0.3
set minCor(CACB) 0.3
set minCor(C)    0.4
set minCor(N)    0.5

getArgD $ARGV -in   inName
getArgD $ARGV -out  outName
getArgD $ARGV -sim  simName
getArgD $ARGV -pdb  pdbName
getArgD $ARGV -keep keepFrac
getArgD $ARGV -disu disuDist

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -cacb]}   {set cacbFlag 1}
if {[flagLoc $ARGV -nocacb]} {set cacbFlag 0}

#
#--------------------------------------------------------------------------

proc absSort { v1 v2 } \
{
   if {abs($v1) > abs($v2)} {return  1}
   if {abs($v1) < abs($v2)} {return -1}

   return 0
}

if {![file exists $pdbName]} \
   {
    puts stderr "Error Finding PDB File $pdbName"
    exit 1
   }

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Shift File $inName"
    exit 1
   }

dynInit

initResNames

set pdbRoot [file root [file tail $pdbName]]
set csRoot  [file root [file tail $inName]]

set pdbSrc  [dynRead -pdb -in $pdbName]
set pdbTab  [gdbRead table -in $pdbName -pdb]
set pInfo   [dynGetInfo -src $pdbSrc]
set a1      [lindex $pInfo 0]
set aN      [lindex $pInfo 1]

set csSrc   [dynRead -cs  -in $inName]
set simInfo [dynSimulate -cs $csSrc -src $pdbSrc]

dynWrite -cs $csSrc -out $simName -verb

set csTab   [gdbRead table -in $simName]
set seqList [getSeqList $csTab r1 rN]

exec /bin/cp $inName $outName

#
# Extract sequence and missing residues from the PDB:

for {set resID $r1} {$resID <= $rN} {incr resID} \
   {
    set pdbSeq($resID) missing
   }

set entry [gdbFirst $pdbTab]
set resIDList ""

while {$entry} \
   {
    set resID [gdbGet $entry RESID]

     if {[lsearch -exact $resIDList $resID] < 0} \
        {
         set resName [getResName1 [gdbGet $entry RESNAME]]
         set pdbSeq($resID) $resName

         lappend resIDList $resID
        }

    set entry [gdbNext $entry]
   }

#
# Extract disulfides info from PDB:

set cond  "!strcmp( RESNAME, 'CYS' ) && !strcmp( ATOMNAME, 'SG' )"
set eList [gdbSelect -from $pdbTab -cond $cond]
set n     [llength $eList]

set disuList ""
set ssList   ""

for {set i 0} {$i < [expr $n - 1]} {incr i} \
   {
    set entryI [lindex $eList $i]

    for {set j [expr $i + 1]} {$j < $n} {incr j} \
       {
        set entryJ [lindex $eList $j]
        set d      [pdbDist $entryI $entryJ]

        if {$d < $disuDist} \
           {
            set resIDI [gdbGet $entryI RESID]
            set resIDJ [gdbGet $entryJ RESID]
      
            if {abs($resIDI - $resIDJ) < 4} continue

            if {[lsearch -exact $disuList $resIDI] >= 0 || \
                [lsearch -exact $disuList $resIDJ] >= 0} \
               {
                puts stderr \
                 "Warning: Ambiguous Disfulfide, Residues $resIDI:$resIDJ."
               }
   
            set pdbSeq($resIDI) c
            set pdbSeq($resIDJ) c
  
            lappend disuList $resIDI
            lappend disuList $resIDJ

            lappend ssList $resIDI:$resIDJ

            break
           }
       }
   }

#
# Test match of residue definitions in shift table and PDB:

set resID    $r1
set misCount 0
set noCount  0

set pdbStr "$pdbRoot $a1:$aN"

foreach resName $seqList \
   {
    if {$resID < $a1 || $resID > $aN} \
       {
        puts -nonewline stderr "Residue Mismatch ($pdbStr): CS Residue "
        puts stderr "$resID $resName is outside the PDB range."

        incr resID
        continue
       }
 
    if {[strcmp $pdbSeq($resID) $resName]} {incr misCount}
    if {![strcmp $pdbSeq($resID) missing]} {incr noCount}

    switch $resName \
       {
        c {
           if {[lsearch -exact $disuList $resID] < 0} \
              {
               puts -nonewline stderr "CYS Mismatch ($pdbStr): CS Residue "
               puts stderr "$resID $resName is not a disulfide in the PDB."
              }
          }

        C {
           if {[lsearch -exact $disuList $resID] >= 0} \
              {
               puts -nonewline stderr "CYS Mismatch ($pdbStr): CS Residue "
               puts stderr "$resID $resName is a disulfide in the PDB ($ssList)"
              }
          }

        default \
           {
            if {[strcmp $resName $pdbSeq($resID)]} \
               {
                puts -nonewline stderr \
                 "Residue Mismatch ($pdbStr): CS Residue "

                puts stderr "$resID $resName is $pdbSeq($resID) in the PDB."
               }
           }
       }

    incr resID
   }

#
# Extract observed and simulated shifts:

set eList [gdbGetInfo $csTab -parms entries]

set atomList ""
set resList  ""

foreach entry $eList \
   {
    set atoms   [gdbGet $entry ATOMNAME]
    set resID   [gdbGet $entry RESID]
    set csObs   [gdbGet $entry CS_OBS]
    set csPred  [gdbGet $entry SHIFT]

    if {$csObs > 999.0 || $csPred > 999.0} continue

    if {[lsearch -exact "HA1 HA2 HA3 HA#" $atoms] >= 0} {set atoms HA}

    if {![strcmp $atoms CA]} {set atoms "CA CACB"}
    if {![strcmp $atoms CB]} {set atoms "CB CACB"}

    foreach atom $atoms \
       {
        if {[lsearch -exact $atomList $atom]  < 0} {lappend atomList $atom}
        if {[lsearch -exact $resList  $resID] < 0} {lappend resList  $resID}

        lappend csInfo($atom,csObs)   $csObs
        lappend csInfo($atom,csPred)  $csPred
        lappend csInfo($atom,csDiff)  [expr $csObs - $csPred]
       }
   }

#
# Display summary if needed:

if {$verbose} \
   {
    if {![llength $ssList]} \
       {
        set ssStr None
       } \
    else \
       {
        set ssStr $ssList
       }

    set csRMS [lindex $simInfo 0]
    set csN   [lindex $simInfo 2]

    set fmt "%s N: %d RMS: %.3f Residue Mismatch: %d Missing: %d Disulfides: %s"
    puts stderr [format $fmt $csRoot $csN $csRMS $misCount $noCount $ssStr]
   }

#
# Show the statistics for the atoms of interest:

if {$cacbFlag} \
   {
    set thisList {HA CA CB C N}
   } \
else \
   {
    set thisList {HA CA CB CACB C N}
   }

foreach atom {HA CA CB CACB C N} \
   {
    if {![info exists csInfo($atom,csDiff)]} continue

    set origList [lsort -command absSort $csInfo($atom,csDiff)]

    set nOrig    [llength $origList]
    set nBest    [expr int($nOrig*$keepFrac)] 

    if {$nOrig < 1 || $nBest < 1} continue

    puts -nonewline [format "%s %-4s " $csRoot $atom]

    set bestList [lrange $origList 0 [expr $nBest - 1]]

    set avg [getAvg    $origList]
    set dev [getStdDev $origList]
    set min [getMin    $origList]
    set max [getMax    $origList]

    set fmt "  %5d % 6.3f %5.3f % 7.3f % 7.3f"

    puts -nonewline [format $fmt $nOrig $avg $dev $min $max]

    set avg [getAvg    $bestList]
    set dev [getStdDev $bestList]
    set min [getMin    $bestList]
    set max [getMax    $bestList]

    set v   [expr $dev*3.0*sqrt($nBest)/$nBest]

    if {abs($avg) > $v} \
       {
        if {abs($avg) < $minCor($atom)} \
           {
            set status ">"
           } \
        elseif {![strcmp $atom CA] || ![strcmp $atom CB]} \
           {
            if {$cacbFlag} \
               {
                set status "*"
               } \
            else \
               {
                set status "+"
               }
           } \
        else \
           {
            set status "*"
           }
       } \
    else \
       {
        set status "."
       }

    set fmt "   %s %5d % 6.3f %5.3f % 7.3f % 7.3f"

    puts -nonewline [format $fmt $status $nBest $avg $dev $min $max]
    puts ""

    if {![strcmp $status "*"]} \
       {
        set c [expr -1.0*$avg]

        if {![strcmp $atom CACB]} \
           {
            set aList "CA CB"
           } \
        else \
           {
            set aList $atom
           }

        catch \
         "exec adjShift.tcl -in $outName -out $outName -atom $aList -adj $c"
       }
   }

exit
