
#
# gdb.tcl: auxillary modules for manipulating GDB database information.

proc getArrayList { arrayPtr indexListPtr valueListPtr } \
{
   upvar $arrayPtr     thisArray
   upvar $indexListPtr indexList
   upvar $valueListPtr valueList

   set indexList ""
   set valueList ""

   if {![info exists thisArray]} return

   set id [array startsearch thisArray]

   while {[array anymore thisArray $id]} \
      {
       set indexName [array nextelement thisArray $id]
       set value     $thisArray($indexName)

       lappend indexList $indexName
       lappend valueList $value
      }

   array donesearch thisArray $id
}

proc showTabText { tab outUnit } \
{
   set text [gdbGetInfo $tab -parms text]
   set n 0

   set prevBlank 1

   while {$text} \
      {
       set str  [gdbGetInfo $text -parms value]

       if {[string length $str]} \
          {
           puts $outUnit $str
           set prevBlank 0
          } \
       else \
          {
           if {!$prevBlank} {puts $outUnit ""}
           set prevBlank 1
          }

       set text [gdbNext $text]
       incr n
      }

   return $n

}

proc getTabTextCount { tab } \
{
   set text [gdbGetInfo $tab -parms text]
   set n 0

   while {$text} \
      {
       set text [gdbNext $text]
       incr n
      }

   return $n
}

proc getTabTextInfo { tab tPtr } \
{
   upvar $tPtr textInfo

   set text [gdbGetInfo $tab -parms text]
   set n 0

   while {$text} \
      {
       set textInfo($n) [gdbGetInfo $text -parms value]
       set text [gdbNext $text]
       incr n
      }

   return $n
}

proc getTabKeyList { tab } \
{
   set text    [gdbGetInfo $tab -parms text]
   set keyList ""

   while {$text} \
      {
       set str [gdbGetInfo $text -parms value]
       set key [lindex $str 0]

       if {![strcasecmp $key "DATA"]} \
          {
           set id [lindex $str 1]
           if {[lsearch -exact $keyList $id] < 0} {lappend keyList $id}
          }

       set text [gdbNext $text]
      }

   return $keyList
}

proc getTabKeyLength { tab keyName } \
{
   set text [gdbGetInfo $tab -parms text]
   set n 0

   while {$text} \
      {
       set str [gdbGetInfo $text -parms value]
       set key [lindex $str 0]

       if {![strcasecmp $key "DATA"]} \
          {
           set id  [lindex $str 1]
           if {![strcasecmp $id $keyName]} {incr n}
          }

       set text [gdbNext $text]
      }

   return $n
}

proc getTabKeyItem { tab keyName loc } \
{
   set text [gdbGetInfo $tab -parms text]
   set n 0

   while {$text} \
      {
       set str [gdbGetInfo $text -parms value]
       set key [lindex $str 0]

       if {![strcasecmp $key "DATA"]} \
          {
           set id  [lindex $str 1]
           set val [lrange $str 2 end]

           if {![strcasecmp $id $keyName]} \
              {
               if {$n == $loc} {return $val} 
               incr n
              }
          }

       set text [gdbNext $text]
      }

   return "" 
}


proc getKeyData { tab arrayPtr } \
{
   upvar $arrayPtr array
  
   set text [gdbGetInfo $tab -parms text]

   while {$text} \
      {
       set str [gdbGetInfo $text -parms value]
       set key [lindex $str 0]

       if {![strcasecmp $key "DATA"]} \
          {
           set id  [lindex $str 1]
           set val [lrange $str 2 end]

           lappend array($id) $val
          }

       set text [gdbNext $text]
      }
}

proc isPDBFile { inName } \
{
   if {![file exists $inName]} {return 0}

   set error [catch {set inUnit [open $inName r]}]
   if {$error} {return 0}

   set hasVARS 0
   set hasFMT  0
   set hasATOM 0

   set vars    ""
   set fmt     ""

   while {[gets $inUnit line] >= 0} \
      {
       set key None

       scan $line %s key

       if {![strcasecmp $key VARS]} \
          {
           set vars $line
           set hasVARS 1
          } \
       elseif {![strcasecmp $key FORMAT]} \
          {
           set fmt $line
           set hasFMT 1
          } \
       elseif {![strcasecmp $key ATOM]} \
          {
           incr hasATOM
          } 
      }

    catch "close $inUnit"

    if {$hasVARS && $hasFMT} {return 0}
    if {$hasATOM} {return 1}

    return 0
}

proc isGDBFile { inName } \
{
   if {![file exists $inName]} {return 0}

   set error [catch {set inUnit [open $inName r]}]
   if {$error} {return 0}

   set hasVARS 0
   set hasFMT  0

   set vars    ""
   set fmt     ""

   while {[gets $inUnit line] >= 0} \
      {
       set key None   

       scan $line %s key 

       if {![strcasecmp $key VARS]} \
          {
           set vars $line
           set hasVARS 1
          } \
       elseif {![strcasecmp $key FORMAT]} \
          {
           set fmt $line
           set hasFMT 1
          }

       if {$hasVARS && $hasFMT} break 
      }

    catch "close $inUnit"

    if {$hasVARS && $hasFMT} {return 1}

    return 0
}

proc gdbGetDef { e var def } \
{
   if {![llength $e] || ![llength $var]}  {return $def}
   if {![strcmp $e 0]}  {return $def}

   if {![strcasecmp $e   None]} {return $def}
   if {![strcasecmp $var None]} {return $def}
   if {![strcasecmp $var Auto]} {return $def}

   set v [gdbGet $e $var]

   return $v
}

proc div3 { x y d } \
{
   if {$y == 0} {return $d}

   set q [expr double($x)/$y]

   return $q
}

#
# Bottom.
