#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Extract 2D Planes from a 4D Spectrum According to a Chemical Shift Table:"
   echo " -in      specName [ft/test%04d.ft4] Input 4D Spectrum."
   echo " -tab     tabName  [test.tab]        Input Table of Chemical Shifts."
   echo " -outDir  outDir   [za]              Output Directory for 2D Planes."
   echo " -outName outName  [test%04d.dat]    Output Template Name."
   echo "Chemical Shift Table and Parameters:"
   echo " -xVar    xVar     [X_PPM]           X-Axis Chemical Shift Variable."
   echo " -yVar    yVar     [Y_PPM]           Y-Axis Chemical Shift Variable."
   echo " -idVar   idVar    [INDEX]           Index Number Variable or Keyword None."
   echo " -assVar  assVar   [ASS]             Assignment Variable, or Keyword None."
   echo "Spectrum Axis Names:"
   echo " -xName   xName    [CUR_XDIM]        Corresponds to Table X-Axis."
   echo " -yName   yName    [CUR_YDIM]        Corresponds to Table Y-Axis."
   echo " -zName   zName    [CUR_ZDIM]        Corresponds to Extracted Plane X-Axis."
   echo " -aName   aName    [CUR_ADIM]        Corresponds to Extracted Plane Y-Axis."
   echo "Spectrum Extracted Region Widths, +/- Pts Hz ppm %:"
   echo " -dx      dx       [0]               Width of Extracted Region in xName Axis."
   echo " -dy      dy       [0]               Width of Extracted Region in yName Axis."
   echo "How to Extract Planes:"
   echo " -nopair                             Plane for Every Table Entry (Default)."
   echo " -pair                               Plane for Every Possible Pair of Entries."
   echo "Notes:"
   echo " 1. When the -pair options is used, the default value for -yVar will be set"
   echo "    to the value for -xVar."
   echo " 2. If the shift table has N entries, the -nopair mode will save N planes,"
   echo "    and the -pair mode will save N*N planes."
   echo " 3. Example: assuming 4D Spectrum with Axis: H1x H1y C13z C13a, and a peak"
   echo "    table X_PPM with 1H ppm values and Y_PPM with 13C ppm values, this"
   echo "    command extracts an H1y,C13a plane at every location in the table:"
   echo ""
   echo "    get4DPlanes.com -tab hc.tab -in ft/test%04d.ft4 -outDir za \\"
   echo "                    -xName H1x -yName C13z -zName H1y -aName C13a"
   echo ""
   echo " 5. For the same spectrum, this command would extract all 1Hx,1Hy planes"
   echo "    at every pairing of 13C,13C locations from the table:"
   echo ""
   echo "    get4DPlanes.com -tab hc.tab -in ft/test%04d.ft4 -outDir hh -pair \\"
   echo "                    -xName C13z -yName C13a -zName H1x -aName H1y    \\"
   echo "                    -xVar Y_PPM"
   exit 0
endif

set pairFlag = 0

if (`flagLoc $argv -pair`) then
   set pairFlag = 1
endif

if (`flagLoc $argv -nopair`) then
   set pairFlag = 0
endif

set specName = (`getArgD $argv -in      ft/test%04d.ft4`)
set tabName  = (`getArgD $argv -tab     test.tab`)
set outDir   = (`getArgD $argv -outDir  za`)
set outName  = (`getArgD $argv -outName test%04d.dat`)

set xName    = (`getArgD $argv -xName   CUR_XDIM`)
set yName    = (`getArgD $argv -yName   CUR_YDIM`)
set zName    = (`getArgD $argv -zName   CUR_ZDIM`)
set aName    = (`getArgD $argv -aName   CUR_ADIM`)

set dx       = (`getArgD $argv -dx      0`)
set dy       = (`getArgD $argv -dy      0`)

if ($pairFlag) then
   set tabVarX  = (`getArgD $argv -xVar    X_PPM`)
   set tabVarY  = (`getArgD $argv -yVar    $tabVarX`)
else
   set tabVarX  = (`getArgD $argv -xVar    X_PPM`)
   set tabVarY  = (`getArgD $argv -yVar    Y_PPM`)
endif

set idVar    = (`getArgD $argv -idVar   INDEX`)
set assVar   = (`getArgD $argv -assVar  ASS`)

#
#---------------------------------------------------------------------------------

if (!(-e $tabName)) then
   echo "Error Finding Input Table $tabName"
   exit 1
endif

set thisName = (`nmrPrintf $specName 1 1`)

if (!(-e $thisName)) then
   echo "Error Finding Input Spectrum $specName ($thisName)."
   exit 1
endif

set xPPMList    = (`getTabCol.tcl -in $tabName -var $tabVarX`)
set yPPMList    = (`getTabCol.tcl -in $tabName -var $tabVarY`)

set idList      = ()
set assList     = ()
set fullXList   = ()
set fullYList   = ()
set fullAssList = ()

@ n = $#xPPMList

if (`string csame $idVar None`) then
   @ i = 1

   while ($i <= $n)
      set idList = ($idList $i)
      @ i++
   end
else
   set idList = (`getTabCol.tcl -in $tabName -var $idVar`)
endif

if (`string csame $assVar None`) then
   @ i = 1

   while ($i <= $n)
      set assList = ($assList None)
      @ i++
   end
else
   set assList = (`getTabCol.tcl -in $tabName -var $assVar`)
endif

if ($pairFlag) then
   @ i = 1

  while ($i <= $n)
     @ j = $i

     if (`string csame $assList[$i] None`) then
        set assI = $idList[$i]
     else
        set assI = $idList[$i]"="$assList[$i]
     endif

     while ($j <= $n)
        if (`string csame $assList[$j] None`) then
           set assJ = $idList[$j]
        else
           set assJ = $idList[$j]"="$assList[$j]
        endif

        set fullXList   = ($fullXList   $xPPMList[$i])
        set fullYList   = ($fullYList   $yPPMList[$j])
        set fullAssList = ($fullAssList $assI","$assJ)

        if ($i != $j) then
           set fullXList   = ($fullXList   $xPPMList[$j])
           set fullYList   = ($fullYList   $yPPMList[$i])
           set fullAssList = ($fullAssList $assJ","$assI)
        endif

        @ j++
     end

     @ i++
  end   
else
   @ i = 1

   while ($i <= $n)
     if (`string csame $assList[$i] None`) then
        set assI = $idList[$i]
     else
        set assI = $idList[$i]"="$assList[$i]
     endif

     set fullXList   = ($fullXList   $xPPMList[$i])
     set fullYList   = ($fullYList   $yPPMList[$i])
     set fullAssList = ($fullAssList $assI)

     @ i++
   end
endif

/bin/rm -rf $outDir
mkdir $outDir

@ nn = $#fullAssList
@ k  = 1

while ($k <= $nn)
   set thisOutName = (`nmrPrintf "$outDir/$outName" $k`)

   set cmnd = "readROI -ndim 2 -in $specName -out $thisOutName -x $zName 0% 100% -y $aName 0% 100% -dz $xName $fullXList[$k]ppm -da $yName $fullYList[$k]ppm $dy"

   echo $fullAssList[$k] $cmnd

   $cmnd -silent

   sethdr $thisOutName -title $fullAssList[$k]

   @ k++
end

series.com $outDir/*

