#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Extract Information About a Non-Unform Sampling (NUS) Schedule."
    puts "Version 2022.343.16.58 frank.delaglio@nist.gov"
    puts " -in          sName  \[Auto\]  NUS Schedule File Input (also -sample)."
    puts " -sampleCount n      \[Auto\]  Number of Samples Actually Acquired."
    puts " -parmDir     pDir   \[.\]     Parameter File Input Directory (For Bruker acq*)."
    puts " -rev                        Reverse Items in Line Before Extraction."
    puts " -pseudo3D                   Data is a Pseudo-3D Experiment (Sets -qzSize 1)."
    puts "Quadrature Sizes (Real = 1, Complex = 2):"
    puts " -qySize      qySize \[Auto\]  Y-Axis Quadrature Size."
    puts " -qzSize      qzSize \[2\]     Z-Axis Quadrature Size."
    puts " -qaSize      qaSize \[2\]     A-Axis Quadrature Size."
    puts "Notes:"
    puts " 1. NUS schedules are expected to have one column of integers for each indirect"
    puts "    dimension, representing the time increment values."
    puts " 2. The lowest increment value found is assumed to be the origin, usually zero."
    puts " 3. Using -parmDir None will suppress reading of acq files."
#   puts " 4. RQD (Random Quadrature Detection) schedules are expected to have an"
#   puts "    additional column of zeros and ones for each indirect dimension,"
#   puts "    representing real or imaginary."
    exit 0
   }

set sampleName   Auto 
set sampleCount  Auto 
set parmDir      .
set pseudo3DFlag 0
set revFlag      0

set qySize       Auto
set qzSize       2
set qaSize       2

getArgD $ARGV -in     sampleName
getArgD $ARGV -sample sampleName

getArgD $ARGV -qySize qySize
getArgD $ARGV -qzSize qzSize
getArgD $ARGV -qaSize qaSize

if {![strcasecmp $sampleName Auto]} \
   {
    set sampleName [lindex [findNUSInput .] 0]
    if {![llength $sampleName]} {set sampleName None}
   }

getArgD $ARGV -sampleCount sampleCount
getArgD $ARGV -parmDir     parmDir

if {[flagLoc $ARGV -rev]}   {set revFlag 1}
if {[flagLoc $ARGV -norev]} {set revFlag 0}

set lineCount    0
set blankCount   0
set commentCount 0
set dataCount    0

set minItems     0
set maxItems     0
set offset       0

set dimCount     0
set nusDimCount  0
set nusFormat    None

set yTMax        0
set zTMax        0
set aTMax        0

if {![file exists $sampleName]} \
   {
    puts stderr "NUS Info Error finding schedule input $sampleName"
    exit 1
   }

set error [catch {set sUnit [open $sampleName r]}]

if {$error} \
   {
    puts stderr "NUS Info Error Opening Sampling Input File $sampleName"
    exit 1
   }

set pList(celem) 0
set incrCount    0

if {![strcasecmp $sampleCount Auto] && [strcasecmp $parmDir None]} \
   {
    if {[file exists $parmDir/procpar]} \
       {
        set status [rdProcPar $parmDir/procpar pList stderr]
       } \
    elseif {[file exists $parmDir/acqus] && [file exists $parmDir/acqu2s]} \
       {
        rdJCAMP $parmDir/acqus  acqus
        rdJCAMP $parmDir/acqu2s acqu2s
    
        getTopspinVersion acqus
   
        if {$acqus(topspinVersion) >= 3} \
           {
            if {[info exists acqu2s(TD)]} \
               {
                if {[string is double $acqu2s(TD)]} {set incrCount [expr int($acqu2s(TD))]}
                if {$incrCount < 0} {set incrCount 0}
               }            
           }

        if {[file exists $parmDir/acqu3s]} \
           {
            rdJCAMP $parmDir/acqu3s acqu3s

            if {[info exists acqu3s(TD)] && $incrCount > 0} \
               {
                if {[string is double $acqu3s(TD)]} {set incrCount [expr $incrCount*int($acqu3s(TD))]}
                if {$incrCount < 0} {set incrCount 0}
               }
           }

        if {[file exists $parmDir/acqu4s]} \
           {
            rdJCAMP $parmDir/acqu4s acqu4s

            if {[info exists acqu4s(TD)] && $incrCount > 0} \
               {
                if {[string is double $acqu4s(TD)]} {set incrCount [expr $incrCount*int($acqu4s(TD))]}
                if {$incrCount < 0} {set incrCount 0}
               }
           }
       }
   }

if {[info exists acqu2s(FnMODE)] && [info exists acqu3s(FnMODE)] && ![info exists acqu4s(FnMODE)]} \
   {
    if {($acqu2s(FnMODE) <= 1 && $acqu3s(FnMODE) > 1) || ($acqu3s(FnMODE) <= 1 && $acqu2s(FnMODE) > 1)} {set pseudo3DFlag 1}
    if {($acqu2s(FnMODE) == 7 && $acqu3s(FnMODE) > 1) || ($acqu3s(FnMODE) == 7)} {set pseudo3DFlag 1}
   }

if {[info exists acqu2s(FnMODE)] && ![info exists acqu3s(FnMODE)]} \
   {
    if {![strcasecmp $qySize Auto]} \
       {
        if {$acqu2s(FnMODE) == 1 || $acqu2s(FnMODE) == 7} {set qySize 1}
       }
   }

if {![strcasecmp $qySize Auto]} {set qySize 2}

if {[info exists acqu2s(FnILOOP)]} \
   {
    if {$acqu2s(FnILOOP) > 1} {set pseudo3DFlag 1} 
   }

if {[flagLoc $ARGV -pseudo3D]}   {set pseudo3DFlag 1}
if {[flagLoc $ARGV -nopseudo3D]} {set pseudo3DFlag 0}

if {$pseudo3DFlag} {set qzSize 1}

set typeList(all) ""
set valList(all)  ""

set typeList(0)   ""
set valList(0)    ""

while {[gets $sUnit str] >= 0} \
   {
    set line [string trim $str]

    if {$revFlag} {set line [lreverse $line]}

    incr lineCount 

    set key [string index [lindex $line 0] 0]

    if {![strcmp $key "#"]} \
       {
        incr commentCount
        continue
       }

    set thisItemCount [llength $line]

    if {!$thisItemCount} \
       {
        incr blankCount
        continue
       }

    incr dataCount

    if {$dataCount == 1} \
       {
        set minItems $thisItemCount
        set maxItems $thisItemCount
       }

    if {$thisItemCount < $minItems} {set minItems $thisItemCount}
    if {$thisItemCount > $maxItems} {set maxItems $thisItemCount}

    for {set i 0} {$i < $thisItemCount} {incr i} \
       {
        if {![info exists typeList($i)]} {set typeList($i) ""}
        if {![info exists valList($i)]}  {set valList($i)  ""}

        set item [lindex $line $i]

        if {[string is integer $item]} \
           {
            if {[lsearch -exact $typeList(all) int] < 0} {lappend typeList(all) int}
            if {[lsearch -exact $typeList($i)  int] < 0} {lappend typeList($i)  int}

            lappend valList(all)   $item
            lappend valList($i)    $item
           } \
        elseif {[string is double $item]} \
           {
            if {[lsearch -exact $typeList(all) float] < 0} {lappend typeList(all) float}
            if {[lsearch -exact $typeList($i)  float] < 0} {lappend typeList($i)  float}
           } \
        else \
           { 
            if {[lsearch -exact $typeList(all) other] < 0} {lappend typeList(all) other}
            if {[lsearch -exact $typeList($i)  other] < 0} {lappend typeList($i)  other}
           }
       }
   }

if {$maxItems < 1} \
   {
    set sampleCountMax 0
    set sampleCount    0 
    set nusFormat      Bad

    puts "schedName $sampleName nusFormat $nusFormat nusDimCount $nusDimCount yTNUS $yTMax offset $offset sampleCount $sampleCount qySize 0 qzSize 0 qaSize 0"
   } \
elseif {$maxItems > 6} \
   {
    set sampleCountMax [llength $valList(all)] 

    if {![strcasecmp $sampleCount Auto]} \
       {
        if {[strcmp $pList(celem) 0]} \
           {
            set qAdjust 1
           
            if {$nusDimCount >= 2 && $qySize > 1} {set qAdjust [expr $qAdjust*$qySize]}
            if {$nusDimCount >= 3 && $qzSize > 1} {set qAdjust [expr $qAdjust*$qzSize]}
            if {$nusDimCount >= 4 && $qaSize > 1} {set qAdjust [expr $qAdjust*$qaSize]}

            set sampleCount [expr int($pList(celem)/$qAdjust)]
           } \
        elseif {$incrCount} \
           {
            set sampleCount [expr int($incrCount/2)]
           } \
        else \
           {
            set sampleCount [llength $valList(all)]
           }
       }
 
    if {$sampleCount > $sampleCountMax} {set sampleCount $sampleCountMax}

    set nusFormat   2D
    set nusDimCount 3
    set vecCount    [expr 2*$sampleCount]
    set nn          [expr $sampleCount - 1]

    set offset   [getMin $valList(all)]
    set yTMax    [expr [getMax [lrange $valList(all) 0 $nn]] + 1 - $offset]

    puts -nonewline "schedName $sampleName nusFormat $nusFormat nusDimCount $nusDimCount colCount $maxItems"
    puts -nonewline " offset $offset sampleCount $sampleCount sampleCountMax $sampleCountMax vecCount $vecCount yTNUS $yTMax qySize $qySize qzSize $qzSize qaSize $qaSize"
    puts ""
   } \
elseif {$minItems == $maxItems && [llength $typeList(all)] == 1 && ![strcasecmp [lindex $typeList(all) 0] int]} \
   {
    set nusFormat      ND
    set nusDimCount    [expr $minItems + 1]
    set sampleCountMax [llength $valList(0)] 

    if {![strcasecmp $sampleCount Auto]} \
       {
        if {[strcmp $pList(celem) 0]} \
           {
            set qAdjust 1

            if {$nusDimCount >= 2 && $qySize > 1} {set qAdjust [expr $qAdjust*$qySize]}
            if {$nusDimCount >= 3 && $qzSize > 1} {set qAdjust [expr $qAdjust*$qzSize]}
            if {$nusDimCount >= 4 && $qaSize > 1} {set qAdjust [expr $qAdjust*$qaSize]}

            set sampleCount [expr int($pList(celem)/$qAdjust)]
           } \
        elseif {$incrCount} \
           {
            set sampleCount $incrCount

            if {$nusDimCount >= 4} {set sampleCount [expr int($sampleCount/$qaSize)]}
            if {$nusDimCount >= 2} {set sampleCount [expr int($sampleCount/$qySize)]}

            if {$nusDimCount >= 3 && !$pseudo3DFlag} {set sampleCount [expr int($sampleCount/$qzSize)]}
           } \
        else \
           {
            set sampleCount [llength $valList(0)]
           }
       }

    if {$sampleCount > $sampleCountMax} {set sampleCount $sampleCountMax}

    set offset [getMin $valList(all)]
    set nn     [expr $sampleCount - 1]

    set vecCount $sampleCount

    for {set i 0} {$i < $minItems} {incr i} {set vecCount [expr 2*$vecCount]}
   
    if {$qySize == 1}  {set vecCount [expr $vecCount/2]}
    if {$pseudo3DFlag} {set vecCount [expr $vecCount/2]}
 
    puts -nonewline "schedName $sampleName nusFormat $nusFormat nusDimCount $nusDimCount colCount $maxItems"
    puts -nonewline " offset $offset sampleCount $sampleCount sampleCountMax $sampleCountMax vecCount $vecCount qySize $qySize qzSize $qzSize qaSize $qaSize"
 
    if {$nusDimCount >= 2} \
       {
        set yTMax [expr [getMax [lrange $valList(0) 0 $nn]] + 1 - $offset]
        puts -nonewline " yTNUS $yTMax"
       }

    if {$nusDimCount >= 3} \
       { 
        set zTMax [expr [getMax [lrange $valList(1) 0 $nn]] + 1 - $offset]
        puts -nonewline " zTNUS $zTMax"
       }

    if {$nusDimCount >= 4} \
       {
        set aTMax [expr [getMax [lrange $valList(2) 0 $nn]] + 1 - $offset]
        puts -nonewline " aTNUS $aTMax"
       }

    puts ""
   } \
else \
   {
    if {![strcasecmp $sampleCount Auto]} {set sampleCount [llength $valList(0)]}

    set sampleCountMax [llength $valList(0)]
    if {$sampleCount > $sampleCountMax} {set sampleCount $sampleCountMax}

    set vecCount       [expr 2*$sampleCount]

    set nusFormat   Unknown
    set nusDimCount 2
    
    puts "schedName $sampleName nusFormat $nusFormat nusDimCount $nusDimCount yTNUS $yTMax offset $offset sampleCount $sampleCount sampleCountMax $sampleCountMax vecCount $vecCount qySize $qySize qzSize $qzSize qaSize $qaSize"
   }

exit 0
