#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "List Information about a PDB File:"
    puts " -in       pdbName \[ref.pdb\]  Input PDB."
    puts " -info     iList   \[All\]      List of Properties to Display."
    puts " -cond     cond    \[1\]        Selection Condition."
    puts " -null     nStr    \[NONE\]     String for Missing/Null Values."
    puts " -thresh   t       \[30.0\]     Percent Threshold for needH Test."
    puts " -hetatm                      Interpret HETATMs as ATOMs."
    puts " -renumber                    Renumber ATOMID on Input."
    puts " -verb                        Verbose Mode ON (Default)."
    puts " -noverb                      Verbose Mode OFF."
    puts "Property Names for Keyword All:"
    puts " name atomCount resCount chainCount segCount locCount"
    puts " molCount gapCount r1 rN rNext type min max range center extent"
    puts " atomNames resNames chainNames segNames locNames sequence resTypeCounts"
    puts "Other Property Names:"
    puts " resID atomID hetatmCount modelCount needH"
    puts "Notes:"
    puts " 1. Property resCount returns the number of unique residue IDs,"
    puts "    without consideration of chain ID etc."
    puts " 2. Property hetatmCount requires the -hetatm option, and only"
    puts "    reports the number of HETATMs in the original PDB file,"
    puts "    before applying the selection condition." 
    puts " 3. Property needH is 1 or 0 depending on whether the percentage"
    puts "    of H atoms is above the specified -thresh value."
    puts " 4. Property type includes one or more of:"
    puts "    protein na other"
    exit 1
   }

set inName    ref.pdb 
set iList     All
set nullStr   NONE
set thresh    30.0
set hetFlag   0
set cond      1
set renumFlag 0
set verbose   1

getArgD     $ARGV -in     inName
getArgD     $ARGV -thresh thresh

getListArgD $ARGV -info iList
getListArgD $ARGV -cond cond
 
if {[flagLoc $ARGV -hetatm]}   {set hetFlag 1}
if {[flagLoc $ARGV -nohetatm]} {set hetFlag 0}

if {[flagLoc $ARGV -verb]}     {set verbose 1}
if {[flagLoc $ARGV -noverb]}   {set verbose 0}

if {[flagLoc $ARGV -renumber]}    {set renumFlag 1}
if {[flagLoc $ARGV -norenumber]}  {set renumFlag 0}

set cond [string trim $cond "{}"]

#
#-------------------------------------------------------------------------------------------

if {![file exists $inName]} {exit 0}

if {![strcmp $iList "*"] || ![strcasecmp $iList All]} \
   {
    set iList {
               name atomCount resCount chainCount segCount locCount molCount gapCount 
               r1 rN rNext type min max range center extent 
               atomNames resNames chainNames segNames locNames sequence resTypeCounts
              }
   }

set aa3List {
             ALA  ARG  ASN  ASP  CYS  GLN  GLU  GLY  HIS  ILE
             LEU  LYS  MET  PHE  PRO  SER  THR  TRP  TYR  VAL
             ANI
            }

set na3List {ADE CYT GUA THY URI DADE DCYT DGUA DTHY RADE RCYT RGUA RURI}

set na1List {A   C   G   T   U   A    C    G    T    A    C     G   U} 

if {$hetFlag} \
   {
    set tOrig [gdbRead table -in $inName -pdb -hetatm]
   } \
else \
   {
    set tOrig [gdbRead table -in $inName -pdb ]
   }

if {![strcmp $cond 1]} \
   {
    set t $tOrig
   } \
else \
   {
    set t  [gdbSelect -new -from $tOrig -cond [string trim $cond "{}"]]
   }

if {$renumFlag} \
   {
    set eList [gdbGetInfo $t -parms entries]
    set id    0

    foreach e $eList \
       {
        incr id
        gdbSet $e ATOMID $id
       }
   }

set n 0

foreach i $iList \
   {
    if {$verbose} {puts -nonewline "$i "}

    switch -exact [string tolower $i] \
       {
        name        {puts $inName}

        atomname    {puts [getTabUniqVals $t ATOMNAME NONE]}
        atomnames   {puts [getTabUniqVals $t ATOMNAME NONE]}
        atomcount   {puts [gdbGetInfo $t -parms entryCount]}
       
        hetatmcount {puts [getHETATMCount $tOrig]}

        resname    {puts [getTabUniqVals $t RESNAME NONE]}
        resnames   {puts [getTabUniqVals $t RESNAME NONE]}
        rescount   {puts [llength [getTabUniqVals $t RESID]]}

        chainname  {puts [getTabUniqVals $t CHAINID NONE]}
        chainnames {puts [getTabUniqVals $t CHAINID NONE]}
        chaincount {puts [llength [getTabUniqVals $t CHAINID]]}

        locname    {puts [getTabUniqVals $t LOCID NONE]}
        locnames   {puts [getTabUniqVals $t LOCID NONE]}
        loccount   {puts [llength [getTabUniqVals $t LOCID]]}

        segname    {puts [getTabUniqVals $t SEGID NONE]}
        segnames   {puts [getTabUniqVals $t SEGID NONE]}
        segcount   {puts [llength [getTabUniqVals $t SEGID]]}

        modelcount {puts [getPDBModelCount $inName]}
        molcount   {puts [getMax "[llength [getTabUniqVals $t SEGID]] [llength [getTabUniqVals $t CHAINID]]"]}

        gapcount   {
                    set idList   [lsort -integer [getTabUniqVals $t RESID]]
                    set nn       [llength $idList]
                    set gapCount 0

                    for {set i 0} {$i < $nn - 1} {incr i} \
                       {
                        set thisID [lindex $idList $i]
                        set nextID [lindex $idList [expr $i + 1]]

                        if {$nextID != $thisID + 1} {incr gapCount}
                       }

                    puts $gapCount
                   }

        resid      {puts [lsort -integer [getTabUniqVals $t RESID]]}

        atomid     {puts [getTabVals $t ATOMID]}

        needh      {puts [pdbNeedHTest $t $thresh]}

        r1         {puts [format %d [getTabMin $t RESID]]}

        rn         {puts [format %d [getTabMax $t RESID]]}
        rnext      {puts [expr 1 + int([getTabMax $t RESID])]}

        min        {
                    set stat getMin

                    set sx [getTabStat $stat $t X] 
                    set sy [getTabStat $stat $t Y] 
                    set sz [getTabStat $stat $t Z]

                    puts [format "%.3f %.3f %.3f" $sx $sy $sz]
                   }

        max        {
                    set stat getMax

                    set sx [getTabStat $stat $t X]    
                    set sy [getTabStat $stat $t Y]   
                    set sz [getTabStat $stat $t Z]

                    puts [format "%.3f %.3f %.3f" $sx $sy $sz]
                   }

        range      {
                    set stat getRange

                    set sx [getTabStat $stat $t X]    
                    set sy [getTabStat $stat $t Y]   
                    set sz [getTabStat $stat $t Z]

                    puts [format "%.3f %.3f %.3f" $sx $sy $sz]
                   }

        center     {
                    set stat getAvg

                    set sx [getTabStat $stat $t X]    
                    set sy [getTabStat $stat $t Y]   
                    set sz [getTabStat $stat $t Z]

                    puts [format "%.3f %.3f %.3f" $sx $sy $sz]
                   }

        extent     {
                    set stat getRange

                    set sx [getTabStat $stat $t X]    
                    set sy [getTabStat $stat $t Y]   
                    set sz [getTabStat $stat $t Z]

                    puts [format "%.3f" [expr sqrt( $sx*$sx +  $sy*$sy + $sz*$sz)]]
                   }

        type       {puts [getPDBType $t]}

        restypecounts {
                       set eList [gdbGetInfo $t -parms entries]
                       set rList ""
                       set nList ""

                       foreach e $eList \
                          {
                           set resID     [gdbGet $e RESID]
                           set resName   [gdbGet $e RESNAME]
                           set thisChain [gdbGet $e CHAINID]
                           set thisSeg   [gdbGet $e CHAINID]

                           if {![info exists rCount($resName)]} \
                              {
                               set rCount($resName) 0
                               lappend nList $resName
                              }

                           if {[lsearch -exact $rList $resID,$thisChain,$thisSeg] < 0} \
                              {
                               lappend rList $resID,$thisChain,$thisSeg
                               incr rCount($resName)                               
                              }
                          }

                       set kk 0

                       foreach resName [lsort $nList] \
                          {
                           if {$kk} {puts -nonewline " "}
                           puts -nonewline "$resName $rCount($resName)"
                           incr kk
                          }

                       puts ""
                      }

        sequence   {
                    set thisChainList [getTabUniqVals $t CHAINID NONE]
                    set thisSegList   [getTabUniqVals $t SEGID   NONE]

                    set thisR1 0
                    set thisRN 0

                    set kk 0 

                    if {[llength $thisSegList] < 2} \
                       {
                        if {[llength $thisChainList] < 2} \
                           {
                            puts [getPDBSeqList $t]
                           } \
                        else \
                           {
                            foreach thisChain $thisChainList \
                               {
                                set tt      [gdbSelect -new -from $t -cond "!strcasecmp( CHAINID, '$thisChain' )"]
                                set thisSeq [getPDBSeqList $tt thisR1 thisRN]

                                if {$kk} {puts -nonewline " "}
                                puts -nonewline "chain=$thisChain\[$thisR1:$thisRN\] $thisSeq"
                                incr kk

                                gdbFree $tt
                               }
                           }
                       } \
                    else \
                       {
                        if {[llength $thisChainList] < 2} \
                           {
                            puts [getPDBSeqList $t]

                            foreach thisSeg $thisSegList \
                               {
                                set tt      [gdbSelect -new -from $t -cond "!strcasecmp( SEGID, '$thisSeg' )"]
                                set thisSeq [getPDBSeqList $tt thisR1 thisRN]

                                if {$kk} {puts -nonewline " "}
                                puts "seg=$thisSeg\[$thisR1:$thisRN\] $thisSeq"
                                incr kk

                                gdbFree $tt
                               }
                           } \
                        else \
                           {
                            foreach thisSeg $thisSegList \
                               {
                                foreach thisChain $thisChainList \
                                   {
                                    set tt      [gdbSelect -new -from $t -cond "!strcasecmp( CHAINID, '$thisChain' ) && !strcasecmp( SEGID, '$thisSeg' )"]
                                    set thisSeq [getPDBSeqList $tt thisR1 thisRN]

                                    if {$kk} {puts -nonewline " "}
                                    puts "chain=$thisName seg=$thisSeg\[$thisR1:$thisRN\] $thisSeq"
                                    incr kk

                                    gdbFree $tt
                                   }
                               }
                           }
                       }

                    puts ""
                   }

        default    {puts "unknown"}
       }

    incr n
   } 
