#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]
  
if {[flagLoc $ARGV -help]} \
{
 puts stderr "Extract Descriptive Data from an NMRPipe format table:"
 puts stderr " -in   tabName \[test.tab\] Input Table."
 puts stderr " -parm pList   \[*\]        Parameters to Extract."
 puts stderr "List Text (REMARK, #, and DATA) Lines (Sets -parm None):"
 puts stderr " -text tList   \[None\]     Text Lines to Extract, First is 0."
 puts stderr "List DATA Lines (Sets -parm None):"
 puts stderr " -key  keyName \[*\]        Key Names for DATA to Extract."
 puts stderr " -data dList   \[None\]     DATA Lines to Extract, First is 0."
 puts stderr "List Entry (Row) Numbers in Sorted Table (Sets -parm None):"
 puts stderr " -sort sVar    \[None\]     Variable Name for Sort."
 puts stderr " -abs                     Sort by Absolute Value."
 puts stderr " -up                      Sort in Increasing Order (Default)."
 puts stderr " -down                    Sort in Decreasing Order."
 puts stderr "PDB Options:"
 puts stderr " -pdb                     Input is PDB Data (Default for .pdb)."
 puts stderr " -nopdb                   Input is NMRPipe-Format Table."
 puts stderr " -hetatm                  Interpret HETATMs as ATOMs."
 puts stderr " -seqText                 Extract DATA SEQUENCE Information."
 puts stderr " -seqInfo                 Extract First RESID and Single-Letter Sequence."
 puts stderr " -seqInfo3                Extract First RESID and Three-Letter Sequence."
 puts stderr "Notes:"
 puts stderr "1. Parameter Names, Output for \"-parm *\" in this order:"
 puts stderr "    varCount    Number of Variables."
 puts stderr "    entryCount  Number of Entries."
 puts stderr "    textCount   Number of Text Lines."
 puts stderr "    keyCount    Number of Different DATA Keywords."
 puts stderr "    varNames    List of All Variable Names."
 puts stderr "    keyNames    List of All DATA Keywords (Keys)."
 puts stderr "    keyLengths  List of Number of DATA Lines for Each Key."
 puts stderr "    keyLength   List of Number of DATA Lines For Specified Keys;"
 puts stderr "                (not output for -parm *)"
 puts stderr "Examples:"
 puts stderr " Find number of Variables in Table:"
 puts stderr "  getTabInfo.tcl -in csObs.tab -parm varCount"
 puts stderr " Find number of DATA SEQUENCE lines:"
 puts stderr "  getTabInfo.tcl -in csObs.tab -key SEQUENCE -parm keyLength"
 puts stderr " Get Contents of First DATA SEQUENCE line:"
 puts stderr "  getTabInfo.tcl -in csObs.tab -key SEQUENCE -data 0"
 puts stderr " Get Contents of All DATA SEQUENCE lines:"
 puts stderr "  getTabInfo.tcl -in csObs.tab -key SEQUENCE -data \"*\""
 exit 1  
}

set inName    test.tab
set tLocList  None
set dLocList  None
set sortVar   None
set pdbMode   nopdb
set hetMode   nohetatm
set pNameList *
set keyList   *

set textCount 0
set dataCount 0

set absFlag   0
set sortOrder increasing 

getArgD $ARGV -in   inName
getArgD $ARGV -sort sortVar

if {![strcasecmp [file extension $inName] .pdb]} {set pdbMode pdb}

if {[flagLoc $ARGV -pdb]}       {set pdbMode pdb}
if {[flagLoc $ARGV -nopdb]}     {set pdbMode nopdb}

if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}

if {[flagLoc $ARGV -up]}   {set sortOrder increasing}
if {[flagLoc $ARGV -down]} {set sortOrder decreasing}

if {[flagLoc $ARGV -abs]}  {set absFlag 1}

getListArgD $ARGV -text tLocList
getListArgD $ARGV -data dLocList
getListArgD $ARGV -key  keyList

if {[strcasecmp $sortVar None] || [strcasecmp $tLocList None] || \
     [strcasecmp $dLocList None] || [strcmp $keyList "*"]} \
   {
    set pNameList ""
   }

getListArgD $ARGV -parm pNameList

if {![strcasecmp $pNameList None]} {set pNameList ""}
if {![strcasecmp $tLocList  None]} {set tLocList  ""}
if {![strcasecmp $dLocList  None]} {set dLocList  ""}

if {![strcmp $pNameList "*"]} \
   {
    set pNameList \
     "varCount entryCount textCount keyCount varNames keyNames keyLengths"
   }

#
#------------------------------------------------------------------------------

proc eSort { loc1 loc2 } \
{
   global sortVar varType absFlag eList

   set v1 [gdbGet [lindex $eList $loc1] $sortVar]
   set v2 [gdbGet [lindex $eList $loc2] $sortVar]

   if {$absFlag} \
      {
       set v1 [expr abs($v1)]
       set v2 [expr abs($v2)]
      }

   if {![strcasecmp $varType text]} \
      {
       return [strcmp $v1 $v2]
      } \
   else \
      {
       if {$v1 > $v2} {return  1}
       if {$v1 < $v2} {return -1}
       return 0
      }
}

#
#------------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input File $inName"
    exit 1
   }

set inTab [gdbRead table -in $inName -$pdbMode -$hetMode]

if {[flagLoc $ARGV -seqInfo]} \
   {
    set r1 0
    set rN 0

    set seqList [getPDBSeqList $inTab r1 rN]
    puts "$r1 $seqList"

    exit 0
   }

if {[flagLoc $ARGV -seqInfo3]} \
   {
    set r1 0
    set rN 0

    set seqList [getPDBSeqList $inTab r1 rN]
    puts -nonewline "$r1"

    foreach resName $seqList {puts -nonewline " [getResName3 $resName]"}

    exit 0
   }

if {[flagLoc $ARGV -seqText]} \
   {
    set seqText [getPDBSeqStr $inTab]
    puts $seqText
    exit 0
   }

set textCount [getTabTextInfo $inTab textInfo]
set idList    [getTabKeyList $inTab]

if {![strcmp $keyList "*"]} {set keyList $idList}

if {![strcmp $tLocList "*"]} \
   {
    set tLocList ""
    for {set i 0} {$i < $textCount} {incr i} {lappend tLocList $i}
   }

#
# Named parameters:

foreach p $pNameList \
   {
    switch -exact [string tolower $p] \
       {
        textcount  {puts $textCount}
        keycount   {puts [llength $idList]}
        keynames   {puts $idList}

        keylength  {
                    set i 0

                    foreach id $keyList \
                       {
                        if {$i} {puts -nonewline " "}
                        puts -nonewline [getTabKeyLength $inTab $id]
                        incr i
                       }

                    puts ""
                   }
 
        keylengths {
                    set i 0

                    foreach id $idList \
                       {
                        if {$i} {puts -nonewline " "}
                        puts -nonewline [getTabKeyLength $inTab $id]
                        incr i
                       }

                    puts ""
                   }

        default    {puts [string trim [gdbGetInfo $inTab -parms $p]]}
       }
   }

#
# Text info:

foreach n $tLocList \
   {
    if {$n < 0 || $n > $textCount - 1} \
       {
        puts "(None)"
       } \
    else \
       {
        puts "$textInfo($n)"
       }
   }

#
# DATA info:

foreach id $keyList \
   {
    if {![strcmp $dLocList "*"]} \
       {
        set locList "" 
        set n [getTabKeyLength $inTab $id]
        for {set i 0} {$i < $n} {incr i} {lappend locList $i}
       } \
    else \
       {
        set locList $dLocList
       }

    foreach n $locList {puts [getTabKeyItem $inTab $id $n]} 
   }

#
# Sort info:

if {[strcasecmp $sortVar None]} \
   {
    set eList   [gdbGetInfo $inTab -parms entries]
    set vList   [gdbGetInfo $inTab -parms varNames]
    set eCount  [llength $eList]
    set locList ""
   
    for {set i 0} {$i < $eCount} {incr i} {lappend locList $i} 

    if {[lsearch -exact $vList $sortVar] >= 0} \
       {
        set vID     [gdbFindByName $inTab $sortVar]
        set varType [gdbGetInfo $vID -parms varType]
        set locList [lsort -command eSort -$sortOrder $locList]
       }

    foreach loc $locList {puts $loc}
   }

exit 0
