#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
 puts stderr "Extract and Print Interpolated Values from 1D Spectra:"
 puts stderr " -in   inList \[test.dat\] List of 1D Spectra Inputs."
 puts stderr " -x1   x1     \[0%\]       Start Coordinate (Pts, Hz, PPM, %)."
 puts stderr " -xn   xn     \[100%\]     End Coordinate (Pts, Hz, PPM, %)."
 puts stderr " -n    n      \[Auto\]     Size of Interpolated Output, Points."
 puts stderr " -xFmt xFmt   \[%.4e\]     Output Format for X-Coord."
 puts stderr " -yFmt yFmt   \[%.4e\]     Output Format for Intensity."
 puts stderr "Notes:"
 puts stderr " 1. Values for -x1 and -xn must have the same units (% ppm etc)."
 puts stderr " 2. Use -n Auto to set the output size to the largest of the"
 puts stderr "    input region sizes."
 puts stderr " 3. The value for -n can also be specified with a % label."
 exit 0
}

set inList test.dat
set x1     0%
set xn     100%
set xFmt   " % .4e"
set yFmt   " % .4e"
set nDest  Auto

getListArgD $ARGV -in inList

getArgD $ARGV -x1 x1
getArgD $ARGV -xn xn
getArgD $ARGV -n  nDest 

getArgD $ARGV -xFmt xFmt
getArgD $ARGV -yFmt yFmt

proc rdInterp1D { id inName } \
{
   global fdata xUnits x1 xn xList yList nMax

   set xList($id) ""
   set yList($id) ""
   set yNew($id)  ""
   set fdata($id) 0

   set error [rdFDATA $inName fdata($id)]
   if {$error} {return $error}

   set px1 [spec2pnt $fdata($id) CUR_XDIM $x1]
   set pxn [spec2pnt $fdata($id) CUR_XDIM $xn]

   if {$px1 > $pxn} \
      {
       set tmp $px1
       set px1 $pxn
       set pxn $tmp
      }

   set ix1 [expr int($px1 - 0.5) - 1]
   set ixn [expr int($pxn + 0.5) + 1]

   readROI -ndim 1 -roi 1 -in $inName -x X_AXIS $ix1 $ixn

   set sInfo   [getROIInfo -roi 1]
   set rdata1D [lindex $sInfo 0]
   set fdata1D [lindex $sInfo 1]
   set size    [lindex [lindex $sInfo 2] 0]

   set xList($id) ""
   set yList($id) ""

   for {set i 0} {$i < $size} {incr i} \
      {
       lappend xList($id) [pnt2spec $fdata1D CUR_XDIM [expr $i + 1] $xUnits]
       lappend yList($id) [getPtrVal $rdata1D $i]
      }

   set n [expr [llength $xList($id)] - 2]
   if {$n < 2} {set n 2}

   if {$nMax < $n} {set nMax $n}

   return 0
}

set x1Info  [spec2val $x1]
set xnInfo  [spec2val $xn]

set x1Val   [lindex $x1Info 0]
set xnVal   [lindex $xnInfo 0]

set x1Units [lindex $x1Info 1]
set xnUnits [lindex $xnInfo 1]

set xUnits  $x1Units

if {[strcasecmp $x1Units $xnUnits]} \
   {
    puts stderr "Interp1D Error: Units for -x1 and -xn Must be the Same."
    exit 1
   }

foreach inName $inList \
   {
    if {![file exists $inName]} \
       {
        puts stderr "Interp1D Error Finding 1D Input $inName"
        exit 1
       }
   }

set inCount [llength $inList]
set xNew    ""
set nMax    0

for {set id 1} {$id <= $inCount} {incr id} \
   {
    set inName [lindex $inList [expr $id - 1]]
    set error  [rdInterp1D $id $inName]

    if {$error} \
       {
        puts stderr "Interp1D Error Reading 1D Input $inName"
        exit 1
       }
   }

if {![strcasecmp $nDest Auto]} \
   {
    set nDest $nMax
   } \
else \
   {
    set nInfo  [spec2val $nDest]
    set nVal   [lindex $nInfo 0]
    set nUnits [lindex $nInfo 1]

    if {![strcmp $nUnits "%"]} \
       {
        set nDest [expr int( 0.5 + $nVal*$nMax/100.0 )]
       }
   }
 
if {!$nDest} \
   {
    set step 0.0
   } \
else \
   {
    set step [expr ($xnVal - $x1Val)/double($nDest - 1)]
   }

set xNew [getIota $x1Val $step $nDest]

for {set id 1} {$id <= $inCount} {incr id} \
   {
    set yNew($id) [linterp -x $xList($id) -y $yList($id) -xNew $xNew]
   }

for {set i 0} {$i < $nDest} {incr i} \
   {
    puts -nonewline [format $xFmt [lindex $xNew $i]]
 
    for {set id 1} {$id <= $inCount} {incr id} \
       {
        puts -nonewline " [format $yFmt [lindex $yNew($id) $i]]"
       }

    puts ""
   }

exit 0
