#!/bin/csh

#
# Isolate a region from a spectrum, and inverse process to create time-domain data.
# frank.delaglio@nist.gov 7/10/2024

if (`flagLoc $argv -help`) then
   echo "Isolate and Inverse-Process a 1D or 2D Spectral Region:"
   echo "frank.delaglio@nist.gov 7/10/2024"
   echo " -in   inName   [test.ft1]  Input Spectrum."
   echo " -mask maskName [mask.ft1]  Output Mask."
   echo " -ft   ftName   [reg.ft1]   Output Isolated Region."
   echo " -fid  fidName  [reg.fid]   Output Inverse-Processed Region."
   echo " -mode mode     [X]         Isolation Mode: X Y XY."
   echo " -ndim dimCount [Auto]      Number of Dimensions in Input."
   echo "Region Specification (Use -x.. -y..):"
   echo " -x1   x1       [10%]       X-Axis Region Start (1D, 2D)."
   echo " -xn   xn       [90%]       X-Axis Region End (1D, 2D)."
   echo " -xrw  xrw      [5%]        X-Axis Ramp Width (Pts Hz ppm %)."
   echo "Inverse Processing:"
   echo " -invApod                   Apply Inverse Apodization (Default)."
   echo " -noinvApod                 No Inverse Apodization."
   echo " -invC                      Apply Inverse First-Point Scaling (Default)."
   echo " -noinvC                    No Inverse First-Point Scaling."
   echo "Notes:"
   echo " 1. The region is isolated by applying a raised cosine filter"
   echo "    that sets the points outside the region to zero."
   echo " 2. Mode X isolates a band selected from the X-Axis, and inverse"
   echo "    transforms the X-Axis."
   echo " 3. Mode Y isolates a band selected from the Y-Axis, and inverse"
   echo "    transforms the Y-Axis."
   echo " 4. Mode XY isolates a 2D region, and inverse transforms both the"
   echo "    X-Axis and the Y-Axis."
   exit 0
endif

set inName    = (`getArgD $argv -in   test.ft1`)
set dimCount  = (`getArgD $argv -ndim Auto`)

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
   echo "IsolateRegion. Error finding input file $inName ($thisName)."
   exit 1
endif

set specInfo  = (`getSpecSizes.com -in $inName`)
set dimCount  = (`getNamedItem trueDimCount $specInfo`)

set maskName  = (`getArgD $argv -mask mask.ft1`)
set ftName    = (`getArgD $argv -ft   region.ft1`)
set fidName   = (`getArgD $argv -fid  region.fid`)
set isoMode   = (`getArgD $argv -mode X`)

set x1        = (`getArgD $argv -x1   10%`)
set xn        = (`getArgD $argv -xn   90%`)
set xrw       = (`getArgD $argv -xrw  5%`)

set y1        = (`getArgD $argv -y1   10%`)
set yn        = (`getArgD $argv -yn   90%`)
set yrw       = (`getArgD $argv -yrw  5%`)

set z1        = (`getArgD $argv -z1   10%`)
set zn        = (`getArgD $argv -zn   90%`)
set zrw       = (`getArgD $argv -zrw  5%`)

@ invApodFlag = 1
@ invCFlag    = 1

if (`flagLoc $argv -noinvApod`) then
   @ invApodFlag = 0
endif

if (`flagLoc $argv -noinvC`) then
   @ invCFlag = 0
endif

#-----------------------------------------------------------------------------------------

#
# The region will be isolated by a band with cosine-square tails on either side.
# The user specifies the band region (x1 xn) and the width of the tails (xrw).
# Convert these to common units (pts).

if (`string csame $isoMode X`) then
   set x1Pts  = (`spec2pnt  $inName X $x1`)
   set xnPts  = (`spec2pnt  $inName X $xn`)
   set xrwPts = (`spec2pntW $inName X $xrw`)

   set x1Pts  = (`IMATH "0.5 + $x1Pts"`)
   set xnPts  = (`IMATH "0.5 + $xnPts"`)
   set xrwPts = (`IMATH "0.5 + $xrwPts"`)

   if ($x1Pts > $xnPts) then
      set itmp  = $x1Pts
      set x1Pts = $xnPts
      set xnPts = $itmp
   endif

   set xFirstPts = (`IMATH "$x1Pts - $xrwPts"`)
   set xLastPts  = (`IMATH "$xnPts + $xrwPts"`)

   echo "IsolateRegion. Input Spectrum:               $inName"
   echo "IsolateRegion. Isolation Mode:               $isoMode"
   echo "IsolateRegion. Requested Region:             $x1 $xn"
   echo "IsolateRegion. Ramp Width:                   $xrw"
   echo "IsolateRegion. Region Locations, Pts:        $xFirstPts $x1Pts $xnPts $xLastPts"
   echo "IsolateRegion. Output Region Mask:           $maskName"
   echo "IsolateRegion. Output Isolated Region:       $ftName"
   echo "IsolateRegion. Output Inverse Transform:     $fidName"

   if ($dimCount == 1) then
      normSpec.tcl -in $inName -out $maskName -noverb \
                   -prog set region 0%         100%      to 0.0 then \
                         set region $xFirstPts $xLastPts to 1.0 then \
                         rampUp   region $xFirstPts $x1Pts      then \
                         rampDown region $xnPts     $xLastPts
   else if ($dimCount == 2) then
      normSpec.tcl -in $inName -out $maskName -noverb \
                   -prog set region 0%         100%      0% 100% to 1.0 then \
                         plateau x1=$xFirstPts,x2=$x1Pts,x3=$xnPts,x4=$xLastPts,y1=0%,y2=0%,y3=100%,y4=100%
   else
      echo ""IsolateRegion. Error: unsupported dimension count $dimCount"
      exit 1
   endif
endif

addNMR -in1 $inName -in2 $maskName -out $ftName -mult

if ($invApodFlag) then
   if ($invCFlag) then
      set invApodCmnd = (SP -inv -hdr) 
   else
      set invApodCmnd = (SP -inv -hdr -c 1.0) 
   endif
else
   if ($invCFlag) then
      set cVal = (`getParm -in $ftName -parm NDC1 -dim CUR_XDIM`)
      set c    = (`MATH "$cVal + 1.0"`)

      set invApodCmnd = (MULT -xn 1 -c $c -inv)
   else
      set invApodCmnd = (NULL)
   endif
endif

if (`string notcsame $fidName None`) then
   nmrPipe -in $ftName \
   | nmrPipe -fn HT \
   | nmrPipe -fn FT -inv \
   | nmrPipe -fn $invApodCmnd \
   | nmrPipe -fn ZF -inv \
      -out $fidName -ov
endif

exit 0

