#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# jcamp.tcl: modules to read/write Bruker JCAMP parameter files.
#
#  Inteded for user:
#
#  rdJCAMP:       main module for reading and returning parameter values.
#  wrJCAMP:       main module for writing parameter values.
#  isJParm:       returns 1 if the given parameter is an original Bruker one.
#
#  Internal functions:
#
#  getJParmKey:    returns keyword describing line type, DATA, HDR, or REMARK.
#  getJParmKeyAll: returns keyword describing line type, PRIVATE, SPECIFIC, GENERIC, or REMARK.
#  getJParmName:   returns parameter name of the given line.
#  getJParmVal:    returns the parameter value following the recent name.
#  getJParmArray:  returns the parameter array following the recent name.

#--------------------------------------------------------------------------------------------
# rdJCAMP
#
# Read parameters from JCAMP parameter list to a TCL array:
#   Entries in the array (e.g. "bruk(TD)") contain the parameter values.
#
#   Types in the array (e.g. "bruk(TD.type)") contain the parameter type. 
#
#   Label types in the array (e.g. "bruk(TD.labelType)") contain the label type
#   ##$ private, ## generic, ##. specific.
#
#   The tallies in the array (e.g. bruk(TD.count)) show the number of times 
#   a parameter was found.
#
# Returns number of parameters read.
# Returns -1 on failure.
#
# JCAMP ## entries are appended to .hdr    parameter in array.
# JCAMP $$ entries are appended to .remark parameter in array
#
# Arguments:
#  inName:  file name of JCAMP input, or "stdin".
#  parmPtr: name of TCL array for storing JCAMP parameters.

proc rdJCAMP { inName parmPtr } \
{
    upvar $parmPtr parmList

#
# Initialize.
# Open the input file.

    pushJCAMPLine ""

    set error     0
    set parmCount 0

    set parmType  NONE
    set labelType NONE

    set parmList(.hdr)      ""
    set parmList(.remark)   ""
    set parmList(.parmList) ""

    switch $inName \
      {
       stdin   {set inUnit stdin}
       default {set error [catch {set inUnit [open $inName r]}]}
      }

    if {$error} \
       {
        puts stderr "RdJCAMP Error opening input file $inName"
        return -1
       }

#
# For each line in the input:
#  If it is a parameter line:
#   Extract the parameter name and value
#   Store the name/value pair.

    while {[getsJCAMP $inUnit line] >= 0} \
       {
        switch [getJParmKey $line] \
           {
            DATA    {
                     set  parmName  [getJParmName $line]
                     set  parmVal   [getJParmVal  $line $inUnit parmType]
                     incr parmCount

                     if {[lsearch -exact $parmList(.parmList) $parmName] < 0} \
                        {
                         lappend parmList(.parmList) $parmName
                        }

                     if {![info exists parmList($parmName)]} \
                        {
                         set parmList($parmName)           ""
                         set parmList($parmName.type)      None 
                         set parmList($parmName.labelType) None 
                         set parmList($parmName.count)     0 
                        }

                     set parmList($parmName)      $parmVal
                     set parmList($parmName.type) $parmType

                     incr parmList($parmName.count)
                    }

            HDR     { lappend parmList(.hdr)    $line }
            REMARK  { lappend parmList(.remark) $line } 
            default {}
           }
       }

    close $inUnit

    return $parmCount 
}

proc rdJCAMPAll { inName parmPtr privateFlag genericFlag specificFlag } \
{
    upvar $parmPtr parmList

#
# Initialize.
# Open the input file.

    pushJCAMPLine ""

    set error     0
    set parmCount 0

    set parmType  NONE
    set labelType NONE

    set parmList(.hdr)      ""
    set parmList(.remark)   ""
    set parmList(.parmList) ""

    switch $inName \
      {
       stdin   {set inUnit stdin}
       default {set error [catch {set inUnit [open $inName r]}]}
      }

    if {$error} \
       {
        puts stderr "RdJCAMP Error opening input file $inName"
        return -1
       }

#
# For each line in the input:
#  If it is a parameter line:
#   Extract the parameter name and value
#   Store the name/value pair.

    while {[getsJCAMP $inUnit line] >= 0} \
       {
        set thisKey [getJParmKeyAll $line]

        if {![strcasecmp $thisKey SPECIFIC] && !$specificFlag} continue
        if {![strcasecmp $thisKey GENERIC]  && !$genericFlag}  continue 
        if {![strcasecmp $thisKey PRIVATE]  && !$privateFlag}  continue 

        switch $thisKey \
           {
            SPECIFIC -
            GENERIC  -
            PRIVATE {
                     set  parmName  [getJParmName $line]
                     set  parmVal   [getJParmVal  $line $inUnit parmType]
                     incr parmCount

                     if {[lsearch -exact $parmList(.parmList) $parmName] < 0} \
                        {
                         lappend parmList(.parmList) $parmName
                        }

                     if {![info exists parmList($parmName)]} \
                        {
                         set parmList($parmName)           ""
                         set parmList($parmName.type)      None 
                         set parmList($parmName.labelType) $thisKey
                         set parmList($parmName.count)     0 
                        }

                     set parmList($parmName)      $parmVal
                     set parmList($parmName.type) $parmType

                     incr parmList($parmName.count)
                    }

            HDR     { lappend parmList(.hdr)    $line }
            REMARK  { lappend parmList(.remark) $line } 
            default {}
           }
       }

    close $inUnit

    return $parmCount 
}

#-------------------------------------------------------------------------
# wrJCAMP
#
# Write parameters from a TCL array to a JCAMP parameter list.
# Returns number of parameters written.
# Returns -1 on failure.
#
#  outName: file name of JCAMP output, or "stdout"/"stderr".
#  parmPtr: name of TCL array containing JCAMP parameters.
#  hdrMode: either ORIG_HDR_YES or ORIG_HDR_NO (replace hdr/remarks)


proc wrJCAMP { outName parmPtr hdrMode } \
{
    upvar $parmPtr parmList

#
# Initialize.
# Open the output file.

    set error     0
    set parmCount 0
    set idList    ""

    switch $outName \
      {
       stdout  {set outUnit stdout}
       stderr  {set outUnit stderr}
       default {set error [catch {set outUnit [open $outName w]}]}
      }

    if {$error} \
       {
        puts stderr "WrJCAMP Error opening output file $outName"
        return -1
       }

#
# Prepare to write parameters in alphabetical order.

    set id [array startsearch parmList]

    while {[array anymore parmList $id]} \
       {
        if {-1 == [string first "." $id]} \
           {
            lappend idList [array nextelement parmList $id]
           }
       }

    array donesearch parmList $id

    set idList [lsort $idList]

#
# Create a header, or use existing one:

    if {![string compare $hdrMode ORIG_HDR_NO]} \
       {
        puts $outUnit "##TITLE= Parameter File, NMRPipe TCL"
        puts $outUnit "##JCAMPDX= 5.0"
        puts $outUnit "##DATATYPE= Parameter Values"
        puts $outUnit "##ORIGIN= NIH Laboratory of Chemical Physics"

        puts $outUnit [format "\$\$ %s"       [getDate]]
        puts $outUnit [format "\$\$ File: %s" $outName]
       } \
    elseif {![string compare $hdrMode ORIG_HDR_YES]} \
       {
        foreach i $parmList(.hdr) \
           {
            puts $outUnit $i
           }

        foreach i $parmList(.remark) \
           {
            puts $outUnit $i
           }
       }
 
#
# For each parameter in the alphabetically sorted list:
#  Write the JCAMP parameter/value output.
#  Write the END.

    foreach i $idList \
       {
        putJParm parmList $i $outUnit
       }

    puts $outUnit "##END=  "

    return $parmCount 
}

#
# Return 1 if the given parameter name is an original Bruker
# JCAMP parameter, rather than a "synthetic" one created by these
# modules, for instance "TD" (original) vs "TD.type" (synthetic).

proc isJParm { parmName } \
{
   if {-1 == [string first "." $parmName]} \
      {
       return 1
      }

   return 0
}

proc getJParmKey { line } \
{
   if {-1 != [string first "##$" $line]} \
      {
       if {-1 != [string first "=" $line]} \
          {
           return DATA
          } \
       else \
          {
           return ITEM
          }
      }

   if {-1 != [string first "##"  $line]} {return HDR}
   if {-1 != [string first "$$"  $line]} {return REMARK}

   return NONE 
}

proc getJParmKeyAll { line } \
{
   set s4 [string range [string trim $line] 0 4]
   if {![strcasecmp $s4 "##END"]} {return END}

   set s2 [string range [string trim $line] 0 1]
   if {![strcasecmp $s2 "$$"]} {return REMARK}

   set s3 [string range [string trim $line] 0 2]

   if {![strcasecmp $s3 "##$"]} \
      {
       if {-1 != [string first "=" $line]} \
          {
           return PRIVATE
          } \
       else \
          {
           return ITEM
          }
      }

   if {![strcasecmp $s3 "##."]} \
      {
       if {-1 != [string first "=" $line]} \
          {
           return SPECIFIC 
          } \
       else \
          {
           return ITEM
          }
      }

   if {![strcasecmp $s2 "##"]} {return GENERIC}

   return NONE
}

proc getJParmName { line } \
{
    set parmName NONE

    set line [string trim $line]

    set s3 [string range $line 0 2]
    set s2 [string range $line 0 1]

    if {![strcmp $s3 "##$"] || ![strcmp $s3 "##."]} \
       {
        set line [string trim [string range $line 3 end]]
       } \
    elseif {![strcmp $s2 "##"]} \
       {
        set line [string trim [string range $line 2 end]]
       }

    set loc [string first "=" $line]

    if {$loc > -1} \
       {
        set parmName [string trim [string range $line 0 [expr $loc - 1]]]
       }

    set n [llength $parmName]

    if {$n > 1} \
       {
        if {$loc > 0 && ![strcasecmp [lindex $parmName 0] DATA]} \
           { 
            if {![strcasecmp [lindex $parmName 1] TABLE]} \
               {
                set parmName [string trim [string range $line [expr $loc + 1] end]]

                if {[string first "(X++(R..R))" $parmName] > -1} {set parmName "R..R"}
                if {[string first "(X++(I..I))" $parmName] > -1} {set parmName "I..I"}
               } \
            else \
               {                
                set parmName [join $parmName "_"]
               }
           } \
        else \
           {
            set parmName [join $parmName "_"]
           }
       }

    return $parmName
}

proc getJParmVal { line inUnit parmTypePtr } \
{
    upvar $parmTypePtr  parmType 

    set loc     [string first "=" $line]
    set parmVal None

    set parmVal [string range $line [expr $loc + 1] end] 
    set loc     [string first "$$" $parmVal]

    if {$loc > -1} {set parmVal [string range $parmVal 0 [expr $loc - 1]]}

    set parmVal [string trim $parmVal]

    if {"<" == [string index $parmVal 0]} \
       {
        set parmType STRING
        set parmVal [string trim $parmVal "<>"]
       } \
    else \
       {
        if {[llength $parmVal] == 1 && [string is double $parmVal]}  \
           {
            set parmType NUMBER 
           } \
        else \
           {
            if {[string first DATA $line] > 0 && [string first TABLE $line] > 0} \
               {
                set parmType TABLE
               } \
            else \
               {
                set parmType VALUE 
               }
           }
       }

    if {![strcasecmp $parmType TABLE]} \
       {
        set parmVal [getJParmTable $inUnit]
       } \
    else \
       {
        set loc [string first "(" $parmVal]

        if {$loc != -1} \
           {
            set parmVal [getJParmArray $parmVal $inUnit]
           }
       }

    return $parmVal
}

proc getJParmTable { inUnit } \
{
   set parmval ""

   while {[getsJCAMP $inUnit line] >= 0} \
      {
       set line [string trim $line]
       set s2   [string range $line 0 1]

       if {![strcmp $s2 "##"] || ![strcmp $s2 "$$"]} \
          {
           pushJCAMPLine $line
           break
          }

       foreach v [lrange $line 1 end] {lappend parmVal $v}
      }

   return $parmVal
}

proc getJParmArray { arrayLims inUnit } \
{
   set loc1 [string first ".." $arrayLims]
   set loc2 [string first ")"  $arrayLims]

   if {$loc1 < 0} \
      {
       set parmVal $arrayLims

       if {$loc2 > 0} \
          {
           return [string trim $parmVal]
          }

       set parmVal [string trim $arrayLims]

       while {[getsJCAMP $inUnit line] >= 0} \
          {
           set line [string trim $line]
           set s2   [string range $line 0 1]

           if {![strcmp $s2 "##"] || ![strcmp $s2 "$$"]} \
              {
               pushJCAMPLine $line
               break
              }

           append parmVal " [string trim $line]"
          }

       return [string trim $parmVal]
      }

   set loc1 0
   set loc2 0

   scan [string trim $arrayLims "()"] "%d..%d" loc1 loc2 

   set count [expr 1 + $loc2 - $loc1]
   set parmVal ""

   set loc   [expr 1 + [string first ")" $arrayLims]]
   set line  [string range $arrayLims $loc end]

   if {[llength $line] > 0} \
      {
       set line [string trim $line]
       append parmVal " $line"
       incr count -[llength [split $line]]
      }

   while {$count > 0} \
      {
       if {[gets $inUnit line] < 0} {return $parmVal}

       set line [string trim $line]
       append parmVal " $line"
       incr count -[llength [split $line]]
      }

   return [split [string trim $parmVal]]
}


proc putJParm { parmPtr id outUnit } \
{
    upvar $parmPtr parmList

    set length   [llength $parmList($id)]
    set parmType NONE
    set count    0

    if {-1 != [string first "." $id]} \
       {
        return 0
       }

    set error [catch [set parmType $parmList($id.type)]]

    if {![string compare STRING $parmType]} \
       {
        puts $outUnit [format "##\$%s= <%s>" $id $parmList($id)]
       } \
    elseif {$length < 2} \
       {
        puts $outUnit [format "##\$%s= %s " $id $parmList($id)]
       } \
    else \
       {
        puts $outUnit [format "##\$%s= (0..%d)" $id [expr $length - 1]]

        foreach i $parmList($id) \
           {
            puts -nonewline $outUnit  "$i "
            incr count [string length "$i "]

            if {$count > 70} \
               {
                puts $outUnit ""
                set count     0
               }
            }

         if {$count} \
            {
             puts $outUnit ""
             set count     0
            }
       }

    return 0
}

proc pushJCAMPLine { line } \
{
   global prevJCAMPLine 

   set prevJCAMPLine $line
}

proc getsJCAMP { inUnit linePtr } \
{
   upvar  $linePtr line

   global prevJCAMPLine

   set n [string length $prevJCAMPLine]

   if {$n == 0} \
      {
       set thisLine ""

       set n    [gets $inUnit thisLine]
       set line $thisLine
      } \
   else \
      {
       set line $prevJCAMPLine
      }

   set prevJCAMPLine ""

   return $n
}

