
#
# Modules for interpolation of numerical lists.

proc linterp args \
{
   set ARGV  [concat listInterp $args]
   set ARGC  [llength $ARGV]

   if {[flagLoc $ARGV -help]} \
      {
       puts "Interpolation from of lists X and Y Values:"
       puts " -x    xList     Input List of X Values."
       puts " -y    yList     Input List of Y Values."
       puts "Output Parameters (Use Only One):"
       puts " -xNew xNewList  List of X Points for Interpolated Y Result."
       puts " -n    nFinal    Number of Equally-Spaced X Points for Y Result."
       puts "Other Options:"
       puts " -prep           Adjust Input for Sort Order and Size (Default)."
       puts " -noprep         Do Not Adjust Input Lists."
       puts " -xPtr tclVar    TCL Variable for Returning Interpolated X Values."
       puts "Notes:"
       puts " 1. The '-prep' mode establishes the following requirements:" 
       puts "    * Input X and Y lists should have the same size."
       puts "    * X points should be in ascending order."
       puts "    * No X point is repeated."
       puts " 2. When the '-n' option is used, the evenly spaced X points"
       puts "    created will start at the minimum value in the X input"
       puts "    list and end at the maximum value in the X input list." 
       return "" 
      }

   set x     ""
   set y     ""

   set xNew  ""
   set yNew  ""
   set xPtr  None

   set nX       0
   set nY       0
   set nNew     0 
   set nFinal   0 
   set prepFlag 1

   set mode "linear"

   getArgD $ARGV -x    x
   getArgD $ARGV -y    y
   getArgD $ARGV -xNew xNew
   getArgD $ARGV -xPtr xPtr
   getArgD $ARGV -n    nFinal

   if {[flagLoc $ARGV -prep]}   {set prepFlag 0}
   if {[flagLoc $ARGV -noprep]} {set prepFlag 1}

#
#---------------------------------------------------------------------------

   if {[strcasecmp $xPtr None]} {upvar $xPtr xUser}

   set nNew  [llength $xNew] 
   set xUser "" 

   if {$nNew < 1} \
      {
       if {$nFinal < 1} {return $yNew}

       prepInterpX x xNew $nFinal

       set nNew $nFinal
      }

   if {[strcasecmp $xPtr None]} {set xUser $xNew}
 
   if {$prepFlag} {prepInterpXY x y}

   set n [llength $x]

   if {$n < 1} \
      {
       set x 0
       set y 0
       set n 1
      }

   if {$n == 1} \
      {
       for {set i 0} {$i < $nNew} {incr i} {lappend yNew $y}        
      } \
   else \
      {
       foreach xI $xNew \
          {
           lappend yNew [findInterpY $x $y $xI]
          }
      }

   return $yNew
} 

proc findInterpY { x y xI } \
{
   set n [llength $x]

   if {!$n} {return 0}

   set j [findSortedXLoc $x $xI]

   if {$j < 0}      {set j [expr $j + $n]} 
   if {$j > $n - 1} {set j [expr $j - $n]} 

   set k [expr $j + 1]

   if {$k < 0}      {set k [expr $k + $n]} 
   if {$k > $n - 1} {set k [expr $k - $n]} 

   set xJ [lindex $x $j]
   set xK [lindex $x $k]

   if {$xJ == $xK} \
      {
       set yI [lindex $y $j]
      } \
   else \
      {
       set yJ [lindex $y $j]
       set yK [lindex $y $k]

       set yI [expr $yJ + double($yK - $yJ)*($xI - $xJ)/($xK - $xJ)]
      }    

   return $yI
}

proc findSortedXLoc { x xI } \
{
   set n [llength $x]

   if {!$n} {return 0} 

   set i 0

   foreach xJ $x \
      {
       if {$xJ > $xI} {return [expr $i - 1]}
       incr i 
      }

   return $n
}

proc prepInterpX { xPtr xNewPtr nNew } \
{
   upvar $xPtr    x
   upvar $xNewPtr xNew

   set xMin [getMin $x]
   set xMax [getMax $x]

   if {$nNew > 1} \
      {
       set step [expr double($xMax - $xMin)/($nNew - 1)]
      } \
   else \
      {
       set step 0
      }

   set xNew [getIota $xMin $step $nNew]
}

proc prepInterpXY { xPtr yPtr } \
{
   upvar $xPtr x
   upvar $yPtr y

   set nX [llength $x]
   set nY [llength $y]
   set n  [getMin "$nX $nY"]

   if {$nX != $nY} \
      {
       if {$nX != $n} {set x [lrange $x 0 [expr $n - 1]]}
       if {$nY != $n} {set y [lrange $y 0 [expr $n - 1]]}
      }

   set n 0

   if {![isAscending $x]} \
      {
       if {[isDescending $x]} \
          {
           set x [getReverse $x]
           set y [getReverse $y]
          } \
       else \
          {
           getSortedXY x y
          }
      }

   if {![noSameNeighbors $x]} \
      {
       set n [getMonotonicXY x y]
      }

    return $n
}

proc getIota { start step n } \
{
   set iList ""

   if {$n < 1} {return $iList}

   for {set i 0} {$i < $n} {incr i} \
      {
       lappend iList [expr $start + $step*$i]
      }

   return $iList
}

proc getMonotonicXY { xPtr yPtr } \
{
   upvar $xPtr x 
   upvar $yPtr y 

   set nX [llength $x]
   set nY [llength $y]
   set n  [getMin "$nX $nY"]

   if {$n < 2} return

   set xNew ""
   set yNew ""

   set j 0

   for {set i 0} {$i < $n} {incr i} \
      {
       set xI [lindex $x $i] 
       set xJ [lindex $x [expr $i + 1]] 

       if {$i == $n - 1 || $xI != xJ} \
          {
           lappend xNew $xI
           lappend yNew [lindex $ty $i]
          } \
       else \
          {
           incr j
          }
      }

   set x $xNew
   set y $yNew

   return j
}

proc isAscending { rList } \
{
   set n [llength $rList]
   if {$n < 2} {return 0}

   set rPrev [lindex $rList 0]
   set i 0

   foreach r $rList \
      {
       if {$i && $r < $rPrev} {return 0}
       set rPrev $r
       incr i
      }

   return 1
}

proc isDescending { rList } \
{
   set n [llength $rList]
   if {$n < 2} {return 0}

   set rPrev [lindex $rList 0]
   set i 0

   foreach r $rList \
      {
       if {$i && $r > $rPrev} {return 0}
       set rPrev $r
       incr i
      }

   return 1
}

proc noSameNeighbors { rList } \
{
   set n [llength $rList]
   if {$n < 2} {return 0}

   set rPrev [lindex $rList 0]
   set i 0

   foreach r $rList \
      {
       if {$i && $r == $rPrev} {return 0}
       set rPrev $r
       incr i
      }

   return 1
}

