#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Prepare a Peak Table for Local Fitting of Peaks:"
    puts " -in       inName   \[master.tab\] Input Peak Table."
    puts " -out      outName  \[local.tab\]  Output Peak Table."
    puts " -ft       ftName   \[test.ft2\]   Reference Spectrum."
    puts " -ndim     ndim     \[Auto\]       Dimension Count of Peak Table."
    puts " -verb                           Verbose Mode ON (Default)."
    puts " -noverb                         Verbose Mode OFF."
    puts "Local Region (Use -x... -y...) (ppm, Hz, pts or %), or Keyword Auto:"
    puts " -dx       dx       \[Auto\]       X-Axis +/- Distance for Local Region."
    puts " -wx       wx       \[Auto\]       X-Axis Weighting Factor for Distance."
    puts " -xwVar    xwVar    \[XW\]         Table Variable for X-Axis Linewidth." 
    puts " -dxMult   dxMult   \[4.0\]        Linewidth Multiplier for -dx Auto Mode."
    puts "Grouping of Peaks:"
    puts " -useClust                       Only Group Peaks with the Same CLUSTID."
    puts " -nouseClust                     Group Peaks According to Distance Only (Default)."
    puts " -maxClust maxClust \[10\]       Maximum Number of Peaks in Cluster."
    puts " -maxPeaks maxPeaks \[10\]       Maximum Number of Peaks Fit Simultaneously."
    puts "Notes:"
    puts " 1. This utility intended to help fit complex spectra via the autoFit.tcl"
    puts "    spectral lineshape fitting module. By default, autoFit.tcl simultaneously"
    puts "    fits peaks belonging to the same continuously overlapping group, as defined"
    puts "    by the peak table CLUSTID column. This doesn't work well when the clusters"
    puts "    contain too many peaks, so this script offers an alternative: each peak"
    puts "    will be fit with a collection of its nearest neighbors, but only the"
    puts "    parameters for the given peak will be kept."
    exit 0
   }

set inName     master.tab
set outName    local.tab
set ftName     test.ft2
set dimCount   Auto

set maxClust   10 
set maxPeaks   10 
set useClust   0
set verbose    1

set dx      Auto
set dy      Auto
set dz      Auto
set da      Auto

set wx      Auto
set wy      Auto 
set wz      Auto 
set wa      Auto 

set xwVar   XW
set ywVar   YW
set zwVar   ZW
set awVar   AW

set dxMult  4.0
set dyMult  4.0
set dzMult  4.0
set daMult  4.0

getArgD $ARGV -in    inName
getArgD $ARGV -out   outName
getArgD $ARGV -ft    ftName

getArgD $ARGV -maxClust maxClust
getArgD $ARGV -maxPeaks maxPeaks
getArgD $ARGV -ndim     dimCount

getArgD $ARGV -dx  dx
getArgD $ARGV -dy  dy
getArgD $ARGV -dz  dz
getArgD $ARGV -da  da

getArgD $ARGV -wx  wx
getArgD $ARGV -wy  wy
getArgD $ARGV -wz  wz
getArgD $ARGV -wa  wa

getArgD $ARGV -xwVar xwVar
getArgD $ARGV -ywVar ywVar
getArgD $ARGV -zwVar zwVar
getArgD $ARGV -awVar awVar

getArgD $ARGV -dxMult dxMult 
getArgD $ARGV -dyMult dyMult 
getArgD $ARGV -dzMult dzMult 
getArgD $ARGV -daMult daMult 

if {[flagLoc $ARGV -useClust]}   {set useClust 1}
if {[flagLoc $ARGV -nouseClust]} {set useClust 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
#-------------------------------------------------------------------------------

proc printEntry { e id clustID memCnt } \
{
   global varList fmtList outUnit

   set i 0

   foreach var $varList \
      {
       set fmt [lindex $fmtList $i]

       if {![strcasecmp $var INDEX]} \
          {
           set thisVal $id
          } \
       elseif {![strcasecmp $var CLUSTID]} \
          {
           set thisVal $clustID
          } \
       elseif {![strcasecmp $var MEMCNT]} \
          {
           set thisVal $memCnt
          } \
       else \
          {
           set thisVal [gdbGet $e $var]
          }

       puts -nonewline $outUnit [format " $fmt" $thisVal]

       incr i
      }

   puts $outUnit ""
}

if {![file exists $inName]} \
   {
    puts stderr "localFit. Error finding input peak table $inName"
    exit 1
   }

set thisName [format $ftName 1 1]

if {![file exists $thisName]} \
   {
    puts stderr "localFit. Error finding input spectrum $ftName ($thisName)"
    exit 1
   }

set t [gdbRead table -in $inName]

set eList     [gdbGetInfo $t -parms entries]
set vPtrList  [gdbGetInfo $t -parms vars]
set varList   ""
set fmtList   ""

foreach v $vPtrList \
   {
    lappend varList [gdbGetInfo $v -parms name]
    lappend fmtList [gdbGetInfo $v -parms varFmt]
   }

if {![strcasecmp $dimCount Auto]} \
   {
    if {[lsearch -exact $varList $xwVar] >= 0} {set dimCount 1}
    if {[lsearch -exact $varList $ywVar] >= 0} {set dimCount 2}
    if {[lsearch -exact $varList $zwVar] >= 0} {set dimCount 3}
    if {[lsearch -exact $varList $awVar] >= 0} {set dimCount 4}
   }

set swx 1.0
set swy 1.0
set swz 1.0
set swa 1.0

set error [rdFDATA $ftName fdata]

if {[strcasecmp $dx Auto]}  {set dx [expr int(0.5+[spec2pntW $fdata CUR_XDIM $dx])]}
if {[strcasecmp $dy Auto]}  {set dy [expr int(0.5+[spec2pntW $fdata CUR_YDIM $dy])]}
if {[strcasecmp $dz Auto]}  {set dz [expr int(0.5+[spec2pntW $fdata CUR_ZDIM $dz])]}
if {[strcasecmp $da Auto]}  {set da [expr int(0.5+[spec2pntW $fdata CUR_ADIM $da])]}

if {$dimCount >= 1} {set swx [getParm $fdata NDSW CUR_XDIM]}
if {$dimCount >= 2} {set swy [getParm $fdata NDSW CUR_YDIM]}
if {$dimCount >= 3} {set swz [getParm $fdata NDSW CUR_ZDIM]}
if {$dimCount >= 4} {set swa [getParm $fdata NDSW CUR_ADIM]}

if {![strcasecmp $wx Auto]} {set wx 1.0}
if {![strcasecmp $wy Auto]} {set wy [safeDiv $swy $swx]}
if {![strcasecmp $wz Auto]} {set wz [safeDiv $swz $swx]}
if {![strcasecmp $wa Auto]} {set wa [safeDiv $swa $swx]}

if {![strcasecmp $outName stdout]} \
   {
    set outUnit stdout
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "localFit. Error opening output file $outName"
        exit 1
       }
   }

if {$verbose} \
   {
    puts ""
    puts "localFit. Dimensions: $dimCount Original Peak Count: [llength $eList]"
   }

puts $outUnit ""
puts $outUnit "#"
puts $outUnit "# [lindex $ARGV 0] [lrange $ARGV 3 end]"
puts $outUnit "#"

set fmt "# %s %s-Axis Local Size, +/- Pts: %4s Weight: %6.4f"

if {$dimCount >= 1} {puts $outUnit [format $fmt $xwVar X $dx $wx]}
if {$dimCount >= 2} {puts $outUnit [format $fmt $ywVar Y $dx $wy]}
if {$dimCount >= 3} {puts $outUnit [format $fmt $zwVar Z $dx $wz]}
if {$dimCount >= 4} {puts $outUnit [format $fmt $awVar A $dx $wa]}

puts $outUnit ""
puts $outUnit "VARS   $varList"
puts $outUnit "FORMAT $fmtList"
puts $outUnit ""

set id 0

foreach e $eList \
   {
    set thisDX $dx 
    set thisDY $dy 
    set thisDZ $dz 
    set thisDA $da 

    set x0 0.0
    set y0 0.0
    set z0 0.0
    set a0 0.0

    set x0PPM 0.0
    set y0PPM 0.0
    set z0PPM 0.0
    set a0PPM 0.0

    set nearCond ""

    if {$dimCount >= 1} \
       {
        if {![strcasecmp $dx Auto]} {set thisDX [expr int(1.5 + $dxMult*[gdbGet $e $xwVar])]}

        set x0        [gdbGet $e X_AXIS]
        set x0PPM     [gdbGet $e X_PPM]

        append nearCond " near( $x0, X_AXIS, $thisDX )"
       }

    if {$dimCount >= 2} \
       {
        if {![strcasecmp $dy Auto]} {set thisDY [expr int(1.5 + $dyMult*[gdbGet $e $ywVar])]}

        set y0        [gdbGet $e Y_AXIS]
        set y0PPM     [gdbGet $e Y_PPM]

        append nearCond " && near( $y0, Y_AXIS, $thisDY )"
       }

    if {$dimCount >= 3} \
       {
        if {![strcasecmp $dz Auto]} {set thisDZ [expr int(1.5 + $dzMult*[gdbGet $e $zwVar])]}

        set z0        [gdbGet $e Z_AXIS]
        set z0PPM     [gdbGet $e Z_PPM]

        append nearCond " && near( $z0, Z_AXIS, $thisDZ )"
       }

    if {$dimCount >= 4} \
       {
        if {![strcasecmp $da Auto]} {set thisDA [expr int(1.5 + $daMult*[gdbGet $e $awVar])]}

        set a0        [gdbGet $e A_AXIS]
        set a0PPM     [gdbGet $e A_PPM]

        append nearCond " && near( $a0, A_AXIS, $thisDA )"
       }

    set origID    [gdbGet $e INDEX]
    set origClust [gdbGet $e CLUSTID]

    if {$useClust} \
       {
        set nearList [gdbSelect -from $t -cond "INDEX != $origID && CLUSTID == $origClust"]
        set n        [llength $nearList]
       
        if {$n > $maxClust - 1} \
           {
            set nearList [gdbSelect -from $t -cond "INDEX != $origID && CLUSTID == $origClust && $nearCond"]
            set n        [llength $nearList]
           }
       } \
    else \
       {
        set nearList [gdbSelect -from $t -cond "INDEX != $origID && $nearCond"]
        set n        [llength $nearList]
       }

    set thisMemCnt [expr $n + 1]
    set distList   ""

    foreach eNear $nearList \
       {
        set ddx 0.0
        set ddy 0.0
        set ddz 0.0
        set dda 0.0

        if {$dimCount >= 1} {set ddx [expr $wx*($x0 - [gdbGet $eNear X_AXIS])]}
        if {$dimCount >= 2} {set ddy [expr $wy*($y0 - [gdbGet $eNear Y_AXIS])]}
        if {$dimCount >= 3} {set ddz [expr $wz*($z0 - [gdbGet $eNear Z_AXIS])]}
        if {$dimCount >= 4} {set dda [expr $wa*($a0 - [gdbGet $eNear A_AXIS])]}

        lappend distList [expr sqrt($ddx*$ddx + $ddy*$ddy + $ddz*$ddz + $dda*$dda)]
       }

    set nearList  [sortByVecAbs $nearList $distList]
    set origCount $thisMemCnt

    if {$origCount > $maxPeaks} \
       {
        set nearList   [lrange $nearList 0 [expr $maxPeaks - 2]]
        set n          [llength $nearList]
        set thisMemCnt [expr $n + 1] 
       }

    set thisIDList ""

    foreach eNear $nearList {lappend thisIDList [gdbGet $eNear INDEX]} 

    incr id

    if {$dimCount == 1} \
       {
        set fmt "# Peak: %d New ID: %d X: %.3f ppm Cluster: %d DX: +/- %.1f pts Neighbors: %d %d Neighbor IDs: $thisIDList"

        puts $outUnit "#"
        puts $outUnit [format $fmt $origID $id $x0PPM $origClust $thisDX $n $origCount]
       } \
    elseif {$dimCount == 2} \
       {
        set fmt "# Peak: %d New ID: %d X: %.3f ppm Y: %.3f ppm Cluster: %d DX: +/- %.1f pts DY: +/- %.1f pts Neighbors: %d %d Neighbor IDs: $thisIDList"

        puts $outUnit "#"
        puts $outUnit [format $fmt $origID $id $x0PPM $y0PPM $origClust $thisDX $thisDY $n $origCount]
       } \
    elseif {$dimCount == 3} \
       {
        set fmt "# Peak: %d New ID: %d X: %.3f ppm Y: %.3f ppm Z: %.3f ppm Cluster: %d DX: +/- %.1f pts DY: +/- %.1f pts DZ: +/- %.1f pts Neighbors: %d %d Neighbor IDs: $thisIDList"

        puts $outUnit "#"
        puts $outUnit [format $fmt $origID $id $x0PPM $y0PPM $z0PPM $origClust $thisDX $thisDY $thisDZ $n $origCount]
       } \
    elseif {$dimCount == 4} \
       {
       set fmt "# Peak: %d New ID: %d X: %.3f ppm Y: %.3f ppm Z: %.3f ppm A: %.3f ppm Cluster: %d DX: +/- %.1f pts DY: +/- %.1f pts DZ: +/- %.1f pts  DA: +/- %.1f pts Neighbors: %d %d Neighbor IDs: $thisIDList"

        puts $outUnit "#"
        puts $outUnit [format $fmt $origID $id $x0PPM $y0PPM $z0PPM $a0PPM $origClust $thisDX $thisDY $thisDZ $thisDA $n $origCount]
       }

    printEntry $e $id $origID $thisMemCnt

    foreach eNear $nearList \
       {
        incr id
        printEntry $eNear $id $origID $thisMemCnt
       }

    puts $outUnit ""
   }

if {$verbose} {puts "localFit. Final Peak Count:    $id"}

exit 0

