
#
# Modules for Metropolis Algorithm for Combinatorial Optimization.

proc metro { nList x \
             scoreFn modFn reportFn \
             tMax tMin nLevels nSteps nTrials alpha } \
{
   global metroN metroNN metroLoc

   metroInit $nList $x

   if {$metroN < 2 || $metroN == $metroNN} {return $x}
   if {$nLevels < 1 || $nSteps < 1}        {return $x}

   if {![strcasecmp $tMax Auto]} {set tMax 100.0}
   if {![strcasecmp $tMin Auto]} {set tMin   1.0}

   if {![strcasecmp $alpha Auto]} \
      {
       set alpha [expr abs(log(0.01)*$tMax)] 
      }

   set xCurrent $x
   set xGlobal  $x
   set qGlobal  [$scoreFn $nList $x]

   if {$nLevels == 1} \
      {
       set dT 0.0
      } \
   else \
      {
       set dT [expr ($tMax - $tMin)/($nLevels - 1)]
      }

   for {set i 1} {$i <= $nTrials} {incr i} \
      {
       set xBest  $xCurrent
       set qBest  [$scoreFn $nList $xBest]

       set t $tMax

       $reportFn $t $i 0 0 $xBest $qBest $xGlobal $qGlobal

       for {set j 1} {$j <= $nLevels} {incr j} \
          {

           for {set k 1} {$k <= $nSteps} {incr k} \
               {
                set xNew [$modFn   $xCurrent]
                set qNew [$scoreFn $nList $xNew]

                if {$qNew < $qGlobal} \
                   {
                    set xGlobal $xNew
                    set qGlobal $qNew
                   }

                if {$qNew < $qBest} \
                   {
                    set xCurrent $xNew
                    set xBest    $xNew 
                    set qBest    $qNew 

                    $reportFn $t $i $j $k $xBest $qBest $xGlobal $qGlobal
                   } \
                else \
                   {
                    if {$t != 0.0} \
                       { 
                        if {[rand] < exp( -$alpha/$t )} \
                           {
                            set xCurrent $xNew
                           }
                       } 
                   }
               }

           set t [expr abs($t - $dT)]
          }

       set xCurrent [metroShuffle $nList $xCurrent 5]
      }

   return $xGlobal
}

proc metroInit { nList x } \
{
   global metroN metroNN metroLoc

   set metroNN [llength $nList]
   set metroN  0

   foreach n $nList \
      {
       for {set i $metroN} {$i < $metroN + $n} {incr i} \
          {
           set metroLoc($i,off) $metroN
           set metroLoc($i,n)   $n
          }

       incr metroN $n
      }
}

proc metroSwap { x } \
{
   global metroN metroNN metroLoc

   set locI [expr int([rand]*$metroN)]
   set nI   [expr $metroN - 1]

   if {$locI > $nI} {set locI $nI}

   set locJ [expr $metroLoc($locI,off) + int([rand]*$metroLoc($locI,n))]
   set nJ   [expr $metroLoc($locI,off) + $metroLoc($locI,n) - 1]

   if {$locJ > $nJ} {set locJ $nJ}

   if {$locI == $locJ} {return $x}

   set itemI [lindex $x $locI]
   set itemJ [lindex $x $locJ]

   lset x $locI $itemJ
   lset x $locJ $itemI

   return $x
}

proc metroShuffle { nList x shufCount } \
{
   global metroN

   metroInit $nList $x

   for {set i 0} {$i < $shufCount} {incr i} \
      {
       for {set j 0} {$j < $metroN} {incr j} {set x [metroSwap $x]}
      }

   return $x
}

proc metroReport { t i j k x q xG qG } \
{
   set fmt "Trial: %3d T: %6.2f Level: %4d Step: %4d Q: %9.3f Global: %9.3f"

   puts [format $fmt $i $t $j $k $q $qG]
}

