#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- -notk "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set dNameList   "dObsA.tab dObsB.tab"
set mfrNameList mfr.tab
set csName      csObs.tab
set refName     ext.pdb
set outPDBName  init.pdb
set outTabName  init.tab
set segLength   7
set maxRank     10 
set verbose     0

set dcW         2.00
set csW         1.00
set homoW       0.50 
set ramaW       0.50

set pdbDir   $env(PDBH_DIR)
set talosDir $env(TALOS_DIR)

if {[flagLoc $ARGV -help]} \
   {
    puts "Estimate Phi/Psi Angles from Molecular Fragment Replacement Search."
    puts " -inD     dcList   \[dObsA.tab dObsB.tab\]   Input DC Tables."
    puts " -inCS    csName   \[csObs.tab\]             Input CS Table."
    puts " -mfr     mfrList  \[mfr.tab\]               Input Fragment Tables."
    puts " -inPDB   pdbName  \[ext.pdb\]               Reference Structure."
    puts " -outPDB  pdbName  \[init.pdb\]              Output Structure."
    puts " -outTab  tabName  \[init.tab\]              Output Phi/Psi Table."
    puts " -maxRank rank     \[10\]                    Max Fragment Rank."
    puts " -pdbDir  pdbDir   \[PDBH_DIR\]              MFR PDB Directory."
    puts " -talosir tDir     \[TALOS_DIR\]             TALOS Directory."
    puts "Notes:"
    puts " Reference structure is used for sequence; its backbone angles"
    puts " will be reset to generate the output PDB."
    exit 0
   }

getListArgD $ARGV -inD dNameList
getListArgD $ARGV -mfr mfrNameList

getArgD $ARGV -inCS      csName
getArgD $ARGV -inPDB     refName
getArgD $ARGV -outPDB    outPDBName
getArgD $ARGV -outTab    outTabName
getArgD $ARGV -segLength segLength
getArgD $ARGV -maxRank   maxRank

getArgD $ARGV -dcW   dcW
getArgD $ARGV -csW   csW
getArgD $ARGV -homoW homoW
getArgD $ARGV -ramaW ramaW

getArgD $ARGV -pdbDir   pdbDir
getArgD $ARGV -talosDir talosDir

#
#------------------------------------------------------------------------------

set error [catch {set outUnit [open $outTabName w]}]

if {$error} \
   {
    puts "Error opening output file $outTabName"
    exit 1
   }

set dNameA [lindex $dNameList 0]
set dNameB [lindex $dNameList 1]
set dSrcA  0
set dSrcB  0

dynInit

initResNames

set csTab [gdbRead table -in $csName]

if {[file exists $dNameA]} {set dSrcA [dynRead -dc -in $dNameA]}
if {[file exists $dNameB]} {set dSrcB [dynRead -dc -in $dNameB]}

set csSrc    [dynRead -cs  -in $csName]
set refSrc   [dynRead -pdb -in $refName]

set seqList  [getSeqList $csTab seqFirstRes seqLastRes]
set refInfo  [dynGetInfo -src $refSrc]

set firstRes [lindex $refInfo 0]
set lastRes  [lindex $refInfo 1]
set resCount [expr 1 + $lastRes - $firstRes]

set mfrList ""

foreach mfrName $mfrNameList \
   {
    lappend mfrList [gdbRead table -in $mfrName]
   }

#
#----------------------------------------------------------------------

proc classInit {} \
{
   global alphaClass betaClass posClass nullClass
   global classNames classTypes mapPtr talosDir

   readROI -ndim 2 -in $talosDir/vina.dat -roi 1

   set roiInfo [getROIInfo -roi 1]
   set mapPtr  [lindex $roiInfo 0]

   set alphaClass [getClassID  -57.0 -46.0]
   set betaClass  [getClassID -102.0 118.0]
   set posClass   [getClassID   90.0   2.0]
   set nullClass  [getClassID    0.0   0.0]

   set classTypes "alpha beta pos ne nw sw se null"

   set classNames($alphaClass) alpha
   set classNames($betaClass)  beta
   set classNames($posClass)   pos
   set classNames($nullClass)  null
}

proc adjustPhiPsi { phiPtr psiPtr class } \
{
    upvar $phiPtr phi
    upvar $psiPtr psi

    if {"$class" == "beta"} \
       {
        if {$phi >=  160} {set phi [expr $phi - 360]}
        if {$psi <= -120} {set psi [expr $psi + 360]}
       }
}

proc tallyClasses { resID cList } \
{
   global classTypes judgeInfo

   set n [llength $cList]

   if {$n < 1} {set n 1} 

   foreach class $classTypes \
      {
       set tally($class) 0
      }

   foreach class $cList \
      {
       switch $class \
          {
           alpha   {set c $class}
           beta    {set c $class}
           pos     {set c $class}
           ne      {set c $class}
           nw      {set c $class}
           se      {set c $class}
           sw      {set c $class}
           default {set c null} 
          }
      
       incr tally($c)
      }

   puts -nonewline [format "%-7s" Count]

   foreach class $classTypes \
      {
       set judgeInfo(Pct,$class) [expr 100.0*$tally($class)/$n]

       puts -nonewline \
        [format " %-7s %7.0f%%" $class: $judgeInfo(Pct,$class)]
      }

   puts ""
}

proc getClass { phi psi } \
{
   set classID   [getClassID   $phi $psi]
   set className [getClassName $classID]

   if {"$className" == "null"} \
      {
       if {$phi >= 0} \
          {
           set c1 e
          } \
       else \
          {
           set c1 w
          }

       if {$psi >= 0} \
          {
           set c2 n 
          } \
       else \
          {
           set c2 s 
          }

       set className $c2$c1
      }

   return $className
}

proc getClassID { phi psi } \
{
    global mapPtr betaClass

    set x   [expr int(0.5 + $phi + 180.0)]
    set y   [expr int(0.5 + $psi + 180.0)]

    if {$x < 0} {set x 0}
    if {$y < 0} {set y 0}

    if {$x > 360} {set x 360}
    if {$y > 360} {set y 360}

    if {$x > 0   && $x <= 120 && $y >= 0 && $y <= 20} {return $betaClass}
    if {$x > 340 && $x <= 360 && $y >= 0 && $y <= 20} {return $betaClass}

    set loc [expr $x + 361*$y]

    set val [expr int([getPtrVal $mapPtr $loc])]

    return $val
}

proc getClassName { id } \
{
   global classNames

   if {[info exists classNames($id)]} {return $classNames($id)}

   return class$id
}

proc getClassCount { cList } \
{
   set count 0

   foreach class $cList \
      {
       if {![info exists tally($class)]} \
          {
           set tally($class) 1
           incr count
          }
      }

   return $count
}

proc getCommonClass { cList } \
{
   set count       0
   set commonClass None

   foreach class $cList \
      {
       if {![info exists tally($class)]} \
          {
           set tally($class) 1
          } \
       else \
          {
           incr tally($class)
          }

       if {$tally($class) > $count} \
          {
           set count $tally($class)
           set commonClass $class
          }
      }

    return $commonClass
}

proc getBestClass { cList scores name } \
{
   global classTypes judgeInfo

   set n [llength $cList]

   set bestScore 9.999e+16
   set bestClass none

   for {set i 0} {$i < $n} {incr i} \
      {
       set class [lindex $cList  $i]
       set cErr  [lindex $scores $i]

       lappend tally($class) $cErr
      }

   puts -nonewline [format "%-7s" $name]

   foreach class $classTypes \
      { 
       if {[info exists tally($class)]} \
          {
           set judgeInfo($name,$class) [getMedian $tally($class)]

           puts -nonewline \
            [format " %-7s %8.3f" $class: $judgeInfo($name,$class)]

           if {$cErr != 0.0 && $cErr < $bestScore} \
              {
               set bestScore $cErr
               set bestClass $class
              }
          } \
       else \
          {
           set judgeInfo($name,$class) None

           puts -nonewline \
            [format " %-7s %8s" $class: $judgeInfo($name,$class)]
          }
      }

   puts ""

   return $bestClass 
}

proc fixByClass { aList cList bestClass } \
{
   set newList ""

   if {"$bestClass" == "null"} {return $aList}

   set n [llength $cList]

   for {set i 0} {$i < $n} {incr i} \
      {
       set class [lindex $cList $i]

       if {"$class" == "$bestClass"} \
          {
           lappend newList [lindex $aList $i]
          }
      }

   return $newList
}

proc getScores { r1 rN r0 a1 aN a0 dcPtr csPtr sPtr hPtr } \
{
   upvar $dcPtr dcErr
   upvar $csPtr csErr
   upvar $sPtr  sErr
   upvar $hPtr  hErr

   global dSrcA dSrcB csSrc pdbSrc refSrc
   global csW dcW homoW ramaW

   set k     [expr $a0 - $a1]

   set dcErr 0.0
   set csErr 0.0
   set hErr  0.0
   set sErr  0.0
   set daRef 1.0

   if {$dSrcA} \
      { 
       set dInfo [dynSimulate -dc $dSrcA -src $pdbSrc -s1 $r1 -sN $rN -a1 $a0]
       set daRef [lindex $dInfo 3] 
       set dcErr [expr $dcErr + [lindex $dInfo 0]]
      }

   if {$dSrcB} \
      { 
       set dInfo [dynSimulate -dc $dSrcB -src $pdbSrc -s1 $r1 -sN $rN -a1 $a0]
       set da    [lindex $dInfo 3] 

       if {$da == 0.0} {set da 1.0}

       set dcErr [expr $dcErr + [lindex $dInfo 0]*$daRef/$da]
      }

   set csInfo [dynSimulate -cs $csSrc -src $pdbSrc -s1 $r0 -sN $r0 -a1 $a0]
   set csChi  [lindex $csInfo 1]
   set csN    [lindex $csInfo 2]

   if {$csN == 0} {set csN 1}

   set csErr [expr $csChi/double($csN)]

   set sInfo [dynRama     -refSrc $refSrc -aSrc $pdbSrc -r1 $r0 -rN $r0 -a1 $a0]
   set sErr  [lindex $sInfo 0]

   set hInfo [dynHomology -refSrc $refSrc -aSrc $pdbSrc -r1 $r0 -rN $r0 -a1 $a0]
   set hErr  [lindex $hInfo 0]

   set score [expr $dcW*$dcErr + $csW*$csErr + $homoW*$hErr + $ramaW*$sErr]

   return $score
}

proc judgeBestClass { resID } \
{
   global judgeInfo bestInfo badList classTypes

   set commonClass $bestInfo(Pct,$resID)
   set bestClass   $commonClass
   set largeCount  0

#
# Good: Largest group is 75% of total:

   if {$judgeInfo(Pct,$commonClass) >= 75.0} \
      {
       return $commonClass
      }

#
# Bad: there are 3 or more groups of 10% or larger:

   foreach class $classTypes \
      {
       if {$judgeInfo(Pct,$class) > 10.0} {incr largeCount}
      }

   if {$largeCount >= 3} \
      {
       lappend badList $resID
       return $commonClass
      }
    
#
# Good: largest group is 25% larger than next largest:

   set nearestPct   $judgeInfo(Pct,$commonClass)
   set nearestClass None
   set minDif       100.0

   foreach class $classTypes \
      {
       if {"$class" == "$commonClass"} continue

       set d [expr abs($judgeInfo(Pct,$commonClass) - $judgeInfo(Pct,$class))]

       if {$d < $minDif} \
          {
           set minDif     $d
           set nearestPct $judgeInfo(Pct,$class)
          } 
      }

   if {$minDif >= 25.0} \
      {
       return $commonClass
      }

#
# Otherwise, bad:

   lappend badList $resID

   return $bestClass
}

#
#------------------------------------------------------------------------

classInit

foreach mfrTab $mfrList \
  {
   set eList [gdbGetInfo $mfrTab -parms entries]
   set j     0

   foreach entry $eList \
      {
       set thisName [gdbGet $entry PDB_NAME]
       set pdbName  $pdbDir/$thisName

       set rank     [gdbGet $entry RANK]

       if {$rank > $maxRank} continue

       set res1     [gdbGet $entry D_RES1]
       set resN     [gdbGet $entry D_RESN]

       set a1       [gdbGet $entry RES1]
       set aN       [gdbGet $entry RESN]
       set id       [gdbGet $entry INDEX]

       set aa1      [expr $a1 - 1]
       set aaN      [expr $aN + 1]

       set i        $res1
       set k        1

       set pdbSrc   [dynRead -pdb -in $pdbName -r1 $aa1 -rN $aaN]
       set rootName [file tail [file root $pdbName]]

       set fmt    "%4d %3d %3d %2d  %8s %3d %3d"

       puts [format $fmt $j $res1 $resN $rank $rootName $a1 $aN]

       for {set resID $a1} {$resID <= $aN} {incr resID} \
          {
           set refInfo [dynGetPhiPsi -src $pdbSrc -res $resID]
           set phi     [lindex $refInfo 0]
           set psi     [lindex $refInfo 1]

           if {$k > 1 && $k < $segLength && $phi < 999 && $psi < 999} \
              {
               lappend scoreInfo($i) \
                [getScores $res1 $resN $i $a1 $aN $resID dcErr csErr sErr hErr]

               set thisClass [getClass $phi $psi]

               adjustPhiPsi  phi psi $thisClass

               lappend classList($i) $thisClass

               lappend dcList($i)    [format %.4f $dcErr]
               lappend csList($i)    [format %.4f $csErr]
               lappend sList($i)     [format %.4f $sErr]
               lappend hList($i)     [format %.4f $hErr]

               lappend phiList($i)   $phi
               lappend psiList($i)   $psi
              }

           incr i 
           incr k
          }

       dynFree $pdbSrc

       incr j
      }
   }

#
# Find the consensus class at each residue of the target:

set badList ""
set dPhi    ""
set dPsi    ""

for {set i $firstRes} {$i <= $lastRes} {incr i} \
   {
    if {![info exists classList($i)]} continue

    set resName($i) [lindex $seqList [expr $i - $seqFirstRes]]
    set n [llength $classList($i)]

    puts [format "Residue %3d %s Entries: %3d" $i $resName($i) $n]

    tallyClasses $i $classList($i)

    set bestInfo(Pct,$i) [getCommonClass $classList($i)]

    set bestInfo(DC,$i)  [getBestClass $classList($i) $dcList($i) DC]
    set bestInfo(CS,$i)  [getBestClass $classList($i) $csList($i) CS]
    set bestInfo(S,$i)   [getBestClass $classList($i) $sList($i)  S]
    set bestInfo(H,$i)   [getBestClass $classList($i) $hList($i)  H]
      
    set bestClass [judgeBestClass $i]
 
    puts -nonewline "Common: $bestInfo(Pct,$i) "
    puts -nonewline "DC:     $bestInfo(DC,$i) "
    puts -nonewline "CS:     $bestInfo(CS,$i) "
    puts -nonewline "Homo:   $bestInfo(H,$i) "
    puts -nonewline "Rama:   $bestInfo(S,$i) "
    puts -nonewline "Pred:   $bestClass "
    puts ""
    puts ""
 
    set devPhiAll($i)  [getStdDevAngle $phiList($i)]
    set devPsiAll($i)  [getStdDevAngle $psiList($i)]

    set phiOrig($i) $phiList($i)
    set psiOrig($i) $psiList($i)
 
    set phiList($i) [fixByClass $phiList($i) $classList($i) $bestClass]
    set psiList($i) [fixByClass $psiList($i) $classList($i) $bestClass]

    set devPhi($i)  [getStdDevAngle $phiList($i)]
    set devPsi($i)  [getStdDevAngle $psiList($i)]

    set predPhi($i) [getMedian $phiList($i)] 
    set predPsi($i) [getMedian $psiList($i)] 

    set class       [getClassID $predPhi($i) $predPsi($i)]
    set thisClass   [getClassName $class]

    dynSetPhiPsi -src $refSrc -phi $predPhi($i) -psi $predPsi($i) -res $i
   }

puts $outUnit "REMARK Ambiguous Residues: $badList"
puts $outUnit ""

puts $outUnit \
 "VARS   RESID RESNAME PHI PSI DPHI DPSI DPHI_ALL DPSI_ALL DIST COUNT CLASS"

puts $outUnit \
 "FORMAT %4d %s %8.2f %8.2f %7.2f %7.2f %7.2f %7.2f %8.3f %2d %s"

puts $outUnit ""
 
set fmt "%3d %s % 7.2f % 7.2f %6.2f %6.2f %6.2f %6.2f %8.3f %2d %s"

for {set i $firstRes} {$i <= $lastRes} {incr i} \
   {
    if {![info exists predPhi($i)]} continue
 
    if {[lsearch -exact $badList $i] >= 0} \
       {
        set status Bad
       } \
    else \
       {
        set status Good 
       }

    set s [getAvg $scoreInfo($i)]
    set n [llength $phiList($i)]

    puts $outUnit [format $fmt $i $resName($i) \
                          $predPhi($i) $predPsi($i) $devPhi($i) $devPsi($i)\
                          $devPhiAll($i) $devPsiAll($i) $s $n $status]
   }

if {$verbose} \
   {
    puts stderr "VARS   RESID RESNAME PHI    PSI"
    puts stderr "FORMAT %3d   %s      %7.2f  %7.2f"
    puts stderr ""

    for {set i $firstRes} {$i < $lastRes} {incr i} \
       {
        if {[info exists phiOrig($i)]} \
           {
            for {set j 0} {$j < [llength $phiOrig($i)]} {incr j} \
               {
                set phi [lindex $phiOrig($i) $j]
                set psi [lindex $psiOrig($i) $j]
   
                set fmt " %3d %s % 7.2f % 7.2f"
 
                puts stderr [format $fmt $i $resName($i) $phi $psi]
               }
           }

        puts stderr ""
       }
   }

puts "Ambiguous Residues: $badList"

dynWrite -pdb -src $refSrc -out $outPDBName

exit
