#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- -notk "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set verbose     0
set refName     ref.pdb
set outDir      linus
set outName     linus%03d.tab
set mfrDir      mfrSurf
set mfrName     mfr%03d.dat
set linusName   $env(SURF_DIR)/linus.tab

if {[flagLoc $ARGV -help]} \
   {
    puts "Create Linus Bins from Molecular Fragment Replacement Surfaces."
    puts " -mfrDir outDir    \[mfrSurf\]       MFR Surface Input Directory."
    puts " -mfr    mfrName   \[mfr%03d.dat\]   MFR Surface Input Template."
    puts " -ref    pdbName   \[ref.pdb\]       PDB Input (For Sequence)."
    puts " -outDir outDir    \[linus\]         Output Directory."
    puts " -out    outName   \[linus%03d.tab\] Output Template."
    exit 1
   }

getArgD $ARGV -mfr     mfrName
getArgD $ARGV -mfrDir  mfrDir
getArgD $ARGV -out     outName
getArgD $ARGV -outDir  outDir

#
#------------------------------------------------------------------------------

if {![file exists $refName]} \
   {
    puts stderr "Error finding PDB input $refName"
    exit 1
   }

if {![file isdirectory $outDir]} \
   {
    catch "file mkdir $outDir"
   }

set pdbTab [gdbRead table -in $refName -pdb]

set seqList [getPDBSeqList $pdbTab r1 rN]
set seqStr  [getPDBSeqStr  $pdbTab]
set lTab    [gdbRead table -in $linusName]

getKeyData $lTab lInfo

set dPhi 15
set dPsi 15

catch {set dPhi [lindex $lInfo(DPHI) 0]}
catch {set dPsi [lindex $lInfo(DPSI) 0]}

for {set resID $r1} {$resID <= $rN} {incr resID} \
   {
    gdbFree $lTab

    set thisInName  [format $mfrDir/$mfrName $resID]
    set thisOutName [format $outDir/$outName $resID]
    set resName     [lindex $seqList [expr $resID - $r1]]

    set lTab  [gdbRead table -in $linusName]
    set eList [gdbGetInfo $lTab -parms entries]
    set sum   0.0

    puts -nonewline stderr [format " %3d %s\015" $resID $resName]
    flush stderr 
    
    if {[file exists $thisInName]} \
       {
        foreach entry $eList \
           {
            set phi [gdbGet $entry PHI]
            set psi [gdbGet $entry PSI]

            set phiLoc [expr $phi + 181]
            set psiLoc [expr $psi + 181]

            readROI -roi 1 -in $thisInName -ndim 2 \
                    -dx X_AXIS $phiLoc $dPhi -dy Y_AXIS $psiLoc $dPsi

            set roiInfo [getROIInfo -roi 1]
            set rdata   [lindex $roiInfo 0]
            set fdata   [lindex $roiInfo 1]
            set sList   [lindex $roiInfo 2]
            set xSize   [lindex $sList 0]
            set ySize   [lindex $sList 0]
            set tSize   [expr $xSize*$ySize]
            set nMFR    [getParm $fdata FDUSER5 NULL_DIM]
            set val     [vSum $rdata $tSize]
            set sum     [expr $sum + $val]

            if {$nMFR == 0.0} \
               {
                set minVal [expr 1.0/$tSize] 
               } \
            else \
               {
                set minVal [expr 1.0/$nMFR]
               }

            if {$val < $minVal} {set val $minVal}

            gdbSet $entry P $val
           }
       }

    if {$sum == 0.0} \
       {
        set val [expr 1.0/[llength $eList]]

        foreach entry $eList \
           {
            gdbSet $entry P $val
           }
       } \
    else \
       {
        foreach entry $eList \
           {
            set val [gdbGet $entry P]
            set val [expr $val/$sum]
            gdbSet $entry P $val
           }
       }

    gdbAddNote $lTab ""
    gdbAddNote $lTab "DATA RESID   $resID"
    gdbAddNote $lTab "DATA RESNAME $resName"
    gdbAddNote $lTab $seqStr

    gdbWrite $lTab $thisOutName
   }

puts stderr "" 

exit 0
