#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- -notk "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set mfrName     mfr.tab
set refName     ext.pdb
set outPDBName  mfrTor.pdb 
set dcNMin      Default
set maxRank     5 
set skip        1
set dPhi        30 
set dPsi        30 
set maxPhi      60
set maxPsi      60

set qList       "TOTAL_Q HOMO_Q RAMA_Q CS_Q TORSION_Q J_Q DC_Q DC_SVD DDC_Q"
set wList       "TOTAL_Q 1.0"
set atomNames   "C CA N"

set firstRes    First
set lastRes     Last
set pdbDir      $env(PDBH_DIR)
set fragDir     None
set useNScale   1 
set useFragName 0 
set verbose     0

if {[flagLoc $ARGV -help]} \
   {
    puts "Extract Named Torsion Restraints from MFR Search Results."
    puts " -mfr       mfrName  \[mfr.tab\]     Input Fragment Table."
    puts " -ref       pdbName  \[ext.pdb\]     PDB File Reference Input."
    puts " -out       pdbName  \[mfrTor.pdb\]  PDB File Output."
    puts " -r1        r1       \[First\]       First Residue in Output Range."
    puts " -rN        rN       \[Last\]        Last Residue in Output Range."
    puts " -skip      skip     \[1\]           Head/Tail Residues to Skip."
    puts " -maxPhi    mPhi     \[60.0\]        Max Allowed Phi Std Deviation."
    puts " -maxPsi    mPsi     \[60.0\]        Max Allowed Psi Std Deviation."
    puts "Clusters and Cluster Scoring (N = Cluster Member Count):"
    puts " -wInfo     wPairs   \[TOTAL_Q 1.0\] Weight Coefficients for Score."
    puts " -dPhi      dPhi     \[30.0\]        Phi Nearest Neighbor Distance."
    puts " -dPsi      dPsi     \[30.0\]        Psi Nearest Neighbor Distance."
    puts " -useNScale                        Scale Scores by Sqrt(N) (Default)."
    puts " -noNScale                         No Member Count Scaling of Score."
    puts "Fragments to Use:"
    puts " -maxRank     maxRank  \[10\]        Max Rank of Fragments."
    puts " -dcNMin      dcNMin   \[15\]        Minimum Allowable DC Values,"
    puts "                                     When DC Values are Used."  
    puts "Source of Input Structures:"
    puts " -pdbDir      pdbDir   \[PDBH_DIR\]  Location of MFR PDB Files."
    puts " -fragDir     fragDir  \[None\]      Location of MFR PDB Fragments."
    puts " -useFragName                      Use FRAG_NAME from MFR Table."
    exit 1
   }

getArgD $ARGV -in      mfrName
getArgD $ARGV -mfr     mfrName
getArgD $ARGV -pdb     refName
getArgD $ARGV -out     outPDBName

getArgD $ARGV -pdbDir  pdbDir
getArgD $ARGV -fragDir fragDir
getArgD $ARGV -maxRank maxRank
getArgD $ARGV -dcNMin  dcNMin

getArgD $ARGV -maxPhi  maxPhi
getArgD $ARGV -maxPsi  maxPsi

getArgD $ARGV -dPhi    dPhi
getArgD $ARGV -dPsi    dPsi
getArgD $ARGV -r1      firstRes
getArgD $ARGV -rN      lastRes

getListArgD $ARGV -wInfo wList

if {[flagLoc $ARGV -useFragName]} {set useFragName 1}
if {[flagLoc $ARGV -useNScale]}   {set useNScale 1}
if {[flagLoc $ARGV -noNScale]}    {set useNScale 0}

foreach q $qList {set wInfo($q) 0.0}

set n [llength $wList]
set i 0

if {$n % 2} \
   {
    puts stderr "Error In -wInfo Name/Value Pairs."
    exit 1
   }

while {$i < $n} \
   {
    set q [lindex $wList $i]
    incr i 

    set v [lindex $wList $i]
    incr i 

    if {[lsearch -exact $qList $q] < 0} \
       {
        puts stderr "Error: Unknown -wInfo Name $q."
        exit 1
       }

    puts stderr "Setting Weight $q to value $v"
    set wInfo($q) $v
   }

#
#------------------------------------------------------------------------------

proc isNear { loc1 loc2 } \
{
   global dPhi dPsi aInfo r0

   set aItem1 [lindex $aInfo($r0) $loc1]
   set aItem2 [lindex $aInfo($r0) $loc2]

   set thisDPhi [expr [lindex $aItem1 0] - [lindex $aItem2 0]]
   set thisDPsi [expr [lindex $aItem1 1] - [lindex $aItem2 1]]

   while {$thisDPhi < -180.0} {set thisDPhi [expr $thisDPhi + 360.0]}
   while {$thisDPhi >  180.0} {set thisDPhi [expr $thisDPhi - 360.0]}

   while {$thisDPsi < -180.0} {set thisDPsi [expr $thisDPsi + 360.0]}
   while {$thisDPsi >  180.0} {set thisDPsi [expr $thisDPsi - 360.0]}

   if {abs($thisDPhi) < $dPhi && abs($thisDPsi) < $dPsi} {return 1}

   return 0
}

dynInit -fast

if {![file exists $refName]} \
   {
    puts stderr "Error Finding PDB Reference Input $refName"
    exit 1
   }

set refSrc [dynRead -pdb -in $refName]
set pInfo  [dynGetInfo -src $refSrc]
set minRes [lindex $pInfo 0]
set maxRes [lindex $pInfo 1]

if {![strcasecmp $firstRes First]} {set firstRes $minRes}
if {![strcasecmp $lastRes  Last]}  {set lastRes  $maxRes}

if {![file exists $mfrName]} \
   {
    puts stderr "Error Finding MFR Input $mfrName"
    exit 1
   }

set mfrTab [gdbRead table -in $mfrName]
set vList  [gdbGetInfo $mfrTab -parms varNames]

if {![strcasecmp $dcNMin Default]} \
   {
    if {[lsearch -exact $vList A_DA] < 0} {set dcNMin 0}
   }

set    cond "RANK <= $maxRank"
append cond " && D_RES1 >= $firstRes && D_RES1 <= $lastRes"
append cond " && DC_NMIN >= $dcNMin"

set eList   [gdbSelect -from $mfrTab -cond $cond]
set resList ""

for {set r1 [expr $minRes - $skip]} {$r1 <= [expr $maxRes + $skip]} {incr r1} \
   {
    set fragList($r1)  ""
    set fragCount($r1) 0
   }

foreach entry $eList \
   {
    set pdbName  [gdbGet $entry PDB_NAME]
    set rootName [file root [file tail $pdbName]]

    set id    [gdbGet $entry INDEX]
    set rank  [gdbGet $entry RANK]
    set r1    [gdbGet $entry D_RES1]
    set rN    [gdbGet $entry D_RESN]
    set s1    [gdbGet $entry RES1]
    set sN    [gdbGet $entry RESN]

    set score 0.0

    foreach q $qList {set score [expr $score + $wInfo($q)*[gdbGet $entry $q]]}

    if {![strcasecmp $fragDir None]} \
       {
        set thisName $pdbDir/$rootName.pdb
        set fragName None

        set a1 $s1
        set aN $sN
       } \
    else \
       {
        if {$useFragName} \
           {
            set thisName [addExt $fragDir/[gdbGet $entry FRAG_NAME] pdb]
           } \
        else \
           {
            set fmt "%s/frag_%03d_%03d_%02d.pdb"
            set thisName [format $fmt $fragDir $r1 $rN $rank]
           }

        set fragName [file root [file tail $thisName]]

        set a1 $r1
        set aN $rN
       }

    set aa1   [expr $a1 - 1]
    set aaN   [expr $aN + 1]
    set fmt   " %4d.  %4d %4d  %2d  %4d %4d %6s %s"

    puts stderr [format $fmt $id $r1 $rN $rank $s1 $sN $rootName $fragName]

    set pdbSrc \
     [dynRead -pdb -in $thisName -r1 $aa1 -rN $aaN -atomName $atomNames]

    set r0 [expr $r1 + $skip]
    set thisFrag ""

    for {set a0 [expr $a1 + $skip]} {$a0 <= [expr $aN - $skip]} {incr a0} \
       {
        set refInfo [dynGetPhiPsi -src $pdbSrc -res $a0]

        set phi [lindex $refInfo 0]
        set psi [lindex $refInfo 1]

        if {$phi < 999.0 && $psi < 999.0} \
           {
            if {[lsearch -exact $resList $r0] < 0} {lappend resList $r0}
            lappend aInfo($r0) "$phi $psi $score $id"
            lappend thisFrag "$phi $psi $score"
           } \
        else \
           {
            set thisFrag ""
           }

        incr r0
       }

    if {[llength $thisFrag]} \
       {
        set r0 [expr $r1 + $skip]

        lappend fragInfo($r0) $thisFrag
        incr fragCount($r0)
       }

    dynFree $pdbSrc
   }

puts -nonewline stderr "VARS   RESID PHI  PSI  SCORE"
puts -nonewline stderr " AVG_PHI AVG_PSI DEV_PHI DEV_PSI FULL_PHI FULL_PSI"
puts -nonewline stderr " AVG_SCORE CLUSTID MEMCNT"
puts stderr ""

puts -nonewline stderr "FORMAT %4d  %8.2f %8.2f %7.5f"
puts -nonewline stderr " %8.2f  %8.2f    %7.2f   %7.2f   %7.2f    %7.2f"
puts -nonewline stderr " %7.5f     %3d     %2d"
puts stderr ""
puts stderr ""

foreach r0 $resList \
   {
    set bestScore($r0) 1.0e+16
   }

foreach r0 $resList \
   {
    set locList ""
    set i 0

    foreach aItem $aInfo($r0) \
       {
        lappend locList $i
        incr i
       }

    set clustIDList [findClusters clustInfo $locList isNear $verbose]

    foreach clustID $clustInfo(clustIDUsed) \
       {
        foreach loc $clustInfo(members,$clustID) \
           {
            set aItem [lindex $aInfo($r0) $loc]

            lappend clustInfo(phiList,$clustID)   [lindex $aItem 0]
            lappend clustInfo(psiList,$clustID)   [lindex $aItem 1]
            lappend clustInfo(scoreList,$clustID) [lindex $aItem 2]
           }
       }

    set phiList ""
    set psiList ""

    set i 0
    
    foreach aItem $aInfo($r0) \
       {
        set clustID [lindex $clustIDList $i]
        set memCnt  $clustInfo(memCount,$clustID)

        set phi     [lindex $aItem 0] 
        set psi     [lindex $aItem 1] 
        set score   [lindex $aItem 2] 

        set avgPhi  [getAvgAngle $clustInfo(phiList,$clustID)]
        set avgPsi  [getAvgAngle $clustInfo(psiList,$clustID)]

        lappend phiList $phi
        lappend psiList $psi

        if {$useNScale} \
           {
            set q \
             [expr 5.0*[getAvg $clustInfo(scoreList,$clustID)]/sqrt($memCnt)]
           } \
        else \
           {
            set q [getAvg $clustInfo(scoreList,$clustID)]
           }

        if {$q < $bestScore($r0)} \
           {
            set bestScore($r0) $q
            set bestPhi($r0)   $avgPhi 
            set bestPsi($r0)   $avgPsi 

            set devPhi($r0)    [getStdDevAngle $clustInfo(phiList,$clustID)]
            set devPsi($r0)    [getStdDevAngle $clustInfo(psiList,$clustID)]
           }
        incr i
       }

    set i 0
   
    foreach aItem $aInfo($r0) \
       {
        set clustID [lindex $clustIDList $i]
        set memCnt  $clustInfo(memCount,$clustID)

        set phi     [lindex $aItem 0]
        set psi     [lindex $aItem 1]
        set score   [lindex $aItem 2]

        set fullPhi($r0) [getStdDevAngle $phiList]
        set fullPsi($r0) [getStdDevAngle $psiList]

        puts -nonewline stderr \
         [format " %3d % 7.2f % 7.2f %7.4f" $r0 $phi $psi $score]

        set fmt " % 7.2f % 7.2f %6.2f %6.2f %6.2f %6.2f %7.4f %3d %2d"

        puts -nonewline stderr \
         [format $fmt $bestPhi($r0) $bestPsi($r0) \
                      $devPhi($r0)  $devPsi($r0) $fullPhi($r0) $fullPsi($r0) \
                      $bestScore($r0) $clustID $memCnt]

        puts stderr ""

        incr i
       }
   }

foreach r0 $resList \
   {
    if {![info exists bestPhi($r0)]} continue
    if {![info exists bestPsi($r0)]} continue

    dynSetPhiPsi -src $refSrc -res $r0 -phi $bestPhi($r0) -psi $bestPsi($r0)

    if {$fullPhi($r0) <= $maxPhi} \
       {
        puts [format "%s %3d % 7.2f %6.2f" phi $r0 $bestPhi($r0) $devPhi($r0)]
       }

    if {$fullPsi($r0) <= $maxPsi} \
       {
        puts [format "%s %3d % 7.2f %6.2f" psi $r0 $bestPsi($r0) $devPsi($r0)]
       }
   }

dynWrite -pdb -src $refSrc -out $outPDBName

exit
