#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set auto_path [linsert $auto_path 0 $env(NMRBASE)/dynamo/tcl]

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Extraction and Refinement of Fragments in an MFR Table:"
    puts " -mfr     mfrName  \[mfr.tab\]   MFR Input Table."
    puts " -ref     pdbName  \[init.pdb\]  PDB Input of Target Protein."
    puts " -r1      r1       \[First\]     First Residue in Range."
    puts " -rN      rN       \[Last\]      Last Residue in Range."
    puts " -minRank rank1    \[1\]         Minimum Rank to Accept."
    puts " -maxRank rankN    \[10\]        Maximum Rank to Accept."
    puts " -head    nHead    \[1\]         Extra Residues at Start of Fragment."
    puts " -tail    nTail    \[1\]         Extra Residues at End of Fragment."
    puts "Input Directories:"
    puts " -pdbDir  pdbDir   \[PDBH_DIR\]  PDB Source Directory for Fragments."
    puts "Output Directories:"
    puts " -fragDir fragDir  \[frag\]      Directory for Refined Fragments."
    puts " -initDir initDir  \[init\]      Directory for Extracted Fragments."
    puts " -logDir  logDir   \[log\]       Directory for Refinement Log."
    puts "Temporary Files:"
    puts " -tmp tmpPDB  \[fragTmp.pdb\]    Temporary PDB Output Name."
    puts " -txt tmpTxt  \[fragTmp.txt\]    Temporary Text Output Name."
    puts " -tor tmpTor  \[fragTor.tab\]    Temporary DYNAMO Torsion Table."
    puts " -ac  tmpAC   \[fragAC.tab\]     Temporary DYNAMO Atom Coord Table."
    puts "Refinement Modes (Use as Last Arguments on the Command Line):"
    puts " -dyn   dArgs \[None\]           DYNAMO Refinement (Default)."
    puts " -xplor xArgs \[None\]           NIH-XPLOR Refinement."
    puts " -lib                          No Refinement; Library Creation Only."
    puts "DYNAMO Refinement Parameters:"
    puts " -gmc    gmcDir  \[mfr.gmc\]     GMC Input Directory."
    puts " -dHi    dHi     \[0.5\]         AC Distance Restraint Limit."
    puts " -aHi    aHi     \[20.0\]        Torsion Restraint Limit."
    puts " -seg    segName \[MFR\]         Segment Name for Target Protein."
    puts " -script sName   \[fragSA.tcl\]  Fragment Annealing Script."
    exit
   }

set parID ""

getArgD $ARGV -parID parID
 
set mode        dynamo
set refName     init.pdb
set mfrName     mfr.tab
set gmcDir      mfr.gmc
set segName     MFR 

set pdbDir      $env(PDBH_DIR)
set initDir     init
set fragDir     frag
set logDir      log

set torName     fragTor$parID.tab
set acName      fragAC$parID.tab
set tmpPDBName  fragTmp$parID.pdb
set tmpTxtName  fragTmp$parID.txt
set scriptName  fragSA.tcl

set dynArgs     ""
set xplorArgs   ""

set firstRes    First
set lastRes     Last
set minRank     1
set maxRank     10
set headLength  1
set tailLength  1

set dHi         0.5
set aHi         20.0

set loc [flagLoc $ARGV -dyn]

if {$loc} \
   {
    set dynArgs [lrange $ARGV [expr $loc+1 end]]
    set mode dynamo
   }

set loc [flagLoc $ARGV -xplor]

if {$loc} \
   {
    set xplorArgs [lrange $ARGV [expr $loc+1 end]]
    set mode xplor
   }

if {![strcasecmp $dynArgs   None]} {set dynArgs   ""}
if {![strcasecmp $xplorArgs None]} {set xplorArgs ""}

if {[flagLoc $ARGV -lib]} {set mode lib}

getArgD $ARGV -ref     refName
getArgD $ARGV -mfr     mfrName

getArgD $ARGV -head    headLength
getArgD $ARGV -tail    tailLength
getArgD $ARGV -r1      firstRes
getArgD $ARGV -rN      lastRes
getArgD $ARGV -minRank minRank
getArgD $ARGV -maxRank maxRank

getArgD $ARGV -tmp     tmpPDBName
getArgD $ARGV -txt     tmpTxtName

getArgD $ARGV -fragDir fragDir
getArgD $ARGV -initDir initDir 
getArgD $ARGV -logDir  logDir 
getArgD $ARGV -pdbDir  pdbDir

getArgD $ARGV -seg     segName
getArgD $ARGV -gmc     gmcDir
getArgD $ARGV -tor     torName
getArgD $ARGV -ac      acName
getArgD $ARGV -script  scriptName

getArgD $ARGV -dHi     dHi
getArgD $ARGV -aHi     aHi

#
#----------------------------------------------------------------------------

dynInit -fast

dynSetChainMode -ignore
dynSetSegMode   -ignore

if {![file exists $mfrName] || ![file exists $refName]} \
   {
    puts stderr "Error Finding Input Files ($mfrName $refName)."
    exit 1
   }

foreach i "$initDir $fragDir $logDir" \
   {
    if {![strcasecmp $i None]} continue
    if {![file isdirectory $i]} {file mkdir $i}
   }

set refSrc [dynRead -pdb -in $refName -atomName "CA C N"]
set rInfo  [dynGetInfo -src $refSrc]
set r1     [lindex $rInfo 0]
set rN     [lindex $rInfo 1]

dynFree $refSrc

if {![strcasecmp $firstRes First]} {set firstRes $r1}
if {![strcasecmp $lastRes  Last]}  {set lastRes  $rN}

catch "file delete $torName $acName $tmpPDBName $tmpTxtName"
 
set mfrTab [gdbRead table -in $mfrName]
set eList  [gdbGetInfo $mfrTab -parms entries]

foreach entry $eList \
   {
    set aName [gdbGet $entry PDB_NAME]
    set r1    [gdbGet $entry D_RES1]
    set rN    [gdbGet $entry D_RESN]
    set a1    [gdbGet $entry RES1]
    set aN    [gdbGet $entry RESN]
    set rank  [gdbGet $entry RANK]

    if {$r1   < $firstRes || $r1   > $lastRes} continue
    if {$rank < $minRank  || $rank > $maxRank} continue

    set rr1   [expr $r1 - $headLength]
    set rrN   [expr $rN + $tailLength]
    set aa1   [expr $a1 - $headLength]
    set aaN   [expr $aN + $tailLength]

    if {![file exists $pdbDir/$aName]} continue

    lappend fragInfo($r1,$rN) "$aName $a1"

    set pdbName $pdbDir/$aName
    set rem     "Fragment $aName $a1 $aN"

    set id [llength $fragInfo($r1,$rN)]

    set pdbSrc   [dynRead -pdb -in $refName  -r1 $rr1 -rN $rrN]
    set pInfo    [dynGetInfo -src $pdbSrc]
    set s1       [lindex $pInfo 0]
    set sN       [lindex $pInfo 1]

    set aSrc     [dynRead -pdb -in $pdbName -r1 $aa1 -rN $aaN]
    set pInfo    [dynGetInfo -src $aSrc]
    set s1       [lindex $pInfo 0]
    set sN       [lindex $pInfo 1]

    set initName [format "%s/frag_%03d_%03d_%02d.pdb" $initDir $r1 $rN $rank]
    set outName  [format "%s/frag_%03d_%03d_%02d.pdb" $fragDir $r1 $rN $rank]
    set logName  [format "%s/frag_%03d_%03d_%02d.log" $logDir  $r1 $rN $rank]

    if {![strcasecmp $logDir None]} {set logName $tmpTxtName}

    dynTranslate -src $aSrc -center -all -verb
    dynAlign -refSrc $aSrc -aSrc $pdbSrc -r1 $a1 -rN $aN -a1 $r1 -apply

    puts [format "%s %10s %4d %4d" $outName $aName $a1 $aN]

    if {[strcasecmp $initDir None]} \
       {
        dynWrite -pdb -src $aSrc -out $initName 
       }

    switch $mode \
       {
        dynamo \
           {
            dynWrite -pdb -src $pdbSrc -out $tmpPDBName 

            exec pdb2ac.tcl -in $pdbName \
             -ref $refName -center -seg $segName -atom CA C N \
             -fc 50.0 -r1 $r1 -rN $rN -a1 $a1 -dHi $dHi > $acName

            exec pdb2torsion.tcl -in $pdbName \
             -ref $refName -seg $segName -tor $gmcDir/torsions.tab \
             -r1 $r1 -rN $rN -a1 $a1 -dPhi $aHi -pPsi $aHi > $torName

            catch {exec $scriptName -r1 $r1 -rN $rN \
                    -gmc $gmcDir -in $tmpPDBName -out $outName \
                    -tor $torName -ac $acName -rem $rem $dynArgs >& $logName}

            catch "file delete $torName $acName $tmpPDBName $tmpTxtName"
           }

        xplor \
           {
            dynWrite -pdb -src $pdbSrc -out $tmpPDBName 

            catch \
             {exec xplorMFRRefine.tcl \
                    -in $tmpPDBName -out $outName $xplorArgs >& $logName}

            catch "file delete $tmpPDBName"
           }

        lib { 
             if {[strcasecmp $fragDir None]} \
                {
                 dynWrite -pdb -src $pdbSrc -out $outName 
                }
            }
       }

    dynFree $pdbSrc
    dynFree $aSrc
   }

exit
