#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {$ARGC == 1 || [flagLoc $ARGV -help]} \
   {
    puts "Fit NMR chemical shift titration curve to estimate Kd."
    puts ""
    puts "Original form:"
    puts " modelTitr.tcl titrAxis ppmSigma protinConc"
    puts "Current arguments:"
    puts " -cx    X Axis Coefficient."
    puts " -cy    Y Axis Coefficient."
    puts " -tab   tabName"
    puts " -axis  titrAxis"
    puts " -sigma ppmSigma"
    puts " -lti   ligandConcList"
    puts " -pti   proteinConcList"
    puts "Notes:"
    puts " 1. Concentrations are in uM."
    puts " 2. Settings for TitrAxis: X Y XY."
    puts " 3. Default Scaling Values for titrAxis Modes:"
    puts ""
    puts "     X  cx: 1.0 cy: 0.0 shift: cx*xPPM"
    puts "     Y  cx: 0.0 cy: 1.0 shift: cy*yPPM"
    puts "     XY cx: 1.0 cy: 0.2 shift: sqrt( cx*xPPM*xPPM + cy*yPPM*yPPM )"
    exit 1
   }

setOSType

set titrAxis  X
set sigma     0.0
set ptiVal    100.0

set titrAxis  [lindex $ARGV 3]
set sigma     [lindex $ARGV 4]
set ptiVal    [lindex $ARGV 5]

set cx        1.0
set cy        0.0

set ltiList   ""
set ptiList   ""

set tabName   titrView.tab
set comName   modelTitr.com

set inList    [lsort -dictionary [glob -nocomplain ft/*]]
set specCount [llength $inList]

getArgD $ARGV -axis  titrAxis 
getArgD $ARGV -tab   tabName
getArgD $ARGV -sigma sigma

getListArgD $ARGV -lti ltiList
getListArgD $ARGV -pti ptiList

set titrAxis [string toupper $titrAxis]

switch -exact $titrAxis \
   {
    X -
    X_PPM   {
             set cx 1.0
             set cy 0.0
            }
    Y -
    Y_PPM   {
             set cx 0.0
             set cy 1.0
            }

    default {
             set cx 1.0
             set cy 0.2
            }
   }

getArgD $ARGV -cx cx
getArgD $ARGV -cy cy

#
#-------------------------------------------------------------------------

#
# Extract a given PPM value or composite.

proc getTitrPPM { entry i } \
{
    global titrAxis cx cy

   switch -exact $titrAxis \
      {
       X -
       X_PPM   {
                set sx    [gdbGet $entry X_PPM$i]
                set shift [expr $cx*$sx]
               }
       Y - 
       Y_PPM   {
                set sy    [gdbGet $entry Y_PPM$i]
                set shift [expr $cy*$sy]
               }

       default {
                set sx    [gdbGet $entry X_PPM$i]
                set sy    [gdbGet $entry Y_PPM$i]
                set shift [expr sqrt($cx*$sx*$sx + $cy*$sy*$sy)]
               }
      }

    return $shift
}

#
# Display a modelXY modeling command given X/Y pairs, etc.

proc showModelCmnd { scriptUnit xList yList sFree sSign id ass sigma ptiList } \
{
    global env

    set sDelta [lindex $yList end]
    set kd     1000.0

    set pList          "$sDelta $kd"
    set pHi            "-5.0  1000000.0"
    set pLo            " 5.0  0.0"
    set macName        $env(NMRTXT)/titr.M
    set guessCount     21
    set confidInterval 0.68
    set outName        txt/fit$id.txt
    set modName        mod/mod$id.tab
    set count          [llength $xList]

    set sList ""
    set noise [format "% .4e" $sigma]

    foreach i $yList \
       {
        lappend sList $noise 
       }

#
# Save the X/Y pairs in a text file for plotting:

    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "ModelTitr Error opening output file $outName"
        return 1
       }

    set ptiInit [lindex $ptiList 0]

    for {set i 0} {$i < $count} {incr i} \
       {
        set pti [lindex $ptiList $i]
        set lti [lindex $xList   $i]
        set y   [lindex $yList   $i]
        set s   [lindex $sList   $i]
        
        set c   [expr $lti*$ptiInit/$pti] 

        puts $outUnit "$c $y $s"
       }

    close $outUnit

#
# Print a nicely formatted modelXY command:

    puts $scriptUnit "modelXY -id $id -str assStr $ass -tol 1e-8 \\"
    puts $scriptUnit "        -p $pList -pLo $pLo -pHi $pHi \\"
    puts $scriptUnit "        -var sFree $sFree sSign $sSign \\"

    puts -nonewline $scriptUnit " -x     "
    foreach i $xList {puts -nonewline $scriptUnit [format " %6s" $i]}
    puts $scriptUnit " \\"

    puts -nonewline $scriptUnit " -y     "
    foreach i $yList {puts -nonewline $scriptUnit [format " %6s" $i]}
    puts $scriptUnit " \\"

    puts -nonewline $scriptUnit " -noise "
    foreach i $sList {puts -nonewline $scriptUnit [format " %6s" $i]}
    puts $scriptUnit " \\"

    puts -nonewline $scriptUnit " -a     "
    foreach i $ptiList {puts -nonewline $scriptUnit [format " %6s" $i]}
    puts $scriptUnit " \\"

    puts $scriptUnit " -gc $guessCount -cf $confidInterval \\"
    puts $scriptUnit " -macro $macName >& $modName"
    puts $scriptUnit ""
    puts $scriptUnit "cat $modName"
    puts $scriptUnit ""
}

#
# Initialize:

#
#-------------------------------------------------------------------------

#
# Read the peak table.
# Extract the ligand concentration values  (saved as CONC_L).
# Extract the protein concentration values (saved as CONC_P).

set titrTab [gdbRead table -in $tabName]
set eList   [gdbGetInfo $titrTab -parms entries]

getKeyData $titrTab keyInfo

if {![llength $ptiList]} \
   {
    foreach c [string trim $keyInfo(CONC_P) \{\}] {lappend ptiList $c}
   }

set ptiVal [lindex $ptiList 0]

while {[llength $ptiList] < $specCount} {lappend ptiList $ptiVal}

if {[info exists keyInfo(CONC_L)]} \
   {
    set clInfo $keyInfo(CONC_L)
   } \
else \
   {
    set clInfo $keyInfo(TAU)
   }

if {![llength $ltiList]} \
   {
    foreach c [string trim $clInfo \{\}] {lappend ltiList $c}
   }

if {[llength $ptiList] != [llength $ltiList]} \
   {
    puts "Error: Concentration List Mismatch."
    puts " CONC_L: $ltiList"
    puts " CONC_P: $ptiList"
   }

#
# Create the script output file.
# Create the directories used for data and graphics output.
#
# For each peak:
#  Extract the list of evolutions.
#  Construct the modeling command.

set error [catch {set scriptUnit [open $comName w 0744]}]

if {$error} \
   {
    puts stderr "Error opening script $comName"
    exit 1
   }

if {![file isdirectory plot]} {file mkdir plot}
if {![file isdirectory gnu]}  {file mkdir gnu}
if {![file isdirectory txt]}  {file mkdir txt}
if {![file isdirectory mod]}  {file mkdir mod}

#
# For each peak:
#   Extract the shifts.
#   Show the extracted results.

puts $scriptUnit "#!/bin/csh -f"
puts $scriptUnit ""
puts $scriptUnit "# Titration Axis: $titrAxis"
puts $scriptUnit "# Noise Estimate: $sigma"
puts $scriptUnit ""

foreach entry $eList \
   {
    set id  [gdbGet $entry INDEX]
    set ass [gdbGet $entry ASS]

    set freeShift [getTitrPPM $entry 1]
    set shiftList ""
    set zeroCount 0
 
    for {set j 1} {$j <= $specCount} {incr j} \
        {
         set shift  [getTitrPPM $entry $j]
         set dShift [expr $shift - $freeShift]

         if {$shift == 0.0} {incr zeroCount}

         lappend shiftList $dShift
        }

#   if {$zeroCount} continue

    set avg       [getAvg $shiftList]
    set deltaSign 1.0

    if {$avg < 0.0} \
       {
        set negList ""

        foreach s $shiftList {lappend negList [expr -($s)]}

        set shiftList $negList
        set deltaSign -1.0
       }

    showModelCmnd $scriptUnit \
       $ltiList $shiftList $freeShift $deltaSign $id $ass $sigma $ptiList
   }

puts $scriptUnit "gnuplot gnu/*"
puts $scriptUnit "sleep 5"

close $scriptUnit

set g 90x12+15+15
set error [catch {exec xterm -bg cyan -fg black -geom $g -e $comName}]

exit
