#!/bin/csh

set thisVersion = "2019.161.15.23"

if (`flagLoc $argv -help`) then
   echo "NMRPipe Wrapper for NESTANMR Reconstruction."
   echo " Requires nestanmr binary executable, see: http://nestanmr.com"
   echo "Wrapper Options:"
   echo " -in        inName   [test.ft1]   Input Interferogram."
   echo " -out       outName  [nesta.ft1]  Output Interferogram."
   echo " -ndim      dimCount [2]          Dimension Count."
   echo " -sample    sName    [Auto]       Sample Schedule Input Name."
   echo " -tmpDir    tmpDir   [tmp_nesta]  Directory for Temporary Files."
   echo "Wrapper Options for NUS Zero Fill Extrapolation:"
   echo " -nusZF                           NUS Zero Fill Mode ON."
   echo " -nonusZF                         No NUS Zero Fill (Default)."
   echo " -yNUSZF    zfArgs   [zf=1]       Amount of Y-Axis NUS Zero-Fill."
   echo " -zNUSZF    zfArgs   [zf=1]       Amount of Z-Axis NUS Zero-Fill."
   echo " -aNUSZF    zfArgs   [zf=1]       Amount of A-Axis NUS Zero-Fill."
   echo "Wrapper Options for NESTA Parameters:"
   echo " -L1                          L1 Regularization (NESTA --method 1, Default)."
   echo " -IRL1                        IRL1 Regularization (NESTA --rwiter > 1)."
   echo " -GAUSS                       Gaussian-SL0 Regularization (NESTA --method 0)."
   echo " -nThread     n       [Auto]  Number of Parallel Threads (NESTA --threads)."
   echo " -nestaIter   iter    [Auto]  Number of IRL1 Iterations (NESTA --iter)."
   echo " -nestaRWIter rwiter  [Auto]  Number of Reweighting Iterations (NESTA --rwiter)."
   echo " -nestaCutoff cuttoff [0.10]  Cutoff for Gaussian-SL0 (NESTA --cutoff)."
   echo " -nestaScale  scale   [0.98]  Scaling for Gaussian-SL0 (NESTA --scaling)."
   echo ""
   echo "NESTANMR Options (nestanmr -help):"
   nestanmr -help
   exit 0
endif

set nusZFFlag  = 0
set cleanFlag  = 1
set vFlag      = 1

set nestaArgs  = (`nesta_args.com $*`) 
set inName     = (`getArgD $argv -in       test.ft1`)
set outName    = (`getArgD $argv -out      nesta.ft1`)
set tmpDir     = (`getArgD $argv -tmpDir   tmp_nesta`)
set schedName  = (`getArgD $argv -sample   Auto`)
set dimCount   = (`getArgD $argv -ndim     2`)
set origInName = $inName

if (`flagLoc $argv -nusZF`) then
   set nusZFFlag = 1
endif

if (`flagLoc $argv -nonusZF`) then
   set nusZFFlag = 0
endif

if ($nusZFFlag) then
   set zfDef = "zf=1"
else
   set zfDef = "zf=0"
endif

set yNUSZF = (`getArgD $argv -yNUSZF $zfDef`)
set zNUSZF = (`getArgD $argv -zNUSZF $zfDef`)
set aNUSZF = (`getArgD $argv -aNUSZF $zfDef`)

if (`flagLoc $argv -verb`) then
   set vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   set vFlag = 0
endif

if (`flagLoc $argv -noclean`) then
   set cleanFlag = 0
endif

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
   echo "NESTANMR Wrapper Error Finding Interferogram Input $thisName"
   exit 1
endif

if (!(-d $tmpDir)) then
   mkdir $tmpDir
endif

if (`string csame $schedName Auto`) then
   set fList = (nuslist sampling.sch `find ./ -maxdepth 1 -name schedule\[0-9\]\* -print` `find ./ -maxdepth 1 -name \*.hdr_3 -print` vclist fake.nuslist)

   foreach f ($fList)
      if (-e $f) then
         set sampleName = $f
         break
      endif
   end
else if (`string csame $schedName None`) then
   set schedName = $tmpDir/fake.nuslist
   fakeNUS.com -in $inName -nus 1.0 > $schedName
endif

if (!(-e $schedName)) then
   echo "NESTANMR Wrapper Error Finding Sampling Schedule Input $schedName"
   exit 1
endif

@ fmtCount = (`echo $outName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if ($fmtCount >= 2) then
   echo "NMRPipe NESTA Wrapper Error: only single-format template names are supported."
   exit 1
endif

if ($vFlag) then
   echo ""
   echo "NESTA Wrapper Version $thisVersion"
endif

if ($nusZFFlag) then
   if ($dimCount == 2) then
      set zfName = $tmpDir/nuszf.ft1 

      if ($vFlag) then
         echo ""
         echo "Executing nusZF:"
         echo " nusZF.com -in $inName -out $zfName -yZFARG $yNUSZF -sample None -mask None"
      endif

      nusZF.com -in $inName -out $zfName -yZFARG $yNUSZF -sample None -mask None
   else if ($dimCount == 3) then
      set zfName = $tmpDir/nuszf%04d.ft1 

      if ($vFlag) then
         echo ""
         echo "Executing nusZF:"
         echo " nusZF.com -in $inName -out $zfName -yZFARG $yNUSZF -zZFARG $zNUSZF -sample None -mask None"
      endif

      nusZF.com -in $inName -out $zfName -yZFARG $yNUSZF -zZFARG $zNUSZF -sample None -mask None
   else if ($dimCount == 4) then
      set zfName = $tmpDir/nuszf%04d.ft1 

      if ($vFlag) then
         echo ""
         echo "Executing nusZF:"
         echo " nusZF.com -in $inName -out $zfName -yZFARG $yNUSZF -zZFARG $zNUSZF -aZFARG -aNUSZF -sample None -mask None"
      endif

      nusZF.com -in $inName -out $zfName -yZFARG $yNUSZF -zZFARG $zNUSZF -aZFARG -aNUSZF -sample None -mask None
   endif
   
   set inName = $zfName 
endif

set dName = (`string path  $outName`)
set tName = (`string tail  $outName`)
set tInfo = (`string split $tName "%"`)

if (!(-d $dName)) then
   mkdir $dName
endif

if ($vFlag) then
   echo ""
   echo "Executing NESTA:"
   echo ""
   prettyPrintCmnd.tcl -nocomment nestanmr --fids $inName --outdir $tmpDir --outname nesta --nuslist $schedName $nestaArgs
   echo ""
endif

nestanmr --fids $inName --outdir $tmpDir --outname nesta --nuslist $schedName $nestaArgs

if ($fmtCount == 1) then
   if (-e $tmpDir/nesta0001.ft1) then
      set thisBase = nesta%04d.ft1
   else if (-e $tmpDir/nesta001.ft1) then
      set thisBase = nesta%03d.ft1
   else
      set thisBase = nesta%05d.ft1
   endif

   if ($vFlag) then
      echo ""
      echo "Updating NESTA Result ($tmpDir/$thisBase to $outName) ..."
   endif

   @ i = 1

   while ($i <= 999999)
      set thisName  = (`nmrPrintf $outName $i`) 
      set nestaName = (`nmrPrintf $tmpDir/$thisBase $i`) 

      if (-e $nestaName) then
         /bin/mv $nestaName $thisName
      else
         break
      endif

      @ i++
   end
else
   /bin/mv $tmpDir/nesta.ft1 $outName
endif

if ($cleanFlag) then
   /bin/rm -rf $tmpDir
endif

if ($vFlag) then
   echo ""
endif
