#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set thisVersion "2020.311.09.12"

if {[flagLoc $ARGV -help]} \
   {
    puts "nusCompress.tcl Version $thisVersion"
    puts stderr "Resample Data According to a Non-Uniform Sampling (NUS) Schedule:"
    puts stderr " -sample sName    \[Auto\]   Input Sampling Schedule."
    puts stderr " -in     inName   \[Auto\]   Input Data, Time-Domain or Interferogram."
    puts stderr " -out    outName  \[Auto\]   Output Data, Compressed According to Schedule."
    puts stderr " -mode   inMode   \[pipe\]   Input Mode: pipe bruker varian."
    puts stderr " -type   inType   \[Auto\]   Input Type: int float double."
    puts stderr " -mask   maskName \[Auto\]   Output Mask (-nocompress Only)."
    puts stderr " -ndim   dimCount \[Auto\]   Input Dimension Count."
    puts stderr " -xSize  xSize    \[Auto\]   Vector Size in Input Points."
    puts stderr " -qSize  qSize    \[Auto\]   Vector Quad Size, 1 = Real, 2 = Complex."
    puts stderr " -ySize  ySize    \[Auto\]   Y-Axis Size in Input Points."
    puts stderr " -zSize  zSize    \[Auto\]   Z-Axis Size in Input Points."
    puts stderr " -aSize  aSize    \[Auto\]   A-Axis Size in Input Points."
    puts stderr " -hw     hdrSize  \[Auto\]   Header Size in Input Words."
    puts stderr " -pw     skipSize \[Auto\]   Pad Size in Input Words."
    puts stderr " -compress                 Compress Data to NUS 2D (Default)."
    puts stderr " -nocompress               Retain Data Size, Create and Apply Mask."
    puts stderr " -verb                     Verbose Mode ON (Default)."
    puts stderr " -noverb                   Verbose Mode OFF."
    puts stderr "Spectrometer Parameter File Locations (For -mode bruker or -mode varian):"
    puts stderr " -parmDir pDir   \[.\]           Directory Containing Parameter Files."
    puts stderr " -acqus   a1Name \[pDir/acqus\]  Bruker Dimension 1 Parameter File."
    puts stderr "Sample Schedule Format, First Item Location in Each Line is 0:"
    puts stderr " -rev                          Reverse Items in Line Before Extraction."
    puts stderr " -yLoc        yLoc   \[0\]       Item Location of Y-Axis Sample."
    puts stderr " -zLoc        zLoc   \[1\]       Item Location of Z-Axis Sample."
    puts stderr " -aLoc        aLoc   \[2\]       Item Location of A-Axis Sample."
    puts stderr " -wLoc        wLoc   \[None\]    Item Location of Weighting Factor."
    puts stderr " -sampleCount sCount \[Auto\]    Number of Valid Samples in Schedule."
    puts stderr "Offset Subtracted from Schedule Values (Usually -1, 0, or 1):"
    puts stderr " -offY        offY   \[0\]       Offset for Y-Axis Schedule Values."
    puts stderr " -offZ        offZ   \[0\]       Offset for Z-Axis Schedule Values."
    puts stderr " -offA        offA   \[0\]       Offset for A-Axis Schedule Values."
    puts stderr "Options to nusExpand.tcl (Should be Last on Command Line):"
    puts stderr " -nusArgs ..."
    puts stderr "Notes:"
    puts stderr " 1. Input data file must be specified explicitly."
    puts stderr " 2. Output may be a file name or keyword stdout."
    puts stderr " 3. Weighting values -wLoc can only be used for NMRPipe-format data."
    puts stderr " 4. Options for -nusArgs are only used for -nocompress mode."
    puts stderr " 5. Auto parameters for -mode bruker require acq*s, pulseprogram*, and"
    puts stderr "    NUS schedule files in the same parameter directory (-parmDir)."
    puts stderr " 6. Auto parameters for -mode varian require fid, procpar, NUS schedule"
    puts stderr "    files in the same parameter directory (-parmDir)."
    exit 0
   }

set nusArgs ""

set loc [flagLoc $ARGV -nusArgs]

if {$loc > 0} \
   {
    set nusArgs [lrange $ARGV [expr $loc + 1] end]
    set ARGV    [lrange $ARGV 0 [expr $loc - 1]]
   }

set inName       Auto
set outName      Auto
set sampleName   Auto
set maskName     Auto
set inType       Auto
set inMode       pipe
set parmDir      .

set bytesPerWord 4
set nusInfo      ""
set revFlag      0
set compressFlag 1
set verbose      1

set sampleCount  Auto
set qSize        Auto 
set xSize        Auto 
set ySize        Auto
set zSize        Auto
set aSize        Auto
set dimCount     Auto 
set fmtCount     0

set yLoc         0
set zLoc         1
set aLoc         2
set wLoc         None
set wFlag        0

set locOffY      0
set locOffZ      0
set locOffA      0

set currentInName None
set inUnit        None
set outUnit       None
set sUnit         None
set hdrSize       Auto 
set skipSize      Auto 

set fdata         0
set hdrBuff       0
set skipBuff      0
set dataBuff      0
set vecCountIn    0
set vecCountOut   0
set lineCount     0
set buffSize      0
set qqSize        0

getArgD $ARGV -parmDir parmDir

set a1Name $parmDir/acqus

getArgD $ARGV -in     inName
getArgD $ARGV -out    outName
getArgD $ARGV -sample sampleName
getArgD $ARGV -acqus  a1Name
getArgD $ARGV -mode   inMode
getArgD $ARGV -type   inType

getArgD $ARGV -xSize  xSize
getArgD $ARGV -qSize  qSize
getArgD $ARGV -ySize  ySize
getArgD $ARGV -zSize  zSize
getArgD $ARGV -aSize  aSize
getArgD $ARGV -ndim   dimCount

getArgD $ARGV -offY   locOffY
getArgD $ARGV -offZ   locOffZ
getArgD $ARGV -offA   locOffA

getArgD $ARGV -mask        maskName
getArgD $ARGV -sampleCount sampleCount

getArgD $ARGV -yLoc   yLoc
getArgD $ARGV -zLoc   zLoc
getArgD $ARGV -aLoc   aLoc
getArgD $ARGV -wLoc   wLoc

getArgD $ARGV -hw     hdrSize
getArgD $ARGV -pw     skipSize

if {[flagLoc $ARGV -compress]}   {set compressFlag 1}
if {[flagLoc $ARGV -nocompress]} {set compressFlag 0}

if {[flagLoc $ARGV -rev]}        {set revFlag 1}
if {[flagLoc $ARGV -norev]}      {set revFlag 0}

if {[flagLoc $ARGV -hasW]}       {set wFlag   1}
if {[flagLoc $ARGV -nohasW]}     {set wFlag   0}

if {[flagLoc $ARGV -verb]}       {set verbose 1}
if {[flagLoc $ARGV -noverb]}     {set verbose 0}

#
#----------------------------------------------------------------------------------------------------

proc getInputHdr { fName } \
{
   global inMode inType fdata hdrBuff skipBuff hdrSize skipSize bytesPerWord
   global a1Name acqus

   set thisName [format $fName 1 1]

   if {![strcasecmp $inMode pipe]} \
      {
       if {![file exists $thisName]} {return 1}

       set hdrSize  512
       set inType   float
       set skipSize 0

       set error [rdFDATA $thisName fdata]

       set hdrBuff $fdata
      } \
   elseif {![strcasecmp $inMode varian]} \
      {
       if {![file exists $thisName]} {return 1}

       if {![strcasecmp $hdrSize  Auto]} {set hdrSize  8}
       if {![strcasecmp $skipSize Auto]} {set skipSize 7}

       if {$hdrSize}  {set hdrBuff  [sysMalloc $hdrSize  float]}
       if {$skipSize} {set skipBuff [sysMalloc $skipSize float]}

       set thisUnit [sysOpen $thisName r]

       if {$hdrSize == 8} \
          {
           set status [sysRead $thisUnit $hdrBuff  $hdrSize]     

           if {$status != $bytesPerWord*$hdrSize} \
              {
               puts stderr "NUS Compress Error Reading Header in $inName"
               exit 1
              }

           set vInfo  [getVHdrInfo $hdrBuff]

           if {![strcasecmp $inType Auto]} \
              {
               if {[lsearch -exact $vInfo float] >= 0} \
                  {
                   set inType float
                  } \
               else \
                  {
                   set inType int
                  }
              }
          }

       if {$skipSize} \
          {
           set status [sysRead $thisUnit $skipBuff $skipSize]

           if {$status != $bytesPerWord*$skipSize} \
              {
               puts stderr "NUS Compress Error Reading Pad in $inName"
               exit 1
              }
          }

       catch "sysClose $thisUnit"
      } \
   elseif {![strcasecmp $inMode bruker]} \
      {
       if {![strcasecmp $hdrSize  Auto]} {set hdrSize  0}
       if {![strcasecmp $skipSize Auto]} {set skipSize 0}
      
       if {![strcasecmp $inType Auto]} \
          {
           if {[file exists $a1Name]} {rdJCAMP $a1Name acqus}

           if {[info exists acqus(DTYPA)]} \
              {
               if {[string is integer $acqus(DTYPA)]} \
                  {
                   if {$acqus(DTYPA) == 2} \
                      {
                       if {![strcasecmp $inType Auto]} {set inType double}
                      } \
                   elseif {$acqus(DTYPA) == 1} \
                      {
                       if {![strcasecmp $inType Auto]} {set inType float}
                      }
                  }

               if {![strcasecmp $inType Auto]} {set inType int}
              }
          } 
      }

   if {![strcasecmp $hdrSize  Auto]} {set hdrSize  0}
   if {![strcasecmp $skipSize Auto]} {set skipSize 0}
   if {![strcasecmp $inType   Auto]} {set inType   int}

   if {![strcasecmp $inType double]} {set bytesPerWord 8}

   return 0
}

proc getInputSizes { fName } \
{
   global inMode qSize qqSize xSize ySize zSize aSize vecCountIn vecCountOut sampleCount
   global skipSize buffSize dimCount hdrBuff fdata sampleName parmDir

   set specInfo ""

   if {![strcasecmp $inMode pipe]} \
      {
       if {![strcasecmp $dimCount Auto]} {set dimCount [getParmI    $fdata FDDIMCOUNT NULL_DIM]}
       if {![strcasecmp $qSize    Auto]} {set qSize    [getQuadSize [getParmI $fdata NDQUADFLAG CUR_XDIM]]}

       if {$dimCount < 2} {set ySize 1}
       if {$dimCount < 3} {set zSize 1}
       if {$dimCount < 4} {set aSize 1}

       if {![strcasecmp $xSize Auto]} {set xSize [getParmI $fdata NDSIZE CUR_XDIM]}
       if {![strcasecmp $ySize Auto]} {set ySize [getParmI $fdata NDSIZE CUR_YDIM]}
       if {![strcasecmp $zSize Auto]} {set zSize [getParmI $fdata NDSIZE CUR_ZDIM]}
       if {![strcasecmp $aSize Auto]} {set aSize [getParmI $fdata NDSIZE CUR_ADIM]}
      } \
   elseif {![strcasecmp $inMode varian]} \
      {
       set specInfo [exec varian -notk -auto -list -sample $sampleName -nus -cd $parmDir -noverb] 

       if {![strcasecmp $dimCount Auto]} {set dimCount [getNamedItem dimCount $specInfo]}

       if {$dimCount < 2} {set ySize 1}
       if {$dimCount < 3} {set zSize 1}
       if {$dimCount < 4} {set aSize 1}

       if {![strcasecmp $xSize Auto]} {set xSize [getNamedItem xT $specInfo]}
       if {![strcasecmp $ySize Auto]} {set ySize [getNamedItem yN $specInfo]}
       if {![strcasecmp $zSize Auto]} {set zSize [getNamedItem zN $specInfo]}
       if {![strcasecmp $aSize Auto]} {set aSize [getNamedItem aN $specInfo]}
       if {![strcasecmp $qSize Auto]} {set qSize 2}
      } \
   elseif {![strcasecmp $inMode bruker]} \
      {
       set specInfo [exec bruker -notk -auto -list -sample $sampleName -nus -cd $parmDir -noverb] 

       if {![strcasecmp $dimCount Auto]} {set dimCount [getNamedItem dimCount $specInfo]}

       if {$dimCount < 2} {set ySize 1}
       if {$dimCount < 3} {set zSize 1}
       if {$dimCount < 4} {set aSize 1}

       if {![strcasecmp $ySize Auto]} {set ySize [getNamedItem yN $specInfo]}
       if {![strcasecmp $zSize Auto]} {set zSize [getNamedItem zN $specInfo]}
       if {![strcasecmp $aSize Auto]} {set aSize [getNamedItem aN $specInfo]}
       if {![strcasecmp $qSize Auto]} {set qSize 2}

       if {![strcasecmp $xSize Auto]} \
          {
           set thisSize [getNamedItem xN $specInfo]
           set xSize    [expr $thisSize/$qSize]
          }
      }

   foreach i "$dimCount $xSize $ySize $zSize $aSize $qSize $xSize" \
      {
       if {![string is integer $i]} {return 1}
       if {$i < 1} {return 1}
      }

   set qqSize 1

   if {$dimCount >= 2} {set qqSize [expr 2*$qqSize]}
   if {$dimCount >= 3} {set qqSize [expr 2*$qqSize]}
   if {$dimCount >= 4} {set qqSize [expr 2*$qqSize]}

   set vecCountIn  [expr $ySize*$zSize*$aSize] 
   set vecCountOut [expr $sampleCount*$qqSize]
   set buffSize    [expr ($xSize*$qSize + $skipSize)*$qqSize]

   return 0
}

proc updateNUSHdr {} \
{
   global inMode inType fdata hdrBuff hdrSize skipSize vecCountOut dimCount

   if {![strcasecmp $inMode pipe]} \
      {
       setParm $fdata FDDIMCOUNT  2            NULL_DIM
       setParm $fdata FDPIPEFLAG  0            NULL_DIM
       setParm $fdata FDCUBEFLAG  0            NULL_DIM
       setParm $fdata FDFILECOUNT 1            NULL_DIM
       setParm $fdata FDNUSDIM    $dimCount    NULL_DIM
       setParm $fdata NDSIZE      $vecCountOut CUR_YDIM

       if {$dimCount >= 3} \
          {
           setParm $fdata NDSIZE     1 CUR_ZDIM
           setParm $fdata NDQUADFLAG 1 CUR_ZDIM
          }

       if {$dimCount >= 4} \
          {
           setParm $fdata NDSIZE     1 CUR_ADIM
           setParm $fdata NDQUADFLAG 1 CUR_ADIM
          }
      } \
   elseif {![strcasecmp $inMode varian] && $hdrSize == 8 && [strcmp $hdrBuff 0]} \
      {
       setVHdrInfo $hdrBuff "FIDCount $vecCountOut"
      }
}

proc scaleNUSInput { v w n } \
{
    global inType

    if {![strcasecmp $inType int]} \
       {
        vsMultInt $v $w $n
       } \
    elseif {![strcasecmp $inType double]} \
       {
        vsMultDouble $v $w $n
       } \
    else \
       {
        vsMult $v $w $n
       }
}

proc wrOutputData { thisUnit } \
{
   global xSize qSize qqSize skipSize buffSize dataBuff skipBuff bytesPerWord

   set jump   [expr $xSize*$qSize]
   set status 0
   set off    0

   if {0} \
      {
       for {set iqq 0} {$iqq < $qqSize} {incr iqq} \
          {
           set status [sysWrite $thisUnit $skipBuff $skipSize]
           if {$status != $bytesPerWord*$skipSize} {return 1}

           set status [sysWrite $thisUnit [incrPtr $dataBuff $off] $jump]
           if {$status != $bytesPerWord*$skipSize} {return 1}

           incr off $jump
          }
      } \
   else \
      {
       set status [sysWrite $thisUnit $dataBuff $buffSize]
       if {$status != $bytesPerWord*$buffSize} {return 1}
      }

   return 0 
}

proc openInputData { thisName } \
{
   global currentInName inUnit

   if {[strcmp $currentInName $thisName]} \
      {
       if {[strcasecmp $currentInName None]} \
          {
           catch "sysClose $inUnit"
           set currentInName None
           set inUnit None
          }

       set error [catch {set inUnit [sysOpen $thisName r]}]

       if {$error} \
          {
           puts stderr "NUS Compress Error Opening Input File $thisName"
           return 1
          }

       set currentInName $thisName
      }

   return 0
}

proc rdInputData { iiy iiz iia w } \
{
   global inMode hdrSize skipSize xSize ySize zSize aSize qSize qqSize dimCount fmtCount inName currentInName inUnit
   global dataBuff buffSize locOffY locOffZ locOffA bytesPerWord

   if {$dimCount >= 2} \
      {
       set iiy [expr $iiy - $locOffY]
       if {$iiy < 0 || $iiy > $ySize/2 - 1} {return 1}
      }

   if {$dimCount >= 3} \
      {
       set iiz [expr $iiz - $locOffZ]
       if {$iiz < 0 || $iiz > $zSize/2 - 1} {return 1}
      }

   if {$dimCount >= 4} \
      {
       set iia [expr $iia - $locOffA]
       if {$iia < 0 || $iia > $aSize/2 - 1} {return 1}
      }

   if {![strcasecmp $inMode pipe]} \
      {
       if {$dimCount == 2} \
          {
           set thisSize $buffSize

           set thisName $inName
           set error [openInputData $thisName]
           if {$error} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*2*$iiy]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit $dataBuff $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}
          } \
       elseif {$dimCount == 3 && $fmtCount == 0} \
          {
           set kz       [expr 2*$iiz]
           set thisSize [expr $buffSize/2]

           set thisName $inName
           set error [openInputData $thisName]
           if {$error} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + $kz*$ySize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit $dataBuff $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + ($kz + 1)*$ySize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit [incrPtr $dataBuff $thisSize] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}
          } \
       elseif {$dimCount == 3 && $fmtCount == 1} \
          {
           set kz       [expr 2*$iiz]
           set thisSize [expr $buffSize/2]

           set thisName [format $inName [expr $kz + 1]]
           set error [openInputData $thisName]
           if {$error} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*2*$iiy]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit $dataBuff $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set thisName [format $inName [expr $kz + 2]]
           set error [openInputData $thisName]
           if {$error} {return 1}
           
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}
       
           set status [sysRead $inUnit [incrPtr $dataBuff $thisSize] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}
          } \
       elseif {$dimCount == 4 && $fmtCount == 0} \
          {
           set kz       [expr 2*$iiz]
           set ka       [expr 2*$iia]
           set thisSize [expr $buffSize/4]

           set thisName $inName
           set error [openInputData $thisName]
           if {$error} {return 1}
           
           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + $kz*$ySize + $ka*$ySize*$zSize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}
      
           set status [sysRead $inUnit $dataBuff $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + ($kz + 1)*$ySize + $ka*$ySize*$zSize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit [incrPtr $dataBuff $thisSize] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + $kz*$ySize + ($ka + 1)*$ySize*$zSize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}
          
           set status [sysRead $inUnit [incrPtr $dataBuff [expr 2*$thisSize]] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}
          
           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + ($kz + 1)*$ySize + ($ka + 1)*$ySize*$zSize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit [incrPtr $dataBuff [expr 3*$thisSize]] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}
          } \
       elseif {$dimCount == 4 && $fmtCount == 1} \
          {
           set kz       [expr 2*$iiz]
           set ka       [expr 2*$iia]
           set thisSize [expr $buffSize/4]

           set thisName [format $inName [expr $ka + 1]] 
           set error [openInputData $thisName]
           if {$error} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + $kz*$ySize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit $dataBuff $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + ($kz + 1)*$ySize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit [incrPtr $dataBuff $thisSize] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set thisName [format $inName [expr $ka + 2]]
           set error [openInputData $thisName]
           if {$error} {return 1}
           
           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + $kz*$ySize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}
           
           set status [sysRead $inUnit [incrPtr $dataBuff [expr 2*$thisSize]] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}
           
           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + ($kz + 1)*$ySize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}
           
           set status [sysRead $inUnit [incrPtr $dataBuff [expr 3*$thisSize]] $thisSize]
           if {$status != $bytesPerWord *$thisSize} {return 1}
          } \
       elseif {$dimCount == 4 && $fmtCount == 2} \
          {
           set kz       [expr 2*$iiz]
           set ka       [expr 2*$iia]
           set thisSize [expr $buffSize/4]

           set thisName [format $inName [expr $ka + 1] [expr $kz + 1]]
           set error [openInputData $thisName]
           if {$error} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*2*$iiy]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit $dataBuff $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set thisName [format $inName [expr $ka + 1] [expr $kz + 2]]
           set error [openInputData $thisName]
           if {$error} {return 1}

           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit [incrPtr $dataBuff $thisSize] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set thisName [format $inName [expr $ka + 2] [expr $kz + 1]]
           set error [openInputData $thisName]
           if {$error} {return 1}

           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit [incrPtr $dataBuff [expr 2*$thisSize]] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set thisName [format $inName [expr $ka + 2] [expr $kz + 2]]
           set error [openInputData $thisName]
           if {$error} {return 1}

           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit [incrPtr $dataBuff [expr 3*$thisSize]] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}
          } \
       else \
          {
           puts "NUS Compress Error: Unsupported Dimension Count $dimCount $fmtCount"
           exit 1
          }

       if {$w != 1.0} {scaleNUSInput $dataBuff $w $buffSize}
      } \
   elseif {![strcasecmp $inMode bruker]} \
      {
       set error [openInputData $inName]
       if {$error} {return 1}

       if {$dimCount == 2} \
          {
           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*2*$iiy]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit $dataBuff $buffSize]
           if {$status != $bytesPerWord*$buffSize} {return 1}
          } \
       elseif {$dimCount == 3} \
          {
           set thisSize [expr $buffSize/2]
           set kz       [expr 2*$iiz]

           set error [openInputData $inName]
           if {$error} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + $kz*$ySize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit $dataBuff $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + ($kz + 1)*$ySize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit [incrPtr $dataBuff $thisSize] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}
          } \
       elseif {$dimCount == 4} \
          {
           set thisSize [expr $buffSize/4]
           set kz       [expr 2*$iiz]
           set ka       [expr 2*$iia]

           set error [openInputData $inName]
           if {$error} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + $kz*$ySize + $ka*$ySize*$zSize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit $dataBuff $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + ($kz + 1)*$ySize + $ka*$ySize*$zSize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit [incrPtr $dataBuff $thisSize] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + $kz*$ySize + ($ka + 1)*$ySize*$zSize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}
      
           set status [sysRead $inUnit [incrPtr $dataBuff [expr 2*$thisSize]] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}

           set loc   [expr $hdrSize + ($xSize*$qSize + $skipSize)*(2*$iiy + ($kz + 1)*$ySize + ($ka + 1)*$ySize*$zSize)]
           set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
           if {$error} {return 1}

           set status [sysRead $inUnit [incrPtr $dataBuff [expr 3*$thisSize]] $thisSize]
           if {$status != $bytesPerWord*$thisSize} {return 1}
          } \
       else \
          {
           puts "NUS Compress Error: Unsupported Dimension Count $dimCount $fmtCount"
           exit 1
          }

       if {$w != 1.0} {scaleNUSInput $dataBuff $w $buffSize}
      } \
   else \
      {
       set error [openInputData $inName]
       if {$error} {return 1}

       set loc [expr $hdrSize + ($xSize*$qSize + $skipSize)*$qqSize*($iiy + ($iiz*$ySize/2) + ($iia*$ySize*$zSize/4))]

       set error [catch {sysSeek $inUnit [expr $bytesPerWord*$loc]}]
       if {$error} {return 1}

       set status [sysRead $inUnit $dataBuff $buffSize]
       if {$status != $bytesPerWord*$buffSize} {return 1}

       if {$w != 1.0} {scaleNUSInput $dataBuff $w $buffSize}
      } 

   return 0
}

if {!$compressFlag && [strcasecmp $inMode pipe]} \
   {
    puts stderr "NUS Compress Error: -nocompress Requires -mode pipe"
    exit 1
   }

if {![strcasecmp $sampleName Auto]} \
   {
    if {![strcmp $parmDir "."] || ![strcmp $parmDir "./"]} \
       {
        set sampleName [lindex [findNUSInput .] 0]
       } \
    else \
       {
        set sampleName [lindex [findNUSInput $parmDir] 0]
       }
   }

if {![strcasecmp $sampleName Auto]} \
   {
    puts stderr "NUS Compress. Error Finding Auto Mode Sampling Schedule."
    exit 1
   }

if {![file exists $sampleName]} \
   {
    puts stderr "NUS Compress. Error Finding Sampling Schedule $sampleName"
    exit 1
   }

if {![strcasecmp $sampleCount Auto]} \
   {
    if {$revFlag} \
       {
        catch {set nusInfo [exec getNUSInfo.tcl -in $sampleName -rev]}
       } \
    else \
       {
        catch {set nusInfo [exec getNUSInfo.tcl -in $sampleName]}
       }

    if {![info exists nusInfo]} {set nusInfo ""}

    set sampleCount [getNamedItem sampleCount $nusInfo]

    if {![llength $sampleCount] || ![string is integer $sampleCount]} \
       {
        puts stderr "NUS Compress. Error Extracting Sample Count from Sampling Schedule $sampleName"
        exit 1
       }
   }

if {![strcasecmp $inMode pipe]} \
   {
    set hdrSize $FDATASIZE
    set inType  float

    if {![strcasecmp $inName Auto]} {set inName [findNMRInput "fid data" fid]}
 
    if {![strcasecmp $inName Auto] || ![strcasecmp $inName None]} \
       {
        puts stderr "NUS Compress. Error Finding Auto Mode Input Name."
        exit 1
       }

    set thisName [format $inName 1 1]

    if {![file exists $thisName]} \
       {
        puts stderr "NUS Compress. Error Finding Input $thisName"
        exit 1
       }

    set error [getInputHdr $thisName]

    if {$error} \
       {
        puts stderr "NUS Compress. Error Reading Header in $thisName"
        exit 1
       }

    set error [getInputSizes $thisName]

    if {$error} \
       {
        puts stderr "NUS Compress. Error Extracting Sizes for $thisName"
        exit 1
       }

    set ftFlag   [getParmI $fdata NDFTFLAG CUR_XDIM]

    if {![strcasecmp $outName Auto]} \
       {
        if {$ftFlag} \
           {
            set outName nus.ft1
           } \
        else \
           {
            set outName nus.fid
           }
       }
   } \
elseif {![strcasecmp $inMode varian]} \
   {
    if {![strcasecmp $inName  Auto]} {set inName  fid}
    if {![strcasecmp $outName Auto]} {set outName fid_nus}

    set error [getInputHdr $inName]

    if {$error} \
       {
        puts stderr "NUS Compress. Error Reading Header in $inName"
        exit 1
       }

    set error [getInputSizes $inName]

    if {$error} \
       {
        puts stderr "NUS Compress. Error Extracting Sizes for $inName"
        exit 1
       }
   } \
else \
   {
    if {![strcasecmp $inName  Auto]} {set inName  ser}
    if {![strcasecmp $outName Auto]} {set outName ser_nus}

    set error [getInputHdr $inName]

    if {$error} \
       {
        puts stderr "NUS Compress. Error Reading Header in $inName"
        exit 1
       }

    set error [getInputSizes $inName]

    if {$error} \
       {
        puts stderr "NUS Compress. Error Extracting Sizes for $inName"
        exit 1
       }
   }

set fmtCount [getFmtCount $inName]

updateNUSHdr

if {![strcasecmp $maskName Auto]} \
   {
    if {!$fmtCount} \
       {
        set maskName mask[file extension $inName]
       } \
    else \
       {
        set maskName mask/[file tail $inName]
       }
   }

if {![strcasecmp $inName $outName]} \
   {
    puts stderr "NUS Compress. Error: In-Place Processing is Not Allowed."
    puts stderr "NUS Compress. Input: $inName Output: $outName"
    exit 1
   }

if {![strcmp $maskName $inName] || ![strcmp $outName $maskName]} \
   {
    puts stderr "NUS Compress. Error: In-Place Processing is Not Allowed."
    puts stderr "NUS Compress. Input: $inName Output: $outName Mask: $maskName"
    exit 1
   }

if {$compressFlag && [getFmtCount $outName]} \
   {
    puts stderr "NUS Compress. Error:  Template Output for -mode pipe -compress is Not Allowed."
    puts stderr "NUS Compress. Output: $outName"
    exit 1
   }

if {$verbose} \
   {
    puts stderr ""
    puts stderr "NUS Compress. Version:             $thisVersion"
    puts stderr "NUS Compress. Binary Type:         [getNMRBinType]"
    puts stderr "NUS Compress. Sampling Schedule:   $sampleName"
    puts stderr "NUS Compress. Sample Count:        $sampleCount"
    puts stderr "NUS Compress. Input Mode:          $inMode"
    puts stderr "NUS Compress. Input Type:          $inType"
    puts stderr "NUS Compress. Input Word Size:     $bytesPerWord"
    puts stderr "NUS Compress. Input Data:          $inName"
    puts stderr "NUS Compress. Output Data:         $outName"

    if {!$compressFlag} \
       {
        puts stderr "NUS Compress. Compress:            No"
        puts stderr "NUS Compress. Output Mask:         $maskName"
       } \
    else \
       {
        puts stderr "NUS Compress. Compress:            Yes"
       }

    puts stderr "NUS Compress. Input Dimensions:    $dimCount"
    puts stderr "NUS Compress. Input Quad Size:     $qSize"
    puts stderr "NUS Compress. Input X-Size:        $xSize"

    if {$dimCount >= 2} {puts stderr "NUS Compress. Input Y-Size:        $ySize"}
    if {$dimCount >= 3} {puts stderr "NUS Compress. Input Z-Size:        $zSize"}
    if {$dimCount >= 4} {puts stderr "NUS Compress. Input A-Size:        $aSize"}

    if {$dimCount == 2} \
       {
        puts stderr "NUS Compress. Index Offset:        Y: $locOffY"
       } \
    elseif {$dimCount == 3} \
       {
        puts stderr "NUS Compress. Index Offsets:       Y: $locOffY Z: $locOffZ"
       } \
    else \
       {
        puts stderr "NUS Compress. Index Offsets:       Y: $locOffY Z: $locOffZ A: $locOffA"
       }

    puts stderr "NUS Compress. Input Vector Count:  $vecCountIn"
    puts stderr "NUS Compress. Output Vector Count: $vecCountOut"
    puts stderr "NUS Compress. Words Per Sample:    $buffSize"
   }

if {$compressFlag} \
   {
    if {![strcasecmp $outName stdout]} \
       {
        set outUnit 1
       } \
    else \
       {
        set outUnit [sysOpen $outName w]
       }

    set error [catch {set sUnit [open $sampleName r]}]

    if {$error} \
       {
        puts stderr "NUS Compress Error Opening Sampling Input File $sampleName"
        exit 1
       }

    set error [catch {set dataBuff [sysMalloc $buffSize $inType]}]

    if {$error} \
       {
        puts stderr "NUS Compress Error Allocating Memory."
        exit 1
       }

    set lineCount 0

    if {$dimCount < 2} {set yLoc None}
    if {$dimCount < 3} {set zLoc None}
    if {$dimCount < 4} {set aLoc None}

    if {$hdrSize} {set status [sysWrite $outUnit $hdrBuff $hdrSize]}

    while {[gets $sUnit origLine] >= 0} \
       {
        set  line [string trim $origLine]
        incr lineCount 

        set itemCount [llength $line]
        if {!$itemCount} continue

        set key [string index [lindex $line 0] 0]
        if {![strcmp $key "#"]} continue

        if {$revFlag} {set line [lreverse $line]}

        set error 0
        set iiy   0 
        set iiz   0 
        set iia   0 
        set w     1.0 

        if {[strcasecmp $yLoc None]} {set iiy [lindex $line $yLoc]}
        if {[strcasecmp $zLoc None]} {set iiz [lindex $line $zLoc]}
        if {[strcasecmp $aLoc None]} {set iia [lindex $line $aLoc]}
        if {[strcasecmp $wLoc None]} {set w   [lindex $line $wLoc]}

        if {![string is integer $iiy]} {incr error}
        if {![string is integer $iiz]} {incr error}
        if {![string is integer $iia]} {incr error}
        if {![string is double  $w]}   {incr error}

        if {$error} \
           {
            puts stderr "NUS Compress Error: Bad Data at Line $lineCount:"
            puts stderr "$origLine"
            continue
           }

        set error [rdInputData $iiy $iiz $iia $w]

        if {$error} \
           {
            puts stderr "NUS Compress Error: Bad Increments at Line $lineCount:"
            puts stderr "$origLine"
            continue
           }

        set error [wrOutputData $outUnit]

        if {$error} \
           {
            puts stderr "NUS Compress Error Writing Increments for Line $lineCount:"
            puts stderr "$origLine"
            continue
           }
       }

    if {[strcasecmp $currentInName None]} {catch "sysClose $inUnit"}
    if {[strcasecmp $outName stdout]}     {catch "sysClose $outUnit"}
   } \
else \
   {
    catch "file delete -force [format $outName 1 1]  [format $maskName 1 1]"

    set nusArgs "$nusArgs -sample $sampleName -mode $inMode"

    if {$fmtCount} \
       {
        catch {eval "exec xyz2pipe -in $inName | nusExpand.tcl $nusArgs -in stdin -out $maskName -noexpand -mask -noverb -silent"}
       } \
    else \
       {
        catch {eval "exec nusExpand.tcl $nusArgs -in $inName -out $maskName -noexpand -mask -noverb -silent"}
       }

    set thisName [format $maskName 1 1]

    if {![file exists $thisName]} \
       {
        puts stderr "NUS Compress. Error Creating Mask $maskName"
        exit 1
       }

    exec addNMR -in1 $inName -in2 $maskName -mult -out $outName

    set thisName [format $outName 1 1]

    if {![file exists $thisName]} \
       {
        puts stderr "NUS Compress. Error Creating Output $outName"
        exit 1
       }
   }

exit 0
