#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Sort/Distribute Non-Uniformly Sampled (NUS) Vectors in NMRPipe Data:"
    puts " -in     inName   \[test.fid\] Input Name."
    puts " -out    outName  \[sort.fid\] Output Name." 
    puts " -sample sName    \[*.hdr_3\]  Sample Point Input Name."
    puts " -par    pName    \[*.in\]     Sampling Parameter Input Name."
    puts " -nusdim dimCount \[Auto\]     Number of NUS Dimensions."
    puts " -prof   profName \[None\]     Optional NUS Profile Output."
    puts " -expand                     Expand to Full Sampling Space (Default)."
    puts " -noexpand                   Sort Only."
    puts " -verb                       Verbose Mode ON (Default)."
    puts " -noverb                     Verbose Mode OFF."
    exit 1
   }

set inTemplate  test.fid
set outTemplate sort.fid
set profName    None
set parName     Auto
set nusDim      Auto 

set sampleName [lindex [glob -nocomplain *.hdr_3] 0]
set expandFlag 1
set verbose    1

getArgD $ARGV -in     inTemplate
getArgD $ARGV -out    outTemplate
getArgD $ARGV -prof   profName
getArgD $ARGV -nusdim nusDim
getArgD $ARGV -sample sampleName

if {![strcasecmp $parName Auto]} \
   {
    set parName [file root $sampleName].in 
   }

getArgD $ARGV -par parName

if {[flagLoc $ARGV -expand]}   {set expandFlag 1}
if {[flagLoc $ARGV -noexpand]} {set expandFlag 0}

set loc [flagLoc $ARGV -verb]

if {$loc} \
   {
    set nextFlag [nextFlag $ARGV $loc]
    set verbose  1

    if {$nextFlag == $loc + 2} \
       {
        getArgD $ARGV -verb verbose
       }
   }

if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
#--------------------------------------------------------------------------

proc getSampleParm { parName } \
{
    global parInfo nusDim

    foreach v {file NDIM NIMAX NIMIN NI SW} {set parInfo($v) "None"}

    if {![strcasecmp $parName None]} {return 0}

    if {![file exists $parName]} \
       {
        puts stderr "Warning: Missing Sampling Parameter File $parName"
        return 0
       }    
   
    set error [catch {set inUnit [open $parName r]}]

    if {$error} \
       {
        puts stderr "Error Opening Sampling Parameter File $parName"
        return n
       }   

    set n 0

    while {[gets $inUnit line] >= 0} \
       {
        if {![llength $line]} continue

        set v [lindex $line 0]
        set parInfo($v) [lrange $line 1 end]

        incr n
       }

    close $inUnit

    if {![strcasecmp $nusDim Auto]} \
       {
        if {[strcasecmp $parInfo(NDIM) None]} \
           {
            set nusDim [expr $parInfo(NDIM) - 1]
           }
       }

    return $n 
}

#
# Interpret sample scheme:

proc getSampleSchedule { sampleName nusDim } \
{
   global mxSize mySize mzSize maSize xSize ySize zSize aSize
   global yInfo zInfo aInfo locInfo

   set sList ""
   set str   ""

   set prevN 0
   set n 0

   set mxSize 0
   set mySize 0
   set mzSize 0
   set maSize 0

   if {![llength $sampleName]} \
      {
       puts stderr "Error: No Sampling Input Name Found."
       return $sList
      }

   if {![file exists $sampleName]} \
      {
       puts stderr "Error Finding Sampling Input File $sampleName"
       return $sList
      }

   set error [catch {set sUnit [open $sampleName r]}]

   if {$error} \
      {
       puts stderr "Error Opening Sampling Input File $sampleName"
       return $sList
      }

   while {[gets $sUnit line] >= 0} \
      {
       set k [expr int(-1*[llength $str])]
 
       append str " $line"
       incr n

       while {[llength $str] >= $nusDim} \
          {
           set iList [lrange $str 0 [expr $nusDim - 1]]
           set str   [lrange $str $nusDim end]
           set j 1

           lappend sList [join $iList ","]

           foreach item $iList \
              {
               incr j
               incr k

               if {![string is integer $item]} \
                  {
                   puts -nonewline stderr "Warning: Bad Sample Value, Near"

                   if {$k} \
                      {
                       puts stderr " Line $n Item $k: $item"
                      } \
                   else \
                      {
                       puts stderr \
                        " Line [expr $n - 1] Item [expr $prevN - $k]: $item"
                      }
                  } \
               else \
                  {
                   if {$item >= 0} \
                      {
                       if {$j == 2} {set mySize [getMax "$mySize $item"]}
                       if {$j == 3} {set mzSize [getMax "$mzSize $item"]}
                       if {$j == 4} {set maSize [getMax "$maSize $item"]}
                      }
                  }
              }
          }

       set prevN [llength $line]
      }

   close $sUnit

   incr mySize
   incr mzSize
   incr maSize

   set sSize [llength $sList]

   if {!$sSize} \
      {
       puts stderr "Error: Empty Sample Schedule."
       exit 1
      }

   if {$sSize < $ySize*$zSize*$aSize} \
      {
       puts stderr "Warning: Sample Count is Smaller than Data Size."
      }

   if {$sSize > $ySize*$zSize*$aSize} \
      {
       puts stderr "Warning: Sample Count is Larger than Data Size."
      }

   for {set ia 0} {$ia < $maSize} {incr ia} \
      {
       for {set iz 0} {$iz < $mzSize} {incr iz} \
          {
           set yInfo($ia,$iz) ""
          }
      }
  
   for {set ia 0} {$ia < $maSize} {incr ia} \
      {
       for {set iy 0} {$iy < $mySize} {incr iy} \
          {
           set zInfo($ia,$iy) ""
          }
      }

   for {set iz 0} {$iz < $mzSize} {incr iz} \
      {
       for {set iy 0} {$iy < $mySize} {incr iy} \
          {
           set aInfo($iz,$iy) ""
          }
      }

   set iy 0
   set iz 0
   set ia 0

   foreach s $sList \
      {
       set iList [split $s ","]

       if {$nusDim > 0} {set iy [lindex $iList 0]}
       if {$nusDim > 1} {set iz [lindex $iList 1]}
       if {$nusDim > 2} {set ia [lindex $iList 2]}

       if {$iy >= 0} {lappend yInfo($ia,$iz) $iy}
       if {$iz >= 0} {lappend zInfo($ia,$iy) $iz}
       if {$ia >= 0} {lappend aInfo($iz,$iy) $ia}
      }

   set i  0

   for {set ia 0} {$ia < $aSize} {incr ia} \
      {
       for {set iz 0} {$iz < $zSize} {incr iz} \
          {
           for {set iy 0} {$iy < $ySize} {incr iy} \
              {
               set locInfo($i) "$iy $iz $ia"
               incr i
              }
          }
      }

   return $sList
}

proc getNUSSizes {} \
{
   global fdata xSize ySize zSize aSize qxSize qySize qzSize qaSize

   set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]

   set xSize    [getParmI $fdata NDSIZE CUR_XDIM]
   set ySize    [getParmI $fdata NDSIZE CUR_YDIM]
   set zSize    [getParmI $fdata NDSIZE CUR_ZDIM]
   set aSize    [getParmI $fdata NDSIZE CUR_ADIM]


   if {$dimCount < 4} {set aSize 1}
   if {$dimCount < 3} {set zSize 1}
   if {$dimCount < 2} {set ySize 1}

   set qxState  [getParmI $fdata NDQUADFLAG CUR_XDIM]
   set qyState  [getParmI $fdata NDQUADFLAG CUR_YDIM]
   set qzState  [getParmI $fdata NDQUADFLAG CUR_ZDIM]
   set qaState  [getParmI $fdata NDQUADFLAG CUR_ADIM]

   set qxSize   [getQuadSize $qxState]
   set qySize   [getQuadSize $qyState]
   set qzSize   [getQuadSize $qzState]
   set qaSize   [getQuadSize $qaState]

   if {$qxSize == 1 && $qySize == 2} \
      {
       set ySize [expr $qySize*$ySize]
      }

   if {$dimCount > 1} {set ySize [expr $ySize/$qySize]}
   if {$dimCount > 2} {set zSize [expr $zSize/$qzSize]}
   if {$dimCount > 3} {set aSize [expr $aSize/$qaSize]}

   set n [expr $xSize*$qxSize*$ySize*$qySize*$zSize*$qzSize*$aSize*$qaSize]

   return $n
}

proc updateYSize { fdata yCount } \
{
   global expandFlag ySize mySize qxSize qySize

   if {$expandFlag} return

   if {$yCount < 1} {set yCount 1}

   if {$qxSize == 2 && $qySize == 2} \
      {
       setParm $fdata NDSIZE [expr 2*$yCount] CUR_YDIM
      } \
   else \
      {
       setParm $fdata NDSIZE $yCount CUR_YDIM
      }

   setParm $fdata NDAPOD   $yCount CUR_YDIM
   setParm $fdata NDTDSIZE $yCount CUR_YDIM

   updateHdr $fdata CUR_YDIM
}

proc setInName { iia iiz } \
{
   global dimCount inTemplate zSize aSize qzSize qaSize verbose

   if {$dimCount == 4} \
      {
       set inName  [format $inTemplate  $iia $iiz]

       if {$verbose} \
          {
           set fmt "Input Plane: %3d %3d of %3d %3d.\015"

           puts -nonewline \
            [format $fmt $iia $iiz [expr $qaSize*$aSize] [expr $qzSize*$zSize]]
          }
      } \
   elseif {$dimCount == 3} \
      {
       set inName  [format $inTemplate  $iiz]

       if {$verbose} \
          {
           set fmt "Input Plane: %3d of %3d.\015"
           puts -nonewline [format $fmt $iiz [expr $qzSize*$zSize]]
          }
      } \
   else \
      {
       set inName  $inTemplate
      }

   if {$verbose} {flush stdout}
   if {$verbose > 1} {puts ""}

   return $inName
}

proc setOutName { iia iiz } \
{
   global dimCount outTemplate mzSize maSize qzSize qaSize verbose

   if {$dimCount == 4} \
      {
       set outName [format $outTemplate $iia $iiz]

       if {$verbose} \
          {
           set fmt "Output Plane: %3d %3d of %3d %3d.\015"

           puts -nonewline \
           [format $fmt $iia $iiz [expr $qaSize*$maSize] [expr $qzSize*$mzSize]]
          }
      } \
   elseif {$dimCount == 3} \
      {
       set outName [format $outTemplate $iiz]

       if {$verbose} \
          {
           set fmt "Plane: %3d of %3d.\015"

           puts -nonewline [format $fmt $iiz [expr $qzSize*$mzSize]]
          }
      } \
   else \
      {
       set outName $outTemplate
      }

   if {$verbose} {flush stdout}
   if {$verbose > 1} {puts ""}

   return $outName
}

proc getNUSOffset { ia iaq iz izq iy vSize loc } \
{
   global xSize ySize zSize qxSize qySize qzSize locInfo nusDim 

   set off 0

   if {$nusDim >= 3} \
      {
       set aLoc  [lindex $locInfo($loc) 2]
       set aJump [expr $xSize*$qxSize*$ySize*$qySize*$zSize*$qzSize]
       set off   [expr $off + (2*$aLoc + $iaq)*$aJump]
      }

   if {$nusDim >= 2} \
      {
       set zLoc  [lindex $locInfo($loc) 1]
       set zJump [expr $xSize*$qxSize*$ySize*$qySize]
       set off   [expr $off + (2*$zLoc + $izq)*$zJump]
      }

   if {$nusDim >= 1} \
      {
       set yLoc [lindex $locInfo($loc) 0]
       set off  [expr $off + $yLoc*$vSize]
      }

   return $off
}

#
#------------------------------------------------------------------------------

getSampleParm $parName

if {$nusDim > 3} \
   {
    puts stderr \
     "Error: Only 3 NUS Dimensions Max are Supported, $nusDim Requested."

    exit 1
   }

set inName  [format $inTemplate  1 1]
set outName [format $outTemplate 1 1]

set dirName [file dirname $outName]

if {![file exists $dirName]} \
   {
    catch {file mkdir $dirName}
   }

if {![file isdirectory $dirName]} \
   {
    puts stderr "Error Finding Output Directory $dirName"
    exit 1
   }

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input FID $inName"
    exit 1
   }

#
# Input header:

set error [rdFDATA $inName fdata]

if {$error} \
   {
    puts stderr "Error Reading Header from Input FID $inName"
    exit 1
   }

set dimCount [getParmI $fdata FDDIMCOUNT   NULL_DIM]
 
if {$dimCount < 2 || $dimCount > 4} \
   {
    puts stderr "Error: NUS Support Requires 2, 3, or 4 Dimensions."
    exit 1
   }

set dim1 [getParmI $fdata FDDIMORDER1 NULL_DIM]
set dim2 [getParmI $fdata FDDIMORDER2 NULL_DIM]

if {$dim1 != 2 || $dim2 != 1} {puts stderr "Warning: Data is Transposed."}

getNUSSizes

for {set i 0} {$i < $dimCount} {incr i} \
   {
    set dim [lindex {X Y Z A B C D} $i]

    set sw($dim)  [getParm    $fdata NDSW    [format CUR_%sDIM $dim]]
    set obs($dim) [getParm    $fdata NDOBS   [format CUR_%sDIM $dim]]
    set car($dim) [getParm    $fdata NDCAR   [format CUR_%sDIM $dim]]
    set lab($dim) [getParmStr $fdata NDLABEL [format CUR_%sDIM $dim]]
   }

set sList [getSampleSchedule $sampleName $nusDim]

#
# Allocate memory.

set vSize [expr $qxSize*$qySize*$xSize]

switch $dimCount \
   {
    2 {set bSize [expr $vSize*$ySize]}
    3 {set bSize [expr $vSize*$ySize*$zSize*$qzSize]}
    4 {set bSize [expr $vSize*$ySize*$zSize*$qzSize*$aSize*$qaSize]}
   } 

set error [catch {set fdataP [sysMalloc $FDATASIZE  float]}]
set error [catch {set fBuff  [sysMalloc $FDATASIZE  float]}]
set error [catch {set pBuff  [sysMalloc $mySize     float]}]
set error [catch {set vBuff  [sysMalloc $vSize      float]}]
set error [catch {set tBuff  [sysMalloc $bSize      float]}]

if {$error} \
   {
    puts stderr "Error Allocating Shuffling Buffers."
    exit 1
   }

if {$verbose} \
   {
    puts ""
    puts -nonewline "V: $vSize"
    if {$dimCount > 0} {puts -nonewline " X: $xSize QX: $qxSize"}
    if {$dimCount > 1} {puts -nonewline " Y: $ySize $mySize QY: $qySize"}
    if {$dimCount > 2} {puts -nonewline " Z: $zSize $mzSize QZ: $qzSize"}
    if {$dimCount > 3} {puts -nonewline " A: $aSize $maSize QA: $qaSize"}
    puts ""
   }

#
# Adjust output header:

if {$expandFlag} \
   {
    if {$nusDim >= 1} \
       {
        if {$qxSize == 2 && $qySize == 2} \
           {
            setParm $fdata NDSIZE [expr 2*$mySize] CUR_YDIM
           } \
        else \
           {
            setParm $fdata NDSIZE $mySize CUR_YDIM
           }

        setParm $fdata NDAPOD   $mySize CUR_YDIM
        setParm $fdata NDTDSIZE $mySize CUR_YDIM

        updateHdr $fdata CUR_YDIM
       }

    if {$nusDim >= 2} \
       {
        setParm $fdata NDSIZE [expr $qzSize*$mzSize] CUR_ZDIM

        setParm $fdata NDAPOD   $mzSize CUR_ZDIM
        setParm $fdata NDTDSIZE $mzSize CUR_ZDIM

        updateHdr $fdata CUR_ZDIM
       }

    if {$nusDim >= 3} \
       {
        setParm $fdata NDSIZE [expr $qaSize*$maSize] CUR_ADIM

        setParm $fdata NDAPOD   $mzSize CUR_ADIM
        setParm $fdata NDTDSIZE $mzSize CUR_ADIM

        updateHdr $fdata CUR_ADIM
       }
   }

#
# Read input:

set inPtr $tBuff
set iia 0
set nn  0

for {set ia 0} {$ia < $aSize} {incr ia} \
   {
for {set iaq 0} {$iaq < $qaSize} {incr iaq} \
   {
    incr iia
    set  iiz 0

    for {set iz 0} {$iz < $zSize} {incr iz} \
       {
    for {set izq 0} {$izq < $qzSize} {incr izq} \
       {
        incr iiz

        set inName [setInName $iia $iiz]

        set n      [expr $xSize*$qxSize*$ySize*$qySize]
        set inUnit [sysOpen $inName r]

        sysRead  $inUnit $fBuff $FDATASIZE
        sysRead  $inUnit $inPtr $n
        sysClose $inUnit

        set inPtr [incrPtr $inPtr $n]]
        incr nn $n
       }
       }
   }
   }

set minVal [vMin $tBuff $nn]
set maxVal [vMax $tBuff $nn]

setParm $fdata FDMIN       $minVal NULL_DIM
setParm $fdata FDMAX       $maxVal NULL_DIM
setParm $fdata FDSCALEFLAG 1.0     NULL_DIM

if {$verbose} {puts ""}

#
# Write the data in sorted order:

set iia 0

for {set ia 0} {$ia < $maSize} {incr ia} \
   {
for {set iaq 0} {$iaq < $qaSize} {incr iaq} \
   {
    incr iia
    set  iiz 0

    for {set iz 0} {$iz < $mzSize} {incr iz} \
       {
    for {set izq 0} {$izq < $qzSize} {incr izq} \
       {
        incr iiz

        set outName [setOutName $iia $iiz]

        set error [catch {set outUnit [sysOpen $outName w]}]

        if {$error} \
           {
            puts stderr "Error Opening 2D Output File $outName"
            exit 1
           }

        updateYSize $fdata [llength $yInfo($ia,$iz)]

        sysWrite $outUnit $fdata $FDATASIZE

        for {set iy 0} {$iy < $mySize} {incr iy} \
           {
            set iList [join [lrange "$iy $iz $ia" 0 [expr $nusDim - 1]] ","]
            set loc   [lsearch -exact $sList $iList]

            if {$loc < 0} \
               {
                if {!$expandFlag} continue
                vsSet $vBuff 0.0 $vSize

                if {$verbose > 1} \
                   {
                    set fmt "   Input: %4s Output: %4d Min: % .4e Max: % .4e"
                    puts [format $fmt None [expr $iy + 1] 0.0 0.0]
                   }
               } \
            else \
               {
                set off [getNUSOffset $ia $iaq $iz $izq $iy $vSize $loc] 
                set src [incrPtr $tBuff $off]

                vvMove $vBuff $src $vSize 

                if {$verbose > 1} \
                   {
                    set minVal [vMin $vBuff $vSize]
                    set maxVal [vMax $vBuff $vSize]

                    set fmt "   Input: %4d Output: %4d Min: % .4e Max: % .4e"

                    puts \
                    [format $fmt [expr $loc + 1] [expr $iy + 1] $minVal $maxVal]
                   }
               }

            sysWrite $outUnit $vBuff $vSize
           }

        sysClose $outUnit
       }
       }
    }
    }

#
# Create the sampling schedule output:

if {![strcasecmp $profName None]} {exit 0}

set thisName [format $profName 1]
set dirName  [file dirname $thisName]

if {![file exists $dirName]} {catch {file mkdir $dirName}}

if {![file isdirectory $dirName]} \
   {
    puts stderr "Error Finding Sampling Schedule Output Directory $dirName"
    exit 1
   }

switch -exact $nusDim \
   {
    1 {
       buildHdr $fdataP -ndim $nusDim -aq2D Real \
          -xMODE Real   -xFT  Time    -xN   $mySize -xT   $mySize \
          -xSW   $sw(Y) -xOBS $obs(Y) -xCAR $car(Y) -xLAB $lab(Y)
      }

    2 {
       buildHdr $fdataP -ndim $nusDim -aq2D Real \
          -xMODE Real   -xFT  Time    -xN   $mySize -xT   $mySize \
          -xSW   $sw(Y) -xOBS $obs(Y) -xCAR $car(Y) -xLAB $lab(Y) \
          -yMODE Real   -yFT  Time    -yN   $mzSize -yT   $mzSize \
          -ySW   $sw(Z) -yOBS $obs(Z) -yCAR $car(Z) -yLAB $lab(Z)
      }

    3 {
       buildHdr $fdataP -ndim $nusDim -aq2D Real \
          -xMODE Real   -xFT  Time    -xN   $mySize -xT   $mySize \
          -xSW   $sw(Y) -xOBS $obs(Y) -xCAR $car(Y) -xLAB $lab(Y) \
          -yMODE Real   -yFT  Time    -yN   $mzSize -yT   $mzSize \
          -ySW   $sw(Z) -yOBS $obs(Z) -yCAR $car(Z) -yLAB $lab(Z) \
          -zMODE Real   -zFT  Time    -zN   $maSize -zT   $maSize \
          -zSW   $sw(A) -zOBS $obs(A) -zCAR $car(A) -zLAB $lab(A)
      }
   }
  
setParm $fdataP FDMIN       0.0 NULL_DIM
setParm $fdataP FDMAX       1.0 NULL_DIM
setParm $fdataP FDSCALEFLAG 1.0 NULL_DIM

for {set ia 0} {$ia < $maSize} {incr ia} \
   {
    set thisName [format $profName [expr $ia + 1]]
    set outUnit  [sysOpen $thisName w]

    if {$error} \
       {
        puts stderr "Error Opening Sampling Schedule Output File $thisName"
        exit 1
       }

    sysWrite $outUnit $fdataP $FDATASIZE

    for {set iz 0} {$iz < $mzSize} {incr iz} \
       { 
        vsSet $pBuff 0.0 $mySize

        foreach iy $yInfo($ia,$iz) \
          {
           setPtrVal $pBuff $iy 1.0    
          }

        sysWrite $outUnit $pBuff $mySize
       } 

    sysClose $outUnit
   }

exit 0
