#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Save NMRPipe processed 3D or 4D result in multiple orientations."
   echo " -in     inName  [ft/spec%03d.ft3]  Input Spectrum."
   echo " -out    outName [spec%03d.ft3]     Output Template."
   echo " -outDir outDir  [.]                Output Directory."
   echo " -orient orList  [xy xz yz]         List of Output Orientations."
   echo "Removing Existing Results:"
   echo " -clean                             Delete Existing Results First."
   echo " -cleanOnly                         Delete Existing Results and Exit."
   echo "Reporting:"
   echo " -progress                          Display Progress Bar (Default)."
   echo " -noprogress                        No Progress Bar Graphics."
   echo " -verb                              Verbose Mode ON (Default)."
   echo " -noverb                            Verbose Mode OFF."
   echo "Notes:"
   echo " 1. Each output orientation will be saved in its own subdirectory,"
   echo "    for example xz/spec%04d.ft3"
   echo " 2. The output template and orient list will be adjusted automatically"
   echo "    for 4D input data."
   echo " 3. The output directory will be created if needed."
   exit 0
endif

set vFlag      = 1
set progFlag   = 1
set orientList = None
set sendName   = nmrProgress.tcl 

if (`flagLoc $argv -verb`) then
   set vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   set vFlag = 0
endif

if (`flagLoc $argv -progress`) then
   set progFlag = 1
   set sendName = nmrProgress.tcl 
endif

if (`flagLoc $argv -noprogress`) then
   set progFlag = 0
   set sendName = None
endif

set inName    = (`getArgD $argv -in ft/spec%03d.ft3`)
set outDir    = (`getArgD $argv -out "."`)
set thisName  = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
   echo "Orient Error Finding Input $thisName"
   exit 1
endif

set dimCount = (`getParm -in $thisName -parm FDDIMCOUNT -fmt %.0f`)

if ($vFlag != 0) then
   echo "Orient Input: $inName Dimensions: $dimCount"
endif

set outName1D  = (`getArgD $argv -out1D spec.ft1`)
set outName2D  = (`getArgD $argv -out2D spec.ft2`)
set outName3D  = (`getArgD $argv -out3D spec%04d.ft3`)
set outName4D  = (`getArgD $argv -out4D spec%04d_%04d.ft4`)

if ($dimCount == 1) then
   set orientList = (x)
   set outName = $outName1D
else if ($dimCount == 2) then
   set orientList = (xy yx)
   set outName = $outName2D
else if ($dimCount == 3) then
   set orientList = (xy xz yz)
   set outName = $outName3D
else if ($dimCount == 4) then
   set orientList = (xy xz xa yz ya za)
   set outName = $outName4D
else
   echo "Orient Error: Unsupported Dimension Count $dimCount."
   exit 1
endif

set outName    = (`getArgD $argv -out $outName`)
set orientList = (`getListArgD $argv -orient -- $orientList`)

if (`flagLoc $argv -clean`) then
   foreach orient ($orientList)
      if (-d $outDir/$orient) then
         if ($vFlag != 0) then
            echo " Orient Cleaning Output Directory $outDir/$orient"
         endif

         /bin/rm -rf $outDir/$orient
      endif
   end
endif

if (`flagLoc $argv -cleanOnly`) then
   if ($vFlag != 0) then
      echo " Orient Clean-Only Mode. Exiting."
   endif

   exit 0
endif

if (!(-e $outDir)) then
   mkdir $outDir
endif

if (!(-e $outDir)) then
   echo "Orient Error Creating Output Directory $outDir"
   exit 1
endif

if ($progFlag != 0) then
   echo "Stopping any existing progress bar displays ..."
   nmrProgress.com -stop -noprogress
   echo "Starting new progress bar display ..."
   nmrProgress.tcl &
   sleep 3
   echo "Done starting new progress bar display."
endif

foreach orient ($orientList) 
  switch( $orient$dimCount )
     case x1:
        nmrPipe -in $inName -out $outDir/${orient}_$outName -ov
        breaksw

     case xy2:
        nmrPipe -in $inName -out $outDir/${orient}_$outName -ov
        breaksw
     case xz2:
        nmrPipe -fn TP -in $inName -out $outDir/${orient}_$outName -ov
        breaksw

     case xy3:
        xyz2pipe -in $inName -x \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -x 
        breaksw

     case yx3:
        xyz2pipe -in $inName -y \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -x 
        breaksw

     case xz3:
        xyz2pipe -in $inName -z \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -y 
        breaksw

     case zx3:
       xyz2pipe -in $inName -z \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -x
        breaksw

     case yz3:
        xyz2pipe -in $inName -x \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -z
        breaksw

     case zy3:
        xyz2pipe -in $inName -x \
        | nmrPipe -fn ZTP       \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -x
        breaksw

     case xy4:
        xyz2pipe -in $inName -x \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -x 
        breaksw

     case yx4:
        xyz2pipe -in $inName -y \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -x 
        breaksw

     case xz4:
        xyz2pipe -in $inName -z \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -y 
        breaksw

     case zx4:
       xyz2pipe -in $inName -z \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -x
        breaksw

     case xa4:
        xyz2pipe -in $inName -a \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -y 
        breaksw

     case ax4:
       xyz2pipe -in $inName -a \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -x
        breaksw

     case yz4:
        xyz2pipe -in $inName -x \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -z
        breaksw

     case zy4:
        xyz2pipe -in $inName -x \
        | nmrPipe -fn ZTP       \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -x
        breaksw

     case ya4:
        xyz2pipe -in $inName -a \
        | nmrPipe -fn TP        \
        | nmrPipe -fn ZTP       \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -x
        breaksw

     case ay4:
        xyz2pipe -in $inName -a \
        | nmrPipe -fn TP        \
        | nmrPipe -fn ZTP       \
        | nmrSend -fn SEND -send $sendName -title Export $orient \
        | pipe2xyz -out $outDir/$orient/$outName -ov -y
        breaksw

     case za4:
        breaksw

     case az4:
        breaksw

     default:
        echo "Orient Error: Unsupported Orientation ${dimCount}D $orient." 
        breaksw
  endsw 
end
