#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"
 
set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
 puts stderr "Create an Empty Table of Dipolar Couplings for a PDB:"
 puts stderr " -in       inName  \[ref.pdb\]    PDB Table Input."
 puts stderr " -out      outName \[dcNull.tab\] DC Table Output."
 puts stderr " -dcInfo   dcInfo  \[HN,N ...\]   Couplings to List (Append +1 or -1 for RESID)."
 puts stderr " -useChain                      Include ChainID Information in Output."
 puts stderr " -hdr                           Display Header (Default)."
 puts stderr " -nohdr                         No Header Output."
 puts stderr " -seq                           Display DATA SEQUENCE with Header (Default)."
 puts stderr " -noseq                         No DATA SEQUENCE Display."
 puts stderr "Notes:"
 puts stderr " 1. When specifying coupling info, append -1 or +1 to the atom names"
 puts stderr "    to specify an I-1 or I+1 coupling.  For example, for a N(I) to C(I-1)"
 puts stderr "    coupling, use: N,C-1."
 puts stderr " 2. Use # or * in atom names, for example: HA#,CA."
 puts stderr " 2. Use hash or star in atom names, for example: HAhash,CA HAstar,CA."
 puts stderr " 3. To add DD and W Values:"
 puts stderr "    scaleDI.tcl -in dc.tab -pdb ref.pdb -out dcAdj.tab -calcDI -calcDD -calcW"
 exit 1
} 

set inName    ref.pdb
set outName   dcNull.tab 
set outUnit   stdout 
set dcInfo    HN,N
set chainFlag 0

set hdrFlag   1
set seqFlag   1
 
getArgD     $ARGV -in     inName
getArgD     $ARGV -out    outName
getListArgD $ARGV -dcInfo dcInfo

if {[flagLoc $ARGV -useChain]}   {set chainFlag 1}
if {[flagLoc $ARGV -nouseChain]} {set chainFlag 0}

if {[flagLoc $ARGV -hdr]}        {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]}      {set hdrFlag 0}

if {[flagLoc $ARGV -seq]}        {set seqFlag 1}
if {[flagLoc $ARGV -noseq]}      {set seqFlag 0}

set n [string length $dcInfo]

for {set i 0} {$i < $n} {incr i} \
   {
    set loc [string first hash $dcInfo] 
    if {$loc < 0} break
    set dcInfo [string replace $dcInfo $loc [expr $loc + 3] "#"] 
   }

set n [string length $dcInfo]

for {set i 0} {$i < $n} {incr i} \
   {
    set loc [string first star $dcInfo]
    if {$loc < 0} break
    set dcInfo [string replace $dcInfo $loc [expr $loc + 3] "*"]
   }

set dcInfo [string trim $dcInfo "{}"]

#
#-----------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "PDB2DC Error Finding PDB Input $inName"
    exit 1
   }

if {![strcasecmp $outName None] || ![strcasecmp $outName stdout]} \
   {
    set outUnit stdout
   } \
elseif {![strcasecmp $outName stderr]} \
   {
    set outUnit stderr
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error Opening DC Output File."
        exit 1
       }
   }

set pdbTab [gdbRead table -in $inName -pdb -hetatm]
set r1     [getTabMin $pdbTab RESID]
set rN     [getTabMax $pdbTab RESID]

if {$chainFlag} \
   {
    set chainList [getTabUniqVals $pdbTab CHAINID]
   } \
else \
   {
    set chainList None
   }

if {$hdrFlag} \
   {
    if {$seqFlag} {puts $outUnit [getPDBSeqStr $pdbTab]} 
      
    if {$chainFlag} \
       { 
        puts $outUnit "VARS   RESID_I RESNAME_I ATOMNAME_I CHAINNAME_I RESID_J RESNAME_J ATOMNAME_J CHAINNAME_J D"
        puts $outUnit "FORMAT %5d     %6s       %6s        %6s         %5d     %6s       %6s        %6s         %9.3f"
        puts $outUnit ""
       } \
    else \
       {
        puts $outUnit "VARS   RESID_I RESNAME_I ATOMNAME_I RESID_J RESNAME_J ATOMNAME_J D"
        puts $outUnit "FORMAT %5d     %6s       %6s        %5d     %6s       %6s        %9.3f"
        puts $outUnit ""
       }
   }
 
foreach dcPair $dcInfo \
   {
    set dcList [split $dcPair ","]

    set typeI  [lindex $dcList 0]
    set typeJ  [lindex $dcList 1]

    set atomI  [lindex [split $typeI "+-"] 0]
    set atomJ  [lindex [split $typeJ "+-"] 0]

    set offI   0
    set offJ   0

    set pLoc [string first "+" $typeI]
    set mLoc [string first "-" $typeI]

    if {$pLoc > 0} {set offI [string range $typeI $pLoc end]}
    if {$mLoc > 0} {set offI [string range $typeI $mLoc end]}

    set pLoc [string first "+" $typeJ]
    set mLoc [string first "-" $typeJ]

    if {$pLoc > 0} {set offJ [string range $typeJ $pLoc end]}
    if {$mLoc > 0} {set offJ [string range $typeJ $mLoc end]}

    if {![strcmp $offI +1]} {set offI 1}
    if {![strcmp $offJ +1]} {set offJ 1}

    foreach thisChain $chainList \
       {
        for {set resID $r1} {$resID <= $rN} {incr resID} \
           {
            set resIDI [expr $resID + $offI]
            set resIDJ [expr $resID + $offJ]

            set cI "RESID == $resIDI && atomMatch( ATOMNAME, '$atomI' )"
            set cJ "RESID == $resIDJ && atomMatch( ATOMNAME, '$atomJ' )"

            if {[strcasecmp $thisChain None] && [strcasecmp $thisChain "(Null)"]} \
               {
                append cI " && strmatch( CHAINID, '$thisChain' )"
                append cJ " && strmatch( CHAINID, '$thisChain' )"
               }

            set eListI [gdbSelect -from $pdbTab -cond $cI]
            set eListJ [gdbSelect -from $pdbTab -cond $cJ]

            set nI [llength $eListI]
            set nJ [llength $eListJ]

            if {!$nI || !$nJ} continue
    
            set eI [lindex $eListI 0]
            set eJ [lindex $eListJ 0]

            set atomNameI  [gdbGet $eI ATOMNAME]
            set atomNameJ  [gdbGet $eJ ATOMNAME]

            set resNameI   [gdbGet $eI RESNAME]
            set resNameJ   [gdbGet $eJ RESNAME]

            set chainNameI [gdbGet $eI CHAINID]
            set chainNameJ [gdbGet $eJ CHAINID]

            set thisAtomI $atomNameI
            set thisAtomJ $atomNameJ

            if {$nI > 1} {set thisAtomI [string range $atomNameI 0 end-1]#}
            if {$nJ > 1} {set thisAtomJ [string range $atomNameJ 0 end-1]#}

            if {$chainFlag} \
               {
                set fmt " %5d %6s %6s %6s  %5d %6s %6s %6s  %9.3f"
                puts $outUnit [format $fmt $resIDI $resNameI $thisAtomI $chainNameI  $resIDJ $resNameJ $thisAtomJ $chainNameJ 0.0]
               } \
            else \
               {
                set fmt " %5d %6s %6s  %5d %6s %6s  %9.3f"
                puts $outUnit [format $fmt $resIDI $resNameI $thisAtomI  $resIDJ $resNameJ $thisAtomJ 0.0]
               }
           }
       }
   }

# scaleDI.tcl -pdb $pdbName -in $outName -out $fixName -calcDI -calcDD -calcW -keepDI -keepDD -keepW -verb

if {[strcasecmp $outUnit stdout] && [strcasecmp $outUnit stderr]} \
   {
    catch "close $outUnit"
   }

exit 0
