#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Create Synthetic PDB Data of Randomly Oriented Vectors:"
    puts stderr " -out     outName \[rand.pdb\]  PDB Output Name."
    puts stderr " -atomI   atomI   \[HN\]        Atom Name I."
    puts stderr " -atomJ   atomJ   \[N\]         Atom Name J."
    puts stderr " -rad     radius  \[1.041\]     Distance Between Atom I and J, Angstroms."
    puts stderr " -resID   resID   \[1\]         First Residue ID."
    puts stderr " -resName resName \[ALA\]       Residue Name."
    puts stderr " -chain   chain   \[None\]      Chain Name, or Keyword Auto or None."
    puts stderr " -iseed   iseed   \[654321\]    Random Number Seed, 0 for Auto."
    puts stderr "Arrangement of Vectors:"
    puts stderr " -nX    nX        \[10\]        Number of Vectors Along X-Axis."
    puts stderr " -nY    nY        \[10\]        Number of Vectors Along Y-Axis."
    puts stderr " -nZ    nZ        \[10\]        Number of Vectors Along Z-Axis."
    puts stderr " -x0    x0        \[0.000\]     Initial X-Axis Coord, Angstroms."
    puts stderr " -y0    y0        \[0.000\]     Initial Y-Axis Coord, Angstroms."
    puts stderr " -z0    z0        \[0.000\]     Initial Z-Axis Coord, Angstroms."
    puts stderr " -dx    dx        \[4.000\]     X-Axis Step Between Vectors, Angstroms."
    puts stderr " -dy    dy        \[4.000\]     Y-Axis Step Between Vectors, Angstroms."
    puts stderr " -dz    dz        \[4.000\]     Z-Axis Step Between Vectors, Angstroms."
    exit 0
   }

set outName rand.pdb 
set outUnit stdout

set resID   1 
set atomI   HN
set atomJ   N
set resName ALA
set chain   None 
set rad     1.041
set iseed   654321
set iterMax 20

set nX      10
set nY      10
set nZ      10

set dx      4.0
set dy      4.0
set dz      4.0

set x0      0.0
set y0      0.0
set z0      0.0

getArgD $ARGV -out     outName
getArgD $ARGV -resName resName
getArgD $ARGV -resID   resID

getArgD $ARGV -atomI   atomI
getArgD $ARGV -atomJ   atomJ
getArgD $ARGV -chain   chain
getArgD $ARGV -rad     rad
getArgD $ARGV -iseed   iseed

getArgD $ARGV -nX nX
getArgD $ARGV -nY nY
getArgD $ARGV -nZ nZ

getArgD $ARGV -dx dx 
getArgD $ARGV -dy dy 
getArgD $ARGV -dz dz 

getArgD $ARGV -x0 x0
getArgD $ARGV -y0 y0 
getArgD $ARGV -z0 z0 

srand $iseed

#
#---------------------------------------------------------------------------------

proc getRandVec { thisX0 thisY0 thisZ0 thisR } \
{
   global iterMax 

   set xV  $thisX0
   set yV  [expr $thisY0 + $thisR]
   set zV  $thisZ0

   set rIJ 0.0

   for {set i 0} {$i < $iterMax} {incr i} \
      {
       set xI [expr 2.0*([rand] - 0.5)]
       set yI [expr 2.0*([rand] - 0.5)]
       set zI [expr 2.0*([rand] - 0.5)]

       set xJ [expr 2.0*([rand] - 0.5)]
       set yJ [expr 2.0*([rand] - 0.5)]
       set zJ [expr 2.0*([rand] - 0.5)]

       set xI 0.0
       set yI 0.0
       set zI 0.0

       set xJ [expr 2.0*[rand -gauss]]
       set yJ [expr 2.0*[rand -gauss]]
       set zJ [expr 2.0*[rand -gauss]]

       set xIJ [expr $xJ - $xI]
       set yIJ [expr $yJ - $yI]
       set zIJ [expr $zJ - $zI]

       set rIJ  [expr sqrt($xIJ*$xIJ + $yIJ*$yIJ + $zIJ*$zIJ)]
       if {$rIJ >= 0.01} break
      }

   if {$rIJ >= 0.01} \
      {
       set xV [expr $thisX0 + $thisR*$xIJ/$rIJ]
       set yV [expr $thisY0 + $thisR*$yIJ/$rIJ]
       set zV [expr $thisZ0 + $thisR*$zIJ/$rIJ]
      }

   return "$xV $yV $zV"
}

if {![strcasecmp $outName None] || ![strcasecmp $outName stdout]} \
   {
    set outUnit stdout
   } \
elseif {![strcasecmp $outName stderr]} \
   {
    set outUnit stderr
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error Opening PDB Output File."
        exit 1
       }
   }

set chainList {A B C D E F G H I J K L P N O P Q R S T U V W X Y Z}
set nC [llength $chainList]

dynInit -fast

if {$nX < 1} {set nx 1}
if {$nY < 1} {set nY 1}
if {$nZ < 1} {set nZ 1} 

set nN [expr $nX*$nY*$nZ]

if {$nN > 1000000} \
   {
    puts stderr "Error: Maximum Atom Count Exceeded ($nN Requested, 100000 Max).
    exit 1
   }

set thisResID $resID
set thisChain None
set zLocJ     $z0
set atomID    0

for {set iz 1} {$iz <= $nZ} {incr iz} \
   {
    if {![strcasecmp $chain None]} \
       {
        set thisChain None
       } \
    elseif {![strcasecmp $chain Auto]} \
       {
        if {$nZ <= $nC} \
           {
            set loc [expr $iz - 1]
           } \
        else \
           {
            set loc [expr int($nC*($iz - 1)/double($nZ))]
            if {$loc > $nC - 1} {set loc [expr $nC - 1]}
           }

        set thisChain [lindex $chainList $loc]
       } \
    else \
       {
        set thisChain $chain
       }

    set yLocJ $y0

    for {set iy 1} {$iy <= $nY} {incr iy} \
       {
        set xLocJ $x0

        for {set ix 1} {$ix <= $nX} {incr ix} \
           {
            set coordList [getRandVec $xLocJ $yLocJ $zLocJ $rad]

            set xLocI [lindex $coordList 0]
            set yLocI [lindex $coordList 1]
            set zLocI [lindex $coordList 2]

            if {![strcasecmp $thisChain None]} \
               {
                incr atomID
                puts $outUnit "[dynGetPDBStr -x $xLocI -y $yLocI -z $zLocI -atomID $atomID -atomName $atomI -resID $resID -resName $resName]"

                incr atomID
                puts $outUnit "[dynGetPDBStr -x $xLocJ -y $yLocJ -z $zLocJ -atomID $atomID -atomName $atomJ -resID $resID -resName $resName]"
               } \
            else \
               {
                incr atomID
                puts $outUnit "[dynGetPDBStr -x $xLocI -y $yLocI -z $zLocI -atomID $atomID -atomName $atomI -resID $resID -resName $resName -chain $thisChain]"

                incr atomID
                puts $outUnit "[dynGetPDBStr -x $xLocJ -y $yLocJ -z $zLocJ -atomID $atomID -atomName $atomJ -resID $resID -resName $resName -chain $thisChain]"
               }

            incr resID

            set xLocJ [expr $xLocJ + $dx]
           }

        set yLocJ [expr $yLocJ + $dy]
       }

    set zLocJ [expr $zLocJ + $dz]
   }

puts $outUnit "END"

if {[strcasecmp $outUnit stdout] && [strcasecmp $outUnit stderr]} \
   {
    catch "close $outUnit"
   }

exit 0
