#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"
 
set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
 puts stderr "Transformation of PDB by Rotation, Translation, or Matrix:"
 puts stderr " -in       inName    \[ref.pdb\]  PDB Table Input."
 puts stderr " -out      outName   \[stdout\]   PDB Table Output."
 puts stderr " -m        tMatrix   \[1.0 ...\]  Initial Transform matrix."
 puts stderr " -t        tInfo     \[None\]     Additional Transform Steps."
 puts stderr " -center                        Center Final Result."
 puts stderr " -seg      segName   \[None\]     Segment Name for Result."
 puts stderr " -chain    chainName \[None\]     Chain Name for Result."
 puts stderr " -report   rInfo     \[None\]     Report Keywords: matrix euler N"
 puts stderr "Notes: "
 puts stderr " 1. The initial transform matrix is the identity matrix." 
 puts stderr " 2. Transform matrix requires 9 (3x3) or 16 (4x4) values."
 puts stderr " 3. Transform steps are supplied as keyword/value pairs,"
 puts stderr "    and are performed in the order given."
 puts stderr ""
 puts stderr "       Translation Keywords: tx ty tz"
 puts stderr "       Rotation Keywords:    rx ry rz"
 puts stderr ""
 puts stderr " 4. Rotation Example:"
 puts stderr "       pdbTrans.tcl -t rx 90 ry 45 rz 135  ..."
 exit 1
} 
 
set m         "1.0 0.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0 0.0 1.0"
set inName    ref.pdb
set outName   stdout
set segName   None
set chainName None
set tInfo     None
set rInfo     None

getArgD $ARGV -in    inName
getArgD $ARGV -out   outName
getArgD $ARGV -seg   segName
getArgD $ARGV -chain chainName

getListArgD $ARGV -m      m
getListArgD $ARGV -t      tInfo 
getListArgD $ARGV -report rInfo 

#
#--------------------------------------------------------------------------

set pdbSrc None

if {![strcasecmp $outName None]} \
   {
    set outUnit stderr
   } \
else \
   {
    set outUnit stdout
   }

if {[llength $m] != 9 && [llength $m] != 16} \
   {
    puts stderr "Error: -m matrix must have 9 or 16 elements."
    exit 1
   }

#
# Read and adjust PDB file if needed:

if {[strcasecmp $inName None]} \
   {
    if {![file exists $inName]} \
       {
        puts stderr "Error Finding Input PDB $inName"
        exit 1
       }

    dynInit -fast

    set pdbSrc [dynRead -pdb -in $inName]

    if {![strcasecmp $chainName None]} \
       {
        dynSetChain -src $pdbSrc -chain ""
       } \
    else \
       {
        dynSetChain -src $pdbSrc -chain $chainName 
       }

    if {![strcasecmp $segName None]} \
       {
        dynSetSeg -src $pdbSrc -seg ""
       } \
    else \
       {
        dynSetSeg -src $pdbSrc -seg $segName
       }
   }

#
# Build the transformation:

dynTransform -reset

dynTransform -m $m

if {[strcasecmp $tInfo None]} \
   {
    set n [llength $tInfo]

    for {set i 0} {$i < $n} {incr i} \
       {
        set s [string tolower [lindex $tInfo $i]] 

        switch -exact $s \
           {
            tx -
            ty -
            tz -
            rx -
            ry -
            rz {
                incr i
                
                if {$i >= $n} \
                   {
                    puts stderr "Error Extracting Value for Transform $s."
                    continue
                   }

                dynTransform -$s [lindex $tInfo $i]
               }

            default {puts stderr "Error: Unknown Transform Keyword: $s"}
           }
       }
   }

#
# Apply the transformation if needed:

if {![strcasecmp $pdbSrc None]} \
   {
    set mm [dynTransform -verb]
   } \
else \
   {
    if {[flagLoc $ARGV -center]} \
       {
        set mm [dynTransform -src $pdbSrc -apply -all -verb -center]
       } \
    else \
       {
        set mm [dynTransform -src $pdbSrc -apply -all -verb]
       }
   }

#
# Report the results.

set nn [llength $mm]

if {[strcasecmp $rInfo None]} \
   {
    set n [llength $rInfo]

    for {set i 0} {$i < $n} {incr i} \
       {
        set s [string tolower [lindex $rInfo $i]]

        switch -exact $s \
           {
            matrix  {puts $outUnit [lrange $mm 0 [expr $nn - 2]]}
            euler   {puts $outUnit [dynEuler [lrange $mm 0 [expr $nn - 2]]]}
            n       {puts $outUnit [lindex $mm end]}
            default {puts stderr "Error: Unknown Report Keyword: $s"}
           }
       }
   }

#
# Write the PDB data if needed.

if {[strcasecmp $pdbSrc None]} \
   {
    if {[strcasecmp $outName None]} \
       {
        if {[strcasecmp $outName stdout]} \
           {
            dynWrite -src $pdbSrc -pdb -out $outName
           } \
        else \
           {
            dynWrite -src $pdbSrc -pdb
           }
       }
   }

exit 0
