#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert a 2D NMRPipe All-Real Spectrum into a TIFF."
    puts " -in      inName  \[test.ft2\]   2D NMRPipe Format Input."
    puts " -out     outName \[Auto\]       TIFF RGB Output."
    puts " -min     minVal  \[Default\]    Lower Spectral Height Range."
    puts " -max     maxVal  \[Default\]    Upper Spectral Height Range."
    puts " -tiff16                       Output 16-Bits per Channel." 
    puts " -verb                         Verbose Mode ON (Default)."
    puts " -noverb                       Verbose Mode OFF."
    puts "Notes:"
    puts " 1. Use % for -max and -min to set values as a percentage of"
    puts "    the largest absolute intensity (Example: -min 20%)."
    puts " 2. Use \"-out Auto\" to create output name by adding/replacing"
    puts "    input name extension with \".tif\""
    exit 1
   }

set inName     test.ft2 
set outName    Auto 
set minVal     Default
set maxVal     Default
set tiffFlag   -tiff

set verbose    1
set bitsPerPix 8

getArgD $ARGV -in  inName
getArgD $ARGV -out outName

getArgD $ARGV -min minVal
getArgD $ARGV -max maxVal

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}
 
if {[flagLoc $ARGV -tiff16]} \
   {
    set tiffFlag   -tiff16
    set bitsPerPix 16
   }

if {![strcasecmp $outName Auto]} \
   {
    set outName [file root $inName].tif
   }

#
#------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input Spectrum $inName"
    exit 1
   }

set error [rdFDATA $inName fdata]

if {$error} \
   {
    puts stderr "Error Reading Header from Input Spectrum $inName"
    exit 1
   }

set dimCount [getParmI $fdata FDDIMCOUNT   NULL_DIM]
set tpFlag   [getParmI $fdata FDTRANSPOSED NULL_DIM]

if {$dimCount < 2} \
   {
    puts stderr "Error: Input Spectrum Must be 2D or Greater."
    exit 1
   }

if {$tpFlag} \
   {
    readROI -in $inName -roi 1 -ndim 2 -x Y_AXIS 0% 100% -y X_AXIS 0% 100% 
   } \
else \
   {
    readROI -in $inName -roi 1 -ndim 2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100%
   }

set rInfo [getROIInfo -roi 1]
set rPtr  [lindex $rInfo 0]
set sList [lindex $rInfo 2]
set xSize [lindex $sList 0]
set ySize [lindex $sList 1]
set n     [expr $xSize*$ySize]

set specMin [vMin $rPtr $n]
set specMax [vMax $rPtr $n]
set maxAbs  [expr abs($specMax)]

if {abs($specMin) > $maxAbs} {set maxAbs [expr abs($specMin)]}

if {![strcasecmp $minVal Default]} {set minVal $specMin}
if {![strcasecmp $maxVal Default]} {set maxVal $specMax}

if {[string first % $minVal] == [string length $minVal] - 1} \
   {
    set minVal [string trim $minVal %]
    set minVal [expr $minVal*$maxAbs/100.0]
   }

if {[string first % $maxVal] == [string length $maxVal] - 1} \
   {
    set maxVal [string trim $maxVal %]
    set maxVal [expr $maxVal*$maxAbs/100.0]
   }

set range   [expr $maxVal - $minVal]
set pMax    [expr int(pow(2,$bitsPerPix)) - 1]

if {$range == 0.0} \
   {
    set iz 0
   } \
else \
   {
    set iz [expr int($pMax*(0.0 - $minVal)/$range)]
   }

if {$verbose} {puts  "Spectrum Input: $inName Image Output: $outName"}

if {$verbose} \
   {
    set fmt "Spectrum Min: %.3e Max: %.3e X-Size: %d Y-Size: %d"

    puts [format $fmt $specMin $specMax $xSize $ySize]

    puts "" 
    puts [format "Image %5d is Spectrum % 6.3e" 0 $minVal]
    puts [format "Image %5d is Spectrum  Zero" $iz]
    puts [format "Image %5d is Spectrum % 6.3e" $pMax $maxVal]
   }

writeImage $tiffFlag \
           -rRMin $minVal -rRMax $maxVal \
           -rGMin $minVal -rGMax $maxVal \
           -rBMin $minVal -rBMax $maxVal \
           -roiR 1 -roiG 1 -roiB 1 -out $outName -ov 

exit 0
