#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set thisVersion "2022.333.19.15"

if {[flagLoc $ARGV -help]} \
   {
    puts "Simulate Time-Domain Data from NMRPipe Peak Table (Version $thisVersion)."
    puts "Input:"
    puts " -in        inName  \[master.tab\] Input Peak Table."
    puts " -fid       fidName \[Auto\]       Corresponding Input Time-Domain Data."
    puts " -fidScript fName   \[fid.com\]    Corresponding Conversion Script."
    puts " -ft        ftName  \[Auto\]       Corresponding Input Spectrum."
    puts " -series                         Input is a Series (Such as Pseudo-3D)."
    puts " -noseries                       Input is Not a Series."
    puts "Output:"
    puts " -out       outName \[Auto\]       Output Time-Domain Data."
    puts " -simScript sName   \[simFID.com\] Output Script to Generate Time-Domain Simulation." 
    puts " -title     title   \[None\]       Title for Output Time-Domain Data."
    puts " -prefix    prefix  \[tmppk\$\$\]    Prefix for Temporary Files (\$\$ is Process ID)."
    puts " -exec                           Execute the Simulation Script (Default)."
    puts " -noexec                         Do Not Execute the Simulation Script."
    puts "Addition of a Synthetic Solvent Signal:"
    puts " -useH2O                         Add Synthetic Solvent Signal (Default for HN Data)."
    puts " -nouseH2O                       No Synthetic Solvent Signal."
    puts " -p0MinH2O  p0Min   \[-6.0\]       Lower Bound for Solvent Phase Distortion, Degrees."
    puts " -p0MinH2O  p0Max   \[-3.0\]       Upper Bound for Solvent Phase Distortion, Degrees."
    puts " -dxMinH2O  dxMin   \[-2.0\]       Lower Bound for Solvent Position Offset, Points."
    puts " -lwH2O     lw      \[2.0\]        Linewidth for Solvent Signal, Points."
    puts " -hiH2O     hiScale \[1.0\]        Scale Factor for Solvent Signal Height."
    puts "Axis-Specific Simulation Parameters (-x -y -z -a):"
    puts " -xName     xName   \[x\]          Axis Name Prefix."
    puts " -xFTARG    ftArg   \[None\]       Fourier Transform Options (None alt neg alt,neg)."
    puts " -xCT       nCT     \[0\]          Number of Constant Time Points."
    puts " -xExp                           Use Exponential Decay (Default)."
    puts " -xGauss                         Use Gaussian Decay."
    puts "Other Simulation Parameters:"
    puts " -df        dfVal   \[Auto\]       Digital Oversampling Value, Points."
    puts " -iseed     iseed   \[37238547\]   Random Number Seed, 0 for Auto."
    puts " -noise     noise   \[0.0\]        Random Noise Level."
    puts " -ts                             Scale Time-Domain by Integral of Decay (Default)."
    puts " -nots                           Do Not Scale Time-Domain."
    puts " -tdd                            Interpret Linewidth as Time-Domain Decay."
    puts " -notdd                          Interpret Linewidth in Frequency-Domain (Default)."
    puts "Additional Arguments for simTimeND (Should be Last on Command Line):"
    puts " -simArgs   simArgs \[None\]       Optional Arguments for simTimeND."
    exit 0
   }

#
#---------------------------------------------------------------------------------------------------------------------------------------------

proc extractScriptCmnd { inName cmndList } \
{
   set cmnd ""

   set error [catch {set inUnit [open $inName r]}]
   if {$error} {return $cmnd}

   while {[gets $inUnit lineOrig] >= 0} \
      {
       set line [string trim $lineOrig]
       set key  None

       foreach thisCmnd $cmndList \
          {
           set loc [string first $thisCmnd $line]

           if {$loc >= 0} \
              { 
               set line [string range $line $loc end]
               set key  $thisCmnd
               break
              }
          }

       if {![strcasecmp $key None]} continue
       
       set cmnd  [string trim $line "\\"]
       set lastC [string index $line end]

       while {![strcmp $lastC "\\"]} \
          {
           if {[gets $inUnit lineOrig] < 0} break 
          
           set line [string trim $lineOrig]

           append cmnd " " 
           append cmnd [string trim $line "\\"]

           set lastC   [string index $lineOrig end]
          }

       catch "close $inUnit"

       return $cmnd
      }

   catch "close $inUnit"

   return $cmnd
}

proc getValInRange { minVal maxVal } \
{
   global gFactor

   if {$minVal == $maxVal} {return $minVal}

   set mid [expr 0.5*($maxVal + $minVal)]
   set val [expr $mid + $gFactor*($maxVal - $minVal)*[rand -gauss]]

   if {$val < $minVal} {set val $minVal}
   if {$val > $maxVal} {set val $maxVal}

   return $val
}

set simArgs None

set loc [flagLoc $ARGV -simArgs]

if {$loc} \
   {
    set simArgs [lrange $ARGV [expr $loc + 1] end]
    set ARGV    [lrange $ARGV 0 [expr $loc - 1]]
   }

set inName         master.tab
set outName        Auto
set fidName        Auto
set ftName         Auto
set title          None

set prefix         tmppk[pid]
set execFlag       1
set seriesFlag     Auto
set tauList        ""
set tauCount       0

set fidScriptName  fid.com
set simScriptName  simFID.com

set dimCount       Auto
set useH2OFlag     Auto 
set df             Auto
set grpdly         None

set gFactor        0.35

set p0MinH2O      -6.0
set p0MaxH2O      -3.0

set dxMinH2O      -2.0
set dxMaxH2O      -1.5
set hiH2O          1.0
set lwH2O          2.0

set tddArg         "notdd"
set tsArg          "ts"

set noise          0.0

set iseed          37238547

getArgD $ARGV -in        inName
getArgD $ARGV -out       outName
getArgD $ARGV -fid       fidName
getArgD $ARGV -ft        ftName
getArgD $ARGV -title     title
getArgD $ARGV -ndim      dimCount

getArgD $ARGV -fidScript fidScriptName
getArgD $ARGV -simScript simScriptName

if {[flagLoc $ARGV -tdd]}   {set tddArg "tdd"}
if {[flagLoc $ARGV -notdd]} {set tddArg "notdd"}

if {[flagLoc $ARGV -ts]}    {set tsArg "ts"}
if {[flagLoc $ARGV -nots]}  {set tsArg "nots"}

if {[flagLoc $ARGV -series]}   {set seriesFlag 1}
if {[flagLoc $ARGV -noseries]} {set seriesFlag 0}

if {[flagLoc $ARGV -exec]}    {set execFlag 1}
if {[flagLoc $ARGV -noexec]}  {set execFlag 0}

foreach v {prefix df p0MinH2O p0MaxH2O dxMinH2O dxMaxH2O hiH2O lwH2O iseed noise gFactor} \
   {
    getArgD $ARGV -$v $v
   }

set logName ${prefix}.log

getArgD $ARGV -log logName

foreach dim {x y z a} \
   {
    set dimName($dim)  $dim

    set p0($dim)       Auto
    set p1($dim)       Auto

    set tSize($dim)    Auto
    set fSize($dim)    Auto
    set qSize($dim)    Auto
    set ctSize($dim)   0

    set lab($dim)      Auto
    set sw($dim)       Auto
    set car($dim)      Auto
    set obs($dim)      Auto
    set mode($dim)     Complex
    set modelArg($dim) Exp

    set minPts($dim)   0.0
    set maxPts($dim)   0.0
    set minPPPM($dim)  0.0
    set maxPPPM($dim)  0.0

    set ftArg($dim) None
    set x1($dim)    0
    set xn($dim)    0 

    getArgD $ARGV -${dim}Name  dimName($dim)
    getArgD $ARGV -${dim}N     fSize($dim)
    getArgD $ARGV -${dim}T     tSize($dim)
    getArgD $ARGV -${dim}CT    ctSize($dim)
    getArgD $ARGV -${dim}MODE  mode($dim)
    getArgD $ARGV -${dim}SW    sw($dim)
    getArgD $ARGV -${dim}CAR   car($dim)
    getArgD $ARGV -${dim}OBS   obs($dim)
    getArgD $ARGV -${dim}LAB   lab($dim)
    getArgD $ARGV -${dim}EXTX1 x1($dim)
    getArgD $ARGV -${dim}EXTXN xn($dim)
    getArgD $ARGV -${dim}P0    p0($dim)
    getArgD $ARGV -${dim}P1    p1($dim)
    getArgD $ARGV -${dim}FTARG ftArg($dim)

    if {[flagLoc $ARGV -${dim}Exp]}   {set modelArg($dim) Exp}
    if {[flagLoc $ARGV -${dim}Gauss]} {set modelArg($dim) Gauss}
   }

if {![file exists $inName]} \
   {
    puts "pk2FID. Error Finding Input Peak Table Table $inName"
    exit 1
   }

set t      [gdbRead table -in $inName]

set eList  [gdbGetInfo $t -parms entries]
set vList  [gdbGetInfo $t -parms varNames]

set fidCmnd   None
set pseudoDim None
set dimList   ""

set fdataF  0
set fdataT  0

if {[lsearch -exact $vList Y_A0] >= 0} {set pseudoDim y}
if {[lsearch -exact $vList Z_A0] >= 0} {set pseudoDim z}
if {[lsearch -exact $vList A_A0] >= 0} {set pseudoDim a}

if {![strcasecmp $seriesFlag Auto]} \
   {
    if {![strcasecmp $pseudoDim None]} \
       {
        set seriesFlag 0
       } \
    else \
       {
        set seriesFlag 1
       }
   }

if {![strcasecmp $dimCount Auto]} \
   {
    set dimCount 0

    if {[lsearch -exact $vList X_AXIS] >= 0} {set dimCount 1}
    if {[lsearch -exact $vList Y_AXIS] >= 0} {set dimCount 2}
    if {[lsearch -exact $vList Z_AXIS] >= 0} {set dimCount 3}
    if {[lsearch -exact $vList A_AXIS] >= 0} {set dimCount 4}

    if {$seriesFlag} {incr dimCount}

    if {[strcasecmp $pseudoDim None]} \
       {
        if {[lsearch -exact "x y z a" $pseudoDim] != $dimCount - 1} \
           {
            puts "pk2FID. Error: Mismatch in _AXIS and _A0 series Variables in Input $inName"
            exit 1
           }
       }
   }

if {$dimCount == 0} \
   {
    puts "pk2FID. Error Finding Variables X_AXIS ... in Input $inName"
    exit 1
   }

set dimList [lrange "x y z a" 0 [expr $dimCount - 1]]

foreach dim $dimList \
   {
    set minPts($dim) [getTabStat getMin $t [string toupper $dim]_AXIS]
    set maxPts($dim) [getTabStat getMax $t [string toupper $dim]_AXIS]
    set minPPM($dim) [getTabStat getMin $t [string toupper $dim]_PPM]
    set maxPPM($dim) [getTabStat getMax $t [string toupper $dim]_PPM]
   }

if {[strcasecmp $fidScriptName None]} \
   {
    if {![file exists $fidScriptName]} \
       {
        puts "pk2FID. Error Finding Input Script $fidScriptName"
        exit 1
       }

    set fidCmnd [extractScriptCmnd $fidScriptName "bruk2pipe var2pipe"]

    if {![llength $fidCmnd]} \
       {
        puts "pk2FID. Error Finding Conversion Command in Input Script $fidScriptName"
        exit 1
       }

    if {![strcasecmp $fidName Auto]} {getArgD $fidCmnd -out  fidName}

    if {$seriesFlag} \
       {
        set thisDimList "x"
       } \
    else \
       {
        set thisDimList $dimList
       }

    foreach dim $thisDimList \
       {
        if {[strcmp $dim x] && ![strcasecmp $tSize($dim) Auto]} {getArgD $fidCmnd -$dimName($dim)T   tSize($dim)}

        if {![strcasecmp $sw($dim)    Auto]} {getArgD $fidCmnd -$dimName($dim)SW  sw($dim)}
        if {![strcasecmp $obs($dim)   Auto]} {getArgD $fidCmnd -$dimName($dim)OBS obs($dim)}
        if {![strcasecmp $car($dim)   Auto]} {getArgD $fidCmnd -$dimName($dim)CAR car($dim)}
        if {![strcasecmp $lab($dim)   Auto]} {getArgD $fidCmnd -$dimName($dim)LAB lab($dim)}

        set thisMode None

        getArgD $fidCmnd -$dimName($dim)MODE thisMode

        if {![strcasecmp $qSize($dim) Auto]} \
           {
            if {![strcasecmp $thisMode Real] || ![strcasecmp $thisMode TPPI] || ![strcasecmp $thisMode Sequential]} \
               {
                set qSize($dim) 1
               } \
            else \
               {
                set qSize($dim) 1
               }
           }
       }

    if {![strcasecmp $df Auto]} \
       {
        if {[flagLoc $fidCmnd -df]} \
           {
            getArgD $fidCmnd -df df
           } \
        elseif {[flagLoc $fidCmnd -grpdly]} \
           {
            getArgD $fidCmnd -grpdly grpdly

            if {[string is double $grpdly]} \
               {
                if {($grpdly > 0.0) && ([string first . $grpdly] >= 0)} {set df $grpdly} 
               }
           }
       } 
   }

if {![strcasecmp $fidName Auto]} \
   {
    if {$dimCount >= 3} \
       {
        if {[file exists data/test001.fid]} \
           {
            set fidName data/test%03d.fid
           } \
        else \
           {
            set fidName fid/test%03d.fid
           }
       } \
    else \
       {
        set fidName test.fid
       }
   }

if {![strcasecmp $outName Auto]} \
   {
    if {$dimCount >= 3} \
       {
        set outName sim/test%03d.fid
       } \
    else \
       {
        set outName sim.fid
       } 
   }

if {![strcasecmp $ftName Auto]} \
   {
    if {$dimCount >= 3} \
       {
        set ftName ft/test%03d.ft$dimCount
       } \
    else \
       {
        set ftName test.ft$dimCount 
       }
   }

if {[strcasecmp $fidName None]} \
   {
    set thisName [format $fidName 1 1]

    if {![file exists $thisName]} \
       {
        puts "pk2FID. Error Finding Time-Domain Input $fidName ($thisName)."
        exit 1
       }

    set error [rdFDATA $fidName fdataT]

    if {$error} \
       {
        puts "pk2FID. Error Reading Header in Empty Time-Domain Input $fidName"
        exit 1
       }

    foreach dim $dimList \
       {
        if {![strcasecmp $tSize($dim) Auto]} {set tSize($dim) [getParmI   $fdataT NDAPOD  CUR_[string toupper $dim]DIM]}
        if {![strcasecmp $sw($dim)    Auto]} {set sw($dim)    [getParm    $fdataT NDSW    CUR_[string toupper $dim]DIM]}
        if {![strcasecmp $obs($dim)   Auto]} {set obs($dim)   [getParm    $fdataT NDOBS   CUR_[string toupper $dim]DIM]}
        if {![strcasecmp $car($dim)   Auto]} {set car($dim)   [getParm    $fdataT NDCAR   CUR_[string toupper $dim]DIM]}
        if {![strcasecmp $lab($dim)   Auto]} {set lab($dim)   [getParmStr $fdataT NDLABEL CUR_[string toupper $dim]DIM]}
       }

    if {![strcasecmp $qSize($dim) Auto]} {set $qSize($dim) [getQuadSize [getParmI $fdataT NDQUADFLAG CUR_[string toupper $dim]DIM]]}
    if {![strcasecmp $df          Auto]} {set df           [getParm $fdataT FDDMXVAL NULL_DIM]}

    set dimCountT [getParmI $fdataT FDDIMCOUNT NULL_DIM]
   }

if {[strcasecmp $ftName None]} \
   {
    set thisName [format $ftName 1 1]

    if {![file exists $thisName]} \
       {
        puts "pk2FID. Error Finding Freq-Domain Input $ftName ($thisName)."
        exit 1
       }

    set error [rdFDATA $ftName fdataF]
    
    if {$error} \
       {
        puts "pk2FID. Error Reading Header in Freq-Domain Input $ftName"
        exit 1
       }

    foreach dim $dimList \
       {
        set x1($dim) [getParmI $fdataF NDX1     CUR_[string toupper $dim]DIM]
        set xn($dim) [getParmI $fdataF NDXN     CUR_[string toupper $dim]DIM]
        set nT       [getParmI $fdataF NDTDSIZE CUR_[string toupper $dim]DIM]
        set nF       [getParmI $fdataF NDFTSIZE CUR_[string toupper $dim]DIM]

        if {$x1($dim) == 0 || $xn($dim) == 0} \
           {
            set x1($dim) 0
            set xn($dim) [expr $nF - 1]
           }

        if {![strcasecmp $tSize($dim) Auto]} {set tSize($dim) [getParmI   $fdataF NDTDSIZE CUR_[string toupper $dim]DIM]}
        if {![strcasecmp $fSize($dim) Auto]} {set fSize($dim) [getParmI   $fdataF NDFTSIZE CUR_[string toupper $dim]DIM]}
        if {![strcasecmp $sw($dim)    Auto]} {set sw($dim)    [getParm    $fdataF NDSW     CUR_[string toupper $dim]DIM]}
        if {![strcasecmp $obs($dim)   Auto]} {set obs($dim)   [getParm    $fdataF NDOBS    CUR_[string toupper $dim]DIM]}
        if {![strcasecmp $car($dim)   Auto]} {set car($dim)   [getParm    $fdataF NDCAR    CUR_[string toupper $dim]DIM]}
        if {![strcasecmp $p0($dim)    Auto]} {set p0($dim)    [getParm    $fdataF NDP0     CUR_[string toupper $dim]DIM]}
        if {![strcasecmp $p1($dim)    Auto]} {set p1($dim)    [getParm    $fdataF NDP1     CUR_[string toupper $dim]DIM]}
        if {![strcasecmp $lab($dim)   Auto]} {set lab($dim)   [getParmStr $fdataF NDLABEL  CUR_[string toupper $dim]DIM]}

        if {![strcasecmp $sw($dim) Auto] && $x1($dim) != 0 && $xn($dim) != 0} \
           {
            set sw($dim) [expr (double($n)/(1 + $xn($dim) - $x1($dim)))*[getParm $fdataF NDSW CUR_[string toupper $dim]DIM]]
           }
       }

    set dimCountF [getParmI $fdataT FDDIMCOUNT NULL_DIM]
   }

if {!$tauCount} {set tauList ""}

if {[flagLoc $fidCmnd -xT]} {getArgD $fidCmnd -xT tSize(x)}

set errList ""

foreach dim $dimList \
   {
    if {![strcasecmp $tSize($dim) Auto]} {lappend errList ${dim}T}
    if {![strcasecmp $sw($dim)    Auto]} {lappend errList ${dim}SW}
    if {![strcasecmp $obs($dim)   Auto]} {lappend errList ${dim}OBS}
    if {![strcasecmp $car($dim)   Auto]} {lappend errList ${dim}CAR}

    if {![strcasecmp $lab($dim)   Auto]} {set lab($dim) [string toupper $dim]}
    if {![strcasecmp $p0($dim)    Auto]} {set p0($dim)  0.0}
    if {![strcasecmp $p1($dim)    Auto]} {set p1($dim)  0.0}
   }

if {[llength $errList]} \
   {
    puts "pk2FID. Error Finding Parameters $errList."
    exit 1
   }

foreach dim $dimList \
   {
    if {![strcasecmp $fSize($dim) Auto]} {set fSize($dim) [nextPower2 [expr int(0.5+$maxPts($dim))]]}
   }

if {[flagLoc $ARGV -useH2O]}   {set useH2OFlag 1}
if {[flagLoc $ARGV -nouseH2O]} {set useH2OFlag 0}

if {![strcasecmp $useH2OFlag Auto]} \
   {
    if {[string first HN [string toupper $lab(x)]] >= 0} \
       {
        set useH2OFlag 1
       } \
    else \
       {
        set useH2OFlag 0
       }
   }

set lastDim  [lindex "x y z a" [expr $dimCount - 1]]
set fmtCount [getFmtCount $outName]

if {$seriesFlag && $fmtCount == 0} \
   {
    puts "pk2FID. Error: Series Data Requires an Output Template, Such as ft/test%03d.dat"
    exit 1
   }

set thisXMin [expr (1 + int($fSize(x)/2)) + $dxMinH2O]
set thisXMax [expr (1 + int($fSize(x)/2)) + $dxMaxH2O]

srand $iseed

set error [catch {set outUnit [open $simScriptName w 0744]}]

if {$error} \
   {
    puts "pk2FID. Error Opening Output Script $simScriptName"
    exit 1
   }

if {$seriesFlag && [strcasecmp $ftName None]} \
   {
    for {set i 1} {$i <= $tSize($lastDim)} {incr i} \
       {
        set thisName [format $ftName $i]
        set tau      0.0

        if {[file exists $thisName]} \
           {
            set error [rdFDATA $thisName thisFDATA]
            if {$error} continue

            set tau [getParm $thisFDATA FDTAU NULL_DIM]

            sysFree $thisFDATA
           }

        lappend tauList $tau
       }
   }

foreach tau $tauList \
   {
    if {$tau != 0.0} \
       {
        set tauCount [llength $tauList]
        break
       }
   }

if {!$tauCount} {set tauList ""}

set hiScale 1.0

puts $outUnit "#!/bin/csh"
puts $outUnit ""
puts $outUnit "set inName  = (`getArgD \$argv -in    $inName`)"
puts $outUnit "set outName = (`getArgD \$argv -out   $outName`)"
puts $outUnit "set title   = (`getArgD \$argv -title $title`)"
puts $outUnit "set noise   = (`getArgD \$argv -noise $noise`)"

puts $outUnit ""

if {$tauCount} \
   {
    puts -nonewline $outUnit "set tauList = ("

    for {set i 0} {$i < $tauCount} {incr i} \
       {
        if {$i} {puts -nonewline $outUnit " "}
        puts -nonewline $outUnit "[format %.4e [lindex $tauList $i]]"
       }

    puts $outUnit ")"
    puts $outUnit ""
   }

if {![strcasecmp $simArgs None]} {set simArgs ""}

if {[lsearch -exact $simArgs -verb] < 0 || [lsearch -exact $simArgs -noverb] < 0} {lappend simArgs "-verb"}


if {$seriesFlag} \
   {
    set specDimCount [expr $dimCount - 1]
    set scaleVarStr  " -scaleVar \$thisVar"
    set indentStr    "   "

    set thisDir [file dirname $outName]

    if {[strcmp $thisDir .]} \
       {
        puts $outUnit "if (!(-e $thisDir)) then"
        puts $outUnit "   mkdir $thisDir"
        puts $outUnit "endif"
        puts $outUnit ""
       }

    puts $outUnit "@ n = $tSize($lastDim)"
    puts $outUnit "@ i = 1"
    puts $outUnit "@ j = 0"
    puts $outUnit ""
    puts $outUnit "while (\$i <= \$n)"
    puts $outUnit "   set thisName = (`nmrPrintf \$outName \$i`)"
    puts $outUnit "   set thisVar  = (`nmrPrintf [string toupper $lastDim]_A%d    \$j`)"
    puts $outUnit ""
   } \
else \
   {
    set specDimCount $dimCount
    set scaleVarStr  ""
    set indentStr    ""
   }

if {$specDimCount == 1} \
   {
    set hiScale  [expr 1.0/$tSize(x)]
    set scaleStr [format "-scale %.4e$scaleVarStr" $hiScale]

    puts $outUnit "${indentStr}simTimeND $simArgs -ndim $specDimCount -in \$inName -noise \$noise $scaleStr -xOff $x1(x) -$tddArg -$tsArg \\"
    puts $outUnit [format "%s -xN     %10s \\" $indentStr $fSize(x)]
    puts $outUnit [format "%s -xT     %10s \\" $indentStr $tSize(x)]
    puts $outUnit [format "%s -xMODE  %10s \\" $indentStr $mode(x)]
    puts $outUnit [format "%s -xSW    %10s \\" $indentStr [format %.3f $sw(x)]]
    puts $outUnit [format "%s -xOBS   %10s \\" $indentStr [format %.3f $obs(x)]]
    puts $outUnit [format "%s -xCAR   %10s \\" $indentStr [format %.4f $car(x)]]
    puts $outUnit [format "%s -xLAB   %10s \\" $indentStr $lab(x)]
    puts $outUnit [format "%s -xP0    %10s \\" $indentStr [format %.1f $p0(x)]]
    puts $outUnit [format "%s -xP1    %10s \\" $indentStr [format %.1f $p1(x)]]
    puts $outUnit [format "%s -xFTARG %10s \\" $indentStr $ftArg(x)]
    puts $outUnit [format "%s -x%-16s \\"      $indentStr $modelArg(x)]
   } \
elseif {$specDimCount == 2} \
   {
    set hiScale  [expr 1.0/($tSize(x)*$tSize(y))]
    set scaleStr [format "-scale %.4e$scaleVarStr" $hiScale]

    puts $outUnit "${indentStr}simTimeND $simArgs -ndim $specDimCount -in \$inName -noise \$noise $scaleStr -xOff $x1(x) -$tddArg -$tsArg -aq2D States \\"
    puts $outUnit [format "%s -xN     %10s  -yN     %10s \\" $indentStr $fSize(x)             $fSize(y)]
    puts $outUnit [format "%s -xT     %10s  -yT     %10s \\" $indentStr $tSize(x)             $tSize(y)]
    puts $outUnit [format "%s -xCT    %10s  -yCT    %10s \\" $indentStr $ctSize(x)            $ctSize(y)]
    puts $outUnit [format "%s -xMODE  %10s  -yMODE  %10s \\" $indentStr $mode(x)              $mode(y)]
    puts $outUnit [format "%s -xSW    %10s  -ySW    %10s \\" $indentStr [format %.3f $sw(x)]  [format %.3f $sw(y)]]
    puts $outUnit [format "%s -xOBS   %10s  -yOBS   %10s \\" $indentStr [format %.3f $obs(x)] [format %.3f $obs(y)]]
    puts $outUnit [format "%s -xCAR   %10s  -yCAR   %10s \\" $indentStr [format %.3f $car(x)] [format %.3f $car(y)]]
    puts $outUnit [format "%s -xLAB   %10s  -yLAB   %10s \\" $indentStr $lab(x)               $lab(y)]
    puts $outUnit [format "%s -xP0    %10s  -yP0    %10s \\" $indentStr [format %.1f $p0(x)]  [format %.1f $p0(y)]]
    puts $outUnit [format "%s -xP1    %10s  -yP1    %10s \\" $indentStr [format %.1f $p1(x)]  [format %.1f $p1(y)]]
    puts $outUnit [format "%s -xFTARG %10s  -yFTARG %10s \\" $indentStr $ftArg(x)             $ftArg(y)]
    puts $outUnit [format "%s -x%-16s  -y%-16s \\"      $indentStr $modelArg(x)          $modelArg(y)]
   } \
elseif {$specDimCount == 3} \
   {
    set hiScale  [expr 1.0/($tSize(x)*$tSize(y)*$tSize(z))]
    set scaleStr [format "-scale %.4e$scaleVarStr" $hiScale]

    puts $outUnit "${indentStr}simTimeND $simArgs -ndim $specDimCount -in \$inName -noise \$noise $scaleStr -xOff $x1(x) -$tddArg -$tsArg -aq2D States \\"
    puts $outUnit [format "%s -xN     %10s  -yN     %10s  -zN     %10s \\" $indentStr $fSize(x)             $fSize(y)             $fSize(z)]
    puts $outUnit [format "%s -xT     %10s  -yT     %10s  -zT     %10s \\" $indentStr $tSize(x)             $tSize(y)             $tSize(z)]
    puts $outUnit [format "%s -xCT    %10s  -yCT    %10s  -zCT    %10s \\" $indentStr $ctSize(x)            $ctSize(y)            $ctSize(z)]
    puts $outUnit [format "%s -xMODE  %10s  -yMODE  %10s  -zMODE  %10s \\" $indentStr $mode(x)              $mode(y)              $mode(z)]
    puts $outUnit [format "%s -xSW    %10s  -ySW    %10s  -zSW    %10s \\" $indentStr [format %.3f $sw(x)]  [format %.3f $sw(y)]  [format %.3f $sw(z)]]
    puts $outUnit [format "%s -xOBS   %10s  -yOBS   %10s  -zOBS   %10s \\" $indentStr [format %.3f $obs(x)] [format %.3f $obs(y)] [format %.3f $obs(z)]]
    puts $outUnit [format "%s -xCAR   %10s  -yCAR   %10s  -zCAR   %10s \\" $indentStr [format %.3f $car(x)] [format %.3f $car(y)] [format %.3f $car(z)]]
    puts $outUnit [format "%s -xLAB   %10s  -yLAB   %10s  -zLAB   %10s \\" $indentStr $lab(x)               $lab(y)               $lab(z)]
    puts $outUnit [format "%s -xP0    %10s  -yP0    %10s  -zP0    %10s \\" $indentStr [format %.1f $p0(x)]  [format %.1f $p0(y)]  [format %.1f $p0(z)]]
    puts $outUnit [format "%s -xP1    %10s  -yP1    %10s  -zP1    %10s \\" $indentStr [format %.1f $p1(x)]  [format %.1f $p1(y)]  [format %.1f $p1(z)]]
    puts $outUnit [format "%s -xFTARG %10s  -yFTARG %10s  -zFTARG %10s \\" $indentStr $ftArg(x)             $ftArg(y)             $ftArg(z)]
    puts $outUnit [format "%s -x%-16s  -y%-16s  -z%-16s \\"      $indentStr $modelArg(x)          $modelArg(y)          $modelArg(z)]
   } \
elseif {$specDimCount == 4} \
   {
    set hiScale  [expr 1.0/($tSize(x)*$tSize(y)*$tSize(z)*$tSize(a))]
    set scaleStr [format "-scale %.4e$scaleVarStr" $hiScale]

    puts $outUnit "simTimeND $simArgs -ndim $specDimCount -in \$inName -noise \$noise $scaleStr -xOff $x1(x) -$tddArg -$tsArg -aq2D States \\"
    puts $outUnit [format " -xN     %10s  -yN     %10s  -zN     %10s  -aN     %10s \\" $fSize(x)             $fSize(y)             $fSize(z)             $fSize(a)]
    puts $outUnit [format " -xT     %10s  -yT     %10s  -zT     %10s  -aT     %10s \\" $tSize(x)             $tSize(y)             $tSize(z)             $tSize(a)]
    puts $outUnit [format " -xCT    %10s  -yCT    %10s  -zCT    %10s  -aCT    %10s \\" $ctSize(x)            $ctSize(y)            $ctSize(z)            $ctSize(a)]
    puts $outUnit [format " -xMODE  %10s  -yMODE  %10s  -zMODE  %10s  -aMODE  %10s \\" $mode(x)              $mode(y)              $mode(z)              $mode(a)]
    puts $outUnit [format " -xSW    %10s  -ySW    %10s  -zSW    %10s  -aSW    %10s \\" [format %.3f $sw(x)]  [format %.3f $sw(y)]  [format %.3f $sw(z)]  [format %.3f $sw(a)]]
    puts $outUnit [format " -xOBS   %10s  -yOBS   %10s  -zOBS   %10s  -aOBS   %10s \\" [format %.3f $obs(x)] [format %.3f $obs(y)] [format %.3f $obs(z)] [format %.3f $obs(a)]]
    puts $outUnit [format " -xCAR   %10s  -yCAR   %10s  -zCAR   %10s  -aCAR   %10s \\" [format %.3f $car(x)] [format %.3f $car(y)] [format %.3f $car(z)] [format %.3f $car(a)]]
    puts $outUnit [format " -xLAB   %10s  -yLAB   %10s  -zLAB   %10s  -aLAB   %10s \\" $lab(x)               $lab(y)               $lab(z)               $lab(a)]
    puts $outUnit [format " -xP0    %10s  -yP0    %10s  -zP0    %10s  -aP0    %10s \\" [format %.1f $p0(x)]  [format %.1f $p0(y)]  [format %.1f $p0(z)]  [format %.1f $p0(a)]]
    puts $outUnit [format " -xP1    %10s  -yP1    %10s  -zP1    %10s  -aP1    %10s \\" [format %.1f $p1(x)]  [format %.1f $p1(y)]  [format %.1f $p1(z)]  [format %.1f $p1(a)]]
    puts $outUnit [format " -x%16s  -y%16s  -z%16s  -a%16s \\"       $modelArg(x)          $modelArg(y)          $modelArg(z)          $modelArg(a)]
   } \
else \
   {
    puts "pk2FID. Error. Unsupported Dimension Count $dimCount."
    exit 1
   }

set usePipeFlag 0
set hiSum       0.0

foreach e $eList {set hiSum [expr $hiSum + $hiH2O*$hiScale*abs([gdbGet $e HEIGHT])]}

if {$useH2OFlag} \
   {
    set usePipeFlag 1

    set thisMin [expr $p0(x) + $p0MinH2O]
    set thisMax [expr $p0(x) + $p0MaxH2O]

    puts $outUnit "${indentStr}| nmrPipe -fn MAC -macro \$NMRTXT/simH2O.M \\"
    puts $outUnit "${indentStr}          -var iseed $iseed fSize $fSize(x) p0Min [format %.1f $thisMin] p0Max [format %.1f $thisMax] hi [format %.3e $hiSum] \\"
    puts $outUnit "${indentStr}               xMinPts [format %.2f $thisXMin] xMaxPts [format %.2f $thisXMax] lwPts $lwH2O \\" 
   }

if {$df != 0.0} \
   {
    set usePipeFlag 1
    puts $outUnit "${indentStr}| nmrPipe -fn DMX -df $df -hdr \\"
   }

if {$seriesFlag} \
   {
    puts $outUnit "${indentStr} -out \$thisName -noverb -ov"
    puts $outUnit ""

    puts -nonewline $outUnit "   "
    puts -nonewline $outUnit "sethdr \$thisName -ndim $dimCount -${lastDim}N $tSize($lastDim) -${lastDim}T $tSize($lastDim) -${lastDim}MODE Real"
    puts -nonewline $outUnit " -${lastDim}SW [format %.3f $sw($lastDim)] -${lastDim}OBS [format %.3f $obs($lastDim)] -${lastDim}CAR [format %.3f $car($lastDim)]"
    puts -nonewline $outUnit " -${lastDim}LAB $lab($lastDim)"

    if {$tauCount} {puts -nonewline $outUnit " -tau \$tauList\[\$i\]"}

    puts $outUnit ""
    puts $outUnit ""
    puts $outUnit "   @ i++"
    puts $outUnit "   @ j++"
    puts $outUnit "end"
    puts $outUnit ""
   } \
else \
   {
    if {$usePipeFlag && $specDimCount >= 3} \
       {
        puts $outUnit "${indentStr}| pipe2xyz -out \$outName -ov"
       } \
    else \
       {
        puts $outUnit "${indentStr} -out \$outName -noverb -ov"
       }
   }

puts $outUnit ""
puts $outUnit "if (`string notcsame \$title None`) then"
puts $outUnit "   sethdr \$outName -title \$title"
puts $outUnit "endif"
puts $outUnit ""

catch "close $outUnit"

if {$execFlag} \
   {
    catch "file delete $logName"
    catch "exec $simScriptName >& $logName"

    if {[file exists $logName]} {printFile stdout $logName}

    catch "file delete $logName"
   }

exit 0
