#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
  {
   puts "Detect Peaks in a 2D Spectrum Over One or More Regions:"
   puts " -in   inName    \[test.ft2\]        Input Spectrum."
   puts " -out  tabName   \[test.tab\]        Peak Table Output."
   puts " -tmp  tmpName   \[tmp.tab\]         Temporary Output."
   puts " -reg  regList   \[0% 100% 0% 100%\] List of Analysis Region Limits (pts ppm %)."
   puts " -verb                             Verbose Mode ON (Default)."
   puts " -noverb                           Verbose Mode OFF."
   puts "Noise Analysis:"
   puts " -noiseX1 nx1    \[None\]   X-Axis Start of Signal-Free Region (pts ppm %)."
   puts " -noiseXN nxn    \[None\]   X-Axis End of Signal-Free Region (pts ppm %)."
   puts " -noiseY1 ny1    \[None\]   Y-Axis Start of Signal-Free Region (pts ppm %)."
   puts " -noiseYN nyn    \[None\]   Y-Axis End of Signal-Free Region (pts ppm %)."
   puts " -noise   sigma  \[Auto\]   Noise Value."
   puts "Peak Detection Parameters:"
   puts " -maxVal  maxVal \[Auto\]   Maximum Value in Spectrum."
   puts " -thresh  t      \[Auto\]   Peak Detection Height Threshold."
   puts " -mult    m      \[12.0\]   Multiplication Factor for -thresh Auto."
   puts " -ddx     ddx    \[Auto\]   X-Axis +/- Width of Detection Region, Pts."
   puts " -ddy     ddy    \[Auto\]   Y-Axis +/- Width of Detection Region, Pts."
   puts " -idx     idx    \[Auto\]   X-Axis +/- Width of Interpolation Region, Pts."
   puts " -idy     idy    \[Auto\]   Y-Axis +/- Width of Interpolation Region, Pts."
   puts " -tolx    tolx   \[Auto\]   Maximum Allowable X-Axis Position Change, +/- Pts."
   puts " -toly    toly   \[Auto\]   Maximum Allowable Y-Axis Position Change, +/- Pts."
   puts " -edge                    Expand Analysis Region to Detect Peaks at Edges."
   puts "Options for nmrWish pkFindROI Command (Should be Last on Command Line):"
   puts " -pkArgs  args   \[None\]   Arguments for pkFindROI."
   puts "Notes:"
   puts " 1. Analysis region limits are specified in groups of four values x1 xn y1 yn."
   puts " 2. The -ddx -ddy options specify how many points are considered when"
   puts "    testing whether a given point could be a peak because it is"
   puts "    larger than all of its neighboring points."
   puts " 3. If no noise region is specified via -noiseX1 -noiseXN etc., and"
   puts "    no noise value is specified via the -noise option, the noise will be"
   puts "    estimated automatically from the distribution of spectral intensities."
   puts " 4. If -thresh Auto is used, the threshold will be set to the noise"
   puts "    value times the -mult multiplication factor."
   puts " 5. If a percent value is used for -thresh (such as -thresh 10%) then"
   puts "    the detection threshold will be set as a percentage of the highest"
   puts "    value in the entire spectrum."
   exit 1
  }

set inName     test.ft2
set tabName    test.tab
set tmpName    tmp.tab

set noiseRMS   Auto
set maxVal     Auto
set thresh     Auto
set pkArgs     None

set noiseX1    None
set noiseXN    None
set noiseY1    None
set noiseYN    None

set threshMult 12.0
set regList    "0% 100% 0% 100%"
set ddx        Auto
set ddy        Auto 
set idx        Auto 
set idy        Auto 
set tolx       Auto 
set toly       Auto 

set verbose    1
set edgeFlag   0

set loc [flagLoc $ARGV -pkArgs]

if {$loc > 0} \
   {
    set pkArgs [lrange $ARGV [expr $loc + 1] end]
    set ARGV   [lrange $ARGV 0 [expr $loc - 1]]
   }

if {![strcasecmp $pkArgs None]} {set pkArgs ""}

getListArgD $ARGV -reg regList

getArgD $ARGV -in  inName
getArgD $ARGV -out tabName
getArgD $ARGV -tab tabName

getArgD $ARGV -ddx  ddx
getArgD $ARGV -ddy  ddy
getArgD $ARGV -idx  idx
getArgD $ARGV -idy  idy
getArgD $ARGV -tolx tolx
getArgD $ARGV -toly toly

getArgD $ARGV -noiseX1 noiseX1
getArgD $ARGV -noiseXN noiseXN

getArgD $ARGV -noiseY1 noiseY1
getArgD $ARGV -noiseYN noiseYN

getArgD $ARGV -thresh thresh
getArgD $ARGV -noise  noiseRMS
getArgD $ARGV -maxVal maxVal
getArgD $ARGV -mult   threshMult 

if {[flagLoc $ARGV -edge]}    {set edgeFlag 1}
if {[flagLoc $ARGV -noedge]}  {set edgeFlag 0}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

if {$verbose} \
   {
    set vArg "verb"
   } \
else \
   {
    set vArg "noverb"
   }

#
#--------------------------------------------------------------------------

set fdata 0

if {![file exists $inName]} \
   {
    puts stderr "2D Peak Detection. Error Finding Input Spectrum $inName"
    exit 1
   }

set error [rdFDATA $inName fdata]

if {$error} \
   {
    puts stderr "2D Peak Detection. Error Reading Header from Spectrum $inName"
    exit 1
   }

set xtSize [getParmI $fdata NDTDSIZE CUR_XDIM]
set ytSize [getParmI $fdata NDTDSIZE CUR_YDIM]

set xfSize [getParmI $fdata NDFTSIZE CUR_XDIM]
set yfSize [getParmI $fdata NDFTSIZE CUR_YDIM]

if {$xtSize > 0 && $xfSize > 0} \
   {
    set dxDef   [expr int(sqrt( 0.5*double($xfSize)/double($xtSize) ))]
    set xTolDef [expr 0.5*double($xfSize)/double($xtSize)]

    if {$dxDef < 1}     {set dxDef   1}
    if {$xTolDef < 2.0} {set xTolDef 2.0}
   } \
else \
   {
    set dxDef   1
    set xTolDef 2.0
   }

if {$ytSize > 0 && $yfSize > 0} \
   {
    set dyDef   [expr int(sqrt( 0.5*double($yfSize)/double($ytSize) ))]
    set yTolDef [expr 0.5*double($yfSize)/double($ytSize)]

    if {$dyDef < 1}     {set dyDef   1}
    if {$yTolDef < 2.0} {set yTolDef 2.0}
   } \
else \
   {
    set dyDef   1
    set yTolDef 2.0
   }

set n [llength $regList]

if {$n % 4} \
   {
    puts stderr "2D Peak Detection. Error: Region List -reg Must Have a Multiple of Four Values."
    puts stderr $regList
    exit 1
   }

if {![strcasecmp $ddx  Auto]} {set ddx  $dxDef}
if {![strcasecmp $idx  Auto]} {set idx  $dxDef}
if {![strcasecmp $tolx Auto]} {set tolx $xTolDef}

if {![strcasecmp $ddy  Auto]} {set ddy  $dyDef}
if {![strcasecmp $idy  Auto]} {set idy  $dyDef}
if {![strcasecmp $toly Auto]} {set toly $yTolDef}

#
# Noise estimate:

if {![strcasecmp $noiseX1 None] || ![strcasecmp $noiseXN None] || ![strcasecmp $noiseY1 None] || ![strcasecmp $noiseYN None]} \
   {
    if {![strcasecmp $noiseRMS Auto]} \
       {
        set noiseRMS [estSpecNoiseND $inName]
        set noiseStr " (Auto)"
       } \
    else \
       {
        set noiseStr " (User-Supplied)"
       }
   } \
else \
   {
    readROI -roi 1 -in $inName -ndim 2 -x X_AXIS $noiseX1 $noiseXN -y Y_AXIS $noiseY1 $noiseYN

    set noiseStr " (RMS of X: $noiseX1 to $noiseXN Y: $noiseY1 to $noiseYN)"
    set noiseRMS [getROIStat 1 vRMS]
   }

#
# Detection threshold:

if {![strcasecmp $maxVal Auto]} \
   {
    readROI -roi 1 -in $inName -ndim 2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100%
    set maxVal [getROIStat 1 vMaxAbs]
   }

set threshStr ""

if {![strcasecmp $thresh Auto]} \
   {
    set threshStr " ($threshMult*Noise RMS)"
    set thresh    [expr $threshMult*$noiseRMS]
   }

if {![strcmp [string index $thresh end] "%"]} \
   {
    set threshStr " ($thresh of Max Value)"
    set pct       [string trim $thresh "%"]
    set thresh    [expr $maxVal*$pct/100.0]
   }

if {$verbose} \
   {
    puts "2D Peak Detection. Input:              $inName"
    puts "2D Peak Detection. Max Value:          [format %.3e $maxVal]"
    puts "2D Peak Detection. Noise RMS:          [format %.3e $noiseRMS]$noiseStr"
    puts "2D Peak Detection. Threshold:          [format %.3e $thresh]$threshStr"
    puts "2D Peak Detection. X-Axis Detection:   ddx $ddx idx $idx tolx $tolx"
    puts "2D Peak Detection. Y-Axis Detection:   ddy $ddy idy $idy toly $toly"
    puts ""
   }

#
# Peak detection for each specified analysis region:

set regCount [expr $n/4]
set loc      0

for {set i 1} {$i <= $regCount} {incr i} \
   {
    if {$regCount == 1} \
       {
        set thisName $tabName
       } \
    else \
       {
        set thisName $tmpName
       }

    set rx1 [lindex $regList $loc]
    incr loc

    set rxn [lindex $regList $loc]
    incr loc

    set ry1 [lindex $regList $loc]
    incr loc

    set ryn [lindex $regList $loc]
    incr loc

    set kx1 [expr int(0.5 + [spec2pnt $fdata CUR_XDIM $rx1])]
    set kxn [expr int(0.5 + [spec2pnt $fdata CUR_XDIM $rxn])]

    set ky1 [expr int([spec2pnt $fdata CUR_YDIM $ry1])]
    set kyn [expr int([spec2pnt $fdata CUR_YDIM $ryn])]

    if {$edgeFlag} \
       {
        if {$kyn == $yfSize} \
           {
            set ky1 [expr $ky1 - $ddy]
            set kyn [expr $kyn + 2*$ddy]
           } \
        else \
           {
            set ky1 [expr $ky1 - $ddy]
            set kyn [expr $kyn + $ddy]
           }

        if {$ky1 < 1} {set ky1 1}
       }

    readROI -roi 1 -ndim 2 -in $inName -x X_AXIS $kx1 $kxn -y Y_AXIS $ky1 $kyn

    if {$verbose} \
       {
        set localMin [getROIStat 1 vMin]
        set localMax [getROIStat 1 vMax]
       }

    eval pkFindROI $pkArgs -roi 1 \
         -sigma $noiseRMS -pChi 1.0e-04 -plus $thresh -minus -$thresh \
         -dx      $ddx  $ddy  \
         -idx     $idx  $idy  \
         -tol     $tolx $toly \
         -hiAdj   1.20  1.80  \
         -lw      1.00  1.00  \
         -parent -noreject -mask -out $thisName -$vArg

    set tabInfo($i)   [gdbRead table -in $thisName]
    set thisCount($i) [gdbGetInfo $tabInfo($i) -parms entryCount]

    if {$verbose} \
       {
        if {$edgeFlag} \
           {
            set eStr " Edge-Adjusted"
           } \
        else \
           {
            set eStr ""
           }

        puts "2D Peak Detection. Region $i Limits X:  $rx1 $rxn ($kx1 $kxn)"
        puts "2D Peak Detection. Region $i Limits Y:  $ry1 $ryn ($ky1 $kyn$eStr)"
        puts "2D Peak Detection. Region $i Min:      [format %.3e $localMin]"
        puts "2D Peak Detection. Region $i Max:       [format %.3e $localMax]"
        puts "2D Peak Detection. Region $i Peaks:     $thisCount($i)"
       }
   }

#
# Join peak tables if multiple regions were analyzed:

if {$regCount > 1} \
   {
    for {set i 2} {$i <= $regCount} {incr i} \
       {
        if {!$thisCount($i)} continue
        gdbAppend -dest $tabInfo(1) -src $tabInfo($i)        
       }
   }

set totalCount [gdbGetInfo $tabInfo(1) -parms entryCount]

if {$verbose} \
   {
    puts "2D Peak Detection. Total Peaks:        $totalCount"
    puts "2D Peak Detection. Output:             $tabName"
   }

gdbWrite $tabInfo(1) $tabName

#
# Renumber peaks if multiple tables were joined:

if {$regCount > 1} \
   {
    set t     [gdbRead table -in $tabName]
    set eList [gdbGetInfo $t -parms entries]

    set n     [llength $eList]
    set i     0

    foreach e $eList \
       {
        incr i
        gdbSet $e INDEX $i
       }

    gdbWrite $t $tabName
   }

if {$regCount > 1} {catch "file delete -force $tmpName"}

exit 0
