#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Find the Distance Between Peaks in Table 1 and Closest Peaks in Table 2:"
    puts " -in1     inName1  \[test1.tab\] Input Table 1."
    puts " -in2     inName2  \[test2.tab\] Input Table 2."
    puts " -out     outName  \[dist.tab\]  Output Table, or Keyword None."
    puts " -var     varList  \[X_PPM ...\] Table Variable Names for Computing Distance."
    puts " -norm    normList \[1.0 ...\]   Normalization Factors for Each Dimension."
    puts " -n       count    \[0\]         Consider Only N Largest Peaks in Table 2 (0 for All)."
    puts " -maxDist maxList  \[None ...\]  Maximum Allowable Distances for Each Dimension."
    puts " -title   title    \[None\]      Title for Report Text."
    puts " -report                       Report Mode ON (Print Distance Parameters)."
    puts " -verb                         Verbose Mode ON."
    exit 0
   }
 
set inName1       test1.tab
set inName2       test2.tab
set outName       dist.tab

set normListUser  "1.0  1.0  1.0  1.0"
set distListUser  "None None None None"
set varListUser   "X_PPM Y_PPM Z_PPM A_PPM"

set reportFlag  0
set hdrFlag     1
set verbose     1
set maxCount    0
set title       None

if {[flagLoc $ARGV -report]}   {set reportFlag 1}
if {[flagLoc $ARGV -noreport]} {set reportFlag 0}

getArgD $ARGV -in1 inName1
getArgD $ARGV -in2 inName2
getArgD $ARGV -out outName
getArgD $ARGV -n   maxCount

getArgD $ARGV -title title

getListArgD $ARGV -norm    normListUser
getListArgD $ARGV -maxDist distListUser
getListArgD $ARGV -var     varListUser

if {[flagLoc $ARGV -hdr]}    {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]}  {set hdrFlag 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {![file exists $inName1]} \
   {
    puts "PkDistance Error Finding Input Table $inName1"
    exit 1
   }

if {![file exists $inName2]} \
   {
    puts "PkDistance Error Finding Input Table $inName2"
    exit 1
   }

#
#-----------------------------------------------------------------------------

proc gdbGetSafe { e v } \
{
   if {![strcmp $e 0]} {return 0}

   return [gdbGet $e $v]
}

proc findNearest { e1 } \
{
   global distList maxDistList normList varList varCount eList2 badCount

   set bestDist  1.0e+16
   set badMatch  0
   set bestEntry 0

   set thisEList ""
   set x1List    ""

   foreach v $varList {lappend x1List [gdbGet $e1 $v]}

   set dist 0.0

   for {set i 0} {$i < $varCount} {incr i} \
      {
       set dMax [lindex $maxDistList $i]
       set q    [lindex $normList $i]

       if {[strcasecmp $dMax None]} {set dist [expr $dist + 4.0*$q*$q*$dMax*$dMax]}
      }

   if {$dist != 0.0} {set bestDist [expr sqrt($dist)]}

   foreach e2 $eList2 \
      {
       set vioCount 0

       for {set i 0} {$i < $varCount} {incr i} \
          {
           set dMax [lindex $maxDistList $i]

           if {[strcasecmp $dMax None]} \
              {   
               set var  [lindex $varList $i]
               set x1   [lindex $x1List $i]
               set x2   [gdbGet $e2 $var]

               set dx [expr $x1 - $x2] 

               if {abs($dx) > $dMax} \
                  {
                   incr vioCount
                   break
                  }
              }
          }

       if {!$vioCount} {lappend thisEList $e2}
      }

   foreach e2 $thisEList \
      {
       set dist 0.0

       for {set i 0} {$i < $varCount} {incr i} \
          {
           set q    [lindex $normList $i]
           set x1   [lindex $x1List   $i]
           set x2   [gdbGet $e2 [lindex $varList $i]]

           set dx   [expr $x1 - $x2] 
           set dist [expr $dist + $q*$dx*$q*$dx]
          }

       set dist [expr sqrt($dist)]

       if {$dist < $bestDist} \
          {
           set bestDist  $dist
           set bestEntry $e2
          }
      }

   lappend distList $bestDist

   if {![strcmp $bestEntry 0]} {incr badCount}

   return $bestEntry
}

if {![strcasecmp $outName None]} \
   {
    set outUnit stdout
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error Opening Output Table $outName"
        exit 1
       }
   }

set t1 [gdbRead table -in $inName1]
set t2 [gdbRead table -in $inName2]

if {$maxCount > 0} {gdbSortTab $t2 HEIGHT -down -abs}

set varList1 [gdbGetInfo $t1 -parms varNames]
set varList2 [gdbGetInfo $t2 -parms varNames]

set eList1   [gdbGetInfo $t1 -parms entries]
set eList2   [gdbGetInfo $t2 -parms entries]

if {$maxCount > 0} {set eList2 [lrange $eList2 0 [expr $maxCount - 1]]}

set pkCount1 [llength $eList1]
set pkCount2 [llength $eList2]

set varList      ""
set maxDistList  ""
set normList     "" 
set varCount     0

set matchList ""
set distList  ""

set usedList    ""
set reusedCount 0
set badCount    0

set i 0

foreach v $varListUser \
   {
    if {[lsearch -exact $varList1 $v] < 0 || [lsearch -exact $varList2 $v] < 0} continue 

    lappend varList     $v
    lappend normList    [lindex $normListUser $i]
    lappend maxDistList [lindex $distListUser $i]

    incr i
   }

set varCount [llength $varList]

if {$verbose && $hdrFlag} \
   {
    puts $outUnit ""
    puts $outUnit "#"
    puts $outUnit "# PkDistance. Input 1: $inName1  Input 2: $inName2 Title: $title"
    puts $outUnit "#"

    puts -nonewline $outUnit "# Variables:      "
    foreach v $varList     {puts -nonewline $outUnit [format " %-8s" $v]}
    puts $outUnit ""

    puts -nonewline $outUnit "# Normalization:  "
    foreach v $normList    {puts -nonewline $outUnit [format " %-8s" $v]}
    puts $outUnit ""

    puts -nonewline $outUnit "# Max Difference: "
    foreach v $maxDistList {puts -nonewline $outUnit [format " %-8s" $v]}
    puts $outUnit ""

    puts -nonewline $outUnit "# Distance:        sqrt( "

    for {set i 0} {$i < $varCount} {incr i} \
       {
        puts -nonewline $outUnit "([lindex $normList $i]*([lindex $varList $i]1-[lindex $varList $i]2))^2"
        if {$i < $varCount - 1} {puts -nonewline $outUnit " + "}
       }

    puts $outUnit " )"
    puts $outUnit ""
   }

foreach e1 $eList1 \
   {
    set eMatch [findNearest $e1]

    if {[strcmp $eMatch 0]} \
       {
        if {[lsearch -exact $matchList $eMatch] >= 0} {incr reusedCount}
       }

    lappend matchList $eMatch
   }

set totalDist [getSum $distList]
set nzCount   0

foreach d $distList \
   {
    if {$d != 0.0} {incr nzCount}
   }

for {set i 0} {$i < $varCount} {incr i} {set dxInfo($i) ""}

set hiInfo1 ""
set hiInfo2 ""
set difInfo ""

set j 0

foreach e1 $eList1 \
   {
    set e2 [lindex $matchList $j]

    for {set i 0} {$i < $varCount} {incr i} {lappend dxInfo($i) [expr [gdbGet $e1 [lindex $varList $i]] - [gdbGetSafe $e2 [lindex $varList $i]]]}

    lappend hiInfo1 [gdbGet     $e1 HEIGHT]
    lappend hiInfo2 [gdbGetSafe $e2 HEIGHT]

    incr j
   }

set c [getScale $hiInfo1 $hiInfo2]

set j 0

foreach hi1 $hiInfo1 \
   {
    set hi2 [lindex $hiInfo2 $j]
    lappend difInfo [expr $hi1 - $c*$hi2]
    incr j
   }

if {$reportFlag} \
   {
    puts "dist [format %.5e $totalDist] inName1 $inName1 count1 $pkCount1 inName2 $inName2 count2 $pkCount2 nomatch $badCount resused $reusedCount nonzero $nzCount title $title"

   }

if {$verbose} \
   {
    if {$hdrFlag} \
       {
        puts $outUnit "DATA TABLE_DISTANCE [format %.5e $totalDist]"
        puts $outUnit ""
        puts $outUnit "DATA PEAK_COUNT1    $pkCount1"
        puts $outUnit "DATA PEAK_COUNT2    $pkCount2"
        puts $outUnit ""
        puts $outUnit "DATA NOMATCH_COUNT  $badCount"
        puts $outUnit "DATA RESUSED_COUNT  $reusedCount"
        puts $outUnit "DATA NONZERO_COUNT  $nzCount"
        puts $outUnit ""

        puts -nonewline $outUnit "DATA DIST_RMS  "
        for {set i 0} {$i < $varCount} {incr i} {puts -nonewline $outUnit [format " %5f" [getRMS $dxInfo($i)]]}

        puts $outUnit ""
        puts $outUnit [format "DATA HEIGHT_RMS %.5e" [getRMS $difInfo]]
        puts $outUnit [format "DATA HEIGHT_COR %.5f" [getPearsonR $hiInfo1 $hiInfo2]]
        puts $outUnit ""

        puts -nonewline $outUnit "VARS   INDEX1"
        for {set i 0} {$i < $varCount} {incr i} {puts -nonewline $outUnit [format " %8s" [lindex $varList $i]1]}
        puts -nonewline $outUnit [format " %8s" HEIGHT1]

        puts -nonewline $outUnit [format " %8s" INDEX2]
        for {set i 0} {$i < $varCount} {incr i} {puts -nonewline $outUnit [format " %8s" [lindex $varList $i]2]}
        puts -nonewline $outUnit [format " %8s" HEIGHT2]

        puts -nonewline $outUnit [format " %8s %8s" DIST ASS]
        puts $outUnit ""

        puts -nonewline $outUnit "FORMAT %5d  "
        for {set i 0} {$i < $varCount} {incr i} {puts -nonewline $outUnit [format " %8s" "%8.4f"]}
        puts -nonewline $outUnit [format " %8s" "%+.5e"]

        puts -nonewline $outUnit [format " %8s" "%5d"]
        for {set i 0} {$i < $varCount} {incr i} {puts -nonewline $outUnit [format " %8s" "%8.4f"]}
        puts -nonewline $outUnit [format " %8s" "%+.5e"]
        puts -nonewline $outUnit [format " %8s %8s" "%.5f" "%s"]
        puts $outUnit ""
        puts $outUnit ""
       }
 
    set j 0

    foreach e1 $eList1 \
       {
        set e2 [lindex $matchList $j]

        puts -nonewline $outUnit [format " %5d" [gdbGet $e1 INDEX]]
        for {set i 0} {$i < $varCount} {incr i} {puts -nonewline $outUnit [format " %8.4f" [gdbGet $e1 [lindex $varList $i]]]}
        puts -nonewline $outUnit [format " % .5e" [gdbGet $e1 HEIGHT]]

        puts -nonewline $outUnit [format " %5d" [gdbGetSafe $e2 INDEX]]
        for {set i 0} {$i < $varCount} {incr i} {puts -nonewline $outUnit [format " %8.4f" [gdbGetSafe $e2 [lindex $varList $i]]]}
        puts -nonewline $outUnit [format " % .5e" [gdbGetSafe $e2 HEIGHT]]

        set thisAss [gdbGet $e1 ASS]

        if {![llength $thisAss]} \
           {
            set thisAss ID=[gdbGetSafe $e2 INDEX]
           } \
        else \
           {
            set thisAss $thisAss,ID=[gdbGetSafe $e2 INDEX]
           }

        puts $outUnit [format " %9.5f %s" [lindex $distList $j] $thisAss]

        incr j
       }
   } \
else \
   {
    puts -nonewline $outUnit [format "ALL_PPM_RMS %.5f" $totalDist]
    for {set i 0} {$i < $varCount} {incr i} {puts -nonewline $outUnit [format " [lindex $varList $i]_RMS %5f" [getRMS $dxInfo($i)]]}
    puts -nonewline $outUnit [format " HEIGHT_RMS %.4e" [getRMS $difInfo]]
    puts -nonewline $outUnit [format " HEIGHT_COR %.5f" [getPearsonR $hiInfo1 $hiInfo2]]
    puts $outUnit "" 
   }

if {[strcasecmp $outName None]} {catch "close $outUnit"}

exit 0
