#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set thisVersion 2021.179.19.42

if {[flagLoc $ARGV -help]} \
  {
   puts "Extract Peak Regions Listing in a Peak Table."
   puts " -in      inName    \[test.ft2\]  Input Spectrum."
   puts " -tab     tabName   \[test.tab\]  Input Peak Table."
   puts " -outDir  outDir    \[ext\]       Output Directory for Extracted Regions."
   puts " -outName outName   \[%04d.dat\]  Output Name."
   puts " -dx    dx          \[0.15ppm\]   X-Axis +/- Size of Regions."
   puts " -dy    dy          \[1.50ppm\]   Y-Axis +/- Size of Regions."
   puts " -useID                         Use Peak Table INDEX for Output Name."
   puts " -nouseID                       Use Region Number for Output Name (Default)."
   exit 0
  }

set inName  test.ft2
set tabName test.tab

set outDir  ext
set outName %04d.dat

set dx      0.15ppm
set dy      1.50ppm

set idFlag   0
set verbose  1

getArgD $ARGV -in      inName
getArgD $ARGV -tab     tabName
getArgD $ARGV -outDir  outDir
getArgD $ARGV -outName outName

getArgD $ARGV -dx      dx 
getArgD $ARGV -dy      dy 

if {[flagLoc $ARGV -useID]}   {set idFlag 1}
if {[flagLoc $ARGV -nouseID]} {set idFlag 0}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

if {![file exists $inName]} \
   {
    puts stderr "pkRegion. Error finding input spectrum $inName"
    exit 1
   }

set error [rdFDATA $inName fdata]

if {$error} \
   {
    puts stderr "pkRegion. Error reading header in input spectrum $inName"
    exit 1
   }

set tStr     [getParmStr $fdata FDTITLE    NULL_DIM]
set dimCount [getParmI   $fdata FDDIMCOUNT NULL_DIM]
set xSize    [getParmI   $fdata NDSIZE     CUR_XDIM]
set ySize    [getParmI   $fdata NDSIZE     CUR_YDIM]

if {$dimCount < 2} \
   {
    puts stderr "pkRegion. Error: input $inName must be a 2D spectrum."
    exit 1
   }

if {![file isdirectory $outDir]} {catch "file mkdir $outDir"} 

if {![file isdirectory $outDir]} \
   {
    puts stderr "pkRegion. Error finding output directory $outDir"
    exit 1
   }

set tab     [gdbRead table -in $tabName]
set varList [gdbGetInfo $tab -parms varNames]
set pkCount [gdbGetInfo $tab -parms entryCount]
set eList   [gdbGetInfo $tab -parms entries]

set xName   [getParmStr $fdata NDLABEL CUR_XDIM]
set yName   [getParmStr $fdata NDLABEL CUR_YDIM]

if {$verbose} \
   {
    puts stderr "pkRegion. Input Spectrum: $inName"
    puts stderr "pkRegion.  X-Axis: $xName $xSize Pts Y-Axis: $yName $ySize Pts."
    puts stderr "pkRegion.  Peak Table: $tabName Total Peaks: $pkCount:"
   }

set regID 0

foreach e $eList \
   {
    incr regID

    set pkID [gdbGet $e INDEX]
    set pkX  [gdbGet $e X_PPM]
    set pkY  [gdbGet $e Y_PPM]

    if {$idFlag} \
       {
        set thisName [format %s/$outName $outDir $pkID]
        set thisID   $pkID
       } \
    else \
       {
        set thisName [format %s/$outName $outDir $regID]
        set thisID   $regID
       }


    readROI -ndim 2 -roi 1 -in $inName -dx X_AXIS ${pkX}ppm $dx -dy Y_AXIS ${pkY}ppm $dy
    writeROI -roi 1 -out $thisName -ov

    set rInfo  [getROIInfo -roi 1]

    set fdataR [lindex $rInfo 1]
    set sList  [lindex $rInfo 2]

    set regX1  [format %.3f [pnt2spec $fdataR CUR_XDIM 1                 ppm]]
    set regXN  [format %.3f [pnt2spec $fdataR CUR_XDIM [lindex $sList 0] ppm]]

    set regY1  [format %.3f [pnt2spec $fdataR CUR_YDIM 1                 ppm]]
    set regYN  [format %.3f [pnt2spec $fdataR CUR_YDIM [lindex $sList 1] ppm]]

    set thisXSize [lindex $sList 0]
    set thisYSize [lindex $sList 1]

    if {$verbose} \
       {
        puts stderr [format "pkRegion.   Peak %3d Region %3d %6.3fppm %7.3fppm Size X: %3d Y: %3d $thisName" $pkID $regID $pkX $pkY $thisXSize $thisYSize]
       }
   }

exit 0

