#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Filter a Peak Table According to a Mask (2022.061.13.26):"
    puts " -in     inName   \[test.tab\]   Input Peak Table." 
    puts " -mask   maskName \[mask.dat\]   Mask Spectrum."
    puts " -out    outName  \[new.tab\]    Output Peak Table." 
    puts " -thresh thresh   \[0.5\]        Mask Threshold."
    puts " -var    varList  \[X_PPM ...\]  Peak Table Variables for Mask Locations."
    puts " -axis   axisList \[X_AXIS ...\] Corresponding Mask Dimensions."
    puts " -ndim   dimCount \[Auto\]       Dimension Count of Mask."
    puts " -verb                         Verbose Mode ON (Default)."
    puts " -noverb                       Verbose Mode OFF."
    puts "Delete Mode (Use Only One):"
    puts " -below                        Delete Peaks Below Mask Threshold (Default)."
    puts " -above                        Delete Entries Above Mask Threshold."
    puts "Notes:"
    puts " 1. In order to allow filtering of a 3D peak table by a 2D mask etc., the mask can"
    puts "    have fewer dimensions than the peak table."
    puts " 2. The number of peak table variables to use must be no greater than the number of"
    puts "    dimensions in the mask."
    puts " 3. Example to delete peaks that correspond with ppm locations in the mask"
    puts "    that have intensities less than 0.5:"
    puts ""
    puts "       pkFilter.tcl -in test.tab -mask mask.ft2 -out filter.tab -below -thresh 0.5" 
    exit 1 
   }

set inName       test.tab
set outName      new.tab
set maskName     mask.dat
set varList      Auto
set dimList      Auto
set unitList     Auto
set dimCount     Auto
set thresh       0.5

set delBelowFlag 1
set verbose      1

set dx           0
set dy           0
set dz           0
set da           0
 
getArgD $ARGV -in   inName
getArgD $ARGV -out  outName
getArgD $ARGV -mask maskName
getArgD $ARGV -ndim dimCount
getArgD $ARGV -thresh thresh

getListArgD $ARGV -var   varList
getListArgD $ARGV -axis  dimList
getListArgD $ARGV -units unitList

if {[flagLoc $ARGV -below]}  {set delBelowFlag 1}
if {[flagLoc $ARGV -above]}  {set delAboveFlag 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1} 
if {[flagLoc $ARGV -noverb]} {set verbose 0} 

#
#--------------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts "pkMaskFilter. Error Finding Input Table $inName"
    exit 1
   }

if {![file exists [format $maskName 1 1]]} \
   {
    puts "pkMaskFilter. Error Finding Mask $maskName"
    exit 1
   }

set sInfo        [getSpecSizeInfo $maskName]
set maskDimCount [lindex $sInfo 1]

set t     [gdbRead table -in $inName]
set eList [gdbGetInfo $t -parms entries]
set vList [gdbGetInfo $t -parms varNames]

set pkDimCount 0

foreach v {X_PPM Y_PPM Z_PPM A_PPM} \
   {
    if {[lsearch -exact $vList $v] < 0} break
    incr pkDimCount
   }

if {![strcasecmp $dimCount Auto]} {set dimCount $maskDimCount}

if {![strcasecmp $varList Auto]}   {set varList [lrange "X_PPM  Y_PPM  Z_PPM  A_PPM"  0 [expr $dimCount - 1]]}
if {![strcasecmp $dimList Auto]}   {set dimList [lrange "X_AXIS Y_AXIS Z_AXIS A_AXIS" 0 [expr $dimCount - 1]]} 

if {![strcasecmp $unitList Auto]} \
   {
    set unitList ""

    foreach v $varList \
       {
        if {[string first PPM [string toupper $v]] >= 0} \
           {
            lappend unitList ppm
           } \
        else \
           {
            lappend unitList pts
           } 
       } 
   }

if {$verbose} \
   {
    puts ""
    puts "pkMaskFilter. Input Peak Table:        $inName"
    puts "pkMaskFilter. Peak Table Dimensions:   $pkDimCount"
    puts "pkMaskFilter. Peak Filter Variables:   $varList"
    puts "pkMaskFilter. Peak Variable Units:     $unitList"
    puts "pkMaskFilter. Initial Peak Count:      [llength $eList]"
    puts "pkMaskFilter. Input Mask:              $maskName"
    puts "pkMaskFilter. Mask Dimensions:         $maskDimCount"
    puts "pkMaskFilter. Mask Filter Axes:        $dimList"
    puts "pkMaskFilter. Mask Threshold:          $thresh"

    if {$delBelowFlag} \
       {
        puts "pkMaskFilter. Delete Mode:             Delete Peaks Below Threshold."
       } \
    else \
       {
        puts "pkMaskFilter. Delete Mode:             Delete Peaks Above Threshold."
       }

    puts "pkMaskFilter. Ouput Table:             $outName"
   }

if {[llength $varList] != [llength $dimList] || [llength $varList] != [llength $unitList]} \
   {
    puts "pkMaskFilter. Error: variable list/axis list/units list mismatch."
    exit 1
   }

if {$dimCount == 1} \
   {
    readROI -roi 1 -in $maskName -ndim 1 -x [lindex $dimList 0] 0% 100%
   } \
elseif {$dimCount == 2} \
   {
    readROI -roi 1 -in $maskName -ndim 2 -x [lindex $dimList 0] 0% 100% \
                                         -y [lindex $dimList 1] 0% 100% 
   } \
elseif {$dimCount == 3} \
   {
    readROI -roi 1 -in $maskName -ndim 3 -x [lindex $dimList 0] 0% 100% \
                                         -y [lindex $dimList 1] 0% 100% \
                                         -z [lindex $dimList 2] 0% 100%
   } \
elseif {$dimCount == 4} \
   {
    readROI -roi 1 -in $maskName -ndim 4 -x [lindex $dimList 0] 0% 100% \
                                         -y [lindex $dimList 1] 0% 100% \
                                         -z [lindex $dimList 2] 0% 100% \
                                         -a [lindex $dimList 3] 0% 100%
   } \
else \
   {
    puts "pkMaskFilter. Error: Unsupported Dimension Count."
    exit 1
   }

set delList ""

foreach e $eList \
   {
    if {$dimCount == 1} \
       {
        readROI -roi 2 -inROI 1 -ndim 1 -dx [lindex $dimList 0] [gdbGet $e [lindex $varList 0]][lindex $unitList 0] $dx
       } \
    elseif {$dimCount == 2} \
       {
        readROI -roi 2 -inROI 1 -ndim 2 -dx [lindex $dimList 0] [gdbGet $e [lindex $varList 0]][lindex $unitList 0] $dx \
                                        -dy [lindex $dimList 1] [gdbGet $e [lindex $varList 1]][lindex $unitList 1] $dy
       } \
    elseif {$dimCount == 3} \
       {
        readROI -roi 2 -inROI 1 -ndim 3 -dx [lindex $dimList 0] [gdbGet $e [lindex $varList 0]][lindex $unitList 0] $dx \
                                        -dy [lindex $dimList 1] [gdbGet $e [lindex $varList 1]][lindex $unitList 1] $dy \
                                        -dz [lindex $dimList 2] [gdbGet $e [lindex $varList 2]][lindex $unitList 2] $dz
       } \
    elseif {$dimCount == 4} \
       {
        readROI -roi 2 -inROI 1 -ndim 4 -dx [lindex $dimList 0] [gdbGet $e [lindex $varList 0]][lindex $unitList 0] $dx \
                                        -dy [lindex $dimList 1] [gdbGet $e [lindex $varList 1]][lindex $unitList 1] $dy \
                                        -dz [lindex $dimList 2] [gdbGet $e [lindex $varList 2]][lindex $unitList 2] $dz \
                                        -da [lindex $dimList 3] [gdbGet $e [lindex $varList 3]][lindex $unitList 3] $da
       }

    set maxVal [getROIStat 2 vMax]

    if {$delBelowFlag} \
       {
        if {$maxVal < $thresh} {lappend delList $e}
       } \
    else \
       {
        if {$maxVal > $thresh} {lappend delList $e}
       }
   }

gdbFree $delList

set newCount [gdbGetInfo $t -parms entryCount]

if {$verbose} {puts "pkMaskFilter. Ouput Peak Count:        $newCount"}

gdbWrite $t $outName

exit 0
