#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"

#
# Replace {} with () to avoid parsing errors.

proc fixBrace { str } \
{
   set n [string length $str]
   set q 0

   set res ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $str $i]

       if {"$c" == "\{"} \
          {
           append res "("
          } \
       elseif {"$c" == "\}"} \
          {
           append res ")"
          } \
       elseif {"$c" == "\""} \
          {
           append res "\""
           if {$q} {append res " "}
           set q [expr !$q]
          } \
       else \
          {
           append res $c
          }
      }

   return $res
}

#
# Identify lines from a Bruker pulse program, with special
# attention to loop and increment/decrement commands.

proc getPProgKey { origLine basePtr comPtr isLabelPtr valuePtr varListPtr } \
{
    upvar  $varListPtr varList
    upvar  $isLabelPtr isLabel
    upvar  $valuePtr   value
    upvar  $basePtr    base
    upvar  $comPtr     comment

    global acqus

#
# Initialize:

    set isLabel  0
    set isQuoted 0
    set value    0
    set base     $origLine
    set comment  ""
    set key      ""

#
# Check for pure comment or blank line:

    scan $origLine "%s" key
    set  c [string index $key 0]

    if {![string compare $c ";"] || ![string compare $c "#"]} \
       {
        set base ""
        set comment $origLine
        return COMMENT
       }

    if {"$key" == ""} {return BLANK}

#
# Remove trailing comments:

    set line  [string trim $origLine]
    set loc   [string first ";" $line]

    if {$loc >= 0} \
       {
        set comment [string range $line $loc end]
        set line    [string range $line 0 [expr $loc - 1]]
        set base    $line
       }

#
# Check for quotes now, that will be lost during parsing:

    set count [llength $line]

    if {$count} \
       {
        set c [string index $line 0]

        if {![string compare $c \"]} {set isQuoted 1}
       }

#
# Remove extra spaces.
# Check for blank lines.

    set str ""
 
    foreach i $line {append str "$i "}

    set line    [string trim $str]
    set count   [llength $line]

    if {!$count} {return BLANK}

    scan $line "%s" key

#
# Identify and remove integer labels.

    if {[regexp {^([0-9])+$} $key]} \
       {
        set isLabel 1
        set line [lrange $line 1 end]
        set base $line

        set count  [llength $line]
        scan $line "%s" key
       }

#
# Statements outside of the pulse program are enclosed in double-quotes.
# Loops within the program have the keyword "lo".
# Other lines with only one token won't be analyzed further.
# Increments are generally of the form (# = integer 0 - 31):
#   "d#" followed by one of: "id#" "dd#" "ipu#" or "dpu#".

    if {$isQuoted} \
       {
        if {[string first "=" $line]} \
           {
            set line [split $line "= "]

            if {[llength $line] > 1} \
               {
                set value ""
                foreach i $line {append value "$i "}
                return ASSIGNMENT
               } \
            else \
               {
                return STATEMENT
               }
           } \
        else \
           {
            return STATEMENT 
           }
       }

    if {![string compare $key "lo"]} \
       {
        set value [lindex $line 4]
        set c     [string index $value 0]
        
        if {[info exists acqus(ass.$value)]} \
           {
            set value [brukExpr $acqus(ass.$value)]
            lappend varList ass.$value
           } \
        elseif {![string compare $c "l"]} \
           {
            if {[info exists acqus(L)]} \
               {
                set loc   [string range $value 1 end]
                set value [lindex $acqus(L) $loc]
                lappend varList L$loc
               }
           }
        
        return LOOP
       }

    if {![strcasecmp $key aqseq]} \
       {
        set value [lindex $line 1]
        set acqus(aqseq) $value 
        return OTHER
       }

    if {$count < 2} \
       {
        return CMND
       }

    set expr   {(.*)( )(id|dd|ipu|dpu)([0-9]+)(.*)}
    set status [regexp $expr $line ans delayLoc x incrType incrLoc tail]

    if {$status} \
       {
        set value "$incrType $incrLoc"

        if {![string compare $incrType "id"] ||
              ![string compare $incrType "dd"]} \
           {
            if {[info exists acqus(IN)]} \
               {
                set value [format "%4e" [lindex $acqus(IN) $incrLoc]]
                lappend varList IN$incrLoc
               }
           }

        if {![string compare $incrType "ipu"] ||
              ![string compare $incrType "dpu"]} \
           {
            if {[info exists acqus(INP)]} \
               {
                set value [format "%4e" [expr 1.0e-6*[lindex $acqus(INP) $incrLoc]]]
                lappend varList INP$incrLoc
               }
           }

        return INCR
       }

    set expr   {(.*)( )(ip|dp)([0-9]+)(.*)}
    set status [regexp $expr $line ans delayLoc x incrType incrLoc tail]

    if {$status} \
       {
        if {$incrLoc == 31} \
           {
            set value Receiver
           } \
        else \
           {
            set value #$incrLoc
           }

        return PHASE 
       }

    return OTHER
}

#
# Aux test for echo-antoecho mode.

proc isEA { pcVal phaseInfo } \
{
   if {$pcVal == 2} {return 1}
   if {[string first " EA" $phaseInfo] >= 0} {return 1}

   return 0 
}

#
# Read and interpret the contents of a Bruker pulse program.

proc rdPProg { ppName \
               loopArrayPtr incrArrayPtr phaseArrayPtr loopCountPtr
               varListPtr verbFlag } \
{
    upvar $loopArrayPtr  loopArray
    upvar $incrArrayPtr  incrArray
    upvar $phaseArrayPtr phaseArray 
    upvar $loopCountPtr  loopCount
    upvar $varListPtr    varList

    global acqus acqu2s acqu3s acqu4s

#
# Initialize:

    set error     0
    set value     0
    set loopCount 0

    set loopArray(0) 0
    set incrArray(0) "" 

    set blanks  "                  "
    set base    ""
    set comment ""
    set varList ""

    if {![info exists acqus(aqseq)]} {set acqus(aqseq) 0}

    if {![info exists acqus(TD)]}  {set acqus(TD)  0}
    if {![info exists acqu2s(TD)]} {set acqu2s(TD) $acqus(TD)}
    if {![info exists acqu3s(TD)]} {set acqu3s(TD) $acqu2s(TD)}
    if {![info exists acqu4s(TD)]} {set acqu4s(TD) $acqu3s(TD)}

    if {![info exists acqus(eaFlag)]} {set acqus(eaFlag) 0}

    set acqus(ass.td0) $acqus(TD)
    set acqus(ass.td1) $acqu3s(TD)
    set acqus(ass.td2) $acqu2s(TD)
    set acqus(ass.td3) $acqu4s(TD)

#
# Open the pulse program file:

    set error [catch {set inUnit [open $ppName r]}]

    if {$error} \
       {
        puts stderr "PProg Error opening pulse program file $ppName"
        return -1
       }

#
# For each line in the input:
#  If it is a parameter line:
#   Extract the parameter name and value
#   Store the name/value pair.

    while {[gets $inUnit buff] >= 0} \
       {
        set base    None
        set comment None
        set value   None

        set line [fixBrace $buff]

        catch {set key [getPProgKey $line base comment isLabel value varList]}

        if {[strcasecmp $key COMMENT]} \
           {
            catch {if {[lsearch -exact $line setgrad] >= 0 || [lsearch -exact $line EA] >= 0} {set acqus(eaFlag) 1}}
            catch {if {[lsearch -exact $line igrad]   >= 0 || [lsearch -exact $line EA] >= 0} {set acqus(eaFlag) 1}}
           }

        set base    [string trim $base]
        set comment [string trim $comment]
        set value   [string trim $value]

        switch $key \
           {
            COMMENT    { }
            BLANK      { }
            STATEMENT  { }
            CMND       { }

            ASSIGNMENT {
                        set varName [lindex $value 0]
                        set acqus(ass.$varName) [lrange $value 1 end]
                       }

            PHASE  {
                    lappend phaseArray($loopCount) "$value $base"

                    if {$verbFlag} \
                       {
                        set fmt "%-8s %s %-20s (%s) %s"
                        puts [format $fmt $key: $blanks $base $value $comment]
                       }
                   }
            INCR   {
                    lappend incrArray($loopCount) $value

                    if {$verbFlag} \
                       {
                        set fmt "%-8s %s %-20s (%s) %s"
                        puts [format $fmt $key: $blanks $base $value $comment]
                       }
                   }
            LOOP   {
                    if {![string is double  $value]} {set value 0}
                    if {![string is integer $value]} {set value [expr int($value)]}

                    set  loopArray($loopCount) $value
                    incr loopCount
                    set  incrArray($loopCount) ""

                    if {$verbFlag} \
                       {
                        set fmt "%-8s %s %-20s (%s) %s\n"
                        puts [format $fmt $key: $blanks $base $value $comment]
                       }

                    set loc [expr [string length $blanks] - 3]

                    if {$loc > 0} \
                       {
                        set blanks [string range $blanks 0 $loc]
                       }
                   }
           }
       }

    close $inUnit
}

#
# Guess the sweep width from the increment list.

proc getPProgSW { incrList defSW swScaleIN } \
{
    set sum 0.0

    foreach i $incrList {set sum [expr $sum + abs($i)]}

    set sw $defSW


    if {$sum != 0.0} \
       {
        if {[llength $incrList] == 1} \
           {
            set sw [expr 1.0/($swScaleIN*$sum)]
           } \
        else \
           {
            set sw [expr 1.0/(0.5*$swScaleIN*$sum)]
           }

        if {$sw < 1000.0} {set sw [expr 2.0*$sw]}
        if {$sw < 1000.0} {set sw [expr 2.0*$sw]}
       }   

    return $sw
}

#
# Parse the pulse program loop and increment arrays.

proc parsePProg { loopArrayPtr incrArrayPtr phaseArrayPtr loopCount verbFlag } \
{
    upvar $loopArrayPtr  loopArray
    upvar $incrArrayPtr  incrArray
    upvar $phaseArrayPtr phaseArray

    global Nx Tx MODEx SWx \
           Ny Ty MODEy SWy \
           Nz Tz MODEz SWz \
           Na Ta MODEa SWa

    global xScaleIN yScaleIN zScaleIN aScaleIN
    global dimCount dimCountName nusDimCount
    global acqus acqu2s acqu3s acqu4s

    if {![info exists nusDimCount]} {set nusDimCount 0}

    set nDim $dimCount 

    if {![info exists xScaleIN]} {set xScaleIN 1.0}
    if {![info exists yScaleIN]} {set yScaleIN 2.0}
    if {![info exists zScaleIN]} {set zScaleIN 2.0}
    if {![info exists aScaleIN]} {set aScaleIN 2.0}

    if {![info exists acqus(eaFlag)]} {set acqus(eaFlag) 0}

#
# Save the number of time and phase increments associated with each loop;
# Phase loops should have 2 steps but no increments.

    for {set i 0} {$i < 16} {incr i} \
       {
        set ic($i) 0
        set pc($i) 0

        if {![info exists phaseArray($i)]} {set phaseArray($i) ""}
        if {![info exists incrArray($i)]}  {set incrArray($i)  ""}
        if {![info exists loopArray($i)]}  {set loopArray($i)  0}
       }

    for {set i 0} {$i < $loopCount} {incr i} \
       {
        set ic($i) [llength $incrArray($i)]
        set pc($i) 0

        if {[info exists phaseArray($i)]} \
           {
            foreach item $phaseArray($i) \
               {
                set j [lindex $item 0]
                if {"$j" != "Receiver"} {incr pc($i)}
               }
           }
       }

    set eaFlag 0

    switch $loopCount \
       {
        1 {
           if {[llength $incrArray(0)]} \
              {
               set Ny    $loopArray(0)
               set Ty    $loopArray(0)
               set SWy   [getPProgSW $incrArray(0) $SWy $yScaleIN]
               set MODEy TPPI
               set nDim  2
              }
          }
        2 { 
           if {($loopArray(0) == 2 && !$ic(0)) && $ic(1)} \
              {
               set Ny    [expr 2*$loopArray(1)]
               set Ty    $loopArray(1)
               set SWy   [getPProgSW $incrArray(1) $SWy $yScaleIN]
               set MODEy Complex 
               set nDim  2

               if {[isEA $pc(0) $phaseArray(0)]} \
                  {
                   set MODEy  Echo-AntiEcho
                   set eaFlag 1
                  }
              }
          }
        3 {
           if {($loopArray(0) == 2 && !$ic(0)) && \
                 ($loopArray(1) == 2 && !$ic(1)) && $ic(2)} \
              {
               set Ny     [expr 2*$loopArray(2)]
               set Ty     $loopArray(2)
               set SWy    [getPProgSW $incrArray(2) $SWy $yScaleIN]
               set MODEy  Echo-AntiEcho
               set eaFlag 1
               set nDim   2
              }
          }
        4 {
           if {($loopArray(0) == 2 && !$ic(0)) && $ic(1)} \
              {
               set Ny    [expr 2*$loopArray(1)]
               set Ty    $loopArray(1)
               set SWy   [getPProgSW $incrArray(1) $SWy $yScaleIN]
               set MODEy Complex
               set nDim  2

               if {[isEA $pc(0) $phaseArray(0)]} \
                  {
                   set MODEy  Echo-AntiEcho
                   set eaFlag 1
                  }
              }

           if {($loopArray(2) == 2 && !$ic(2)) && $ic(3)} \
              {
               set Nz    [expr 2*$loopArray(3)]
               set Tz    $loopArray(3)
               set SWz   [getPProgSW $incrArray(3) $SWz $zScaleIN]
               set MODEz Complex
               set nDim  3

               if {[isEA $pc(2) $phaseArray(2)]} \
                  {
                   set MODEz  Echo-AntiEcho
                   set eaFlag 1
                  }
              }
          }
        5 {
           if {($loopArray(1) == 2 && !$ic(0) && !$ic(1)) && $ic(2)} \
              {
               set Ny    [expr 2*$loopArray(2)]
               set Ty    $loopArray(2)
               set SWy   [getPProgSW $incrArray(2) $SWy $yScaleIN]
               set MODEy Complex
               set nDim  2

               if {[isEA $pc(1) $phaseArray(1)]} \
                  {
                   set MODEy  Echo-AntiEcho
                   set eaFlag 1
                  }
              }

           if {($loopArray(3) == 2 && !$ic(0) && !$ic(3)) && $ic(4)} \
              {
               set Nz    [expr 2*$loopArray(4)]
               set Tz    $loopArray(4)
               set SWz   [getPProgSW $incrArray(4) $SWz $zScaleIN]
               set MODEz Complex
               set nDim  3

               if {[isEA $pc(3) $phaseArray(3)]} \
                  {
                   set MODEz  Echo-AntiEcho
                   set eaFlag 1
                  }
              }
          }
        6 {
           if {($loopArray(0) == 2 && !$ic(0)) && $ic(1)} \
              {
               set Ny    [expr 2*$loopArray(1)]
               set Ty    $loopArray(1)
               set SWy   [getPProgSW $incrArray(1) $SWy $yScaleIN]
               set MODEy Complex
               set nDim  2

               if {[isEA $pc(0) $phaseArray(0)]} \
                  {
                   set MODEy  Echo-AntiEcho
                   set eaFlag 1
                  }
              }

           if {($loopArray(2) == 2 && !$ic(2)) && $ic(3)} \
              {
               set Nz    [expr 2*$loopArray(3)]
               set Tz    $loopArray(3)
               set SWz   [getPProgSW $incrArray(3) $SWz $zScaleIN]
               set MODEz Complex
               set nDim  3

               if {[isEA $pc(2) $phaseArray(2)]} \
                  {
                   set MODEz  Echo-AntiEcho
                   set eaFlag 1
                  }
              }

           if {($loopArray(4) == 2 && !$ic(4)) && $ic(5)} \
              {
               set Na    [expr 2*$loopArray(5)]
               set Ta    $loopArray(5)
               set SWa   [getPProgSW $incrArray(5) $SWa $aScaleIN]
               set MODEa Complex
               set nDim  4

               if {[isEA $pc(4) $phaseArray(4)]} \
                  {
                   set MODEz  Echo-AntiEcho
                   set eaFlag 1
                  }
              }
          }
        7 {
           if {($loopArray(1) == 2 && !$ic(0) && !$ic(1)) && $ic(2)} \
              {
               set Ny    [expr 2*$loopArray(2)]
               set Ty    $loopArray(2)
               set SWy   [getPProgSW $incrArray(2) $SWy $yScaleIN]
               set MODEy Complex
               set nDim  2

               if {[isEA $pc(1) $phaseArray(1)]} {set MODEy Echo-AntiEcho}
              }

           if {($loopArray(3) == 2 && !$ic(0) && !$ic(3)) && $ic(4)} \
              {
               set Nz    [expr 2*$loopArray(4)]
               set Tz    $loopArray(4)
               set SWz   [getPProgSW $incrArray(4) $SWz $zScaleIN]
               set MODEz Complex
               set nDim  3

               if {[isEA $pc(3) $phaseArray(3)]} \
                  {
                   set MODEz  Echo-AntiEcho
                   set eaFlag 1
                  }
              }

           if {($loopArray(5) == 2 && !$ic(0) && !$ic(5)) && $ic(6)} \
              {
               set Na    [expr 2*$loopArray(6)]
               set Ta    $loopArray(6)
               set SWa   [getPProgSW $incrArray(6) $SWa $aScaleIN]
               set MODEa Complex
               set nDim  4

               if {[isEA $pc(5) $phaseArray(5)]} \
                  {
                   set MODEa  Echo-AntiEcho
                   set eaFlag 1
                  }
              }
          }
       }

    if {$nusDimCount >= 2} \
       {
        set dimCount $nusDimCount

        if {![strcasecmp $dimCountName N]} {setDimCount $dimCountName}
       } \
    elseif {![strcasecmp $dimCountName N]} \
       {
        set dimCount $nDim

        if {$dimCount == 4 && $Ta < 2} {set dimCount 3}
        if {$dimCount == 3 && $Tz < 2} {set dimCount 2}

        setDimCount  $dimCountName
       }

#
# Update guesses about Echo-AntiEcho if non-zero FnMODE parameters exist;
# Assumes that non-zero FnMODE values are definitive.

    set eaYes 0
    set eaNo  0

if {$acqu3s(FnMODE) != 0 && $dimCount < 2} {set dimCount 3}
if {$acqu4s(FnMODE) != 0 && $dimCount < 4} {set dimCount 4}
 
    if {$dimCount >= 2 && $acqu2s(FnMODE) == 6} {incr eaYes}
    if {$dimCount >= 2 && $acqu3s(FnMODE) == 6} {incr eaYes}
    if {$dimCount >= 2 && $acqu4s(FnMODE) == 6} {incr eaYes}

    if {$dimCount >= 2 && $acqu2s(FnMODE) != 0 && $acqu2s(FnMODE) != 6} {incr eaNo}
    if {$dimCount >= 3 && $acqu3s(FnMODE) != 0 && $acqu2s(FnMODE) != 6} {incr eaNo}
    if {$dimCount >= 4 && $acqu4s(FnMODE) != 0 && $acqu2s(FnMODE) != 6} {incr eaNo}

    if {$dimCount == 2} \
       {
        if {$eaNo == 1} \
           {
            set MODEy  Complex
            set eaFlag 0
           } \
        else \
           {
            if {$eaYes == 1 || $acqus(eaFlag)} \
               {
                set MODEy  Echo-AntiEcho
                set eaFlag 1
               }
           }
       } \
    elseif {$dimCount == 3} \
       {
        if {$acqu2s(FnMODE) != 0 && $acqu3s(FnMODE) != 0} \
           {
            set MODEy  Complex
            set MODEz  Complex
            set eaFlag 0
           }

        if {$acqu2s(FnMODE) == 6} \
           {
            set MODEy  Echo-AntiEcho
            set eaFlag 1
           }

        if {$acqu3s(FnMODE) == 6} \
           {
            set MODEz Echo-AntiEcho
            set eaFlag 1
           }
       } \
    elseif {$dimCount == 4} \
       {
        if {$acqu2s(FnMODE) != 0 && $acqu3s(FnMODE) != 0 && $acqu4s(FnMODE) != 0} \
           {
            set MODEy  Complex
            set MODEz  Complex
            set MODEa  Complex
            set eaFlag 0
           }

        if {$acqu2s(FnMODE) == 6} \
           {
            set MODEy  Echo-AntiEcho
            set eaFlag 1
           }

        if {$acqu3s(FnMODE) == 6} \
           {
            set MODEz Echo-AntiEcho
            set eaFlag 1
           }

        if {$acqu4s(FnMODE) == 6} \
           {
            set MODEa Echo-AntiEcho
            set eaFlag 1
           }
      }

    if {($acqu2s(FnMODE) == 1 || $acqu2s(FnMODE) == 7)} {set MODEy Real}
    if {($acqu3s(FnMODE) == 1 || $acqu3s(FnMODE) == 7)} {set MODEz Real}
    if {($acqu4s(FnMODE) == 1 || $acqu4s(FnMODE) == 7)} {set MODEa Real}

    if {$verbFlag} \
       {
        puts [format "%dD Data:" $nDim]

        set fmt "%d. %5d Pts %8s, SW %8.3f Hz"

        for {set i 1} {$i <= $nDim} {incr i} \
           {
            switch $i \
               {
                1 {puts [format $fmt $i $Tx $MODEx $SWx]}
                2 {puts [format $fmt $i $Ty $MODEy $SWy]}
                3 {puts [format $fmt $i $Tz $MODEz $SWz]}
                4 {puts [format $fmt $i $Ta $MODEa $SWa]}
               }
           }
       }
}

#
# Substitute a Bruker expression; right now only td0-3 is used:

proc brukExpr { origStr } \
{
   global acqus acqu2s acqu3s acqu4s

   set varList "td0 td1 td2 td3"
   set valList "$acqus(TD) $acqu3s(TD) $acqu2s(TD) $acqu4s(TD)"

   set str $origStr
   set k   0

   foreach i $varList \
      {
       set loc [string first $i $str]

       if {$loc > -1} \
          {
           set    newStr [string range $str 0 [expr $loc-1]]
           append newStr [lindex $valList $k]
           append newStr [string range $str [expr $loc+[string length $i]] end]
           set    str    $newStr
          }

       incr k
      }

    set error [catch {set str [expr $str]}]

    if {$error} {set $str 0}

    return $str
}
 
#
# Bottom.
