#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Create 2D projections from 3D data."
    puts " -in        inName    \[ft/test%03d.ft3\] Name of 3D Input."
    puts " -outDir    outDir    \[.\]               Output Directory."
    puts " -xyOutName xyOutName \[Auto\]            Name of 2D X,Y-Axis Output."
    puts " -xzOutName xzOutName \[Auto\]            Name of 2D X,Z-Axis Output."
    puts " -yzOutName yzOutName \[Auto\]            Name of 2D Y,Z-Axis Output."
    puts " -skyline                               Skyline Projection (Default)."
    puts " -sum                                   Summation projection."
    puts " -abs                                   Use Absolute Value."
    puts " -axis                                  Auto Names Use Axis Names X Y Z."
    puts " -noaxis                                Auto Names Use Axis Labels (Default)."
    puts " -verb                                  Verbose Mode ON (Default)."
    puts " -noverb                                Verbose Mode OFF."
    puts "Spectral Region (Use Prefix -x -y -z):"    
    puts " -x1  x1 \[0%\]                           X-Axis Region Start."
    puts " -xn  xn \[100%\]                         X-Axis Region End."
    puts "Notes:"
    puts " 1. Projections are relative to original orientation of FID."
    puts " 2. Auto mode output names are generated from axis labels."
    exit 1
   }

set inName      ft/test%03d.ft3
set outDir      .
set projMode    proj
set absFlag     0
set axisFlag    0
set verbose     1

set xyOutName   Auto
set xzOutName   Auto
set yzOutName   Auto

set ix1         0%
set ixn         100%

set iy1         0%
set iyn         100%

set iz1         0%
set izn         100%

getArgD $ARGV -in     inName
getArgD $ARGV -outDir outDir

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -axis]}   {set axisFlag 1}
if {[flagLoc $ARGV -noaxis]} {set axisFlag 0}

if {$axisFlag} \
   {
    set xyOutName projXY.dat
    set xzOutName projXZ.dat
    set yzOutName projYZ.dat
   }

getArgD $ARGV -xyOutName xyOutName
getArgD $ARGV -xzOutName xzOutName
getArgD $ARGV -yzOutName yzOutName

getArgD $ARGV -x1 ix1
getArgD $ARGV -xn ixn

getArgD $ARGV -y1 iy1
getArgD $ARGV -yn iyn

getArgD $ARGV -z1 iz1
getArgD $ARGV -zn izn

if {[flagLoc $ARGV -skyline]} {set projMode proj}
if {[flagLoc $ARGV -sum]}     {set projMode sum}
if {[flagLoc $ARGV -abs]}     {set absFlag  1}

if {$absFlag} \
   {
    set absArg abs
   } \
else \
   {
    set absArg noabs
   }

#
#--------------------------------------------------------------------------

set thisName [format $inName 1 1]

if {![file exists $thisName]} \
   {
    puts stderr "Proj3D Error Finding Input $inName"
    exit 1
   }

#
# Extract header:

set error [rdFDATA $inName fdata]

if {$error} \
   {
    puts stderr "Proj3D Error Reading header from $inName"
    exit 1
   }

#
# Prepare output directory if needed:

if {![strcasecmp $outDir "."] || ![strcasecmp $outDir "./"]} \
   {
    set outPrefix ""
   } \
else \
   {
    if {![file exists $outDir]} {catch "file mkdir $outDir"}

    if {![file isdirectory $outDir]} \
       {
        puts stderr "Proj3D Error Finding or Creating Output Directory $outDir"
        exit 1
       }

    set outPrefix $outDir/
   }

#
# Find axis labels:

set tpFlag [getParmI $fdata FDTRANSPOSED 0]

if {$tpFlag} \
   {
    set xName [getParmStr $fdata NDLABEL CUR_YDIM]
    set yName [getParmStr $fdata NDLABEL CUR_XDIM]
    set zName [getParmStr $fdata NDLABEL CUR_ZDIM]

    set it1 $ix1
    set itn $ixn

    set ix1 $iy1
    set ixn $iyn

    set iy1 $it1
    set iyn $itn

    set xAlt  Y
    set yAlt  X
    set zAlt  Z
   } \
else \
   {
    set xName [getParmStr $fdata NDLABEL CUR_XDIM]
    set yName [getParmStr $fdata NDLABEL CUR_YDIM]
    set zName [getParmStr $fdata NDLABEL CUR_ZDIM]

    set xAlt  X
    set yAlt  Y
    set zAlt  Z
   }

#
# Adjust axis labels if any are identical:

if {![strcmp $xName $yName]} {set xName $xAlt}
if {![strcmp $xName $zName]} {set xName $xAlt}
if {![strcmp $yName $zName]} {set yName $yAlt}

#
# Read the projections regions.
# Write the projections.

if {$verbose} {puts "Reading Projection: $xName/$yName"}

readROI -roi 1 -in $inName -ndim 2 -$projMode -$absArg -x $xName $ix1 $ixn -y $yName $iy1 $iyn -z $zName $iz1 $izn

if {$verbose} {puts "Reading Projection: $xName/$zName"}

readROI -roi 2 -in $inName -ndim 2 -$projMode -$absArg -x $xName $ix1 $ixn -y $zName $iz1 $izn -z $yName $iy1 $iyn

if {$verbose} {puts "Reading Projection: $yName/$zName"}

readROI -roi 3 -in $inName -ndim 2 -$projMode -$absArg -x $yName $iy1 $iyn -y $zName $iz1 $izn -z $xName $ix1 $ixn

if {$absFlag} \
   {
    foreach id {1 2 3} \
       {
        set roiInfo [getROIInfo -roi $id]
        set rdata   [lindex $roiInfo 0]
        set sList   [lindex $roiInfo 2]
        set nx      [lindex $sList 0]
        set ny      [lindex $sList 1]
        set n       [expr $nx*$ny]

        vAbs $rdata $n
       }
   }

if {![strcasecmp $xyOutName Auto]} \
   {
    set thisName $outPrefix$xName.$yName.dat
   } \
else \
   {
    set thisName $outPrefix$xyOutName
   }

if {$verbose} {puts "Writing Projection: $thisName"}

writeROI -roi 1 -out $thisName -ov 

if {![strcasecmp $xzOutName Auto]} \
   {
    set thisName $outPrefix$xName.$zName.dat
   } \
else \
   {
    set thisName $outPrefix$xzOutName
   }

if {$verbose} {puts "Writing Projection: $thisName"}

writeROI -roi 2 -out $thisName -ov

if {![strcasecmp $yzOutName Auto]} \
   {
    set thisName $outPrefix$yName.$zName.dat
   } \
else \
   {
    set thisName $outPrefix$yzOutName
   }

if {$verbose} {puts "Writing Projection: $thisName"}

writeROI -roi 3 -out $thisName -ov

exit 0
