#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# TALOS: Torsion Angle Likeness Optimized by Shifts
# (C) Shen, Cornilescu, Delaglio, and Bax 1997-2011
#     NIH Lab of Chemical Physics NIDDK
#
# talos+:    finds chemical shift homology matches in database.
# rama+.tcl: interactive display and adjustment of predictions.

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set talosVersion "Version 3.40F1 Rev 2011.030.17.13"

#
# Initialize variables:

proc ramaInit {} \
{
   global ramaTab shiftTab sumTab rcTab adjTab prevTab nextTab refTab 
   global talosTab annTab ssTab s2Tab predAllTab predAllFlag

   global inName rcName adjName prevName nextName
   global phiAvg psiAvg phiDev psiDev

   global sumName refName dbName predAllName tabDir predDir talosDir surfDir
   global seqList npList currentRes r1 rN matchCount roiCount drawResFlag
   global xOff yOff xExtra yExtra autoFlag rasmolFlag currentStat
   global shiftLimits

   global clipFlag clipName clipTab

   global centerColor zeroColor defColor1 defColor2 activeColor1 activeColor2 
   global goodColor1 goodColor2 badColor1 badColor2 
   global warnColor1 warnColor2 dynColor1 dynColor2

   global colCount firstRes lastRes avgFunc devFunc sdHeight sdWidth sdFlag
   global refColor color1 color2 cellX cellY refPhi refPsi refInfo

   global env

   setFonts

   if {[info exists env(TALOSP_DIR)]} \
      {
       set talosDir $env(TALOSP_DIR)
      } \
   else \
      {
       set talosDir .
      }

   if {[info exists env(SURF_DIR)]} \
      {
       set surfDir $env(SURF_DIR)
      } \
   else \
      {
       set surfDir None
      }

#                   HN   N    CO  CA  CB  HA
   set shiftLimits "4.0 12.0 6.0 6.0 6.0 2.0"

   set tabDir      tab
   set inName      None
   set refName     None 
   set clipName    sslimit.tab
   set sumName     pred.tab
   set dbName      talos.tab
   set currentStat None
   set predDir     pred 
   set predAllName predAll.tab

   set rcName   None
   set adjName  None
   set prevName None
   set nextName None

   set predAllFlag 1
   set autoFlag    0
   set rasmolFlag  0
   set clipFlag    0
   set predAllTab  0
   set shiftTab    0
   set talosTab    0
   set clipTab     0
   set ramaTab     0
   set annTab      0
   set rcTab       0
   set adjTab      0
   set prevTab     0
   set nextTab     0
   set sumTab      0
   set refTab      0
   set sdFlag      1

   set drawResFlag 1
   set roiCount    0

   set avgFunc     getAvgAngle
   set devFunc     getStdDevAngle

   set firstRes    First 
   set lastRes     Last 
   set r1          None 
   set rN          None 
   set currentRes  None 

   set refInfo     "? ? ?"
   set refPhi      0.0
   set refPsi      0.0

   set phiAvg      0.0
   set psiAvg      0.0
   set phiDev      0.0
   set psiDev      0.0

   set matchCount  10
   set colCount    10
   set seqList     ""
   set npList      ""

   set sdWidth  300
   set sdHeight  14 

   set xOff    24
   set yOff    24
   set xExtra  80 
   set yExtra  24

   set color1       white
   set color2       black
   set defColor1    gray85
   set defColor2    gray90
   set activeColor1 gray50
   set activeColor2 gray40
   set goodColor1   green3
   set goodColor2   green2
   set dynColor1    blue
   set dynColor2    cyan 
   set warnColor1   yellow2
   set warnColor2   yellow1
   set badColor1    firebrick2
   set badColor2    firebrick1
   set centerColor  gold3
   set zeroColor    dimGray
   set refColor     blue

   set wide         1024
   set hi           900

   catch {set wide [winfo screenwidth  .]}
   catch {set hi   [winfo screenheight .]}

   if {$hi > 1000} \
      {
       set cellX       10 
       set cellY       18
      } \
   else \
      {
       set cellX       8
       set cellY       16
      }

   initResNames
}

#
# Extract command line arguments.

proc ramaArgs {} \
{
 global inName sumName refName dbName rcName adjName prevName nextName
 global tabDir predDir talosDir surfDir predAllName

 global matchCount autoFlag rasmolFlag firstRes lastRes drawResFlag
 global ARGC ARGV talosVersion colCount sdFlag predAllFlag

 puts "*----------------------------------------------------------------*"
 puts "This is RAMA+, part of the TALOS+ system for predicting protein"
 puts "backbone torsion angles from chemical shifts."
 puts ""
 puts "TALOS+ is an enhanced version of the earlier TALOS system"
 puts "that improves upon the original TALOS database mining"
 puts "approach by including a artificial neural network (ANN)"
 puts "classification scheme. This improved method allows"
 puts "prediction of backbone angles for 88% of residues in"
 puts "a given protein on average."
 puts ""
 puts "There is also a complementary system, called SPARTA+, that"
 puts "can predict backbone chemicals shifts from a PDB structure."
 puts ""
 puts "For more information, see:"
 puts ""
 puts "   http://spin.niddk.nih.gov/bax/software/TALOS+"
 puts "   http://spin.niddk.nih.gov/bax/software/SPARTA+"
 puts ""
 puts "Reference:"
 puts "   Y. Shen, F. Delaglio, G. Cornilescu, and A. Bax:"
 puts "   TALOS plus: A hybrid method for predicting protein"
 puts "   torsion angles from NMR chemical shifts."
 puts "   J. Biomol. NMR 44, 213-223 (2009)."
 puts ""
 puts "RAMA+ Version $talosVersion"
 puts "*----------------------------------------------------------------*"
 puts ""

 if {[flagLoc $ARGV -help]} \
 {
  puts "rama+.tcl: display and adjust TALOS+ prediction results."
  puts " -in      inName  \[None\]         Shifts used for TALOS predictions."
  puts " -sum     sumName \[pred.tab\]     Prediction summary input/output."
  puts " -ref     refName \[None\]         Optional PDB Input for comparison."
  puts "RAMA+ Prediction Input, as Produced by TALOS: (Use Only One):"
  puts " -predAll pName   \[predAll.tab\]  Table of existing predictions."
  puts " -predDir pDir    \[pred\]         Directory of existing predictions."
  puts "RAMA+ Other Options:"
  puts " -count   mc      \[10\]           Max match count per triplet."
  puts " -cols    cCount  \[10\]           Column Count for Sequence Display."
  puts " -first   resID1  \[First\]        First residue to display."
  puts " -last    resIDN  \[Last\]         Last residue to display."
  puts " -sd                             Display shift distributions (Default)."
  puts " -nosd                           Don't display shift distributions."
  puts " -ras                            Display PDB Reference using RasMol."
  puts " -auto                           Display predictions in progress."
  puts " -old                            Use the old version of RAMA."
  puts "RAMA+ Database Settings:"
  puts " -talosDir mDir   \[TALOSP_DIR\]   TALOS+ Main Directory."
  puts " -tabDir   tDir   \[tab\]          Subdirectory of TALOS master tables."
  puts " -db      dbName  \[talos.tab\]    TALOS+ triplet database file."
  puts " -rand    rcName  \[randcoil.tab\] Random coil shifts table."
  puts ""
  puts "RAMA+ Examples:"
  puts " Typical inspection:"
  puts "   talos+ -in ubiq.tab"
  puts "   rama+  -in ubiq.tab"
  puts " Inspection when TALOS+ is used with BMRB NMR-Star Format Input:"
  puts "   talos+ -in ubiq.str -bmrb bmrb.tab"
  puts "   rama+  -in bmrb.tab"
  puts " Inspection when a PDB structure is known:"
  puts "   talos+ -in ubiq.tab -ref ubiq.pdb"
  puts "   rama+  -in ubiq.tab -ref ubiq.pdb"
  puts " Inspection when a PDB structure is known, including Molecular Viewer:"
  puts "   rama+  -in ubiq.tab -ref ubiq.pdb -ras"
  puts ""
  puts "Notes:"
  puts " 1. Commands rama+ and rama+.tcl are equivalent."
  puts " 2. RAMA+ expects to find the following TALOS+ output files:"
  puts "    pred.tab predAll.tab predSS.tab predS2.tab pdbABP.tab" 
  puts " 3. The prediction input file (-predAll) will be updated"
  puts "    automatically by RAMA+ to reflect any interactive adjustments"
  puts "    made to select the database triplets used to form a prediction." 
  puts " 4. Option -ras requires that RasMol2 is installed and available"
  puts "    via the command 'rasmol', and that no other instances of RasMol"
  puts "    are using the screen. RasMol Reference: Roger Sayle and E. James"
  puts "    Milner-White.  RasMol: Biomolecular graphics for all; Trends in"
  puts "    Trends in Biochemical Sciences (TIBS), Sep 1995, Vol. 20, No. 9,"
  puts "    p. 374. http://www.umass.edu/microbio/rasmol"
  exit 0
 }

   getArgD $ARGV -talosDir talosDir

   set tabDir $talosDir/$tabDir

   getArgD $ARGV -predAll  predAllName
   getArgD $ARGV -predDir  predDir
   getArgD $ARGV -surfDir  surfDir
   getArgD $ARGV -tabDir   tabDir

   if {[file exists $predAllName]} \
      {
       set predAllFlag 1
      } \
   else \
      {
       set predAllFlag 0
      }
  
   if {!$predAllFlag} \
      {
       if {![file isdirectory $predDir]} \
          {
           puts stderr "Error Finding Prediction Directory $predDir"
           exit 1
          }
      } 

   set dbName   $tabDir/$dbName

   set rcName   $tabDir/randcoil.tab
   set adjName  $tabDir/rcadj.tab
   set prevName $tabDir/rcprev.tab
   set nextName $tabDir/rcnext.tab

   getArgD $ARGV -rand rcName
   getArgD $ARGV -adj  adjName
   getArgD $ARGV -prev prevName
   getArgD $ARGV -next nextName

   getArgD $ARGV -count    matchCount
   getArgD $ARGV -cols     colCount
   getArgD $ARGV -in       inName
   getArgD $ARGV -db       dbName
   getArgD $ARGV -sum      sumName
   getArgD $ARGV -out      sumName
   getArgD $ARGV -ref      refName
   getArgD $ARGV -first    firstRes
   getArgD $ARGV -last     lastRes

   if {[flagLoc $ARGV -sd]}     {set sdFlag 1}
   if {[flagLoc $ARGV -nosd]}   {set sdFlag 0}

   if {[flagLoc $ARGV -drawRes]}   {set drawResFlag 1}
   if {[flagLoc $ARGV -nodrawRes]} {set drawResFlag 0}

   if {[flagLoc $ARGV -auto]}   {set autoFlag 1}
   if {[flagLoc $ARGV -noauto]} {set autoFlag 0}

   if {[flagLoc $ARGV -ras]}    {set rasmolFlag 1}
   if {[flagLoc $ARGV -noras]}  {set rasmolFlag 0}
}

#
# Read or create the summary table.

proc ramaReadSummary {} \
{
   global inName sumName predAllName predDir 
   global sumTab annTab ssTab s2Tab predAllTab
   global xListH yListH xListE yListE xListS2 yListS2

   gdbInit
   gdbCreate dBase

   if {![file exists $sumName]} \
      {
       catch {exec vina.tcl -in $inName -sum $sumName -predDir $predDir -AUTO}
      }

   if {![file exists $sumName]} \
      {
       puts stderr "Error Finding Summary File $sumName"
       exit 1
      }

   set sumTab [gdbRead table -in $sumName -name "TALOS Prediction Summary"]    

   if {![file exists $predAllName]} \
      {
       puts stderr "Error Finding Prediction File $predAllName"
       exit 1
      }

   set predAllTab [gdbRead table -in $predAllName -name "TALOS Predictions"]    

   set rName [file rootname [file tail $sumName]]
   set fName [format %sABP.tab $rName]

   if {[file exists $fName]} \
      {
       set annTab [gdbRead table -in $fName -name "ANN Prediction Summary"]
      }\
     else \
      {
        set annTab [gdbCreate table -name "ANN Prediction Summary"]
      }


   set fName [format %sSS.tab $rName]

   if {[file exists $fName]} \
      {
       set ssTab \
       [gdbRead table -in $fName -name "Secondary Structure Prediction Summary"]
      }\
     else \
      {
       set ssTab \
       [gdbCreate table -name "Secondary Structure Prediction Summary"]
      }

   set eList  [gdbGetInfo $ssTab -parms entries]
   set xListH ""
   set xListE ""
   set yListH ""
   set yListE ""

   foreach entry $eList \
      {
       set xVal [gdbGet $entry RESID]
       set hVal [gdbGet $entry Q_H]
       set eVal [gdbGet $entry Q_E]
       set lVal [gdbGet $entry Q_L]

       set maxVal [expr max($hVal,max($eVal,$lVal))] 
       set minVal [expr min($hVal,min($eVal,$lVal))] 
       set conf   [expr 2.0*$maxVal-$hVal-$eVal-$lVal+$minVal]

       if {$hVal == $maxVal && $maxVal != 0 && $conf != 0 } \
          {
           lappend xListH  $xVal
           lappend yListH  [expr -1.0*$hVal]
          }\
	 else \
          {
           lappend xListH  $xVal
           lappend yListH  0.0
          }

       if {$eVal == $maxVal && $maxVal != 0 && $conf != 0 } \
          {
           lappend xListE  $xVal
           lappend yListE  [expr 1.0*$eVal]
	   }\
	 else \
          {
           lappend xListE  $xVal
           lappend yListE  0.0
          }

       if {$lVal == $maxVal && $maxVal != 0} {set yVal 0}
      }

   set fName [format %sS2.tab $rName]

   if {[file exists $fName]} \
      {
       set s2Tab [gdbRead table -in $fName -name "RCI-S2 Prediction Summary"]
      }\
     else \
      {
       set s2Tab [gdbCreate table -name "RCI-S2 Prediction Summary"]
      }

   set eList   [gdbGetInfo $s2Tab -parms entries]
   set xListS2 ""
   set yListS2 ""

   foreach entry $eList \
      {
       set xVal [gdbGet $entry RESID]
       set yVal [gdbGet $entry S2]

       if {$yVal < 1 && $yVal >= 0} \
          { 
           lappend xListS2  $xVal
           lappend yListS2  [expr $yVal]
          }
      }

}

#
# Initialize table of random coil reference shifts.

proc ramaReadRandCoil {} \
{
   global sdFlag

   if {!$sdFlag} return

   initRandCoil
}

#
# Read the triplet table.

proc ramaReadTriplets {} \
{
   global dbName talosTab sdFlag predAllFlag

   if {!$sdFlag} return
   if {$predAllFlag} return

   set talosTab [gdbRead table -in $dbName -name "TALOS Database"]
}

#
# Read the reference PDB table for displaying tests on known structures.

proc ramaReadRef {} \
{
    global refName refTab

    if {![strcasecmp $refName None]} return

    if {![file exists $refName]} \
       {
        puts stderr "Error Finding PDB Reference $refName"
        exit 1
       }

    set refTab [gdbRead table -pdb -in $refName -name "Reference PDB"]
}

#
# Read the input shift table used previously by TALOS.

proc ramaReadShifts {} \
{
   global shiftTab seqList npList statusArray firstRes lastRes r1 rN 
   global currentRes inName predDir predAllFlag

   if {![strcasecmp $inName None]} return

   if {![file exists $inName]} \
      {
       puts stderr "Error Finding Chemical Shift Input $inName"
       exit 1
      }
 
   set shiftTab [gdbRead table -in $inName]
   set seqList  [getSeqList $shiftTab r1 rN]
   set npList   "?"

   if {![strcasecmp $firstRes First]} {set firstRes $r1}
   if {![strcasecmp $lastRes  Last]}  {set lastRes  $rN}

   if {$firstRes < $r1} {set firstRes $r1}
   if {$lastRes  < $r1} {set lastRes  $r1}

   if {$firstRes > $rN} {set firstRes $rN}
   if {$lastRes  > $rN} {set lastRes  $rN}

   if {$firstRes > $lastRes} \
      {
       set itemp    $firstRes
       set firstRes $lastRes
       set $lastRes $itemp
      }
      
   set currentRes [expr $firstRes + 1] 

   foreach res $seqList {lappend npList $res}

   lappend npList "?"

   for {set i $r1} {$i <= $rN} {incr i} \
      {
       set entry  [ramaGetEntry $i]
       
       set stat   [gdbGet $entry CLASS]

       if {$predAllFlag} \
          {
           set statusArray($i) "$stat 1"
          } \
       else \
          {
           set fName  [format $predDir/res%03d.tab $i]

           if {[file exists $fName]} \
              {
               set statusArray($i) "$stat [file mtime $fName]"
              } \
           else \
              {
               set statusArray($i) "None 0"
               gdbSet $entry CLASS None 
              }
          }
      }

   set currentRes [expr $firstRes + 1]
}

#
# Configure the given residue button according to its status.

proc ramaButton { b resID } \
{
   global centerColor zeroColor defColor1 defColor2 activeColor1 activeColor2
   global goodColor1 goodColor2 badColor1 badColor2 warnColor1 warnColor2 dynColor1 dynColor2
   global color1 color2

   global statusArray currentRes 

   if {![info exists statusArray($resID)]} \
      {
       set entry [ramaGetEntry $resID]
       set statusArray($resID) "None 0"
       set stat None 

       gdbSet $entry CLASS None 
      } \
   else \
      {
       set stat [lindex $statusArray($resID) 0]
      }

   switch $stat \
      {
       None    {
                set s   disabled
                set bg  $defColor1
                set abg $defColor2
                set afg $color1
                set fg  $color2
               }

       Good    {
                set s   normal
                set bg  $goodColor1
                set abg $goodColor2
                set afg $bg
                set fg  $color2
               } 

       Dyn     {
                set s   normal
                set bg  $dynColor1
                set abg $dynColor2
                set afg $bg
                set fg  $color2
               }


       Bad     {
                set s   normal
                set bg  $badColor1
                set abg $badColor2
                set afg $bg
                set fg  $color2
               }

       Warn    {
                set s   normal
                set bg  $warnColor1
                set abg $warnColor2
                set afg $bg
                set fg  $color2
               }

       default {
                set s   normal
                set bg  $defColor1
                set abg $defColor2
                set afg $color1
                set fg  $color2
               } 
      }

   if {$resID == $currentRes} \
      {
       set fg  $afg
       set bg  $activeColor1
       set abg $activeColor2
      }

   $b configure -fg $fg -bg $bg -activebackground $abg -state $s
}

#
# Update the grid of residue choices.

proc ramaUpdate {} \
{
   global seqList r1 rN firstRes lastRes colCount

   if {![strcasecmp $r1 None]} return
   if {![strcasecmp $rN None]} return

   set row      1
   set col      0
   set newCount 0

   for {set i $firstRes} {$i <= $lastRes} {incr i} \
      {
       if {$i < $r1 || $i > $rN} \
          {
           set resName ?
          } \
       else \
          {
           set resName [lindex $seqList [expr $i - $r1]] 
          }

       set  rowName .t.row$row
       incr col

       set packFlag 0

       if {![winfo exists $rowName]} \
          {
           set packFlag 1
           frame $rowName
           incr newCount
          }

       set bName $rowName.col$col 

       if {![winfo exists $bName]} \
          {
           button $bName -text $resName$i -width 4 -padx 2 -pady 1 \
                  -command "ramaSet $i"

           pack $bName -in $rowName -side left
          }

       if {$packFlag} {pack $rowName -in .t -side top -anchor w}

       ramaButton $bName $i

       if {$col == $colCount} \
          {
           set  col 0
           incr row
          }
      }

   if {[info exists env(BINVAL)]} \
      {
       set binVal $env(BINVAL)
      } \
   else \
      {
       set binVal other
      }

   if {$newCount} \
      {
       if {![strcasecmp $binVal winxp]} \
          {
           set bwide 29 
           set bhi   21 

           set wide [expr $bwide*(4 + $colCount)]
           set hi   [expr $bhi*(2 + $row)]

           wm geom .t ${wide}x${hi} 
          } \
       else \
          {
           set wide [expr 2 + 4*$colCount]c
           set hi   [expr 1 + $col]c
    
           .t configure -width $wide -height $hi
          }
      }

   update
}

#
# Update display, and set for future automated update if needed.

proc ramaAuto {} \
{
   global autoFlag

   ramaScan
   ramaUpdate

   if {$autoFlag} {after 30000 ramaAuto}
}

#
# Scan the contents of the prediction directory; updates ranges and currentRes.

proc ramaScan {} \
{
   global predDir firstRes lastRes r1 rN currentRes statusArray predAllFlag

   if {$predAllFlag} \
      {
       set minRes $r1 
       set minRes $rN 
      } \
   else \
      {
       set fList [lsort -dictionary [glob -nocomplain $predDir/res*.tab]]
       set n     0

       foreach fName $fList \
          {
           set  rName  [file rootname [file tail $fName]]
           scan $rName "res%d" resID
        
           if {!$n} \
              {
               set minRes $resID
               set maxRes $resID
              }

           if {$resID < $minRes} {set minRes $resID}
           if {$resID > $maxRes} {set maxRes $resID}

           if {[info exists statusArray($resID)]} \
              {
               set tPrev [lindex $statusArray($resID) 1] 
              } \
           else \
              {
               set tPrev 0
              }

           set t [file mtime $fName]

           if {$t > $tPrev} \
              {
               set statusArray($resID) "New $t"
              }
    
           incr n
          }
      }

   if {![strcasecmp $r1 None]} {set r1 $minRes}
   if {![strcasecmp $rN None]} {set rN $maxRes}

   if {![strcasecmp $firstRes First]} {set firstRes $r1}
   if {![strcasecmp $lastRes  Last]}  {set lastRes  $rN}

   if {$firstRes < $r1} {set firstRes $r1}
   if {$lastRes  < $r1} {set lastRes  $r1}

   if {$firstRes > $rN} {set firstRes $rN}
   if {$lastRes  > $rN} {set lastRes  $rN}

   if {$firstRes > $lastRes} \
      {
       set itemp    $firstRes
       set firstRes $lastRes
       set $lastRes $itemp
      }

   if {![strcasecmp $currentRes None]} {set currentRes $firstRes}
}



#
#
# Read the predicted angle table.

proc ramaReadAngles {} \
{
   global currentRes predDir ramaTab predAllFlag

   if {$predAllFlag} return

   if {$ramaTab} \
      {
       gdbDestroy $ramaTab
       set ramaTab 0
      }

   set fName [format $predDir/res%03d.tab $currentRes]

   if {[file exists $fName]} \
      {
       set ramaTab [gdbRead table -in $fName -name $fName]
      }
}

proc ramaWriteAngles {} \
{
   global currentRes predDir ramaTab predAllName predAllTab predAllFlag

   if {$predAllFlag} \
      {
       if {!$predAllTab} {return 0}
       gdbWrite $predAllTab $predAllName
      } \
   else \
      {
       if {!$ramaTab} {return 0}
       set fName [format $predDir/res%03d.tab $currentRes]
       gdbWrite $ramaTab $fName
      }
}

#
# Mark current residue prediction as good, bad, etc.

proc ramaMark { type } \
{
   global statusArray currentRes currentStat sumTab

   if {[info exists statusArray($currentRes)]} \
      {
       set t [lindex $statusArray($currentRes) 1]
      } \
   else \
      {
       set t 0
      }

   set statusArray($currentRes) "$type $t"
   set currentStat $type

   set entry [gdbSelect -from $sumTab -cond "RESID == $currentRes"]

   if {[llength $entry] == 1} \
      {
       gdbSet $entry CLASS $type
      }

   update

   ramaUpdate 

   catch ramaShowMol
}

#
# Clear current residue display.

proc ramaClear {} \
{
   global matchCount selArray boxArray 

   .c delete label box outline center
   .c itemconfigure info -text "" 
   .s.summary configure  -text ""

   set selArray(Ref) 0
   set boxArray(Ref) 0

   for {set i 0} {$i < $matchCount} {incr i} \
      {
       set selArray($i) 0
       set boxArray($i) 0

       .s.b$i configure -command "ramaSelect $i 0" -text "Prediction $i"
      }
}

#
# Show the current angles.

proc ramaShowAngles {} \
{
   global currentRes currentStat seqList selArray statusArray r1 rN 
   global centerColor zeroColor activeColor goodColor1 badColor1 warnColor1
   global sumTab ramaTab avgFunc devFunc talosVersion inName annTab s2Tab ssTab
   global predAllTab predAllFlag

   global xOff yOff color1 color2 cellX cellY boxArray stdfont
   global phiAvg psiAvg phiDev psiDev
   global xListH yListH xListE yListE xListS2 yListS2

   ramaClear
   drawResBG

   set entry  [annGetEntry $currentRes]
   
   set QA [gdbGet $entry Q_A]
   set QB [gdbGet $entry Q_B]
   set QP [gdbGet $entry Q_P]
 
   set maxVal [expr max($QA,max($QB,$QP))] 
   set minVal [expr min($QA,min($QB,$QP))] 
   set conf   [expr 2.0*$maxVal-$QA-$QB-$QP+$minVal]

#   set fmt " ABP %3d %8.3f %8.3f %8.3f"
#   puts [format $fmt $currentRes $QA $QB $QP]
   set fmt "%4.2f"

   set x0 [expr $xOff + 60]
   set y0 [expr $yOff + 210]
   set txtColor $warnColor1
   if {$conf >= 0.8 && $QA == $maxVal} {set txtColor $goodColor1}
   .c create text $x0 $y0 -anchor c -text [format $fmt $QA] -tags center -justify c -fill $txtColor 

   set x0 [expr $xOff + 30]
   set y0 [expr $yOff + 10]
   set txtColor $warnColor1
   if {$conf >= 0.8 && $QB == $maxVal} {set txtColor $goodColor1}
   .c create text $x0 $y0 -anchor c -text [format $fmt $QB] -tags center -justify c -fill $txtColor

   set x0 [expr $xOff + 240]
   set y0 [expr $yOff + 150]
   set txtColor $warnColor1
   if {$conf >= 0.8 && $QP == $maxVal} {set txtColor $goodColor1}
   .c create text $x0 $y0 -anchor c -text [format $fmt $QP] -tags center -justify c -fill $txtColor

#
# Extract angles and stats:

   if {$predAllFlag} \
      {
       if {!$predAllTab} return

       set currentStat [lindex $statusArray($currentRes) 0]

       set eList   [gdbSelect -from $predAllTab -cond "RESID == $currentRes"]

       set idList  [getIota 1 1 [llength $eList]]
       set phiList [getEListVals $eList PHI]
       set psiList [getEListVals $eList PSI]
       set dList   [getEListVals $eList DIST]
       set wList   [getEListVals $eList W]
       set srcList [getEListVals $eList SOURCE]
      } \
   else \
      {
       if {!$ramaTab} return

       set currentStat [lindex $statusArray($currentRes) 0]

       set eList   [gdbGetInfo $ramaTab -parms entries]

       set var     [gdbFindByName $ramaTab INDEX]
       set idList  [gdbGetInfo $var -parms values]

       set var     [gdbFindByName $ramaTab PHI]
       set phiList [gdbGetInfo $var -parms values]

       set var     [gdbFindByName $ramaTab PSI]
       set psiList [gdbGetInfo $var -parms values]

       set var     [gdbFindByName $ramaTab DIST]
       set dList   [gdbGetInfo $var -parms values]
    
       set var     [gdbFindByName $ramaTab W]
       set wList   [gdbGetInfo $var -parms values]

       set var     [gdbFindByName $ramaTab SOURCE]
       set srcList [gdbGetInfo $var -parms values]
      }

   set phiAvg [format "%.0f" [$avgFunc [lselect $phiList $wList]]]
   set psiAvg [format "%.0f" [$avgFunc [lselect $psiList $wList]]]

   set phiDev [format "%.0f" [$devFunc [lselect $phiList $wList]]]
   set psiDev [format "%.0f" [$devFunc [lselect $psiList $wList]]]

   set dAvg   [format "%.2f" [getAvg   [lselect $dList   $wList]]]

   set n [llength $idList]

#
# Label center and extent:

   set x0 [expr $xOff + $phiAvg + 180]
   set y0 [expr $yOff + 180 - $psiAvg]
   set xn [expr $xOff + 360]
   set yn [expr $yOff + 360]

   .c create line $xOff $y0 $xn $y0 -fill $centerColor -tags center
   .c create line $x0 $yOff $x0 $yn -fill $centerColor -tags center

   set x1 [expr $x0 - $phiDev]
   set xn [expr $x0 + $phiDev]
   set y1 [expr $y0 - $psiDev]
   set yn [expr $y0 + $psiDev]

   .c create oval $x1 $y1 $xn $yn -outline $centerColor -tags center

   set xn [expr $xOff + 360 + $cellX]
   set yn [expr $yOff + 360 + $cellY]

   .c create text $x0 $yn -anchor c -text "$phiAvg +/- $phiDev\nPhi" \
      -font $stdfont -tags center -justify c -fill $color2

   .c create text $xn $y0 -anchor w -text "$psiAvg +/- $psiDev\nPsi" \
      -font $stdfont -tags center -justify c -fill $color2

#
# Individual labels:
 
   for {set i 0} {$i < $n} {incr i} \
      {
       set phi [lindex $phiList $i]
       set psi [lindex $psiList $i]
       set src [lindex $srcList $i]
       set id  [lindex $idList  $i]
       set d   [lindex $dList   $i]
       set w   [lindex $wList   $i]
       set e   [lindex $eList   $i]
     
       set x0 [expr $xOff + $phi + 180]
       set y0 [expr $yOff + 180 - $psi]

       set x1 [expr $x0 - 3]
       set y1 [expr $y0 - 3]
       set xn [expr $x0 + 3]
       set yn [expr $y0 + 3]

       if {$w == 0.0} \
          {
           set color $badColor1
           set selArray($i) 0
          } \
       else \
          {
           set color $goodColor1
           set selArray($i) 1
          }

       set str [getResData $e]

       set boxArray($i) \
        [.c create rectangle $x1 $y1 $xn $yn \
           -outline $color1 -fill $color -tags "box box$i"]

       .c bind $boxArray($i) <Any-Enter>       "+ ramaEnterBox $i"
       .c bind $boxArray($i) <Any-Leave>       "+ ramaLeaveBox $i"

       .c bind $boxArray($i) <ButtonRelease-1> "+ ramaToggle $i $e"

       set fmt "Phi: %4.0f Psi: %4.0f D: %5.2f %s %s"

       .s.b$i configure -command "ramaSelect $i $e" -text \
          [format $fmt $phi $psi $d $str $src]
      }

   ramaRef

#
# Update the Prediction Select Title:

   set resID1 [expr $currentRes - 1]
   set resID2 [expr $resID1 + 1]
   set resID3 [expr $resID2 + 1]

   set rList ""

   set str "Triplet"

   foreach i "$resID1 $resID2 $resID3" \
      {
       if {$i < $r1 || $i > $rN} \
          {
           lappend rList ?
          } \
       else \
          {
           lappend rList [lindex $seqList [expr $i - $r1]]
          }
      }

   set name1 [lindex $rList 0]
   set name2 [lindex $rList 1]
   set name3 [lindex $rList 2]

   set    str "Residue $name2$resID2, Triplet"
   append str " $name1$resID1 $name2$resID2 $name3$resID3"

   wm title . "TALOS $talosVersion   Residue $name2$resID2 $inName"
   wm title .s $str

   set str "               "
   set src "Average"

   set fmt "Phi: %4.0f Psi: %4.0f D: %5.2f %s %s"

   .s.summary configure -text [format $fmt $phiAvg $psiAvg $dAvg $str $src]


# update ANN frame
    .r.c.g marker create line -coords {$currentRes -0.2 $currentRes 1.2} -outline yellow -name currentLine -under yes
    set resName [lindex $seqList [expr $currentRes-$r1]] 
    set entry [gdbSelect -from $s2Tab -cond "RESID == $currentRes"]
    if {[llength $entry] == 0} return
    
    set s2 [gdbGet $entry S2]
#    puts [lindex $yListS2 $currentRes] 
    set fmt "Predicted RCI S2 value %d %s: %4.2f"
    .r.c.g configure -title [format $fmt $currentRes $resName $s2 ]

 
    .r.css.b marker create line -coords {$currentRes -1.2 $currentRes 1.2} -outline yellow -name currentLine -under yes
    set entry [gdbSelect -from $ssTab -cond "RESID == $currentRes"]
    if {[llength $entry] == 0} return
    set a [gdbGet $entry Q_H]
    set b [gdbGet $entry Q_E]
    set p [gdbGet $entry Q_L]
    set fmt "Predicted Secondary Structure %d %s: \[%4.2f|%4.2f|%4.2f\]"
    .r.css.b configure -title [format $fmt $currentRes $resName $a $b $p ]


   update

#
# Update the summary table:

   set entry [ramaGetEntry $resID2]
   set n     [llength [lselect $dList $wList]]

   gdbSet $entry PHI     $phiAvg
   gdbSet $entry PSI     $psiAvg
   gdbSet $entry DPHI    $phiDev
   gdbSet $entry DPSI    $psiDev
   gdbSet $entry DIST    $dAvg

   gdbSet $entry COUNT   $n
   gdbSet $entry CLASS   $currentStat
}

#
# Show shift distributions.

proc ramaShowShifts {} \
{
   global shiftLimits sdWidth sdHeight selArray sdArray sdFlag predAllFlag
   global predAllTab rcArray adjArray prevArray nextArray r1 rN

   global goodColor1 goodColor2 badColor1 badColor2 warnColor1 warnColor2
   global currentRes shiftTab ramaTab talosTab seqList npList matchCount
   global refColor color1 color2

#
# Erase the current distribution:

   if {!$sdFlag}  return

   if {$predAllFlag} \
      {
       if {!$predAllTab} return
      } \
   else \
      {
       if {!$ramaTab} return
      }

   set sdArray(Ref)  ""

   for {set i 0} {$i < 18} {incr i} \
      {
       .d.c$i delete box
      }

   for {set i 0} {$i < $matchCount} {incr i} \
      {
       set sdArray($i)  ""
      }

   update

#
# Show the distribution of shifts from the database:

   if {$predAllFlag} \
      {
       set eListI [gdbSelect -from $predAllTab -cond "RESID == $currentRes"]
      } \
   else \
      {
       set eListI [gdbGetInfo $ramaTab -parms entries]
      }

   set ii 0

   foreach entryI $eListI \
      {
       set src [gdbGet $entryI SOURCE]

       if {$predAllFlag} \
          {
           set resName [gdbGet $entryI RESNAME_R1]
           set resID   [gdbGet $entryI RESID_R1]
           set res     $resName$resID

           set entryJ $entryI
          } \
       else \
          {
           set res [gdbGet $entryI R1]
           scan $res "%c%d" resName resID

           set cond   "RESID_R1 == $resID && !strcasecmp( '$src', SOURCE )"
           set entryJ [gdbSelect -from $talosTab -cond $cond]
          }

       if {[llength $entryJ] != 1} \
          {
           incr ii
           set  entryI [gdbNext $entryI]
           puts "Warning: no match, $src $resID ..."
           continue
          }

       if {$selArray($ii) == 0.0} \
          {
           set color $badColor1
          } \
       else \
          {
           set color $goodColor1
          }

       set k 0

       foreach i {_R1 _R2 _R3} \
          {
           set kk   0

           foreach j {HN N C CA CB HA} \
              {
               set maxShift [lindex $shiftLimits $kk]
               set shift    [gdbGet $entryJ $j$i]
               set secShift [format "%.3f" $shift]

               if {$shift > 999.0} \
                  {
                   incr k
                   incr kk
                   continue
                  }

               if {$shift >  $maxShift} {set shift  $maxShift}
               if {$shift < -$maxShift} {set shift -$maxShift}

               set x0 [expr $sdWidth*($shift + $maxShift)/(2.0*$maxShift)]
               set x1 [expr $x0 - 2]
               set xn [expr $x0 + 2]

               set val \
                 [.d.c$k create rectangle $x1 2 $xn $sdHeight \
                    -fill $color -outline $color2 -tags box -width 1]

               lappend sdArray($ii) "$k $val $secShift"

               .d.c$k bind $val <Any-Enter>       "+ ramaEnterBox $ii"
               .d.c$k bind $val <Any-Leave>       "+ ramaLeaveBox $ii"
               .d.c$k bind $val <ButtonRelease-1> "+ ramaToggle   $ii $entryI"

               incr kk
               incr k
              }
          }

       incr ii
      }

#
# Show the shifts from the unknown:

   set k     0
   set color $refColor

   for {set i [expr $currentRes-1]} {$i <= [expr $currentRes+1]} {incr i} \
       {
        if {!$shiftTab} continue
        if {$i < $r1}   continue

        set resName  [lindex $seqList [expr $i - $r1]]
        set prevName [lindex $npList  [expr $i - $r1]]
        set nextName [lindex $npList  [expr $i - $r1 + 1]]
        set kk   0

        foreach j {HN N C CA CB HA} \
           {
            set maxShift [lindex $shiftLimits $kk]
            set shift    9999.0

            if {![strcasecmp $resName G] && ![strcasecmp $j HA]} \
               {
                set cond  "RESID == $i && !strcasecmp( ATOMNAME, 'HA2' )"
                set entry [gdbSelect -from $shiftTab -cond $cond]

                if {[llength $entry] == 1} \
                   {
                    set shift [gdbGet $entry SHIFT]
                   }

                set cond  "RESID == $i && !strcasecmp( ATOMNAME, 'HA3' )"
                set entry [gdbSelect -from $shiftTab -cond $cond]

                if {[llength $entry] == 1} \
                   {
                    set shift \
                       [expr 0.5*($shift + [gdbGet $entry SHIFT])]
                   }
               } \
            else \
               {
                set cond  "RESID == $i && !strcasecmp( ATOMNAME, \"$j\" )"
                set entry [gdbSelect -from $shiftTab -cond $cond]

                if {[llength $entry] == 1} \
                   {
                    set shift [gdbGet $entry SHIFT]
                   }
               }

            if {$shift > 999.0} \
               {
                incr k
                incr kk
                continue
               }

            set rcVal [expr $rcArray($resName,$j) + $adjArray($resName,$j)]
            set rcVal [expr $rcVal + $prevArray($prevName,$j)]
            set rcVal [expr $rcVal + $nextArray($nextName,$j)]

            set shift    [expr $shift - $rcVal]
            set secShift [format "%.3f" $shift]

            if {$shift >  $maxShift} {set shift  $maxShift}
            if {$shift < -$maxShift} {set shift -$maxShift}

            set x0 [expr $sdWidth*($shift + $maxShift)/(2.0*$maxShift)]
            set x1 [expr $x0 - 2]
            set xn [expr $x0 + 2]

            set val \
              [.d.c$k create rectangle $x1 2 $xn $sdHeight \
                 -fill $color -outline $color2 -tags box -width 1]

            lappend sdArray(Ref) "$k $val $secShift"

            .d.c$k bind $val <Any-Enter>  "+ ramaEnterBox Ref"
            .d.c$k bind $val <Any-Leave>  "+ ramaLeaveBox Ref"

            incr kk
            incr k
           }
       }
}

#
# Show the reference info.

proc ramaRef {} \
{
   global xOff yOff refTab color1 refColor currentRes
   global refPhi refPsi refInfo boxArray

   if {!$refTab} {return}

   set aList   [getAngles [expr $currentRes - 1] $refTab]
   set phi     [lindex $aList 0]
   set psi     [lindex $aList 1]
   set refInfo [lindex $aList 2]
   set refPhi  $phi
   set refPsi  $psi

   if {$phi > 999.0 || $psi > 999.0} {return}

   set x0 [expr $xOff + $phi + 180]
   set y0 [expr $yOff + 180 - $psi]

   set x1 [expr $x0 - 3]
   set y1 [expr $y0 - 3]
   set xn [expr $x0 + 3]
   set yn [expr $y0 + 3]

   set boxArray(Ref) \
     [.c create rectangle $x1 $y1 $xn $yn \
           -outline $color1 -fill $refColor -tags "box boxRef"]

   .c bind $boxArray(Ref) <Any-Enter>       "+ ramaEnterBox Ref"
   .c bind $boxArray(Ref) <Any-Leave>       "+ ramaLeaveBox Ref"

}



#
# Find or add an entry in the ANN summary table.

proc annGetEntry { resID } \
{
   global seqList r1 rN annTab
   set entry [gdbSelect -from $annTab -cond "RESID == $resID"]

   if {[llength $entry] == 0} \
      {
       set entry [gdbCreate entry -parent $annTab]

       if {$resID < $r1 || $resID > $rN} \
          {
           set resName ?
          } \
       else \
          {
           set resName [lindex $seqList [expr $resID - $r1]]
          }

       gdbSet $entry RESID   $resID
       gdbSet $entry RESNAME $resName

       gdbSet $entry Q_A        0.0
       gdbSet $entry Q_B        0.0
       gdbSet $entry Q_P        0.0

       gdbSet $entry CS_CNT        0
       gdbSet $entry CS_CNT_R2     None
      }

   return $entry
}


#
# Find or add an entry in the summary table.

proc ramaGetEntry { resID } \
{
   global seqList r1 rN sumTab

   set entry [gdbSelect -from $sumTab -cond "RESID == $resID"]


   if {[llength $entry] == 0} \
      {
       set entry [gdbCreate entry -parent $sumTab]

       if {$resID < $r1 || $resID > $rN} \
          {
           set resName ?
          } \
       else \
          {
           set resName [lindex $seqList [expr $resID - $r1]]
          }

       gdbSet $entry RESID   $resID
       gdbSet $entry RESNAME $resName

       gdbSet $entry PHI     9999.0
       gdbSet $entry PSI     9999.0
       gdbSet $entry DPHI       0.0
       gdbSet $entry DPSI       0.0
       gdbSet $entry DIST       0.0

       gdbSet $entry COUNT        0
       gdbSet $entry CLASS     None
      }

   return $entry
}

#
# Get the triplet data from the given entry.

proc getResData { entry } \
{
   global predAllFlag

   set f   " %-4s"
   set str ""

   if {$predAllFlag} \
      {
       append str [format $f [gdbGet $entry RESNAME_R1][gdbGet $entry RESID_R1]]
       append str [format $f [gdbGet $entry RESNAME_R2][gdbGet $entry RESID_R2]]
       append str [format $f [gdbGet $entry RESNAME_R3][gdbGet $entry RESID_R3]]
      } \
   else \
      {
       append str [format $f [gdbGet $entry R1]]
       append str [format $f [gdbGet $entry R2]]
       append str [format $f [gdbGet $entry R3]]
      }

   return $str
}

#
# Set the current residue number on display.

proc ramaNext {} \
{
   global currentRes r1 rN

   incr currentRes

   if {$currentRes > $rN - 1} {set currentRes [expr $r1 + 1]}

   ramaSet $currentRes
}

proc ramaPrev {} \
{
   global currentRes r1 rN 

   incr currentRes -1

   if {$currentRes < [expr $r1 + 1]} {set currentRes [expr $rN - 1]}

   ramaSet $currentRes
}

proc ramaSet { thisRes } \
{
   global currentRes r1 rN 

   if {$thisRes >= $r1 && $thisRes <= $rN} \
      {
       set currentRes $thisRes

       ramaReadAngles
       ramaUpdate
       ramaShowAngles
       ramaShowShifts

       catch ramaShowMol
      }
}

#
# Save all current results.

proc ramaSave {} \
{
   global sumName sumTab

   if {!$sumTab} return

   set types \
      {
       { {All Files}          *      }
       { {tab}                {.tab} }
       { {angle constraints}  {.aco} }
      }

   set thisName \
     [tk_getSaveFile -title "Prediction Summary Output" \
       -filetypes $types -initialfile $sumName]

   if {![strcasecmp $thisName None]} return
   if {![string length $thisName]} return

   gdbWrite $sumTab $thisName
   puts "Wrote prediction summary table $thisName"
}

#
# Save results and exit.

proc ramaQuit {} \
{
   global rasmolFlag

   ramaSave

   if {$rasmolFlag} {catch "send rasmol quit"}

   exit
}

#
# Create the graphics interface.

proc ramaFrames {} \
{
   global talosVersion cellX cellY xOff yOff xExtra yExtra matchCount r1 rN
   global sdArray selArray boxArray currentStat sdHeight sdWidth sdFlag currentRes
   global color1 color2 goodColor1 badColor1 warnColor1 dynColor1 defColor1 stdfont
   global axisfont talosDir inName
   global xListH yListH xListE yListE xListS2 yListS2
   
   toplevel .t
   toplevel .s

#
# Initialize:

   set selArray(Ref) 0
   set boxArray(Ref) 0
   set sdArray(Ref)  ""

   for {set i 0} {$i < $matchCount} {incr i} \
      {
       set selArray($i) 0
       set boxArray($i) 0
       set sdArray($i)  ""
      }

#
# Rama Frame:

   frame    .bottom1
   frame    .bottom2

   set xw [expr 1 + 360 + 2*$xOff + $xExtra]
   set yw [expr 1 + 360 + 2*$yOff + $yExtra]

   image create photo ramaImage -file $talosDir/rama.gif -palette 16

   canvas .c -width $xw -height $yw

   .c create text $cellX $cellY -font $stdfont -text "" -anchor sw -tags info 

   checkbutton .bGood -text Good         -command "ramaMark Good" -width 12 \
      -indicatoron 0 -variable currentStat -onvalue Good -offvalue None     \
      -selectcolor $goodColor1

   checkbutton .bWarn -text Ambiguous    -command "ramaMark Warn" -width 12 \
      -indicatoron 0 -variable currentStat -onvalue Warn -offvalue None     \
      -selectcolor $warnColor1

   checkbutton .bDyn  -text Dynamic      -command "ramaMark Dyn" -width 12 \
      -indicatoron 0 -variable currentStat -onvalue Dyn  -offvalue None     \
      -selectcolor $dynColor1

   checkbutton .bBad  -text Bad          -command "ramaMark Bad"  -width 12 \
      -indicatoron 0 -variable currentStat -onvalue Bad -offvalue None      \
      -selectcolor $badColor1

   checkbutton .bNew  -text Unclassified -command "ramaMark New"  -width 12 \
      -indicatoron 0 -variable currentStat -onvalue New -offvalue None      \
      -selectcolor $defColor1

   button .bNext  -text Next    -command ramaNext         -width 7
   button .bPrev  -text Prev    -command ramaPrev         -width 7
   button .bShow  -text Redraw  -command ramaShowAngles   -width 7
   button .bClear -text Clear   -command ramaClear        -width 7
   button .bSave  -text Save    -command ramaSave         -width 7
   button .bQuit  -text Quit    -command ramaQuit         -width 7

   pack .bGood .bWarn .bDyn .bBad .bNew \
      -in .bottom1 -side left -expand 1 -fill x

   pack .bPrev .bNext .bShow .bClear .bSave .bQuit \
      -in .bottom2 -side left -expand 1 -fill x

   pack .c -in .
   pack .bottom1 .bottom2 -in . -side top -expand 1 -fill x

   wm title . "TALOS $talosVersion"

#
# Residue Select Frame:

   if {[strcasecmp $inName None]} \
      {
       wm title .t "TALOS $inName Residues $r1 to $rN"
      } \
   else \
      {
       wm title .t "TALOS Residue Select"
      }

#
# Prediction Select Frame:

   wm title .s "TALOS Prediction Select"

   for {set i 0} {$i < $matchCount} {incr i} \
      {
       checkbutton .s.b$i -command "ramaSelect $i 0" -variable selArray($i)  \
          -selectcolor $goodColor1 -text "Prediction $i" -width 60 -anchor w \
          -font $axisfont

       bind .s.b$i <Any-Enter> "+ ramaEnterButton $i" 
       bind .s.b$i <Any-Leave> "+ ramaLeaveButton $i" 

       pack .s.b$i -side top
      }

   set str "               "
   set src "Average"

   checkbutton .s.summary -width 60 -anchor w -state disabled \
      -disabledforeground black -font $axisfont

   set fmt "Phi: %4.0f Psi: %4.0f D: %5.2f %s %s"

   .s.summary configure -text [format $fmt 0.0 0.0 0.0 $str $src]

   pack .s.summary -side top -expand 1 -fill x

#
# Shift Distribution Frame:
 
   if {$sdFlag} \
      {
       toplevel .d 

       set k    0

       foreach i {"i-1" "i" "i+1"} \
          {
           foreach j {HN N CO CA CB HA} \
              {
               set pInfo($i,$j) .d.f$k

               frame  $pInfo($i,$j)  
               canvas .d.c$k -width $sdWidth -height $sdHeight
               label  .d.l$k -text [format "%s(%s)" $j $i] -anchor w -width 7
               label  .d.s$k -anchor e -width 7

               pack   .d.l$k .d.c$k .d.s$k -in .d.f$k -side left -padx 2
             

               set xm [expr $sdWidth/2]

              .d.c$k create line $xm 1 $xm $sdHeight \
                 -fill $color2 -tags bg 

              .d.c$k create line  1 $sdHeight $sdWidth $sdHeight \
                 -fill $color2 -tags bg 

               incr k
              }
          }

       update
 
       set k 0
 
       foreach j {HN N CO CA CB HA} \
          {
           foreach i {"i-1" "i" "i+1"} \
              {
               pack $pInfo($i,$j) -side top -pady 1
              }

           frame .d.space$k -height 1
           pack  .d.space$k -side top -pady 4

           incr k
          }

       wm title .d "TALOS Secondary Shift Distributions"
       update
      }

#
# RCI-S2 Distribution Frame:
   toplevel .r

   wm title .r "RCI-S2 and Secondary Structure Distributions"
   set w 5
   if {$rN-$r1 < 99 } {set w 8}
   set sw [expr ($rN-$r1+1)*$w]

   frame  .r.t
   canvas .r.t.c -width $sw
   .r.t.c create text $cellX $cellY -font $stdfont -text "XX" -anchor sw -tags ssinfo 
   pack .r.t.c -in .r.t -side top -expand 1 -fill both

   frame  .r.f

   canvas .r.c   -width $sw -height 150
   graph  .r.c.g -plotbackground $color2 -width $sw -height 150 -plotpadx 8 -plotpady 8 -title "RCI-S2"
   .r.c.g crosshairs on
   .r.c.g crosshairs configure -color white

   pack .r.c.g -in .r.c -side top -expand 1 -fill both
   pack .r.c -in .r.f -side top -expand 1 -fill both
# -side left -padx 2

   .r.c.g axis configure x -tickfont $axisfont -ticklength 0.05i
   .r.c.g axis configure y -tickfont $axisfont -ticklength 0.05i -min 0 -max 1 -majorticks {0 0.5 1} -minorticks {1.25} 
   .r.c.g axis configure x -step 10.0
   .r.c.g legend configure -hide 1

# plot RCI-S2 
   catch {.r.c.g element create "S2" -xdata $xListS2 -ydata $yListS2 -pixels 2 -linewidth 1 -color "green" -fill "white"}  

   .r.c.g element bind "S2" <ButtonRelease-1> {
       set newX [%W axis invtransform x %x]
       ramaSet [expr {round($newX)} ]
    }


   .r.c.g marker create line -coords {-99 0.5 999 0.5} \
         -outline yellow -name cutoffLine -under yes -dashes 1

#   .r.c.g marker create text -coord "0 0" -anchor sw -foreground white -text "" -name s2Marker

   bind .r.c.g <Motion> {
       .r.c.g crosshairs configure -position @%x,%y
        set maxY [.r.css.b axis cget y -max]
        set newY [.r.css.b axis transform y $maxY]
        set gX   [.r.c.g   axis invtransform x %x]
        set newX [.r.css.b axis transform x $gX]
       .r.css.b   crosshairs configure -position @$newX,$newY

       set mouseX [expr {round($gX)} ]
#       set gY [.r.c.g   axis invtransform y %y]
       
       if {$mouseX <= $r1} { set mouseX [expr $r1+1] }
       if {$mouseX >= $rN} { set mouseX [expr $rN-1] }
 #      if {$gY >= 0.9} { set gY 0.9 }
#       if {$gX >= $rN-5} { set gX [expr $rN-5] }

       set entry [gdbSelect -from $s2Tab -cond "RESID == $mouseX"]
       if {[llength $entry] == 0} {set s2 0.0} else {set s2 [gdbGet $entry S2]}
       set resName  [lindex $seqList [expr $mouseX-$r1]] 

       set entry [gdbSelect -from $s2Tab -cond "RESID == $currentRes"]
       if {[llength $entry] == 0} {set s2C 0.0} else {set s2C [gdbGet $entry S2]}
       set resNameC [lindex $seqList [expr $currentRes-$r1]] 

#       .r.c.g marker configure s2Marker -text "$mouseX$resName: $s2 $currentRes$resNameC: $s2C" -coord "$gX $gY"
       set fmt "Predicted RCI S2 value \[%%d %%s: %%4.2f\] \[%%d %%s: %%4.2f\]"
       .r.c.g configure -title [format $fmt $currentRes $resNameC $s2C $mouseX $resName $s2]


       # adjust text in SS frame
       set entry [gdbSelect -from $ssTab -cond "RESID == $mouseX"]
       if {[llength $entry] == 0} \
          {
	   set h 0.0
	   set e 0.0
	   set l 0.0
	  }\
       else\
          {
           set h [gdbGet $entry Q_H]
           set e [gdbGet $entry Q_E]
           set l [gdbGet $entry Q_L]
          }

       set entry [gdbSelect -from $ssTab -cond "RESID == $currentRes"]
       if {[llength $entry] == 0} \
          {
	   set hC 0.0
	   set eC 0.0
	   set lC 0.0
	  }\
       else\
          {
           set hC [gdbGet $entry Q_H]
           set eC [gdbGet $entry Q_E]
           set lC [gdbGet $entry Q_L]
          }
       set fmt "Predicted Secondary Structure \[%%d %%s: %%4.2f|%%4.2f|%%4.2f\] \[%%d %%s: %%4.2f|%%4.2f|%%4.2f\]"
       .r.css.b configure -title [format $fmt $currentRes $resNameC $hC $eC $lC $mouseX $resName $h $e $l]
#       .r.css.b marker configure ssMarker -text "$mouseX$resName:$h $e $l" -coord "$gX 0.8"

    }

#
# SS distribution barlpot frame
   frame  .r.fss
   canvas .r.css  -width $sw -height 150
   barchart  .r.css.b -plotbackground $color2 -width $sw -height 150 -plotpadx 8 -plotpady 8 -title "Predicted Secondary Structure"

   .r.css.b axis configure x -tickfont $axisfont -ticklength 0.05i -step 10
   .r.css.b axis configure y -tickfont $axisfont -ticklength 0.05i -min -1 -max 1 -majorticks {-1 0 1} -minorticks {1} 
   .r.css.b legend configure -hide 1
   .r.css.b crosshairs on
   .r.css.b crosshairs configure -color white

 
   pack .r.css.b -in .r.css -side top -expand 1 -fill both
   pack .r.css   -in .r.fss -side top -expand 1 -fill both

#   pack .r.t .r.f .r.fss -side top -expand 1 -fill both
   pack .r.f .r.fss -side top -expand 1 -fill both


# plot SS
   catch {.r.css.b element create "H" -xdata $xListH -ydata $yListH -fg red  -barwidth 1}  

   catch {.r.css.b element create "E" -xdata $xListE -ydata $yListE -fg cyan -barwidth 1}  

   .r.css.b marker create text -coord "0 0" -anchor sw -foreground white -text "" -name ssMarker

   bind .r.css.b <Motion> {
       .r.css.b crosshairs configure -position @%x,%y
        set minY [.r.c.g   axis cget y -min]
        set newY [.r.c.g   axis transform y $minY]
        set bX   [.r.css.b axis invtransform x %x]
        set newX [.r.c.g   axis transform x $bX]
       .r.c.g   crosshairs configure -position @$newX,$newY

       set mouseX [expr {round($bX)} ]
       
       if {$mouseX <= $r1} { set mouseX [expr $r1+1] }
       if {$mouseX >= $rN} { set mouseX [expr $rN-1] }

       # adjust text in S2 frame
       set entry [gdbSelect -from $s2Tab -cond "RESID == $mouseX"]
       if {[llength $entry] == 0} {set s2 0.0} else {set s2 [gdbGet $entry S2]}
       set resName  [lindex $seqList [expr $mouseX-$r1]] 

       set entry [gdbSelect -from $s2Tab -cond "RESID == $currentRes"]
       if {[llength $entry] == 0} {set s2C 0.0} else {set s2C [gdbGet $entry S2]}
       set resNameC [lindex $seqList [expr $currentRes-$r1]] 

       set fmt "Predicted RCI S2 value \[%%d %%s: %%4.2f\] \[%%d %%s: %%4.2f\]"
       .r.c.g configure -title [format $fmt $currentRes $resNameC $s2C $mouseX $resName $s2]


       # adjust text in SS frame
       set entry [gdbSelect -from $ssTab -cond "RESID == $mouseX"]
       if {[llength $entry] == 0} \
          {
	   set h 0.0
	   set e 0.0
	   set l 0.0
	  }\
       else\
          {
           set h [gdbGet $entry Q_H]
           set e [gdbGet $entry Q_E]
           set l [gdbGet $entry Q_L]
          }
       set entry [gdbSelect -from $ssTab -cond "RESID == $currentRes"]
       if {[llength $entry] == 0} \
          {
	   set hC 0.0
	   set eC 0.0
	   set lC 0.0
	  }\
       else\
          {
           set hC [gdbGet $entry Q_H]
           set eC [gdbGet $entry Q_E]
           set lC [gdbGet $entry Q_L]
          }
       set fmt "Predicted Secondary Structure \[%%d %%s: %%4.2f|%%4.2f|%%4.2f\] \[%%d %%s: %%4.2f|%%4.2f|%%4.2f\]"
       .r.css.b configure -title [format $fmt $currentRes $resNameC $hC $eC $lC $mouseX $resName $h $e $l]
   }

  .r.css.b marker create line -coords {-99 0.0 999 0.0} -outline white -name zeroLine 

  .r.css.b element bind all <ButtonRelease-1> {
      set newX [%W axis invtransform x %x]
      .r.css.b crosshairs configure -position @%x,%y
      ramaSet [expr {round($newX)} ]
  }

#.r.css.b postscript output "ss.ps" -decorations 0 -colormode mono
 
#   .r.css.b marker create text -coord "$currentRes 0" -anchor s -foreground white -text "XXX" 


   ramaBG
   ramaUpdate
}

#
# Handlers for entering and leaving buttons or boxes:

proc ramaEnterButton { i } \
{
    global goodColor1 badColor1 color1
    global selArray sdArray sdFlag

    if {$selArray($i) == 0.0} \
       {
        set color $badColor1
       } \
    else \
       {
        set color $goodColor1
       }

   .c itemconfigure info  -text "[string trim [.s.b$i cget -text]]"
   .c itemconfigure box$i -outline $color -fill $color1
   .c raise box$i

   if {$sdFlag} \
      {
       foreach j $sdArray($i) \
          {
           set k     [lindex $j 0]
           set val   [lindex $j 1]
           set shift [lindex $j 2]

           .d.c$k itemconfigure $val -outline $color1
           .d.c$k raise $val
           .d.s$k configure -text $shift
          }
      }
}

proc ramaLeaveButton { i } \
{
   global goodColor1 badColor1 color1 color2
   global selArray sdArray sdFlag

   if {$selArray($i) == 0.0} \
      {
       set color $badColor1
      } \
   else \
      {
       set color $goodColor1
      }

   .c itemconfigure info  -text ""
   .c itemconfigure box$i -fill $color -outline $color1

   if {$sdFlag} \
      {
       foreach j $sdArray($i) \
          {
           set k   [lindex $j 0]
           set val [lindex $j 1]
 
           .d.c$k itemconfigure $val -outline $color2
           .d.s$k configure -text ""
          }
      }
}

proc ramaEnterBox { i } \
{
   global selArray sdArray sdFlag goodColor1 badColor1 refColor color1
   global refPhi refPsi refName refInfo phiAvg psiAvg

   if {![strcasecmp $i "Ref"]} \
      {
       set color $refColor

       set fmt \
        "Phi: %.0f Psi: %.0f dPhi: %.0f dPsi: %.0f Reference %s"

       set dPhi [angleDiff $refPhi $phiAvg]
       set dPsi [angleDiff $refPsi $psiAvg]

       set msg [format $fmt $refPhi $refPsi $dPhi $dPsi $refName]
      } \
   elseif {$selArray($i) == 0.0} \
      {
       set color $badColor1
       set msg   "[.s.b$i cget -text]"
       .s.b$i configure -state active
      } \
   else \
      {
       set color $goodColor1
       set msg   "[.s.b$i cget -text]"
       .s.b$i configure -state active
      }

  .c itemconfigure info  -text "[string trim $msg]"
  .c itemconfigure box$i -outline $color -fill $color1
  .c raise box$i

   if {$sdFlag} \
      {
       foreach j $sdArray($i) \
          {
           set k     [lindex $j 0]
           set val   [lindex $j 1]
           set shift [lindex $j 2]
    
           .d.c$k itemconfigure $val -outline $color1
           .d.c$k raise $val
           .d.s$k configure -text $shift
          }
      }
}

proc ramaLeaveBox { i } \
{
   global goodColor1 badColor1 refColor color1 color2
   global boxArray selArray sdArray sdFlag

   if {![strcasecmp $i "Ref"]} \
      {
       set color $refColor
      } \
   elseif {$selArray($i) == 0.0} \
      {
       set color $badColor1
       .s.b$i configure -state normal 
      } \
   else \
      {
       set color $goodColor1
       .s.b$i configure -state normal 
      }

   .c itemconfigure info  -text ""
   .c itemconfigure box$i -fill $color -outline $color1

   if {$sdFlag} \
      {
       foreach j $sdArray($i) \
          {
           set k   [lindex $j 0]
           set val [lindex $j 1]

           .d.c$k itemconfigure $val -outline $color2
           .d.s$k configure -text ""
          }
      }
}

proc ramaToggle { i entry } \
{
   global selArray

   if {$selArray($i)} \
      {
       set selArray($i) 0
      } \
   else \
      {
       set selArray($i) 1
      }

   ramaSelect $i $entry
}

#
# Handle change in selection state of a given prediction.

proc ramaSelect { i entry } \
{
   global sdArray sdFlag selArray badColor1 goodColor1

   if {!$entry} return

   gdbSet $entry W $selArray($i)

   ramaShowAngles
   ramaWriteAngles

   .c raise box$i

   if {$selArray($i) == 0.0} \
      {
       set color $badColor1
      } \
   else \
      {
       set color $goodColor1
      }

   if {$sdFlag} \
      {
       foreach j $sdArray($i) \
          {
           set k     [lindex $j 0]
           set val   [lindex $j 1]
           set shift [lindex $j 2]

           .d.c$k itemconfigure $val -fill $color
           .d.c$k raise $val
           .d.s$k configure -text $shift
          }
      }
}

#
# Positioning:

proc ramaPosition {} \
{
   global xOff yOff sdFlag

#
# Rama frame:

   set x0 [expr $xOff] 
   set y0 [expr 2*$yOff] 

   wm geometry . +$x0+$y0
   update

#
# Residue Selection frame:

   incr x0 [winfo width .]
   incr x0 $xOff 
   incr y0 $yOff 

   wm geometry .t +$x0+$y0
   update

#
# Prediction Selection frame:

#   set x0 [expr $xOff]
#   set y0 [expr 2*$yOff]

#   incr x0 $xOff 
   incr y0 [winfo height .t]
   incr y0 [expr 2*$yOff] 

   wm geometry .s +$x0+$y0
   update


#
# RCI-S2 and Secondary Structure Prediction Frame:

   set x0 [expr $xOff]
   set y0 [expr 2*$yOff]

   incr x0 $xOff 
   incr y0 [winfo height .]
   incr y0 [expr 2*$yOff] 

   wm geometry .r +$x0+$y0
   update


#
# Shift Distribution frame:

   if {$sdFlag} \
      {
       set x0 [expr $xOff]
       set y0 [expr 2*$yOff]

       incr x0 [winfo width .]
       incr x0 $xOff 
       incr x0 $xOff 

       incr y0 [winfo height .t]
       incr y0 [expr 3*$yOff] 

       wm geometry .d +$x0+$y0
       update
      }
}

#
# Molecular Viewer.

proc ramaRasMol {} \
{
   global rasmolFlag refName refTab sumTab

   if {!$rasmolFlag} return
   if {!$refTab}     return

#
# Desired Geometry 550x450-30-20

   exec rasmol &
   exec sleep 5

   send rasmol load $refName
#  send rasmol wireframe off
   send rasmol ribbons   on
   send rasmol zoom      150
   send rasmol set fontsize 16

   send rasmol select *
   send rasmol colour gray

   set entry   [gdbFirst $sumTab]

   while {$entry} \
      {
       set resID [gdbGet $entry RESID]
       set stat  [gdbGet $entry CLASS]

       switch $stat \
          {
           Good    {set c green}
           Bad     {set c red}
           Warn    {set c yellow}
           Dyn     {set c blue}
           default {set c gray}
          }

       send rasmol select $resID
       send rasmol colour $c

       set entry [gdbNext $entry]
      }

   send rasmol select *
}

#
# Show current residue.

proc ramaShowMol {} \
{
   global rasmolFlag refTab currentRes

   if {!$rasmolFlag} return
   if {!$refTab}     return

   set entry    [ramaGetEntry $currentRes]
   set resName  [gdbGet $entry RESNAME]
   set stat     [gdbGet $entry CLASS]

   switch $stat \
      {
       Good    {set c green}
       Bad     {set c red}
       Warn    {set c yellow}
       Dyn     {set c blue}
       default {set c gray}
      }

   send rasmol label off

   send rasmol select $currentRes
   send rasmol colour $c 

   set resType [getResName3 $resName]

   send rasmol select $resType$currentRes.C
   send rasmol label  $resName$currentRes
   send rasmol colour label magenta 
   send rasmol select *
}

#
# Show the ramachandran plot background.

proc ramaBG {} \
{
    global xOff yOff zeroColor centerColor surfDir drawResFlag 

    set x0 [expr $xOff + 180]
    set y0 [expr $yOff + 180]

    set xn [expr $xOff + 360]
    set yn [expr $yOff + 360]

    set pxw [expr 1 + $xn - $xOff]
    set pyw [expr 1 + $yn - $yOff]

    if {![strcasecmp $surfDir None] || ![file isdir $surfDir]} \
       {
        set drawResFlag 0
       }

    if {$drawResFlag} \
       {
        .c create pixmap $xOff $yOff -width $pxw -height $pyw -win 1

        defineCell -x 1 $pxw -y 1 $pyw -cell 1 -verb -win 1
       } \
    else \
       {
        .c create image $xOff $yOff -image ramaImage -anchor nw -tags image
       }

   .c create rectangle $xOff $yOff $xn $yn \
       -outline $centerColor -tags border -width 2

   .c create line      $x0   $yOff $x0 $yn \
       -fill $zeroColor -tags border

   .c create line      $xOff $y0   $xn $y0 \
       -fill $zeroColor -tags border

   drawResBG
}

#
# Draw residue-specific Rama background.

proc drawResBG {} \
{
   global talosDir surfDir surfInfo drawResFlag roiCount 
   global currentRes seqList r1 rN

   if {!$drawResFlag} return

   set inName $talosDir/rama.dat
   set r0 $currentRes

  if {[strcasecmp $r0 None] && [strcasecmp $r1 None] && [strcasecmp $rN None]} \
     {
      if {$currentRes >= $r1 && $currentRes <= $rN} \
         { 
          set resName [getResName3 [lindex $seqList [expr $r0 - $r1]]]
          set inName  $surfDir/$resName.dat
         }
     }

   if {![file exists $inName]} {set inName $talosDir/rama.dat}

   if {![file exists $inName]} \
      {
       drawRec -cell 1 -x 0% 100% -y 0% 100% -fg black
       return
      }

   if {![info exists surfInfo($inName,roi)]} \
      {
       incr roiCount
       set surfInfo($inName,roi) $roiCount 
       readROI -roi $roiCount -ndim 2 -in $inName

       set rInfo  [getROIInfo -roi $roiCount]
       set fdata  [lindex $rInfo 1]
       set thresh [getParm $fdata FDTAU NULL_DIM]

       if {$thresh <= 0.0} {set thresh 3.0}

       set surfInfo($inName,hi) $thresh
      } 
   
   drawROI -roi $surfInfo($inName,roi) -hi $surfInfo($inName,hi) \
           -pSat1 0 -pSat2 0 -pVal1 0.5 -pVal2 0.2 -fg None \
           -cell 1 -pix -bg black -plev 10 -nlev 10
}

#
# Main application:
#  Initialize values.
#  Extract command line arguments.
#  Read the original shift table of the protein being analyzed.
#  Scan the prediction directory for any predictions.
#  Initialize the graphical interface.
#  Initialize auto-scan for predictions in progress.
#  Show the first existing prediction, if any.
 
ramaInit
ramaArgs
ramaReadSummary
ramaReadRandCoil
ramaReadShifts
ramaReadRef
ramaReadTriplets
ramaScan
ramaFrames
ramaAuto
ramaPosition

catch ramaRasMol

ramaSet $currentRes
