#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# TALOS: Torsion Angle Likeness Optimized by Shifts
# (C) Shen, Cornilescu, Delaglio, and Bax 1997-2010
#     NIH Lab of Chemical Physics NIDDK
#
# talos.tcl: finds chemical shift homology matches in database.
# vina.tcl:  classifies and summarizes predictions.
# rama.tcl:  interactive display and adjustment of predictions.

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set talosVersion 2010.160.15.01 

#
# Initialize variables:

proc ramaInit {} \
{
   global ramaTab shiftTab sumTab rcTab adjTab prevTab nextTab refTab talosTab
   global inName rcName adjName prevName nextName
   global phiAvg psiAvg phiDev psiDev

   global sumName refName dbName tabDir predDir talosDir surfDir
   global seqList npList currentRes r1 rN matchCount roiCount drawResFlag
   global xOff yOff xExtra yExtra autoFlag rasmolFlag currentStat
   global shiftLimits

   global clipFlag clipName clipTab

   global centerColor zeroColor defColor1 defColor2 activeColor1 activeColor2 
   global goodColor1 goodColor2 badColor1 badColor2 warnColor1 warnColor2
   global colCount firstRes lastRes avgFunc devFunc sdHeight sdWidth sdFlag
   global refColor color1 color2 cellX cellY refPhi refPsi refInfo

   global env

   setFonts

   if {[info exists env(TALOS_DIR)]} \
      {
       set talosDir $env(TALOS_DIR)
      } \
   else \
      {
       set talosDir .
      }

   if {[info exists env(SURF_DIR)]} \
      {
       set surfDir $env(SURF_DIR)
      } \
   else \
      {
       set surfDir None
      }

#                   HN   N    CO  CA  CB  HA
   set shiftLimits "4.0 12.0 6.0 6.0 6.0 2.0"

   set tabDir      tab
   set inName      None
   set refName     None 
   set clipName    sslimit.tab
   set sumName     pred.tab
   set dbName      talos.tab
   set currentStat None
   set predDir     pred 

   set rcName   None
   set adjName  None
   set prevName None
   set nextName None

   set autoFlag    0
   set rasmolFlag  0
   set clipFlag    0
   set shiftTab    0
   set talosTab    0
   set clipTab     0
   set ramaTab     0
   set rcTab       0
   set adjTab      0
   set prevTab     0
   set nextTab     0
   set sumTab      0
   set refTab      0
   set sdFlag      0

   set drawResFlag 1
   set roiCount    0

   set avgFunc     getAvgAngle
   set devFunc     getStdDevAngle

   set firstRes    First 
   set lastRes     Last 
   set r1          None 
   set rN          None 
   set currentRes  None 

   set refInfo     "? ? ?"
   set refPhi      0.0
   set refPsi      0.0

   set phiAvg      0.0
   set psiAvg      0.0
   set phiDev      0.0
   set psiDev      0.0

   set matchCount  10
   set colCount    10
   set seqList     ""
   set npList      ""

   set sdWidth  300
   set sdHeight  14 

   set xOff    24
   set yOff    24
   set xExtra  80 
   set yExtra  24

   set color1       white
   set color2       black
   set defColor1    gray85
   set defColor2    gray90
   set activeColor1 gray50
   set activeColor2 gray40
   set goodColor1   green3
   set goodColor2   green2
   set warnColor1   yellow2
   set warnColor2   yellow1
   set badColor1    firebrick2
   set badColor2    firebrick1
   set centerColor  gold3
   set zeroColor    dimGray
   set refColor     blue

   set wide         1024
   set hi           900

   catch {set wide [winfo screenwidth  .]}
   catch {set hi   [winfo screenheight .]}

   if {$hi > 1000} \
      {
       set cellX       10 
       set cellY       18
      } \
   else \
      {
       set cellX       8
       set cellY       16
      }

   initResNames
}

#
# Extract command line arguments.

proc ramaArgs {} \
{
 global inName sumName refName dbName rcName adjName prevName nextName
 global tabDir predDir talosDir surfDir 

 global matchCount autoFlag rasmolFlag firstRes lastRes drawResFlag
 global ARGC ARGV talosVersion colCount sdFlag

 puts "TALOS: Torsion Angle Likeness Optimized By Shifts."
 puts "Version $talosVersion Shen, Cornilescu, Delaglio, and Bax"
 puts ""

 if {[flagLoc $ARGV -help]} \
  {
   puts "rama.tcl: display and adjust TALOS prediction results (OLD VERSION)."
   puts ""
   puts "NOTICE: this help text is for the old version of RAMA;"
   puts "        the TALOS system has been replaced with an improved"
   puts "        version called TALOS+ (TALOS plus)."
   puts ""
   puts "Old Version Arguments:"
   puts " -old                         Use the old version of RAMA." 
   puts "Directory Settings:"
   puts " -talosDir mDir   \[.\]         TALOS Main Directory: TALOS_DIR."
   puts " -predDir  pDir   \[pred\]      Directory of existing predictions."
   puts " -tabDir   tDir   \[tab\]       Subdirectory of TALOS master tables."
   puts "Other Options:"
   puts " -in      inName  \[None\]         Input shifts for predictions."
   puts " -sum     sumName \[pred.tab\]     Prediction summary input/output."
   puts " -ref     refName \[None\]         PDB Reference (for testing)."
   puts " -db      dbName  \[talos.tab\]    TALOS triplet database file."
   puts " -rand    rcName  \[randcoil.tab\] Random coil shifts table."
   puts " -count   mc      \[10\]           Max match count per triplet."
   puts " -cols    cCount  \[10\]           Column Count for Sequence Display."
   puts " -first   resID1  \[First\]        First residue to display."
   puts " -last    resIDN  \[Last\]         Last residue to display."
   puts " -sd                             Display shift distributions."
   puts " -ras                            Display PDB Reference using RasMol."
   puts " -auto                           Display predictions in progress."
   puts "Examples:"
   puts "Typical inspection:"
   puts "  rama.tcl -in ubiq.tab"
   puts "Inspection when a PDB structure is known:"
   puts " rama.tcl -in ubiq.tab -ref ubiq.pdb"
   puts "Inspection when a PDB structure is known, including Molecular Viewer:"
   puts " rama.tcl -in ubiq.tab -ref ubiq.pdb -ras"
   puts "Display predictions in progress while talos.tcl is running:"
   puts " rama.tcl -in ubiq.tab -auto"
   puts "See also:"
   puts " talos.tcl                 Creates predictions by shift homology."
   puts " vina.tcl                  Summarize predictions in a single table."
   puts "Notes:"
   puts " Option -ras requires that RasMol2 is installed and available"
   puts " via the command 'rasmol', and that no other instances of RasMol"
   puts " are using the screen.  " 
   puts "RasMol Reference:"
   puts " Roger Sayle and E. James Milner-White."
   puts " RasMol: Biomolecular graphics for all"
   puts " Trends in Biochemical Sciences (TIBS),"
   puts " September 1995, Vol. 20, No. 9, p. 374."
   puts " http://www.umass.edu/microbio/rasmol"
   exit 0
  }

   getArgD $ARGV -talosDir talosDir

   set tabDir $talosDir/$tabDir

   getArgD $ARGV -predDir  predDir
   getArgD $ARGV -surfDir  surfDir
   getArgD $ARGV -tabDir   tabDir

   set dbName   $tabDir/$dbName

   set rcName   $tabDir/randcoil.tab
   set adjName  $tabDir/rcadj.tab
   set prevName $tabDir/rcprev.tab
   set nextName $tabDir/rcnext.tab

   getArgD $ARGV -rand rcName
   getArgD $ARGV -adj  adjName
   getArgD $ARGV -prev prevName
   getArgD $ARGV -next nextName

   getArgD $ARGV -count    matchCount
   getArgD $ARGV -cols     colCount
   getArgD $ARGV -in       inName
   getArgD $ARGV -db       dbName
   getArgD $ARGV -sum      sumName
   getArgD $ARGV -out      sumName
   getArgD $ARGV -ref      refName
   getArgD $ARGV -first    firstRes
   getArgD $ARGV -last     lastRes

   if {[flagLoc $ARGV -sd]}     {set sdFlag 1}
   if {[flagLoc $ARGV -nosd]}   {set sdFlag 0}

   if {[flagLoc $ARGV -drawRes]}   {set drawResFlag 1}
   if {[flagLoc $ARGV -nodrawRes]} {set drawResFlag 0}

   if {[flagLoc $ARGV -auto]}   {set autoFlag 1}
   if {[flagLoc $ARGV -noauto]} {set autoFlag 0}

   if {[flagLoc $ARGV -ras]}    {set rasmolFlag 1}
   if {[flagLoc $ARGV -noras]}  {set rasmolFlag 0}
}

#
# Read or create the summary table.

proc ramaReadSummary {} \
{
   global inName sumName predDir sumTab

   gdbInit
   gdbCreate dBase

if {![file exists $sumName]} \
   {
    catch \
     {exec vinaOrig.tcl -in $inName -sum $sumName -predDir $predDir -AUTO}
   }

   set sumTab [gdbRead table -in $sumName -name "TALOS Prediction Summary"]    
}

#
# Initialize table of random coil reference shifts.

proc ramaReadRandCoil {} \
{
   global sdFlag

   if {!$sdFlag} return

   initRandCoil
}

#
# Read the triplet table.

proc ramaReadTriplets {} \
{
   global dbName talosTab sdFlag

   if {!$sdFlag} return

   set talosTab [gdbRead table -in $dbName -name "TALOS Database"]
}

#
# Read the reference PDB table for displaying tests on known structures.

proc ramaReadRef {} \
{
    global refName refTab

    if {"$refName" != "None"} \
       {
        set refTab [gdbRead table -pdb -in $refName -name "Reference PDB"]
       }
}

#
# Read the input shift table used previously by TALOS.

proc ramaReadShifts {} \
{
   global shiftTab seqList npList statusArray firstRes lastRes r1 rN 
   global currentRes inName predDir

   if {![strcasecmp $inName None]} return
 
   set shiftTab [gdbRead table -in $inName]
   set seqList  [getSeqList $shiftTab r1 rN]
   set npList   "?"

   if {![strcasecmp $firstRes First]} {set firstRes $r1}
   if {![strcasecmp $lastRes  Last]}  {set lastRes  $rN}

   if {$firstRes < $r1} {set firstRes $r1}
   if {$lastRes  < $r1} {set lastRes  $r1}

   if {$firstRes > $rN} {set firstRes $rN}
   if {$lastRes  > $rN} {set lastRes  $rN}

   if {$firstRes > $lastRes} \
      {
       set itemp    $firstRes
       set firstRes $lastRes
       set $lastRes $itemp
      }

   foreach res $seqList {lappend npList $res}

   lappend npList "?"

   for {set i $r1} {$i <= $rN} {incr i} \
      {
       set fName  [format $predDir/res%03d.tab $i]
       set entry  [ramaGetEntry $i]
       set stat   [gdbGet $entry CLASS]

       if {[file exists $fName]} \
          {
           set statusArray($i) "$stat [file mtime $fName]"
          } \
       else \
          {
           set statusArray($i) "None 0"
           gdbSet $entry CLASS None 
          }
      }

   set currentRes [expr $firstRes + 1] 
}

#
# Configure the given residue button according to its status.

proc ramaButton { b resID } \
{
   global centerColor zeroColor defColor1 defColor2 activeColor1 activeColor2
   global goodColor1 goodColor2 badColor1 badColor2 warnColor1 warnColor2
   global color1 color2

   global statusArray currentRes 

   if {![info exists statusArray($resID)]} \
      {
       set entry [ramaGetEntry $resID]
       set statusArray($resID) "None 0"
       set stat None 

       gdbSet $entry CLASS None 
      } \
   else \
      {
       set stat [lindex $statusArray($resID) 0]
      }

   switch $stat \
      {
       None    {
                set s   disabled
                set bg  $defColor1
                set abg $defColor2
                set afg $color1
                set fg  $color2
               }

       Good    {
                set s   normal
                set bg  $goodColor1
                set abg $goodColor2
                set afg $bg
                set fg  $color2
               } 

       Bad     {
                set s   normal
                set bg  $badColor1
                set abg $badColor2
                set afg $bg
                set fg  $color2
               }

       Warn    {
                set s   normal
                set bg  $warnColor1
                set abg $warnColor2
                set afg $bg
                set fg  $color2
               }

       default {
                set s   normal
                set bg  $defColor1
                set abg $defColor2
                set afg $color1
                set fg  $color2
               } 
      }

   if {$resID == $currentRes} \
      {
       set fg  $afg
       set bg  $activeColor1
       set abg $activeColor2
      }

   $b configure -fg $fg -bg $bg -activebackground $abg -state $s
}

#
# Update the grid of residue choices.

proc ramaUpdate {} \
{
   global seqList r1 rN firstRes lastRes colCount axisfont

   if {![strcasecmp $r1 None]} return
   if {![strcasecmp $rN None]} return

   set row      1
   set col      0
   set newCount 0

   for {set i $firstRes} {$i <= $lastRes} {incr i} \
      {
       if {$i < $r1 || $i > $rN} \
          {
           set resName ?
          } \
       else \
          {
           set resName [lindex $seqList [expr $i - $r1]] 
          }

       set  rowName .t.row$row
       incr col

       set packFlag 0

       if {![winfo exists $rowName]} \
          {
           set packFlag 1
           frame $rowName
           incr newCount
          }

       set bName $rowName.col$col 

       if {![winfo exists $bName]} \
          {
           button $bName -text $resName$i -width 4 -padx 2 -pady 1 \
                  -command "ramaSet $i" -font $axisfont 

           pack $bName -in $rowName -side left
          }

       if {$packFlag} {pack $rowName -in .t -side top -anchor w}

       ramaButton $bName $i

       if {$col == $colCount} \
          {
           set  col 0
           incr row
          }
      }

   if {[info exists env(BINVAL)]} \
      {
       set binVal $env(BINVAL)
      } \
   else \
      {
       set binVal other
      }

   if {$newCount} \
      {
       if {![strcasecmp $binVal winxp]} \
          {
           set bwide 29 
           set bhi   21 

           set wide [expr $bwide*(4 + $colCount)]
           set hi   [expr $bhi*(2 + $row)]

           wm geom .t ${wide}x${hi} 
          } \
       else \
          {
           set wide [expr 2 + 4*$colCount]c
           set hi   [expr 1 + $col]c
    
           .t configure -width $wide -height $hi
          }
      }

   update
}

#
# Update display, and set for future automated update if needed.

proc ramaAuto {} \
{
   global autoFlag

   ramaScan
   ramaUpdate

   if {$autoFlag} {after 30000 ramaAuto}
}

#
# Scan the contents of the prediction directory; updates ranges and currentRes.

proc ramaScan {} \
{
   global predDir firstRes lastRes r1 rN currentRes statusArray

   set fList [lsort -dictionary [glob -nocomplain $predDir/res*.tab]]
   set n     0

   foreach fName $fList \
      {
       set  rName  [file rootname [file tail $fName]]
       scan $rName "res%d" resID

       if {!$n} \
          {
           set minRes $resID
           set maxRes $resID
          }

       if {$resID < $minRes} {set minRes $resID}
       if {$resID > $maxRes} {set maxRes $resID}

       if {[info exists statusArray($resID)]} \
          {
           set tPrev [lindex $statusArray($resID) 1] 
          } \
       else \
          {
           set tPrev 0
          }

       set t [file mtime $fName]

       if {$t > $tPrev} \
          {
           set statusArray($resID) "New $t"
          }

       incr n
      }

   if {![strcasecmp $r1 None]} {set r1 $minRes}
   if {![strcasecmp $rN None]} {set rN $maxRes}

   if {![strcasecmp $firstRes First]} {set firstRes $r1}
   if {![strcasecmp $lastRes  Last]}  {set lastRes  $rN}

   if {$firstRes < $r1} {set firstRes $r1}
   if {$lastRes  < $r1} {set lastRes  $r1}

   if {$firstRes > $rN} {set firstRes $rN}
   if {$lastRes  > $rN} {set lastRes  $rN}

   if {$firstRes > $lastRes} \
      {
       set itemp    $firstRes
       set firstRes $lastRes
       set $lastRes $itemp
      }

   if {![strcasecmp $currentRes None]} {set currentRes $firstRes}
}

#
# Read the predicted angle table.

proc ramaReadAngles {} \
{
   global currentRes predDir ramaTab

   if {$ramaTab} \
      {
       gdbDestroy $ramaTab
       set ramaTab 0
      }

   set fName [format $predDir/res%03d.tab $currentRes]

   if {[file exists $fName]} \
      {
       set ramaTab [gdbRead table -in $fName -name $fName]
      }
}

proc ramaWriteAngles {} \
{
   global currentRes predDir ramaTab

   if {!$ramaTab} {return 0}

   set fName [format $predDir/res%03d.tab $currentRes]
   gdbWrite  $ramaTab $fName
}

#
# Mark current residue prediction as good, bad, etc.

proc ramaMark { type } \
{
   global statusArray currentRes currentStat sumTab

   if {[info exists statusArray($currentRes)]} \
      {
       set t [lindex $statusArray($currentRes) 1]
      } \
   else \
      {
       set t 0
      }

   set statusArray($currentRes) "$type $t"
   set currentStat $type

   set entry [gdbSelect -from $sumTab -cond "RESID == $currentRes"]

   if {[llength $entry] == 1} \
      {
       gdbSet $entry CLASS $type
      }

   update

   ramaUpdate 
   ramaShowMol
}

#
# Clear current residue display.

proc ramaClear {} \
{
   global matchCount selArray boxArray 

   .c delete label box outline center
   .c itemconfigure info -text "" 
   .s.summary configure  -text ""

   set selArray(Ref) 0
   set boxArray(Ref) 0

   for {set i 0} {$i < $matchCount} {incr i} \
      {
       set selArray($i) 0
       set boxArray($i) 0

       .s.b$i configure -command "ramaSelect $i 0" -text "Prediction $i"
      }
}

#
# Show the current angles.

proc ramaShowAngles {} \
{
   global currentRes currentStat seqList selArray statusArray r1 rN 
   global centerColor zeroColor activeColor goodColor1 badColor1
   global sumTab ramaTab avgFunc devFunc talosVersion inName
   global xOff yOff color1 color2 cellX cellY boxArray stdfont
   global phiAvg psiAvg phiDev psiDev

   ramaClear
   drawResBG

#
# Extract angles and stats:

   if {!$ramaTab} return

   set currentStat [lindex $statusArray($currentRes) 0]

   set eList   [gdbGetInfo $ramaTab -parms entries]

   set var     [gdbFindByName $ramaTab INDEX]
   set idList  [gdbGetInfo $var -parms values]

   set var     [gdbFindByName $ramaTab PHI]
   set phiList [gdbGetInfo $var -parms values]

   set var     [gdbFindByName $ramaTab PSI]
   set psiList [gdbGetInfo $var -parms values]

   set var     [gdbFindByName $ramaTab DIST]
   set dList   [gdbGetInfo $var -parms values]

   set var     [gdbFindByName $ramaTab W]
   set wList   [gdbGetInfo $var -parms values]

   set var     [gdbFindByName $ramaTab SOURCE]
   set srcList [gdbGetInfo $var -parms values]

   set phiAvg [format "%.0f" [$avgFunc [lselect $phiList $wList]]]
   set psiAvg [format "%.0f" [$avgFunc [lselect $psiList $wList]]]

   set phiDev [format "%.0f" [$devFunc [lselect $phiList $wList]]]
   set psiDev [format "%.0f" [$devFunc [lselect $psiList $wList]]]

   set dAvg   [format "%.2f" [getAvg   [lselect $dList   $wList]]]

   set n [llength $idList]

#
# Label center and extent:

   set x0 [expr $xOff + $phiAvg + 180]
   set y0 [expr $yOff + 180 - $psiAvg]
   set xn [expr $xOff + 360]
   set yn [expr $yOff + 360]

   .c create line $xOff $y0 $xn $y0 -fill $centerColor -tags center
   .c create line $x0 $yOff $x0 $yn -fill $centerColor -tags center

   set x1 [expr $x0 - $phiDev]
   set xn [expr $x0 + $phiDev]
   set y1 [expr $y0 - $psiDev]
   set yn [expr $y0 + $psiDev]

   .c create oval $x1 $y1 $xn $yn -outline $centerColor -tags center

   set xn [expr $xOff + 360 + $cellX]
   set yn [expr $yOff + 360 + $cellY]

   .c create text $x0 $yn -anchor c -text "$phiAvg +/- $phiDev\nPhi" \
      -font $stdfont -tags center -justify c -fill $color2

   .c create text $xn $y0 -anchor w -text "$psiAvg +/- $psiDev\nPsi" \
      -font $stdfont -tags center -justify c -fill $color2

#
# Individual labels:
 
   for {set i 0} {$i < $n} {incr i} \
      {
       set phi [lindex $phiList $i]
       set psi [lindex $psiList $i]
       set src [lindex $srcList $i]
       set id  [lindex $idList  $i]
       set d   [lindex $dList   $i]
       set w   [lindex $wList   $i]
       set e   [lindex $eList   $i]
     
       set x0 [expr $xOff + $phi + 180]
       set y0 [expr $yOff + 180 - $psi]

       set x1 [expr $x0 - 3]
       set y1 [expr $y0 - 3]
       set xn [expr $x0 + 3]
       set yn [expr $y0 + 3]

       if {$w == 0.0} \
          {
           set color $badColor1
           set selArray($i) 0
          } \
       else \
          {
           set color $goodColor1
           set selArray($i) 1
          }

       set str [getResData $e]

       set boxArray($i) \
        [.c create rectangle $x1 $y1 $xn $yn \
           -outline $color1 -fill $color -tags "box box$i"]

       .c bind $boxArray($i) <Any-Enter>       "+ ramaEnterBox $i"
       .c bind $boxArray($i) <Any-Leave>       "+ ramaLeaveBox $i"

       .c bind $boxArray($i) <ButtonRelease-1> "+ ramaToggle $i $e"

       set fmt "Phi: %4.0f Psi: %4.0f D: %5.2f %s %s"

       .s.b$i configure -command "ramaSelect $i $e" -text \
          [format $fmt $phi $psi $d $str $src]
      }

   ramaRef

#
# Update the Prediction Select Title:

   set resID1 [expr $currentRes - 1]
   set resID2 [expr $resID1 + 1]
   set resID3 [expr $resID2 + 1]

   set rList ""

   set str "Triplet"

   foreach i "$resID1 $resID2 $resID3" \
      {
       if {$i < $r1 || $i > $rN} \
          {
           lappend rList ?
          } \
       else \
          {
           lappend rList [lindex $seqList [expr $i - $r1]]
          }
      }

   set name1 [lindex $rList 0]
   set name2 [lindex $rList 1]
   set name3 [lindex $rList 2]

   set    str "Residue $name2$resID2, Triplet"
   append str " $name1$resID1 $name2$resID2 $name3$resID3"

   wm title . "TALOS $talosVersion   Residue $name2$resID2 $inName"
   wm title .s $str

   set str "               "
   set src "Average"

   set fmt "Phi: %4.0f Psi: %4.0f D: %5.2f %s %s"

   .s.summary configure -text [format $fmt $phiAvg $psiAvg $dAvg $str $src]

   update

#
# Update the summary table:

   set entry [ramaGetEntry $resID2]
   set n     [llength [lselect $dList $wList]]

   gdbSet $entry PHI     $phiAvg
   gdbSet $entry PSI     $psiAvg
   gdbSet $entry DPHI    $phiDev
   gdbSet $entry DPSI    $psiDev
   gdbSet $entry DIST    $dAvg

   gdbSet $entry COUNT   $n
   gdbSet $entry CLASS   $currentStat
}

#
# Show shift distributions.

proc ramaShowShifts {} \
{
   global shiftLimits sdWidth sdHeight selArray sdArray sdFlag 
   global rcArray adjArray prevArray nextArray r1 rN

   global goodColor1 goodColor2 badColor1 badColor2 warnColor1 warnColor2
   global currentRes shiftTab ramaTab talosTab seqList npList matchCount
   global refColor color1 color2

#
# Erase the current distribution:

   if {!$sdFlag}  return
   if {!$ramaTab} return

   set sdArray(Ref)  ""

   for {set i 0} {$i < 18} {incr i} \
      {
       .d.c$i delete box
      }

   for {set i 0} {$i < $matchCount} {incr i} \
      {
       set sdArray($i)  ""
      }

   update

#
# Show the distribution of shifts from the database:

   set entryI [gdbFirst $ramaTab]
   set ii     0

   while {$entryI} \
      {
       set src [gdbGet $entryI SOURCE]
       set res [gdbGet $entryI R1]
       
       scan $res "%c%d" resName resID

       if {$selArray($ii) == 0.0} \
          {
           set color $badColor1
          } \
       else \
          {
           set color $goodColor1
          }

       set cond   "RESID_R1 == $resID && !strcasecmp( '$src', SOURCE )"
       set entryJ [gdbSelect -from $talosTab -cond $cond]

       if {[llength $entryJ] != 1} \
          {
           incr ii
           set  entryI [gdbNext $entryI]
           puts "Warning: no match, $src $resID ..."
           continue
          }

       set k    0

       foreach i {_R1 _R2 _R3} \
          {
           set kk   0

           foreach j {HN N C CA CB HA} \
              {
               set maxShift [lindex $shiftLimits $kk]
               set shift    [gdbGet $entryJ $j$i]
               set secShift [format "%.3f" $shift]

               if {$shift > 999.0} \
                  {
                   incr k
                   incr kk
                   continue
                  }

               if {$shift >  $maxShift} {set shift  $maxShift}
               if {$shift < -$maxShift} {set shift -$maxShift}

               set x0 [expr $sdWidth*($shift + $maxShift)/(2.0*$maxShift)]
               set x1 [expr $x0 - 2]
               set xn [expr $x0 + 2]

               set val \
                 [.d.c$k create rectangle $x1 2 $xn $sdHeight \
                    -fill $color -outline $color2 -tags box -width 1]

               lappend sdArray($ii) "$k $val $secShift"

               .d.c$k bind $val <Any-Enter>       "+ ramaEnterBox $ii"
               .d.c$k bind $val <Any-Leave>       "+ ramaLeaveBox $ii"
               .d.c$k bind $val <ButtonRelease-1> "+ ramaToggle   $ii $entryI"

               incr kk
               incr k
              }
          }

       incr ii
       set  entryI [gdbNext $entryI]
      }

#
# Show the shifts from the unknown:

   set k     0
   set color $refColor

   for {set i [expr $currentRes-1]} {$i <= [expr $currentRes+1]} {incr i} \
       {
        if {!$shiftTab} continue
        if {$i < $r1}   continue

        set resName  [lindex $seqList [expr $i - $r1]]
        set prevName [lindex $npList  [expr $i - $r1]]
        set nextName [lindex $npList  [expr $i - $r1 + 1]]
        set kk   0

        foreach j {HN N C CA CB HA} \
           {
            set maxShift [lindex $shiftLimits $kk]
            set shift    9999.0

            if {![strcasecmp $resName G] && ![strcasecmp $j HA]} \
               {
                set cond  "RESID == $i && !strcasecmp( ATOMNAME, 'HA2' )"
                set entry [gdbSelect -from $shiftTab -cond $cond]

                if {[llength $entry] == 1} \
                   {
                    set shift [gdbGet $entry SHIFT]
                   }

                set cond  "RESID == $i && !strcasecmp( ATOMNAME, 'HA3' )"
                set entry [gdbSelect -from $shiftTab -cond $cond]

                if {[llength $entry] == 1} \
                   {
                    set shift \
                       [expr 0.5*($shift + [gdbGet $entry SHIFT])]
                   }
               } \
            else \
               {
                set cond  "RESID == $i && !strcasecmp( ATOMNAME, \"$j\" )"
                set entry [gdbSelect -from $shiftTab -cond $cond]

                if {[llength $entry] == 1} \
                   {
                    set shift [gdbGet $entry SHIFT]
                   }
               }

            if {$shift > 999.0} \
               {
                incr k
                incr kk
                continue
               }

            set rcVal [expr $rcArray($resName,$j) + $adjArray($resName,$j)]
            set rcVal [expr $rcVal + $prevArray($prevName,$j)]
            set rcVal [expr $rcVal + $nextArray($nextName,$j)]

            set shift    [expr $shift - $rcVal]
            set secShift [format "%.3f" $shift]

            if {$shift >  $maxShift} {set shift  $maxShift}
            if {$shift < -$maxShift} {set shift -$maxShift}

            set x0 [expr $sdWidth*($shift + $maxShift)/(2.0*$maxShift)]
            set x1 [expr $x0 - 2]
            set xn [expr $x0 + 2]

            set val \
              [.d.c$k create rectangle $x1 2 $xn $sdHeight \
                 -fill $color -outline $color2 -tags box -width 1]

            lappend sdArray(Ref) "$k $val $secShift"

            .d.c$k bind $val <Any-Enter>  "+ ramaEnterBox Ref"
            .d.c$k bind $val <Any-Leave>  "+ ramaLeaveBox Ref"

            incr kk
            incr k
           }
       }
}

#
# Show the reference info.

proc ramaRef {} \
{
   global xOff yOff refTab color1 refColor currentRes
   global refPhi refPsi refInfo boxArray

   if {!$refTab} {return}

   set aList   [getAngles [expr $currentRes - 1] $refTab]
   set phi     [lindex $aList 0]
   set psi     [lindex $aList 1]
   set refInfo [lindex $aList 2]
   set refPhi  $phi
   set refPsi  $psi

   if {$phi > 999.0 || $psi > 999.0} {return}

   set x0 [expr $xOff + $phi + 180]
   set y0 [expr $yOff + 180 - $psi]

   set x1 [expr $x0 - 3]
   set y1 [expr $y0 - 3]
   set xn [expr $x0 + 3]
   set yn [expr $y0 + 3]

   set boxArray(Ref) \
     [.c create rectangle $x1 $y1 $xn $yn \
           -outline $color1 -fill $refColor -tags "box boxRef"]

   .c bind $boxArray(Ref) <Any-Enter>       "+ ramaEnterBox Ref"
   .c bind $boxArray(Ref) <Any-Leave>       "+ ramaLeaveBox Ref"

}

#
# Find or add an entry in the summary table.

proc ramaGetEntry { resID } \
{
   global seqList r1 rN sumTab

   set entry [gdbSelect -from $sumTab -cond "RESID == $resID"]

   if {[llength $entry] == 0} \
      {
       set entry [gdbCreate entry -parent $sumTab]

       if {$resID < $r1 || $resID > $rN} \
          {
           set resName ?
          } \
       else \
          {
           set resName [lindex $seqList [expr $resID - $r1]]
          }

       gdbSet $entry RESID   $resID
       gdbSet $entry RESNAME $resName

       gdbSet $entry PHI     9999.0
       gdbSet $entry PSI     9999.0
       gdbSet $entry DPHI       0.0
       gdbSet $entry DPSI       0.0
       gdbSet $entry DIST       0.0

       gdbSet $entry COUNT        0
       gdbSet $entry CLASS     None
      }

   return $entry
}

#
# Get the triplet data from the given entry.

proc getResData { entry } \
{
   set str ""

   append str [format " %-4s" [gdbGet $entry R1]]
   append str [format " %-4s" [gdbGet $entry R2]]
   append str [format " %-4s" [gdbGet $entry R3]]

   return $str
}

#
# Set the current residue number on display.

proc ramaNext {} \
{
   global currentRes r1 rN

   incr currentRes

   if {$currentRes > $rN - 1} {set currentRes [expr $r1 + 1]}

   ramaSet $currentRes
}

proc ramaPrev {} \
{
   global currentRes r1 rN 

   incr currentRes -1

   if {$currentRes < [expr $r1 + 1]} {set currentRes [expr $rN - 1]}

   ramaSet $currentRes
}

proc ramaSet { thisRes } \
{
   global currentRes r1 rN 

   if {$thisRes >= $r1 && $thisRes <= $rN} \
      {
       set currentRes $thisRes

       ramaReadAngles
       ramaUpdate
       ramaShowAngles
       ramaShowShifts
       ramaShowMol
      }
}

#
# Save all current results.

proc ramaSave {} \
{
   global predDir sumName sumTab

   if {!$sumTab} return

   set types \
      {
       { {All Files}  *      }
       { {tab}        {.tab} }
       { {ser}        {ser*} }
       { {fid}        {fid*} }
       { {dat}        {.dat .DAT} }
       { {seq}        {.seq} }
       { {Text}       {.txt} }
       { {TCL}        {.tcl} }
       { {C-Shell}    {.com .csh} }
       { {K-Shell}    {.ksh} }
       { {Shell}      {.sh}  }
      }

   set thisName \
     [tk_getSaveFile -title "Prediction Summary Output" \
       -filetypes $types -initialfile $sumName]

   if {"$thisName" != "None" && "$thisName" != ""} \
      {
       gdbWrite $sumTab $thisName
       puts "Wrote prediction summary table $thisName"
      }
}

#
# Save results and exit.

proc ramaQuit {} \
{
   global rasmolFlag

   ramaSave

   if {$rasmolFlag} {catch "send rasmol quit"}

   exit
}

#
# Create the graphics interface.

proc ramaFrames {} \
{
   global talosVersion cellX cellY xOff yOff xExtra yExtra matchCount r1 rN
   global sdArray selArray boxArray currentStat sdHeight sdWidth sdFlag
   global color1 color2 goodColor1 badColor1 warnColor1 defColor1 stdfont
   global axisfont talosDir inName
   
   toplevel .t
   toplevel .s

#
# Initialize:

   set selArray(Ref) 0
   set boxArray(Ref) 0
   set sdArray(Ref)  ""

   for {set i 0} {$i < $matchCount} {incr i} \
      {
       set selArray($i) 0
       set boxArray($i) 0
       set sdArray($i)  ""
      }

#
# Rama Frame:

   frame    .bottom1
   frame    .bottom2

   set xw [expr 1 + 360 + 2*$xOff + $xExtra]
   set yw [expr 1 + 360 + 2*$yOff + $yExtra]

   image create photo ramaImage -file $talosDir/rama.gif -palette 16

   canvas .c -width $xw -height $yw

   .c create text $cellX $cellY -font $stdfont -text "" -anchor sw -tags info 

   checkbutton .bGood -text Good         -command "ramaMark Good" -width 12 \
      -indicatoron 0 -variable currentStat -onvalue Good -offvalue None     \
      -selectcolor $goodColor1

   checkbutton .bWarn -text Ambiguous    -command "ramaMark Warn" -width 12 \
      -indicatoron 0 -variable currentStat -onvalue Warn -offvalue None     \
      -selectcolor $warnColor1

   checkbutton .bBad  -text Bad          -command "ramaMark Bad"  -width 12 \
      -indicatoron 0 -variable currentStat -onvalue Bad -offvalue None      \
      -selectcolor $badColor1

   checkbutton .bNew  -text Unclassified -command "ramaMark New"  -width 12 \
      -indicatoron 0 -variable currentStat -onvalue New -offvalue None      \
      -selectcolor $defColor1

   button .bNext  -text Next    -command ramaNext         -width 7
   button .bPrev  -text Prev    -command ramaPrev         -width 7
   button .bShow  -text Redraw  -command ramaShowAngles   -width 7
   button .bClear -text Clear   -command ramaClear        -width 7
   button .bSave  -text Save    -command ramaSave         -width 7
   button .bQuit  -text Quit    -command ramaQuit         -width 7

   pack .bGood .bWarn .bBad .bNew \
      -in .bottom1 -side left -expand 1 -fill x

   pack .bPrev .bNext .bShow .bClear .bSave .bQuit \
      -in .bottom2 -side left -expand 1 -fill x

   pack .c -in .
   pack .bottom1 .bottom2 -in . -side top -expand 1 -fill x

   wm title . "TALOS $talosVersion"

#
# Residue Select Frame:

   if {[strcasecmp $inName None]} \
      {
       wm title .t "TALOS $inName Residues $r1 to $rN"
      } \
   else \
      {
       wm title .t "TALOS Residue Select"
      }

#
# Prediction Select Frame:

   wm title .s "TALOS Prediction Select"

   for {set i 0} {$i < $matchCount} {incr i} \
      {
       checkbutton .s.b$i -command "ramaSelect $i 0" -variable selArray($i)  \
          -selectcolor $goodColor1 -text "Prediction $i" -width 60 -anchor w \
          -font $axisfont

       bind .s.b$i <Any-Enter> "+ ramaEnterButton $i" 
       bind .s.b$i <Any-Leave> "+ ramaLeaveButton $i" 

       pack .s.b$i -side top
      }

   set str "               "
   set src "Average"

   checkbutton .s.summary -width 60 -anchor w -state disabled \
      -disabledforeground black -font $axisfont

   set fmt "Phi: %4.0f Psi: %4.0f D: %5.2f %s %s"

   .s.summary configure -text [format $fmt 0.0 0.0 0.0 $str $src]

   pack .s.summary -side top -expand 1 -fill x

#
# Shift Distribution Frame:
 
   if {$sdFlag} \
      {
       toplevel .d 

       set k    0

       foreach i {"i-1" "i" "i+1"} \
          {
           foreach j {HN N CO CA CB HA} \
              {
               frame  .d.f$k
               canvas .d.c$k -width $sdWidth -height $sdHeight
               label  .d.l$k -text [format "%s(%s)" $j $i] -anchor e -width 7
               label  .d.s$k -anchor e -width 7

               pack   .d.l$k .d.c$k .d.s$k -in .d.f$k -side left -padx 2
               pack   .d.f$k -side top -pady 1

               set xm [expr $sdWidth/2]

              .d.c$k create line $xm 1 $xm $sdHeight \
                 -fill $color2 -tags bg 

              .d.c$k create line  1 $sdHeight $sdWidth $sdHeight \
                 -fill $color2 -tags bg 

               incr k
              }
          }
   
       wm title .d "TALOS Secondary Shift Distributions"
       update
      }

   ramaBG
   ramaUpdate
}

#
# Handlers for entering and leaving buttons or boxes:

proc ramaEnterButton { i } \
{
    global goodColor1 badColor1 color1
    global selArray sdArray sdFlag

    if {$selArray($i) == 0.0} \
       {
        set color $badColor1
       } \
    else \
       {
        set color $goodColor1
       }

   .c itemconfigure info  -text "[string trim [.s.b$i cget -text]]"
   .c itemconfigure box$i -outline $color -fill $color1
   .c raise box$i

   if {$sdFlag} \
      {
       foreach j $sdArray($i) \
          {
           set k     [lindex $j 0]
           set val   [lindex $j 1]
           set shift [lindex $j 2]

           .d.c$k itemconfigure $val -outline $color1
           .d.c$k raise $val
           .d.s$k configure -text $shift
          }
      }
}

proc ramaLeaveButton { i } \
{
   global goodColor1 badColor1 color1 color2
   global selArray sdArray sdFlag

   if {$selArray($i) == 0.0} \
      {
       set color $badColor1
      } \
   else \
      {
       set color $goodColor1
      }

   .c itemconfigure info  -text ""
   .c itemconfigure box$i -fill $color -outline $color1

   if {$sdFlag} \
      {
       foreach j $sdArray($i) \
          {
           set k   [lindex $j 0]
           set val [lindex $j 1]
 
           .d.c$k itemconfigure $val -outline $color2
           .d.s$k configure -text ""
          }
      }
}

proc ramaEnterBox { i } \
{
   global selArray sdArray sdFlag goodColor1 badColor1 refColor color1
   global refPhi refPsi refName refInfo phiAvg psiAvg

   if {"$i" == "Ref"} \
      {
       set color $refColor

       set fmt \
        "Phi: %.0f Psi: %.0f dPhi: %.0f dPsi: %.0f Reference %s"

       set dPhi [angleDiff $refPhi $phiAvg]
       set dPsi [angleDiff $refPsi $psiAvg]

       set msg [format $fmt $refPhi $refPsi $dPhi $dPsi $refName]
      } \
   elseif {$selArray($i) == 0.0} \
      {
       set color $badColor1
       set msg   "[.s.b$i cget -text]"
       .s.b$i configure -state active
      } \
   else \
      {
       set color $goodColor1
       set msg   "[.s.b$i cget -text]"
       .s.b$i configure -state active
      }

  .c itemconfigure info  -text "[string trim $msg]"
  .c itemconfigure box$i -outline $color -fill $color1
  .c raise box$i

   if {$sdFlag} \
      {
       foreach j $sdArray($i) \
          {
           set k     [lindex $j 0]
           set val   [lindex $j 1]
           set shift [lindex $j 2]
    
           .d.c$k itemconfigure $val -outline $color1
           .d.c$k raise $val
           .d.s$k configure -text $shift
          }
      }
}

proc ramaLeaveBox { i } \
{
   global goodColor1 badColor1 refColor color1 color2
   global boxArray selArray sdArray sdFlag

   if {"$i" == "Ref"} \
      {
       set color $refColor
      } \
   elseif {$selArray($i) == 0.0} \
      {
       set color $badColor1
       .s.b$i configure -state normal 
      } \
   else \
      {
       set color $goodColor1
       .s.b$i configure -state normal 
      }

   .c itemconfigure info  -text ""
   .c itemconfigure box$i -fill $color -outline $color1

   if {$sdFlag} \
      {
       foreach j $sdArray($i) \
          {
           set k   [lindex $j 0]
           set val [lindex $j 1]

           .d.c$k itemconfigure $val -outline $color2
           .d.s$k configure -text ""
          }
      }
}

proc ramaToggle { i entry } \
{
   global selArray

   if {$selArray($i)} \
      {
       set selArray($i) 0
      } \
   else \
      {
       set selArray($i) 1
      }

   ramaSelect $i $entry
}

#
# Handle change in selection state of a given prediction.

proc ramaSelect { i entry } \
{
   global sdArray sdFlag selArray badColor1 goodColor1

   if {!$entry} return

   gdbSet $entry W $selArray($i)

   ramaShowAngles
   ramaWriteAngles

   .c raise box$i

   if {$selArray($i) == 0.0} \
      {
       set color $badColor1
      } \
   else \
      {
       set color $goodColor1
      }

   if {$sdFlag} \
      {
       foreach j $sdArray($i) \
          {
           set k     [lindex $j 0]
           set val   [lindex $j 1]
           set shift [lindex $j 2]

           .d.c$k itemconfigure $val -fill $color
           .d.c$k raise $val
           .d.s$k configure -text $shift
          }
      }
}

#
# Positioning:

proc ramaPosition {} \
{
   global xOff yOff sdFlag

#
# Rama frame:

   set x0 [expr $xOff] 
   set y0 [expr 2*$yOff] 

   wm geometry . +$x0+$y0
   update

#
# Residue Selection frame:

   incr x0 [winfo width .]
   incr x0 $xOff 
   incr y0 $yOff 

   wm geometry .t +$x0+$y0
   update

#
# Prediction Selection frame:

   set x0 [expr $xOff]
   set y0 [expr 2*$yOff]

   incr x0 $xOff 
   incr y0 [winfo height .]
   incr y0 [expr 2*$yOff] 

   wm geometry .s +$x0+$y0
   update

#
# Shift Distribution frame:

   if {$sdFlag} \
      {
       set x0 [expr $xOff]
       set y0 [expr 2*$yOff]

       incr x0 [winfo width .]
       incr x0 $xOff 
       incr x0 $xOff 

       incr y0 [winfo height .t]
       incr y0 [expr 3*$yOff] 

       wm geometry .d +$x0+$y0
       update
      }
}

#
# Molecular Viewer.

proc ramaRasMol {} \
{
   global rasmolFlag refName refTab sumTab

   if {!$rasmolFlag} return
   if {!$refTab}     return

#
# Desired Geometry 550x450-30-20

   exec rasmol &
   exec sleep 3

   send rasmol load $refName
#  send rasmol wireframe off
   send rasmol ribbons   on
   send rasmol zoom      150
   send rasmol set fontsize 16

   send rasmol select *
   send rasmol colour gray

   set entry   [gdbFirst $sumTab]

   while {$entry} \
      {
       set resID [gdbGet $entry RESID]
       set stat  [gdbGet $entry CLASS]

       switch $stat \
          {
           Good    {set c green}
           Bad     {set c red}
           Warn    {set c yellow}
           default {set c gray}
          }

       send rasmol select $resID
       send rasmol colour $c

       set entry [gdbNext $entry]
      }

   send rasmol select *
}

#
# Show current residue.

proc ramaShowMol {} \
{
   global rasmolFlag refTab currentRes

   if {!$rasmolFlag} return
   if {!$refTab}     return

   set entry    [ramaGetEntry $currentRes]
   set resName  [gdbGet $entry RESNAME]
   set stat     [gdbGet $entry CLASS]

   switch $stat \
      {
       Good    {set c green}
       Bad     {set c red}
       Warn    {set c yellow}
       default {set c gray}
      }

   send rasmol label off

   send rasmol select $currentRes
   send rasmol colour $c 

   set resType [getResName3 $resName]

   send rasmol select $resType$currentRes.C
   send rasmol label  $resName$currentRes
   send rasmol colour label magenta 
   send rasmol select *
}

#
# Show the ramachandran plot background.

proc ramaBG {} \
{
    global xOff yOff zeroColor centerColor surfDir drawResFlag 

    set x0 [expr $xOff + 180]
    set y0 [expr $yOff + 180]

    set xn [expr $xOff + 360]
    set yn [expr $yOff + 360]

    set pxw [expr 1 + $xn - $xOff]
    set pyw [expr 1 + $yn - $yOff]

    if {![strcasecmp $surfDir None] || ![file isdir $surfDir]} \
       {
        set drawResFlag 0
       }

    if {$drawResFlag} \
       {
        .c create pixmap $xOff $yOff -width $pxw -height $pyw -win 1

        defineCell -x 1 $pxw -y 1 $pyw -cell 1 -verb -win 1
       } \
    else \
       {
        .c create image $xOff $yOff -image ramaImage -anchor nw -tags image
       }

   .c create rectangle $xOff $yOff $xn $yn \
       -outline $centerColor -tags border -width 2

   .c create line      $x0   $yOff $x0 $yn \
       -fill $zeroColor -tags border

   .c create line      $xOff $y0   $xn $y0 \
       -fill $zeroColor -tags border

   drawResBG
}

#
# Draw residue-specific Rama background.

proc drawResBG {} \
{
   global talosDir surfDir surfInfo drawResFlag roiCount 
   global currentRes seqList r1 rN

   if {!$drawResFlag} return

   set inName $talosDir/rama.dat
   set r0 $currentRes

  if {[strcasecmp $r0 None] && [strcasecmp $r1 None] && [strcasecmp $rN None]} \
     {
      if {$currentRes >= $r1 && $currentRes <= $rN} \
         { 
          set resName [getResName3 [lindex $seqList [expr $r0 - $r1]]]
          set inName  $surfDir/$resName.dat
         }
     }

   if {![file exists $inName]} {set inName $talosDir/rama.dat}

   if {![file exists $inName]} \
      {
       drawRec -cell 1 -x 0% 100% -y 0% 100% -fg black
       return
      }

   if {![info exists surfInfo($inName,roi)]} \
      {
       incr roiCount
       set surfInfo($inName,roi) $roiCount 
       readROI -roi $roiCount -ndim 2 -in $inName

       set rInfo  [getROIInfo -roi $roiCount]
       set fdata  [lindex $rInfo 1]
       set thresh [getParm $fdata FDTAU NULL_DIM]

       if {$thresh <= 0.0} {set thresh 3.0}

       set surfInfo($inName,hi) $thresh
      } 
   
   drawROI -roi $surfInfo($inName,roi) -hi $surfInfo($inName,hi) \
           -pSat1 0 -pSat2 0 -pVal1 0.5 -pVal2 0.2 -fg None \
           -cell 1 -pix -bg black -plev 10 -nlev 10
}

#
# Main application:
#  Initialize values.
#  Extract command line arguments.
#  Read the original shift table of the protein being analyzed.
#  Scan the prediction directory for any predictions.
#  Initialize the graphical interface.
#  Initialize auto-scan for predictions in progress.
#  Show the first existing prediction, if any.
 
ramaInit
ramaArgs
ramaReadSummary
ramaReadRandCoil
ramaReadShifts
ramaReadRef
ramaReadTriplets
ramaScan
ramaFrames
ramaAuto
ramaPosition
ramaRasMol

ramaSet $currentRes
