#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set inList    test.ft1
set outName   None
set tabName   None
set refPPM    0.0
set dxPPM     Auto
set x1        0.2ppm
set xn       -0.2ppm
set idx       2
set skipFlag  1
set leftFlag  0
set f         0.05
set briefFlag 0
set verbose   1

set fdataX 0
set fdata  0

if {[flagLoc $ARGV -help]} \
  {
   puts "Re-Reference PPM Calibration by Max Point or Peak in Region (2022.129.16.15):"
   puts " -in   inName  \[test.ft1\]   Input Spectrum or List."
   puts " -out  outName \[None\]       Optional Output of Reference Region."
   puts " -tab  tabName \[None\]       Optional Peak Table Output."
   puts " -ref          \[0.0\]        Desired PPM of Reference Signal."
   puts " -x1   x1      \[0.2ppm\]     Start of Analysis Region."
   puts " -xn   xn      \[-0.2ppm\]    End of Analyis Region."
   puts " -idx  idx     \[2\]          +/- Width of Interpolation Region, Pts."
   puts " -noskip                    Allow Calibration of Non-H Spectra."
   puts " -verb                      Verbose Mode ON (Default)."
   puts " -noverb                    Verbose Mode OFF."
   puts " -brief                     Brief Report Mode ON."
   puts "Specifying a Specific Adjustment to Calibration:"
   puts " -xAdj  xAdj   \[Auto\]       Amount to Adjust X-Axis Calibration, PPM"
   puts "Referencing via Peak Detection:"
   puts " -f    fThresh \[0.05\]       Height Fraction for Detect Threshold." 
   puts " -right                     Use Right-Most (Upfield) Peak (Default)."
   puts " -left                      Use Left-Most (Downfield) Peak."
   puts " -max                       Use Maximum Peak."
   puts "Notes:"
   puts " 1. If a peak table name is given, peak analysis will be used."
   puts "    Otherwise, the largest point in the reference region will be used."
   puts " 2. Calibration will be skipped if X-Axis Label does not match: *H* or"
   puts "    *Proton*."
   puts " 3. The -brief report format is:"
   puts "    xAdjPPM 0.0 xAdjHz 0.0"

   exit 1
  }

getListArgD $ARGV -in inList

getArgD $ARGV -out  outName
getArgD $ARGV -tab  tabName
getArgD $ARGV -ref  refPPM 
getArgD $ARGV -xAdj dxPPM
getArgD $ARGV -x1   x1
getArgD $ARGV -xn   xn
getArgD $ARGV -idx  idx
getArgD $ARGV -f    f

if {[flagLoc $ARGV -verb]}    {set verbose   1}
if {[flagLoc $ARGV -noverb]}  {set verbose   0}

if {[flagLoc $ARGV -brief]}   {set briefFlag 1}
if {[flagLoc $ARGV -nobrief]} {set briefFlag 0}

if {$briefFlag} {set verbose 0}

if {[flagLoc $ARGV -skip]}    {set skipFlag  1}
if {[flagLoc $ARGV -noskip]}  {set skipFlag  0}

if {[flagLoc $ARGV -left]}  {set leftFlag  1}
if {[flagLoc $ARGV -right]} {set leftFlag  0}
if {[flagLoc $ARGV -left]}  {set leftFlag  1}
if {[flagLoc $ARGV -max]}   {set leftFlag -1}

#
#--------------------------------------------------------------------------

set inName  [lindex $inList 0]
set inCount [llength $inList]

set fdata     0
set fdataX    0

if {!$inCount} {exit 0}

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input Spectrum $inName"
    exit 1
   }

set error [rdFDATA $inName fdata]

if {$error} \
   {
    puts stderr "Error Reading Header from Spectrum $inName"
    exit 1
   }

set xLab     [string toupper [getParmStr $fdata NDLABEL CUR_XDIM]]
set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]

if {$skipFlag && (![strmatch $xLab *H*] && ![strmatch $xLab *PROTON*])} \
   {
    if {$verbose} {puts "Skipping Calibration for $inName with X-Axis $xLab"}
    exit 0
   }

set x1Pts [expr int( 0.5 + [spec2pnt $fdata CUR_XDIM $x1] )]
set xnPts [expr int( 0.5 + [spec2pnt $fdata CUR_XDIM $xn] )]

if {$dimCount == 2} \
   {
    readROI -ndim 1 -roi 1 -x X_AXIS $x1 $xn -in $inName
   } \
else \
   {
    readROI -ndim 1 -roi 1 -x X_AXIS $x1 $xn -y Y_AXIS 1 1 -in $inName
   }

if {[strcasecmp $outName None]} {writeROI -roi 1 -out $outName -ov}

set roiInfo [getROIInfo -roi 1]
set rdataX  [lindex $roiInfo 0]
set fdataX  [lindex $roiInfo 1]
set sInfo   [lindex $roiInfo 2]
set xSize   [lindex $sInfo 0]

set ix1 [expr $idx + 1]
set ixn [expr $xSize - $idx]

set maxVal [getPtrVal $rdataX 0]
set maxLoc $ix1
set refLoc $ix1

if {![strcasecmp $dxPPM Auto]} \
   {
    for {set ix $ix1} {$ix <= $ixn} {incr ix} \
       {
        set y [getPtrVal $rdataX [expr $ix - 1]]

        if {abs($y) > abs($maxVal)} \
           {
            set maxVal $y
            set maxLoc $ix
           } 
       }

    if {$maxVal == 0.0} {set maxVal 1.0}

    set yList ""
    set xList ""
    set mList ""

    for {set ix -$idx} {$ix <= $idx} {incr ix} \
       {
        set y [getPtrVal $rdataX [expr $maxLoc + $ix - 1]]

        lappend yList [expr $y/$maxVal - 1.0]

        lappend xList $ix

        lappend mList [expr $ix*$ix] 
        lappend mList $ix
        lappend mList 1.0
       }

    set q [svdLinSol -x $mList -y $yList]
    
    set a [lindex $q 0]
    set b [lindex $q 1]
    set c [lindex $q 2]

    if {$a != 0.0} \
       {
        set ddx [expr -1.0*$b/(2.0*$a)]
       } \
    else \
       {
        set ddx 0.0
       }

    if {abs($ddx) > $idx} {set ddx 0.0}

    set refLoc [expr $maxLoc + $ddx]

    if {$verbose} {puts "Reference PPM. Interpolated Maximum:  [format %.3f [expr $x1Pts + $refLoc - 1]] pts."}
   }

if {![strcasecmp $dxPPM Auto]} \
   {
    if {[strcasecmp $tabName None]} \
       {
        set thresh [expr $f*abs($maxVal)]
        set noise  [expr 0.1*$thresh]

        pkFindROI -roi 1 \
             -sigma $noise -pChi 1.0e-04 -plus $thresh -minus -$thresh -ppmPrec 5 \
             -dx      $idx \
             -idx     $idx \
             -tol     $idx \
             -hiAdj   1.20 \
             -lw     15.00 \
             -reject -mask -out $tabName -noverb

       set pkTab [gdbRead table -in $tabName]
       set n     [gdbGetInfo $pkTab -parms entryCount]
 
       if {$n > 1} \
          { 
           if {$leftFlag == 0} \
              {
               if {$verbose} {puts -nonewline "Reference PPM. Rightmost Peak:        "}
               gdbSortTab $pkTab X_PPM -up
              } \
           elseif {$leftFlag == 1} \
              {
               if {$verbose} {puts -nonewline "Reference PPM. Leftmost Peak:         "}
               gdbSortTab $pkTab X_PPM -down
              } \
           else \
              {
               if {$verbose} {puts -nonewline "Reference PPM. Largest Peak:          "}
               gdbSortTab $pkTab HEIGHT -down
              } 
    
           set eList [gdbGetInfo $pkTab -parms entries]
    
           set e [lindex $eList 0]
           set refLoc [gdbGet $e X_AXIS]

           if {$verbose} \
              {
               puts "[format %.5f [expr $x1Pts + $refLoc - 1]] pts."
               puts "Reference PPM. Peaks Found:           $n"
              }
          }
       }
   }

set xPPM  [pnt2spec $fdataX CUR_XDIM $refLoc ppm]

set xCAR  [getParm $fdata NDCAR  CUR_XDIM]
set xOBS  [getParm $fdata NDOBS  CUR_XDIM]
set xORIG [getParm $fdata NDORIG CUR_XDIM]

if {$verbose} \
   {
    set fmt "% 10.5f"

    if {![strcasecmp $dxPPM Auto]} \
       {
        puts "Reference PPM. Reference Location: [format $fmt $xPPM] ppm."
       } \
    else \
       {
        puts "Reference PPM. Reference Location: [format %10s None]."
       }
   }

if {![strcasecmp $dxPPM Auto]} {set dxPPM [expr $refPPM - $xPPM]}

set dxHz [expr $xOBS*$dxPPM]

if {$verbose} \
   {
    set fmt "% 10.5f"
    puts "Reference PPM. Adjustment:         [format $fmt $dxPPM] ppm [format %.5f $dxHz] Hz."
   }

for {set i 0} {$i < $inCount} {incr i} \
   {
    if {$i} \
       {
        set inName [lindex $inList $i]
        set error  [rdFDATA $inName fdata]
       }

    if {$verbose} {puts "Reference PPM. Saving Calibration:    $inName"}

    setParm $fdata NDCAR  [expr $xCAR  + $dxPPM] CUR_XDIM
    setParm $fdata NDORIG [expr $xORIG + $dxHz]  CUR_XDIM

    set error [wrFDATA $inName $fdata]

    sysFree $fdata

    set fdata 0
   }

if {$briefFlag} {puts "xAdjPPM $dxPPM xAdjHz $dxHz"}
 

exit 0
