#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
# Residue type probabilities from chemical shift.

set talosDir  $env(TALOS_DIR)
set priorFlag 0 
set verbose   0

set resList    {A R N D C c Q E G H h I L K M F P S T W Y V}
set atomList   {HA CA CB C N}
set inName     csObs.tab
set outName    resProb.tab
set idVar      INDEX
set offsetList 0

set q(HA) 2.0
set q(CA) 1.0
set q(CB) 5.0 
set q(C)  1.0
set q(N)  2.0

if {[flagLoc $ARGV -help]} \
   {
    puts "Find Residue Type Probabilties from Chemical Shifts (2022.164.11.43):"
    puts " -in    inName   \[csObs.tab\]      Input Shift Table."
    puts " -out   outName  \[resProb.tab\]    Output Residue Probability Table."
    puts " -idVar varName  \[INDEX\]          ID Variable in Input Table."
    puts " -verb                            Verbose Mode ON."
    puts "Notes:"
    puts " 1. Ths input table should include the amino acid sequence of the target"
    puts "    protein as DATA statements (see below)."
    puts " 2. The input table should include INDEX ATOMNAME and SHIFT columns."
    puts " 3. The INDEX values are arbitrary integers that define chemical shifts"
    puts "    that belong to the same residue."
    puts " 4. Recognized ATOMNAME values: HN N HA HB CO CA CB"
    puts " 5. ATOMNAME values can include (I-1) (I+1) to indicate that the chemical shift"
    puts "    belongs to the previous or next residue in the sequence. Example: CA(I-1)."
    puts " 6. Example input table excerpt:"
    puts "" 
    puts "    DATA FIRST_RESID 1"
    puts ""
    puts "    DATA SEQUENCE MQQVRQSPQS LTVWEGETAI LNCSYENSAF DYFPWYQQFP GEGPALLISI"
    puts "    DATA SEQUENCE LSVSNKKEDG RFTIFFNKRE KKLSLHIADS QPGDSATYFC AASASFGDNS"
    puts "    DATA SEQUENCE KLIWGLGTSL VVNP"
    puts ""
    puts "    VARS   INDEX ATOMNAME SHIFT"
    puts "    FORMAT %5d   %-16s    %9.3f"
    puts ""
    puts "    3 HN                   8.832"
    puts "    3 N                  121.066"
    puts "    3 CA(I-1)             55.838"
    puts "    3 CB(I-1)             29.653"
    puts "    3 CA                  58.252"
    puts "    3 HA(I-1)              4.519"
    puts "    3 HB(I-1)              2.125"
    puts "    3 CO(I-1)            175.964"
    puts "    4 HN                   7.742"
    puts "    4 N                  118.147"
    puts "    4 CA                  62.431"
    puts "    4 CA(I-1)             58.240"
    puts "    4 CB                  34.820 "
    puts "..."
    exit 1
   }

getArgD $ARGV -in    inName
getArgD $ARGV -out   outName
getArgD $ARGV -talos talosDir
getArgD $ARGV -idVar idVar
getArgD $ARGV -atom  atomList

getArgD $ARGV -wHA   q(HA)
getArgD $ARGV -wCA   q(CA)
getArgD $ARGV -wCB   q(CB)
getArgD $ARGV -wC    q(C)
getArgD $ARGV -wN    q(N)

if {[flagLoc $ARGV -prev]}    {lappend offsetList -1}
if {[flagLoc $ARGV -next]}    {lappend offsetList  1}

if {[flagLoc $ARGV -prior]}   {set priorFlag 1}
if {[flagLoc $ARGV -noprior]} {set priorFlag 0}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

set w(HA) 0.38
set w(CA) 1.23
set w(CB) 1.34
set w(C)  1.45
set w(N)  3.95

#
#------------------------------------------------------------

proc pSort { i1 i2 } \
{
   set s1 [lindex $i1 1]
   set s2 [lindex $i2 1]

   if {$s1 < $s2} {return  1}
   if {$s1 > $s2} {return -1}

   return 0
}

proc getShifts { shiftPtr i eList offset } \
{
   global atomList

   upvar $shiftPtr s

   set thisAtomList $atomList

   lappend thisAtomList HN

   foreach atom $thisAtomList \
      {
       set s($i,$atom,atomList) ""
       set s($i,$atom,N) 0
       set s($i,$atom)   9999.0 
      }

   if {$offset < 0} \
      {
       set loc (I$offset)
      } \
   elseif {$offset > 0} \
      {
       set loc (I+$offset)
      } \
   else \
      {
       set loc (I)
      }

   set unambigList ""

   foreach entry $eList \
      {
       set atomName [string toupper [gdbGet $entry ATOMNAME]]
       if {[string first "|" $atomName] < 0} {lappend unambigList $atomName}
      }

   foreach entry $eList \
      {
       set atomInfo  [string toupper [gdbGet $entry ATOMNAME]]
       set thisList  [split $atomInfo "|"]
       set thisN     [llength $thisList]

       set thisShift [gdbGet $entry SHIFT]

       if {$thisShift > 999} continue
          
       foreach atomName $thisList \
          {
           if {$thisN > 1 && [lsearch -exact $unambigList $atomName] >= 0} \
              {
               continue
              }

           set j [string last $loc $atomName]
           set n [string length $atomName]

           if {$j > 0 && $j == $n - [string length $loc]} \
              {
               set atomName [string range $atomName 0 [expr $j - 1]]
              }

           if {[string first "(" $atomName] >= 0} continue

           switch -exact $atomName \
              {
               HA   -
               HA1  -
               HA2  -
               HA3  -
               HA   -
               HA# {set atom HA}
    
               HB  -
               HB1 -
               HB2 -
               HB3 -
               HB# {set atom HB}
    
               CA -
               CA {set atom CA}

               CB -
               CB {set atom CB}

               C  -
               CO -
               CO {set atom C}

               N -
               N  {set atom N}

               default {set atom $atomName} 
              }

           lappend s($i,$atom,atomList) $thisShift 
          }
      }

   foreach atom $thisAtomList \
      {
       if {![llength $s($i,$atom,atomList)]} \
          {
           set s($i,$atom)   9999.0
           set s($i,$atom,N) 0
          } \
       else \
          {
           set s($i,$atom,N) [llength $s($i,$atom,atomList)]
           set s($i,$atom)   [getAvg  $s($i,$atom,atomList)]
          }
      }
}

proc initTalosInfo {} \
{
   global talosDir talosInfo talosCount classInfo verbose

   set csList     [glob -nocomplain $talosDir/shifts/*.tab]
   set talosCount 0
   set i          0

   if {$verbose} {puts stderr ""}

   foreach csName $csList \
      {
       set rootName [file rootname [file tail $csName]]
       set className $talosDir/class/$rootName.tab

       set csTab    [gdbRead table -in $csName]
       set seqList  [getSeqList $csTab r1 rN]

       set classTab [gdbRead table -in $csName]
       set entry    [gdbFirst $classTab]

       if {$verbose} {puts stderr "Reading TALOS data for $rootName"}

       for {set resID $r1} {$resID <= $rN} {incr resID} \
          {
           set classInfo($rootName,$resID) None 
          }
 
       while {$entry} \
          {
           set resID  [gdbGet $entry RESID]
           set class  [gdbGet $entry ANGLE_CLASS]

           set classInfo($rootName,$resID) $class

           set entry  [gdbNext $entry]
          }

       for {set resID $r1} {$resID <= $rN} {incr resID} \
          {
           set eList   [gdbSelect -from $csTab -cond "RESID == $resID"]
           set n       [llength $eList]

           if {!$n} continue

           set talosInfo($i,res)   [lindex $seqList [expr $resID - $r1]] 
           set talosInfo($i,class) $classInfo($rootName,$resID)
 
           getShifts talosInfo $i $eList 0

           incr i
          }

       gdbFree $classTab
       gdbFree $csTab
      }

   set talosCount $i
}

proc findResProb { thisName } \
{
   global resList atomList talosCount talosInfo priorFlag scoreInfo maxScoreLoc
   global offsetList w q idVar outName verbose 

   set csTab    [gdbRead table -in $thisName]
   set seqList  [getSeqList $csTab r1 rN]
   set varList  [gdbGetInfo $csTab -parms varNames]

   if {[lsearch -exact $varList $idVar]} \
      {
       puts stderr "Error Finding Input table Variable $idVar."
       exit 1
      }

   set error [catch {set outUnit [open $outName w]}]

   if {$error} \
      {
       puts stderr "Error Opening Output File $outName"
       exit 1
      }

   puts $outUnit [getSeqListSeqStr $seqList $r1 $rN]

   set maxScoreLoc 0
   set i 0

   puts $outUnit "VARS   INDEX RESNAME P    RANK"
   puts $outUnit "FORMAT %4d   %s      %.5f %2d"

   foreach res $resList \
      {
       set priorInfo($res) 0
       set scoreInfo($i)   0

       incr i
      }

   for {set resID $r1} {$resID <= $rN} {incr resID} \
      {
       set res [lindex $seqList [expr $resID - $r1]]
       incr priorInfo($res)
      }

   set minID [getTabMin $csTab $idVar]
   set maxID [getTabMax $csTab $idVar]

   set n     [llength $atomList]
   set dMax  [expr log( 0.5*$talosCount/double($n) )]

   for {set thisID $minID} {$thisID <= $maxID} {incr thisID} \
     {
      foreach offset $offsetList \
         {
          set id [expr $thisID + $offset]
          set actualRes None

          if {![strcasecmp $idVar RESID]} \
             {
              if {$id >= $r1 && $id <= $rN} \
                 {
                  set actualRes [lindex $seqList [expr $id - $r1]]
                 }
             }

          set eList [gdbSelect -from $csTab -cond "$idVar == $thisID"]
          set n     [llength $eList]

          if {!$n} continue

          foreach res $resList {set probInfo($res) ""}

          getShifts shift query $eList $offset 

          for {set i 0} {$i < $talosCount} {incr i} \
             {
              set atomCount 0
              set dd 1.0

              set res $talosInfo($i,res)

              foreach atom $atomList \
                 {
                  if {$talosInfo($i,$atom) > 999} continue
                  if {!$shift(query,$atom,N)} continue
 
                  if {$w($atom) == 0.0 || $q($atom) == 0.0} \
                     { 
                      set s 1.0
                      set d 0.0
                     } \
                  else \
                     { 
                      set s     [expr $w($atom)/$q($atom)]
                      set dList ""

                      foreach shiftVal $shift(query,$atom,atomList) \
                         {
                          set d [expr abs($talosInfo($i,$atom) - $shiftVal)]
                          lappend dList $d
                         }

                      set d [getMin $dList]
                     }

                  if {$d > $dMax} {set d $dMax}

                  set dd [expr $dd*exp( -0.5*$d*$d )]

                  incr atomCount
                 }

              if {$atomCount} \
                 {
                  lappend probInfo($res) [expr pow( $dd, 1.0/$atomCount )]
                 }
             }

          set sum 0.0

          foreach res $resList \
             {
              set probInfo($res) [getAvg $probInfo($res)]

              if {![strcasecmp $res P] && $shift(query,HN,N) > 0} \
                 {
                  set probInfo($res) 0.0
                 }

              if {$priorFlag} \
                 {
                  set probInfo($res) [expr $probInfo($res)*$priorInfo($res)]
                 } \
              else \
                 {
                  if {!$priorInfo($res)} {set probInfo($res) 0.0}
                 }

              set sum [expr $sum + $probInfo($res)]
             }

          set pList ""

          if {$sum == 0.0} {set sum 1.0}

          foreach res $resList \
             {
              set probInfo($res) [expr $probInfo($res)/$sum]
              lappend pList "$res $probInfo($res)"
             }

          if {$verbose} \
             {
              if {![strcasecmp $idVar RESID]} \
                 {
                  puts stderr "$id $actualRes Offset $offset:"
                 } \
              else \
                 {
                  if {$offset} \
                     {
                      puts stderr "$idVar $id Offset $offset:"
                     } \
                  else \
                     {
                      puts stderr "$idVar $id:"
                     }
                 }
             }

          set pList [lsort -command pSort $pList]
          set k     0

          puts $outUnit ""

          foreach pItem $pList \
             {
              set res [lindex $pItem 0]
              set p   [lindex $pItem 1]
   
              if {$verbose} {puts stderr [format "  %s %.5f" $res $p]}
   
              puts $outUnit \
               [format " %3d %s %.5f %2d" $id $res $p [expr $k + 1]]
   
              if {![strcmp $actualRes $res]} \
                 {
                  incr scoreInfo($k)
   
                  if {$k > $maxScoreLoc} {set maxScoreLoc $k}
                 }

              incr k
             }
          }
      }

    if {$verbose && ![strcasecmp $idVar RESID]} \
       {
        for {set k 0} {$k <= $maxScoreLoc} {incr k} \
           {
            puts -nonewline stderr " $scoreInfo($k)"
           }

        puts stderr ""
       }

   close $outUnit
}

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input Shift Table $inName"
    exit 1
   }

initTalosInfo

findResProb $inName

exit 0 
