#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert PBox pulseshape format to Bruker format."
    puts " -in  inName   \[Pbox.RF]    RF Info, Produced by PBox."
    puts " -sim simName  \[pbox.sim]   Produced by PBox (Excitation Profile)."
    puts " -out brukName \[bruk.RF]    RF Info in Bruker Format."
    puts "Graphics:"
    puts " -graph                     Graph Mode ON (Default)."
    puts " -nograph                   Graph Mode Off."
    exit 1
   }

set inName    Pbox.RF
set simName   pbox.sim
set outName   bruk.RF
set graphFlag 1

getArgD $ARGV -in  inName
getArgD $ARGV -sim simName
getArgD $ARGV -out outName

if {[flagLoc $ARGV -graph]}   {set graphFlag 0}
if {[flagLoc $ARGV -nograph]} {set graphFlag 1}

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input $inName"
    exit 1
   }

if {![file exists $simName]} \
   {
    puts stderr "Error Finding Input $simName"
    exit 1
   }

#
#-------------------------------------------------------------

proc initRFVars {} \
{
   global objInfo

   set obj RF

   set objInfo($obj,phase)       ""
   set objInfo($obj,amp)         ""
   set objInfo($obj,incrLength)  "" 
   set objInfo($obj,gate)        ""

   set objInfo($obj,freq)        ""
   set objInfo($obj,mx)          ""
   set objInfo($obj,my)          "" 
   set objInfo($obj,mz)          ""
}

proc rdPBoxRF {} \
{
   global inName objInfo

   set obj RF

   set error [catch {set inUnit [open $inName r]}]

   if {$error} \
      {
       showError RF "Error Opening PBox RF File $inName"
       exit 1
      }

   set objInfo($obj,rfCount) 0

   while {[gets $inUnit line] >= 0} \
      {
       set key [lindex $line 0]
       set c   [string index $key 0]
       set n   [llength $line]

       if {![strcmp $c "#"]} \
          {
           puts stderr $line
           continue
          }

       if {$n != 4} continue

       set phase       [lindex $line 0]
       set amp         [lindex $line 1]
       set incrLength  [lindex $line 2]
       set gate        [lindex $line 3]

       if {$incrLength <= 0.0} continue

       if {$gate <= 0.0} {set amp 0.0}
 
       lappend objInfo($obj,phase)      $phase 
       lappend objInfo($obj,amp)        $amp
       lappend objInfo($obj,incrLength) $incrLength
       lappend objInfo($obj,gate)       $gate 

       incr objInfo($obj,rfCount)
      }
}

proc rdPBoxSim {} \
{
   global simName objInfo

   set obj RF

   set error [catch {set inUnit [open $simName r]}]

   if {$error} \
      {
       showError RF "Error Opening PBox RF Simulation $simName"
       exit 1
      }

   set objInfo($obj,simCount) 0

   while {[gets $inUnit line] >= 0} \
      {
       set key [lindex $line 0]
       set c   [string index $key 0]
       set n   [llength $line]

       if {![strcmp $c "#"]} \
          {
           puts stderr $line
           continue
          }

       if {$n != 4} continue

       lappend objInfo($obj,freq)  [expr 1000.0*[lindex $line 0]]
       lappend objInfo($obj,mx)    [lindex $line 1]
       lappend objInfo($obj,my)    [lindex $line 2]
       lappend objInfo($obj,mz)    [lindex $line 3]

       incr objInfo($obj,simCount)
      }
}

proc normPBoxRF {} \
{
   global objInfo

   set obj RF

   set maxVal  [getMax $objInfo($obj,amp)]
   set ampList ""

   if {$maxVal == 0.0} {set maxVal 1.0}

   for {set i 0} {$i < $objInfo($obj,rfCount)} {incr i} \
      {
       set amp [lindex $objInfo($obj,amp) $i]
       lappend ampList [expr 100.0*$amp/$maxVal]
      }

   set objInfo($obj,amp) $ampList
}

proc savePBoxRF {} \
{
   global objInfo outName

   set obj     RF

   set error [catch {set outUnit [open $outName w]}]

   if {$error} \
      {
       showError RF "Error Opening Output RF File $outName"
       exit 1
      }

   puts $outUnit "##TITLE= $outName" 
   puts $outUnit "##JCAMP-DX= 5.00 Bruker JCAMP library"
   puts $outUnit "##DATA TYPE= Shape Data"
   puts $outUnit "##ORIGIN= NIH PBox Translation "
   puts $outUnit "##OWNER= NIH"
   puts $outUnit "##DATE= [sysTime %m/%d/%Y]"
   puts $outUnit "##TIME= [sysTime %H:%M:%S]"
   puts $outUnit "##\$SHAPE_EXMODE= Excitation"
   puts $outUnit "##\$SHAPE_TOTROT= 9.000000e+01"
   puts $outUnit "##\$SHAPE_BWFAC= 4.000000e+00"
   puts $outUnit "##\$SHAPE_INTEGFAC= 6.00000e-02"
   puts $outUnit "##\$SHAPE_PARAMETERS= Type: PBox ;"
   puts $outUnit "##MINX= [format %.6e [getMin $objInfo($obj,amp)]]"
   puts $outUnit "##MAXX= [format %.6e [getMax $objInfo($obj,amp)]]" 
   puts $outUnit "##MINY= [format %.6e 0.0]"
   puts $outUnit "##MAXY= [format %.6e 359.999999]"
   puts $outUnit "##\$SHAPE_MODE= 1"
   puts $outUnit "##NPOINTS= $objInfo($obj,rfCount) "
   puts $outUnit "##XYPOINTS= (XY..XY)"

   set fmt "%.6e, %.6e"

   for {set i 0} {$i < $objInfo($obj,rfCount)} {incr i} \
      {
       set amp   [lindex $objInfo($obj,amp)   $i]
       set phase [lindex $objInfo($obj,phase) $i]

       puts $outUnit [format $fmt $amp $phase] 
      }

   puts $outUnit "##END= "

   close $outUnit
}

proc showPBoxWin {} \
{
   global objInfo graphFlag

   set obj RF

   if {!$graphFlag} return 

   set hi   400
   set wide 900

   set sHigh [winfo screenheight .]
   set sWide [winfo screenwidth  .]

   incr sHigh -16
   incr sWide -4

   wm title . "RF Profile"
   wm geom  . ${sWide}x${sHigh}+4+4

   set xList ""
 
   for {set i 0} {$i < $objInfo($obj,rfCount)} {incr i} \
      {
       lappend xList $i
      }

   graph .rf  -plotbackground black \
         -title "RF Profile" 

   .rf axis configure y  -hide 1 -min 0 -max 100
   .rf axis configure y2 -hide 1 -min 0 -max 360 

   .rf element create amp   -xdata $xList -ydata $objInfo($obj,amp) \
       -color cyan -pixels 0 -mapy y

   .rf element create phase -xdata $xList -ydata $objInfo($obj,phase) \
       -color yellow -pixels 0 -mapy y2

   graph .sim  -plotbackground black \
         -title "Frequency Profile"

   .sim element create Mx -xdata $objInfo($obj,freq) -ydata $objInfo($obj,mx) \
        -color cyan -pixels 0

   .sim element create My -xdata $objInfo($obj,freq) -ydata $objInfo($obj,my) \
        -color green -pixels 0

   .sim element create Mz -xdata $objInfo($obj,freq) -ydata $objInfo($obj,mz) \
        -color yellow -pixels 0

   button .q -text Quit -command exit

   pack .rf .sim -in . -side top -expand 1 -fill both
   pack .q -in . -side top -expand 1 -fill x

   update
}

initRFVars

rdPBoxRF
rdPBoxSim

normPBoxRF
showPBoxWin
savePBoxRF

