#!/bin/csh

set thisVersion = "2023.067.12.51"

if (`flagLoc $argv -help`) then
   echo "Extract Results from SAND 1D Spectral Decomposition (Script Version $thisVersion)."
   echo " Y Wu, O Sanati, M Uchimiya, K Krishnamurthy, A Edison, and F Delaglio"
   echo " -sandDir sandDir [sand0001]  Existing SAND Master Directory."
   echo " -firstID fID     [1]         First Spectrum ID to Update."
   echo " -lastID  lID     [Last]      Last Spectrum ID to Update, or Keyword Last."
   echo "Options for Creation of NMRPipe Simulated Data (Default = Use Value from Input Data):"
   echo " -sim                            Generate Simulations from SAND Results (Default)."
   echo " -nosim                          Do Not Generate Simulations from SAND Results."
   echo " -simAll                         Simulate Data for Each Individual Peak (Sets -sim)."
   echo " -rescale                        Rescale SAND Results (Default)."
   echo " -norescale                      No Rescale of SAND Results."
   echo " -lb      lb      [Default]      Exponential Broaden in Hz."
   echo " -p0      p0      [Default]      Zero Order Phase."
   echo " -p1      p1      [Default]      First Order Phase."
   echo "Notes:"
   echo " This utility is used by sand1D.com, see sand1D.com -help for details."
   exit 0
endif

@ simFlag     = 1
@ rescaleFlag = 1

set sandDir = (`getArgD $argv -sandDir sand0001`)
set firstID = (`getArgD $argv -firstID 1`)
set lastID  = (`getArgD $argv -lastID  Last`)

set lbVal   = (`getArgD $argv -lb Default`)
set p0Val   = (`getArgD $argv -p0 Default`)
set p1Val   = (`getArgD $argv -p1 Default`)

if (`flagLoc $argv -rescale`) then
   @ rescaleFlag = 1
endif

if (`flagLoc $argv -norescale`) then
   @ rescaleFlag = 0
endif

if (`flagLoc $argv -sim`) then
   @ simFlag = 1
endif

if (`flagLoc $argv -nosim`) then
   @ simFlag = 0
endif

if (`flagLoc $argv -simAll`) then
   @ simFlag = 2
endif

if (!(-d $sandDir)) then
   echo "sand1DUpdate. Error finding SAND Master Directory $sandDir"
   exit 1
endif

set tabDir    = $sandDir/tab
set csvDir    = $sandDir/csv
set simFIDDir = $sandDir/simFID
set simDir    = $sandDir/sim
set ftDir     = $sandDir/ft

set inDir     = $sandDir/sand_data

echo "sand1DUpdate. Version: $thisVersion"
echo "sand1DUpdate. Options: $*"
echo ""

if (!(-d $sandDir/ft)) then
   echo "sand1DUpdate. Creating directory $sandDir/ft"
   mkdir $sandDir/ft
endif

if (!(-d $sandDir/tab)) then
   echo "sand1DUpdate. Creating directory $sandDir/tab"
   mkdir $sandDir/tab 
endif

if (!(-d $sandDir/csv)) then
   echo "sand1DUpdate. Creating directory $sandDir/csv"
   mkdir $sandDir/csv 
endif

if ($simFlag != 0) then
   if (!(-d $sandDir/simFID)) then
      echo "sand1DUpdate. Creating directory $sandDir/simFID"
      mkdir $sandDir/simFID
   endif

   if (!(-d $sandDir/sim)) then
      echo "sand1DUpdate. Creating directory $sandDir/sim"
      mkdir $sandDir/sim
   endif
endif

if ($simFlag == 2) then
   set simAllDir    = $sandDir/simAll
   set simFIDAllDir = $sandDir/simFIDAll

   foreach d ($sandDir/simAll $sandDir/simFIDAll)
      if (!(-d $d)) then
         echo "sand1DUpdate. Creating directory $d"
         mkdir $d
      endif
   end
else
   set simAllDir    = None
   set simFIDAllDir = None
endif

if (`string csame $lastID Last`) then
   set i      = $firstID 
   set lastID = $firstID

   while ($i <= 9999) 
      if (!(-d $inDir/$i)) then
         break
      endif

      set lastID = $i
      @ i++
   end
endif

@ i = $firstID 

while ($i <= $lastID) 
   if (!(-d $inDir/$i)) then
      break
   endif

   echo "sand1DUpdate. Processing spectrum $i of $firstID to $lastID ..."

   set txtName     = $inDir/$i/SAND/sand_output/deconvoluted_peaks_refined.txt
   set tabName     = $inDir/$i/sim.tab
   set csvName     = $inDir/$i/sim.csv
   set ftRefName   = $inDir/$i/test.ft
   set fidRefName  = $inDir/$i/test.fid
   set simFIDName  = $inDir/$i/sim.fid
   set simFTName   = $inDir/$i/sim.ft1

   foreach f ($ftRefName $inDir/$i/SAND $txtName)
      if (!(-e $f)) then
         echo "sand1DUpdate. Error finding expected file $f"
         exit 1
      endif
   end

   if (!(-e $fidRefName)) then
      echo "sand1DUpdate. Creating $fidRefName from $ftRefName"
      nmrPipe -in $ftRefName -fn HT | nmrPipe -fn FT -inv -nodf | nmrPipe -fn ZF -inv | nmrPipe -fn SP -inv -hdr -out $fidRefName -ov
   endif

   if (`string csame $simAllDir None`) then
      set thisSimAllDir = None
   else
      set thisSimAllDir = $simAllDir/$i 

       if (!(-d $simAllDir/$i)) then
          echo "sand1DUpdate. Creating directory $simAllDir/$i"
          mkdir $simAllDir/$i
       endif
   endif

   if (`string csame $simFIDAllDir None`) then
      set thisSimFIDAllDir = None
   else 
      set thisSimFIDAllDir = $simFIDAllDir/$i
   
      if (!(-d $simFIDAllDir/$i)) then
         echo "sand1DUpdate. Creating directory $simFIDAllDir/$i"
         mkdir $simFIDAllDir/$i
      endif
   endif

   if ($rescaleFlag) then
      set scaleArgs = "-rescale"
      echo "sand1DUpdate. Using auto rescale mode."
   else
      set scaleArgs = "-norescale"
      echo "sand1DUpdate. No auto-rescaling will be used."
   endif
 
   set cmnd = "sand2fid.tcl -sandDir $sandDir -simDir $thisSimAllDir -simFIDDir $thisSimFIDAllDir -in $txtName -out $tabName -ftRef $ftRefName -fidRef $fidRefName -fid $simFIDName -ft $simFTName -lb $lbVal -p0 $p0Val -p1 $p1Val $scaleArgs" 

   echo "sand1DUpdate. $cmnd"
   $cmnd

   if (!(-e $tabName)) then
      echo "sand1DUpdate. Error generating NMRPipe signal table ($tabName)."
      exit 1
   endif

   if ($simFlag != 0) then
      if (!(-e $simFIDName)) then
         echo "sand1DUpdate. Error generating NMRPipe simulated time-domain data ($simFIDName)."
         exit 1
      endif

      if (!(-e $simFTName)) then
         echo "sand1DUpdate. Error generating NMRPipe simulated ($simFTName)."
         exit 1
      endif
   endif

   set thisFTName = (`nmrPrintf "%s/%04d.ft1" $ftDir      $i`)
   /bin/cp $ftRefName $thisFTName

   set thisTabName = (`nmrPrintf "%s/%04d.tab" $tabDir    $i`)
   /bin/cp $tabName $thisTabName

   set thisCSVName = (`nmrPrintf "%s/%04d.csv" $csvDir    $i`)
   nmrPrintf "%s, %10s, %10s, %s\n" index freq_ppm decay_hz amplitude                        >  $csvName
   getTabCol.tcl -in $thisTabName -var INDEX X_PPM XW_HZ HEIGHT -fmt %4d %10s %10s %15s -csv >> $csvName
   /bin/cp $csvName $thisCSVName
 
   if ($simFlag != 0) then
      set thisFIDName = (`nmrPrintf "%s/%04d.fid" $simFIDDir $i`)
      /bin/cp $simFIDName $thisFIDName

      set thisFTName = (`nmrPrintf "%s/%04d.ft1"  $simDir    $i`)
      /bin/cp $simFTName $thisFTName
   endif

   @ i++
end
