#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Scale or Set DI, DD, or W Values in a Dipolar Coupling Table:"
    puts " -in    inName  \[A.tab\]      Input Table"
    puts " -out   outName \[scaleA.tab\] Output Table"
    puts " -scale sInfo   \[None\]       AtomI AtomJ Value Triplets."
    puts " -pdb   pdbName \[ref.pdb\]    Optional PDB Input for DI Calc."
    puts " -var   varName \[DI\]         Variable to Adjust."
    puts " -fmt   fmt     \[%9.3f\]      Format for New Variables."
    puts " -wRef  wRef    \[21585.20\]   Factor for Calculating W from DI."
    puts " -tmp   tmpName \[d_tmp.tab\]  Temporary DC Output File."
    puts "Special Modes, Performed in this Order:"
    puts " -calcDI Calculate DI from PDB Structure (Requires PDB Input)."
    puts " -calcDD Calculate DD from Existing DI Values."
    puts " -calcW  Calculate W  from Existing DI Values."
    puts "Keeping Existing Values (Use with -calcDI etc):"
    puts " -keepDI Keep Existing DI Values if Found."
    puts " -keepDD Keep Existing DD Values if Found."
    puts " -keepW  Keep Existing W Values if Found."
    puts "Scale or Set Modes (Use Only One):"
    puts " -mult   Multiply Variable by Value (Default)."
    puts " -set    Set Variable to Value."
    puts "Notes:" 
    puts " 1. To Scale existing HN-N DI values by 1.1 and C-N values by 1.2:"
    puts "    scaleDI.tcl -var DI -scale HN N 1.1 C N 1.2 ..." 
    puts " 2. To Add DI Values and Create W Values Based on DI:"
    puts "    scaleDI.tcl -calcDI -calcW ..."
    exit 1
   }

set inName    A.tab 
set outName   scaleA.tab 
set pdbName   ref.pdb
set tmpName   d_tmp.tab
set varName   DI
set varFmt    %9.3f
set scaleInfo None
set wRef      21585.20
set multFlag  1
set verbose   0

set diFlag    0
set ddFlag    0
set wFlag     0

getArgD $ARGV -in    inName
getArgD $ARGV -out   outName
getArgD $ARGV -pdb   pdbName
getArgD $ARGV -var   varName
getArgD $ARGV -fmt   varFmt
getArgD $ARGV -tmp   tmpName
getArgD $ARGV -wRef  wRef

getListArgD $ARGV -scale scaleInfo

if {[flagLoc $ARGV -mult]} {set multFlag 1}
if {[flagLoc $ARGV -set]}  {set multFlag 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
#-----------------------------------------------------------------

#
# Add missing variables:

proc addVar { t vName vFmt } \
{
   set vList [gdbGetInfo $t -parms varNames]

   if {[lsearch -exact $vList $vName] >= 0} {return $vList}

   set v [gdbCreate variable -parent $t -varType float -name $vName -varFmt $vFmt]

   set vList [gdbGetInfo $t -parms varNames]

   return $vList
}

#
#-----------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error Finding DC Input $inName"
    exit 1
   }

set dTab [gdbRead table -in $inName]

#
# Calculate DI values if requested:

if {[flagLoc $ARGV -calcDI]} {set diFlag 1}

if {[flagLoc $ARGV -keepDI]} \
   {
    set vList [gdbGetInfo $dTab -parms varNames]
    if {[lsearch -exact $vList DI] >= 0} {set diFlag 0}
   }

if {$diFlag} \
   {
    if {$verbose} {puts stderr "Adding DI Values to DC Table."}
 
    set vList [addVar $dTab DI %9.3f]

    if {![file exists $pdbName]} \
       {
        puts stderr "Error Finding PDB Input $pdbName for DI Calculation."
        exit 1
       }

    catch "file delete $tmpName"
    catch "exec DC -inD $inName -outD $tmpName -pdb $pdbName"

    if {![file exists $tmpName]} \
       {
        puts stderr "Error Calculating DI Values."
        exit 1
       }

    set tTab [gdbRead table -in $tmpName]

    set eListA [gdbGetInfo $dTab -parms entries]
    set eListB [gdbGetInfo $tTab -parms entries]

    set nA     [llength $eListA]
    set nB     [llength $eListB]

    if {$nA != $nB} \
       {
        puts stderr "Error: Table Mismatch When Calculating DI Values."
        exit 1
       }

    for {set k 0} {$k < $nA} {incr k} \
       {
        set eA [lindex $eListA $k]
        set eB [lindex $eListB $k]

        set resIDI_A [gdbGet $eA RESID_I]
        set resIDJ_A [gdbGet $eA RESID_J]
        set atomI_A  [gdbGet $eA ATOMNAME_I]
        set atomJ_A  [gdbGet $eA ATOMNAME_J]

        set resIDI_B [gdbGet $eB RESID_I]
        set resIDJ_B [gdbGet $eB RESID_J]
        set atomI_B  [gdbGet $eB ATOMNAME_I]
        set atomJ_B  [gdbGet $eB ATOMNAME_J]

        if {$resIDI_A != $resIDI_B || $resIDJ_A != $resIDJ_B} \
           {
            puts stderr "Error: RESID Mismatch When Calculating DI Values."
            exit 1
           }

        if {[strcasecmp $atomI_A $atomI_B] || [strcasecmp $atomJ_A $atomJ_B]} \
           {
            puts stderr "Error: ATOMNAME Mismatch When Calculating DI Values."
            exit 1
           }

        gdbSet $eA DI [gdbGet $eB DI]
       }

    gdbFree $tTab

    catch "file delete $tmpName"
   }

#
# Calculate DD values if requested:

if {[flagLoc $ARGV -calcDD]} {set ddFlag 1}

if {[flagLoc $ARGV -keepDD]} \
   {
    set vList [gdbGetInfo $dTab -parms varNames]
    if {[lsearch -exact $vList DD] >= 0} {set ddFlag 0}
   }

if {$ddFlag} \
   {
    if {$verbose} {puts stderr "Adding DD Values to DC Table."}

    set vList [addVar $dTab DD %6.3f]
    set eList [gdbGetInfo $dTab -parms entries]

    foreach e $eList \
       {
        set di [gdbGet $e DI]

        if {$wRef == 0.0} \
           {
            set dd 0.0
           } \
        else \
           {
            set dd [expr abs($di/$wRef)]
           }

        gdbSet $e DD $dd
       }
   }

#
# Calculate W values if requested:

if {[flagLoc $ARGV -calcW]} {set wFlag 1}

if {[flagLoc $ARGV -keepW]} \
   {
    set vList [gdbGetInfo $dTab -parms varNames]
    if {[lsearch -exact $vList W] >= 0} {set wFlag 0}
   }

if {$wFlag} \
   {
    if {$verbose} {puts stderr "Adding W  Values to DC Table."}

    set vList [addVar $dTab W %6.3f]
    set eList [gdbGetInfo $dTab -parms entries]

    foreach e $eList \
       {
        set di [gdbGet $e DI]

        if {$di == 0.0} \
           {
            set w 0.0
           } \
        else \
           {
            set w [expr abs($wRef/$di)]
           }

        gdbSet $e W $w
       }
   }

#
# Adjust triplet-specified items according to atom assignment:

if {[strcasecmp $scaleInfo None]} \
   {
    set vList [addVar $dTab $varName $varFmt]

    set n [llength $scaleInfo]
    set i 0

    if {($n % 3)} \
       {
        puts stderr "Error: Scale Info Should Consist of Triplets:"
        puts stderr " $scaleInfo"
        exit 1
       }

    while {$i < $n} \
       {
        set  atomI [lindex $scaleInfo $i]
        incr i 

        set  atomJ [lindex $scaleInfo $i]
        incr i 

        set  s     [lindex $scaleInfo $i]
        incr i

        set    cond "(atomMatch(ATOMNAME_I, \"$atomI\" ) && "
        append cond " atomMatch(ATOMNAME_J, \"$atomJ\" )) || "
        append cond "(atomMatch(ATOMNAME_I, \"$atomJ\" ) && "
        append cond " atomMatch(ATOMNAME_J, \"$atomI\" ))"

        set eList [gdbSelect -from $dTab -cond $cond]
    
        foreach entry $eList \
           {
            set v [gdbGet $entry $varName]
    
            if {$multFlag} \
               {
                gdbSet $entry $varName [expr $v*$s]
               } \
            else \
               {
                gdbSet $entry $varName $s
               }
           }
       }
    }

if {![strcasecmp $outName None] || ![strcasecmp $outName stdout]} \
   {
    gdbWrite $dTab
   } \
else \
   {
    gdbWrite $dTab $outName
   }

exit
