#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Predict Protein Chemical Shifts (CS) via TALOS Database Search:"
    puts " -pdb  pdbName  \[ref.pdb\]          PDB Input."
    puts " -in   csName   \[csObs.tab\]        Optional CS Input (or -cs)."
    puts " -out  outName  \[csSearch.tab\]     Chemical Shift (CS) Output."
    puts " -clip cLimit   \[0.0\]              Clip Factor for Database Shifts;"    puts "                                     Use 0.0 for No Clipping."
    puts " -tmp  tmpName  \[csSim.tab\]        Temporary CS Output."
    puts " -excl exList   \[None\]             Sources to Exclude."
    puts " -db   dbName   \[cs.tab\]           TALOS Database File."
    puts " -verb                             Verbose Mode ON."
    puts "Ring Current Shift Adjustment:"
    puts " -hmsdb        Adjust Database Shifts for Ring Current Effects."
    puts " -hmscs        Adjust Predicted Shifts for Ring Current Effects."
    puts "Notes:"
    puts " 1. If provided, the CS Input File must include:"
    puts "    RC_SHIFT, CS_OBS, SHIFT, and SIGMA." 
    puts " 2. If no CS Input is provided, or if the supplied CS table"
    puts "    does not include RC_SHIFT, etc., a new CS table will"
    puts "    be created via the script simCS.tcl"
    puts " 3. Currently, predicts: HN HA N CA CB C"
    exit 1
   }

if {[info exists env(TALOS_DIR)]} \
   {
    set dbName $env(TALOS_DIR)/tab/cs.tab
   } \
else \
   {
    set dbName cs.tab
   }

set pdbName    ref.pdb
set outName    csSearch.tab 
set tmpName    csSim.tab
set csName     csObs.tab 
set exclList   ""

set clipFactor  4.0
set hmsCSFlag   0
set hmsDBFlag   0
set rotamerFlag 0
set verbose     1

getArgD $ARGV -pdb   pdbName
getArgD $ARGV -in    csName
getArgD $ARGV -cs    csName
getArgD $ARGV -out   outName
getArgD $ARGV -db    dbName
getArgD $ARGV -clip  clipFactor

getListArgD $ARGV -excl exclList

if {[flagLoc $ARGV -rotamer]}   {set rotamerFlag 1}
if {[flagLoc $ARGV -norotamer]} {set rotamerFlag 1}

if {[flagLoc $ARGV -hmscs]}     {set hmsCSFlag 1}
if {[flagLoc $ARGV -nohmscs]}   {set hmsCSFlag 0}

if {[flagLoc $ARGV -hmsdb]}     {set hmsDBFlag 1}
if {[flagLoc $ARGV -nohmsdb]}   {set hmsDBFlag 0}

if {[flagLoc $ARGV -verb]}      {set verbose 1}
if {[flagLoc $ARGV -noverb]}    {set verbose 0}

#
#------------------------------------------------------------------------

proc initCSDB {} \
{
   global exclList dbName dbTab verbose

   if {![file exists $dbName]} \
      {
       puts stderr "Error Finding CS Database File $dbName"
       exit 1
      }

    if {$verbose} {puts stderr "Reading:      $dbName"}

    set t  [gdbRead table -in $dbName]
    set nb [gdbGetInfo $t -parms entryCount]

    if {$verbose} {puts stderr "Filtering:    $dbName"}

    set c "PHI < 999 && PSI < 999 && PHI_NEXT < 999 && PSI_PREV < 999"

    foreach eName $exclList \
       {
        set str [file root [file tail $eName]]
        append c " && strcmp( SOURCE, '$str' )"
       }

    set dbTab [gdbSelect -from $t -new -cond $c]
    set na    [gdbGetInfo $dbTab -parms entryCount]

    if {$verbose} \
       {
        puts stderr "Excluded:     [expr $nb - $na] Entries."
       }
}

proc initResClass {} \
{
   global resClassInfo

   set classInfo { "A" "C c" "D E N Q" "G" "H h" "M I L V" 
                   "K R" "P" "S T" "F W Y" "X ?" "<" ">" }

   foreach resList $classInfo \
      {
       foreach resName $resList \
          {
           set resClassInfo($resName) $resList
          }
      }
}

proc getResClassCond { resName } \
{
   global resClassInfo

   if {![info exists resClassInfo($resName)]} \
      {
       set resClassInfo($resName) $resName
      }

   set n [llength $resClassInfo($resName)]

   set c ""
   set i 0

   foreach resName $resClassInfo($resName) \
      {
       if {$i} {append c " || "}
       append c "!strcmp( RESNAME, '$resName' )"
       incr i
      }
  
   return $c 
}

proc getDBSelection { resID resName off } \
{
    global psiPrev phi psi phiNext chi dbTab eList rotamerFlag

    set dPsiPrev [lindex $off 0]
    set dPhi     [lindex $off 1]
    set dPsi     [lindex $off 2]
    set dPhiNext [lindex $off 3]

    set    c "angleNear( PHI, $phi, $dPhi ) && "
    append c "angleNear( PSI, $psi, $dPsi ) && "
    append c "angleNear( PHI_NEXT, $phiNext, $dPhiNext ) && "
    append c "angleNear( PSI_PREV, $psiPrev, $dPsiPrev )"
    append c " && ([getResClassCond $resName])"

    if {$rotamerFlag} \
       {
        append c " && (strcasecmp( ATOMNAME, 'CB' ) ||"
        append c " (!strcasecmp( ATOMNAME, 'CB' ) &&"
        append c " rotamerTest( RESNAME, '$resName', CHI, $chi, 1 )))"
       }

    set eList [gdbSelect -from $dbTab -cond $c]
}

proc getPDBAngles { resID } \
{
   global pdbSrc phi psi phiNext psiPrev chi

   set n 0

   set psiPrev [dynGetPsi -src $pdbSrc -res [expr $resID - 1]]
   if {$psiPrev > 999} {return 1}

   set phiNext [dynGetPhi -src $pdbSrc -res [expr $resID + 1]]
   if {$phiNext > 999} {return 1}

   set aInfo [dynGetPhiPsi -src $pdbSrc -res $resID]
   set phi   [lindex $aInfo 0]
   set psi   [lindex $aInfo 1]

   if {$phi > 999 || $psi > 999} {return 1}

   set chi [dynGetChi -src $pdbSrc -res $resID -chi1]
   if {![llength $chi]} {set chi 9999.0}

   return 0
}

proc getCSInfo { csName pdbName } \
{
   global tmpName csInfo seqList r1 rN verbose

   if {![strcmp $csName None]} \
      {
       set csName $tmpName
       catch "file delete $csName"
      }

   if {![file exists $csName]} \
      {
       if {$verbose} {puts stderr "Creating:     $csName"}
       catch "exec simCS.tcl -pdb $pdbName -in None -out $csName -hmsNone"
      }

   if {![file exists $csName]} \
      {
       puts stderr "Error Finding CS Input $csName"
       exit 1
      }

   if {$verbose} {puts stderr "Reading:      $csName"}

   set csTab [gdbRead table -in $csName]
   set vList [gdbGetInfo $csTab -parms varNames]

   foreach v {RESID RESNAME ATOMNAME CS_OBS SHIFT RC_SHIFT SIGMA}  \
      {
       if {[lsearch -exact $vList $v] < 0} \
          {
           gdbFree $csTab

           if {$verbose} {puts stderr "Initializing: $tmpName"}

           catch \
            "exec simCS.tcl -pdb $pdbName -in $csName -out $tmpName -hmsNone"

           if {$verbose} {puts stderr "Re-Reading:   $tmpName"}
           set csTab [gdbRead table -in $tmpName]
           break
          }
      }

   adjGLYHA $csTab CS_OBS

   catch "unset csInfo"

   set r1 0
   set rN 0

   set seqList   [getSeqList $csTab r1 rN]
   set entryList [gdbGetInfo $csTab -parms entries]

   foreach e $entryList \
      {
       set resID     [gdbGet $e RESID]
       set atomName  [gdbGet $e ATOMNAME]
       set obsShift  [gdbGet $e CS_OBS]
       set predShift [gdbGet $e SHIFT]
       set rcShift   [gdbGet $e RC_SHIFT]
       set csSigma   [gdbGet $e SIGMA]
       set hmShift   [gdbGet $e HM_SHIFT]
      
       if {![llength $hmShift]} {set hmShift 9999.0}

       if {$obsShift > 999 || $predShift > 999 || $rcShift > 999} continue

       set csInfo($resID,$atomName,obs)  [expr $obsShift  - $rcShift] 
       set csInfo($resID,$atomName,pred) [expr $predShift - $rcShift]

       set csInfo($resID,$atomName,hm)    $hmShift
       set csInfo($resID,$atomName,rc)    $rcShift
       set csInfo($resID,$atomName,sigma) $csSigma
   
       lappend csInfo($resID,atomNames) $atomName
      }

   gdbFree $csTab
}

proc getPDBInfo { pdbName } \
{
   global pdbSrc

   if {![file exists $pdbName]} \
      {
       puts stderr "Error Finding PDB Input $pdbName"
       exit 1
      }

   if {[strcmp $pdbSrc 0]} {dynFree $pdbSrc}
   set pdbSrc [dynRead -pdb -in $pdbName]
}

proc updatePDBHist { srcName resID } \
{
    global pdbUsedList pdbUsedInfo

    if {[lsearch -exact $pdbUsedList $srcName] < 0} \
       {
        lappend pdbUsedList $srcName
        set pdbUsedInfo($srcName) "" 
       }

    lappend pdbUsedInfo($srcName) $resID
}

proc reportPDBStats { outName outTab } \
{
    global pdbUsedList pdbUsedInfo r1 rN verbose

    set nList ""
    set nn 0

    foreach srcName $pdbUsedList \
       {
        set n [llength $pdbUsedInfo($srcName)]
        incr nn $n

        if {[lsearch -exact $nList $n] < 0} \
           {
            set pdbCountInfo($n) "" 
            lappend nList $n
           }

        lappend pdbCountInfo($n) $srcName
       } 

    set np  [llength $pdbUsedList]
    set str "$nn Total Shifts Used, From $np PDB Files."

    if {$verbose} \
       { 
        puts stderr ""
        puts stderr $str
        puts stderr ""
       }

    if {[strcasecmp $outName None]} \
       {
        gdbAddNote $outTab ""
        gdbAddNote $outTab "REMARK TALOS Search Chemical Shift Prediction."
        gdbAddNote $outTab "REMARK $str"
       }

    set sList [lsort -decreasing -integer $nList]

    foreach n [lrange $sList 0 4] \
       {
        foreach srcName $pdbCountInfo($n) \
           {
            set nr 0

            for {set resID $r1} {$resID <= $rN} {incr resID} \
               {
                if {[lsearch -exact $pdbUsedInfo($srcName) $resID] >= 0} \
                   {
                    incr nr
                   }
               }

            set str \
             [format " %4d Shifts to %3d Residues From PDB %s" $n $nr $srcName]

            if {$verbose} {puts stderr $str} 

            if {[strcasecmp $outName None]} {gdbAddNote $outTab "REMARK $str"}
           }
       }

    if {$verbose} {puts stderr ""}
    if {[strcasecmp $outName None]} {gdbAddNote $outTab ""} 
}

proc reportCSStats { outName outTab } \
{
   global shiftInfo verbose

   foreach atomName {HA HA2 HA3 CA CB C N HN} \
      {
       if {![info exists shiftInfo($atomName)]} continue

       set n   [llength $shiftInfo($atomName)]
       set rms [getRMS  $shiftInfo($atomName)]

       set str [format " %-4s RMS: %6.3f ppm  Count: %3d"  $atomName $rms $n]

       if {[strcasecmp $outName None]} {gdbAddNote $outTab "REMARK $str"}

       if {$verbose} {puts stderr $str}
      }
}

dynInit

initResNames
initResClass

initCSDB

set offList  {
              {10 10 10 10 100} 
              {15 15 15 15  50} 
              {20 20 20 20  30} 
              {25 25 25 25  30}
              {25 25 25 90  25}
             }

set pdbUsedList ""
set eList       ""

set pdbSrc 0
set csTab  0

getPDBInfo $pdbName
getCSInfo  $csName $pdbName

if {[strcasecmp $outName None]} \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error Opening CS Output File $outName"
        exit 1
       }

    puts $outUnit [dynGetPDBSeqStr $pdbSrc]
    puts $outUnit ""

    puts -nonewline $outUnit "VARS   RESID RESNAME ATOMNAME"
    puts $outUnit "      CS_OBS CS_SURF SHIFT RC_SHIFT HM_SHIFT CS_DIFF SIGMA"

    puts -nonewline $outUnit "FORMAT %4d   %-4s    %-4s    "
    puts $outUnit "      %8.3f  %8.3f   %8.3f %8.3f    %8.3f    %8.3f   %8.3f" 
    puts $outUnit ""
   }

for {set resID [expr $r1 + 1]} {$resID < $rN} {incr resID} \
   {
    set nUndef [getPDBAngles $resID]
    if {$nUndef} continue

    set resName [lindex $seqList [expr $resID - $r1]]

    set i 0

    foreach item $offList \
       {
        set off [lrange $item 0 3]
        set n   [lindex $item end]

        incr i 

        getDBSelection $resID $resName $off
        if {[llength $eList] >= $n} break
       }
 
    foreach e $eList \
       {
        set obsShift [gdbGet $e CS_OBS]
        set rcShift  [gdbGet $e RC_SHIFT]
        set csSigma  [gdbGet $e SIGMA]
        set atomName [gdbGet $e ATOMNAME]

        if {$obsShift > 999 || $rcShift > 999} continue

        if {$clipFactor != 0.0 && $csSigma < 999} \
           {
            set csMax [expr $rcShift + $clipFactor*$csSigma]
            set csMin [expr $rcShift - $clipFactor*$csSigma]

            if {$obsShift < $csMin} {set obsShift $csMin}
            if {$obsShift > $csMax} {set obsShift $csMax}
           }

        set s [expr $obsShift - $rcShift]

        if {$hmsDBFlag} \
           {
            set hmShift [gdbGet $e HM_SHIFT]
            if {$hmShift < 999} {set s [expr $s - $hmShift]}
           }

        lappend predInfo($resID,$atomName) $s 

        updatePDBHist [gdbGet $e SOURCE] $resID
       }

    if {![info exists csInfo($resID,atomNames)]} continue

    foreach atomName $csInfo($resID,atomNames) \
       {
        if {![info exists predInfo($resID,$atomName)]} continue
        if {![info exists csInfo($resID,$atomName,obs)]} continue

        set n [llength $predInfo($resID,$atomName)]

        set surfShift $csInfo($resID,$atomName,pred)
        set obsShift  $csInfo($resID,$atomName,obs)
        set rcShift   $csInfo($resID,$atomName,rc)
        set hmShift   $csInfo($resID,$atomName,hm)

        if {$n < 5} \
           {
            set predShift $csInfo($resID,$atomName,pred)
            set csSigma   $csInfo($resID,$atomName,sigma)
           } \
        else \
           {
            set predShift [getAvg    $predInfo($resID,$atomName)]
            set csSigma   [getStdDev $predInfo($resID,$atomName)]
           }

        if {$hmsCSFlag} \
           {
            if {$hmShift < 999} {set predShift [expr $predShift + $hmShift]}
           }

        set csDiff [expr $obsShift - $predShift]

        lappend shiftInfo($atomName) $csDiff 

        if {$verbose} \
           {
            set fmt \
             "%4d %-4s Obs: %8.3f Pred: %8.3f Sigma: %8.3f N: %5d Iter: %d"

            puts stderr \
             [format $fmt $resID $atomName $obsShift $predShift $csSigma $n $i]
           }

        if {[strcasecmp $outName None]} \
           {
            set fmt " %4d %-4s %-4s %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f" 

            puts $outUnit \
             [format $fmt $resID $resName $atomName \
                     [expr $obsShift  + $rcShift] \
                     [expr $surfShift + $rcShift] \
                     [expr $predShift + $rcShift] \
                           $rcShift $hmShift $csDiff $csSigma]
           }
       }
   }

close $outUnit

if {[strcasecmp $outName None]} {set csTab [gdbRead table -in $outName]}

reportPDBStats $outName $csTab
reportCSStats  $outName $csTab

gdbWrite $csTab $outName

exit 0

