#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set refName    ref.pdb
set rmsFlag    0
set angleFlag  0 
set segLength  4

set pdbBase    $env(PDBH_BASE)
set pdbDir     $env(PDBH_DIR)

set seqTabName   $pdbBase/seqList.tab
set angleTabName $pdbBase/angles.tab

if {[flagLoc $ARGV -help]} \
   {
    puts "Locate Sequence Segments in the PDB Sequence List Table:"
    puts " -pdb       pdbName  \[ref.pdb\] PDB Target Input."
    puts " -segLength sLength  \[4\]       Segment Length for Search."
    puts " -rms                          Report RMS of Sequence Matches"
    puts " -angles                       Report Angles of Sequence Matches"
    exit 1
   }

getArgD $ARGV -pdb       refName
getArgD $ARGV -pdbDir    pdbDir
getArgD $ARGV -seqName   seqTabName
getArgD $ARGV -segLength segLength 

if {[flagLoc $ARGV -rms]}    {set rmsFlag   1}
if {[flagLoc $ARGV -angles]} {set angleFlag 1}

#
#----------------------------------------------------------------------

proc getEquivInfo {} \
{
   global eqInfo

   set resList {A R N D C c Q E G H h I L K M F P S T W Y V ?}
   set eqList  {{S T} {L I} {K R} {D E} {N Q} {A V}}

   foreach res $resList {set eqInfo($res) $res}

   foreach item $eqList \
      {
       set rI [lindex $item 0]
       set rJ [lindex $item 1]

       lappend eqInfo($rI) $rJ
       lappend eqInfo($rJ) $rI
      }
}

proc getEquivSeq { seq } \
{
    global eqInfo

    set n [string length $seq]

    switch $n \
       {
        0 {return ""}

        1 {
           return $eqInfo($seq)
          }

        default \
           {
            set head  [string index $seq 0]
            set tail  [string range $seq 1 [expr $n - 1]]
            set tList [getEquivSeq $tail]
            set sList ""

            foreach res $eqInfo($head) \
               {
                foreach t $tList \
                   {
                    lappend sList $res$t
                   }
               }

            return $sList
           }
       }
}

proc getSubsCount { origSeq seq } \
{
   set n     [string length $seq]
   set count 0
   
   for {set i 0} {$i < $n} {incr i} \
      {
       set cI [string index $origSeq $i]
       set cJ [string index $seq     $i]

       if {[string compare $cI $cJ]} {incr count}
      }

    return $count
}

proc getAngleInfo {} \
{
   global angleTabName angleInfo angleFlag

   if {!$angleFlag} return

   set aTab  [gdbRead table -in $angleTabName]
   set entry [gdbFirst $aTab]

   while {$entry} \
      {
       set pdbName [gdbGet $entry PDB_NAME]
       set res     [gdbGet $entry RESID]

       set angleInfo($pdbName,$res,phi) [gdbGet $entry PHI]
       set angleInfo($pdbName,$res,psi) [gdbGet $entry PSI]
       set angleInfo($pdbName,$res,chi) [gdbGet $entry CHI1]

       set entry [gdbNext $entry]
      } 
}

proc getSeq { src r1 rN } \
{
   set seqList ""

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       set resName [dynGetInfo -src $src -resName -res $res]
       set resCode [getResName1 $resName]

       append seqList $resCode
      }

   return $seqList
}

proc findSubSeq { seq res1 resN subSeq } \
{
   set locList ""
   set str     $seq
   set offset  $res1

   while {[string length $str]} \
      {
       set loc [string first $subSeq $str]

       if {$loc < 0} {return $locList}

       lappend locList [expr $loc + $offset]

       set str    [string range $str [expr $loc + 1] end]
       set offset [expr $offset + $loc + 1]
      }

   return $locList
}

proc getPDBSrc { pdbDir pdbName rmsFlag } \
{
   if {!$rmsFlag} {return 0}
 
   if {[file exists $pdbDir/$pdbName]} \
      {
       set pdbSrc [dynRead -pdb -in $pdbDir/$pdbName]
      } \
   else \
      {
       puts stderr "Skipping $pdbName"
       set pdbSrc  0
      }

   return $pdbSrc
}

proc getRMS { refSrc aSrc r1 rN a1 aN rmsFlag } \
{
    if {!$rmsFlag} {return 0.0}
    if {!$refSrc}  {return 0.0}
    if {!$aSrc}    {return 0.0}

    set aInfo \
     [dynAlign -refSrc $refSrc -aSrc $aSrc -r1 $r1 -rN $rN -a1 $r1]

    set rms [lindex $aInfo end]

    return $rms 
}

dynInit -fast

initResNames

getEquivInfo
getAngleInfo

set seqTab  [gdbRead table -in $seqTabName]
set refSrc  [dynRead -pdb -in $refName]

puts "VARS    INDEX RANK D_RES1 D_RESN D_SEQ PDB_NAME RES1 RESN SEQ NSUBS RMS"
puts "FORMAT  %4d   %2d  %4d    %4d    %8s   %16s     %3d  %3d  %8s %d    %9.4f"
puts ""

set pdbInfo [dynGetInfo -src $refSrc]
set r1      [lindex $pdbInfo 0]
set rN      [lindex $pdbInfo 1]
set seqList [getSeq $refSrc $r1 $rN] 

set s1 $r1
set sN [expr $r1 + $segLength - 1]
set id 0

while {$sN <= $rN} \
   {
    set a1       [expr $s1 - $r1]
    set aN       [expr $sN - $r1]
    set origSeq  [string range $seqList $a1 $aN]
    set equivSeq [getEquivSeq $origSeq]

    foreach seq $equivSeq \
       {
        set cond  "strstr( SEQUENCE, '$seq' ) >= 0"
        set eList [gdbSelect -from $seqTab -cond $cond]
        set n     [llength $eList]
        set rank  0
   
        set fmt "  %4d %2d %4d %4d %8s %16s %3d %3d %8s %d %9.4f" 
    
        if {$n} \
           {
            foreach entry $eList \
               {
                set pdbName  [gdbGet $entry PDB_NAME]
                set res1     [gdbGet $entry RES1]
                set resN     [gdbGet $entry RESN]
                set totalSeq [gdbGet $entry SEQUENCE]
                set nn       [getSubsCount $origSeq $seq]

                set pdbSrc   [getPDBSrc $pdbDir $pdbName $rmsFlag]
                set resList  [findSubSeq $totalSeq $res1 $resN $seq]
 
                foreach m1 $resList \
                   {
                    incr id
                    incr rank 

                    set mN  [expr $m1 + $segLength - 1]
                    set rms [getRMS $refSrc $pdbSrc $s1 $sN $m1 $mN $rmsFlag]

                    puts -nonewline [format $fmt \
                     $id $rank $s1 $sN $origSeq $pdbName $m1 $mN $seq $nn $rms]

                    if {$angleFlag} \
                       {
                        set f " % 8.2f"

                        for {set r0 [expr $m1 + 1]} {$r0 < $mN} {incr r0} \
                           {
                            puts -nonewline \
                             [format $f $angleInfo($pdbName,$r0,phi)]

                            puts -nonewline \
                             [format $f $angleInfo($pdbName,$r0,psi)]

                            puts -nonewline \
                             [format $f $angleInfo($pdbName,$r0,chi)]

                            puts ""
                           }
                       }

                    puts ""
                   }

                if {$pdbSrc} {dynFree $pdbSrc}
               }
           } 
       } 
     
    incr s1
    incr sN
   }

exit 0
