#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Prepare Titles to Process Spectra for Pool Blank or Screen Data:"
    puts " -mode    modeName \[blank\]                Keyword: blank or screen"
    puts " -rackID  rackID   \[1\]                    Rack ID to Use."
    puts " -rackDir rackInfo \[Blanks_Rack_%d_Pools\] Corresponding Rack Directory."
    puts " -cIDTab  tabInfo  \[id/plate%d.tab ...\]   File Names of Compound ID Tables." 
    puts " -jobID   jobID    \[sysTime %Y%m%d\]       Job ID for Titles."
    puts " -poolLib libName  \[poolLibrary.tab\]      Input Table of Compound Library Data."
    puts " -nc      numC     \[5\]                    Expected Number of Compounds per Pool."
    puts " -force                                   Replace existing title files."
    puts " -verb                                    Verbose Mode ON."
    puts " -noverb                                  Verbose Mode OFF (Default)."
    puts "Notes:"
    puts " 1. Each rack directory is expected to have subdirectories 1 up to 96." 
    puts ""
    puts " 2. The Compound ID Tables should have the following format:"
    puts ""
    puts "   VARS    EXPERIMENT CMPD_ID"
    puts "   FORMAT  %11s       %10s"
    puts ""
    puts "     1  1948979#1"
    puts "     2  1375692#1"
    puts "     3    11994#2"
    puts "     4  2031826#1"
    exit 0
   }

set jobID     [sysTime %Y%m%d]
set libName   poolLibrary.tab
set mode      blank
set rackID    1
set nc        5
set forceFlag 0
set verbose   0

getArgD $ARGV -rackID rackID
getArgD $ARGV -mode   mode
getArgD $ARGV -nc     nc

set j [expr 1 + $nc*($rackID - 1)]
set   tabList ""

for {set i 1} {$i <= $nc} {incr i} \
   {
    lappend tabList [format id/plate%d.tab $j]
    incr j
   }

if {![strcasecmp $mode blank]} \
   {
    set rackDir Blanks_Rack_${rackID}_Pools
    set mode blank
   } \
else \
   {
    set rackDir Screen_Rack_${rackID}_Pools
    set mode screen
   }

getListArgD $ARGV -cIDTab  tabList

getArgD $ARGV -libName libName
getArgD $ARGV -rackDir rackDir
getArgD $ARGV -jobID   jobID

if {[flagLoc $ARGV -force]}   {set forceFlag 1}
if {[flagLoc $ARGV -noforce]} {set forceFlag 0}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

#
#------------------------------------------------------------------------------------------

proc getPoolLibDir { t cID } \
{
   global nDup nBad verbose

   if {![strcmp $t 0]} {return None}

   set eList [gdbSelect -from $t -cond "!strcasecmp( CMPD_ID, '$cID' )"]
   set n     [llength $eList]

   if {!$n} \
      {
       if {$verbose} {puts "Pool Title Warning: Missing Library Compound ID $cID"}
       incr nBad
       return None
      }

   if {$n > 1} \
      {
       if {$verbose} {puts "Pool Title Warning: Duplicate Data for Compound ID $cID"}
       incr nDup
      }

   set entry   [lindex $eList end]
   set dirName [gdbGet $entry DIR_NAME]

   return $dirName
}

#
#---------------------------------------------------------------------------------------------

set libTab 0
set nBad   0
set nGood  0
set nMiss  0
set nDup   0

foreach tabName $tabList \
   {
    if {![file exists $tabName]} \
       {
        puts "Pool Title Error Finding Compound ID File $tabName"
        exit 1
       }
   }

if {![file isdirectory $rackDir]} \
   {
    puts "Pool Title Error Finding Rack Directory $rackDir"
    exit 1
   }

if {[file exists $libName]} {set libTab [gdbRead table -in $libName]}

set nc       [llength $tabList]
set expList  ""
set missList ""

for {set i 1} {$i <= $nc} {incr i} \
   {
    set tabName [lindex $tabList [expr $i - 1]]

    set t     [gdbRead table -in $tabName]
    set eList [gdbGetInfo $t -parms entries]
    set vList [gdbGetInfo $t -parms varNames]

    if {[lsearch -exact $vList EXPERIMENT] < 0} \
       {
        puts "Pool Title Error: Table $tabName is missing EXPERIMENT column."
        exit 1
       }

    if {[lsearch -exact $vList CMPD_ID] < 0} \
       {
        puts "Pool Title Error: Table $tabName is missing CMPD_ID column."
        exit 1
       }

    foreach e $eList \
       {
        set cID     [join [split [gdbGet $e CMPD_ID] "#"] "_"]
        set expName [gdbGet $e EXPERIMENT]
        set eID     [file root $expName]

        if {[lsearch -exact $expList $expName] < 0} \
           {
            lappend expList $expName
           }

        if {![file isdirectory $rackDir/$expName]} \
           {
            if {$verbose} \
               {
                puts "Pool Title Warning: Missing Spectral Data $rackDir/$expName $cID"
               }

            if {[lsearch -exact $missList $expName] < 0} {lappend missList $expName}
            continue
           }

        set expInfo($expName,$i)  $cID
        set expInfo($expName,eID) $eID

        if {$verbose} \
           {
            puts "$rackDir/$expName $tabName Job $jobID Rack $rackID Exp $eID $mode C$i CID $cID"
           }
       }
   }

set nMiss [llength $missList]
set nGood 0
set nExp  0
set nBad  0
set nDup  0

foreach expName $expList \
   {
    if {![file isdirectory $rackDir/$expName]} continue

    if {!$forceFlag} \
       {
        if {[file exists $rackDir/$expName/title]} \
           {
            puts "Pool Title Notice: Existing title file for Data $rackDir/$expName"
            puts "   Use -force flag to replace existing title files."
            continue
           }
       }

    incr nExp

    set error [catch {set outUnit [open $rackDir/$expName/title w]}]

    if {$error} \
       {
        puts "Pool Title Error Opening File $rackDir/$expName/title"
        puts "Check file permissions and try again."
        exit 1
       }

    set eID $expInfo($expName,eID)

    puts $outUnit "${jobID}-r${rackID}-e${eID}-${mode}-pool"

    catch "close $outUnit"

    set error [catch {set outUnit [open $rackDir/$expName/pool.txt w]}]

    if {$error} \
       {
        puts "Pool Title Error Opening File $rackDir/$expName/pool.txt"
        puts "Check file permissions and try again."
        exit 1
       }

    for {set i 1} {$i <= $nc} {incr i} \
       {
        if {[info exists expInfo($expName,$i)]} \
           {
            puts $outUnit "$expInfo($expName,$i)"
            incr nGood
           } \
        else \
           {
            puts $outUnit "None"
           }
       }
    
    catch "close $outUnit"

    if {![strcmp $libTab 0]} continue 

    set error [catch {set outUnit [open $rackDir/$expName/pool_lib.txt w]}]

    if {$error} \
       {
        puts "Pool Title Error Opening File $rackDir/$expName/pool_lib.txt"
        puts "Check file permissions and try again."
        exit 1
       }

    for {set i 1} {$i <= $nc} {incr i} \
       {
        if {[info exists expInfo($expName,$i)]} \
           {
            puts $outUnit [getPoolLibDir $libTab $expInfo($expName,$i)]
           } \
        else \
           {
            puts $outUnit "None"
            incr nBad
           }
       }

    catch "close $outUnit"

    if {![strcmp $libTab 0]} continue
   }

if {$nMiss} \
   {
    puts "Pool Title Warning: $nMiss Entries for $rackDir Rack $rackID $mode are missing spectral data."
   }

if {$nDup} \
   {
    puts "Pool Title Warning: $nDup Entries for $rackDir Rack $rackID $mode have more than one library match."
   }

if {$nBad} \
   {
    puts "Pool Title Warning: $nBad Entries for $rackDir Rack $rackID $mode are missing from the library."
   }

puts "$rackDir Job: $jobID Rack: $rackID Experiments: $nExp Compounds: $nGood"

exit 0
