#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

proc initVars {} \
{
   global userSpecList userNoiseList userTauList specList noiseList tauList
   global tauCount listName tabName hiName evolveDim fitArgs
   global sTab eList vList ppmList 

   global currentLoc currentID currentAss orient
   global minHi maxHi relFlag revFlag graphFitFlag autoFitFlag exitFlag exitFitFlag verbose

   set listName nlin.spec.list
   set tabName  nlin.tab
   set hiName   HEIGHT
   set fitArgs  ""
   set orient   ne

   set userSpecList  None
   set userNoiseList None
   set userTauList   None
   set evolveDim     Z

   set specList  ""
   set tauList   ""
   set noiseList ""

   set eList     ""
   set vList     ""
   set ppmList   ""

   set tauCount 0
   set sTab     0 
   set minHi    0.0
   set maxHi    0.0
   set revFlag  0

   set relFlag      1
   set exitFlag     0
   set exitFitFlag  1
   set autoFitFlag  0
   set graphFitFlag 1

   set currentLoc   0
   set currentID   -1
   set currentAss   None

   set verbose     0
}

proc initArgs {} \
{
   global orient fitArgs revFlag autoFitFlag exitFlag graphFitFlag exitFitFlag verbose ARGV ARGV
   global hiName listName tabName userSpecList userNoiseList userTauList 

   if {[flagLoc $ARGV -help]} \
      {
       puts "Display Evolution Curves from a Series Table:"
       puts " -in     inName    \[nlin.tab\]       Series Table Input."
       puts " -noise  noiseList \[None\]           Noise Values for Series."
       puts " -tau    tauList   \[None\]           Tau Values for Series."
       puts " -var    vName     \[HEIGHT\]         Scaling Variable in Table."
       puts " -orient orient    \[NW\]             Location on Screen: N S E W NW NE SW SE."
       puts " -rev                               Reverse X-Axis."
       puts " -verb                              Verbose Mode ON."
       puts "Corresponding Spectral Data, Use Only One:"
       puts " -spec  specList  \[None\]            List of Spectra Names."
       puts " -list  listName  \[nlin.spec.list\]  File Containing Name List."
       puts "XY Fitting Options:"
       puts " -exitFit         Send Exit Commands to fitXY.tcl (Default)."
       puts " -noexitFit       No Exit Commands to fitXY.tcl"
       puts " -autoFit         Automatically Fit all Curves via fitXY.tcl."
       puts " -noautoFit       Manually Invoke fitXY.tcl via 'Fit' Button."
       puts " -nograph         Suppress fitXY Graphics."
       puts " -exit            Exit This Application After Auto Fit."
       puts "XY Fitting Arguments (Should be Last Arguments to showEvolve.tcl):"
       puts " -fitArgs fArgs   \[-exp ...\]        Arguments to fitXY.tcl" 
       exit 1
      }

   set loc [flagLoc $ARGV -fitArgs]

   if {$loc} \
      {
       set fitArgs [lrange $ARGV [expr $loc+1] end]
       set ARGV    [lrange $ARGV 0 [expr $loc-1]]
       set ARGC    [llength $ARGV]
      }

   getArgD $ARGV -list   listName 
   getArgD $ARGV -in     tabName
   getArgD $ARGV -var    hiName
   getArgD $ARGV -orient orient

   getListArgD $ARGV -spec  userSpecList 
   getListArgD $ARGV -noise userNoiseList
   getListArgD $ARGV -tau   userTauList

   if {[flagLoc $ARGV -rev]}       {set revFlag     1}

   if {[flagLoc $ARGV -autoFit]}   {set autoFitFlag 1}
   if {[flagLoc $ARGV -noautoFit]} {set autoFitFlag 0}

   if {$autoFitFlag} {set exitFitFlag 0}

   if {[flagLoc $ARGV -graph]}     {set graphFitFlag 1}
   if {[flagLoc $ARGV -nograph]}   {set graphFitFlag 0}

   if {[flagLoc $ARGV -exit]}      {set exitFlag 1}
   if {[flagLoc $ARGV -noexit]}    {set exitFlag 0}

   if {[flagLoc $ARGV -exitFit]}   {set exitFitFlag 1}
   if {[flagLoc $ARGV -noexitFit]} {set exitFitFlag 0}

   if {[flagLoc $ARGV -verb]}   {set verbose 1}
   if {[flagLoc $ARGV -noverb]} {set verbose 0}
}

proc getSpecList {} \
{
   global listName specList userSpecList
  
   set specList ""

   if {[strcasecmp $userSpecList None]} \
      {
       set specList $userSpecList
      } \
   else \
      {
       if {![strcasecmp $listName None]} return

       set error [catch {set inUnit [open $listName r]}]

       if {$error} \
          {
           puts "Error Opening Spectral List File $listName"
           exit 1
          }

       while {[gets $inUnit line] >= 0} \
          {
           set thisName [string trim $line]
           if {![string length $thisName]} continue
           lappend specList  $thisName
          }

       close $inUnit
      }

   foreach thisName $specList \
      {
       if {![file exists $thisName]} \
          {
           puts "Error Finding Spectrum $thisName"
           exit 1
          } 
      }
}

proc getNoiseList {} \
{
   global specList noiseList userNoiseList tauCount verbose

   if {[strcasecmp $userNoiseList None]} \
      {
       set noiseList $userNoiseList
      }

   if {[llength $noiseList] == 1} \
      {
       set noise     [lindex $noiseList 0]
       set noiseList ""

       for {set i 0} {$i < $tauCount} {incr i} {lappend noiseList $noise}
      } \
   elseif {![llength $noiseList]} \
      {
       if {![llength $specList]} \
          {
           for {set i 0} {$i < $tauCount} {incr i} \
              {
               lappend noiseList 1.0
              }
          } \
       else \
          {
           foreach thisName $specList \
              {
               if {$verbose} \
                  {
                   puts -nonewline "Estimating Noise for Spectrum $thisName"
                   flush stdout
                  }

               set noise [estSpecNoise $thisName]
               lappend noiseList $noise

               if {$verbose} {puts ": [format %.4e $noise]"}
              }

           if {$verbose} {puts ""}
          } 
      }

   if {[llength $noiseList] != $tauCount} \
      {
       puts "Error: Mismatched Noise List and Tau Count."
       puts $noiseList
       puts $tauCount
       exit 1
      }
}

proc getTauList {} \
{
   global specList tauList userTauList tauCount verbose

   if {[strcasecmp $userTauList None]} \
      {
       set tauList $userTauList
      }

   if {![llength $tauList]} \
      {
       if {![llength $specList]} \
          {
           for {set i 0} {$i < $tauCount} {incr i} {lappend tauList $i}
          } \
       else \
          {
           set zeroCount 0

           foreach thisName $specList \
              {
               if {$verbose} \
                  {
                   puts -nonewline "Extracting Tau from Spectrum $thisName"
                   flush stdout
                  }

               set tau [getTau $thisName]
               lappend tauList $tau 

               if {$verbose} {puts ": [format %.4e $tau]"}
               if {$tau == 0.0} {incr zeroCount}
              }

           if {$verbose} {puts ""}

           if {$zeroCount == [llength $specList]} \
              {
               if {$verbose} {puts "Reseting All-Zero Tau Values."}

               set tauCount [llength $specList]
               set tauList  ""

               for {set i 0} {$i < $tauCount} {incr i} {lappend tauList $i}
              }
          } 
      }

   if {[llength $tauList] != $tauCount} \
      {
       puts "Error: Mismatched Tau List and Tau Count."
       exit 1
      }
}

proc getSeriesInfo {} \
{
   global tabName sTab eList vList ppmList tauCount evolveDim 

   set sTab   [gdbRead table -in $tabName]
   set vList  [gdbGetInfo $sTab -parms varNames]
   set eList  [gdbGetInfo $sTab -parms entries]
   set n      [llength $vList]

   set tauCount 0

   for {set i 0} {$i <= $n} {incr i} \
      {
       if {[lsearch -exact $vList A_A$i] < 0} break
       set evolveDim A
       set ppmList {X_PPM Y_PPM Z_PPM}
       incr tauCount
      }

   if {$tauCount} return

   for {set i 0} {$i <= $n} {incr i} \
      {
       if {[lsearch -exact $vList Z_A$i] < 0} break 
       set evolveDim Z
       set ppmList {X_PPM Y_PPM}
       incr tauCount
      }

   if {$tauCount} return

   for {set i 0} {$i <= $n} {incr i} \
      {
       if {[lsearch -exact $vList Y_A$i] < 0} break 
       set evolveDim Y
       set ppmList {X_PPM}
       incr tauCount
      }
}

proc getSeriesData {} \
{
   global sTab eList tauList tauCount noiseList idList idCount hiName sInfo
   global evolveDim currentID currentAss currentLoc
   global minHi maxHi noiseList ppmList

   set hiList ""
   set idList ""

   foreach entry $eList \
      {
       set id  [gdbGet $entry INDEX]
       set ass [gdbGet $entry ASS]

       lappend idList $id

       foreach p $ppmList {set sInfo($id,$p) [gdbGet $entry $p]}

       set sInfo($id,hi)  [gdbGet $entry $hiName]
       set sInfo($id,ass) $ass

       set xyList ""

       for {set i 0} {$i < $tauCount} {incr i} \
          {
           set val [gdbGet $entry ${evolveDim}_A$i]
           set tau [lindex $tauList $i]

           lappend xyList "$tau $val $i"
           lappend hiList [expr $sInfo($id,hi)*$val]
          }

       set sInfo($id,xyList) [lsort -command xSort $xyList]
      }

   set idCount [llength $idList]

   if {!$idCount} \
      {
       puts "Error: No data was found."
       exit
      }

   set maxNoise [getMax $noiseList]

   set minHi [expr [getMin $hiList] - $maxNoise]
   set maxHi [expr [getMax $hiList] + $maxNoise] 

   set currentLoc 0
   set currentID  [lindex $idList $currentLoc]
   set currentAss $sInfo($id,ass)
}

proc exitProc {} \
{
   global exitFitFlag

   if {$exitFitFlag} exitFitXY

   exit 0
}

proc initWin {} \
{
   global currentLoc currentID currentAdd tabName hiName orient relFlag revFlag
   global axisfont

   setFonts

   set xSize 500
   set ySize 350
           
   switch [string tolower $orient] \
      {
       n  {wm geom . +400+20}
       s  {wm geom . +400-20}
       e  {wm geom . -20+400}
       w  {wm geom . +20+400}
       nw {wm geom . +20+20}
       ne {wm geom . -20+20}
       se {wm geom . -20-20}
       sw {wm geom . +20-20}
      }

   graph .g -height $ySize -width $xSize -plotbackground black -title $tabName 

   .g axis configure x -tickfont $axisfont -ticklength 0.05i -title Tau     -descending $revFlag
   .g axis configure y -tickfont $axisfont -ticklength 0.05i -title $hiName

   .g legend configure -hide 1

   frame .cmnd1
   frame .cmnd2

   radiobutton .rel -text "Relative Scale" \
     -value 1 -variable relFlag -anchor w -command updateDisplay

   radiobutton .abs -text "Absolute Scale" \
     -value 0 -variable relFlag -anchor w -command updateDisplay

   button .next -text "Next" -command "incrID  1"
   button .prev -text "Prev" -command "incrID -1"
   button .fit  -text "Fit"  -command "fitXY"
   button .quit -text "Quit" -command "exitProc"

   label .idLab -text "ID: "
   entry .idEntry -textvariable currentID

   label .assLab -text "ASS: "
   entry .assEntry -textvariable currentAss

   pack .rel .abs -in .cmnd1 -side left

   pack .next .prev .fit .idLab .idEntry .assLab .assEntry \
        -in .cmnd2 -side left -fill x

   pack .quit -in .cmnd2 -side right

   pack .g     -in . -side top -expand 1 -fill both
   pack .cmnd1 -in . -side top 
   pack .cmnd2 -in . -side top -fill x

   bind  .idEntry   <Return> "+ getNewID"
   bind  .assEntry  <Return> "+ getNewAss"

   update
}

proc incrID { off } \
{
   global idCount currentLoc

   incr currentLoc $off
   
   if {$currentLoc < 0} {set currentLoc [expr $idCount - 1]}
   if {$currentLoc > $idCount - 1} {set currentLoc 0}

   updateDisplay
}

proc getNewID {} \
{
   global idCount idList currentLoc currentID

   set newLoc -1

   for {set i [expr $currentLoc + 1]} {$i < $idCount} {incr i} \
      {
       set id [lindex $idList $i] 

       if {$id == $currentID} \
          {
           set newLoc $i
           break
          }          
      }

   if {$newLoc >= 0} \
      {
       set currentLoc $newLoc
       updateDisplay
       return
      }

   for {set i 0} {$i < $currentLoc} {incr i} \
      {
       set id [lindex $idList $i] 

       if {$id == $currentID} \
          {          
           set newLoc $i
           break
          }
      }

   if {$newLoc >= 0} \
      {
       set currentLoc $newLoc
       updateDisplay
       return
      }

   updateDisplay
}

proc getNewAss {} \
{
   global idCount idList currentLoc currentID currentAss sInfo

   set newLoc -1

   for {set i [expr $currentLoc + 1]} {$i < $idCount} {incr i} \
      {
       set id  [lindex $idList $i]
       set ass $sInfo($id,ass)

       if {![strcasecmp $ass $currentAss]} \
          {
           set newLoc $i
           break
          }         
      }

   if {$newLoc >= 0} \
      {
       set currentLoc $newLoc
       updateDisplay
       return
      }

   for {set i 0} {$i < $currentLoc} {incr i} \
      {
       set id [lindex $idList $i]
       set ass $sInfo($id,ass)

       if {![strcasecmp $ass $currentAss]} \
          {        
           set newLoc $i
           break
          }
      }

   if {$newLoc >= 0} \
      {
       set currentLoc $newLoc
       updateDisplay
       return
      }

   updateDisplay
}

proc updateDisplay {} \
{
   global idList currentLoc currentID currentAss noiseList sInfo
   global minHi maxHi relFlag

   set currentID  [lindex $idList $currentLoc]
   set currentAss $sInfo($currentID,ass)

   catch ".g element delete evolution"
   catch ".g marker delete zero"

   set x ""
   set y ""
   set s ""

   set i  0
   set pN 0
   set nN 0

   foreach xy $sInfo($currentID,xyList) \
      {
       catch ".g marker delete m$i"

       set tau [lindex $xy 0]
       set hi  [lindex $xy 1]
       set loc [lindex $xy 2]

       set noise [lindex $noiseList $loc] 
       set v     [expr $hi*$sInfo($currentID,hi)]

       lappend x $tau
       lappend y $v
       lappend s $noise

       if {$v >= 0.0} {incr pN}
       if {$v < 0.0}  {incr nN}

       set y1 [expr $v + $noise]
       set y2 [expr $v - $noise]

       .g marker create line -coords "$tau $y1 $tau $y2" -linewidth 2 \
                             -outline gray -under yes -name m$i

       incr i
      }

   set sInfo(xList) $x
   set sInfo(yList) $y
   set sInfo(sList) $s

   if {$relFlag} \
      {
       if {$nN == 0} \
          {
           .g axis configure y -min 0.0 -max ""
          } \
       elseif {$pN == 0} \
          {
           .g axis configure y -min "" -max 0.0 
          } \
       else \
          {
           .g axis configure y -min "" -max ""
          }
      } \
   else \
      {
       .g axis configure y -min $minHi -max $maxHi
      }

   .g element create evolution \
      -xdata $x -ydata $y -color yellow -fill white -pixels 3 -linewidth 2 

   .g marker create line -coords "-Inf 0.0 Inf 0.0" -name zero \
                         -outline gray -under yes

   update
}

proc exitFitXY {} \
{
   foreach i [winfo interps] \
      {
       if {![strcmp [lindex $i 0] fitXY.tcl]} \
          {
           catch "send {$i} exit"
          }
      }
}

proc getFitTitle { id } \
{
   global sInfo ppmList hiName

   set str "Peak $id Ass $sInfo($id,ass)" 

   foreach p $ppmList {append str [format " %.3fppm" $sInfo($id,$p)]}

   append str " $hiName $sInfo($id,hi)"

   return $str 
}

proc fitXY {} \
{
   global sInfo currentID tabName hiName fitArgs graphFitFlag exitFitFlag autoFitFlag verbose

   set    cmnd "fitXY.tcl $fitArgs -anchor NE"
   append cmnd " -x $sInfo(xList) -y $sInfo(yList) -noise $sInfo(sList)"
   append cmnd " -id $currentID -xLab Tau -yLab $hiName"
   append cmnd " -title [getFitTitle $currentID]"

   if {!$graphFitFlag} {append cmnd " -notk"}
   if {!$autoFitFlag}  {append cmnd " &"}

   set info ""

   if {$verbose} \
      {
       puts ""
       puts $cmnd
      }

   if {$exitFitFlag} exitFitXY 

   if {$verbose} \
      {
       puts ""
       puts $cmnd
       puts ""
      }

   catch {set info [eval "exec $cmnd"]}
}

proc autoFitEvolve {} \
{
   global exitFlag autoFitFlag idCount

   if {!$autoFitFlag} return

   for {set i 0} {$i < $idCount} {incr i} \
      {
       fitXY
       incrID 1
      }

   if {$exitFlag} exitProc
}

proc xSort { xy1 xy2 } \
{
   set t1 [lindex $xy1 0]
   set t2 [lindex $xy2 0]

   if {$t1 > $t2} {return  1}
   if {$t1 < $t2} {return -1}
   return 0
}

initVars
initArgs
getSeriesInfo
getSpecList
getNoiseList
getTauList
getSeriesData
initWin
updateDisplay
autoFitEvolve
