#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

#
# Display and adjust components of a 1D simulation.
#--------------------------------------------------------------------

set thisVersion "2022.119.13.29"

proc initArgs {} \
{
   global objInfo

   set objInfo(inTabName)   test.tab
   set objInfo(fidName)     test.fid
   set objInfo(specName)    test.ft1
   set objInfo(baseName)    adjsim

   if {[flagLoc $ARGV -help]} \
      {
       puts "Display and adjust components of a 1D simulation ($thisVersion):"
       puts " -in    inTab    \[test.tab\]   Input Peak Table Table."
       puts " -fid   fidName  \[test.fid\]   Input Time-Domain Data."
       puts " -ft    ftName   \[test.ft1\]   Input Spectrum."
       puts " -base  baseName \[adjsim\]     Base Name for Outputs (Table, Log, Time- and Freq-Domain)."
       puts "Notes:"
       puts " 1. Outputs will have the form:"
       puts "    adjsim.fid adjsim_001.fid ... adjsim.ft1 adjsim_001.ft1 adjsim.log adjsim.tab"
       exit 0
      }
 
   getArgD $ARGV -in   objInfo(inTabName)
   getArgD $ARGV -fid  objInfo(inFIDName)
   getArgD $ARGV -ft   objInfo(inFTName)
   getArgD $ARGV -base objInfo(baseName)

   set objInfo(outTabName)     $objInfo(baseName).tab
   set objInfo(outFIDName)     $objInfo(baseName).fid
   set objInfo(outFTName)      $objInfo(baseName).ft1
   set objInfo(logName)        $objInfo(baseName).log

   set objInfo(outFIDTemplate) $objInfo(baseName)_%03d.fid
   set objInfo(outFTTemplate)  $objInfo(baseName)_%03d.ft1

   set objInfo(pkTab)   0
   set objInfo(pkCount) 0
   set objInfo(eList)   ""
}

proc initInputs {} \
{
   global objInfo

   if {![file exists $objInfo(inTabName)]} \
      {
       puts "Error Finding Input Peak Table $objInfo(inTabName)"
       exit 1
      }

   set objInfo(pkTab)   [gdbRead table -in $objInfo(inTabName)]
   set objInfo(eList)   [gdbGetInfo $objInfo(pkTab) -parms entries]
   set objInfo(pkCount) [llength $objInfo(eList)]

   set roiID 0 

   incr roiID
   set objInfo(fid,roiID) $roiID
   
}

#
# Extract options from command line.

proc initArgs {} \
{
   global ARGC ARGV
   global inName apodName

   if {[flagLoc $ARGV -help]} \
      {
       puts "Demo of Apodization Window Display for 2D FID data:"
       puts " apod.tcl fidName \[apodName\]" 
       exit 0
      }

   if {$ARGC > 1} {set inName   [lindex $ARGV 1]}
   if {$ARGC > 2} {set apodName [lindex $ARGV 2]}

   if {![strcasecmp $inName None]} \
      {
       if {[file exists xz001.fid]} {set inName xz001.fid}
       if {[file exists test.fid]}  {set inName test.fid}

       if {[file exists fid/test001.fid]}  {set inName fid/test001.fid}
       if {[file exists data/test001.fid]} {set inName data/test001.fid}
      }

   if {![strcasecmp $inName None]} \
      {
       puts "Error Finding Input FID:"
       puts " apod.tcl fidName \[apodName\]" 
       exit 0
      }

   set apodName [string toupper $apodName]
}

#
# Draw the current 2D FID.

proc draw2D {} \
{
   drawROI -roi 1 -cell 1 -dot -hi 25% -bg black -color1 blue -color2 dimGray -axis Bottom Right
}

#
# Draw the current 1D vector. 

proc draw1D {} \
{
  global minVal maxVal

  drawROI -roi 4 -cell 2 -bg black -fg red    -fhi $minVal -lhi $maxVal
  drawROI -roi 3 -cell 2 -bg None  -fg yellow -fhi $minVal -lhi $maxVal -zero

  update
}

#
# Move the 1D cursor.

proc moveCursor { id ix iy } \
{
   global cx1 cxn cy1 cyn

   if {$ix < $cx1 || $ix > $cxn} return
   if {$iy < $cy1 || $iy > $cyn} return

   .c coords $id $cx1 $iy $cxn $iy

   select1D $iy
   draw1D
}

#
# Vertical scale adjustment for the 1D display.

proc scale1D { iy } \
{
   global minVal maxVal iyPrev

   if {$iy > $iyPrev} \
      {
       set minVal [expr $minVal/0.9]
       set maxVal [expr $maxVal/0.9]
      } \
   else \
      {
       set minVal [expr $minVal*0.9]
       set maxVal [expr $maxVal*0.9]
      }

   draw1D

   set iyPrev $iy
}

#
# Vertical offset adjustment for the 1D display.

proc move1D { iy } \
{
   global minVal maxVal iyPrev

   set range [expr $maxVal - $minVal]

   if {$iy > $iyPrev} \
      {
       set minVal [expr $minVal + $range*0.05]
       set maxVal [expr $maxVal + $range*0.05]
      } \
   else \
      {
       set minVal [expr $minVal - $range*0.05]
       set maxVal [expr $maxVal - $range*0.05]
      }

   draw1D

   set iyPrev $iy
}

#
# Creation of Tk windows:

proc initWindows {} \
{
   global q1  q1Min  q1Max  q1Name q2  q2Min  q2Max  q2Name q3   q3Min   q3Max  q3Name
   global elb elbMin elbMax        glb glbMin glbMax        goff goffMin goffMax
   global q1Res q2Res q3Res elbRes glbRes goffRes

   global apodName xSize1D ySize1D xSize2D ySize2D
   global xOff yOff

   toplevel .t
   toplevel .a
   toplevel .f

   canvas   .c   -width $xSize2D -height $ySize2D -border 0 -closeenough 25
   canvas   .t.c -width $xSize1D -height $ySize1D -border 0
   canvas   .a.c -width $xSize1D -height $ySize1D -border 0
   canvas   .f.c -width $xSize1D -height $ySize1D -border 0

#
# Main window and 1D viewing window:

   frame    .cmndFrame
   button   .bQuit -text  Quit -command exit
   pack     .c .bQuit -side top -fill x -expand 1

   pack     .t.c
   pack     .f.c

#
# Apodize window:

   frame   .a.f1
   frame   .a.f2
   frame   .a.f3
   frame   .a.f4
   frame   .a.f5
   frame   .a.f6

   set hi   [expr 1 + $xOff/3]
   set wide [expr $xSize1D - 6*$xOff]

   label    .a.l1 -width 12 -anchor e -text "$q1Name:"

   entry    .a.e1 -width 8  -textvariable q1

   scale    .a.q1 -resolution $q1Res -width $hi -length $wide -showvalue 0 \
                  -from $q1Min -to $q1Max -orient horiz -command "setApod q1" \
                  -variable q1

   pack     .a.l1 .a.e1 .a.q1 -in .a.f1 -side left

   label    .a.l2 -width 12 -anchor e -text "$q2Name:"

   entry    .a.e2 -width 8  -textvariable q2 

   scale    .a.q2 -resolution $q2Res -width $hi -length $wide -showvalue 0 \
                  -from $q2Min -to $q2Max -orient horiz -command "setApod q2" \
                  -variable q2

   pack     .a.l2 .a.e2 .a.q2 -in .a.f2 -side left

   label    .a.l4 -width 12 -anchor e -text "ELB:"

   entry    .a.e4 -width 8  -textvariable elb 

   scale    .a.q4 -resolution $elbRes -width $hi -length $wide -showvalue 0 \
                  -from $elbMin -to $elbMax -orient horiz -command "setApod elb" \
                  -variable elb

   pack     .a.l4 .a.e4 .a.q4 -in .a.f4 -side left

   label    .a.l5 -width 12 -anchor e -text "GLB:"

   entry    .a.e5 -width 8  -textvariable glb 

   scale    .a.q5 -resolution $glbRes -width $hi -length $wide -showvalue 0 \
                  -from $glbMin -to $glbMax -orient horiz -command "setApod glb" \
                  -variable glb

   pack     .a.l5 .a.e5 .a.q5 -in .a.f5 -side left

   label    .a.l6 -width 12 -anchor e -text "GOFF:"

   entry    .a.e6 -width 8  -textvariable goff

   scale    .a.q6 -resolution $goffRes -width $hi -length $wide -showvalue 0 \
                  -from $goffMin -to $goffMax -orient horiz -command "setApod goff" \
                  -variable goff

   pack     .a.l6 .a.e6 .a.q6 -in .a.f6 -side left

   label    .a.l3 -width 12 -anchor e -text "$q3Name:"

   bind .a.e1 <Return> "+ setApod q1"
   bind .a.e2 <Return> "+ setApod q2"

   if {![strcmp $apodName SP]} \
      {
       set c showApod

       radiobutton .a.q13 -text 1 -value 1 -variable q3 -anchor w -command $c 
       radiobutton .a.q23 -text 2 -value 2 -variable q3 -anchor w -command $c
       radiobutton .a.q33 -text 3 -value 3 -variable q3 -anchor w -command $c 

       pack        .a.l3 .a.q13 .a.q23 .a.q33  -in .a.f3 -side left
   
      } \
   elseif {![strcmp $apodName GM]} \
      {
       entry    .a.e3 -width 8  -textvariable q3

       scale    .a.q3 -resolution $q3Res -width $hi -length $wide -showvalue 0 \
                      -from $q3Min -to $q3Max -orient horiz \
                      -command "setApod q3" -variable q3

       pack     .a.l3 .a.e3 .a.q3 -in .a.f3 -side left

       bind .a.e3 <Return> "+ setApod q3"
      } \
   elseif {![strcmp $apodName GMB]} \
      {
      }
 
   pack     .a.c .a.f1 .a.f2 .a.f4 .a.f5 .a.f6 .a.f3 -side top

#
# Position each window on screen:

   wm geometry  . +10+10
   wm title     . {2D Window [Left: Select 1D]}

   wm geometry .t +$xOff+$yOff
   wm title    .t {1D Window [Left: Adjust Scale] [Middle: Adjust Offset]}

   wm geometry .a -$xOff+10
   wm title    .a "$apodName Time Domain"

   wm geometry .f -$xOff+460
   wm title    .f "$apodName Frequency Domain"

   raise .

   update
}

#
# Create pixmaps for spectral rendering in the windows.
# Define drawing areas in each of the windows.

proc initSpectralDrawing {} \
{
   global xSize1D ySize1D xSize2D ySize2D
   global cx1 cxn cy1 cyn qx1 qxn qy1 qyn

   .c   create pixmap 0 0 -width $xSize2D -height $ySize2D -win 1
   .t.c create pixmap 0 0 -width $xSize1D -height $ySize1D -win 2
   .a.c create pixmap 0 0 -width $xSize1D -height $ySize1D -win 3
   .f.c create pixmap 0 0 -width $xSize1D -height $ySize1D -win 4

   defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 1 -win 1
   defineCell -x $qx1 $qxn -y $qy1 $qyn -cell 2 -win 2
   defineCell -x $qx1 $qxn -y $qy1 $qyn -cell 3 -win 3
   defineCell -x $qx1 $qxn -y $qy1 $qyn -cell 4 -win 4

   newScreen

   update
}

#
# Read the 2D spectral data; ROI 1 and 2 are the real and imaginary channels.
# Get the min and max values of the data.

proc initSpectralData {} \
{
   global minVal maxVal specXSize specYSize sw inName

   readROI -ndim 2 -roi 1 -in $inName -x X_AXIS -y Y_AXIS -real -verb
   readROI -ndim 2 -roi 2 -in $inName -x X_AXIS -y Y_AXIS -imag -verb

   set roiInfo [getROIInfo -roi 1]
   set rdata   [lindex $roiInfo 0]
   set fdata   [lindex $roiInfo 1]
   set sInfo   [lindex $roiInfo 2]

   set specXSize 1
   set specYSize 1

   if {[llength $sInfo] > 0} {set specXSize [lindex $sInfo 0]}
   if {[llength $sInfo] > 1} {set specYSize [lindex $sInfo 1]}

   set n       [expr $specXSize*$specYSize]

   set minVal  [vMin $rdata $n]
   set maxVal  [vMax $rdata $n]

   set sw      [getParm $fdata NDSW CUR_XDIM]
}

#
# Read the selected 1D vector; ROI 3 and 4 are the real and imaginary channels.

proc select1D { iy } \
{
  global cx1 cxn cy1 cyn

  set yLoc [expr 100.0*($cyn - $iy)/($cyn - $cy1)]%

  readROI -roi 3 -ndim 1 -inROI 1 -x X_AXIS -dy Y_AXIS $yLoc
  readROI -roi 4 -ndim 1 -inROI 2 -x X_AXIS -dy Y_AXIS $yLoc
}

#
# Initialize window function parameter limits.

proc initApodParams {} \
{
   global q1 q1Min q1Max q1Name q2 q2Min q2Max q2Name q3 q3Min q3Max q3Name
   global elbMin elbMax glbMin glbMax goffMin goffMax
   global q1Res q2Res q3Res elbRes glbRes goffRes

   global apodName sw

   set elb      0.0
   set elbMin   0.0
   set elbMax  50.0
   set elbRes   0.1

   set glb      0.0
   set glbMin   0.0
   set glbMax  50.0
   set glbRes   0.1

   set goff    0.0
   set goffMin 0.0
   set goffMax 1.0
   set goffRes 0.01

   switch $apodName \
      {
       SP {
           set q1     0.50
           set q1Res  0.001
           set q1Min  0.00
           set q1Max  1.00
           set q1Name Offset 

           set q2     0.95
           set q2Res  0.001
           set q2Min  0.00
           set q2Max  1.00
           set q2Name Endpoint

           set q3     2
           set q3Res  1
           set q3Min  1.00
           set q3Max  3.00
           set q3Name Power
          }

       GM {
           set q1       10.0
           set q1Res    0.01
           set q1Min  -20.00
           set q1Max  100.00
           set q1Name Exponential

           set q2      20.00
           set q2Res    0.01
           set q2Min  -20.00
           set q2Max  200.00
           set q2Name Gaussian

           set q3     0.000 
           set q3Res  0.001
           set q3Min  0.000
           set q3Max  1.000
           set q3Name Center
          }

       GMB {
            set q1      -10.0
            set q1Res    0.01
            set q1Min -100.00
            set q1Max   50.00
            set q1Name Exponential

            set q2       0.250
            set q2Res    0.001
            set q2Min    0.000
            set q2Max    1.000
            set q2Name Gaussian-B

            set q3     0.000
            set q3Res  0.001
            set q3Min  0.000
            set q3Max  1.000
            set q3Name None 
           }
      }
}

#
# Initialize the window function buffer; ROI 5.

proc initApodData {} \
{
   global apodName apodData fdata

   readROI -roi 5 -ndim 1 -inROI 1 -x X_AXIS -dy Y_AXIS 1

   set roiInfo  [getROIInfo -roi 5]
   set apodData [lindex $roiInfo 0]
   set fdata    [lindex $roiInfo 1]

   showApod
}

#
# Create the window function.

proc applyApod {} \
{
   global apodName apodData fdata specXSize q1 q2 q3 elb glb goff

   set rdata 0
   set idata 0

   set aPtr [nmrInit $fdata rdata idata -fn APOD -qName $apodName -q1 $q1 -q2 $q2 -q3 $q3 -elb $elb -glb $glb -goff $goff]

   vsSet $rdata 1.0 $specXSize
 
   nmrExec $aPtr

   vvMove $apodData $rdata $specXSize

   nmrFree $aPtr

   writeROI -roi 5 -out apodT.dat -ov

   exec nmrPipe -in apodT.dat -fn MULT -c 0.5 -xn 1 | nmrPipe -fn ZF -zf 4 -auto -ai | nmrPipe -fn FT -di -out apodF.dat -ov

   readROI -ndim 1 -roi 7 -x X_AXIS 49% 51% -in apodF.dat 
}

#
# Create the 1D cursor.
# Draw the initial 1D vector.

proc initInteractiveGraphics {} \
{
   global cx1 cxn cy1 cyn minVal maxVal
   global iyInit iyPrev

   set iyInit [expr 0.5*($cy1 + $cyn)]
   set iyPrev $iyInit

   set id [.c create line $cx1 $iyInit $cxn $iyInit -fill yellow]
   .c bind $id <B1-Motion> "moveCursor $id %x %y"

   bind .t.c <B1-Motion> "scale1D %y"
   bind .t.c <B2-Motion> "move1D  %y"

   update
}

#
# Adjustment of apodize parameters

proc setApod args \
{
   global apodData

   if {![strcasecmp $apodData None]} return

   showApod
}

#
# Calculation and display of current apodize function.

proc showApod {} \
{
   global apodName

   applyApod

   set minVal  0.0
   set maxVal  1.0

   set thisMin  [getROIStat 5 vMin]
   set thisMax  [getROIStat 5 vMax]

   if {$thisMin < $minVal} {set minVal $thisMin}
   if {$thisMax > $maxVal} {set maxVal $thisMax}
   
   drawROI -roi 5 -cell 3 -zero -bg black -fg yellow -fhi $minVal -lhi $maxVal
   drawROI -roi 7 -cell 4 -zero -bg black -fg cyan -fhi -20% -lhi 110% 

   wm title .a [format "%s Time Domain Min: %.4f Max: %.4f" $apodName $thisMin $thisMax]

   set thisMin  [getROIStat 7 vMin]
   set thisMax  [getROIStat 7 vMax]

   if {$thisMax != 0.0} \
      {
       set thisMin [expr $thisMin/$thisMax]
       set thisMax 1.0
      }

   wm title .f [format "%s Freq Domain Min: %.4f Max: %.4f" $apodName $thisMin $thisMax]

   update
}

initVals
initArgs

initSpectralData
initApodParams

initWindows
initSpectralDrawing
initInteractiveGraphics
initApodData

select1D $iyInit

draw1D
draw2D
