#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

proc initVars {} \
{
   global eList vList pcList lcList minVal maxVal relFlag cCount
   global sortFlag tabName fitArgs orient noise evolveVar evolveCoef
   global tTab currentLoc currentID currentAss verbose

   set tabName    titrView.tab
   set fitArgs    ""
   set orient     nw

   set noise      0.002
   set evolveVar  X_PPM
   set evolveCoef 1.0

   set pcList     ""
   set lcList     ""
   set eList      ""
   set vList      ""

   set cCount    0
   set tTab      0 

   set minVal   0.0
   set maxVal   0.0
   set relFlag  0
   set sortFlag 1

   set currentLoc  0
   set currentID  -1
   set currentAss  None

   set verbose     0
}

proc initArgs {} \
{
   global tabName orient xNoise yNoise evolveVar evolveCoef evolveTitle
   global ARGV ARGV sortFlag verbose fitArgs

   if {[flagLoc $ARGV -help]} \
      {
       puts "Display Evolution Curves from a Titration Table:"
       puts " -in     inName    \[titrView.tab\] Titration Table Input."
       puts " -var    vNameList \[X_PPM\]        Evolution Variables in Table."
       puts " -c      cList     \[1.0\]          Evolution Value Scale Factors."
       puts " -noise  noise     \[0.002\]        Noise Value for Variable."
       puts " -sort                              Sort by Max Range (Default)."
       puts " -nosort                            No Sort."
       puts " -verb                              Verbose Mode ON."
       puts "XY Fitting Option (Should be Last on Command Line):"
       puts " -fitArgs fArgs   \[-titr ...\]       Arguments to fitXY.tcl" 
       exit 1
      }

   set loc [flagLoc $ARGV -fitArgs]

   if {$loc} \
      {
       set fitArgs [lrange $ARGV [expr $loc+1] end]
       set ARGV    [lrange $ARGV 0 [expr $loc-1]]
       set ARGC    [llength $ARGV]
      }

   getArgD $ARGV -in    tabName
   getArgD $ARGV -noise noise 

   getListArgD $ARGV -var evolveVar 
   getListArgD $ARGV -c   evolveCoef

   if {[llength $evolveVar] != [llength $evolveCoef]} \
      {
       puts stderr "Error: Mismatch in Evolve Variable/Coefficient List."
       exit 1
      }

   if {![file exists $tabName]} \
      {
       puts stderr "Error Finding Input Table $tabName"
       exit 1
      }

   if {[flagLoc $ARGV -sort]}   {set sortFlag 1}
   if {[flagLoc $ARGV -nosort]} {set sortFlag 0}

   if {[flagLoc $ARGV -verb]}   {set verbose 1}
   if {[flagLoc $ARGV -noverb]} {set verbose 0}

   set n [llength $evolveVar]
   set j 0

   if {$n == 1} \
      {
       set c [lindex $evolveCoef 0]
       set v [lindex $evolveVar 0]

       if {$c == 1.0} \
          {
           set evolveTitle "$v"
          } \
       else \
          {
           set evolveTitle "$c*$v"
          } 
       } \
    else \
       { 
        set evolveTitle "Sqrt( "

        foreach v $evolveVar \
           {
            set c [lindex $evolveCoef $j]

            if {$j == 0} \
               {
                append evolveTitle "$c*$v^2" 
               } \
            else \
               {
                append evolveTitle "+ $c*$v^2" 
               }

            incr j
           }

       append evolveTitle " )"
      }
}

proc getSeriesInfo {} \
{
   global tabName tTab eList vList pcList lcList cCount sortFlag

   set tTab   [gdbRead table -in $tabName]
   set vList  [gdbGetInfo $tTab -parms varNames]
   set eList  [gdbGetInfo $tTab -parms entries]
   set n      [llength $vList]

   getKeyData $tTab tInfo

   if {![info exists tInfo(CONC_L)] || ![info exists tInfo(CONC_P)]} \
      {
       puts stderr "Error Finding CONC_L or CONC_P DATA in $tabName"
       exit 1
      }

   set pcList [string trim $tInfo(CONC_P) \{\}]
   set lcList [string trim $tInfo(CONC_L) \{\}]

   set cCount [llength $lcList]

   set newList ""

   foreach entry $eList \
      {
       set id [gdbGet $entry INDEX]
                                                                                
       for {set i 1} {$i <= $cCount} {incr i} \
          {
           set mode [gdbGet $entry MODE$i]
                                                                                
           if {![strcasecmp $mode None]} \
              {
               set id -1
               break
              }
          }

       if {$id < 0} continue

       lappend newList $entry
      }

   set eList $newList

   if {$sortFlag} \
      {
       set eList [lsort -command titrSort $eList] 
      }

   if {![llength $eList]} \
      {
       puts stderr "Notice: No Complete Titration Curves Found."
       exit 1
      }
}

proc titrSort { e1 e2 } \
{
   set r1 [titrRange $e1]
   set r2 [titrRange $e2]

   if {$r1 < $r2} {return  1}
   if {$r1 > $r2} {return -1}

   return 0
}

proc titrRange { entry } \
{
   global cCount

   set valList ""

   for {set i 1} {$i <= $cCount} {incr i} \
      {
       lappend valList [getTitrEvolveVal $entry $i]
      } 

   set minVal [getMin $valList]
   set maxVal [getMax $valList]

   set r [expr $maxVal - $minVal]

   return $r
}

proc getTitrEvolveVal { entry i } \
{
   global evolveVar evolveCoef
 
   set n [llength $evolveVar]
   set shift 0.0
   set j 0
 
   foreach titrAxis $evolveVar \
      { 
       set s [gdbGet $entry $titrAxis$i]
       set c [lindex $evolveCoef $j]

       if {$n == 1} \
          {
           set shift [expr $shift + $c*$s]
          } \
       else \
          {
           set shift [expr $shift + $c*$s*$s]
          }

       incr j
      }

    if {$n > 1} {set shift [expr sqrt( $shift )]}

    return $shift
}

proc getSeriesData {} \
{
   global eList cCount idList idCount sInfo lcList pcList
   global currentID currentAss currentLoc
   global minVal maxVal noise

   set allList ""
   set idList  ""

   foreach entry $eList \
      {
       set xPPM [gdbGet $entry X_PPM1]
       set yPPM [gdbGet $entry Y_PPM1]
       set id   [gdbGet $entry INDEX]
       set ass  [gdbGet $entry ASS]

       lappend idList $id

       set sInfo($id,ass) $ass

       set v0 [getTitrEvolveVal $entry 1]
       set nP 0
       set nN 0

       set valList ""
       set xyList  ""

       for {set i 1} {$i <= $cCount} {incr i} \
          {
           set v1  [getTitrEvolveVal $entry $i]
           set val [expr $v1 - $v0]

           if {$val >= 0.0} {incr nP}
           if {$val <  0.0} {incr nN}

           lappend valList $val
          }

       set pc0 [lindex $pcList 0]

       if {$pc0 == 0.0} {set pc0 1.0}

       for {set i 0} {$i < $cCount} {incr i} \
          {
           set val [lindex $valList $i]
           set lc  [lindex $lcList  $i]
           set pc  [lindex $pcList  $i]

           if {$pc == 0.0} {set pc 1.0}

           set conc [expr $pc0*$lc/$pc]
           
           if {$nN > $nP} {set val [expr -1.0*$val]}

           lappend xyList "$conc $val $i"
           lappend allList $val
          }

       set sInfo($id,xyList) [lsort -command xSort $xyList]
      }

   set idCount [llength $idList]

   if {!$idCount} \
      {
       puts "Error: No data was found."
       exit
      }

   set minVal [expr [getMin $allList] - $noise]
   set maxVal [expr [getMax $allList] + $noise] 

   set currentLoc 0
   set currentID  [lindex $idList $currentLoc]
   set currentAss $sInfo($id,ass)
}

proc initWin {} \
{
   global currentLoc currentID currentAdd tabName orient relFlag evolveTitle
 
   set axisFont "-*-lucida-bold-r-*-*-*-120-*-*-*-*-*-*"

   set xSize 500
   set ySize 350
           
   switch [string tolower $orient] \
      {
       nw {wm geom . +20+20}
       ne {wm geom . -20+20}
       se {wm geom . -20-20}
       sw {wm geom . +20-20}
      }

   graph .g -height $ySize -width $xSize -plotbackground black -title $tabName 

   .g axis configure x -tickfont $axisFont -ticklength 0.05i -title "Conc uM"
   .g axis configure y -tickfont $axisFont -ticklength 0.05i -title $evolveTitle

   .g legend configure -hide 1

   frame .cmnd1
   frame .cmnd2

   radiobutton .rel -text "Relative Scale" \
     -value 1 -variable relFlag -anchor w -command updateDisplay

   radiobutton .abs -text "Absolute Scale" \
     -value 0 -variable relFlag -anchor w -command updateDisplay

   button .next -text "Next" -command "incrID  1"
   button .prev -text "Prev" -command "incrID -1"
   button .fit  -text "Fit"  -command "fitXY"
   button .quit -text "Quit" -command "exit"

   label .idLab -text "ID: "
   entry .idEntry -textvariable currentID

   label .assLab -text "ASS: "
   entry .assEntry -textvariable currentAss

   pack .rel .abs -in .cmnd1 -side left

   pack .next .prev .fit .idLab .idEntry .assLab .assEntry \
        -in .cmnd2 -side left -fill x

   pack .quit -in .cmnd2 -side right

   pack .g     -in . -side top -expand 1 -fill both
   pack .cmnd1 -in . -side top 
   pack .cmnd2 -in . -side top -fill x

   bind  .idEntry   <Return> "+ getNewID"
   bind  .assEntry  <Return> "+ getNewAss"

   update
}

proc incrID { off } \
{
   global idCount currentLoc

   incr currentLoc $off

   if {$currentLoc < 0} {set currentLoc [expr $idCount - 1]}
   if {$currentLoc > $idCount - 1} {set currentLoc 0}

   updateDisplay
}

proc getNewID {} \
{
   global idCount idList currentLoc currentID

   set newLoc -1

   for {set i [expr $currentLoc + 1]} {$i < $idCount} {incr i} \
      {
       set id [lindex $idList $i] 

       if {$id == $currentID} \
          {
           set newLoc $i
           break
          }          
      }

   if {$newLoc >= 0} \
      {
       set currentLoc $newLoc
       updateDisplay
       return
      }

   for {set i 0} {$i < $currentLoc} {incr i} \
      {
       set id [lindex $idList $i] 

       if {$id == $currentID} \
          {          
           set newLoc $i
           break
          }
      }

   if {$newLoc >= 0} \
      {
       set currentLoc $newLoc
       updateDisplay
       return
      }

   updateDisplay
}

proc getNewAss {} \
{
   global idCount idList currentLoc currentID currentAss sInfo

   set newLoc -1

   for {set i [expr $currentLoc + 1]} {$i < $idCount} {incr i} \
      {
       set id  [lindex $idList $i]
       set ass $sInfo($id,ass)

       if {![strcasecmp $ass $currentAss]} \
          {
           set newLoc $i
           break
          }         
      }

   if {$newLoc >= 0} \
      {
       set currentLoc $newLoc
       updateDisplay
       return
      }

   for {set i 0} {$i < $currentLoc} {incr i} \
      {
       set id [lindex $idList $i]
       set ass $sInfo($id,ass)

       if {![strcasecmp $ass $currentAss]} \
          {        
           set newLoc $i
           break
          }
      }

   if {$newLoc >= 0} \
      {
       set currentLoc $newLoc
       updateDisplay
       return
      }

   updateDisplay
}

proc updateDisplay {} \
{
   global idList currentLoc currentID currentAss noise sInfo
   global minVal maxVal relFlag

   set currentID  [lindex $idList $currentLoc]
   set currentAss $sInfo($currentID,ass)

   catch ".g element delete evolution"
   catch ".g marker delete zero"

   set x ""
   set y ""
   set s ""

   set i  0
   set pN 0
   set nN 0

   foreach xy $sInfo($currentID,xyList) \
      {
       catch ".g marker delete m$i"

       set conc [lindex $xy 0]
       set val  [lindex $xy 1]
       set loc  [lindex $xy 2]

       lappend x $conc
       lappend y $val
       lappend s $noise

       if {$val >= 0.0} {incr pN}
       if {$val < 0.0}  {incr nN}

       set y1 [expr $val + $noise]
       set y2 [expr $val - $noise]

       .g marker create line -coords "$conc $y1 $conc $y2" -linewidth 2 \
                             -outline gray -under yes -name m$i

       incr i
      }

   set sInfo(xList) $x
   set sInfo(yList) $y
   set sInfo(sList) $s

   if {$relFlag} \
      {
       if {$nN == 0} \
          {
           .g axis configure y -min 0.0 -max ""
          } \
       elseif {$pN == 0} \
          {
           .g axis configure y -min "" -max 0.0 
          } \
       else \
          {
           .g axis configure y -min "" -max ""
          }
      } \
   else \
      {
       .g axis configure y -min $minVal -max $maxVal
      }

   .g element create evolution \
      -xdata $x -ydata $y -color yellow -fill white -pixels 3 -linewidth 2 

   .g marker create line -coords "-Inf 0.0 Inf 0.0" -name zero \
                         -outline gray -under yes

   update
}

proc fitXY {} \
{
   global sInfo currentID tabName evolveTitle fitArgs verbose

   set    cmnd "fitXY.tcl $fitArgs -quad -fixed None -anchor NE"
   append cmnd " -x $sInfo(xList) -y $sInfo(yList) -noise $sInfo(sList)"
   append cmnd " -id $currentID -xLab Conc uM -yLab $evolveTitle"
   append cmnd " -title $tabName &"

   set info ""

   if {$verbose} \
      {
       puts ""
       puts $cmnd
      }

   catch {set info [eval "exec $cmnd"]}
}

proc xSort { xy1 xy2 } \
{
   set t1 [lindex $xy1 0]
   set t2 [lindex $xy2 0]

   if {$t1 > $t2} {return  1}
   if {$t1 < $t2} {return -1}
   return 0
}

initVars
initArgs
getSeriesInfo
getSeriesData
initWin
updateDisplay
