#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set dNames    "dObsA.tab dObsB.tab"
set refName   ref.pdb
set a1        First 
set aN        Last 
set briefFlag 0

if {[flagLoc $ARGV -help]} \
   {
    puts "Simultaneous Estimate of Two Tensors:"
    puts " -in  dcList  \[dObsA.tab dObsB.tab\]  List of Two DC Tables."
    puts " -pdb pdbName \[ref.pdb\]              PDB Input Name."
    puts " -r1  r1      \[First\]                First Residue in Segment."
    puts " -rN  rN      \[Last\]                 Last Residue in Segment."
    puts " -brief                              Brief Output"
    exit 0
   }

getListArgD $ARGV -in dNames

getArgD $ARGV -pdb refName
getArgD $ARGV -r1  a1
getArgD $ARGV -rN  aN

if {[flagLoc $ARGV -brief]} {set briefFlag 1}

proc formatEuler { e } \
{
   set rx [lindex $e 0]
   set ry [lindex $e 1]
   set rz [lindex $e 2]

   return [format " % 7.2f % 7.2f % 7.2f" $rx $ry $rz]
}

#
#------------------------------------------------------------------------

dynInit

set refSrc  [dynRead -pdb -in $refName]
set dSrcAll [dynRead -dc  -in $dNames]

set dSrcA  [lindex $dSrcAll 0] 
set dSrcB  [lindex $dSrcAll 1]
set dSrcN  [lindex $dSrcAll end]

set pInfo  [dynGetInfo -src $refSrc]
set r1     [lindex $pInfo 0]
set rN     [lindex $pInfo 1]

if {![strcasecmp $a1 First]} {set a1 [expr $r1 + 1]}
if {![strcasecmp $aN Last]}  {set aN [expr $rN - 1]}

#
# SVD fit for initial parameters:

set dInfoN [dynSimulate -dcList $dSrcN -src $refSrc -r1 $a1 -rN $aN -svd]
set dInfoA [getNthTensor $dInfoN 1]
set dInfoB [getNthTensor $dInfoN 2]

set n      [llength $dInfoN]
set dRMS   [format "%7.4f" [lindex $dInfoN [expr $n - 3]]]
set dCount [lindex $dInfoN end]

set DaA    [format "% .4e" [lindex $dInfoA 3]]
set DrA    [format "% .4e" [lindex $dInfoA 4]]

set DaB    [format "% .4e" [lindex $dInfoB 3]]
set DrB    [format "% .4e" [lindex $dInfoB 4]]

set dInfoA [getNthTensor $dInfoN 1]
set dInfoB [getNthTensor $dInfoN 2]

extractSaupe $dInfoA sA
extractSaupe $dInfoB sB

set dRMSA   [format "%7.4f" [lindex $dInfoA 0]]
set eA      [formatEuler [lrange $dInfoA 6 8]]
set dCountA [lindex $dInfoA end]

set dRMSB   [format "%7.4f" [lindex $dInfoB 0]]
set eB      [formatEuler [lrange $dInfoB 6 8]]
set dCountB [lindex $dInfoB end]

set aAB     [format "% 7.2f" [getSaupeAngle sA sB]]
set eAB     [formatEuler [dynEuler $eA $eB]]

if {!$briefFlag} \
   {
    puts ""
    puts "SVD Fit:"
    puts " AB RMS: $dRMS N: $dCount Generalized Angle: $aAB"
    puts " A $DaA $DrA RMS: $dRMSA N: $dCountA Euler: $eA"
    puts " B $DaB $DrB RMS: $dRMSB N: $dCountB Euler: $eB"
   } \
else \
   {
    puts "$DaA $DrA $eA $DaB $DrB $eB $eAB $aAB"
   }

#
# Simultaneous Fit:

dynSetTensor -dc $dSrcA -dadr -da $DaA -dr $DrA -auto 
dynSetTensor -dc $dSrcB -dadr -da $DaB -dr $DrB -auto 

set dInfoN \
 [dynSimulate -dcList $dSrcN -src $refSrc -r1 $a1 -rN $aN \
              -simultaneous -keepTensor -retry 256]

set n       [llength $dInfoN]
set dRMS    [format "%7.4f" [lindex $dInfoN [expr $n - 3]]]
set dCount  [lindex $dInfoN end]

set dInfoA  [getNthTensor $dInfoN 1]
set dInfoB  [getNthTensor $dInfoN 2]

extractSaupe $dInfoA sA
extractSaupe $dInfoB sB

set aAB     [format "% 7.2f" [getSaupeAngle sA sB]]

set DaA     [format "% .4e" [lindex $dInfoA 3]]
set DrA     [format "% .4e" [lindex $dInfoA 4]]

set DaB     [format "% .4e" [lindex $dInfoB 3]]
set DrB     [format "% .4e" [lindex $dInfoB 4]]

set dRMSA   [format "%7.4f" [lindex $dInfoA 0]]
set eA      [formatEuler [lrange $dInfoA 6 8]]
set dCountA [lindex $dInfoA end]

set dRMSB   [format "%7.4f" [lindex $dInfoB 0]]
set eB      [formatEuler [lrange $dInfoB 6 8]]
set dCountB [lindex $dInfoB end]
set eAB     [formatEuler [dynEuler $eA $eB]]

if {!$briefFlag} \
   {
    puts ""
    puts "Simultaneous Fit:"
    puts " AB RMS: $dRMS N: $dCount Generalized Angle: $aAB"
    puts " A $DaA $DrA RMS: $dRMSA N: $dCountA Euler: $eA"
    puts " B $DaB $DrB RMS: $dRMSB N: $dCountB Euler: $eB"
    puts ""
    puts " AB Relative Euler Angles A:B $eAB"
   } \
else \
   {
    puts "$DaA $DrA $eA $DaB $DrB $eB $eAB $aAB"
   }

dynSetTensor -dc $dSrcA -dadr  -da $DaA -dr $DrA -auto 

dynSetTensor -dc $dSrcB -fixed -da $DaB -dr $DrB -rotref \
             -psiRef [lindex $eAB 0] -thetaRef [lindex $eAB 1] -phiRef \
             [lindex $eAB 2]
             

set dInfoN \
 [dynSimulate -dcList $dSrcN -src $refSrc -r1 $a1 -rN $aN \
              -simultaneous -keepTensor -retry 256]

set n       [llength $dInfoN]
set dRMS    [format "%7.4f" [lindex $dInfoN [expr $n - 3]]]
set dCount  [lindex $dInfoN end]

set dInfoA  [getNthTensor $dInfoN 1]
set dInfoB  [getNthTensor $dInfoN 2]

extractSaupe $dInfoA sA
extractSaupe $dInfoB sB

set aAB     [format "% 7.2f" [getSaupeAngle sA sB]]

set DaA     [format "% .4e" [lindex $dInfoA 3]]
set DrA     [format "% .4e" [lindex $dInfoA 4]]

set DaB     [format "% .4e" [lindex $dInfoB 3]]
set DrB     [format "% .4e" [lindex $dInfoB 4]]

set dInfoA [getNthTensor $dInfoN 1]
set dInfoB [getNthTensor $dInfoN 2]

set dRMSA   [format "%7.4f" [lindex $dInfoA 0]]
set eA      [formatEuler [lrange $dInfoA 6 8]]
set dCountA [lindex $dInfoA end]

set dRMSB   [format "%7.4f" [lindex $dInfoB 0]]
set eB      [formatEuler [lrange $dInfoB 6 8]]
set dCountB [lindex $dInfoB end]
set eAB     [formatEuler [dynEuler $eA $eB]]

if {!$briefFlag} \
   {
    puts ""
    puts "Simultaneous Fit, Restrained Relative Orientation:"
    puts " AB RMS: $dRMS N: $dCount Generalized Angle: $aAB"
    puts " A $DaA $DrA RMS: $dRMSA N: $dCountA Euler: $eA"
    puts " B $DaB $DrB RMS: $dRMSB N: $dCountB Euler: $eB"
    puts ""
    puts " AB Relative Euler Angles A:B $eAB"
   } \
else \
   {
    puts "$DaA $DrA $eA $DaB $DrB $eB $eAB $aAB"
   }

exit 0
