#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Sort Planes of a Pseudo-3D Series According to Tau Values:"
   echo " -in      inName  [fid/test%03d.fid]  3D Input Name."
   echo " -outName outName [sort/test%03d.fid] 3D Output Name (Can be the Same as Input)."
   echo " -tau     tList   [0.0 ...]           List of Tau Values."
   echo " -sign                                Perform Z-Axis Sign Adjustment."
   echo " -title                               Update Spectrum Title to Include Tau."
   echo " -up                                  Sort by Increasing Tau Value (Default)."
   echo " -down                                Sort by Decreasing Tau Value."
   exit 0
endif

if (`flagLoc $argv -title`) then
   set titleFlag = 1
else
   set titleFlag = 0
endif

if (`flagLoc $argv -down`) then
   set upFlag = 0
else
   set upFlag = 1
endif

if (`flagLoc $argv -sign`) then
   set signFlag = 1
else
   set signFlag = 0
endif

set inName  = (`getArgD $argv -in  fid/test%03d.fid`)
set outName = (`getArgD $argv -out sort/test%03d.fid`)
set tmpName = (`getArgD $argv -tmp tmp_sort$$/test%03d.fid`)

set tauList = (`getListArgD $argv -tau -- 0.0`)

#
#------------------------------------------------------------------------

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
   echo "sortPlane Error finding Input $inName ($thisName)."
   exit 1
endif

set outDir = (`string path $outName`)
set tmpDir = (`string path $tmpName`)

if (!(-d $outDir)) then
   mkdir $outDir 
endif

if (!(-d $tmpDir)) then
   mkdir $tmpDir
endif

set nZ = (`getParm -in $inName -parm NDSIZE -dim CUR_ZDIM -fmt %.0f`)
set n  = $#tauList

if ($n != $nZ) then
   echo "sortPlane Error: Size Mismatch. Tau Values: $n Z-Axis Size: $nZ"
   exit 1
endif

if ($upFlag) then
   set iList = (`sortRList.tcl -r $tauList -up`)
else
   set iList = (`sortRList.tcl -r $tauList -down`)
endif

@ j = 1

foreach i ($iList)
   set thisInName  = (`nmrPrintf $inName  $i`)
   set thisTmpName = (`nmrPrintf $tmpName $j`)
   set thisOutName = (`nmrPrintf $outName $j`)

   set tau = $tauList[$i]

   if ($signFlag && `IMATH "($i - 1) % 2"`) then
      @ k = -1
   else
      @ k =  1
   endif

   nmrPrintf " Input: %s Output: %s Tau: %.3e Sign: %2d\n" $thisInName $thisOutName $tau $k

   if ($k == -1) then
      nmrPipe -in $thisInName -fn SIGN -i -out $thisTmpName -ov
   else
      nmrPipe -in $thisInName -fn NULL    -out $thisTmpName -ov
   endif

   if ($titleFlag) then
      set tStr = (`getParm -in $thisTmpName -parm FDTITLE -dim NULL_DIM -text -fmt %s`)

      if (`string length $tStr`) then
         set tStr = ${tStr}-$tau
      else
         set tStr = (`nmrPrintf "%03d-%s" $i $tau`)
      endif

      sethdr $thisTmpName -tau $tau -title $tStr
   else
      sethdr $thisTmpName -tau $tau
   endif

   @ j++
end

@ j = 1

while ($j <= $n)
   set thisTmpName = (`nmrPrintf $tmpName $j`)
   set thisOutName = (`nmrPrintf $outName $j`)

   /bin/mv $thisTmpName $thisOutName
   @ j++
end

/bin/rm -rf $tmpDir

