#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set thisVersion "2024.303.21.10"

set inName   spin.txt
set outName  spin.tab
set multDir  None
set multName %03d.tab

set fidName  test.fid
set ftName   test.ft1

set baseAmp  100.0
set lwList   0.0

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert Spin Matrix Text File to 1D Peak Table ($thisVersion):"
    puts " -in   inName     \[spin.txt\]  Spin Matrix Text File Input"
    puts " -out  outName    \[spin.tab\]  Peak Table Output."
    puts " -fid  fidName    \[test.fid\]  Corresponding Time-Domain Data."
    puts " -ft   ftName     \[test.ft1\]  Corresponding Spectrum."
    puts " -lw   lwList     \[1.0 ...\]   Linewidths in Hz."
    puts " -amp  amp        \[100.0\]     Amplitide for a Singlet Time-Domain Signal."
    puts "Optional Output of Multiplet Tables:"
    puts " -multDir  multDir  \[None\]    Optional Output Directory for Multiplet Tables."
    puts " -multName multName \[%3d.tab\] Template Name for Optional Multiplet Table Output."
    puts "Notes:"
    puts " 1. For a molecule of N atoms, the spin matrix is a square matrix of"
    puts "    NxN values, where the diagonal of the matrix is the chemical shift"
    puts "    in ppm of atom I, and the off-diagonal elements are the couplings"
    puts "    in Hz between atom I and atom J. In practice, while this matrix"
    puts "    should be symmetric, only the coupling values above the diagonal"
    puts "    are set, and the corresponding values below the diagonal are zero."
    puts " 2. The first line of the spin matrix file lists the N unique integer"
    puts "    atom IDs. The following lines each start with an atom ID, followed"
    puts "    by N values. Blank lines and lines begining with # will be ignored."
    puts " 3. The spin matrix can have an optional 'atoms' line, listing the atom"
    puts "    names of the N atoms. (See below)."
    puts " 4. If less than N linewidth values are specified via the -lw option,"
    puts "    the last value specified will be used for the remaining atoms."
    puts " 5. The input time-domain data and spectrum are used to convert between"
    puts "    points, Hz, and ppm."
    puts ""
    puts " Example spin matrix for alanine, with atom IDs 7, 8, 9, and 10, and"
    puts " optional 'atoms' line:"
    puts ""
    puts "              7       8       9      10"
    puts "    7     1.466 -12.500 -12.500   7.299"
    puts "    8         0   1.466 -12.500   7.299"
    puts "    9         0       0   1.466   7.299"
    puts "    10        0       0       0   3.767"
    puts "    atoms   HB1     HB2     HB3      HA"
    puts ""
    puts " As an alternative to including the 'atoms' line in the spin matrix,"
    puts " this is example atom name file contents for the alanine matrix above:"
    puts ""
    puts "    7  HB1"
    puts "    8  HB2"
    puts "    9  HB3"
    puts "   10  HA"
    puts ""
    puts "It can be useful to include a DSS signal as an extra row/column of zeros:"
    puts ""
    puts "              7       8       9      10    0"
    puts "    7     1.466 -12.500 -12.500   7.299    0"
    puts "    8         0   1.466 -12.500   7.299    0"
    puts "    9         0       0   1.466   7.299    0"
    puts "    10        0       0       0   3.767    0"
    puts :    0         0       0       0      0   0.0"
    puts "    atoms   HB1     HB2     HB3      HA  DSS"
    exit 0
   }

getArgD $ARGV -in  inName
getArgD $ARGV -out outName
getArgD $ARGV -fid fidName
getArgD $ARGV -ft  ftName
getArgD $ARGV -amp baseAmp

getArgD $ARGV -multDir  multDir
getArgD $ARGV -multName multName

getListArgD $ARGV -lw lwList

#
#-------------------------------------------------------------------------------------------

proc pkSplit { pkStr jVal idI idJ } \
{
   set newStr ""

   foreach pk $pkStr \
      {
       lappend newStr $pk,+,j($idI,$idJ)=$jVal
       lappend newStr $pk,-,j($idI,$idJ)=$jVal
      }

   return $newStr
}

if {![file exists $inName]} \
   {
    puts "spin2tab. Error finding spin matrix input $inName"
    exit 1
   }

if {![file exists $fidName]} \
   {
    puts "spin2tab. Error finding time-domain input $fidName"
    exit 1
   }

if {![file exists $ftName]} \
   {
    puts "spin2tab. Error finding input spectrum $ftName"
    exit 1
   }

set error [catch {set inUnit [open $inName r]}]

if {$error} \
   {
    puts "spin2tab. Error opening spin matrix input $inName"
    exit 1
   }

set error [rdFDATA $fidName fdataT]

if {$error} \
   {
    puts "spin2tab. Error Reading Header in Time-Domain Input $fidName"
    exit 1
   }

set error [rdFDATA $ftName fdataF]

if {$error} \
   {
    puts "spin2tab. Error Reading Header in Spectrum Input $ftName"
    exit 1
   }

set error [catch {set outUnit [open $outName w]}]

if {$error} \
   {
    puts "spin2tab. Error opening peak table output $outName"
    exit 1
   }

if {[strcasecmp $multDir None]} \
   {
    if {![file isdirectory $multDir]} {catch "file mkdir $multDir"}

    if {![file isdirectory $multDir]} \
       {
        puts "spin2tab. Error finding or creating output directort $multDir"
        exit 1
       }
   }

set atomList ""
set idList   ""
set idCount  0

set n 1

while {[gets $inUnit line] >= 0} \
   {
    if {![llength $line]} continue

    set item [lindex $line 0]
    set key  [string index $item 0]

    if {![strcmp $key "#"]} continue

    if {![strcasecmp $item atoms] || ![strcasecmp $item atom]} \
       {
        set atomList  [lrange $line 1 end]
        set thisCount [llength $atomList]

        if {$thisCount != $idCount} \
           {
            puts "spin2tab. Error: bad item count in spin matrix input $inName atom line $n"
            exit
           }

        continue
       }

    if {$n == 1} \
       {
        foreach id $line {lappend idList $id}
        set idCount [llength $idList]
       } \
    else \
       {
        set thisCount [llength $line]

        if {$thisCount != $idCount + 1} \
           {
            puts "spin2tab. Error: bad item count in spin matrix input $inName line $n"
            exit
           }

        set idI [lindex $line 0]

        for {set i 1} {$i <= $idCount} {incr i} \
           {
            set idJ [lindex $idList [expr $i - 1]]

            if {$idI == $idJ} \
               {
                set shiftInfo($idI)  [lindex $line $i]
               } \
            else \
               {
                if {![info exists jInfo($idI,$idJ)]} {set jInfo($idI,$idJ) [lindex $line $i]}
                if {$jInfo($idI,$idJ) != 0}          {set jInfo($idJ,$idI) $jInfo($idI,$idJ)} 
               }
           }
       }

    incr n
   }

catch "close $inUnit"

if {$n < 2} \
   {
    puts "spin2tab. Error: missing data in spin matrix input $inName"
    exit 1
   }

set thisLW [lindex $lwList end]

foreach idI $idList \
   {
    lappend lwList   $thisLW
    lappend atomList None
   }

for {set i 0} {$i < $idCount} {incr i} \
   {
    set idI            [lindex $idList   $i]
    set lwInfo($idI)   [lindex $lwList   $i]
    set atomInfo($idI) [lindex $atomList $i]
   }

foreach idI $idList \
   {
    puts -nonewline " idI $idI:"

    foreach idJ $idList \
       {
        if {$idI == $idJ} \
           {
            puts -nonewline [format " %16s" s($idI)=$shiftInfo($idI)]
           } \
        else \
           {
            puts -nonewline [format " %16s" j($idI,$idJ)=$jInfo($idI,$idJ)]
           }
        }

     puts ""
    }


puts ""

foreach idI $idList \
   {
    set pkInfo($idI) s($idI)=$shiftInfo($idI)

    foreach idJ $idList \
       {
        if {$idI != $idJ} \
           {
            if {$jInfo($idI,$idJ) != 0} {set pkInfo($idI) [pkSplit $pkInfo($idI) $jInfo($idI,$idJ) $idI $idJ]}
           }
       }

    puts "$idI $pkInfo($idI)"
   }

set varStr "VARS   INDEX X_AXIS DX    X_PPM X_HZ  XW    XW_HZ X1  X3  HEIGHT DHEIGHT VOL PCHI2 TYPE ASS CLUSTID MEMCNT"
set fmtStr "FORMAT %5d   %9.3f  %6.3f %9.4f %9.3f %7.3f %8.3f %6d %6d %+e    %+e     %+e %.5f  %d   %s  %4d     %4d"

puts $outUnit $varStr
puts $outUnit $fmtStr
puts $outUnit ""

set pkID 0

foreach idI $idList \
   {
    if {[strcasecmp $multDir None]} \
       {
        set thisMultName [format "%s/$multName" $multDir $idI]

        set error [catch {set multUnit [open $thisMultName w]}]

        if {$error} \
           {
            puts "spin2tab. Error opening multiplet table output $thisMultName"
            exit 1
           }
 
        puts $multUnit ""
        puts $multUnit "DATA PEAK_INFO $idI $pkInfo($idI)"
        puts $multUnit ""

        puts $multUnit $varStr
        puts $multUnit $fmtStr
        puts $multUnit ""
       }

    set sigCount [llength $pkInfo($idI)]

    puts ""
    puts "$idI $pkInfo($idI)"
    puts "$sigCount Signals:"

    set lwHz  $lwInfo($idI)
    set lwPts [spec2pntW $fdataT CUR_XDIM ${lwHz}Hz]

    set i 0

    foreach pk $pkInfo($idI) \
       {
        incr i
        incr pkID

        set jList    [split $pk ",="]
        set shiftPPM [lindex $jList 1]
        set jCount   [expr ([llength $jList] - 2)/4]

        set shiftPts [spec2pnt $fdataF CUR_XDIM ${shiftPPM}ppm]

        set jOffHz 0.0
        set loc    2

        if {$jCount == 0} \
           {
            set hi $baseAmp
           } \
        else \
           {
            set hi [expr $baseAmp/(2*$jCount)]
           }

        set vol [expr $hi*$lwPts]

        for {set j 0} {$j < $jCount} {incr j} \
           {
            set thisSign [lindex $jList $loc]
            set jHz      [lindex $jList [expr $loc + 3]]

            if {![strcmp $thisSign "+"]} \
               {
                set sMult  1
               } \
            else \
               {
                set sMult -1
               }

            set jOffHz [expr $jOffHz + 0.5*$sMult*$jHz]

            incr loc 4
           }

        set jOffPts [spec2pntW $fdataF CUR_XDIM ${jOffHz}Hz]

        set xPts    [expr $shiftPts + $jOffPts]
        set xPPM    [pnt2spec $fdataF CUR_XDIM $xPts ppm]
        set xHz     [pnt2spec $fdataF CUR_XDIM $xPts Hz]

        set x1      [expr int($xPts - 3*$lwPts) - 1]
        set x3      [expr int($xPts + 3*$lwPts) + 1]

        set fmt " %5d %9.3f %6.3f %9.4f %9.3f %7.3f %8.3f %6d %6d %+e %+e %+e %.5f %d %s %5d %4d"

        set thisAss H$idI

        if {[strcasecmp $atomInfo($idI) None]} {append thisAss "-$atomInfo($idI)"}

        if {![strcasecmp $multDir None]} \
           {
            puts $outUnit  [format $fmt $pkID $xPts 0.1 $xPPM $xHz $lwPts $lwHz $x1 $x3 $hi 1.0e-3 $vol 0.0 1 $thisAss $idI 1]
           } \
        else \
           {
            puts $outUnit  [format $fmt $pkID $xPts 0.1 $xPPM $xHz $lwPts $lwHz $x1 $x3 $hi 1.0e-3 $vol 0.0 1 $thisAss $idI 1]
            puts $multUnit [format $fmt $pkID $xPts 0.1 $xPPM $xHz $lwPts $lwHz $x1 $x3 $hi 1.0e-3 $vol 0.0 1 $thisAss $idI 1]
           }

        puts " ID: $idI Atom: $atomInfo($idI) Signal: $i Shift: $shiftPPM ppm J Offset: $jOffHz Hz J Terms: $jCount Info: $jList"
       }

    if {[strcasecmp $multDir None]} {catch "close $multUnit"} 
   }

catch "close $outUnit"

exit 0
