#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "splitPDB.tcl: split a PDB file at MODEL/ENDMDL or END bounds:"
    puts " -in     inPDBName   \[ref.pdb\]  Input PDB File."
    puts " -out    outPDBName  \[Auto\]     Output PDB Series Template."
    puts " -rem    remPDBName  \[None\]     Optional Output of REMARKs etc."
    puts " -hetatm hetPDBName  \[None\]     Optional Output of HETATMs."
    puts " -clean                         Clear SEGID and Line Numbers."
    puts " -probe                         Report Model Count Only, and Exit."
    puts " -verb                          Verbose Mode ON (Default)."
    puts " -noverb                        Verbose Mode OFF."
    puts " -silent                        All Messages Suppressed."
    puts "Generation of Output Name:"
    puts " -model                       Use MODEL ID for Output Name (Default)."
    puts " -nomodel                     Use Structure Number for Output Name." 
    puts "Notes:"
    puts " 1. Use an -out template such as ubiq%03d.pdb" 
    puts " 2. When -out Auto is used, the output template will be based on the"
    puts "    input name. For example, if the input is 1d3z.pdb, the output"
    puts "    template would be: 1d3z_%03d.pdb"
    exit 1
   }

set inName     ref.pdb
set outName    Auto
set remName    None
set hetName    None
set cleanFlag  0
set modelFlag  1
set probeFlag  0
set silentFlag 1
set verbose    1

if {[flagLoc $ARGV -probe]} \
   {
    set probeFlag 1
    set modelFlag 0
    set outName   None
   }

getArgD $ARGV -in     inName
getArgD $ARGV -out    outName
getArgD $ARGV -rem    remName
getArgD $ARGV -hetatm hetName

if {[flagLoc $ARGV -model]}   {set modelFlag 1}
if {[flagLoc $ARGV -nomodel]} {set modelFlag 0}

if {[flagLoc $ARGV -clean]}   {set cleanFlag 1}
if {[flagLoc $ARGV -noclean]} {set cleanFlag 0}

if {[flagLoc $ARGV -verb]}    {set verbose   1}
if {[flagLoc $ARGV -noverb]}  {set verbose   0}

if {[flagLoc $ARGV -silent]} \
   {
    set silentFlag 1
    set verbose    0
   }

if {![strcasecmp $outName Auto]} \
   {
    set outName [file root $inName]_%03d.pdb
   }

#
#----------------------------------------------------------------------------

proc writeOutput { line } \
{
   global outName outUnit

   if {![strcasecmp $outUnit None]} openOutput
   if {![strcasecmp $outName None]} return

   putPDBLine $outUnit $line 0
}

proc openOutput {} \
{
   global outName thisOutName outUnit atomCount modelCount id openFlag

   if {!$openFlag} {incr modelCount}

   closeOutput

   if {!$id} {incr id}

   set openFlag 1

   if {![strcasecmp $outName None]} return

   set thisOutName [format $outName $id]

   set error [catch {set outUnit [open $thisOutName w]}]

   if {$error} \
      {
       puts stderr "SplitPDB Error Opening PDB Output $thisOutName"
       exit 1
      }
}

proc closeOutput {} \
{
   global outName thisOutName outUnit atomCount openFlag silentFlag verbose

   if {[strcasecmp $outUnit None]} \
      {
       puts $outUnit "END"

       if {$verbose} \
          {
           puts "$thisOutName, $atomCount atoms."
          } \
       else \
          {
           if {!$silentFlag} {puts $thisOutName}
          }

       close $outUnit
      }

   set outUnit   None
   set openFlag  0
   set atomCount 0
}

proc openRem {} \
{
   global remName remUnit

   if {![strcasecmp $remName None]} {return 0}

   if {![strcasecmp $remName stdout]} \
      {
       set remUnit stdout
       return 0
      }

   if {![strcasecmp $remName stderr]} \
      {
       set remUnit stderr
       return 0
      }

   set error [catch {set remUnit [open $remName w]}]

   if {$error} \
      {
       puts stderr "SplitPDB Error Opening PDB REMARK Output $remName"
       exit 1
      }
}

proc closeRem {} \
{
   global remUnit

   if {![strcasecmp $remUnit None]}   {return 0}
   if {![strcasecmp $remUnit stdout]} {return 0}
   if {![strcasecmp $remUnit stderr]} {return 0}

   catch "close $remUnit"

   return 1
}

proc openHet {} \
{
   global hetName hetUnit

   if {![strcasecmp $hetName None]} {return 0}

   if {![strcasecmp $hetName stdout]} \
      {
       set hetUnit stdout
       return 0
      }

   if {![strcasecmp $hetName stderr]} \
      {
       set hetUnit stderr
       return 0
      }

   set error [catch {set hetUnit [open $hetName w]}]

   if {$error} \
      {
       puts stderr "SplitPDB Error Opening PDB HETATM Output $hetName"
       exit 1
      }
}

proc closeHet {} \
{
   global hetUnit

   if {![strcasecmp $hetUnit None]}   {return 0}
   if {![strcasecmp $hetUnit stdout]} {return 0}
   if {![strcasecmp $hetUnit stderr]} {return 0}

   catch "close $hetUnit"

   return 1
}

proc putPDBLine { thisUnit thisLine keepElemFlag } \
{
   global cleanFlag

   if {$cleanFlag} \
      {
       if {$keepElemFlag} \
          {
           set eStr [string range $thisLine 76 77]

           if {![string length $eStr]} \
              {
               puts $thisUnit [string range $thisLine 0 71]
              } \
           else \
              {
               set c1 [string index $eStr 0]
               set c2 [string index $eStr 1]
   
               set newStr [string replace $thisLine 72 75 "    "]
               set newStr [string replace $newStr   78 79 "  "]

               if {[lsearch -exact {0 1 2 3 4 5 6 7 8 9 " "} $c1] >= 0 && \
                    [lsearch -exact {0 1 2 3 4 5 6 7 8 9 " "} $c2] >= 0} \
                  {
                   set newStr [string replace $newStr 76 77 "  "]
                  } 
    
               puts $thisUnit $newStr
              }
          } \
       else \
          {
           puts $thisUnit [string range $thisLine 0 71]
          }
      } \
   else \
      {
       puts $thisUnit $thisLine
      }
}

if {![file exists $inName]} \
   {
    puts stderr "SplitPDB Error Finding Input PDB $inName"
    exit 1
   }

set error [catch {set inUnit [open $inName r]}]

if {$error} \
   {
    puts stderr "SplitPDB Error Opening Input PDB $inName"
    exit 1
   }

set dirName [file dir $outName]

if {![file isdir $dirName]} \
   {
    if {$verbose} {puts "SplitPDB: Creating Directory $dirName"}

    set error [catch {file mkdir $dirName}]

    if {$error} \
       {
        puts stderr "SplitPDB Error Opening Output Directory $dirName"
        exit 1
       }
   }


set thisOutName None
set outUnit     None 
set remUnit     None
set hetUnit     None
set atomFlag    0
set atomCount   0
set id          0
set nModelKey   0
set modelCount  0
set openFlag    0\

openRem
openHet

while {[gets $inUnit line] >= 0} \
   {
    scan $line %s key

    switch -- $key \
       {
        ATOM    {
                 set atomFlag 1
                 incr atomCount
                 writeOutput $line
                }
 
        HETATM  {
                 set atomFlag 1
                 incr atomCount
                 writeOutput $line

                 if {[strcasecmp $hetUnit None]} {putPDBLine $hetUnit $line 1}
                }

        MODEL   {
                 incr nModelKey

                 if {$modelFlag} \
                    {
                     scan $line "%s %s" key id
                    } \
                 else \
                    {
                     incr id
                    }

                 openOutput
                }

        ENDMDL  {closeOutput}

        END     {
                 if {!$nModelKey && $atomCount} {incr id}
                 closeOutput
                }

        default {
                 if {!$atomFlag && [strcasecmp $remUnit None]} \
                    {
                     putPDBLine $remUnit $line 0
                    }

                 set key ""
                }
       }
   }

closeOutput
closeRem
closeHet

if {$probeFlag} {puts $modelCount}

exit
