#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

#
# Extract NMR-star chemical shifts to NMRPipe format CS table.

set auto_path "[split $env(TCLPATH) :] $auto_path"

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Convert BMRB NMR-star Chemical Shifts to NMRPipe Table Format:"
    puts stderr " $argv0 < inName > outName"
    puts stderr " -in    inName  \[stdin\]  BMRB Star Format Input."
    puts stderr " -out   outName \[stdout\] NMRPipe Format Output (or None)."
    puts stderr " -err   logName \[stderr\] Output Name for Status Messages."
    puts stderr " -title tName   \[None\]   Name Used for Conversion REMARK." 
    puts stderr " -src   sName   \[None\]   Name for SOURCE column, or Keyword Auto."
    puts stderr " -verb                   Verbose Mode ON (Default)."
    puts stderr " -noverb                 Verbose Mode OFF."
    puts stderr "NMR-Star Parameter Naming:"
    puts stderr " -auto                   Determine Automatically (Default)."
    puts stderr " -old                    Assume Old Format (_Chem_shift_value  etc)."
    puts stderr " -new                    Assume New Format (_Atom_chem_shift.Val etc)."
    puts stderr "Output Control:"
    puts stderr " -hdr                    Display Header (Default)."
    puts stderr " -nohdr                  No Header."
    puts stderr " -showseq                Display Sequence (Default)."
    puts stderr " -noshowseq              No Sequence DATA."
    puts stderr " -showcs                 Display Chemical Shifts (Default)."
    puts stderr " -noshowcs               No Chemical Shift Data."
    puts stderr "Residue Code Output (For No Output Variable, Use Keyword None):"
    puts stderr " -res1  vName \[RESNAME\] Variable Name for Single-Letter Codes."
    puts stderr " -res3  vName \[RESTYPE\] Variable Name for Three-Letter Codes."
    puts stderr " -cysCB cLim  \[34.0\]    CB PPM Threshold for CYS-ox vs reduced."
    puts stderr " -hispH hLim  \[6.0\]     pH Threshold for HIS-protonated form."
    puts stderr "Atom Name Selection and Conversion:"
    puts stderr " -atom aList  \[*\]       List of Input Atom Names to be Used."
    puts stderr " -aIn  aList  \[H\]       Any Input Atom Names to be Converted." 
    puts stderr " -aOut aList  \[HN\]      Corresponding Output Atom Names."
    puts stderr "Format Checking (Use Keyword None for no limit):"
    puts stderr " -refCS    rName \[Auto\]  Name of Reference Shift Table, or None."
    puts stderr " -maxDev   dMax  \[0.001\] Max Std Deviation for Multiple Shifts."
    puts stderr " -maxLen   lMax  \[5000\]  Max Allowable Line Length."
    puts stderr " -maxLines nMax  \[32000\] Max Allowable Line Count."
    puts stderr "Notes:"
    puts stderr " Depends on finding a loop that includes all of (Old Format):"
    puts stderr "  _Residue_seq_code"
    puts stderr "  _Residue_label"
    puts stderr "  _Atom_name"
    puts stderr "  _Chem_shift_value"
    puts stderr "Or (New Format):"
    puts stderr " _Atom_chem_shift.Seq_ID"
    puts stderr " _Atom_chem_shift.Comp_ID"
    puts stderr " _Atom_chem_shift.Atom_ID"
    puts stderr " _Atom_chem_shift.Val"
    exit 1
   }

set inName       stdin 
set outName      stdout 
set srcName      None 
set refName      Auto
set titleName    None
set atomList     *

set maxLineCount  32000
set maxLineLength 5000
set maxShiftDev   0.001
set cysCB         34.0
set hispH          6.0
set pH            None

set oldFlag       Auto 

set res1Var  RESNAME
set res3Var  RESTYPE

set errName   stderr
set errUnit   stderr

set fullSeqFlag 0
set showSeqFlag 1
set showREMFlag 1
set showCSFlag  1
set hdrFlag     1
set verbose     1

set aNamesIn  "H"
set aNamesOut "HN"

set errCount  0
set warnCount 0
set pushCount 0

getArgD $ARGV -in       inName
getArgD $ARGV -out      outName
getArgD $ARGV -src      srcName
getArgD $ARGV -title    titleName
getArgD $ARGV -err      errName
getArgD $ARGV -refCS    refName
getArgD $ARGV -res1     res1Var
getArgD $ARGV -res3     res3Var
getArgD $ARGV -cysCB    cysCB
getArgD $ARGV -hispH    hispH
getArgD $ARGV -pH       pH

getArgD $ARGV -maxLines maxLineCount
getArgD $ARGV -maxLen   maxLineLength
getArgD $ARGV -maxDev   maxShiftDev

getListArgD $ARGV -atom atomList
getListArgD $ARGV -aIn  aNamesOut
getListArgD $ARGV -aOut aNamesIn

if {[flagLoc $ARGV -hdr]}    {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]}  {set hdrFlag 0}

if {[flagLoc $ARGV -showrem]}    {set showREMFlag 1}
if {[flagLoc $ARGV -noshowrem]}  {set showREMFlag 0}

if {[flagLoc $ARGV -showseq]}    {set showSeqFlag 1}
if {[flagLoc $ARGV -noshowseq]}  {set showSeqFlag 0}

if {[flagLoc $ARGV -showcs]}     {set showCSFlag 1}
if {[flagLoc $ARGV -noshowcs]}   {set showCSFlag 0}

if {[flagLoc $ARGV -auto]}       {set oldFlag Auto}
if {[flagLoc $ARGV -old]}        {set oldFlag 1}
if {[flagLoc $ARGV -new]}        {set oldFlag 0}

if {[flagLoc $ARGV -verb]}       {set verbose 1}
if {[flagLoc $ARGV -noverb]}     {set verbose 0}

if {[flagLoc $ARGV -fullSeq]}    {set fullSeqFlag 1}
if {[flagLoc $ARGV -nofullSeq]}  {set fullSeqFlag 0}

set nIn  [llength $aNamesIn]
set nOut [llength $aNamesOut]

#
#------------------------------------------------------------------------------

proc star_exit {} \
{
   global errCount warnCount

   if {$errCount || $warnCount} \
      {
       puts "star2cs.  NMR-Star Format Conversion Errors: $errCount Warnings: $warnCount"
      }

   exit 0
}

proc showFormatInfo {} \
{
   global errUnit

   puts $errUnit ""
   puts $errUnit "NMR-Star Table to NMRPipe-Format Table Conversion requires"
   puts $errUnit "that the input table has a loop that includes all of:"
   puts $errUnit ""
   puts $errUnit "  _Residue_seq_code"
   puts $errUnit "  _Residue_label"
   puts $errUnit "  _Atom_name"
   puts $errUnit "  _Chem_shift_value"
   puts $errUnit "  _Chem_shift_value"
   puts $errUnit ""
   puts $errUnit "Or:"
   puts $errUnit ""
   puts $errUnit " _Atom_chem_shift.Seq_ID"
   puts $errUnit " _Atom_chem_shift.Comp_ID"
   puts $errUnit " _Atom_chem_shift.Atom_ID"
   puts $errUnit " _Atom_chem_shift.Val"
   puts $errUnit ""
   puts $errUnit "If only sequence data is needed, the input table must have"
   puts $errUnit "a loop that includes:"
   puts $errUnit ""
   puts $errUnit "  _Residue_seq_code"
   puts $errUnit "  _Residue_label"
   puts $errUnit ""
   puts $errUnit "Or:"
   puts $errUnit ""
   puts $errUnit " _Entity_comp_index.ID"
   puts $errUnit " _Entity_comp_index.Comp_ID"
   puts $errUnit ""
   puts $errUnit "Sample pH is extracted from a loop that includes a variable"
   puts $errUnit "type 'pH' in a look that includes:"
   puts $errUnit ""
   puts $errUnit " _Variable_type"
   puts $errUnit " _Variable_value"
   puts $errUnit ""
   puts $errUnit "Or:"
   puts $errUnit ""
   puts $errUnit " _Sample_condition_variable.Type"
   puts $errUnit " _Sample_condition_variable.Val"
}

proc starAtomSelected { atomName } \
{
   global atomList

   foreach a $atomList \
      {
       if {![strcmp $a "*"]} {return 1}
       if {![strcasecmp $a "All"]} {return 1}
       if {[strmatch $atomName $a]} {return 1}
      }

   return 0
}

proc updateLineCount {} \
{
   global lineCount maxLineCount errUnit errCount

   incr lineCount

   if {[strcasecmp $maxLineCount None]} \
      {
       if {$lineCount > $maxLineCount} \
          {
           puts $errUnit "star2cs. Error: Max Line Count $maxLineCount Exceeded."
           incr errCount
           star_exit 
          }
      }

   return $lineCount
}

proc star_pushs { str } \
{
   global pushCount pushInfo

   incr pushCount
   set  pushInfo($pushCount) $str
}

proc star_gets { inUnit linePtr } \
{
   upvar $linePtr line

   global maxLineLength errUnit errCount pushCount pushInfo

   if {$pushCount > 0} \
      {
       set line $pushInfo($pushCount)
       set n [string length $line]
      
       incr pushCount -1
       return $n 
      }

   set lineOrig ""
   set line     ""
   set n        -1 

   set error [catch {set n [gets $inUnit lineOrig]}]
   if {$error} {return -1}

   if {[strcasecmp $maxLineLength None]} \
      {
       if {$n > $maxLineLength} \
          {
           puts $errUnit "star2cs. Error: Max Line Length $maxLineLength Exceeded."
           incr errCount
           star_exit 
          }
      }

   if {$n < 0} \
      {
       set line ""
       return $n
      }

   set line [string map {\" ' \$ % \\ / \{ ) \} ) \[ ( \] )} $lineOrig]
   set n    [string length $line]

   set loc [string first ' $line]
   if {$loc < 0} {return $n}

   set line [fixStarText $line]
   set n    [string length $line]

   return $n
}

proc fixStarText { line } \
{
    set n [string length $line]
    set inQuote 0
    set s ""

    for {set i 0} {$i < $n} {incr i} \
       {
        set c [string index $line $i]

        if {![strcmp $c ']} \
           {
            if {$inQuote} \
               {
                set inQuote 0
               } \
            else \
               {
                set inQuote 1
               }
           }

        if {$inQuote && [string is space $c]} {set c "_"} 
        
        append s $c 
       }

   if {$inQuote} {append s '}

   return $s
}

proc fixAtomName { r3 a } \
{
   global aNamesIn aNamesOut

   set i 0

   foreach aInfo $aNamesIn \
      {
       set aList [split $aInfo ,]

       if {[llength $aList] == 1} \
          {
           set atom [lindex $aList 0]

           if {![strcasecmp $atom $a]} {return [lindex $aNamesOut $i]}
          } \
       else \
          {
           set res  [lindex $aList 0]
           set atom [lindex $aList 1]

           if {![strcasecmp $res $r3] && ![strcasecmp $atom $a]} \
              {
               return [lindex $aNamesOut $i]
              }
          }

       incr i
      }

   return $a 
}

#
# Extract contents for a sample conditions loop to find pH.

proc getSampleLoop { inUnit vCount vPairs } \
{
   global pH vLoc lineCount errUnit warnCount

   set buff ""

   while {[star_gets $inUnit origLine] >= 0} \
      {
       updateLineCount

       set line [string trim $origLine]
       set n    [string length $line]
       set key  ""

       scan $line %s key
       set nn [string length $key]

       if {!$n || !$nn} continue

       if {![strcasecmp $key stop_]} break

       append buff " $line"

       while {[llength $buff] >= $vCount} \
          {
           set str  [lrange $buff 0 [expr $vCount - 1]]
           set buff [lrange $buff $vCount end]

           set sampleVarName  None
           set sampleVarValue None

           foreach v $vPairs \
              {
               set vNew  [lindex $v 0]
               set vOld  [lindex $v 1]
               set $vNew [string toupper [lindex $str $vLoc($vNew)]]
              }

           if {[strcasecmp $pH None]} continue
           if {![strcmp $sampleVarValue .]} continue 
           if {[strcasecmp $sampleVarName  pH]}  continue

           if {![string is double $sampleVarValue]} continue

           set pH $sampleVarValue
          }
      }

   if {[llength $buff]} \
      {
       puts $errUnit "star2cs. Warning: Unused data near line $lineCount."
       puts $errUnit "star2cs. Data: $buff"
       incr warnCount
      }
}

#
# Extract contents for a sequence or shift loop.

proc getSeqCSLoop { inUnit vCount vPairs shiftFlag } \
{ 
   global seqInfo shiftInfo resList vLoc lineCount errUnit warnCount

   set buff ""

   while {[star_gets $inUnit origLine] >= 0} \
      {
       updateLineCount 

       set line [string trim $origLine]
       set n    [string length $line]
       set key  ""

       scan $line %s key
       set nn [string length $key]

       if {!$n || !$nn} continue

       if {![strcasecmp $key stop_]} break

       append buff " $line"

       while {[llength $buff] >= $vCount} \
          {
           set str  [lrange $buff 0 [expr $vCount - 1]]
           set buff [lrange $buff $vCount end]

           set resID    None
           set resName  None
           set atomName None
           set shift    9999.0

           foreach v $vPairs \
              {
               set vNew  [lindex $v 0]
               set vOld  [lindex $v 1]
               set $vNew [string toupper [lindex $str $vLoc($vNew)]]
              }

           if {![strcmp $resID "."] || ![llength $resID]} continue
           if {![strcasecmp $resID None]} continue
           if {![string is integer $resID]} continue

           if {[strcmp $resName "."] && [strcasecmp $resName None]} \
              {
               lappend seqInfo($resID) $resName
               if {$shiftFlag} {lappend seqInfo(cs,$resID) $resName}
              }

           lappend resList $resID

           if {![info exists shiftInfo($resID,resNameList]} \
              {
               set shiftInfo($resID,resNameList) "" 
              }

           if {$shiftFlag} \
              {
               if {![info exists shiftInfo($resID,atomList)]} \
                  {
                   set shiftInfo($resID,atomList) "" 
                  }

               if {![strcmp $shift "."] || ![strcasecmp $shift None]} continue
               if {![strcasecmp $atomName None]} continue
               if {![strcmp $atomName "."]} continue
         
               if {![string is double $shift]} continue

               lappend shiftInfo($resID,$atomName)   $shift

               set loc [lsearch -exact $shiftInfo($resID,atomList) $atomName]
               if {$loc < 0} {lappend shiftInfo($resID,atomList) $atomName}

               if {![strcasecmp $resName None]} continue
               if {![strcmp $resName "."]} continue

               set loc [lsearch -exact $shiftInfo($resID,resNameList) $resName]
               if {$loc < 0} {lappend shiftInfo($resID,resNameList) $resName}
              }
          }
      }

   if {[llength $buff]} \
      {
       puts $errUnit "star2cs. Warning: Unused data near line $lineCount."
       puts $errUnit "star2cs. Data: $buff"
       incr warnCount
      }
}

proc testLoopContents { vList vPairs } \
{
    set missing   0
    set found     0

    foreach v $vPairs \
       {
        set vNew  [lindex $v 0]
        set vOld  [lindex $v 1]

        if {[lsearch -exact $vList $vOld] < 0} \
           {
            incr missing
           } \
        else \
           {
            incr found
           }
       }

   return $missing
}

proc adjResName { resID resName } \
{
   global shiftInfo cysCB hispH pH errUnit verbose

   if {![strcasecmp $resName CYS] || ![strcasecmp $resName C]} \
      {
       if {![info exists shiftInfo($resID,CB)]} {return $resName}
       if {![llength $shiftInfo($resID,CB)]} {return $resName}

       set cb [lindex $shiftInfo($resID,CB) 0]

       if {$cb <= $cysCB} \
          {
           if {$verbose} \
              {
               puts $errUnit [format "star2cs.  Residue: %3d CYS Reduced" $resID]
              }

           return [string toupper $resName]
          } \
       else \
          {
           if {$verbose} \
              {
               puts $errUnit [format "star2cs.  Residue: %3d CYS Oxidized" $resID]
              }

           return [string tolower $resName]
          }
      } \
   elseif {![strcasecmp $resName HIS] || ![strcasecmp $resName H]} \
      {
       if {![strcasecmp hispH None]} {return $resName}
       if {![strcasecmp pH None]}    {return $resName}

       if {$pH <= $hispH} \
          {
           if {$verbose} \
              {
               puts $errUnit [format "star2cs.  Residue: %3d HIS Protonated" $resID]
              }

           return [string tolower $resName]
          } \
       else \
          {
           if {$verbose} \
              {
               puts $errUnit [format "star2cs.  Residue: %3d HIS Unprotonated" $resID]
              }

           return [string toupper $resName]
          }
      }

   return $resName
}

proc isOldStar { thisUnit } \
{
   set oldList {
                _Variable_type _Variable_value 
                _Residue_seq_code _Residue_label _Atom_name _Chem_shift_value
               }

   set newList {
                _Sample_condition_variable.Type _Sample_condition_variable.Val
                _Atom_chem_shift.Seq_ID _Atom_chem_shift.Comp_ID _Atom_chem_shift.Atom_ID _Atom_chem_shift.Val
               }

   set oldCount 0
   set newCount 0

   while {[gets $thisUnit lineOrig] >= 0} \
      {
       set line [string map {\" ' \$ % \\ / \{ ) \} ) \[ ( \] )} $lineOrig]
       set line [fixStarText $line]

       foreach v $oldList \
          {
           if {[string first $v $line] >= 0} {incr oldCount}
          }

       foreach v $newList \
          {
           if {[string first $v $line] >= 0} {incr newCount}
          }
      }

   seek $thisUnit 0

   if {$newCount > $oldCount} {return 0}

   return 1
}

#
#------------------------------------------------------------------------------

initResNames

set inUnit  stdin
set outUnit stdout

if {![strcasecmp $errName stderr] || ![strcasecmp $errName None]} \
   {
    set errName stderr
    set errUnit stderr
   } \
else \
   {
    set error [catch {set errUnit [open $errName w]}]
    
    if {$error} \
       {
        set errName stderr
        set errUnit stderr

        puts $errUnit "star2cs. Error Opening NMR-Star Conversion Log File."
        incr errCount
        star_exit
       }
   }
 
if {[llength $aNamesIn] != [llength $aNamesOut]} \
   {
    puts $errUnit "star2cs. Error: Atom Name Conversion List Mismatch:"
    puts $errUnit "star2cs.  Input Names, $nIn entries:   $aNamesIn" 
    puts $errUnit "star2cs.  Output Names, $nOut entries: $aNamesOut" 
    exit 1
   }

if {![strcasecmp $inName None]} \
   {
    puts $errUnit "star2cs. Error: No Input File Specified for NMR-Star Format Conversion."
    incr errCount 
    star_exit
   }

if {[strcasecmp $inName stdin]} \
   {
    set error [catch {set inUnit [open $inName r]}]

    if {$error} \
       {
        puts $errUnit "star2cs. Error Opening NMR-Star Format Input File."
        incr errCount
        star_exit
       }
   }

if {![strcasecmp $srcName Auto]} \
   {
    if {[strcasecmp $inName stdin]} \
       {
        set srcName [file root [file tail $inName]]
       } \
    else \
       {
        set srcName STAR 
       }
   }

if {[strcasecmp $refName None]} \
   {
    set error [getRefCS $refName refCSInfo]

    if {$error} \
       {
        puts $errUnit "star2cs. Warning: Error Opening Chemical Shift Limits File."
        puts $errUnit "star2cs. No chemical shift limit checking will be performed."
        puts $errUnit ""

        set refName None
        incr warnCount
       }
   } 

#
# Define mapping of BMRB variables to our variables:

if {![strcasecmp $oldFlag Auto]} \
   {
    set oldFlag [isOldStar $inUnit]
   }

if {$oldFlag} \
   {
    set vPairsForSample {
                         {sampleVarName   _Variable_type}
                         {sampleVarValue  _Variable_value}
                        }

    set vPairsForShift {
                        {resID    _Residue_seq_code}
                        {resName  _Residue_label}
                        {atomName _Atom_name}
                        {shift    _Chem_shift_value}
                       }

    set vPairsForSeq {
                      {resID    _Residue_seq_code}
                      {resName  _Residue_label}
                     }
   } \
else \
   {
    set vPairsForSample {
                         {sampleVarName   _Sample_condition_variable.Type}
                         {sampleVarValue  _Sample_condition_variable.Val}
                        }

    set vPairsForShift {
                        {resID    _Atom_chem_shift.Seq_ID}
                        {resName  _Atom_chem_shift.Comp_ID}
                        {atomName _Atom_chem_shift.Atom_ID}
                        {shift    _Atom_chem_shift.Val}
                       }

    set vPairsForSeq   {
                        {resID   _Entity_comp_index.ID}
                        {resName _Entity_comp_index.Comp_ID}
                       }
   }

foreach v $vPairsForSample \
   {
    set vNew [lindex $v 0]
    set vOld [lindex $v 1]

    set vEquiv($vOld) $vNew
    set vLoc($vNew)   -1
   }

foreach v $vPairsForShift \
   {
    set vNew [lindex $v 0]
    set vOld [lindex $v 1]

    set vEquiv($vOld) $vNew 
    set vLoc($vNew)   -1
   }

#
# Parse the file:

set frameName None
set resList   ""
set vList     ""

set minVal     0.0
set maxVal     0.0

set minRes       0
set maxRes       0
set thisMin      0
set thisMax      0

set seqLoopCount 0
set csLoopCount  0
set lineCount    0

if {$verbose} \
   {
    puts $errUnit "star2cs. Converting BMRB NMR-Star Format. Input: $inName"
   }
 
while {[star_gets $inUnit origLine] >= 0} \
   {
    updateLineCount

    set line [string trim $origLine]
    set n    [string length $line]
    set key  ""

    scan $line %s key
    set nn [string length $key]

    if {!$n || !$nn} continue

#
# Find any frame names:

    if {![strcasecmp $key _Saveframe_category] || [string first .Sf_category $key] >= 0} \
       {
        if {[llength $line] > 1} \
           {
            set frameName [lindex $line 1]
           } \
        else \
           {
            set frameName None
           }
       } 
#
# Find start of loop:

    if {[strcasecmp $key loop_]} continue

    set vList ""

    while {[star_gets $inUnit origLine] >= 0} \
       {
        updateLineCount

        set line [string trim $origLine]
        set n    [string length $line]
        set key  ""

        scan $line %s key
        set nn [string length $key]

        if {!$n || !$nn} break

        set c [string index $key 0]

        if {[strcmp $c "_"]} \
           {
            star_pushs $line
            break
           }

        set     vOld  $key
        lappend vList $vOld
       }

#
# Loop has shift info:

    set missingForShift [testLoopContents $vList $vPairsForShift] 

    if {!$missingForShift} \
       {
        if {$csLoopCount} continue

        set vCount [llength $vList]

        for {set i 0} {$i < $vCount} {incr i} \
           {
            set vOld [lindex $vList $i]
    
            if {![info exists vEquiv($vOld)]} continue
    
            set vNew $vEquiv($vOld)
            set vLoc($vNew) $i
           }

        getSeqCSLoop $inUnit $vCount $vPairsForShift 1
        incr csLoopCount

        if {$csLoopCount == 1 && [llength $resList]} \
           {
            set minRes [getMin $resList]
            set maxRes [getMax $resList]
           }

        continue
       }

#
# Loop has sequence info:

    set missingForSeq [testLoopContents $vList $vPairsForSeq] 

    if {!$missingForSeq} \
       {
        if {$seqLoopCount} continue

        set vCount [llength $vList]

        for {set i 0} {$i < $vCount} {incr i} \
           {
            set vOld [lindex $vList $i]

            if {![info exists vEquiv($vOld)]} continue

            set vNew $vEquiv($vOld)
            set vLoc($vNew) $i
           }

        getSeqCSLoop $inUnit $vCount $vPairsForSeq 0
        incr seqLoopCount

        if {$seqLoopCount == 1 && [llength $resList]} \
           {
            set minRes [getMin $resList]
            set maxRes [getMax $resList]
           }

        continue
       }

#
# Loop has sample conditions:

    if {[strcasecmp $pH None]} continue
    if {[strcasecmp $frameName sample_conditions]} continue 

    set missingForSample [testLoopContents $vList $vPairsForSample]

    if {!$missingForSample} \
       {
        set vCount [llength $vList]

        for {set i 0} {$i < $vCount} {incr i} \
           {
            set vOld [lindex $vList $i]

            if {![info exists vEquiv($vOld)]} continue

            set vNew $vEquiv($vOld)
            set vLoc($vNew) $i
           }

        getSampleLoop $inUnit $vCount $vPairsForSample
       } 
   }

#
# Now that file is parsed, test sequence and shift information:

    if {!$minRes && !$maxRes && [llength $resList]} \
       {
        set minRes [getMin $resList]
        set maxRes [getMax $resList]
       }

    if {[llength $resList]} \
       { 
        set thisMin [getMin $resList]
        set thisMax [getMax $resList]
       }

    if {!$seqLoopCount && !$csLoopCount || ![llength $resList]} \
       {
        puts $errUnit "star2cs. Error: No Properly Formatted Sequence or Shift Data in NMR-Star Input."

        incr errCount

        showFormatInfo
        star_exit
       }
 
    if {$seqLoopCount > 1 || $csLoopCount > 1 || \
         $thisMin != $minRes || $thisMax != $maxRes} \
       {
        puts $errUnit "star2cs. Warning: Multiple Sequences or Shift Lists in NMR-Star Format Input."

        incr warnCount
       }

    for {set resID $minRes} {$resID <= $maxRes} {incr resID} \
       {
        if {![info exists seqInfo($resID)]} \
           {
            puts $errUnit "star2cs. Warning: No Sequence Information for Residue $resID."
            incr warnCount
            continue
           }

        set rList [listUniq $seqInfo($resID)]

        if {[llength $rList] > 1} \
           {
            puts $errUnit "star2cs. Warning: Multiple Residue Types for Residue $resID:"
            puts $errUnit "star2cs.  $rList"
            incr warnCount
           }
  
        if {![info exists shiftInfo($resID,atomList)]} continue
 
        set resName [lindex $rList 0]
 
        foreach atomName $shiftInfo($resID,atomList) \
           {
            if {![info exists shiftInfo($resID,$atomName)]} continue

            if {![strcasecmp $maxShiftDev None] && \
                 [llength $shiftInfo($resID,$atomName)] > 1} \
               {
                set dev [getStdDev $shiftInfo($resID,atomList)]

                if {$dev > $maxShiftDev} \
                   {
                    puts $errUnit "star2cs. Warning: Multiple Shifts for $resID $resName $atomName:"
                    puts $errUnit "star2cs.  $shiftInfo($resID,$atomName)"
                    incr warnCount
                   }
               }
           }
       }
 
#
# Adjust and Report sequence:

   if {[strcasecmp $outName None] && [strcasecmp $outName stdout]} \
      {
       set error [catch {set outUnit [open $outName w]}]
   
       if {$error} \
          {
           puts $errUnit "star2cs. Error Opening NMR-Star Conversion Output File."
           incr errCount
           star_exit
          }
      }

    if {[string is double $pH]} \
       {
        set pHStr $pH
       } \
    else \
       {
        set pHStr Unknown
       }

    if {$verbose} \
       {
        puts $errUnit "star2cs. Residues $minRes to $maxRes pH: $pHStr"
       }

    set nMiss  0

    for {set resID $minRes} {$resID <= $maxRes} {incr resID} \
       {
        if {![info exists seqInfo($resID)]} {incr nMiss}
       }

    if {$showREMFlag && [strcasecmp $outName None]} \
       {
        puts $outUnit ""

        if {$showCSFlag} \
           {
            puts $outUnit "REMARK Conversion of BMRB NMR-Star Format Chemical Shifts"
            puts $outUnit "REMARK Input: $inName pH: $pHStr"
           } \
        else \
           {
            puts $outUnit "REMARK Conversion of BMRB NMR-Star Format Sequence Data"
           }

        if {[strcasecmp $titleName None]} \
           {
            puts $outUnit "REMARK Input: $titleName"
           }

        puts $outUnit ""
       }

    if {$showSeqFlag && [strcasecmp $outName None]} \
       {
        if {$fullSeqFlag} \
           {
            puts $outUnit "DATA MISSING_RES_COUNT $nMiss"
            puts $outUnit "DATA FIRST_RESID       $minRes"
           } \
        else \
           {
            puts $outUnit "DATA FIRST_RESID $minRes"
           }

        puts $outUnit ""

        set j 0

        for {set resID $minRes} {$resID <= $maxRes} {incr resID} \
           {
            if {[info exists seqInfo($resID)]} \
               {
                set resName  [adjResName $resID [lindex $seqInfo($resID) 0]]
                set resName1 [getResName1 $resName X]
               } \
            else \
               {
                set resName  XXX
                set resName1 X 
               }

            set seqInfo($resID) $resName

            if {!$j} {puts -nonewline $outUnit "DATA SEQUENCE "}

            puts -nonewline $outUnit $resName1
            incr j

            if {$j == 50} \
               {
                puts $outUnit ""
                set j 0
               } \
            elseif {!($j % 10)} \
               {
                puts -nonewline $outUnit " "
               }
          }

        puts $outUnit ""
        puts $outUnit ""
       }

#
# Report shifts:

    if {$hdrFlag && [strcasecmp $outName None]} \
       {
        puts -nonewline $outUnit "VARS   RESID"

        if {[strcasecmp $res1Var None]} {puts -nonewline $outUnit " $res1Var"}
        if {[strcasecmp $res3Var None]} {puts -nonewline $outUnit " $res3Var"}

        puts -nonewline $outUnit " ATOMNAME SHIFT"

        if {[strcasecmp $srcName None]} {puts -nonewline $outUnit " SOURCE"}

        puts $outUnit ""

        puts -nonewline $outUnit "FORMAT %4d  "

        if {[strcasecmp $res1Var None]} {puts -nonewline $outUnit " %1s    "}
        if {[strcasecmp $res3Var None]} {puts -nonewline $outUnit " %4s    "}
 
        puts -nonewline  $outUnit " %-4s     %8.3f"

        if {[strcasecmp $srcName None]} {puts -nonewline $outUnit " %s"}

        puts $outUnit ""
       }

    set minVal 0.0
    set maxVal 0.0

    if {$showCSFlag && [strcasecmp $outName None]} \
       {
        puts $outUnit ""

        for {set resID $minRes} {$resID <= $maxRes} {incr resID} \
           {
            if {[info exists seqInfo($resID)]} \
               {
                set resName  [lindex $seqInfo($resID) 0]
                set resName1 [getResName1 $resName X]
               } \
            else \
               {
                set resName  XXX
                set resName1 X
               }

            if {![info exists shiftInfo($resID,atomList)]} continue

            foreach atomName $shiftInfo($resID,atomList) \
               {
                if {![starAtomSelected $atomName]} continue

                if {![info exists shiftInfo($resID,$atomName)]} continue
                if {![llength $shiftInfo($resID,$atomName)]} continue
    
                set shift [lindex $shiftInfo($resID,$atomName)  0]

                if {[strcasecmp $refName None]} \
                   {
                    set status [refCSTest $resName $atomName $shift refCSInfo]

                    if {!$status} \
                       {
                        set msg1 "star2cs. Warning: Chemical Shift is Outside Expected Limits:"
                        set fmt  "star2cs. Residue %3d %3s %-4s Shift: %6.2f ppm" 
                        set msg2 [format $fmt $resID $resName $atomName $shift]

                        set fmt  "Limits %6.2f to %6.2f ppm" 

                        set msg3 [format $fmt $refCSInfo(minVal) $refCSInfo(maxVal)]

                        puts $errUnit $msg1
                        puts $errUnit "$msg2 $msg3"

                        puts $outUnit "# Expected $msg3:"
                        puts -nonewline $outUnit "#"

                        incr warnCount
                       }
                   }
    
                puts -nonewline $outUnit " [format %4d $resID]"

                if {[strcasecmp $res1Var None]} \
                   {
                    puts -nonewline $outUnit [format " %1s" $resName1]
                   }

                if {[strcasecmp $res3Var None]} \
                   {
                    puts -nonewline $outUnit [format " %4s" $resName]
                   }

                set newName [fixAtomName $resName $atomName]

                puts -nonewline $outUnit [format " %-4s %8.3f" $newName $shift]

                if {[strcasecmp $srcName None]} \
                   {
                    puts -nonewline $outUnit $srcName
                   }

                puts $outUnit ""
               }
           }
       }

   if {[strcasecmp $outName None] && [strcasecmp $outName stdout]} \
      {
       close $outUnit
      }

   if {[strcasecmp $errName None] && [strcasecmp $errName stderr]} \
      {
       close $errUnit
      }

star_exit
