#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set auto_path [linsert $auto_path 0 $env(NMRBASE)/dynamo/tcl]

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
#-------------------------------------------------------------------------------

set talosName   pred.tab
set pdbName     None 
set csName      None 
set segName     None 
set id          1
set group       1
set hdrFlag     1
set keepBadFlag 0

set factor    2.0
set fc        1.0
set dPhiMin   0.0
set dPsiMin   0.0
set w         1.0
set s         1.0

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert TALOS prediction output to DYNAMO Torsion Table:"
    puts " -in       talosName  \[pred.tab\]  TALOS Prediction Output."
    puts " -seg      segName    \[None\]      Default Seg Name."
    puts " -id       firstID    \[1\]         First Restraint ID."
    puts " -fc       forceConst \[1.0]\       Force Constant Value."
    puts " -dPhiMin  pMin       \[0.0]\       Minimum Value for DPHI Input."
    puts " -dPsiMin  pMin       \[0.0]\       Minimum Value for DPSI Input."
    puts " -f        factor     \[2.0]\       Factor for +/- Angle Range."
    puts " -hdr                             Print Header (Default)." 
    puts " -nohdr                           No Header."
    puts " -keepBad                         Use All TALOS Predictions."
    puts "Sequence Info (Use Only One):"
    puts " -cs       csName     \[None\]      TALOS Chemical Shift Table Input."
    puts " -pdb      pdbName    \[None\]      PDB Input File."
    exit 0
   }

getArgD $ARGV -in      talosName
getArgD $ARGV -pdb     pdbName
getArgD $ARGV -cs      csName
getArgD $ARGV -seg     segName
getArgD $ARGV -id      id
getArgD $ARGV -fc      fc
getArgD $ARGV -f       factor
getArgD $ARGV -dPhiMin dPhiMin
getArgD $ARGV -dPsiMin dPsiMin

if {[flagLoc $ARGV -hdr]}   {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]} {set hdrFlag 0}

if {[flagLoc $ARGV -keepBad]} {set keepBadFlag 1}

dynInit -fast

showDynLogo

#
#-------------------------------------------------------------------------------

proc getThisRes { resID } \
{
   global pdbTab seqList r1 rN

   set resName NONE

   if {$pdbTab} \
      {
       set resName [getNthResName $pdbTab $resID]
      } \
   else \
      {
       if {$resID >= $r1 && $resID <= $rN} \
          {
           set resType [lindex $seqList [expr $resID - $r1]]
           set resName [getResName3 $resType]
          }
      }

   return $resName
}

proc printThisHdr {} \
{
   global hdrFlag

   if {!$hdrFlag} return

    set varStr "VARS   INDEX GROUP"
    set fmtStr "FORMAT %4d   %2d"

    foreach i {I J K L} \
       {
        append varStr " RESID_$i RESNAME_$i ATOMNAME_$i SEGNAME_$i"
        append fmtStr " %4d %4s %4s %4s"
       }

    append varStr " ANGLE ANGLE_LO ANGLE_HI FC W S"
    append fmtStr " %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f"

    puts "REMARK TALOS Results"
    puts ""
    puts $varStr
    puts $fmtStr
    puts ""
}

proc printThisAngle { aName id aVal daVal aLo aHi } \
{
    global ATOMNAME_I ATOMNAME_J ATOMNAME_K ATOMNAME_L
    global RESNAME_I  RESNAME_J  RESNAME_K  RESNAME_L
    global SEGNAME_I  SEGNAME_J  SEGNAME_K  SEGNAME_L
    global RESID_I    RESID_J    RESID_K    RESID_L

    global RESNAME RESID
    global fc w s group

    wm title . "Residue $RESID $aName $aVal"
    update

    set str ""

    append str [format " %3d %2d" $id $group]

    append str [format " %3d %3s %-4s" $RESID_I $RESNAME_I $ATOMNAME_I]
    append str [format " %4s" $SEGNAME_I]

    append str [format " %3d %3s %-4s" $RESID_J $RESNAME_J $ATOMNAME_J]
    append str [format " %4s" $SEGNAME_J]

    append str [format " %3d %3s %-4s" $RESID_K $RESNAME_K $ATOMNAME_K]
    append str [format " %4s" $SEGNAME_K]

    append str [format " %3d %3s %-4s" $RESID_L $RESNAME_L $ATOMNAME_L]
    append str [format " %4s" $SEGNAME_L]

    set fmt " % 7.2f % 7.2f % 7.2f %.3f %.2f %.2f"
    append str [format $fmt $aVal $aLo $aHi $fc $w $s]

    puts $str
}

#
#-------------------------------------------------------------------------------

set seqList   "" 
set pdbTab    0
set r1        0
set rN        0

initResNames

if {![file exists $talosName]} \
   {
    puts stderr "Error Finding TALOS prediction file $talosName"
    exit 1
   } 

if {[file exists $pdbName]} \
   {
    set pdbTab [gdbRead table -in $pdbName -pdb]
   }

if {[file exists $csName]} \
   {
    set csTab   [gdbRead table -in $csName]
    set seqList [getSeqList $csTab r1 rN]
   }

if {!$pdbTab && ![llength $seqList]} \
   {
    puts stderr "Error: No Sequence Input; use the -cs or -pdb arguments."
    exit 1
   }

set predTab [gdbRead table -in $talosName]
set eList   [gdbGetInfo $predTab -parms entries]

#-------------------------------------------------------------------------------
# phi(i): C(i-1) N(i)  CA(i) C(i)

printThisHdr

set SEGNAME_I  $segName
set SEGNAME_J  $segName
set SEGNAME_K  $segName
set SEGNAME_L  $segName

set ATOMNAME_I C
set ATOMNAME_J N
set ATOMNAME_K CA
set ATOMNAME_L C 
 
foreach entry $eList \
   {
    foreach v {RESID RESNAME PHI PSI DPHI DPSI CLASS} \
       {
        set $v [gdbGet $entry $v]
       }

    if {!$keepBadFlag} \
       {
        if {![strcasecmp $CLASS Bad]}  continue
        if {![strcasecmp $CLASS Dyn]}  continue
        if {![strcasecmp $CLASS Warn]} continue
        if {![strcasecmp $CLASS None]} continue
       }
 
    set  str ""

    if {$DPHI < $dPhiMin} {set DPHI $dPhiMin}

    set DPHI [expr $factor*$DPHI]

    set aLo  [expr $PHI - $DPHI]
    set aHi  [expr $PHI + $DPHI]

    set RESID_I   [expr $RESID - 1]
    set RESID_J   $RESID
    set RESID_K   $RESID
    set RESID_L   $RESID

    set RESNAME_I [getThisRes $RESID_I]
    set RESNAME_J [getThisRes $RESID_J]
    set RESNAME_K [getThisRes $RESID_K]
    set RESNAME_L [getThisRes $RESID_L]

    printThisAngle Phi $id $PHI $DPHI $aLo $aHi

    incr id
   }

puts ""

#-------------------------------------------------------------------------------
# psi(i): N(i) CA(i) C(i)  N(i+1)

set ATOMNAME_I N
set ATOMNAME_J CA 
set ATOMNAME_K C
set ATOMNAME_L N 
 
foreach entry $eList \
   {
    foreach v {RESID RESNAME PHI PSI DPHI DPSI CLASS} \
       {
        set $v [gdbGet $entry $v]
       }

    if {!$keepBadFlag} \
       {
        if {![strcasecmp $CLASS Bad]}  continue
        if {![strcasecmp $CLASS Dyn]}  continue
        if {![strcasecmp $CLASS Warn]} continue
        if {![strcasecmp $CLASS None]} continue
       }
 
    set  str ""

    if {$DPSI < $dPsiMin} {set DPSI $dPsiMin}

    set DPSI [expr $factor*$DPSI]
    set aLo  [expr $PSI - $DPSI]
    set aHi  [expr $PSI + $DPSI]

    set RESID_I   $RESID
    set RESID_J   $RESID
    set RESID_K   $RESID
    set RESID_L   [expr $RESID + 1]

    set RESNAME_I [getThisRes $RESID_I]
    set RESNAME_J [getThisRes $RESID_J]
    set RESNAME_K [getThisRes $RESID_K]
    set RESNAME_L [getThisRes $RESID_L]

    printThisAngle Psi $id $PSI $DPSI $aLo $aHi

    incr id
   }

exit
