#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert TALOS shift table to multi-column shift table:"
    puts " -in      csName    \[csObs.tab\]  TALOS Format Shift Table."
    puts " -out     outName   \[ass.tab\]    NMRPipe Format Assignment Table."
    puts " -prefix  prefix    \[None\]       Optional Chain Prefix for ASS Output Values." 
    puts " -postfix postfix   \[None\]       Optional Chain Postfix for ASS Output Values."
    puts " -prev                           Include Values for CA_PREV CB_PREV C_PREV."
    puts " -dash                           Use Dash Between Residue ID and Residue Name."
    puts "Notes:"
    puts " 1. Input table is expected to have DATA SEQUENCE lines."
    puts " 2. Input table is expected to have RESID RESNAME ATOMNAME SHIFT."
    puts " 3. Each line of the output table will give the backbone chemical"
    puts "    shifts for a given residue, HN N HA CA CB C and OTHER."
    exit 0
   }

set inName   csObs.tab
set outName  ass.tab
set prefix   None
set postfix  None

set prevFlag 0
set dashFlag 0

getArgD $ARGV -in      inName
getArgD $ARGV -out     outName
getArgD $ARGV -prefix  prefix
getArgD $ARGV -postfix postfix

if {[flagLoc $ARGV -dash]}   {set dashFlag 1}
if {[flagLoc $ARGV -nodash]} {set dashFlag 0}

if {[flagLoc $ARGV -prev]}   {set prevFlag 1}
if {[flagLoc $ARGV -noprev]} {set prevFlag 0}

#
#--------------------------------------------------------------------------

set fmt(HA)    %8.3f
set fmt(HN)    %8.3f
set fmt(CA)    %7.2f
set fmt(CB)    %7.2f
set fmt(C)     %7.2f
set fmt(N)     %7.2f

initResNames

if {![file exists $inName]} \
   {
    puts "Error Findinging Input Table $inName"
    exit
   }

set tTab [gdbRead table -in $inName]

set error [catch {set outUnit [open $outName w]}]

if {$error} \
   {
    puts "Error Opening Output Table $outName"
    exit
   }

getKeyData $tTab tInfo

set seqList [getSeqList $tTab r1 rN]


set eList   [gdbGetInfo $tTab -parms entries]
set resList ""

foreach entry $eList \
   {
    set resID    [gdbGet $entry RESID]
    set resName  [lindex $seqList [expr $resID - $r1]]
    set atomName [gdbGet $entry ATOMNAME]
    set shift    [gdbGet $entry SHIFT]

    if {$shift > 999} continue

    if {[lsearch -exact $resList $resID] < 0} \
       {
        foreach a {HA CA CB C N other} \
           {
            set csInfo($resID,$a) ""
           }

        set csInfo($resID,resName) [getResName3 $resName]

        lappend resList $resID
       }

    switch -glob $atomName \
       {
        HN -
        HA -
        CA -
        CB -
        C  -
        N  {lappend csInfo($resID,$atomName) $shift}

        HA* {
             lappend csInfo($resID,HA) $shift
             lappend csInfo($resID,$atomName) $shift
             lappend csInfo($resID,other) [format "%s=%.3f" $atomName $shift]
            } 
       }
   }

puts $outUnit "DATA FIRST_RESID $r1"
puts $outUnit ""

foreach str $tInfo(SEQUENCE) \
   {
    puts $outUnit "DATA SEQUENCE $str"
   }

puts $outUnit ""

if {$prevFlag} \
   {
    puts $outUnit "VARS    ASS RESID RESNAME HN    N     HA    CA    CB    C     CA_PREV CB_PREV  C_PREV OTHER"
    puts $outUnit "FORMAT  %8s %4d   %s      %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f   %8.3f    %8.3f  %s" 
   } \
else \
   {
    puts $outUnit "VARS    ASS RESID RESNAME HN    N     HA    CA    CB    C     OTHER"
    puts $outUnit "FORMAT  %8s %4d   %s      %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %s" 
   }

puts $outUnit ""

foreach resID $resList \
   {
    if {$dashFlag} \
       {
        set ass ${resID}-$csInfo($resID,resName)
       } \
    else \
       {
        set ass $resID$csInfo($resID,resName)
       }

    if {[strcasecmp $prefix  None]} {set ass $prefix$ass}
    if {[strcasecmp $postfix None]} {set ass $ass$postfix}

    set prevID [expr $resID - 1]

    puts -nonewline $outUnit [format " %8s %4d %s" $ass $resID $csInfo($resID,resName)]

    foreach a {HN N HA CA CB C} \
       {
        set s 9999.0

        if {[info exists csInfo($resID,$a)]} \
           {
            if {[llength $csInfo($resID,$a)]} \
               {
                set s [getAvg $csInfo($resID,$a)]
               }
           } 

        puts -nonewline $outUnit [format " $fmt($a)" $s]
       }

    if {$prevFlag} \
       {
        foreach a {CA CB C} \
           {
            set s 9999.0

            if {[info exists csInfo($prevID,$a)]} \
               {
                if {[llength $csInfo($prevID,$a)]} \
                   {
                    set s [getAvg $csInfo($prevID,$a)]
                   }
               }

            puts -nonewline $outUnit [format " $fmt($a)" $s]
           }    
       }

    if {[llength $csInfo($resID,other)]} \
       {
        puts $outUnit " [join $csInfo($resID,other) ,]"
       } \
    else \
       {
        puts $outUnit " None" 
       }
   }

close $outUnit

exit
