#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# titrView.tcl: auxillary modules for viewing 2D HSQC Titrations.

proc initTitrVars {} \
{
   global objInfo

   set obj titr 

   setFonts

   set objInfo($obj,initFlag)    1
   set objInfo($obj,saveNeeded)  0

#
# PPM limits for postage stamp plots:

   set objInfo($obj,regXW)       0.20
   set objInfo($obj,regYW)       1.00

#
# Pixel size of initial TitrView window and postage stamp plots:

   set objInfo($obj,viewXSize)  1020 
   set objInfo($obj,viewYSize)   400
   set objInfo($obj,viewYMax)  32500
   set objInfo($obj,cellXSize)   100
   set objInfo($obj,cellYSize)   100

#
# Initializations:

   set objInfo($obj,dx)          10
   set objInfo($obj,dy)          10
   set objInfo($obj,xAdj)        15
   set objInfo($obj,yAdj)        20

   set objInfo($obj,resizeProc)  resizeTitrView
   set objInfo($obj,redrawProc)  showTitrView
   set objInfo($obj,specCount)   0
   set objInfo($obj,pkCount)     0
   set objInfo($obj,pkList)      ""
   set objInfo($obj,regID)       0
   set objInfo($obj,ovCellList)  ""
   set objInfo($obj,cellList)    ""
   set objInfo($obj,roiList)     ""
   set objInfo($obj,cellCount)   0
   set objInfo($obj,roiCount)    0
   set objInfo($obj,activeCell)  0

   set objInfo($obj,showPkID)    1 
   set objInfo($obj,showAss)     0 

   set objInfo($obj,dirName)     titr
   set objInfo($obj,outName)     titrView.tab
   set objInfo($obj,clearFlag)   1

   set objInfo($obj,winTitle) "TitrView 2023.355.17.09"

   append objInfo($obj,winTitle) " \[Adjust Position\]"
   append objInfo($obj,winTitle) " \[Insert Peak\]"
   append objInfo($obj,winTitle) " \[Delete Peak\] Active: Scroll Shift-Scroll Arrows"
}

proc titrView args \
{
   global objInfo

   set obj titr

   set objInfo($obj,selList) ""

   if {[flagLoc $args -help]} \
      {
       showTitrArgHelp
       return
      }

   if {[flagLoc $args -quit]} \
      {
       titrViewDone
       return
      }

   set parent [lindex $args 0]
   set sx1    [lindex $args 1]
   set sxn    [lindex $args 2]
   set sy1    [lindex $args 3]
   set syn    [lindex $args 4]

   if {[llength $args] > 5} {set objInfo($obj,selList) [lindex $args 5]}

   showVerb "TitrView: $parent"

   set objInfo($obj,specX1)  $sx1
   set objInfo($obj,specXN)  $sxn
   set objInfo($obj,specY1)  $sy1
   set objInfo($obj,specYN)  $syn

   if {![info exists objInfo($obj,initFlag)]} \
      {
       initTitrVars
       initTitrArgs
       initTitrDir

       initTitrSpecWindow $parent
      } \
   else \
      {
       set tmpName [getTitrSaveName]

       if {[okSaveDiscard $tmpName $objInfo(titr,window)]} \
          {
           saveTitrCurves $tmpName
          } \
       else \
          {
           unhiliteTitrView titr_$objInfo($obj,regID)_propose
          }
      }

   incr objInfo($obj,regID)

   hiliteTitrView   titr_$objInfo($obj,regID)_propose $objInfo(color,propose)
   clearTitrView

   set error [mapTitrPeaks]
   if {$error} return

   set error [readTitrView]
   if {$error} return

   drawTitrView
   drawTitrLabels
}

proc initTitrDir {} \
{
   global objInfo

   set obj titr

   if {$objInfo($obj,clearFlag)} \
      {
       catch "file delete -force $objInfo($obj,dirName)"
      }

   if {![file isdirectory $objInfo($obj,dirName)]} \
      {
       set error [catch "file mkdir $objInfo($obj,dirName)"]

       if {$error} \
          {
           showError TitrView "Error Creating Output Directory $objInfo($obj,dirName)"
           return 1
          }
      }
}

proc showTitrView {} \
{
   clearTitrView
   drawTitrView
   drawTitrLabels
}

proc titrViewExit {} \
{
   titrViewDone
   exit 
}

proc titrViewDone {} \
{
   global objInfo

   set obj titr

   if {![info exists objInfo($obj,initFlag)]} return

   saveAllTitrCurves
}

proc showTitrArgHelp {} \
{
   global objInfo ARGC ARGV

   set obj titr 

   puts "TitrView Arguments:"
   puts " -regX      regXPPM     Peak Region X Width +/- PPM."
   puts " -regY      regYPPM     Peak Region Y Width +/- PPM."
   puts " -cellX     cellXSize   Peak Window X Size, Pixels"
   puts " -cellY     cellXSize   Peak Window Y Size, Pixels"
   puts " -viewX     viewXSize   TitrView Window X Size, Pixels"
   puts " -viewY     viewXSize   TitrView Window Y Size, Pixels"
   puts " -titrDir   dirName     TitrView Intermediate Output Directory."
   puts " -titrName  tabName     TitrView Final Output File."
#  puts " -clearTitr             Clear Existing Intermediate Data on Start."
}

proc initTitrArgs {} \
{
   global objInfo ARGC ARGV

   set obj titr 

   if {[flagLoc $ARGV -help]} return 

   getArgD $ARGV -regX objInfo($obj,regXW)
   getArgD $ARGV -regY objInfo($obj,regYW)

   getArgD $ARGV -viewX objInfo($obj,viewXSize)
   getArgD $ARGV -viewY objInfo($obj,viewYSize)
   getArgD $ARGV -cellX objInfo($obj,cellXSize)
   getArgD $ARGV -cellY objInfo($obj,cellYSize)

   getArgD $ARGV -titrDir  objInfo($obj,dirName)
   getArgD $ARGV -titrName objInfo($obj,outName)

   if {[flagLoc $ARGV -clearTitr]}   {set objInfo($obj,clearFlag) 1}
   if {[flagLoc $ARGV -noclearTitr]} {set objInfo($obj,clearFlag) 0}
}

proc unhiliteTitrView { id } \
{
   delLabelItem $id
}

proc hiliteTitrView { id color } \
{
   global objInfo

   set obj titr 

   set sx1 $objInfo($obj,specX1)
   set sxn $objInfo($obj,specXN)
   set sy1 $objInfo($obj,specY1)
   set syn $objInfo($obj,specYN)

   addLabelItem "box $id $sx1 $sxn $sy1 $syn 1 $color"
}

proc acceptTitrView {} \
{
   global objInfo

   set obj titr 

   hiliteTitrView titr_$objInfo($obj,regID)_accept $objInfo(color,accept)
}

proc getTitrSaveName {} \
{
   global objInfo

   set obj titr

   for {set i 0} {$i < 5} {incr i} \
      {
       set saveName $objInfo($obj,dirName)/[lindex [sysTime] 0].tab

       if {![file exists $saveName]} \
          {
           return $saveName
          }

       catch "exec sleep 1"
      }

   return $saveName
}

proc saveAllTitrCurves {} \
{
   global objInfo

   set obj titr

   set tmpName [getTitrSaveName]

   saveTitrCurves $tmpName

   set nameList [lsort -dictionary [glob -nocomplain $objInfo($obj,dirName)/*]]

   if {![llength $nameList]} \
      {
       showError TitrView "Error: No titration data was found."
       return
      }

   set types \
      {
       { {tab}        {.tab} }
       { {text}       {.txt} }
       { {All Files}  *      }
      }

   set outName [tk_getSaveFile -title "Scroll Script" -filetypes $types -initialfile $objInfo($obj,outName)]
   if {![llength $outName]} return

#  set status [okWrite $outName $objInfo(titr,window)]
#  if {!$status} return

   set error [catch {set outUnit [open $outName w]}]

   if {$error} \
      {
       showError TitrView "Error opening output table $outName"
       return 1
      }

   set tmpList ""
   set tabList ""
   set idList  ""
   
   foreach inName $nameList {lappend tmpList [file tail $inName]}

   set nameList [lsort -integer $tmpList]

   foreach inName $nameList \
      {
       set inTab [gdbRead table -in $objInfo($obj,dirName)/$inName]
       set eList [gdbGetInfo $inTab -parms entries]

       foreach entry $eList \
          {
           set id [gdbGet $entry INDEX]
         
           set tInfo($id) $entry

           if {[lsearch -exact $idList $id] < 0} {lappend idList $id}
          } 

       lappend tabList $inTab
      }

   set idList [lsort -integer $idList]

   writeTitrHdr $outUnit $objInfo($obj,specCount) [llength $idList] 

   foreach id $idList \
      {
       for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
          {
           set entry $tInfo($id)

           set ass  [gdbGet $entry ASS]
           set x    [gdbGet $entry X_PPM$i]
           set y    [gdbGet $entry Y_PPM$i]
           set hi   [gdbGet $entry Z_A[expr $i - 1]]
           set mode [gdbGet $entry MODE$i]

           if {$i == 1} \
              {
               catch {puts -nonewline $outUnit [format " %4d %8s" $id $ass]}
              }

           set fmt " % 9.4f % 9.4f % .4e %-8s"

           catch {puts -nonewline $outUnit [format $fmt $x $y $hi $mode]}
          }

       puts $outUnit [format " % .4e" [gdbGet $entry HEIGHT]]
      }

   gdbFree $tabList

   close $outUnit

   set txt "Saved $objInfo($obj,pkCount) entries in file \"$outName\"."

   tk_messageBox -type ok -title Save -message $txt -icon info -parent $objInfo(titr,window)
}

proc writeTitrHdr { outUnit specCount pkCount } \
{
   global objInfo

   set obj titr

   puts $outUnit "# TitrView Table Created on [sysTime %Y.%j.%H.%M]"
   puts $outUnit ""
   puts $outUnit "DATA SPEC_COUNT $specCount"
   puts $outUnit "DATA PEAK_COUNT $pkCount"
   puts $outUnit ""

   puts -nonewline $outUnit "DATA TAU   "

   foreach c $objInfo(data,tauList) \
      {
       puts -nonewline $outUnit [format " %.4f" $c]
      }

   puts $outUnit ""

   puts -nonewline $outUnit "DATA CONC_L"

   foreach c $objInfo(data,user1List) \
      {
       puts -nonewline $outUnit [format " %.4f" $c]
      }

   puts $outUnit ""

   puts -nonewline $outUnit "DATA CONC_P"

   foreach c $objInfo(data,user2List) \
      {
       puts -nonewline $outUnit [format " %.4f" $c]
      }

   puts $outUnit ""
   puts $outUnit ""

   puts -nonewline $outUnit "VARS   INDEX ASS"

   for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
      {
       puts -nonewline $outUnit " X_PPM$i Y_PPM$i Z_A[expr $i -1 ] MODE$i"
      }

   puts $outUnit " HEIGHT"

   puts -nonewline $outUnit "FORMAT %4d %8s"

   for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
      {
       puts -nonewline $outUnit " %9.4f %9.4f %+e %8s"
      }

   puts $outUnit " %e"
   puts $outUnit ""
}
 
proc saveTitrCurves { outName } \
{
   global objInfo

   set obj titr 

   set error [catch {set outUnit [open $outName w]}]

   if {$error} \
      {
       showError TitrView "Error opening output table $outName"
       return 1
      }

   writeTitrHdr $outUnit $objInfo($obj,specCount) $objInfo($obj,pkCount)

   for {set j 1} {$j <= $objInfo($obj,pkCount)} {incr j} \
      {
       set firstHi 0.0

       for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
          {
           set cellID   $objInfo($obj,cellGrid,$i,$j)
           set cellInfo $objInfo($obj,cellInfo,$cellID)
           set n        [llength $cellInfo]

           if {$n < 12} \
              {
               set id   0
               set ass  None
               set x    0.0
               set y    0.0
               set mode None
               set hi   0.0
              } \
           else \
              {
               set id   [getNamedItem id   $cellInfo]
               set ass  [getNamedItem ass  $cellInfo]
               set x    [getNamedItem xPPM $cellInfo]
               set y    [getNamedItem yPPM $cellInfo]
               set mode [getNamedItem mode $cellInfo]
               set hi   [getNamedItem hi   $cellInfo]
              }

           if {$i == 1} \
              {
               catch {puts -nonewline $outUnit [format " %4d %8s" $id $ass]}
               set firstHi $hi
              }

           set fmt " % 9.4f % 9.4f % .4e %-8s"

           catch {puts -nonewline $outUnit [format $fmt $x $y [safeDiv $hi $firstHi] $mode]}
          }

        puts $outUnit [format " % .4e" $firstHi]
       }

   close $outUnit
}

proc readTitrView {} \
{
   global objInfo

   set obj titr 

   set parent $objInfo($obj,parent)
   set j 1

   if {!$objInfo($obj,pkCount)} \
      {
       for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
          {
           set objInfo($obj,inName$i)   $objInfo($obj,inName$i)
           set objInfo($obj$j,inName$i) $objInfo($obj,inName$i)
           set objInfo($obj$j,window)   $objInfo($obj,window) 
           set objInfo($obj$j,roi$i)    0

           set objInfo($obj$j,hi$i)       $objInfo($parent,hi$i)

           set objInfo($obj$j,$i,specX1)  $objInfo($obj,specX1)
           set objInfo($obj$j,$i,specXN)  $objInfo($obj,specXN)
           set objInfo($obj$j,$i,specY1)  $objInfo($obj,specY1)
           set objInfo($obj$j,$i,specYN)  $objInfo($obj,specYN)
          }

       set objInfo($obj,pkList)  "0" 
       set objInfo($obj,pkCount) 1

       wm title $objInfo($obj,window) "Reading Region $j of $objInfo($obj,pkCount)"

       readView $obj$j

       incr objInfo($obj,roiCount)

       return 0
      }

   foreach entry $objInfo($obj,pkList) \
      {
       for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
          {
           set objInfo($obj,inName$i)   $objInfo($obj,inName$i)
           set objInfo($obj$j,inName$i) $objInfo($obj,inName$i)
           set objInfo($obj$j,window)   $objInfo($obj,window) 
           set objInfo($obj$j,hi$i)     $objInfo($parent,hi$i)
           set objInfo($obj$j,roi$i)    0
          }

       set objInfo($obj$j,parent)  $objInfo($obj,parent)
       set objInfo($obj$j,window)  $objInfo($obj,window)

       wm title $objInfo($obj,window) "Reading Region $j of $objInfo($obj,pkCount)"

       readView $obj$j

       incr objInfo($obj,roiCount)

       incr j
      }

   return 0
}

proc readThisTitrView { obj thisROI specID pkID titrID mode } \
{
   global objInfo xUnits yUnits

   set x1 $objInfo($obj$pkID,$specID,specX1)
   set xn $objInfo($obj$pkID,$specID,specXN)

   set y1 $objInfo($obj$pkID,$specID,specY1)
   set yn $objInfo($obj$pkID,$specID,specYN)

   readROI -in $objInfo($obj,inName$specID) -ndim 2 -roi $thisROI \
           -x $objInfo(data,hAxis) $x1$xUnits $xn$xUnits \
           -y $objInfo(data,vAxis) $y1$yUnits $yn$yUnits \
           -$objInfo(data,realFlag) -naux 1 
}

proc clearTitrView {} \
{
   global objInfo

   set obj titr

   clearTitrLabels

   newScreen -win $objInfo($obj,pixmap)

   drawRec -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
           -x 0% 100% -y 0% 100% -fg $objInfo(color,neutral)

   freeThisID $objInfo($obj,ovCellList)
   freeThisID $objInfo($obj,cellList)
   freeThisID $objInfo($obj,roiList)

   set objInfo($obj,ovCellList)  ""
   set objInfo($obj,cellList)    ""
   set objInfo($obj,roiList)     ""
   set objInfo($obj,cellCount)   0
   set objInfo($obj,roiList)     0 

   set objInfo($obj,activeCell)  0
}

proc drawThisTitrView { cellID mode } \
{
   global objInfo

   set obj titr

   set drawInfo $objInfo($obj,drawInfo,$cellID)

   set thisROI    [getNamedItem roiID  $drawInfo]
   set specID     [getNamedItem specID $drawInfo]
   set titrID     [getNamedItem titrID $drawInfo]
   set pkID       [getNamedItem pkID   $drawInfo]
   set w          [getNamedItem win    $drawInfo]
   set hi         [getNamedItem hi     $drawInfo]
   set thisColor1 [getNamedItem color1 $drawInfo]
   set thisColor2 [getNamedItem color2 $drawInfo]

   set x1 $objInfo($obj$pkID,$specID,specX1)
   set xn $objInfo($obj$pkID,$specID,specXN)
   set x0 [expr 0.5*($x1 + $xn)]

   if {$objInfo(data,$specID,xFirst) > $objInfo(data,$specID,xLast)} \
      {
       set xSign -1
      } \
   else \
      {
       set xSign  1
      }

   if {$objInfo(data,$specID,yFirst) > $objInfo(data,$specID,yLast)} \
      {
       set ySign -1
      } \
   else \
      {
       set ySign  1
      }

   set y1 $objInfo($obj$pkID,$specID,specY1)
   set yn $objInfo($obj$pkID,$specID,specYN)
   set y0 [expr 0.5*($y1 + $yn)]

   set readFlag 0

   if {![strcasecmp $mode incrHi]} \
      {
       set hi [expr 1.3*$hi]
      } \
   elseif {![strcasecmp $mode decrHi]} \
      {
       set hi [expr $hi/1.3]
      } \
   elseif {![strcasecmp $mode zoomIn]} \
      {
       set dx [expr $xSign*0.5*0.9*($xn - $x1)]
       set dy [expr $ySign*0.5*0.9*($yn - $y1)]
        
       set objInfo($obj$pkID,$specID,specX1) [expr $x0 - $dx]
       set objInfo($obj$pkID,$specID,specXN) [expr $x0 + $dx]

       set objInfo($obj$pkID,$specID,specY1) [expr $y0 - $dy]
       set objInfo($obj$pkID,$specID,specYN) [expr $y0 + $dy]

       set readFlag 1
      } \
   elseif {![strcasecmp $mode zoomOut]} \
      {
       set dx [expr $xSign*0.5*1.1*($xn - $x1)]
       set dy [expr $ySign*0.5*1.1*($yn - $y1)]

       set objInfo($obj$pkID,$specID,specX1) [expr $x0 - $dx]
       set objInfo($obj$pkID,$specID,specXN) [expr $x0 + $dx]
       
       set objInfo($obj$pkID,$specID,specY1) [expr $y0 - $dy]
       set objInfo($obj$pkID,$specID,specYN) [expr $y0 + $dy]

       set readFlag 1
      } \
   elseif {![strcasecmp $mode up]} \
      {
       set dy [expr 0.1*($yn - $y1)]

       set objInfo($obj$pkID,$specID,specY1) [expr $y1 + $dy]
       set objInfo($obj$pkID,$specID,specYN) [expr $yn + $dy]

       set readFlag 1
      } \
   elseif {![strcasecmp $mode down]} \
      {
       set dy [expr 0.1*($yn - $y1)]
          
       set objInfo($obj$pkID,$specID,specY1) [expr $y1 - $dy]
       set objInfo($obj$pkID,$specID,specYN) [expr $yn - $dy]

       set readFlag 1
      } \
   elseif {![strcasecmp $mode left]} \
      {
       set dx [expr 0.1*($xn - $x1)]
          
       set objInfo($obj$pkID,$specID,specX1) [expr $x1 - $dx]
       set objInfo($obj$pkID,$specID,specXN) [expr $xn - $dx]

       set readFlag 1
      } \
   elseif {![strcasecmp $mode right]} \
      {
       set dx [expr 0.1*($xn - $x1)]

       set objInfo($obj$pkID,$specID,specX1) [expr $x1 + $dx]
       set objInfo($obj$pkID,$specID,specXN) [expr $xn + $dx]

       set readFlag 1
      }

   if {$readFlag} \
      {
       if {$xSign == 1} \
          {
           if {$objInfo($obj$pkID,$specID,specX1) < $objInfo(data,$specID,xFirst)} {set objInfo($obj$pkID,$specID,specX1) $objInfo(data,$specID,xFirst)}
           if {$objInfo($obj$pkID,$specID,specXN) > $objInfo(data,$specID,xLast)}  {set objInfo($obj$pkID,$specID,specXN) $objInfo(data,$specID,xLast)}
          } \
       else \
          {
           if {$objInfo($obj$pkID,$specID,specX1) > $objInfo(data,$specID,xFirst)} {set objInfo($obj$pkID,$specID,specX1) $objInfo(data,$specID,xFirst)}
           if {$objInfo($obj$pkID,$specID,specXN) < $objInfo(data,$specID,xLast)}  {set objInfo($obj$pkID,$specID,specXN) $objInfo(data,$specID,xLast)}
          }

       if {$ySign == 1} \
          {
           if {$objInfo($obj$pkID,$specID,specY1) < $objInfo(data,$specID,yFirst)} {set objInfo($obj$pkID,$specID,specY1) $objInfo(data,$specID,yFirst)}
           if {$objInfo($obj$pkID,$specID,specYN) > $objInfo(data,$specID,yLast)}  {set objInfo($obj$pkID,$specID,specYN) $objInfo(data,$specID,yLast)}
          } \
       else \
          {
           if {$objInfo($obj$pkID,$specID,specY1) > $objInfo(data,$specID,yFirst)} {set objInfo($obj$pkID,$specID,specY1) $objInfo(data,$specID,yFirst)}
           if {$objInfo($obj$pkID,$specID,specYN) < $objInfo(data,$specID,yLast)}  {set objInfo($obj$pkID,$specID,specYN) $objInfo(data,$specID,yLast)}
          }

       readThisTitrView $obj $thisROI $specID $pkID $titrID $mode 
      }

   drawROI -cell $cellID -roi $thisROI -win $w -hi $hi \
           -color1 $thisColor1 -color2 $thisColor2 -plev $objInfo(prog,plev) -nlev $objInfo(prog,nlev) \
           -$objInfo(prog,drawType) -bg $objInfo(color,bg) -fg $objInfo(color,box)
 
   set objInfo($obj,drawInfo,$cellID) \
    "roiID $thisROI specID $specID pkID $pkID titrID $titrID win $w hi $hi color1 $thisColor1 color2 $thisColor2"
}

proc drawTitrView {} \
{
   global objInfo

   set obj titr

   busyView hold $objInfo($obj,window)

   set parent $objInfo($obj,parent)

   set cx1 $objInfo($obj,dx)
   set cxn [expr $cx1 + $objInfo($obj,cellXSize)]
   set cy1 $objInfo($obj,dy)
   set cyn [expr $cy1 + $objInfo($obj,cellYSize)]

   set titrLoc 1

   for {set j 1} {$j <= $objInfo($obj,pkCount)} {incr j} \
      {
       wm title $objInfo($obj,window) "Drawing Region $j of $objInfo($obj,pkCount)"

       update

       set cellID $objInfo($obj$j,cell)

       set objInfo($obj,ovCellGridX1,$j)     $cx1
       set objInfo($obj,ovCellGridXN,$j)     $cxn
       set objInfo($obj,ovCellGridY1,$j)     $cy1
       set objInfo($obj,ovCellGridYN,$j)     $cyn
       set objInfo($obj,ovCellGridI,$cellID) $j
       set objInfo($obj,ovCellGridJ,$cellID) $j

       lappend objInfo($obj,ovCellList) $cellID

       defineCell -cell $cellID -win $objInfo($obj,pixmap) -x $cx1 $cxn -y $cy1 $cyn

       for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
          {
           if {$i == 1} \
              {
               set bg $objInfo(color,bg)
              } \
           else \
              {
               set bg None
              }

           if {[info exists objInfo(color,posColor$i)]} \
              {
               set thisColor1 $objInfo(color,posColor$i)
               set thisColor2 $objInfo(color,negColor$i)
              } \
           else \
              {
               set nC [llength $objInfo(color,colorList)]
               set ii $i
 
               while {$ii > $nC} {set ii [expr $ii - $nC]}

               set thisColor1 [lindex $objInfo(color,colorList) [expr $ii - 1]]
               set thisColor2 $thisColor1

               set objInfo(color,posColor$i) $thisColor1
               set objInfo(color,negColor$i) $thisColor2
              }

           if {![info exists objInfo(prog,plev)]}     {set objInfo(prog,plev) 10}
           if {![info exists objInfo(prog,nlev)]}     {set objInfo(prog,nlev) 10}
           if {![info exists objInfo(prog,drawType)]} {set objInfo(prog,drawType) cont}

           drawROI -cell $cellID \
                   -roi $objInfo($obj$j,roi$i)             \
                   -win $objInfo($obj,pixmap)              \
                   -hi $objInfo($parent,hi$i) -bg $bg      \
                   -color1 $thisColor1 -color2 $thisColor2 \
                   -plev $objInfo(prog,plev) -nlev $objInfo(prog,nlev)  \
                   -$objInfo(prog,drawType) -fg $objInfo(color,box)
          }

       set cx1 [expr $cxn + 2*$objInfo($obj,dx)]
       set cxn [expr $cx1 + $objInfo($obj,cellXSize)]

       for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
          {
           set cellID $objInfo($obj,cellGrid,$i,$j)

           set objInfo(cell$cellID,roi)     $objInfo($obj$j,roi$i)
           set objInfo(cell$cellID,specID)  $i
           set objInfo(cell$cellID,pkID)    $j

           defineCell -cell $cellID -win $objInfo($obj,pixmap) -x $cx1 $cxn -y $cy1 $cyn

           set objInfo($obj,cellGridX1,$i)     $cx1
           set objInfo($obj,cellGridXN,$i)     $cxn
           set objInfo($obj,cellGridY1,$j)     $cy1
           set objInfo($obj,cellGridYN,$j)     $cyn
           set objInfo($obj,cellGridI,$cellID) $i
           set objInfo($obj,cellGridJ,$cellID) $j
 
           lappend objInfo($obj,cellList) $cellID

           incr objInfo($obj,cellCount)

           set objInfo($obj,drawInfo,$cellID) \
            "roiID $objInfo($obj$j,roi$i) specID $i pkID $j titrID $titrLoc win $objInfo($obj,pixmap) hi $objInfo($parent,hi$i) color1 $objInfo(color,posColor$i) color2 $objInfo(color,negColor$i)"

           drawROI -cell $cellID \
                   -roi $objInfo($obj$j,roi$i)  \
                   -win $objInfo($obj,pixmap)   \
                   -hi $objInfo($parent,hi$i) -bg $objInfo(color,bg)     \
                   -color1 $objInfo(color,posColor$i)                    \
                   -color2 $objInfo(color,negColor$i)                    \
                   -plev $objInfo(prog,plev) -nlev $objInfo(prog,nlev)   \
                   -$objInfo(prog,drawType) -fg $objInfo(color,box) 

           set cx1 [expr $cxn + $objInfo($obj,dx)]
           set cxn [expr $cx1 + $objInfo($obj,cellXSize)]

           incr titrLoc
          }

       set cx1 $objInfo($obj,dx)
       set cxn [expr $cx1 + $objInfo($obj,cellXSize)]
       set cy1 [expr $cyn + $objInfo($obj,dy)]
       set cyn [expr $cy1 + $objInfo($obj,cellYSize)]
      }

   wm title $objInfo($obj,window) $objInfo($obj,winTitle)

   busyView release $objInfo($obj,window)

   return 0
}

proc showTitrSignal { obj sxPix syPix mode } \
{
   global objInfo xUnits yUnits

   set ix0 [$objInfo($obj,canvas) canvasx $sxPix]
   set iy0 [$objInfo($obj,canvas) canvasy $syPix]

   set cellID [findTitrGrid $obj $ix0 $iy0]

   if {!$cellID} \
      {
       $objInfo(titr,window).infoText configure -text " "
       update
       return
      } 

   set roiID  $objInfo(cell$cellID,roi)
   set specID $objInfo(cell$cellID,specID)
   set pkLoc  $objInfo(cell$cellID,pkID)

   set e [lindex $objInfo($obj,pkList) [expr $pkLoc - 1]]

   if {[llength $e]} \
      {
       set pkID [gdbGet $e INDEX]
      } \
   else \
      {
       set pkID None
      }

   set cellInfo $objInfo($obj,cellInfo,$cellID)
   set winID    $objInfo($obj,pixmap)

   set mode [getNamedItem mode $cellInfo]

   if {![info exists objInfo(cell$cellID,idV)] || ![info exists objInfo(cell$cellID,idH)]} return

   set ix0 [lindex [$objInfo($obj,canvas) coords $objInfo(cell$cellID,idV)] 0]
   set iy0 [lindex [$objInfo($obj,canvas) coords $objInfo(cell$cellID,idH)] 1]

   set xPPM [pix2spec -cell $cellID -win $winID -roi $roiID -x $ix0 -units ppm]
   set yPPM [pix2spec -cell $cellID -win $winID -roi $roiID -y $iy0 -units ppm]

   if {![strcasecmp $mode peak]} \
      {
       set hi [getNamedItem hi $cellInfo]
       set modeStr "Peak Table"
      } \
   else \
      {
       set hi [getTitrHi $roiID $cellID $xPPM $yPPM]
       set modeStr "Adjusted"
      }

   $objInfo(titr,window).infoText configure -text [format "Cell $cellID Spectrum: %s  Peak: %s  X: %.3f ppm Y: %.3f ppm  Hi: %.4e Mode: $modeStr" $specID $pkID $xPPM $yPPM $hi]
}

proc adjTitrSignal { obj sxPix syPix mode } \
{
   global objInfo

   set ix0 [$objInfo($obj,canvas) canvasx $sxPix]
   set iy0 [$objInfo($obj,canvas) canvasy $syPix]

   set cellID [findTitrGrid $obj $ix0 $iy0]

   if {!$cellID} return
   
   set objInfo($obj,saveNeeded) 1

   set cellInfo $objInfo($obj,cellInfo,$cellID)
   set n        [llength $cellInfo]

   switch $mode \
      {
       insert {if {$n} return} 

       delete {
               if {!$n} return

               $objInfo($obj,canvas) delete $objInfo(cell$cellID,idH)
               $objInfo($obj,canvas) delete $objInfo(cell$cellID,idV)

               set objInfo($obj,cellInfo,$cellID) ""
               set objInfo(cell$cellID,idH)       ""
               set objInfo(cell$cellID,idV)       ""

               update

               return
              }

      incrHi  {
               drawThisTitrView $cellID $mode 
               return
              }

      decrHi  {
               drawThisTitrView $cellID $mode
               return
              }

      zoomIn  {
               drawThisTitrView $cellID $mode 
              }
     
      zoomOut {
               drawThisTitrView $cellID $mode 
              }

      up      {
               drawThisTitrView $cellID $mode
              }

      down    {
               drawThisTitrView $cellID $mode
              }

      left    {
               drawThisTitrView $cellID $mode
              }

      right   {
               drawThisTitrView $cellID $mode
              }
 
      adjust  {if {$n < 2} return}

      default {return}
     }
     
#  if {$n < 2} return
 
   set roiID   $objInfo(cell$cellID,roi)
   set winID   $objInfo($obj,pixmap)

   set cInfo [getCellInfo -cell $cellID -win $objInfo($obj,pixmap)]
   set xInfo [lindex $cInfo 1]
   set yInfo [lindex $cInfo 2]

   set ix1   [expr [lindex $xInfo 0] + 1]
   set ixn   [expr [lindex $xInfo 1] - 1]
   set iy1   [expr [lindex $yInfo 0] + 1]
   set iyn   [expr [lindex $yInfo 1] - 1]

   if {[strcasecmp $mode adjust] && [strcasecmp $mode insert]} \
      {
       set ix0 [lindex [$objInfo($obj,canvas) coords $objInfo(cell$cellID,idV)] 0]
       set iy0 [lindex [$objInfo($obj,canvas) coords $objInfo(cell$cellID,idH)] 1] 
      }

   set xPPM [pix2spec -cell $cellID -win $winID -roi $roiID -x $ix0 -units ppm]
   set yPPM [pix2spec -cell $cellID -win $winID -roi $roiID -y $iy0 -units ppm]
   set hi   [getTitrHi $roiID $cellID $xPPM $yPPM]

   if {![strcasecmp $mode insert]} \
      {
       insertTitrCursor $cellID 666666 None $xPPM $yPPM white insert
      } \
   else \
      {
       set id  [getNamedItem id  $cellInfo]
       set ass [getNamedItem ass $cellInfo]

       set objInfo($obj,cellInfo,$cellID) "id $id ass $ass xPPM $xPPM yPPM $yPPM mode adjust hi $hi"

       $objInfo($obj,canvas) coords $objInfo(cell$cellID,idH) $ix1 $iy0 $ixn $iy0
       $objInfo($obj,canvas) coords $objInfo(cell$cellID,idV) $ix0 $iy1 $ix0 $iyn
      }

   $objInfo($obj,canvas) raise $objInfo(cell$cellID,idH)
   $objInfo($obj,canvas) raise $objInfo(cell$cellID,idV) 

   showTitrSignal $obj $sxPix $syPix $mode

   update
}

proc clearTitrLabels {} \
{
   global objInfo

   set obj titr

   $objInfo($obj,canvas) delete label cursorH cursorV
}

proc mapTitrPeaks {} \
{
   global objInfo

   set obj titr

   set objInfo($obj,saveNeeded) 1

   if {[llength $objInfo($obj,selList)]} \
      {
       set pkList $objInfo($obj,selList)
      } \
   else \
      {
       set pkList [getViewPeakList $obj $objInfo(peak,tab1)]
      }

   set pkCount   [llength $pkList]
   set yTestSize [expr $pkCount*($objInfo($obj,cellYSize) + $objInfo($obj,dy))]

   if {$yTestSize > $objInfo($obj,viewYMax)} \
      {
       showError titrView "Too Many Peaks; Select a Smaller Region."
       return 1
      }

   set objInfo($obj,pkList)  $pkList
   set objInfo($obj,pkCount) $pkCount
   set objInfo(peak,inUse)   $objInfo($obj,pkList)

   set k 1
   set j 1

   set ddx [expr 0.4*$objInfo($obj,regXW)]
   set ddy [expr 0.4*$objInfo($obj,regYW)]

   foreach entry $objInfo($obj,pkList) \
      {
       set x0 [gdbGet $entry $objInfo(peak,xName)]
       set y0 [gdbGet $entry $objInfo(peak,yName)]

       set sx1 [expr $x0 + $objInfo($obj,regXW)]
       set sxn [expr $x0 - $objInfo($obj,regXW)]
       set sy1 [expr $y0 + $objInfo($obj,regYW)]
       set syn [expr $y0 - $objInfo($obj,regYW)]

       set mx1 $sx1
       set mxn $sxn
       set my1 $sy1
       set myn $syn

       set objInfo($obj$j,expanded) 0

       set objInfo($obj$j,specX1)  $sx1
       set objInfo($obj$j,specXN)  $sxn
       set objInfo($obj$j,specY1)  $sy1
       set objInfo($obj$j,specYN)  $syn

       set dx 0.0
       set dy 0.0

       set cellID [getNextID]

       set objInfo($obj$j,cell) $cellID

       for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
          {
           set sx0 [expr $x0 + $dx]
           set sy0 [expr $y0 + $dy]

           set sx1 [expr $sx0 + $objInfo($obj,regXW)]
           set sxn [expr $sx0 - $objInfo($obj,regXW)]
           set sy1 [expr $sy0 + $objInfo($obj,regYW)]
           set syn [expr $sy0 - $objInfo($obj,regYW)]

           set objInfo($obj$j,$i,specX1)  $sx1
           set objInfo($obj$j,$i,specXN)  $sxn
           set objInfo($obj$j,$i,specY1)  $sy1
           set objInfo($obj$j,$i,specYN)  $syn

           set cellID [getNextID]

           set objInfo($obj,cellInfo,$cellID) ""
           set objInfo($obj,cellGrid,$i,$j)   $cellID
           set objInfo($obj$j,cell$i)         $cellID

           if {$i != 1} \
              {
               set pkList [getViewPeakList $obj$j,$i $objInfo(peak,tab$i)]
               if {![llength $pkList]} continue

               set entry [findNearestPeakEntry $sx0 $sy0 $pkList]
              }

           set x0   [gdbGet $entry $objInfo(peak,xName)]
           set y0   [gdbGet $entry $objInfo(peak,yName)]
           set id   [gdbGet $entry INDEX]
           set hi   [gdbGet $entry HEIGHT]
           set ass  [gdbGet $entry ASS]

           if {$x0 > $mx1} \
              {
               showVerb "Expanded Peak $id Region x1."
               set objInfo($obj$j,expanded) 1
               set mx1 [expr $x0 + $ddx]
              }

           if {$x0 < $mxn} \
              {
               showVerb "Expanded Peak $id Region xn."
               set objInfo($obj$j,expanded) 1
               set mxn [expr $x0 - $ddx]
              }

           if {$y0 > $my1} \
              {
               showVerb "Expanded Peak $id Region y1."
               set objInfo($obj$j,expanded) 1
               set my1 [expr $y0 + $ddy]
              }

           if {$y0 < $myn} \
              {
               showVerb "Expanded Peak $id Region yn."
               set objInfo($obj$j,expanded) 1
               set myn [expr $y0 - $ddy]
              }

           showVerb "Peak $id Spectrum $i $ass $x0 $y0"

           set objInfo($obj,cellInfo,$cellID) "id $id ass $ass xPPM $x0 yPPM $y0 mode peak hi $hi"

           set dx [expr $x0 - $sx0]
           set dy [expr $y0 - $sy0]

           incr k
          }

       if {$objInfo($obj$j,expanded)} \
          {
           set objInfo($obj$j,specX1)  $mx1
           set objInfo($obj$j,specXN)  $mxn
           set objInfo($obj$j,specY1)  $my1
           set objInfo($obj$j,specYN)  $myn
          }

       incr j
      }

   return 0
}

proc drawTitrLabels {} \
{
   global objInfo stdfont

   set obj titr 

   clearTitrLabels

   set winID $objInfo($obj,pixmap)
   set fg    white

   for {set j 1} {$j <= $objInfo($obj,pkCount)} {incr j} \
      {
       set cellID   $objInfo($obj,cellGrid,1,$j)
       set cellInfo $objInfo($obj,cellInfo,$cellID)

       set id   [getNamedItem id  $cellInfo]
       set ass  [getNamedItem ass $cellInfo]

       set cellID $objInfo($obj$j,cell)

       set xPix [spec2pix -cell $cellID -win $winID -x 10%]
       set yPix [spec2pix -cell $cellID -win $winID -y 90%]

       if {$objInfo($obj,showPkID)} \
          {
           if {$objInfo($obj,showAss)} \
              {
               set str "$id=$ass"
              } \
           else \
              {
               set str "$id"
              }
          } \
       else \
          {
           if {$objInfo($obj,showAss)} \
              {
               set str "$ass"
              } \
           else \
              {
               set str ""
              }
          }

       if {[string length $str]} \
          {
           $objInfo($obj,canvas) create text \
              $xPix $yPix -font $stdfont -fill $fg -anchor w \
              -tags label -text $str
          }

       for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
          {
           set cellID   $objInfo($obj,cellGrid,$i,$j)
           set cellInfo $objInfo($obj,cellInfo,$cellID)

           set id   [getNamedItem id   $cellInfo]
           set ass  [getNamedItem ass  $cellInfo]
           set x0   [getNamedItem xPPM $cellInfo]
           set y0   [getNamedItem yPPM $cellInfo]
           set mode [getNamedItem mode $cellInfo]
   
           insertTitrCursor $cellID $id $ass $x0 $y0 $objInfo(color,grid) $mode
          }
      } 

   update
}

proc insertTitrCursor { cellID id ass x0 y0 color mode } \
{
   global objInfo

   set obj titr

   if {![llength $id]} return

   set i $objInfo($obj,cellGridI,$cellID)
   set j $objInfo($obj,cellGridJ,$cellID)

   set roiID  $objInfo($obj$j,roi$i)
   set winID  $objInfo($obj,pixmap)

   set cInfo [getCellInfo -cell $cellID -win $objInfo($obj,pixmap)]
   set xInfo [lindex $cInfo 1]
   set yInfo [lindex $cInfo 2]

   set ix1   [expr [lindex $xInfo 0] + 1]
   set ixn   [expr [lindex $xInfo 1] - 1]
   set iy1   [expr [lindex $yInfo 0] + 1]
   set iyn   [expr [lindex $yInfo 1] - 1]

   set xUnits ppm
   set yUnits ppm

   set hi [getTitrHi $roiID $cellID $x0 $y0]

   if {![strcasecmp $xUnits "pix"]} \
      {
       set ix0 [expr int($x0)]
      } \
   else \
      {
       set ix0 [spec2pix -roi $roiID -cell $cellID -win $winID -x $x0$xUnits]
      }

   if {![strcasecmp $yUnits "pix"]} \
      {
       set iy0 [expr int($y0)]
      } \
   else \
      {
       set iy0 [spec2pix -roi $roiID -cell $cellID -win $winID -y $y0$yUnits]
      }

   if {$ix0 < $ix1} {set ix0 $ix1}
   if {$ix0 > $ixn} {set ix0 $ixn}
   if {$iy0 < $iy1} {set iy0 $iy1}
   if {$iy0 > $iyn} {set iy0 $iyn}
           
   set idH [$objInfo($obj,canvas) create line $ix1 $iy0 $ixn $iy0 -fill $color -tags cursorH]
   set idV [$objInfo($obj,canvas) create line $ix0 $iy1 $ix0 $iyn -fill $color -tags cursorV]

   set objInfo($obj,cellInfo,$cellID) "id $id ass $ass xPPM $x0 yPPM $y0 mode $mode hi $hi"

   set objInfo(cell$cellID,idH) $idH
   set objInfo(cell$cellID,idV) $idV
}

proc findNearestPeakEntry { x0 y0 pkList } \
{
   global objInfo

   set minDist    1000000.0
   set bestEntry  0

   set dx 1.0
   set dy 5.0

   foreach entry $pkList \
      {
       set x [gdbGet $entry $objInfo(peak,xName)]
       set y [gdbGet $entry $objInfo(peak,yName)]
       set d [expr pow(($x-$x0)/$dx,2) + pow(($y-$y0)/$dy,2)]
       set d [expr pow($d,0.5)]

       if {$d < $minDist} \
          {
           set minDist    $d
           set bestEntry  $entry
          }
      }

    return $bestEntry
}

proc findTitrGrid { obj xPix yPix } \
{
   global objInfo

   if {!$objInfo($obj,cellCount)} {return 0}

   if {[testTitrCell $obj $xPix $yPix $objInfo($obj,activeCell)]} \
      {
       return $objInfo($obj,activeCell)
      }

   for {set j 1} {$j <= $objInfo($obj,pkCount)} {incr j} \
      {
       set iy1 $objInfo($obj,cellGridY1,$j)
       set iyn $objInfo($obj,cellGridYN,$j)

       if {$yPix < $iy1 || $yPix > $iyn} continue

       for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
          {
           set ix1 $objInfo($obj,cellGridX1,$i)
           set ixn $objInfo($obj,cellGridXN,$i)

           if {$xPix < $ix1 || $xPix > $ixn} continue

           set id $objInfo($obj,cellGrid,$i,$j)

           set objInfo(titr,activeCell)  $id

           return $id
          }
      }
 
   return 0
}

proc findTitrCell { obj xPix yPix } \
{
   global objInfo

   set n [llength $objInfo($obj,cellList)]

   for {set i -1} {$i < $n} {incr i} \
      {
       if {$i < 0} \
          {
           set id $objInfo(titr,activeCell)
          } \
       else \
          {
           set id [lindex $objInfo(titr,cellList) $i]
          }

       if {!$id} continue

       set cInfo [getCellInfo -cell $id -win  $objInfo($obj,pixmap)]
       set xInfo [lindex $cInfo 1]
       set yInfo [lindex $cInfo 2]

       set ix1   [lindex $xInfo 0]
       set ixn   [lindex $xInfo 1]
       set iy1   [lindex $yInfo 0]
       set iyn   [lindex $yInfo 1]

       if {$xPix < $ix1 || $xPix > $ixn} continue
       if {$yPix < $iy1 || $yPix > $iyn} continue

       set objInfo(titr,activeCell)  $id

       return $id
      }

   return 0
}

proc testTitrCell { obj xPix yPix id } \
{
   global objInfo

   if {!$id} {return 0}

   set cInfo [getCellInfo -cell $id -win  $objInfo($obj,pixmap)]
   set xInfo [lindex $cInfo 1]
   set yInfo [lindex $cInfo 2]

   set ix1   [lindex $xInfo 0]
   set ixn   [lindex $xInfo 1]
   set iy1   [lindex $yInfo 0]
   set iyn   [lindex $yInfo 1]

   if {$xPix < $ix1 || $xPix > $ixn} {return 0} 
   if {$yPix < $iy1 || $yPix > $iyn} {return 0} 

   return $id
}

proc initTitrSpecWindow { parent } \
{
    global objInfo

    set obj titr
    set w   .$obj

    if {[winfo exists $w]} return

    showVerb "Initializing TitrView spectrum window."

    toplevel $w

    wm title $w $objInfo($obj,winTitle)

    set id  [getNextID]

    set objInfo($obj,window)       $w
    set objInfo($obj,canvas)       $w.c
    set objInfo($obj,scrollH)      $w.sh
    set objInfo($obj,scrollV)      $w.sv
    set objInfo($obj,pixmap)       $id 
    set objInfo($obj,cell)         $id 
    set objInfo($obj,parent)       $parent
    set objInfo($obj,specCount)    $objInfo($parent,specCount)

    set objInfo($obj,resizeProc)   resizeTitrView
    set objInfo($obj,redrawProc)   showTitrView

    set objInfo($obj,workROI)      [getNextID]

    for {set j 1} {$j <= $objInfo(peak,pkCount1)} {incr j} \
       {
        for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
           {
            set objInfo($obj,inName$i)   $objInfo($parent,inName$i)
            set objInfo($obj$j,inName$i) $objInfo($parent,inName$i)
            set objInfo($obj$j,window)   $objInfo($obj,window)
            set objInfo($obj$j,roi$i)    0
           }
       }

    set dx $objInfo($obj,dx)
    set dy $objInfo($obj,dy)
    set xw $objInfo($obj,cellXSize)
    set yw $objInfo($obj,cellXSize)
    set n  $objInfo($obj,specCount)
    set m  $objInfo(peak,pkCount1)

    set objInfo($obj,xSize) [expr (1 + $n)*($xw + $dx) + 2*$dx]
    set objInfo($obj,ySize) [expr (1 + $m)*($yw + $dy) + 1*$dy]

    if {$objInfo($obj,xSize) < $objInfo($obj,viewXSize)} \
       {
        set objInfo($obj,xSize) $objInfo($obj,viewXSize)
       }

    if {$objInfo($obj,ySize) < $objInfo($obj,viewYSize)} \
       {
        set objInfo($obj,ySize) $objInfo($obj,viewYSize)
       }

    if {$objInfo($obj,ySize) > $objInfo($obj,viewYMax)} \
       {
        set objInfo($obj,ySize) $objInfo($obj,viewYMax)
       }

    set objInfo($obj,winXSize) \
     [expr $objInfo($obj,xAdj) + $objInfo($obj,viewXSize)]

    set objInfo($obj,winYSize) \
     [expr $objInfo($obj,yAdj) + $objInfo($obj,viewYSize)]

    frame $w.top
    frame $w.bottom

    button $w.save -text "Save" -command saveAllTitrCurves
    button $w.exit -text "Quit" -command titrViewExit
    
    checkbutton $w.pk  -text "Show Peak ID"    -anchor w -command drawTitrLabels -variable objInfo($obj,showPkID) 
    checkbutton $w.ass -text "Show Assignment" -anchor w -command drawTitrLabels -variable objInfo($obj,showAss)

    label $w.space1   -text " " -width 2
    label $w.infoText -text " "

    pack $w.save $w.exit $w.pk $w.ass $w.space1 $w.infoText -in $w.top -side left

    scrollbar $objInfo($obj,scrollV) \
       -orient vert -command "$objInfo($obj,canvas) yview"

    scrollbar $objInfo($obj,scrollH) \
       -orient horiz -command "$objInfo($obj,canvas) xview"

    canvas $objInfo($obj,canvas) -border 0 \
       -xscrollcommand "$objInfo($obj,scrollH) set"                     \
       -yscrollcommand "$objInfo($obj,scrollV) set"                     \
       -scrollregion "0 0 $objInfo($obj,xSize) $objInfo($obj,ySize)"    \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize)         \
       -closeenough 10

    wm geom  $objInfo($obj,window) \
       =$objInfo($obj,winXSize)x$objInfo($obj,winYSize)+$dx-$dy

    update

    pack $objInfo($obj,scrollV) -in $w.bottom -side right  -expand 0 -fill y
    pack $objInfo($obj,scrollH) -in $w.bottom -side bottom -expand 0 -fill x
    pack $objInfo($obj,canvas)  -in $w.bottom -side left   -expand 1 -fill both

    pack $w.top    -side top -expand 0 -fill x
    pack $w.bottom -side top -expand 1 -fill both

    update

    $objInfo($obj,canvas) create pixmap 0 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
       -win $objInfo($obj,pixmap) -tags $obj

    defineCell -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
               -x 1 $objInfo($obj,xSize) -y 1 $objInfo($obj,ySize)

    focus $w

    bind $objInfo($obj,canvas) <B1-Motion>        "adjTitrSignal  $obj %x %y adjust"
    bind $objInfo($obj,canvas) <ButtonRelease-2>  "adjTitrSignal  $obj %x %y insert"
    bind $objInfo($obj,canvas) <ButtonRelease-3>  "adjTitrSignal  $obj %x %y delete"
    bind $objInfo($obj,canvas) <Button-4>         "adjTitrSignal  $obj %x %y incrHi"
    bind $objInfo($obj,canvas) <Button-5>         "adjTitrSignal  $obj %x %y decrHi"
    bind $objInfo($obj,canvas) <Shift-Button-4>   "adjTitrSignal  $obj %x %y zoomIn"
    bind $objInfo($obj,canvas) <Shift-Button-5>   "adjTitrSignal  $obj %x %y zoomOut"

    bind $objInfo($obj,window) <Key-Up>           "adjTitrSignal  $obj %x %y up"
    bind $objInfo($obj,window) <Key-Down>         "adjTitrSignal  $obj %x %y down"
    bind $objInfo($obj,window) <Key-Left>         "adjTitrSignal  $obj %x %y left"
    bind $objInfo($obj,window) <Key-Right>        "adjTitrSignal  $obj %x %y right"

    bind $objInfo($obj,window) <Motion>           "showTitrSignal $obj %x %y motion"
}

proc getTitrHi { roiID cellID x0 y0 } \
{
    global objInfo

    readROI -inROI $roiID -roi $objInfo(titr,workROI) -dx X_AXIS ${x0}ppm 0 -dy Y_AXIS ${y0}ppm 0 -ndim 2

    set rdata  [lindex [getROIInfo -roi $objInfo(titr,workROI)] 0]
    set maxVal [getPtrVal $rdata]

    return $maxVal
}

proc resizeTitrView args \
{
}
