#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Convert Numerical Text Data to NMRPipe Format:"
    puts stderr " txt2pipe.tcl < textInput > nmrPipeOutput"
    puts stderr "Options:"
    puts stderr " -in     inName \[stdin\] Input Text File to Convert, or Keyword stdin."
    puts stderr " -inHdr  inName \[None\]  Use Header from NMRPipe-Format File."
    puts stderr " -xSize  xSize  \[0\]     Number of Points per Vector, Zero for Auto."
    puts stderr " -verb                  Verbose Mode ON."
    puts stderr "Location Format Options (Use Only One):"
    puts stderr " -xyza     Lines start with: xLoc yLoc zLoc aLoc  (4D Only)."
    puts stderr " -xyz      Lines start with: xLoc yLoc zLoc       (Default)."
    puts stderr " -xy       Lines start with: xLoc yLoc            (1D and 2D Only)."
    puts stderr " -x        Lines start with: xLoc                 (1D Only)."
    puts stderr " -none     Lines have no location values."
    puts stderr " -int      Locations are Point Indices (Default)."
    puts stderr " -flt      Locations are Floating Point Values."
    puts stderr " -useIndex Use Line Number as Point Index         (1D Only)."
    puts stderr "Data Format Options (Use Only One):"
    puts stderr " -real    Lines end with one real value (Default)."
    puts stderr " -complex Lines end with a real value and an imaginary value."
    puts stderr " -ri      Lines end with one value, xSize real lines followed"
    puts stderr "          by xSize imaginary lines, etc."
    puts stderr "Domain Options: (Use Only One):"
    puts stderr " -time    Input Data are Time-Domain."
    puts stderr " -freq    Input Data are Frequency-Domain (Default)."
    puts stderr "Axis-Specific Header Adjustments for Result (Use Prefix -x -y -z):"
    puts stderr " -xN    xN       Total Number of Points."
    puts stderr " -xT    xT       Number of Valid Time-Domain Points."
    puts stderr " -xMODE xMODE    Acquisition Mode Keyword (Real, Complex, etc)."
    puts stderr " -xSW   xSW      Spectral Width, Hz."
    puts stderr " -xOBS  xOBS     Observe Frequency, MHz."
    puts stderr " -xCAR  xCar     Position of Zero Freq, PPM."
    puts stderr " -xFT   xFT      Domain Keyword (Time or Freq)."
    puts stderr " -xLAB  xLAB     Axis Label."
    puts stderr "Other Header Adjustments for Result:"
    puts stderr " -aq2D  aqMode   2D Acquisition Mode Keyword (Real, States, Magnitude, etc)."
    puts stderr " -title title    Optional Title."
    puts stderr " -ndim  nDim     Dimension Count."
    puts stderr " -hdr   argList  Additional Header Arguments, Should be Last on Command Line." 
    puts stderr "Notes:"
    puts stderr " 1. Data Locations xLoc etc. from the text input are only used to"
    puts stderr "    find dimension sizes.  They are not used to re-order the data." 
    puts stderr " 2. All lines in the input text should contain the same number of"
    puts stderr "    values, with no missing values, blank lines, or comments."
    puts stderr " 3. Use an explicit input file (-in inName) for large data."
    puts stderr " 4. Header adjustment arguments correspond to those for stand-alone"
    puts stderr "    programs bin2pipe/sethdr etc."
    puts stderr " 5. Adjust specific header values via -hdr -fdata ... which should"
    puts stderr "    last on the command line, for example:"
    puts stderr ""
    puts stderr "       txt2pipe.tcl -in test.txt -xy -time -complex \\"
    puts stderr "                     -hdr -fdata FDDMXVAL 71.625000 > txt.fid"
    puts stderr ""
    puts stderr " 6. If the -inHdr option is used, this overrides all header"
    puts stderr "    adjustment arguments -xN -xT etc."
    exit 1
   }

set outUnit   1 
set freqFlag  1
set origin    1
set riFlag    0
set indexFlag 0
set floatFlag 0
set verbose   0

set inName   stdin
set inHdr    None
set title    None
set hdrArgs  ""

set locList  "x y z"
set riList   "r"
set xSize    0

getArgD $ARGV -in     inName
getArgD $ARGV -inHdr  inHdr
getArgD $ARGV -xSize  xSize
getArgD $ARGV -origin origin
getArgD $ARGV -title  title

if {[flagLoc $ARGV -verb]}   {set verbose  1}
if {[flagLoc $ARGV -noverb]} {set verbose  0}

if {[flagLoc $ARGV -time]}   {set freqFlag 0}
if {[flagLoc $ARGV -freq]}   {set freqFlag 1}

if {[flagLoc $ARGV -useIndex]}   {set indexFlag 1}
if {[flagLoc $ARGV -nouseIndex]} {set indexFlag 0}

if {[flagLoc $ARGV -none]}   {set locList ""}
if {[flagLoc $ARGV -x]}      {set locList "x"}
if {[flagLoc $ARGV -xy]}     {set locList "x y"}
if {[flagLoc $ARGV -xyz]}    {set locList "x y z"}
if {[flagLoc $ARGV -xyza]}   {set locList "x y z a"}

if {[flagLoc $ARGV -int]}    {set floatFlag 0}
if {[flagLoc $ARGV -flt]}    {set floatFlag 1}

if {[flagLoc $ARGV -real]} \
   {
    set riList   "r"
    set riFlag   0
   }

if {[flagLoc $ARGV -complex]} \
   {
    set riList   "r i"
    set riFlag   0
   }

if {[flagLoc $ARGV -ri]} \
   {
    set riList   "r"
    set riFlag   1
   }

set loc [flagLoc $ARGV -hdr]

if {$loc} {set hdrArgs [lrange $ARGV [expr $loc + 1] end]}

set offset [expr 1 - $origin]

#
#---------------------------------------------------------------------------

#
# Allocation:
 
set dList  [concat $locList $riList]
set nn     [llength $dList]
set inUnit None

set dimCount 0 
set size(x)  0
set size(y)  0
set size(z)  0
set size(a)  0

set valCount  0
set lineCount 0
set minWord   0
set maxWord   0
set maxLen    0

if {![strcasecmp $inName stdin] || ![strcmp $inName "-"] || ![strcasecmp $inName None]} \
   {
    set inUnit stdin

    set valList  [read $inUnit]
    set valCount [llength $valList]
   } \
else \
   {
    if {![file exists $inName]} \
       {
        puts stderr "Txt2Pipe Error Finding Input File $inName"
        exit 1
       }

    set valCount [getTextFileInfo $inName lineCount minWord maxWord maxLen]

    set error [catch {set inUnit [open $inName r]}]

    if {$error} \
       {
        puts stderr "Txt2Pipe Error Opening Input File $inName"
        exit 1
       }
   }

if {$verbose} {puts stderr "Input Format: $dList."}
if {$verbose} {puts stderr "Read $valCount Input Text Items."}

if {[llength $riList] == 1} \
   {
    if {$riFlag} \
       {
        set nR      [expr $valCount/(2*$nn)]
        set size(r) [expr $FDATASIZE + $valCount/($nn)]
        set size(i) $FDATASIZE 

        if {$verbose} \
           {
            puts stderr "Extracting $nR Real Entries and $nR Imaginary Entries."
           }
       } \
    else \
       {
        set nR      [expr $valCount/$nn]
        set size(r) [expr $FDATASIZE + $valCount/($nn)]
        set size(i) $FDATASIZE 

        if {$verbose} \
           {
            puts stderr "Extracting $nR Real Entries."
           }
       }
   } \
else \
   {
    set nR      [expr $valCount/($nn - 1)]
    set size(r) [expr $FDATASIZE + $nR]
    set size(i) $size(r)

    if {$verbose} \
       {
        puts stderr "Extracting $nR Complex Entries."
       }
   }

set fdata    [sysMalloc $FDATASIZE float] 
set rdata    [sysMalloc $size(r)   float] 
set idata    [sysMalloc $size(i)   float] 

vsSet $fdata 0.0 $FDATASIZE
vsSet $rdata 0.0 $size(r)
vsSet $idata 0.0 $size(i)

set rCount   0
set iCount   0
set j        0
set k        0

#
# Value extraction:

foreach item {x y z a} {set itemList($item) ""}

if {![strcasecmp $inUnit stdin]} \
   {
    if {($j % $nn) == 0} {incr k}

    foreach val $valList \
       {
        set item [lindex $dList [expr $j % $nn]]

        switch -exact $item \
           {
            x -
            y -
            z -
            a {
               if {$indexFlag} \
                  {
                   set thisIndex $k
                  } \
               else \
                  {
                   set thisIndex $val
                  }

               if {$floatFlag} \
                  {
                   if {[lsearch -exact $itemList($item) $thisIndex] < 0} \
                      {
                       lappend itemList($item) $thisIndex
                       set size($item) [llength $itemList($item)]
                      }
                  } \
               else \
                  {
                   set valN [expr $thisIndex + $offset]
                   if {$valN > $size($item)} {set size($item) $valN}
                  }
              }

            r {
               setPtrVal $rdata $rCount $val
               incr rCount 
              }

            i {
               setPtrVal $idata $iCount $val
               incr iCount
              }
           }

        incr j
       }
   } \
else \
   {
    while {[gets $inUnit line] >= 0} \
       {
        if {[llength $line] != $nn} continue

        incr k

        set key [lindex $line 0]
        set c   [string index $key 0]

        if {![strcmp $c "#"]} continue
 
        set j 0

        foreach item $dList \
           {
            set val [lindex $line $j]

            switch -exact $item \
               {
                x -
                y -
                z -
                a {
                   if {$indexFlag} \
                      {
                       set thisIndex $k
                      } \
                   else \
                      {
                       set thisIndex $val
                      }

                   if {$floatFlag} \
                      {
                       if {[lsearch -exact $itemList($item) $thisIndex] < 0} \
                          {
                           lappend itemList($item) $thisIndex
                           set size($item) [llength $itemList($item)]
                          }
                      } \
                   else \
                      {
                       set valN [expr $thisIndex + $offset]
                       if {$valN > $size($item)} {set size($item) $valN}
                      }
                  }

                r {
                   setPtrVal $rdata $rCount $val
                   incr rCount
                  }
    
                i {
                   setPtrVal $idata $iCount $val
                   incr iCount
                  }
               }

            incr j
           }
       }

    catch "close $inUnit"

    set inUnit None
   }

#
# Header values:

if {$size(x) == 0} {set size(x) $rCount}

if {$size(x) > 0} {set dimCount 1}
if {$size(y) > 1} {set dimCount 2}
if {$size(z) > 1} {set dimCount 3}
if {$size(a) > 1} {set dimCount 3}

if {[llength $riList] == 2} \
   {
    set xN [expr 2*$size(x)]
    set yN [expr $size(y)]
    set zN [expr $size(z)]
    set aN [expr $size(a)]

    set xT $size(x)
    set yT [expr $size(y)/2]
    set zT [expr $size(z)/2]
    set aT [expr $size(a)/2]

    set xMODE Complex
    set yMODE Complex 
    set zMODE Complex 
    set aMODE Complex 

    set aq2D  States 
   } \
else \
   {
    if {$riFlag} \
       {
        set xN $size(x)
        set xT [expr $size(x)/2]

        set xMODE Complex
        set aq2D  States 
       } \
    else \
       {
        set xN $size(x)
        set xT $size(x)

        set xMODE Real
        set aq2D  Real
       }

    set yN $size(y)
    set zN $size(z)
    set aN $size(a)

    set yT $size(y)
    set zT $size(z)
    set aT $size(a)

    set yMODE Real
    set zMODE Real 
    set aMODE Real 
   }

if {!$xSize} {set xSize $xT}

if {$freqFlag} \
   {
    set xFT Freq
    set yFT Freq
    set zFT Freq
    set aFT Freq
   } \
else \
   {
    set xFT Time
    set yFT Time 
    set zFT Time 
    set aFT Time 
   }

set xSW   $xT
set ySW   $yT
set zSW   $zT
set aSW   $aT

set xOBS  1.0
set yOBS  1.0
set zOBS  1.0
set aOBS  1.0

set xCAR  0.0
set yCAR  0.0
set zCAR  0.0
set aCAR  0.0

set xLAB  X
set yLAB  Y
set zLAB  Z
set aLAB  Z

set ndim $dimCount

foreach dim {x y z a} \
   {
    foreach v {N T SW MODE OBS CAR FT LAB} \
       {
        getArgD $ARGV -$dim$v $dim$v
       }
   }

getArgD $ARGV -aq2D aq2D
getArgD $ARGV -ndim ndim

if {$dimCount == 4} \
   {
    if {$verbose} \
       {
        puts stderr "${dimCount}D Data: $size(x) x $size(y) x $size(z) x $size(a). Vector Size: $xSize."
       }

    eval buildHdr $fdata $hdrArgs -pipeFlag 1 -cubeFlag 0 -fileCount 1 \
       -xN      $xN       -yN      $yN        -zN    $zN      -aN    $aN  \
       -xT      $xT       -yT      $yT        -zT    $zT      -aT    $aT \
       -xSW     $xSW      -ySW     $ySW       -zSW   $zSW     -aSW   $aSW \
       -xMODE   $xMODE    -yMODE   $yMODE     -zMODE $zMODE   -aMODE $aMODE \
       -xOBS    $xOBS     -yOBS    $yOBS      -zOBS  $zOBS    -aOBS  $aOBS \
       -xCAR    $xCAR     -yCAR    $yCAR      -zCAR  $zCAR    -aCAR  $aCAR \
       -xFT     $xFT      -yFT     $yFT       -zFT   $zFT     -aFT   $aFT \
       -xLAB    $xLAB     -yLAB    $yLAB      -zLAB  $zLAB    -aLAB  $aLAB \
       -ndim    $ndim     -aq2D    $aq2D
   } \
elseif {$dimCount == 3} \
   {
    if {$verbose} \
       {
        puts stderr "${dimCount}D Data: $size(x) x $size(y) x $size(z). Vector Size: $xSize."
       }

    eval buildHdr $fdata $hdrArgs -pipeFlag 1 -cubeFlag 0 -fileCount 1 \
       -xN      $xN       -yN      $yN        -zN    $zN    \
       -xT      $xT       -yT      $yT        -zT    $zT    \
       -xSW     $xSW      -ySW     $ySW       -zSW   $zSW   \
       -xMODE   $xMODE    -yMODE   $yMODE     -zMODE $zMODE \
       -xOBS    $xOBS     -yOBS    $yOBS      -zOBS  $zOBS  \
       -xCAR    $xCAR     -yCAR    $yCAR      -zCAR  $zCAR  \
       -xFT     $xFT      -yFT     $yFT       -zFT   $zFT   \
       -xLAB    $xLAB     -yLAB    $yLAB      -zLAB  $zLAB  \
       -ndim    $ndim     -aq2D    $aq2D 
   } \
elseif {$dimCount == 2} \
   {
    if {$verbose} \
       {
        puts stderr "${dimCount}D Data: $size(x) x $size(y). Vector Size: $xSize."
       }

    eval buildHdr $fdata $hdrArgs -cubeFlag 0 -fileCount 1 \
       -xN      $xN       -yN      $yN      \
       -xT      $xT       -yT      $yT      \
       -xSW     $xSW      -ySW     $ySW     \
       -xMODE   $xMODE    -yMODE   $yMODE   \
       -xOBS    $xOBS     -yOBS    $yOBS    \
       -xCAR    $xCAR     -yCAR    $yCAR    \
       -xFT     $xFT      -yFT     $yFT     \
       -xLAB    $xLAB     -yLAB    $yLAB    \
       -ndim    $ndim     -aq2D    $aq2D 
   } \
else \
   {
    if {$verbose} \
       {
        puts stderr "${dimCount}D Data: $size(x) points. Vector Size: $xSize."
       }

    eval buildHdr $fdata $hdrArgs -cubeFlag 0 -fileCount 1 \
       -xN      $xN         \
       -xT      $xT         \
       -xSW     $xSW        \
       -xMODE   $xMODE      \
       -xOBS    $xOBS       \
       -xCAR    $xCAR       \
       -xFT     $xFT        \
       -xLAB    $xLAB       \
       -ndim    1      
   }

if {[strcasecmp $inHdr None]} \
   {
    if {![file exists $inHdr]} \
       {
        puts stderr "Txt2Pipe Error Finding Header Input $inHdr"
       } \
    else \
       {
        if {$verbose} {puts stderr "Using Header from $inHdr"}

        set error [rdFDATA $inHdr fdataOut]
        if {$error} {set fdataOut $fdata}
       }
   } \
else \
   {
    set fdataOut $fdata
   }

if {[strcasecmp $title None]} {setParmStr $fdataOut FDTITLE $title NULL_DIM}

sysWrite $outUnit $fdataOut $FDATASIZE

if {[llength $riList] == 2} \
   {
    set rPtr $rdata
    set iPtr $idata

    for {set rLeft $rCount} {$rLeft > 0} {incr rLeft -$xSize} \
       {
        if {$rLeft < $xSize} \
           {
            set n $rLeft
           } \
        else \
           {
            set n $xSize
           }

        sysWrite $outUnit $rPtr $n
        sysWrite $outUnit $iPtr $n

        set rPtr [incrPtr $rPtr $n]
        set iPtr [incrPtr $iPtr $n]
       }
   } \
else \
   {
    sysWrite $outUnit $rdata [expr $rCount + $iCount]
   }

exit 0
