#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

#
# Updates PPM and HZ values in a peak table.

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set inName    None
set outName   None
set dataName  None
set listName  None
set varList   Auto

set updatePPM 1
set extFlag   0

#
# Print help text if needed:

if {$ARGC == 1 || [flagLoc $ARGV -help]} \
   {
    puts stderr "$argv0: Update ppm and Hz, or Pts Values in a Peak Table:"
    puts stderr "Peak Table Specification:"
    puts stderr " -in   inName             Name of Input Table."
    puts stderr " -out  outName  \[stderr\]  Name of Output Table."
    puts stderr " -var  varList  \[Auto\]    List of Variables to Update."
    puts stderr "Spectral Data Specification (Use only one):"
    puts stderr " -data dataName           Name or Template of Spectral Data."
    puts stderr " -list listName           Text list of Spectral Data Planes."
    puts stderr "Conversion Modes (Use Only One):"
    puts stderr " -ppm     Update PPM and Hz Values (Default)."
    puts stderr " -pts     Update Values in Points."
    puts stderr "Extract Region Adjustment (Only For -pts Mode):"
    puts stderr " -ext     Adjust Point Positions for Extract Region;"
    puts stderr "          Locations Will Correspond to Unextracted Data."
    exit
   }

#
# Extract arguments:

getArgD $ARGV -in   inName
getArgD $ARGV -out  outName
getArgD $ARGV -data dataName
getArgD $ARGV -list listName

getListArgD $ARGV -var varList

if {[flagLoc $ARGV -ppm]} {set updatePPM 1}
if {[flagLoc $ARGV -pts]} {set updatePPM 0}
if {[flagLoc $ARGV -ext]} {set extFlag   1}

if {![strcasecmp $inName None]} \
   {
    puts stderr "Error: missing -in argument."
    exit 1
   }

if {![strcasecmp $dataName None] && ![strcasecmp $listName None]} \
   {
    puts stderr "Error: missing -data/-list argument."
    exit 1
   }

#
# Find the data file:

if {[strcasecmp $listName None]} \
   {
    set error [catch {set inUnit [open $listName r]}]

    if {$error} \
       {
        puts stderr "Error opening list file $listName"
        exit 1
       }

    set error [catch {gets $inUnit dataName}]

    if {$error} \
       {
        puts stderr "Error reading list file $listName"
        exit 1
       }

    close $inUnit
   }

#
# Open the data file and extract the header:

set thisName [format $dataName 1 1]
set error    [rdFDATA $thisName fdata]

if {$error} \
   {
    puts stderr "Error reading FDATA header from $thisName"
    exit 1
   }

#
# Read the table.
# Convert the PPM and Hz values. 

set pkTab [gdbRead table -in $inName]

set parmList   "_AXIS _PPM _HZ W W_HZ TW"
set dimList    "X Y Z A"
set updateList ""

if {![strcasecmp $varList Auto]} \
   {
    set vars [gdbGetInfo $pkTab -parms varNames]

    foreach vName $vars \
       {
        set head [string index $vName 0]
        set tail [string range $vName 1 end]

        if {[lsearch -exact $parmList $tail] < 0} continue
        if {[lsearch -exact $dimList  $head] < 0} continue 
    
        lappend varList $vName
       }
   }

set entry [gdbGetInfo $pkTab -parms entry]

while {$entry} \
   {
    foreach vName $varList \
       {
        set head  [string index $vName 0]
        set tail  [string range $vName 1 end]

        switch -exact $tail \
           {
            _AXIS {
                   if {!$updatePPM} \
                      {
                       set specVal [gdbGet $entry [format %s_PPM $head]]ppm
                       set pntVal  [spec2pnt $fdata $head $specVal]

                       if {$extFlag} \
                          {
                           set ox1 [getParm $fdata NDX1 $head]
                           set oxN [getParm $fdata NDXN $head]

                           if {$ox1 != 0.0 || $oxN != 0.0} \
                              {
                               set pntVal [expr $pntVal + $ox1 - 1.0]
                              }
                          }

                       gdbSet $entry $vName $pntVal

                       if {[lsearch -exact $updateList $vName] < 0} \
                          {
                           lappend updateList $vName
                          }
                      }
                  }

            W     {
                   if {!$updatePPM} \
                      {
                       set specVal [gdbGet $entry [format %sW_HZ $head]]hz
                       set pntVal  [spec2pntW $fdata $head $specVal]

                       gdbSet $entry $vName $pntVal

                       if {[lsearch -exact $updateList $vName] < 0} \
                          {
                           lappend updateList $vName
                          }
                      }
                  }

            TW    {
                   if {!$updatePPM} \
                      {
                       set pntVal  [gdbGet $entry [format %sTW $head]]
                       set tSize   [getParmI $fdata NDTDSIZE $head]
                       set fSize   [getParmI $fdata NDFTSIZE $head]
                       set pntVal  [expr $pntVal*(1.0*$tSize/$fSize)]

                       gdbSet $entry $vName $pntVal

                       if {[lsearch -exact $updateList $vName] < 0} \
                          {
                           lappend updateList $vName
                          }
                      }
                  }

            _PPM  {
                   if {$updatePPM} \
                      {
                       set pntVal  [gdbGet $entry [format %s_AXIS $head]]
                       set specVal [pnt2spec $fdata $head $pntVal ppm]

                       gdbSet $entry $vName $specVal

                       if {[lsearch -exact $updateList $vName] < 0} \
                          {
                           lappend updateList $vName
                          }
                      }
                  }

            _HZ   {
                   if {$updatePPM} \
                      {
                       set pntVal  [gdbGet $entry [format %s_AXIS $head]]
                       set specVal [pnt2spec $fdata $head $pntVal hz]

                       gdbSet $entry $vName $specVal

                       if {[lsearch -exact $updateList $vName] < 0} \
                          {
                           lappend updateList $vName
                          }
                      }
                  } 

            W_HZ  {
                   if {$updatePPM} \
                      {
                       set pntVal  [gdbGet $entry [format %sW $head]]
                       set specVal [pnt2specW $fdata $head $pntVal hz]

                       gdbSet $entry $vName $specVal

                       if {[lsearch -exact $updateList $vName] < 0} \
                          {
                           lappend updateList $vName
                          }
                      }
                  }

            default {
                    }
           }
       }
 
    set entry [gdbNext $entry]
   }

#
# Write the output:

gdbAddNote $pkTab "REMARK Updated Variables: $updateList"

if {![strcasecmp $outName None]} \
   {
    gdbWrite $pkTab    
   } \
else \
   {
    gdbWrite $pkTab $outName 
   }

exit
