#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

#
# vj2pipe.tcl (c) Frank Delaglio 2010-2015
#  Extracts parameters for conversion, processing, and display of VNMRJ data.
#-----------------------------------------------------------------------------

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
# For arguments handled by wrapper script such as vj2pipe.com:

set msgUnit     stderr
set wrapperFlag 0

if {[flagLoc $ARGV -wrapper]}   {set wrapperFlag 1}
if {[flagLoc $ARGV -nowrapper]} {set wrapperFlag 0}

proc showHelp {} \
{
   global ARGV argv0
 
   if {![flagLoc $ARGV -help]} return
 
   showMsg  "vj2pipe.tcl Version 3.2 Rev 2017.209.12.10:"
   showMsg  "Generate VJ to NMRPipe Conversion and Processing Commands."
   showMsgS " -vjPath     vPath \[.\]               Directory with VJNMR Input Data." 
   showMsg  " -vjfid      fName \[fid\]             Name of VNMRJ binary fid Input, or Auto"  
   showMsg  " -procpar    pName \[procpar\]         Name of VNMRJ procpar Input, or Auto."  
   showMsg  " -nusSched   sName \[Auto\]            Name of Optional NUS Schedule, or Auto."  
   showMsgW " -cd         cdDir \[.\]               Default Working Directory."
   showMsgW " -pipeDir    pDir  \[.\]               Directory for Output Scripts and Files." 
   showMsg  " -outPrefix  pName \[spec\]            File Prefix for Processed Results."
   showMsg  " -msgName    mName \[stderr\]          Output File for Status Messages."
   showMsg  " -vjAuxDir   aDir  \[NMRBASE/vj2pipe\] Directory of Auxiliary Files." 
   showMsgW " -args       aList \[None\]            Arguments to Pass to Processing Scripts."
   showMsg  "User-Defined Scripts and Parameters:"
   showMsg  " -sVar       sList \[None\]            Simple Single-Value Parameters."
   showMsg  " -lVar       lList \[None\]            Parameters Which Are Lists."
   showMsg  " -dVar       dList \[None\]            Dimension Specific Parameters."
   showMsg  " -subs       sList \[None\]            Script Templates to Substitute."
   showMsg  "Graphics Options:"
   showMsg  " -graph         Use Spectral Graphics Commands (Default)."
   showMsg  " -nograph       Do Not Use Spectral Graphics Commands."
   showMsg  " -drawPlane     Use NMRDraw 2D Plane Display (Default)."
   showMsg  " -nodrawPlane   Do Not Use NMRDraw 2D Plane Display."
   showMsg  " -drawCube      Use 3D Cube Display for 3D Data (Default)."
   showMsg  " -nodrawCube    Do Not Use 3D Cube Display."
   showMsg  " -drawStrips    Use Strip Display for 3D Data (Default)."
   showMsg  " -nodrawStrips  Do Not Use Strip Display."
   showMsg  " -progress      Use Progress Bar Graphics (Default)."
   showMsg  " -noprogress    Do Not Use Progress Bar Graphics."
   showMsg  "Other Options:"
   showMsg  " -info          List All Parameters, then Exit."
   showMsg  " -sinfo         List All Parameters on Single Line, then Exit."
   showMsg  " -vjexpert      Generate Expert Scripts (Default)."
   showMsg  " -novjexpert    No Expert Script Generation."
   showMsg  " -vjhdr         Read and Interpret VnmrJ fid Header (Default)."
   showMsg  " -novjhdr       Do Not Read and Interpret VnmrJ fid Header."
   showMsgW " -clean         Delete Previous Results and Scripts Before Processing."
   showMsgW " -cleanOnly     Delete Previous Results and Scripts, then Exit."
   showMsg  " -exec          Execute Conversion and Processing Scripts (Default)."
   showMsg  " -noexec        Do Not Execute Conversion and Processing Scripts."
   showMsg  " -partial       Accommodate Partially-Completed Acquisition (Default)." 
   showMsg  " -nopartial     No Adjustment for Partially-Completed Acquisition." 
   showMsg  " -skip          Skip Listing of Single-Point Dimensions (Default)."
   showMsg  " -noskip        List Single-Point Dimensions."
   showMsg  " -vjshow        List VJNMR-Specific Parameters."
   showMsg  " -novjshow      Do Not List VJNMR-Specific Parameters."
   showMsg  " -vjapod        Use VJNMR Window Function Values."
   showMsg  " -novjapod      Use NMRPipe Default Window Function Values."
   showMsg  " -nonusadj      Do Not Adjust Parameters for NUS Data."
   showMsg  " -verb          Verbose Mode ON (Default)."
   showMsg  " -noverb        Verbose Mode OFF."
   showMsg  "Option to Reset procpar Values, Can be used more than once:"
   showMsg  " -pset pName valList --"
   showMsg  "Other values can also be replaced at the command line:"
   showMsg  " [file tail $argv0] -pset BPpipetype HNHA -- -xLAB HN -yLAB HA"
}

proc showMsg { msg } \
{
   global msgUnit infoFlag

   if {![strcasecmp $msgUnit None]} return

   if {$infoFlag == 2} \
      {
       puts -nonewline $msgUnit " $msg"
      } \
   else \
      {
       puts $msgUnit $msg
      }
}

proc showMsgS { msg } \
{
   global wrapperFlag

   if {!$wrapperFlag} {showMsg $msg}
}

proc showMsgW { msg } \
{
   global wrapperFlag

   if {$wrapperFlag} {showMsg $msg}
}

proc setDefDir {} \
{
   global parmInfo wrapperFlag verbose

   if {$wrapperFlag} {return 0}

   if {[strcmp $parmInfo(cd) .] && [strcasecmp $parmInfo(cd) None]} \
      {
       if {[file isdirectory $parmInfo(cd)]} \
          {
           if {$verbose} \
              {
               showMsg "VJ2Pipe setting directory to $parmInfo(cd)"
               catch "cd $parmInfo(cd)"
              }
          } \
       else \
          {
           showMsg "VJ2Pipe Error finding directory $parmInfo(cd)"
           return 1
          }
      }

   return 0
}

proc setParmNames {} \
{
   global procpar parmInfo dimCount env

   if {[info exists env(NMRBASE)]} \
      {
       set vjAuxDir $env(NMRBASE)/vj2pipe
      } \
   else \
      {
       set vjAuxDir vj2pipe
      }

   set parmInfo(hideList) ""
   set parmInfo(listVars,userNames)   ""
   set parmInfo(scalarVars,userNames) ""
   set parmInfo(dimVars,userNames)    ""

   set parmInfo(listVars,usedNames)   ""
   set parmInfo(listVars,names)       "dimension tau nusMethods"

   set parmInfo(scalarVars,usedNames) ""

   set parmInfo(scalarVars,names)     "cd outPrefix vjPath vjfid procpar nusSched title type ndim nusDim celem arrayDim mcFlag aq2D aqORD phaseSign temperature h2oM      h2oB     coThresh fidScript ft1Script ft2Script xyScript xzScript xaScript xyzScript xyzaScript scrollScript tmpScript vjAuxDir inName   outName  auxName  hi   stripHi cubeHi planeHi projHi noise tdNoise ftNoise istTMult istCMult istIter istMaxRes"

   set parmInfo(scalarVars,defaults)  ".  spec      .      fid   procpar Auto     NMR   NMR  0    0      0     1        0      Real 0     1         293.0       -0.009552 5.011718 150.0    vj_fid    vj_ft1    vj_ft2    vj_xy    vj_xz    vj_xa    vj_xyz    vj_xyza    vj_scroll    vj_tmp    $vjAuxDir spec.fid spec.dat aux.dat 2.5% 3.0%    10%    10%     Full   Auto  Auto    Auto   default  default  default default"

   set parmInfo(scalarVars,formats)   "%s %s        %s     %s    %s      %s       %s    %s   %d   %d     %d    %d       %d     %s   %d    %d        %.2f        %.6f      %.6f     %.4f     %s        %s        %s        %s       %s       %s        %s       %s         %s           %s        %s        %s       %s       %s      %s   %s      %s     %s      %s     %s     %s     %s     %s       %s       %s      %s"

   set parmInfo(dimVars,usedNames)    ""

   set parmInfo(dimVars,names)        "N  T MODE CHAN SW   AQTIME OBS  OBSMID CAR   CARSET LAB NUC GAMMA P0   P1   P0EXTRA P1EXTRA APOD Q1   Q2   Q3   ELB  GLB  GOFF C1   VJSB VJSBS VJLB VJGF VJGFS DECAY SOL  ZFARG     FTARG EXTX1 EXTXN LSTIME LSFREQ BASEARG LPARG"

   set parmInfo(dimVars,defaults)     "0  0 Real 1    1.0  1.0    1.0  1.0    0.0   0      X   H1  1.0   0.0  0.0  0.0     0.0     SP   0.5  0.95 1    0.0  0.0  0.0  1.0  0.0  0.0   0.0  0.0  0.0   0.0   None zf=1,auto None  0%    100%  0      0      NULL    None"

   set parmInfo(dimVars,formats)      "%s %s %s  %s   %.4f %.6f   %.4f %.4f   %.4f  %d     %s  %s  %.9f  %.2f %.2f %.2f    %.2f    %s   %.3f %.3f %.3f %.3f %.3f %.3f %.3f %.3f %.3f  %.3f %.3f %.3f  %.3f  %s   %s        %s    %s    %s    %s     %s     %s      %s"

   set parmInfo(nusXList) ""
   set parmInfo(nusYList) ""
   set parmInfo(nusZList) ""
   set parmInfo(nusAList) ""

   set parmInfo(nusMODE,1) Complex 
   set parmInfo(nusMODE,2) Complex 
   set parmInfo(nusMODE,3) Complex 
   set parmInfo(nusMODE,4) Complex 

   set parmInfo(nusInitSize,1)  0
   set parmInfo(nusInitSize,2)  0
   set parmInfo(nusInitSize,3)  0
   set parmInfo(nusInitSize,4)  0

   set parmInfo(nusFlag)        0
   set parmInfo(nusSampleCount) 0

#
# VnmrJ fid header:
# LengthType  WordType   QuadTYpe      WordSize FIDSize       FIDCOunt      ExpCount
# short|long, float|int, complex|real, 2|4|8,   1024 (X-Size) 256 (Y-Size), 1 (Interleave Count)

   set procpar(vjhdr,LengthType) None 
   set procpar(vjhdr,WordType)   None 
   set procpar(vjhdr,QuadType)   None 
   set procpar(vjhdr,WordSize)   0
   set procpar(vjhdr,FIDSize)    0 
   set procpar(vjhdr,FIDCount)   0 
   set procpar(vjhdr,ExpCount)   0

   set parmInfo(nusMethods) "clean irls ist mdd mem ml scrub"
}

#
# Exit procedure:

proc quitProc { status } \
{
   global infoFlag

   restorePrecision

   if {$infoFlag} {set status 0}

   exit $status
}

#
# Reset the precision for calculations to original value.

proc restorePrecision {} \
{
   global old_tcl_prec tcl_precision 

   if {![info exists old_tcl_precision]} return

   if {[strcasecmp old_tcl_prec None]} \
      {
       set tcl_precision $old_tcl_precision
      }
}

#
# Set the precision for the PPM expressions, enable restore of precision later.

proc setPrecision {} \
{
   global old_tcl_prec tcl_precision 

   set old_tcl_prec None

   if {[info exists tcl_precision]} {set old_tcl_prec $tcl_precision}

   set tcl_precision 17
}

proc updateProcpar {} \
{
   global procpar verbose ARGC ARGV

   for {set loc 0} {$loc < $ARGC} {incr loc} \
      {
       set aStr [lindex $ARGV $loc]
       if {[strcmp $aStr -pset]} continue

       incr loc
       if {$loc > $ARGC - 1} break

       set i [lindex $ARGV $loc]
       set vStr ""

       if {[info exists procpar($i)]} \
          {
           set oldVal $procpar($i)
          } \
       else \
          {
           set oldVal None
          }

       incr loc

       while {$loc < $ARGC} \
          {
           set v [lindex $ARGV $loc]
           if {![strcmp $v "--"]} break

           lappend vStr $v
           incr loc
          }

       if {[llength $vStr]} \
          {
           set newVal $vStr
          } \
       else \
          {
           set newVal None
          }

       set procpar($i) $vStr

       if {$verbose} \
          {
           showMsg "Setting procpar($i) Old: $oldVal New: $newVal"
          }
      }

   foreach p {BPpipeprocinfo BPpipedrawinfo} \
      {
       if {![info exists procpar($p)]} continue
       if {![strcasecmp [string trim $procpar($p)] None]} continue

       if {$verbose} \
          {
           if {[llength $procpar($p)]} \
              {
               showMsg "Adding Arguments for $p: $procpar($p)"
              }
          }

       foreach i $procpar($p) {lappend ARGV $i}
      }
}

proc readVHdr {} \
{
   global parmInfo parmCount procpar vjHdrFlag verbose

   if {$vjHdrFlag} \
      {
       set LengthType None 
       set WordType   None 
       set QuadType   None 
       set WordSize   0
       set FIDSize    0
       set FIDCount   0 
       set ExpCount   0

       if {[file exists $parmInfo(vjfid)]} \
          {
           catch {set hInfo [exec var2pipe -info -tcl -noaswap -in $parmInfo(vjfid)]}

           if {![info exists hInfo]} \
              {
               showMsg "VJ2Pipe Error Interpreting Header in file $parmInfo(vjfid)"
               return 1
              }

           catch "eval $hInfo"
          }

       if {[info exists LengthType]} {set procpar(vjhdr,LengthType) $LengthType}
       if {[info exists WordType]}   {set procpar(vjhdr,WordType)   $WordType}
       if {[info exists QuadType]}   {set procpar(vjhdr,QuadType)   $QuadType}
       if {[info exists WordSize]}   {set procpar(vjhdr,WordSize)   $WordSize}
       if {[info exists FIDSize]}    {set procpar(vjhdr,FIDSize)    $FIDSize}
       if {[info exists FIDCount]}   {set procpar(vjhdr,FIDCount)   $FIDCount}
       if {[info exists ExpCount]}   {set procpar(vjhdr,ExpCount)   $ExpCount}
      }

   if {[info exists procpar(celem)]} \
      {
       if {[string is int $procpar(celem)]} \
          {
           if {$procpar(celem) > 0} {set parmInfo(celem) $procpar(celem)}
          }
      }

   if {[info exists procpar(vjhdr,FIDCount)]} \
      {
       if {[string is int $procpar(vjhdr,FIDCount)]} \
          {
           if {$procpar(vjhdr,FIDCount) > 0} \
              {
               if {$parmInfo(celem) == 0} \
                  {
                   set parmInfo(celem) $procpar(vjhdr,FIDCount)

                   if {$verbose} \
                      {
                       showMsg "VJ2Pipe Setting Element Count from fid Header: $parmInfo(celem)"
                      }
                  } \
               else \
                  {
                   if {$procpar(vjhdr,FIDCount) < $parmInfo(celem)} \
                      {
                       if {$verbose} \
                          {
                           showMsg "VJ2Pipe Updating Element Count from fid Header. Old: $parmInfo(celem) New: $procpar(vjhdr,FIDCount)"
                          }

                       set parmInfo(celem) $procpar(vjhdr,FIDCount)
                      }
                  }
              }
          }
      }

   getScalarArgs celem

   return 0 
}

proc readProcpar {} \
{
   global parmInfo parmCount procpar msgUnit

   set parmCount 0

   if {![file exists $parmInfo(procpar)]} \
      {
       showMsg "VJ2Pipe Error finding procpar file $parmInfo(procpar)"
       return 1
      }

   set status    [rdProcPar $parmInfo(procpar) procpar $msgUnit]
   set parmCount [array size procpar]

   set id [array startsearch procpar]

   while {[array anymore procpar $id]} \
      {
       set v [array nextelement procpar $id]
       if {![info exists procpar($v)]} continue

       if {![strcasecmp $procpar($v) "(Null)"]} {set procpar($v) ""}
      }

   array donesearch procpar $id

   return $parmCount
}

proc getDimCount {} \
{
   global procpar parmInfo dimCount

   set parmInfo(dimCount)  0
   set parmInfo(ndim)      0
   set parmInfo(ndimOrig)  0
   set parmInfo(arrayDim)  1

   set vList    "np ni ni2 ni3 ni4"
   set dimCount 0
   set thisMax  0
   set i        0

   if {[info exists procpar(arraydim)]} \
      {
       if {![info exists procpar(ni)]} {set procpar(ni) 1}
      }

   for {set dim 1} {$dim < [llength $vList]} {incr dim} \
      {
       set v [lindex $vList [expr $dim - 1]]
       if {![info exists procpar($v)]} continue 
       if {$procpar($v) > 0} {set thisMax $dim}
      }

   foreach v $vList \
      {
       if {![info exists procpar($v)]} \
          {
           set procpar($v) 1
           break
          }

       if {$procpar($v) == 0} {set procpar($v) 1}
       if {$procpar($v) < 1} break

       incr i
      }

   set dimCount $i

   if {$dimCount > $thisMax} {set dimCount $thisMax}

   if {[info exists procpar(arraydim)]} \
      {
       if {[string is integer $procpar(arraydim)]} {set parmInfo(arrayDim) $procpar(arraydim)}
      } 

   set parmInfo(dimCount) $dimCount

   getScalarArgs dimCount arrayDim

   set parmInfo(ndim)     $parmInfo(dimCount)
   set parmInfo(ndimOrig) $parmInfo(ndim)

   return $dimCount
}

proc getAxisList {} \
{
   global parmInfo

    set aList "x y z a b c d e f g h i j k l m n o p q r s t u v w"
    
   set parmInfo(axisPrefix) $aList
   set parmInfo(axisList)   $aList

   getListArgs axisList
}

proc getAQOrd {} \
{
   global procpar parmInfo dimCount

   set parmInfo(aqORD) 0

   if {![info exists procpar(array)]} return

   set sList [split $procpar(array) ","]
   set loc1  [lsearch -exact $sList phase]

   if {$loc1 >= 0} \
      {
       set nBefore 0

       for {set i 2} {$i < $dimCount} {incr i} \
          {
           set locN [lsearch -exact $sList phase$i]
           if {$locN < 0} continue

           if {$locN < $loc1} {incr nBefore}
          }

       if {!$nBefore} {set parmInfo(aqORD) 1}
      }

   getScalarArgs aqORD 
}

proc updatePartialExp {} \
{
   global partialFlag procpar parmInfo dimCount verbose

   set n 1

   if {$parmInfo(nusFlag)} return

   for {set dim 2} {$dim <= $dimCount} {incr dim} \
      {
       set n [expr $n*$parmInfo(N,$dim)]
      }

   if {$parmInfo(celem) < 1 || $parmInfo(celem) == $n} return

   if {$partialFlag} \
      {
       if {$parmInfo(celem) > $n} \
          {
           showMsg "VJ2Pipe Notice: Expected Element Count: $n Completed: $parmInfo(celem)"
          } \
       elseif {$parmInfo(celem) < $n} \
          {
           showMsg "VJ2Pipe Updating Sizes. Expected Element Count: $n Completed: $parmInfo(celem)"

           if {$dimCount == 2} \
              {
               set parmInfo(T,2) [expr int($parmInfo(celem)/$parmInfo(quadAdjust,2))]
               set parmInfo(N,2) [expr int($parmInfo(T,2)*$parmInfo(quadAdjust,2))]

               if {$parmInfo(T,2) < 1} \
                  {
                   set parmInfo(N,2) 1
                   set parmInfo(T,2) 1
                   set parmInfo(quadAdjust,2) 1
                   set parmInfo(MODE,2) Real 
                   set parmInfo(aq2D)   Real 
                   set dimCount 1
                  }
              } \
           elseif {$dimCount == 3} \
              {
               set parmInfo(T,3) [expr int($parmInfo(celem)/($parmInfo(T,2)*$parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)))]
               set parmInfo(N,3) [expr int($parmInfo(T,3)*$parmInfo(quadAdjust,3))]

               if {$parmInfo(T,3) < 1} \
                  {
                   set parmInfo(N,3) 1
                   set parmInfo(T,3) 1
                   set parmInfo(quadAdjust,3) 1
                   set parmInfo(MODE,3) Real
                   set dimCount 2
                  }
              } \
           elseif {$dimCount == 4} \
              {
               set parmInfo(T,4) [expr int($parmInfo(celem)/($parmInfo(T,2)*$parmInfo(quadAdjust,2)*$parmInfo(T,3)*$parmInfo(quadAdjust,3)*$parmInfo(quadAdjust,4)))]
               set parmInfo(N,4) [expr int($parmInfo(T,4)*$parmInfo(quadAdjust,4))]

               if {$parmInfo(T,4) < 1} \
                  {
                   set parmInfo(N,4) 1
                   set parmInfo(T,4) 1
                   set parmInfo(quadAdjust,4) 1
                   set parmInfo(MODE,4) Real
                   set dimCount 3
                  }
              }

           getAxisArgs T N
          }
      } \
   else \
      {
       if {$parmInfo(celem) < $n} \
          {
           showMsg "VJ2Pipe Notice: Parameters indicate a partially-completed experiment."
           showMsg "VJ2Pipe Notice: Expected Element Count: $n Completed: $parmInfo(celem)"
           showMsg "VJ2Pipe Notice: Use the -partial option to adjust accordingly."
          } \
       else \
          {
           showMsg "VJ2Pipe Notice: Expected Element Count: $n Completed: $parmInfo(celem)"
          }
      }
}

proc updateArrayExp {} \
{
   global procpar parmInfo dimCount verbose

   if {$parmInfo(arrayDim) <= 1} return

   if {![info exists procpar(ni)]} {set procpar(ni) 1}

   if {$parmInfo(ndim) == 1 && $procpar(ni) <= 1} \
      {
       if {$verbose} \
          {
           showMsg "VJ2Pipe Updating for Arrayed 1D Experiment with $parmInfo(arrayDim) elements."

           set parmInfo(ndim)   2
           set dimCount         2

           set parmInfo(T,2)    $parmInfo(arrayDim)
           set parmInfo(N,2)    $parmInfo(arrayDim)
           set parmInfo(SW,2)   $parmInfo(arrayDim)
           set parmInfo(MODE,2) Real 
           set parmInfo(CAR,2)  0.0
           set parmInfo(OBS,2)  0.0
           set parmInfo(LAB,2)  ID
           set parmInfo(aq2D)   Real
          }
      } 
}

proc getArrayExp {} \
{
   global procpar parmInfo dimCount

   set parmInfo(arrayList)         ""
   set parmInfo(interleavedFlag)   0
   set parmInfo(interleavedInside) 0
   set parmInfo(interleavedType)   None 

   if {[info exists procpar(array)]} \
      {
       set parmInfo(arrayList) [split $procpar(array) ","]
      }

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set parmInfo(array,$dim) None
      }

   set n [llength $parmInfo(arrayList)]

   if {$n > $dimCount - 1} \
      {
       set parmInfo(interleavedFlag) 1
       set parmInfo(interleaveType)  [lindex $parmInfo(arrayList) 0]

       if {[string first phase $parmInfo(interleaveType)] != 0} \
          {
           set parmInfo(interleavedInside) 0
          }
      }

   set dim 2
  
   foreach i $parmInfo(arrayList) \
      {
       if {[lsearch -exact "phase phase2 phase3 phase4" $i] >= 0} \
          {
           set parmInfo(array,$dim) phase
          } \
       else \
          {
           set parmInfo(array,$dim) $i 
          }

       incr dim
      }
}

proc getTSize {} \
{
   global procpar parmInfo dimCount partialFlag

   set vList "np ni ni2 ni3 ni4"

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set i [lindex $vList [expr $dim - 1]]

       if {[info exists procpar($i)]} \
          {
           if {$dim == 1} \
              {
               set parmInfo(T,$dim) [expr int($procpar($i)/2)]
              } \
           else \
              {
               set parmInfo(T,$dim) [expr int($procpar($i))]
              }

           if {$parmInfo(T,$dim) < 1} {set parmInfo(T,$dim) 1}

           set parmInfo(N,$dim) $parmInfo(T,$dim)
          }
      }

   getAxisArgs   T
}

proc getLShift {} \
{
   global procpar parmInfo dimCount

   set vList "lsfid lsfid1 lsfid2 lsfid3 lsfid4"

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set i [getVJName $vList $dim]

       if {[info exists procpar($i)]} \
          {
           set parmInfo(LSTIME,$dim) $procpar($i)
          }
      }

   set vList "lsfrq lsfrq1 lsfrq2 lsfrq3 lsfrq4"
   
   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set i [getVJName $vList $dim]
   
       if {[info exists procpar($i)]} \
          {
           set n [scan $procpar($i) "%f" val]

           if {$n == 1} \
              {
               if {$procpar($i) == 0.0} \
                  {
                   set parmInfo(LSFREQ,$dim) 0
                  } \
               else \
                  {
                   set parmInfo(LSFREQ,$dim) [safeFormat "%.2fHz" $procpar($i)]
                  }
              }
          }
      }

   getAxisArgs LSTIME LSFREQ 
}


proc getSW {} \
{
   global procpar parmInfo dimCount

   set vList "sw sw1 sw2 sw3 sw4"

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set i [getVJName $vList $dim]

       if {[info exists procpar($i)]} \
          {
           set parmInfo(SW,$dim) $procpar($i)
          }
      }

   getAxisArgs SW
}

proc getBASE {} \
{
   global procpar parmInfo dimCount

   if {![strcasecmp $parmInfo(type) HN]} \
      {
       set parmInfo(BASEARG,1) "POLY,auto"
      }

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       if {[strcasecmp $parmInfo(BASEARG,$dim) NULL]} continue

       if {abs($parmInfo(P1,$dim) - 360) < 10.0} \
          {
           set parmInfo(BASEARG,$dim) "POLY,auto,ord=0"
          }
      }

   getAxisArgs BASEARG 
}

proc getEXT {} \
{
   global procpar parmInfo

   set useVJExt 0

   if {[info exists procpar(BPpipevjregion)]} \
      {
       set val [string trim $procpar(BPpipevjregion)]
       if {![strcasecmp $val y]} {set useVJExt 1}
      }

   if {[info exists procpar(BPpiperegion)]} \
      {
       set val [string trim $procpar(BPpiperegion)]
       if {![strcasecmp $val y]} {set useVJExt 1}
      }

   set obs $parmInfo(OBS,1)

   if {$obs != 0.0 && $useVJExt && \
        [info exists procpar(wp)] && [info exists procpar(sp)]} \
      {
       set wp [string trim $procpar(wp)]
       set wu [string index $wp end]

       set sp [string trim $procpar(sp)]
       set su [string index $sp end]      

       if {![strcasecmp $wu p]} \
          {
           set wp [string range $wp 0 end-1]
          } \
       else \
          {
           set wp [expr $wp/$obs]
          } 

       if {![strcasecmp $su p]} \
          {
           set sp [string range $sp 0 end-1]
          } \
       else \
          {
           set sp [expr $sp/$obs]
          }

       if {$wp > 0.0} \
          {
           set parmInfo(EXTXN,1) [safeFormat "%.3fppm" $sp]
           set parmInfo(EXTX1,1) [safeFormat "%.3fppm" [expr $sp + $wp]]
          } \
       elseif {$wp < 0.0} \
          {
           set parmInfo(EXTX1,1) [safeFormat "%.3fppm" $sp]
           set parmInfo(EXTXN,1) [safeFormat "%.3fppm" [expr $sp + $wp]]
          }
      }

   getAxisArgs EXTX1 EXTXN
}

proc getApod {} \
{
   global procpar parmInfo dimCount vjApodFlag
   
   set parmInfo(Q3,1) 2
 
   set vList "lb lb1 lb2 lb3 lb4"

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       if {$parmInfo(SW,$dim) != 0.0} \
          {
           set parmInfo(AQTIME,$dim) \
            [expr double($parmInfo(T,$dim))/$parmInfo(SW,$dim)]
          }
      }

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set i [getVJName $vList $dim]

       if {[info exists procpar($i)]} \
          {
           set parmInfo(VJLB,$dim) $procpar($i)
          }
      }

   set vList "gf gf1 gf2 gf3 gf4"

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set i [getVJName $vList $dim]

       if {[info exists procpar($i)]} \
          {
           set parmInfo(VJGF,$dim) $procpar($i)
          }
      }
  
   set vList "gfs gfs1 gfs2 gfs3 gfs4"

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set i [getVJName $vList $dim]

       if {[info exists procpar($i)]} \
          {
           set parmInfo(VJGFS,$dim) $procpar($i)
          }
      }

   set vList "sb sb1 sb2 sb3 sb4"

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set i [getVJName $vList $dim]

       if {[info exists procpar($i)]} \
          {
           set parmInfo(VJSB,$dim) $procpar($i)
          }
      }

   set vList "sbs sbs1 sbs2 sbs3 sbs4"

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {   
       set i [getVJName $vList $dim]

       if {[info exists procpar($i)]} \
          {
           set parmInfo(VJSBS,$dim) $procpar($i)
          }
      }

# 
# NMRPipe: exp( -PI*i*lb/sw )
# VNMRJ:   exp( -lb/(0.31831*sw)*i )
#
#   lbconst  = lb/(0.31831*sw)
#   gfconst  = 1.0/(sw*gf)
#   maxpoint = sw*gfs
#
#  i = 0; i < n; i++:
#     gfvar = (i - maxpoint)*gfconst;
#     w = exp( -i*lbconst )*exp( -gfvar*gfvar );
#
# NMRPipe SP(off,end,pow): sin( PI*off + PI*(end-off)*i/(n-1) )^pow

   if {$vjApodFlag} \
      {
       for {set dim 1} {$dim <= $dimCount} {incr dim} \
          {   
           set parmInfo(APOD,$dim) SP
           set aT 1.0

           if {[info exists parmInfo(AQTIME,$dim)]} \
              {
               set aT [expr double($parmInfo(AQTIME,$dim))]
              }

           if {$aT == 0.0} {set aT 1.0}

           if {[info exists parmInfo(VJLB,$dim)]} \
              {
               set elb $parmInfo(VJLB,$dim)
              }

# gT = 0.6*PI*g2*i/sw for exp( -gT*gT )

           if {[info exists parmInfo(VJGF,$dim)]} \
              {
               set glb  [expr $parmInfo(VJGF,$dim)/$aT]
              }
 
           if {[info exists parmInfo(VJGFS,$dim)]} \
              {
               set goff [expr $parmInfo(VJGFS,$dim)/$aT]
              }
 
           if {[info exists parmInfo(VJSB,$dim)] \
                && [info exists parmInfo(VJSBS,$dim)] \
                 && $parmInfo(T,$dim) > 1} \
              {
               set sb  [expr $parmInfo(VJSB,$dim)/$aT]
               set sbs [expr $parmInfo(VJSBS,$dim)/$aT]
              }
          }
      }

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {   
       if {$parmInfo(T,$dim) < 2} \
          {
           set parmInfo(APOD,$dim) SP
           set parmInfo(Q1,$dim)   0.5
           set parmInfo(Q2,$dim)   0.5 
           set parmInfo(Q3,$dim)   1.0
          }
      }

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {   
       if {![info exists parmInfo(P1,$dim)]} continue
       if {abs($parmInfo(P1,$dim)) < 10.0} {set parmInfo(C1,$dim) 0.5}
      }

   if {![strcasecmp $parmInfo(aq2D) Magnitude]} \
      {
       foreach dim {1 2} \
          {
           set parmInfo(APOD,$dim) SP
           set parmInfo(Q1,$dim)   0.05
           set parmInfo(Q2,$dim)   0.95 
           set parmInfo(Q3,$dim)   2.0
           set parmInfo(C1,$dim)   1.0
          }
      }


   if {[info exists procpar(pipeQ1)] && [info exists procpar(pipeQ2)] && [info exists procpar(pipeQ3)]} \
      {
       set q1List "$procpar(pipeQ1) None None None None"
       set q2List "$procpar(pipeQ2) None None None None"
       set q3List "$procpar(pipeQ3) None None None None"

       set nVal 0

       for {set dim 1} {$dim <= $dimCount} {incr dim} \
          {
           set q1Val [lindex $q1List [expr $dim - 1]]
           set q2Val [lindex $q2List [expr $dim - 1]]
           set q3Val [lindex $q3List [expr $dim - 1]]

           if {![strcasecmp $q1Val None] || ![strcasecmp $q1Val default] || ![strcasecmp $q1Val unused]} \
              {
               set q1Val _default_
              } \
           else \
              {
               if {[string is double $q1Val]} \
                  {
                   if {$q1Val != 0.0} {incr nVal}
                  } \
               else \
                  {
                   set $q1Val _default_
                  }
              }

           if {![strcasecmp $q2Val None] || ![strcasecmp $q2Val default] || ![strcasecmp $q2Val unused]} \
              {
               set q2Val _default_
              } \
           else \
              {
               if {[string is double $q2Val]} \
                  {
                   if {$q2Val != 0.0} {incr nVal}
                  } \
               else \
                  {
                   set $q2Val _default_
                  }
              }

           if {![strcasecmp $q3Val None] || ![strcasecmp $q3Val default] || ![strcasecmp $q3Val unused]} \
              {
               set q3Val _default_
              } \
           else \
              {
               if {[string is double $q3Val]} \
                  {
                   if {$q2Val != 0.0} {incr nVal}
                  } \
               else \
                  {
                   set $q3Val _default_
                  }
              }

           if {!$nVal} continue

#          set parmInfo(Q1,$dim) $q1Val
#          set parmInfo(Q2,$dim) $q2Val
#          set parmInfo(Q3,$dim) $q3Val
          }
      }

# %%% p {Q1 Q2 Q3 ELB GLB GOFF} 

   foreach p {ELB GLB GOFF} \
      {
       if {![info exists procpar(pipe$p)]} continue

       set valList "$procpar(pipe$p) None None None None"

       for {set dim 1} {$dim <= $dimCount} {incr dim} \
          {
           set val [lindex $valList [expr $dim - 1]]

           if {![strcasecmp $val None] || ![strcasecmp $val default] || ![strcasecmp $val unused]} \
              {
               set parmInfo($p,$dim) _default_ 
              } \
           else \
              {
               if {[string is double $val]} {set parmInfo($p,$dim) $val}
              }
          }
      } 

   getAxisArgs APOD Q1 Q2 Q3 C1 ELB GLB GOFF VJLB VJGF VJGFS JVSB VJSBS
}

proc getSpecial {} \
{
   global procpar parmInfo dimCount

   foreach p {istTMult istCMult istMaxRes} \
      {
       if {![info exists procpar(pipe$p)]} continue 
       if {![string is double $procpar(pipe$p)]} continue 
       set parmInfo($p) $procpar(pipe$p)
      }

   foreach p {istIter} \
      {
       if {![info exists procpar(pipe$p)]} continue 
       if {![string is integer $procpar(pipe$p)]} continue 
       set parmInfo($p) $procpar(pipe$p)
      }

   getScalarArgs istTMult istCMult istMaxRes istIter

   foreach p {istTMult istCMult istMaxRes istIter} \
      {
       if {![strcasecmp $parmInfo($p) None] || ![strcasecmp $parmInfo($p) default] || ![strcasecmp $parmInfo($p) unused]} \
          {
           set parmInfo($p) _default_
          }
      }
}

proc getPhase {} \
{
   global procpar parmInfo dimCount

   set vList0 "rp rp1 rp2 rp3 rp4"
   set vList1 "lp lp1 lp2 lp3 lp4"

# Delaglio
# Previous scheme, direct P0/P1 same, possibly negate indirect P0/P1.
# New scheme, NMRPipe P0 = lp + rp, P1 = -lp

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set thisP0 0.0
       set thisP1 0.0

       set i [getVJName $vList0 $dim]
       if {[info exists procpar($i)]} {set thisP0 $procpar($i)} 

       set i [getVJName $vList1 $dim]
       if {[info exists procpar($i)]} {set thisP1 $procpar($i)} 

       if {$thisP0 != 0.0 || $thisP1 != 0.0} \
          {
           set parmInfo(P1,$dim) [expr -1.0*$thisP1]
           set parmInfo(P0,$dim) [expr $thisP0 + $thisP1]
          }
      }

   set vList "f1180 f2180 f3180 f4180"

   for {set dim 2} {$dim <= $dimCount} {incr dim} \
      {
       set i [lindex $vList [expr $dim - 1]]

       if {[info exists procpar($i)]} \
          {
           if {[isON $procpar($i)]} \
              {
               if {abs($parmInfo(P0,$dim)) == 0.0 && abs($parmInfo(P1,$dim)) == 0.0} \
                  {
                   set parmInfo(P0,$dim) -90.0
                   set parmInfo(P1,$dim) 180.0
                  }
              }
          }
      }

   if {$dimCount == 2 && [info exists procpar(f1coef)]} \
      {
       if {![lcompare $procpar(f1coef) "1 0 1 0 0 -1 0 1"]} \
          {
           set parmInfo(P0EXTRA,1)  90.0
           set parmInfo(P0EXTRA,2) -90.0
          } \
       elseif {![lcompare $procpar(f1coef) "1 0 1 0 0 1 0 -1"]} \
          {
           set parmInfo(P0EXTRA,1) 90.0
           set parmInfo(P0EXTRA,2) 90.0

           if {![strcasecmp $parmInfo(FTARG,2) None]} {set parmInfo(FTARG,2) neg}
          }
      }

   getAxisArgs P0EXTRA P1EXTRA P0 P1 FTARG

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set parmInfo(P0,$dim) [expr $parmInfo(P0EXTRA,$dim) + $parmInfo(P0,$dim)]
       set parmInfo(P1,$dim) [expr $parmInfo(P1EXTRA,$dim) + $parmInfo(P1,$dim)]
      }
}

proc getQuadSize {} \
{
   global procpar parmInfo dimCount skipFlag

   set parmInfo(aq2D) Complex

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set parmInfo(quadAdjust,$dim) 2
       set parmInfo(MODE,$dim) Complex
      }

   if {$dimCount == 2} \
      {
       if {![strcasecmp $parmInfo(array,2) None]} \
          {
           set parmInfo(aq2D) Real
          }

       if {[info exists procpar(dmg)] && $parmInfo(T,2) > 1} \
          {
           set key [string range [string trim $procpar(dmg)] 0 1]
           if {![strcasecmp $key av] || ![strcasecmp $key pw]} {set parmInfo(mcFlag) 1}
          }

       if {[info exists procpar(dmg1)] && $parmInfo(T,2) > 1} \
          {
           set key [string range [string trim $procpar(dmg1)] 0 1]

           if {![strcasecmp $key av] || ![strcasecmp $key pw]} \
              {
               set parmInfo(aq2D)   Magnitude
               set parmInfo(mcFlag) 1
              }
          }
      }

   getScalarArgs aq2D

   if {![strcasecmp $parmInfo(aq2D) Magnitude] || ![strcasecmp $parmInfo(aq2D) Real]} \
      {
       set parmInfo(MODE,2)       Real
       set parmInfo(quadAdjust,2) 1
       set parmInfo(FTARG,2)      "neg"
      }

   set parmInfo(axisPrefix) [lindex $parmInfo(axisList) 0] 
   set parmInfo(ndim) 1

   set i  0
   set nG 0

   for {set dim 2} {$dim <= $dimCount} {incr dim} \
      {
       incr i

       if {$dim == 2 && $parmInfo(quadAdjust,$dim) == 1 && [strcasecmp $parmInfo(aq2D) Magnitude]} continue
       
       set v f${i}coef

       if {![info exists procpar($v)]} \
          {
           set coefList ""
          } \
       else \
          {
           set coefList $procpar($v)
          }

       set n 0

       foreach ic $coefList \
          {
           if {[strcmp $ic 0]} {incr n} 
          }

       if {$n != 0 && $n != 2} \
          {
           set parmInfo(MODE,$dim) "Rance-Kay"
           incr nG
          }

       if {$parmInfo(T,$dim) < 2} \
          {
           set parmInfo(MODE,$dim)       "Real"
           set parmInfo(quadAdjust,$dim) 1
          }

       if {$parmInfo(T,$dim) > 1 || !$skipFlag} \
          {
           lappend parmInfo(axisPrefix) \
            [lindex $parmInfo(axisList) [expr $dim - 1]]

           incr parmInfo(ndim)
          }
      }

   foreach a $parmInfo(axisList) \
      {
       if {[lsearch -exact $parmInfo(axisPrefix) $a] < 0} \
          {
           lappend parmInfo(axisPrefix) $a
          }
      }

   if {$nG > 1} \
      {
       set parmInfo(FTARG,2) "alt,neg"
      }

   getListArgs axisPrefix
   getAxisArgs MODE FTARG

   set realList {real tppi real-n tppi-n}

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set loc [lsearch -exact $realList [string tolower $parmInfo(MODE,$dim)]]

       if {$loc < 0} \
          {
           set parmInfo(quadAdjust,$dim) 2
          } \
       else \
          {
           set parmInfo(quadAdjust,$dim) 1
          }

       set parmInfo(N,$dim) [expr $parmInfo(quadAdjust,$dim)*$parmInfo(T,$dim)]
      }

   getScalarArgs ndim
   getAxisArgs   N

   set parmInfo(ndimOrig) $parmInfo(ndim)
}

proc getTemperature {} \
{
   global procpar parmInfo

   if {[info exists procpar(temp)]} \
      {
       set parmInfo(temperature) [expr 273.0 + $procpar(temp)]
      }

   getScalarArgs temperature
}

proc getNUC {} \
{
   global procpar parmInfo dimCount verbose

   set vList   "sfrq dfrq dfrq2 dfrq3 dfrq4"
   set nList   "tn   dn   dn2   dn3   dn4"
   set nucList ""

   if {[info exists procpar(dimension)]} \
      {
       set parmInfo(dimension) $procpar(dimension)
      } \
   else \
      {
       set parmInfo(dimension) ""
      }

   getListArgs dimension

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set j [expr 1 + $dimCount - $dim]

       set parmInfo(NUC,$dim) H1

       set i [lindex $nList [expr $dim - 1]]
       if {![info exists procpar($i)]} continue
 
       set parmInfo(NUC,$dim) $procpar($i)
       lappend nucList $procpar($i)
      }

   for {set dim 2} {$dim <= $dimCount} {incr dim} \
      {
       set j [expr $dim - 1]
       set i refsource$j

       if {![info exists procpar($i)]} continue
 
       set loc [lsearch -exact $vList [string tolower $procpar($i)]]
       if {$loc < 0} continue

       set i [lindex $nList $loc]
       if {![info exists procpar($i)]} continue 

       set parmInfo(NUC,$dim) $procpar($i)
      }

   set n [llength $parmInfo(dimension)]

   if {$n} \
      {
       set nuc [lindex $parmInfo(dimension) end]
       set loc 0

       set parmInfo($nuc,1) $nuc

       for {set dim 2} {$dim <= $dimCount} {incr dim} \
           {
            if {$dim > $n} break

            set nuc [lindex $parmInfo(dimension) $loc]

            if {[lsearch -exact $nucList $nuc] >= 0} \
               {
                set parmInfo(NUC,$dim) $nuc
               }

            incr loc
           }
      }

   getAxisArgs NUC
}

proc getLAB {} \
{
   global procpar parmInfo dimCount

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       if {$dim == 1} \
          {
           set parmInfo(LAB,$dim) F$dimCount
          } \
       else \
          {
           set j [expr $dim - 1]
           set parmInfo(LAB,$dim) F$j
          }
      
       if {![info exists parmInfo(NUC,$dim)]} continue
 
       set parmInfo(LAB,$dim) $parmInfo(NUC,$dim)
      }

   parseExpType

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
#      if {![strcasecmp $parmInfo(LAB,$dim) N15]} {set parmInfo(LAB,$dim) N}
      }

   set aList $parmInfo(axisPrefix)

   for {set dimI 1} {$dimI <= $dimCount} {incr dimI} {set adjLab($dimI) 0}
 
   for {set dimI 1} {$dimI < $dimCount} {incr dimI} \
      {
       for {set dimJ [expr $dimI + 1]} {$dimJ <= $dimCount} {incr dimJ} \
           {
            if {[strcasecmp $parmInfo(LAB,$dimI) $parmInfo(LAB,$dimJ)]} continue

            if {!$adjLab($dimI)} \
               {
                append parmInfo(LAB,$dimI) [lindex $aList [expr $dimI - 1]]
                set adjLab($dimI) 1
               }

            if {!$adjLab($dimJ)} \
               {
                append parmInfo(LAB,$dimJ) [lindex $aList [expr $dimJ - 1]]
                set adjLab($dimJ) 1
               }
           }
      }

   getAxisArgs   LAB
   getScalarArgs type

   if {![strcasecmp $parmInfo(type) HN]} \
      {
       set parmInfo(SOL,1)   SOL
       set parmInfo(EXTX1,1)  3%
       set parmInfo(EXTXN,1) 47%
      }
}

proc parseExpType {} \
{
   global parmInfo dimCount

   set nucList ""
   
   set nN15   0
   set nC13   0
   set nH1    0
 
   set h1Dim  0
   set c13Dim 0
   set n15Dim 0
  
   set dim 1

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set nuc [string toupper $parmInfo(NUC,$dim)]
       lappend nucList $nuc

       if {![strcasecmp $nuc H1]}  \
          {
           set  h1Dim $dim
           incr nH1
          }

       if {![strcasecmp $nuc C13]} \
          {
           set  c13Dim $dim
           incr nC13
          }

       if {![strcasecmp $nuc N15]} \
          {
           set  n15Dim $dim
           incr nN15
          }
      }

   if {![llength $nucList]} return

   set nuc1 [lindex $nucList 0]

   if {$dimCount == 1} \
      {
       set parmInfo(type) $nuc1
       return
      }

   if {[llength [listUniq $nucList]] == 1} \
      {
       set parmInfo(type) HOMO
       return
      }

   if {[strcasecmp $nuc1 H1]} return

   set eName [string tolower $parmInfo(title)]

   if {$nN15} \
      {
       set parmInfo(type)     HN
       set parmInfo(LAB,1)    HN
       set parmInfo(SOL,1)   SOL
       set parmInfo(EXTX1,1)  3%
       set parmInfo(EXTXN,1) 47%
      } \
   elseif {$nC13} \
      {
       set parmInfo(type)  HC

       if {[string first haca $eName] >= 0 || [string first hca $eName] >= 0} \
          {
           set parmInfo(LAB,1) HA
          }

       if {![strcasecmp haca $eName] && $dimCount == 2} \
          {
           if {![strcasecmp C13 $parmInfo(NUC,2)]} {set parmInfo(LAB,2) CA}
          }
      }

   if {[string first n15 $eName] >= 0 \
        || [string first hn $eName] >= 0 \
         || [string first nh $eName] >= 0 \
          || [string first h(n) $eName] >= 0} \
      {
       set parmInfo(LAB,1) HN
       set parmInfo(type)  HN
      }

   if {![strcasecmp hnco $eName] && $nC13 == 1} \
      {
       set parmInfo(LAB,$c13Dim) CO
       return
      }

   if {![strcasecmp hnca $eName] && $nC13 == 1} \
      {
       set parmInfo(LAB,$c13Dim) CA
       return
      }

   set nCACO 0

   foreach t {hcaco hacaco hncoca hncaco} \
      {
       if {[string first $t $eName] >= 0} {incr nCACO}
      }

   if {$nC13 \
        && [string first co $eName] >= 0 \
         && [strcasecmp $parmInfo(coThresh) None]} \
      {
       for {set dim 2} {$dim <= $dimCount} {incr dim} \
          {
           if {!$parmInfo(CARSET,$dim) && !$parmInfo(CAR,$dim,argc)} continue

           if {![strcasecmp $parmInfo(NUC,$dim) C13]} \
              {
               if {$parmInfo(CAR,$dim) > $parmInfo(coThresh)} \
                  {
                   set parmInfo(LAB,$dim) CO
                  } \
               else \
                  {
                   if {$nCACO} \
                      {
                       set parmInfo(LAB,$dim) CA
                      }
                  }
              }
          }
      }

   if {!$nN15} return

   set caList {hnca h(n)ca h(nco)ca hn(co)ca}

   if {$nC13 == 1} \
      {
       if {[string first cbca $eName] >= 0 || [string first cacb $eName] >= 0} \
          {
           set parmInfo(LAB,$c13Dim) CACB
           return
          } 

       if {[lsearch -exact $caList $eName] >= 0} \
          {
           set parmInfo(LAB,$c13Dim) CA
           return
          }

       if {[string first hn(coca)cb $eName] >= 0} \
          {
           set parmInfo(LAB,$c13Dim) CB
           return
          } 
      } 

   set haList   {hnha h(n)ha h(nco)ha hn(co)ha hncaha hn(co)caha}

   if {$nH1 == 2} \
      {
       if {[string first hahb $eName] >= 0 || [string first hbha $eName] >= 0} \
          {
           set parmInfo(LAB,$h1Dim) HAHB 
           return
          } 

       foreach t $haList \
          {
           if {[string first $t $eName] >= 0} \
              {
               set parmInfo(LAB,$h1Dim) HNHA
               set parmInfo(LAB,$h1Dim) H1
               return
              }
          } 
      }
}

proc getOBS {} \
{
   global procpar parmInfo dimCount verbose

   set vList   "sfrq dfrq dfrq2 dfrq3 dfrq4"
   set nList   "tn   dn   dn2   dn3   dn4"
   set nucList ""

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set i [lindex $nList [expr $dim - 1]]

       if {[info exists procpar($i)]} \
          {
           lappend nucList $procpar($i)
          } \
       else \
          {
           lappend nucList xx 
          }

       set i tn
       if {[info exists procpar($i)]} {set parmInfo(OBS,$dim) $procpar($i)}
      } 

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set i [lindex $vList [expr $dim - 1]]
       if {![info exists procpar($i)]} continue
 
       set parmInfo(OBS,$dim) $procpar($i)
      }

   for {set dim 2} {$dim <= $dimCount} {incr dim} \
      {
       set j [expr $dim - 1]
       set i refsource$j

       if {[info exists procpar($i)]} \
          {
           set loc [lsearch -exact $vList [string tolower $procpar($i)]]
           if {$loc < 0} continue

           set i [lindex $vList $loc]
           if {![info exists procpar($i)]} continue 

           set parmInfo(OBS,$dim)  $procpar($i)
           set parmInfo(CHAN,$dim) [expr $loc + 1]
          }
      }

   if {[llength $parmInfo(dimension)] || $parmInfo(NUC,all,argc)} \
      {
       for {set dim 1} {$dim <= $dimCount} {incr dim} \
          {
           set loc [lsearch -exact $nucList $parmInfo(NUC,$dim)]
           if {$loc < 0} continue 

           set i [lindex $vList $loc]
           if {![info exists procpar($i)]} continue 

           set parmInfo(OBS,$dim)  $procpar($i)
           set parmInfo(CHAN,$dim) [expr $loc + 1]
          }
      }

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set parmInfo(OBSMID,$dim)  $parmInfo(OBS,$dim)
      }

   getAxisArgs OBS OBSMID CHAN
}

proc getCAR {} \
{
   global procpar parmInfo dimCount verbose

   if {[strcasecmp $parmInfo(NUC,1) H1]} {set parmInfo(CAR,1) 0.0}

   if {[info exists procpar(obscenterppm)]} \
      {
       set n [scan $procpar(obscenterppm) "%f" cPPM]
       if {$n == 1} {set parmInfo(CAR,1) $cPPM}
      }

   getAxisArgs CAR

   if {![strcasecmp $parmInfo(CAR,1) H2O]} \
      {
       setPrecision

       set parmInfo(CAR,1) [getVJH2Oppm $parmInfo(temperature)]

       adjCARZero 1

       set hOBS $parmInfo(OBS,1)
       set hPPM $parmInfo(CAR,1)
 
       for {set dim 2} {$dim <= $dimCount} {incr dim} \
          {
           set g   $parmInfo(GAMMA,$dim)
           set val $parmInfo(OBS,$dim)

           if {$val == 0.0} continue
 
           set parmInfo(CAR,$dim) \
             [expr (1.0e+6*$val-((($hOBS*1.0e+6)-($hOBS*$hPPM))*$g))/$val]

           adjCARZero $dim
          }

       restorePrecision
      }

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set parmInfo(CARSET,$dim) 0
 
       if {$dim == 1} \
          {
           set nN $dimCount
           set nF f$nN
          } \
       else \
          {
           set nN [expr $dim - 1]
           set nF f$nN
          }

       foreach nuc {h1 c13 n15} \
          {
           set v ${nuc}ref$nF 

           if {[info exists procpar($v)]} \
              {
               set r $procpar($v)
               if {$r == 0.0} continue

               if {![strcasecmp $parmInfo(NUC,$dim) $nuc]} \
                  {
                   set parmInfo(CAR,$dim)    $r
                   set parmInfo(CARSET,$dim) $nN 
                  }
              }
          }
      }

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set parmInfo(CAR,$dim) [safeFormat %.4f $parmInfo(CAR,$dim)]
      }

   if {[info exists procpar(pipexCAR)]} \
      {
       if {[string is double $procpar(pipexCAR)]} \
          {
           if {$procpar(pipexCAR) > 0.0} {set parmInfo(CAR,1) [safeFormat %.4f $procpar(pipexCAR)]}
          }
      }

   if {[info exists procpar(pipeyCAR)]} \
      {
       if {[string is double $procpar(pipeyCAR)]} \
          {
           if {$procpar(pipeyCAR) > 0.0} {set parmInfo(CAR,2) [safeFormat %.4f $procpar(pipeyCAR)]}
          }
      }

   if {[info exists procpar(pipezCAR)]} \
      {
       if {[string is double $procpar(pipezCAR)]} \
          {
           if {$procpar(pipezCAR) > 0.0} {set parmInfo(CAR,3) [safeFormat %.4f $procpar(pipezCAR)]}
          }
      }

   if {[info exists procpar(pipeaCAR)]} \
      {
       if {[string is double $procpar(pipeaCAR)]} \
          {
           if {$procpar(pipeaCAR) > 0.0} {set parmInfo(CAR,4) [safeFormat %.4f $procpar(pipeaCAR)]}
          }
      }

   getAxisArgs CAR
}

proc adjCARZero { dim } \
{
   global parmInfo

   foreach i {1 2 3 4 5} \
      {
       set d [expr 1.0e-6*$parmInfo(CAR,$dim)*$parmInfo(OBS,$dim)]
       set parmInfo(OBS,$dim) [expr $parmInfo(OBSMID,$dim) - $d] 
      }
}

proc getOFF {} \
{
   global procpar parmInfo dimCount

   set vList "sfrq dfrq dfrq2 dfrq3 dfrq4"
   set oList "tof  dof  dof1  dof2  dof3"

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set j [expr 1 + $dimCount - $dim]

       set i [lindex $oList [expr $dim - 1]]
       if {![info exists procpar($i)]} continue

       set parmInfo(OFF,$dim) $procpar($i)
      }

   for {set dim 2} {$dim <= $dimCount} {incr dim} \
      {
       set j [expr $dim - 1]
       set i refsource$j

       if {![info exists procpar($i)]} continue
 
       set loc [lsearch -exact $vList [string tolower $procpar($i)]]
       if {$loc < 0} continue

       set i [lindex $oList $loc]
       if {![info exists procpar($i)]} continue

       set parmInfo(OFF,$dim) $procpar($i)
      }

   getAxisArgs OFF
}

proc getGAMMA {} \
{
   global parmInfo dimCount

    set parmInfo(gamma,H)    1.0
    set parmInfo(gamma,1H)   1.0
    set parmInfo(gamma,H1)   1.0

    set parmInfo(gamma,2H)   0.153506088
    set parmInfo(gamma,H2)   0.153506088

    set parmInfo(gamma,C)    0.251449530
    set parmInfo(gamma,13C)  0.251449530
    set parmInfo(gamma,C13)  0.251449530

    set parmInfo(gamma,N)    0.101329118
    set parmInfo(gamma,15N)  0.101329118
    set parmInfo(gamma,N15)  0.101329118

    set parmInfo(gamma,P)    0.40480864
    set parmInfo(gamma,31P)  0.40480864
    set parmInfo(gamma,P31)  0.40480864

    set parmInfo(gamma,off)  0.000000000
    set parmInfo(gamma,NONE) 0.000000000

    for {set dim 1} {$dim <= $dimCount} {incr dim} \
       {
        set parmInfo(GAMMA,$dim) 1.0
        if {![info exists parmInfo(NUC,$dim)]} continue

        set nuc [string toupper $parmInfo(NUC,$dim)]
        if {![info exists parmInfo(gamma,$nuc)]} continue

        set parmInfo(GAMMA,$dim) $parmInfo(gamma,$nuc)
       }

    getAxisArgs GAMMA
}

proc showParms args \
{
   global parmInfo dimCount vjShowFlag

   set thisArgv [concat showParms $args]

   set outUnit    stderr
   set scalarVars $parmInfo(scalarVars,names)
   set listVars   $parmInfo(listVars,names)
   set dimVars    $parmInfo(dimVars,names)

   set thisSkipFlag 1

   getArgD $thisArgv -scalarVars scalarVars
   getArgD $thisArgv -listVars   listVars
   getArgD $thisArgv -axisVars   dimVars
   getArgD $thisArgv -outUnit    outUnit
   getArgD $thisArgv -skip       thisSkipFlag

   set n 0

   foreach v $scalarVars \
      {
       if {![strcasecmp $v None]} continue
       if {[lsearch $parmInfo(hideList) $v] >= 0} continue

       set fmt %s
       set val None

       set i [lsearch -exact $parmInfo(scalarVars,names) $v]

       if {$i >= 0} \
          {
           set fmt [lindex $parmInfo(scalarVars,formats)  $i]
           set val [lindex $parmInfo(scalarVars,defaults) $i]       
          }

       if {[info exists parmInfo($v)]} {set val $parmInfo($v)} 

       set str " -$v [addQuotes [safeFormat $fmt $val]]"
       incr n [string length $str]

       puts -nonewline $outUnit $str

       if {$n > 50} \
          {
           puts $outUnit " \\"
           set n 0
          }
      }

   if {$n} {puts $outUnit " \\"}

   foreach v $listVars \
      {
       if {![strcasecmp $v None]} continue
       if {[lsearch $parmInfo(hideList) $v] >= 0} continue
       if {![info exists parmInfo($v)]} continue
       if {![llength $parmInfo($v)]} continue

       set key [string range $v 0 1]
       if {!$vjShowFlag && ![strcasecmp $key VJ]} continue

       puts $outUnit " -$v $parmInfo($v) \\"
      }

   foreach v $dimVars \
      {
       if {![strcasecmp $v None]} continue
       if {[lsearch $parmInfo(hideList) $v] >= 0} continue

       set key [string range $v 0 1]
       if {!$vjShowFlag && ![strcasecmp $key VJ]} continue 

       set fmt  %s
       set dval None

       set i [lsearch -exact $parmInfo(dimVars,names) $v]

       if {$i >= 0} \
          {
           set fmt  [lindex $parmInfo(dimVars,formats)  $i]
           set dval [lindex $parmInfo(dimVars,defaults) $i]
          }

       for {set dim 1} {$dim <= $dimCount} {incr dim} \
          {
           if {$parmInfo(nusFlag)} \
              {
               if {$thisSkipFlag && $parmInfo(nusInitSize,$dim) == 1} continue 
              } \
           else \
              {
               if {$thisSkipFlag && $parmInfo(N,$dim) == 1} continue 
              }

           set val [safeFormat $fmt $dval]
           set a   [lindex $parmInfo(axisPrefix) [expr $dim - 1]]
 
           if {$parmInfo(nusFlag) && $parmInfo(ndim) == 2 && [strcasecmp $a x]} {set a y}
    
           if {[lsearch $parmInfo(hideList) $a$v] >= 0} continue
           if {[info exists parmInfo($v,$dim)]} {set val $parmInfo($v,$dim)} 

           puts -nonewline $outUnit [format " %s%-7s %11s" -$a $v [safeFormat $fmt $val]]
          }

       puts $outUnit " \\"
      }
}

proc showParmsS args \
{
   global parmInfo dimCount vjShowFlag

   set thisArgv [concat showParms $args]

   set outUnit    stderr
   set scalarVars $parmInfo(scalarVars,names)
   set listVars   $parmInfo(listVars,names)
   set dimVars    $parmInfo(dimVars,names)

   set thisSkipFlag 1

   getArgD $thisArgv -scalarVars scalarVars
   getArgD $thisArgv -listVars   listVars
   getArgD $thisArgv -axisVars   dimVars
   getArgD $thisArgv -outUnit    outUnit
   getArgD $thisArgv -skip       thisSkipFlag

   set n 0

   foreach v $scalarVars \
      {
       if {![strcasecmp $v None]} continue
       if {[lsearch $parmInfo(hideList) $v] >= 0} continue

       set fmt %s
       set val None

       set i [lsearch -exact $parmInfo(scalarVars,names) $v]

       if {$i >= 0} \
          {
           set fmt [lindex $parmInfo(scalarVars,formats)  $i]
           set val [lindex $parmInfo(scalarVars,defaults) $i]       
          }

       if {[info exists parmInfo($v)]} {set val $parmInfo($v)} 

       set str " -$v [addQuotes [safeFormat $fmt $val]]"
       incr n [string length $str]

       puts -nonewline $outUnit $str
      }

   foreach v $listVars \
      {
       if {![strcasecmp $v None]} continue
       if {[lsearch $parmInfo(hideList) $v] >= 0} continue
       if {![info exists parmInfo($v)]} continue
       if {![llength $parmInfo($v)]} continue

       set key [string range $v 0 1]
       if {!$vjShowFlag && ![strcasecmp $key VJ]} continue

       puts -nonewline $outUnit " -$v $parmInfo($v)"
      }

   foreach v $dimVars \
      {
       if {![strcasecmp $v None]} continue
       if {[lsearch $parmInfo(hideList) $v] >= 0} continue

       set key [string range $v 0 1]
       if {!$vjShowFlag && ![strcasecmp $key VJ]} continue 

       set fmt  %s
       set dval None

       set i [lsearch -exact $parmInfo(dimVars,names) $v]

       if {$i >= 0} \
          {
           set fmt  [lindex $parmInfo(dimVars,formats)  $i]
           set dval [lindex $parmInfo(dimVars,defaults) $i]
          }

       for {set dim 1} {$dim <= $dimCount} {incr dim} \
          {
           if {$parmInfo(nusFlag)} \
              {
               if {$thisSkipFlag && $parmInfo(nusInitSize,$dim) == 1} continue 
              } \
           else \
              {
               if {$thisSkipFlag && $parmInfo(N,$dim) == 1} continue 
              }

           set val [safeFormat $fmt $dval]
           set a   [lindex $parmInfo(axisPrefix) [expr $dim - 1]]
 
           if {$parmInfo(nusFlag) && $parmInfo(ndim) == 2 && [strcasecmp $a x]} {set a y}
    
           if {[lsearch $parmInfo(hideList) $a$v] >= 0} continue
           if {[info exists parmInfo($v,$dim)]} {set val $parmInfo($v,$dim)} 

           puts -nonewline $outUnit [format " %s%s %s" -$a $v [safeFormat $fmt $val]]
          }
      }
}

proc getExpName {} \
{
   global procpar parmInfo

   set parmInfo(title) NMR
   set delStr "-_ "

   if {[info exists procpar(seqfil)]} \
      {
       set parmInfo(title) [strDel $procpar(seqfil) $delStr]
       set c [string index $parmInfo(title) 0]

       if {![strcasecmp $c "g"]} \
          {
           set parmInfo(title) [string range $parmInfo(title) 1 end]
          }
      }

   if {[info exists procpar(BPpipetype)]} \
      {
       set parmInfo(title) $procpar(BPpipetype)
      }

   if {![string length $parmInfo(title)]} {set parmInfo(title) NMR}

   getScalarArgs title
}

proc getListArgs args \
{
   global parmInfo dimCount ARGC ARGV

   set resetFlag 1

   if {![info exists parmInfo(listVars,usedNames)]} \
      {
       set parmInfo(listVars,usedNames) ""
      }

   foreach vv $args \
      {
       if {![strcasecmp $vv -reset]} \
          {
           set resetFlag 1
           continue
          }

       if {![strcasecmp $vv -noreset]} \
          {
           set resetFlag 0
           continue
          }

       foreach v $vv \
          {
           if {[lsearch -exact $parmInfo(listVars,usedNames)] >= 0} continue
           lappend parmInfo(listVars,usedNames) $v

           if {![info exists parmInfo($v,argc)]} {set parmInfo($v,argc) 0}
           if {$parmInfo($v,argc) && !$resetFlag} continue

           set loc [getListArgD $ARGV -$v parmInfo($v)]
           if {$loc} {incr parmInfo($v,argc)}
          }
      }
}

proc getScalarArgs args \
{
   global parmInfo dimCount ARGC ARGV

   if {![info exists parmInfo(scalarVars,usedNames)]} \
      {
       set parmInfo(scalarVars,usedNames) ""
      }

   set resetFlag 1
      
   foreach vv $args \
      { 
       if {![strcasecmp $vv -reset]} \
          {
           set resetFlag 1
           continue
          }

       if {![strcasecmp $vv -noreset]} \
          {
           set resetFlag 0
           continue
          }

       foreach v $vv \
          {
           if {[lsearch -exact $parmInfo(scalarVars,usedNames)] >= 0} continue
           lappend parmInfo(scalarVars,usedNames) $v
       
           if {![info exists parmInfo($v,argc)]} {set parmInfo($v,argc) 0}
           if {$parmInfo($v,argc) && !$resetFlag} continue

           set loc [getArgD $ARGV -$v parmInfo($v)]
           if {$loc} {incr parmInfo($v,argc)}
          }
      }
}

proc getNUSInfo {} \
{
   global parmInfo procpar dimCount ARGV

   getListArgs nusMethods 

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set parmInfo(nusInitSize,$dim) $parmInfo(N,$dim)
      }

   if {[info exists parmInfo(MODE,1)]} {set parmInfo(nusMODE,1) $parmInfo(MODE,1)}
   if {[info exists parmInfo(MODE,2)]} {set parmInfo(nusMODE,2) $parmInfo(MODE,2)}
   if {[info exists parmInfo(MODE,3)]} {set parmInfo(nusMODE,3) $parmInfo(MODE,3)}
   if {[info exists parmInfo(MODE,4)]} {set parmInfo(nusMODE,4) $parmInfo(MODE,4)}

   set parmInfo(nusXList) ""
   set parmInfo(nusYList) ""
   set parmInfo(nusZList) ""
   set parmInfo(nusAList) ""

   set parmInfo(nusFlag) 0
   set parmInfo(nusSampleCount) 0

   if {$dimCount == 1} {return 0}
   if {![strcasecmp $parmInfo(nusSched) None]} {return 0}

   if {![info exists procpar(SPARSE)]} \
      {
       if {[info exists procpar(sparse)]} \
          {
           set procpar(SPARSE) $procpar(sparse)
          } \
       else \
          {
           set procpar(SPARSE) n
          }
      }

   if {![strcasecmp $procpar(SPARSE) n] || ![strcasecmp $procpar(SPARSE) no]} \
      {
       set parmInfo(nusFlag) 0
      } \
   else \
      { 
       set parmInfo(nusFlag) 1
      }

   if {[info exists procpar(sampling)]} \
      {
       if {![strcasecmp $procpar(sampling) sparse] || ![strcasecmp $procpar(sampling) e]} \
          {
           set parmInfo(nusFlag) 1
          } \
       elseif {![strcasecmp $procpar(sampling) expanded] || ![strcasecmp $procpar(sampling) inflated]} \
          {
           set parmInfo(nusFlag) 2
          }
      }

   if {[info exists procpar(CSinflated)]} \
      {
       foreach key {1 y yes true} \
          {
           if {![strcasecmp $procpar(CSinflated) $key]} \
              {
               set parmInfo(nusFlag) 2
              }
          }
      }


   if {[flagLoc $ARGV -nonusadj]} {set parmInfo(nusFlag) 0}

   if {!$parmInfo(nusFlag)} {return 0}

   set error [getNUSSched]
   if {$error} {return 1}

   set parmInfo(nusSampleCount) [llength $parmInfo(nusYList)]
   if {$parmInfo(nusSampleCount) < 1} {return 0}

   if {$dimCount == 2} \
      {
       if {$parmInfo(nusFlag) == 2} \
          {
           set parmInfo(T,2) [expr 1 + int([getMax $parmInfo(nusYList)])]
           set parmInfo(N,2) [expr $parmInfo(quadAdjust,2)*$parmInfo(T,2)]
          } \
       else \
          {
           set parmInfo(N,2) [expr $parmInfo(quadAdjust,2)*$parmInfo(nusSampleCount)]
           set parmInfo(T,2) [expr 1 + int([getMax $parmInfo(nusYList)])]
          }
      } \
   elseif {$dimCount == 3} \
      {
       if {$parmInfo(N,2) == 1} \
          {
           set parmInfo(N,2)    1
           set parmInfo(T,2)    [expr 1 + int([getMax $parmInfo(nusZList)])]
           set parmInfo(MODE,2) Real

           if {$parmInfo(nusFlag) == 2} \
              {
               set parmInfo(T,3) [expr 1 + int([getMax $parmInfo(nusYList)])]
               set parmInfo(N,3) [expr $parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)*$parmInfo(T,3)]
              } \
           else \
              {
               set parmInfo(N,3) [expr $parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)*$parmInfo(nusSampleCount)]
               set parmInfo(T,3) [expr 1 + int([getMax $parmInfo(nusYList)])]
              }
          } \
       else \
          {
           set parmInfo(N,3)    1
           set parmInfo(T,3)    [expr 1 + int([getMax $parmInfo(nusZList)])]
           set parmInfo(MODE,3) Real

           if {$parmInfo(nusFlag) == 2} \
              {
               set parmInfo(T,2) [expr 1 + int([getMax $parmInfo(nusYList)])]
               set parmInfo(N,2) [expr $parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)*$parmInfo(T,2)*$parmInfo(T,3)]
               set parmInfo(MODE,2) Complex
              } \
           else \
              {
               set parmInfo(N,2) [expr $parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)*$parmInfo(nusSampleCount)]
               set parmInfo(T,2) [expr 1 + int([getMax $parmInfo(nusYList)])]
               set parmInfo(MODE,2) Complex
              }
          }
      } \
   elseif {$dimCount == 4} \
      {
       if {$parmInfo(N,3) == 1 && $parmInfo(N,4) == 1} \
          {
           set parmInfo(N,3)    1
           set parmInfo(T,3)    [expr 1 + int([getMax $parmInfo(nusZList)])]
           set parmInfo(MODE,3) Real

           set parmInfo(N,4)    1
           set parmInfo(T,4)    [expr 1 + int([getMax $parmInfo(nusAList)])]
           set parmInfo(MODE,4) Real

           if {$parmInfo(nusFlag) == 2} \
              {
               set parmInfo(T,2) [expr 1 + int([getMax $parmInfo(nusYList)])]
               set parmInfo(N,2) [expr $parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)*$parmInfo(quadAdjust,4)*$parmInfo(T,2)]
              } \
           else \
              {
               set parmInfo(N,2) [expr $parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)*$parmInfo(quadAdjust,4)*$parmInfo(nusSampleCount)]
               set parmInfo(T,2) [expr 1 + int([getMax $parmInfo(nusYList)])]
              }
          }
       elseif {$parmInfo(N,2) == 1 && $parmInfo(N,4) == 1} \
          {
           set parmInfo(N,2)    1
           set parmInfo(T,2)    [expr 1 + int([getMax $parmInfo(nusZList)])]
           set parmInfo(MODE,2) Real

           set parmInfo(N,4)    1
           set parmInfo(T,4)    [expr 1 + int([getMax $parmInfo(nusAList)])]
           set parmInfo(MODE,4) Real

           if {$parmInfo(nusFlag) == 2} \
              {
               set parmInfo(T,3) [expr 1 + int([getMax $parmInfo(nusYList)])]
               set parmInfo(N,3) [expr $parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)*$parmInfo(quadAdjust,4)*$parmInfo(T,3)]
              } \
           else \
              {
               set parmInfo(N,3) [expr $parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)*$parmInfo(quadAdjust,4)*$parmInfo(nusSampleCount)]
               set parmInfo(T,3) [expr 1 + int([getMax $parmInfo(nusYList)])]
              }
          } \
       elseif {$parmInfo(N,2) == 1 && $parmInfo(N,3) == 1} \
          {
           set parmInfo(N,2)    1
           set parmInfo(T,2)    [expr 1 + int([getMax $parmInfo(nusZList)])]
           set parmInfo(MODE,2) Real

           set parmInfo(N,3)    1
           set parmInfo(T,3)    [expr 1 + int([getMax $parmInfo(nusAList)])]
           set parmInfo(MODE,4) Real

           if {$parmInfo(nusFlag) == 2} \
              {
               set parmInfo(T,4) [expr 1 + int([getMax $parmInfo(nusYList)])]
               set parmInfo(N,4) [expr $parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)*$parmInfo(quadAdjust,4)*$parmInfo(T,4)]
              } \
           else \
              {
               set parmInfo(N,4) [expr $parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)*$parmInfo(quadAdjust,4)*$parmInfo(nusSampleCount)]
               set parmInfo(T,4) [expr 1 + int([getMax $parmInfo(nusYList)])]
              }
          } \
       else \
          {
           set parmInfo(N,3)    1
           set parmInfo(T,3)    [expr 1 + int([getMax $parmInfo(nusZList)])]
           set parmInfo(MODE,3) Real

           set parmInfo(N,4)    1
           set parmInfo(T,4)    [expr 1 + int([getMax $parmInfo(nusAList)])]
           set parmInfo(MODE,4) Real

           if {$parmInfo(nusFlag) == 2} \
              {
               set parmInfo(T,2)    [expr 1 + int([getMax $parmInfo(nusYList)])]
               set parmInfo(N,2)    [expr $parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)*$parmInfo(quadAdjust,4)*$parmInfo(T,2)*$parmInfo(T,3)*$parmInfo(T,4)]
               set parmInfo(MODE,3) Complex
              } \
           else \
              {
               set parmInfo(N,2)    [expr $parmInfo(quadAdjust,2)*$parmInfo(quadAdjust,3)*$parmInfo(quadAdjust,4)*$parmInfo(nusSampleCount)]
               set parmInfo(T,2)    [expr 1 + int([getMax $parmInfo(nusYList)])]
               set parmInfo(MODE,3) Complex
              }
          }
      }
}

proc getNUSSched {} \
{
   global procpar parmInfo dimCount partialFlag sampleCount elemCount verbose

   if {![strcasecmp $parmInfo(nusSched) Auto]} \
      {
       if {[file exists sampling.sch]} \
          {
           set parmInfo(nusSched) sampling.sch
          } \
       elseif {[file exists ../sampling.sch]} \
          {
           set parmInfo(nusSched) ../sampling.sch
          } \
       else \
          {
           set fList [glob -nocomplain *.hdr_3]
           if {![llength $fList]} {return 1} 

           set parmInfo(nusSched) [lindex $fList 0]
          }
      }

   if {![file exists $parmInfo(nusSched)]} \
      {
       showMsg "VJ2Pipe Error Finding NUS Schedule $parmInfo(nusSched)"
       return 1
      }

   set error [catch {set inUnit [open $parmInfo(nusSched) r]}]

   if {$error} \
      {
       showMsg "VJ2Pipe Error Opening NUS Schedule $parmInfo(nusSched)"
       return 1
      }

   if {$verbose} \
      {
       showMsg "VJ2Pipe Reading NUS Schedule: $parmInfo(nusSched)"
      }

   if {$parmInfo(ndim) == 2} \
      {
       while {[gets $inUnit line] >= 0} \
          {
           set key [string index [lindex $line 0] 0]
           if {![strcmp $key "#"]} continue

           foreach i $line \
              {
               if {[string is integer $i]} {lappend parmInfo(nusYList) $i}
              }
          }
      } \
   elseif {$parmInfo(ndim) == 3} \
      {
       while {[gets $inUnit line] >= 0} \
          {
           set key [string index [lindex $line 0] 0]
           if {![strcmp $key "#"]} continue

           if {[llength $line] < 2} continue

           set ny [lindex $line 0]               
           set nz [lindex $line 1]

           if {![string is integer $ny] || ![string is integer $nz]} continue

           lappend parmInfo(nusYList) $ny
           lappend parmInfo(nusZList) $nz
          }
      } \
   elseif {$parmInfo(ndim) == 4} \
      {
       while {[gets $inUnit line] >= 0} \
          {
           set key [string index [lindex $line 0] 0]
           if {![strcmp $key "#"]} continue

           if {[llength $line] < 3} continue

           set ny [lindex $line 0]
           set nz [lindex $line 1]
           set na [lindex $line 2]

           if {![string is integer $ny] || ![string is integer $nz] || ![string is integer $na]} continue

           lappend parmInfo(nusYList) $ny
           lappend parmInfo(nusZList) $nz
           lappend parmInfo(nusAList) $na
          }
      }

   catch "close $inUnit" 

   set nS [llength $parmInfo(nusYList)]
   set nE $parmInfo(celem)

   set sampleCount $nS
   set elemCount   $nE

   if {$parmInfo(ndim) >= 2} {set nE [expr $nE/$parmInfo(quadAdjust,2)]}
   if {$parmInfo(ndim) >= 3} {set nE [expr $nE/$parmInfo(quadAdjust,3)]}
   if {$parmInfo(ndim) >= 4} {set nE [expr $nE/$parmInfo(quadAdjust,4)]}

   if {$nE < 1} {set nE $nS}

   if {$nE < $nS} \
      {
       if {$partialFlag} \
          {
           showMsg "VJ2Pipe Updating Sizes. Expected Element Count: $nS Completed: $nE"
           set parmInfo(nusYList) [lrange $parmInfo(nusYList) 0 [expr $nE - 1]]
           set parmInfo(nusZList) [lrange $parmInfo(nusZList) 0 [expr $nE - 1]]
           set parmInfo(nusAList) [lrange $parmInfo(nusAList) 0 [expr $nE - 1]]

           set sampleCount $nS
           set elemCount   $nE
          } \
       else \
          {
           showMsg "VJ2Pipe Notice: Parameters indicate a partially-completed NUS experiment."
           showMsg "VJ2Pipe Notice: Expected Element Count: $nS Completed: $nE"
           showMsg "VJ2Pipe Notice: Use the -partial option to adjust accordingly."
          }
      } \
   elseif {$nE > $nS} \
      {
       showMsg "VJ2Pipe Warning: Completed Element Count is Larger than NUS Sample Count."
       showMsg "VJ2Pipe Warning: NUS Sample Count: $nS Expected Completed Count: $nE"
      }

   return 0
}

proc getAxisArgs args \
{
   global parmInfo dimCount ARGC ARGV

   if {![info exists parmInfo(dimVars,usedNames)]} \
      {
       set parmInfo(dimVars,usedNames) ""
      }

   set resetFlag 1

   foreach vv $args \
      {
       if {![strcasecmp $vv -reset]} \
          {
           set resetFlag 1
           continue
          }

       if {![strcasecmp $vv -noreset]} \
          {
           set resetFlag 0
           continue
          }

       foreach v $vv \
          {
           if {[lsearch -exact $parmInfo(dimVars,usedNames)] >= 0} continue
           lappend parmInfo(dimVars,usedNames) $v

           if {![info exists parmInfo($v,all,argc)]} \
              {
               set parmInfo($v,all,argc) 0
              }

           for {set dim 1} {$dim <= $dimCount} {incr dim} \
              {
               set a [lindex $parmInfo(axisList) [expr $dim - 1]]
           
               if {![info exists parmInfo($v,$dim,argc)]} \
                  {
                   set parmInfo($v,$dim,argc) 0
                  }

               if {$parmInfo($v,$dim,argc) && !$resetFlag} continue

               set loc [getArgD $ARGV -$a$v parmInfo($v,$dim)]

               if {$loc} \
                  {
                   incr parmInfo($v,$dim,argc)
                   incr parmInfo($v,all,argc)
                  }
              }
          }
      }
}

proc setParmDefaults {} \
{
   global parmInfo dimCount maxDim

   set parmInfo(dimCount) 0
   set parmInfo(ndimOrig) 0
   set parmInfo(ndim)     0
   set parmInfo(arrayDim) 1

   set i 0

   foreach v $parmInfo(listVars,names) \
      {
       if {![info exists parmInfo($v)]} {set parmInfo($v) ""}
       set parmInfo($v,argc) 0
       incr i
      }

   set i 0

   foreach v $parmInfo(scalarVars,names) \
      {
       set parmInfo($v) [lindex $parmInfo(scalarVars,defaults) $i]
       set parmInfo($v,argc) 0
       incr i
      }

   set i 0

   foreach v $parmInfo(dimVars,names) \
      {
       set def [lindex $parmInfo(dimVars,defaults) $i]
       set parmInfo($v,all,argc) 0

       for {set dim 1} {$dim <= $maxDim} {incr dim} \
          {
           set parmInfo($v,$dim)      $def
           set parmInfo($v,$dim,argc) 0
          }

       incr i
      }

   set parmInfo(CAR,1) H2O
}

proc fixNullParms {} \
{
   global parmInfo dimCount verbose
 
   set i 0

   foreach v $parmInfo(listVars,names) \
      {
       if {![info exists parmInfo($v)]} {set parmInfo($v) ""}
      }

   foreach v $parmInfo(scalarVars,names) \
      {
       set def [lindex $parmInfo(scalarVars,defaults) $i]
       set n   0

       if {![info exists parmInfo($v)]} \
          {
           set parmInfo($v) $def
           incr n 1
          }

       if {![llength $parmInfo($v)]} \
          {
           set parmInfo($v) $def
           incr n 2
          }

       if {$verbose && $n} \
          {
           showMsg "Reset Variable [format %12s $v] |$parmInfo($v)|"
          }

       incr i
      }

   set i 0

   foreach v $parmInfo(dimVars,names) \
      {
       set def [lindex $parmInfo(dimVars,defaults) $i]

       for {set dim 1} {$dim <= $dimCount} {incr dim} \
          {
           set n 0

           if {![info exists parmInfo($v,$dim)]} \
              {
               set parmInfo($v,$dim) $def
               incr n 1
              }

           if {![llength $parmInfo($v,$dim)]} \
              {
               set parmInfo($v,$dim) $def
               incr n 2
              }

           if {$verbose && $n} \
              {
               showMsg "Reset Variable [format %12s $v,$dim] |$parmInfo($v,$dim)|"
              }
          }

       incr i
      }
}

proc getVJName { vList dim } \
{
   if {$dim < 1} {set dim 1}

   if {$dim == 1} {return [lindex $vList 0]}

   set var [lindex $vList 1]
   set v   [string range $var 0 end-1] 
   set i   [expr $dim - 1]

   return $v$i
}

proc isON { val } \
{
   set c [string index $val 0]

   if {![strcasecmp $c "Y"]} {return 1}
   if {![strcasecmp $c "T"]} {return 1}
   if {![strcasecmp $c "1"]} {return 1}

   return 0
}

proc strDel { str delStr } \
{
   set n [string length $str]
   set s ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $str $i]
       if {[string first $c $delStr] >= 0} continue
       append s $c
      }

   return $s
}

proc initMsgFile {} \
{
   global msgName msgUnit 

   set msgUnit stderr

   if {![strcasecmp $msgName None]} \
      {
       set msgUnit None 
       return $msgUnit
      }

   if {![strcasecmp $msgName stderr]} \
      {
       set msgUnit stderr
       return $msgUnit
      }

   if {![strcasecmp $msgName stdout]} \
      {
       set msgUnit stdout
       return $msgUnit
      }

   set error [catch {set msgUnit [open $msgName w]}]

   if {$error} \
      {
       set msgUnit stderr
       puts $msgUnit "VJ2Pipe Error Opening Output Message File $msgName"
       return $msgUnit
      }
}

#
# Return H2O position based on degrees K.

proc getVJH2Oppm { t } \
{
   global parmInfo

   getScalarArgs h2oM h2oB

   set m $parmInfo(h2oM)
   set b $parmInfo(h2oB)

   set h2oPPM  [expr $m*($t - 273.0) + $b]

   return $h2oPPM
}

#
# Find the location of fid and procpar files.

proc getVJInput {} \
{
   global parmInfo

   set thisName $parmInfo(procpar)
   set thisDir  $parmInfo(vjPath)

   if {![strcmp $thisDir "."] || ![strcmp $thisDir "./"]} \
      {
       set thisDir ""
       set upDir   "../"
      } \
   else \
      {
       set thisDir [file normalize $thisDir]/
       set upDir   [file normalize ${thisDir}../]/

       showMsg "VJ2Pipe Using Input Directory:"
       showMsg " $thisDir"
      }

   if {![strcasecmp $thisName Auto]} \
      {    
       showMsg "VJ2Pipe procpar Auto Mode:"

       if {[file exists ${thisDir}procpar]} \
          {   
           set thisName ${thisDir}procpar
          } \
       elseif {[file exists ${thisDir}acqfil/procpar]} \
          {
           set thisName ${thisDir}acqfil/procpar
          } \
       elseif {[file exists ${upDir}procpar]} \
          {
           set thisName ${upDir}procpar
          } \
       else \
          {
           showMsg "VJ2Pipe Error finding procpar file."
           return 1
          }

       set parmInfo(procpar) $thisName
       showMsg " Found $thisName"
      }

   set thisName $parmInfo(vjfid)

   if {![strcasecmp $thisName Auto]} \
      {
       showMsg "VJ2Pipe fid Auto Mode:"

       if {[file exists ${thisDir}fid] && ![file isdirectory ${thisDir}fid]} \
          {
           set thisName ${thisDir}fid
          } \
       elseif {[file exists ${thisDir}{acqfil}/fid] && ![file isdirectory ${thisDir}acqfil/fid]} \
          {
           set thisName ${thisDir}acqfil/fid
          } \
       elseif {[file exists ${upDir}fid] && ![file isdirectory ${upDir}fid]} \
          {
           set thisName ${upDir}fid
          } \
       else \
          {
           showMsg "VJ2Pipe Warning: Error finding fid file."
           return -1
          }

       set parmInfo(vjfid) $thisName
       showMsg " Found $thisName"
      }

    return 0
}

#
# Create the conversion script.

proc createConvScript {} \
{
   global execFlag execList parmInfo sampleCount elemCount partialFlag verbose

   set outUnit [openScript $parmInfo(fidScript)]
   if {![strcasecmp $outUnit None]} {return 1}

   if {$verbose} \
      {
       set vArg "verb"
      } \
   else \
      {
       set vArg "noverb"
      }

   puts $outUnit "#!/bin/csh"
   puts $outUnit ""

   puts $outUnit "if (!(-e $parmInfo(vjfid))) then"
   puts $outUnit "   exit 0"
   puts $outUnit "endif"
   puts $outUnit ""

   set sArg Auto

   if {$partialFlag} \
      {
       if {$elemCount < $sampleCount} {set sArg $elemCount}
      }

   if {$parmInfo(nusFlag) == 2} \
      {
       set expandArg "-defSign -1 -noexpand"
       set expandArg "-noexpand"
      } \
   else \
      {
       set expandArg ""
      }

   switch $parmInfo(ndim) \
      {
       1 {
          if {![strcasecmp [file extension $parmInfo(vjfid)] .Z]} \
             {
              puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -$vArg \\"
             } \
          elseif {![strcasecmp [file extension $parmInfo(vjfid)] .gz]} \
             {
              puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -$vArg \\"
             } \
          else \
             {
              puts $outUnit "var2pipe -in $parmInfo(vjfid) -noaswap -$vArg \\"
             }

          showParms -listVars None   \
           -outUnit $outUnit -skip 1 \
           -scalarVars "ndim title"  \
           -axisVars   "N T MODE SW OBS CAR LAB"

          puts $outUnit " -out $parmInfo(outPrefix).fid -ov"
         }

       2 {
          if {$parmInfo(nusFlag)} \
             {
              if {![strcasecmp [file extension $parmInfo(vjfid)] .Z]} \
                 {
                  puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -ndim 2 -nusDim 2 -$vArg \\"
                 } \
              elseif {![strcasecmp [file extension $parmInfo(vjfid)] .gz]} \
                 {
                  puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -ndim 2 -nusDim 2 -$vArg \\"
                 } \
              else \
                 {
                  puts $outUnit "var2pipe -in $parmInfo(vjfid) -noaswap -ndim 2 -nusDim 2 -$vArg \\"
                 }

              showParms -listVars None   \
               -outUnit $outUnit -skip 1 \
               -scalarVars "title aq2D aqORD" \
               -axisVars   "N T MODE SW OBS CAR LAB"

              puts $outUnit " -out $parmInfo(outPrefix)_nus.fid -ov"
              puts $outUnit ""

              puts $outUnit "nusExpand.tcl $expandArg -in $parmInfo(outPrefix)_nus.fid -out $parmInfo(outPrefix).fid      -sample $parmInfo(nusSched) -sampleCount $sArg"
              puts $outUnit "nusExpand.tcl $expandArg -in $parmInfo(outPrefix)_nus.fid -out $parmInfo(outPrefix)_mask.fid -sample $parmInfo(nusSched) -sampleCount $sArg -mask"
             } \
          else \
             {
              if {![strcasecmp [file extension $parmInfo(vjfid)] .Z]} \
                 {
                  puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -$vArg \\"
                 } \
              elseif {![strcasecmp [file extension $parmInfo(vjfid)] .gz]} \
                 {
                  puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -$vArg \\"
                 } \
              else \
                 {
                  puts $outUnit "var2pipe -in $parmInfo(vjfid) -noaswap -$vArg \\"
                 }

              showParms -listVars None   \
               -outUnit $outUnit -skip 1 \
               -scalarVars "ndim title aq2D aqORD"   \
               -axisVars   "N T MODE SW OBS CAR LAB"

              puts $outUnit " -out $parmInfo(outPrefix).fid -ov"
              puts $outUnit ""
             }
         }

       3 {
          if {$parmInfo(nusFlag)} \
             {
             if {![strcasecmp [file extension $parmInfo(vjfid)] .Z]} \
                 {
                  puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -ndim 2 -nusDim 3 -aqORD 0 -$vArg \\"
                 } \
              elseif {![strcasecmp [file extension $parmInfo(vjfid)] .gz]} \
                 {
                  puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -ndim 2 -nusDim 3 -aqORD 0 -$vArg \\"
                 } \
              else \
                 {
                  puts $outUnit "var2pipe -in $parmInfo(vjfid) -noaswap -ndim 2 -nusDim 3 -aqORD 0 -$vArg \\"
                 }

              showParms -listVars None   \
               -outUnit $outUnit -skip 0 \
               -scalarVars "title aq2D"  \
               -axisVars   "N T MODE SW OBS CAR LAB"
  
              if {![strcasecmp $parmInfo(nusMODE,2) Rance-Kay] || ![strcasecmp $parmInfo(nusMODE,2) Echo-AntiEcho]} \
                 {
                  if {$parmInfo(aqORD) == 0} \
                     {
                      puts $outUnit "| nmrPipe -fn MAC -macro \$NMRTXT/var_ranceN.M -noRd -noWr -var nShuf 1 nusDim $parmInfo(ndim) \\"
                     } \
                  else \
                     {
                      puts $outUnit "| nmrPipe -fn MAC -macro \$NMRTXT/var_ranceN.M -noRd -noWr -var nShuf 2 nusDim $parmInfo(ndim) \\"
                     }
                 }

             if {![strcasecmp $parmInfo(nusMODE,3) Rance-Kay] || ![strcasecmp $parmInfo(nusMODE,3) Echo-AntiEcho]} \
                 {
                  if {$parmInfo(aqORD) == 1} \
                     {
                      puts $outUnit "| nmrPipe -fn MAC -macro \$NMRTXT/var_ranceN.M -noRd -noWr -var nShuf 1 nusDim $parmInfo(ndim) \\"
                     } \
                  else \
                     {
                      puts $outUnit "| nmrPipe -fn MAC -macro \$NMRTXT/var_ranceN.M -noRd -noWr -var nShuf 2 nusDim $parmInfo(ndim) \\"
                     }
                 }
 
              puts $outUnit " -out $parmInfo(outPrefix)_nus.fid -ov"
              puts $outUnit ""

              puts $outUnit "nusExpand.tcl $expandArg -in $parmInfo(outPrefix)_nus.fid -out data/$parmInfo(outPrefix)%03d.fid    -sample $parmInfo(nusSched) -sampleCount $sArg -aqORD $parmInfo(aqORD) -sign"
              puts $outUnit "nusExpand.tcl $expandArg -in $parmInfo(outPrefix)_nus.fid -out nusmask/$parmInfo(outPrefix)%03d.fid -sample $parmInfo(nusSched) -sampleCount $sArg -aqORD $parmInfo(aqORD) -mask"
             } \
          else \
             {
              if {![strcasecmp [file extension $parmInfo(vjfid)] .Z]} \
                 {
                  puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -$vArg \\"
                 } \
              elseif {![strcasecmp [file extension $parmInfo(vjfid)] .gz]} \
                 {
                  puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -$vArg \\"
                 } \
              else \
                 {
                  puts $outUnit "var2pipe -in $parmInfo(vjfid) -noaswap -$vArg \\"
                 }

              showParms -listVars None   \
               -outUnit $outUnit -skip 1 \
               -scalarVars "ndim title aq2D aqORD"   \
               -axisVars   "N T MODE SW OBS CAR LAB"
    
              puts $outUnit " -out data/$parmInfo(outPrefix)%03d.fid -ov"
             }
         }

       4 {
          if {$parmInfo(nusFlag)} \
             {
             if {![strcasecmp [file extension $parmInfo(vjfid)] .Z]} \
                 {
                  puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -ndim 2 -nusDim 4 -aqORD 0 -$vArg \\"
                 } \
              elseif {![strcasecmp [file extension $parmInfo(vjfid)] .gz]} \
                 {
                  puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -ndim 2 -nusDim 4 -aqORD 0 -$vArg \\"
                 } \
              else \
                 {
                  puts $outUnit "var2pipe -in $parmInfo(vjfid) -noaswap -ndim 2 -nusDim 4 -aqORD 0 -$vArg \\"
                 }

              showParms -listVars None   \
               -outUnit $outUnit -skip 0 \
               -scalarVars "title aq2D"   \
               -axisVars   "N T MODE SW OBS CAR LAB"

              if {![strcasecmp $parmInfo(nusMODE,2) Rance-Kay] || ![strcasecmp $parmInfo(nusMODE,2) Echo-AntiEcho]} \
                 {
                  if {$parmInfo(aqORD) == 0} \
                     {
                      puts $outUnit "| nmrPipe -fn MAC -macro \$NMRTXT/var_ranceN.M -noRd -noWr -var nShuf 1 nusDim $parmInfo(ndim) \\"
                     } \
                  else \
                     {
                      puts $outUnit "| nmrPipe -fn MAC -macro \$NMRTXT/var_ranceN.M -noRd -noWr -var nShuf 4 nusDim $parmInfo(ndim) \\"
                     }
                 }

              if {![strcasecmp $parmInfo(nusMODE,3) Rance-Kay] || ![strcasecmp $parmInfo(nusMODE,3) Echo-AntiEcho]} \
                 {
                  if {$parmInfo(aqORD) == 0} \
                     {
                      puts $outUnit "| nmrPipe -fn MAC -macro \$NMRTXT/var_ranceN.M -noRd -noWr -var nShuf 2 nusDim $parmInfo(ndim) \\"
                     } \
                  else \
                     {
                      puts $outUnit "| nmrPipe -fn MAC -macro \$NMRTXT/var_ranceN.M -noRd -noWr -var nShuf 2 nusDim $parmInfo(ndim) \\"
                     }
                 }

              if {![strcasecmp $parmInfo(nusMODE,4) Rance-Kay] || ![strcasecmp $parmInfo(nusMODE,4) Echo-AntiEcho]} \
                 {
                  if {$parmInfo(aqORD) == 0} \
                     {
                      puts $outUnit "| nmrPipe -fn MAC -macro \$NMRTXT/var_ranceN.M -noRd -noWr -var nShuf 4 nusDim $parmInfo(ndim) \\"
                     } \
                  else \
                     {
                      puts $outUnit "| nmrPipe -fn MAC -macro \$NMRTXT/var_ranceN.M -noRd -noWr -var nShuf 1 nusDim $parmInfo(ndim) \\"
                     }
                 }

              puts $outUnit " -out $parmInfo(outPrefix)_nus.fid -ov"
              puts $outUnit ""

              puts $outUnit "nusExpand.tcl $expandArg -in $parmInfo(outPrefix)_nus.fid -out data/$parmInfo(outPrefix)%03d_%03d.fid -sample $parmInfo(nusSched) -sampleCount $sArg -aqORD $parmInfo(aqORD) -sign"
             } \
          else \
             {
              if {![strcasecmp [file extension $parmInfo(vjfid)] .Z]} \
                 {
                  puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -$vArg \\"
                 } \
              elseif {![strcasecmp [file extension $parmInfo(vjfid)] .gz]} \
                 {
                  puts $outUnit "cat $parmInfo(vjfid) | zcat | var2pipe -noaswap -$vArg \\"
                 } \
              else \
                 {
                  puts $outUnit "var2pipe -in $parmInfo(vjfid) -noaswap -$vArg \\"
                 }

              showParms -listVars None   \
               -outUnit $outUnit -skip 1 \
               -scalarVars "ndim title aq2D aqORD"   \
               -axisVars   "N T MODE SW OBS CAR LAB"
       
              puts $outUnit "| pipe2xyz -out data/$parmInfo(outPrefix)%03d_%03d.fid -ov"
             }
         } 
      }

   catch "close $outUnit"

   addExecCommand $parmInfo(fidScript).com

   if {$verbose} {showMsg "VJ2Pipe Creating Script: $parmInfo(fidScript).com"}

   return 0
}

#
# Add a script or command to be executed.

proc addExecCommand { cmnd } \
{
   global execList

   if {[llength $execList]} \
      {
       append execList "; $cmnd"
      } \
   else \
      {
       append execList " $cmnd"
      }
}

#
# Create the processing scripts.

proc createProcScripts {} \
{
   global graphFlag execFlag execList parmInfo vjExpertFlag verbose

   if {$verbose} \
      {
       set vArg "verb"
      } \
   else \
      {
       set vArg "noverb"
      }

   if {$graphFlag} \
      {
       set gArg graph
      } \
   else \
      {
       set gArg nograph
      }

   set methodList None

   if {$parmInfo(nusFlag)} \
      {
       foreach i $parmInfo(nusMethods) \
          {
           set ii [string tolower $i]
           if {[lsearch -exact $methodList $ii] < 0} {lappend methodList $ii}
          }
      }

   switch $parmInfo(ndimOrig) \
      {
       1 {
          if {![strcasecmp $parmInfo(ft1Script) None]} {return 0}

          set inScript $parmInfo(vjAuxDir)/com/$parmInfo(ft1Script).com

          if {![file exists $inScript]} \
             {
              showMsg "VJ2Pipe Error Finding Template Script $inScript"
              return 1
             }
 
          set outUnit [openScript $parmInfo(ft1Script)_user]
          if {![strcasecmp $outUnit None]} {return 1}

          set parmInfo(inName)  $parmInfo(outPrefix).fid
          set parmInfo(outName) $parmInfo(outPrefix).ft1

          set error [subsScript $outUnit $inScript]
          catch "close $outUnit"

          if {$vjExpertFlag} \
             {
              set inScript $parmInfo(vjAuxDir)/expert/$parmInfo(ft1Script).com
   
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Expert Template Script $inScript"
                  return 1
                 }
 
              set outUnit [openScript $parmInfo(tmpScript)_expert]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  $parmInfo(outPrefix).fid
              set parmInfo(outName) $parmInfo(outPrefix).ft1

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"

              catch "file delete $parmInfo(ft1Script)_expert.com"
              catch "exec $parmInfo(tmpScript)_expert.com > $parmInfo(ft1Script)_expert.com"

              if {![file exists $parmInfo(ft1Script)_expert.com]} \
                 {
                  showMsg "VJ2Pipe Error Generating Expert Script $parmInfo(ft1Script)_expert.com"
                  return 1
                 }

              catch "exec chmod a+rx $parmInfo(ft1Script)_expert.com"
              catch "file delete $parmInfo(tmpScript)_expert.com" 
             }

          addExecCommand "$parmInfo(ft1Script)_user.com -$gArg"
          subsUserScripts
         }

       2 {
          if {![strcasecmp $parmInfo(ft2Script) None]} {return 0}

          foreach m $methodList \
             {
              set specPrefix   ""
              set scriptPrefix ""

              if {[strcasecmp $m None]} \
                 {
                  set scriptPrefix _$m 
                  set specPrefix   _$m 
                 }

              if {$parmInfo(mcFlag)} \
                 {
                  if {![strcasecmp $parmInfo(aq2D) Magnitude]} \
                     {
                      set scriptPrefix mc$scriptPrefix
                     } \
                  else \
                     {
                      set scriptPrefix av$scriptPrefix 
                     }
                 }

              set inScript $parmInfo(vjAuxDir)/com/$parmInfo(ft2Script)$scriptPrefix.com

              if {![file exists $inScript]} \
                 {
                  if {![strcasecmp $methodList None]} \
                     {
                      showMsg "VJ2Pipe Error Finding Template Script $inScript"
                      return 1
                     }

                  continue
                 }

              if {$verbose} {showMsg "VJ2Pipe Creating Script: $parmInfo(ft2Script)${scriptPrefix}_user.com"}

              set outUnit [openScript $parmInfo(ft2Script)${scriptPrefix}_user]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  $parmInfo(outPrefix).fid
              set parmInfo(outName) $parmInfo(outPrefix)$specPrefix.ft2

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"

              set inScript $parmInfo(vjAuxDir)/expert/$parmInfo(ft2Script)$scriptPrefix.com

              if {$vjExpertFlag && [file exists $inScript]} \
                 {
                  set outUnit [openScript $parmInfo(tmpScript)${scriptPrefix}_expert]
                  if {![strcasecmp $outUnit None]} {return 1}

                  set parmInfo(inName)  $parmInfo(outPrefix).fid
                  set parmInfo(outName) $parmInfo(outPrefix)$specPrefix.ft2

                  set error [subsScript $outUnit $inScript]
                  catch "close $outUnit"

                  catch "file delete $parmInfo(ft2Script)${scriptPrefix}_expert.com"
                  catch "exec $parmInfo(tmpScript)${scriptPrefix}_expert.com > $parmInfo(ft2Script)${scriptPrefix}_expert.com"

                  if {![file exists $parmInfo(ft2Script)${scriptPrefix}_expert.com]} \
                     {
                      showMsg "VJ2Pipe Error Generating Expert Script $parmInfo(ft2Script)${scriptPrefix}_expert.com"
                      return 1
                     }

                  catch "exec chmod a+rx $parmInfo(ft2Script)${scriptPrefix}_expert.com"
                  catch "file delete $parmInfo(tmpScript)${scriptPrefix}_expert.com" 
                 }

              addExecCommand "$parmInfo(ft2Script)${scriptPrefix}_user.com -$gArg"
             }

          subsUserScripts
         }

       3 {
          if {[strcasecmp $parmInfo(xyScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/com/$parmInfo(xyScript).com

              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Template Script $inScript"
                  return 1
                 }

              set outUnit [openScript $parmInfo(xyScript)_user]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d.fid
              set parmInfo(outName) $parmInfo(outPrefix)_xy.ft2

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"
             }

          if {$vjExpertFlag && [strcasecmp $parmInfo(xyScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/expert/$parmInfo(xyScript).com
   
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Expert Template Script $inScript"
                  return 1
                 }
 
              set outUnit [openScript $parmInfo(tmpScript)_expert]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d.fid
              set parmInfo(outName) $parmInfo(outPrefix)_xy.ft2

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"

              catch "file delete $parmInfo(xyScript)_expert.com"
              catch "exec $parmInfo(tmpScript)_expert.com > $parmInfo(xyScript)_expert.com"

              if {![file exists $parmInfo(xyScript)_expert.com]} \
                 {
                  showMsg "VJ2Pipe Error Generating Expert Script $parmInfo(xyScript)_expert.com"
                  return 1
                 }

              catch "exec chmod a+rx $parmInfo(xyScript)_expert.com"
              catch "file delete $parmInfo(tmpScript)_expert.com" 
             }
          
          if {[strcasecmp $parmInfo(xzScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/com/$parmInfo(xzScript).com

              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Template Script $inScript"
                  return 1
                 }

              set outUnit [openScript $parmInfo(xzScript)_user]
              if {![strcasecmp $outUnit None]} {return 1}
   
              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d.fid
              set parmInfo(outName) $parmInfo(outPrefix)_xz.ft2
              set parmInfo(auxName) $parmInfo(outPrefix)_xz.fid

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"
             }

          if {$vjExpertFlag && [strcasecmp $parmInfo(xzScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/expert/$parmInfo(xzScript).com
   
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Expert Template Script $inScript"
                  return 1
                 }
 
              set outUnit [openScript $parmInfo(tmpScript)_expert]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d.fid
              set parmInfo(outName) $parmInfo(outPrefix)_xz.ft2
              set parmInfo(auxName) $parmInfo(outPrefix)_xz.fid

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"

              catch "file delete $parmInfo(xzScript)_expert.com"
              catch "exec $parmInfo(tmpScript)_expert.com > $parmInfo(xzScript)_expert.com"

              if {![file exists $parmInfo(xzScript)_expert.com]} \
                 {
                  showMsg "VJ2Pipe Error Generating Expert Script $parmInfo(xzScript)_expert.com"
                  return 1
                 }

              catch "exec chmod a+rx $parmInfo(xzScript)_expert.com"
              catch "file delete $parmInfo(tmpScript)_expert.com" 
             }
          
          if {[strcasecmp $parmInfo(xyzScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/com/$parmInfo(xyzScript).com
   
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Template Script $inScript"
                  return 1
                 }

              set outUnit [openScript $parmInfo(xyzScript)_user]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d.fid
              set parmInfo(outName) ft/$parmInfo(outPrefix)%03d.ft3
              set parmInfo(auxName) ft/$parmInfo(outPrefix)%03d.ft2
 
              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"
             }

          if {$vjExpertFlag && [strcasecmp $parmInfo(xyzScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/expert/$parmInfo(xyzScript).com
   
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Expert Template Script $inScript"
                  return 1
                 }
 
              set outUnit [openScript $parmInfo(tmpScript)_expert]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d.fid
              set parmInfo(outName) ft/$parmInfo(outPrefix)%03d.ft3
              set parmInfo(auxName) ft/$parmInfo(outPrefix)%03d.ft2

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"

              catch "file delete $parmInfo(xyzScript)_expert.com"
              catch "exec $parmInfo(tmpScript)_expert.com > $parmInfo(xyzScript)_expert.com"

              if {![file exists $parmInfo(xyzScript)_expert.com]} \
                 {
                  showMsg "VJ2Pipe Error Generating Expert Script $parmInfo(xyzScript)_expert.com"
                  return 1
                 }

              catch "exec chmod a+rx $parmInfo(xyzScript)_expert.com"
              catch "file delete $parmInfo(tmpScript)_expert.com" 
             }
         
          if {[strcasecmp $parmInfo(scrollScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/com/$parmInfo(scrollScript).com
       
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Template Script $inScript"
                  return 1
                 }

              set outUnit [openScript $parmInfo(scrollScript)_user]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  ft/$parmInfo(outPrefix)%03d.ft3

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"
             }

          if {$vjExpertFlag && [strcasecmp $parmInfo(scrollScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/expert/$parmInfo(scrollScript).com
   
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Expert Template Script $inScript"
                  return 1
                 }
 
              set outUnit [openScript $parmInfo(tmpScript)_expert]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  ft/$parmInfo(outPrefix)%03d.ft3

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"

              catch "file delete $parmInfo(scrollScript)_expert.com"
              catch "exec $parmInfo(tmpScript)_expert.com > $parmInfo(scrollScript)_expert.com"

              if {![file exists $parmInfo(scrollScript)_expert.com]} \
                 {
                  showMsg "VJ2Pipe Error Generating Expert Script $parmInfo(scrollScript)_expert.com"
                  return 1
                 }

              catch "exec chmod a+rx $parmInfo(scrollScript)_expert.com"
              catch "file delete $parmInfo(tmpScript)_expert.com" 
             }
         
          addExecCommand "$parmInfo(xyScript)_user.com  -$gArg"
          addExecCommand "$parmInfo(xzScript)_user.com  -$gArg"
          addExecCommand "$parmInfo(xyzScript)_user.com -$gArg"

          if {$graphFlag} \
             {
              addExecCommand "sleep 10"
              addExecCommand $parmInfo(scrollScript)_user.com
             }

          foreach m $methodList \
             {
              if {![strcasecmp $m None]} \
                 {
                  set specPrefix   ""
                  set scriptPrefix ""
                  continue
                 } \
              else \
                 {
                  set scriptPrefix _$m
                  set specPrefix   _$m
                 }

              set inScript $parmInfo(vjAuxDir)/com/$parmInfo(xyzScript)$scriptPrefix.com

              if {![file exists $inScript]} continue

              set outUnit [openScript $parmInfo(xyzScript)${scriptPrefix}_user]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d.fid
              set parmInfo(outName) $m/$parmInfo(outPrefix)%03d.ft3
              set parmInfo(auxName) $m/$parmInfo(outPrefix)%03d.ft2

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"

              addExecCommand "$parmInfo(xyzScript)${scriptPrefix}_user.com -$gArg"

              if {$graphFlag} \
                 {
                  addExecCommand "$parmInfo(scrollScript)_user.com -in $parmInfo(outName) -hi 10% -$gArg"
                 }
             }

          subsUserScripts
         }

       4 {
          if {[strcasecmp $parmInfo(xyScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/com/$parmInfo(xyScript).com

              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Template Script $inScript"
                  return 1
                 }

              set outUnit [openScript $parmInfo(xyScript)_user]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d_%03d.fid
              set parmInfo(outName) $parmInfo(outPrefix)_xy.ft2

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"
             }

          if {$vjExpertFlag && [strcasecmp $parmInfo(xyScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/expert/$parmInfo(xyScript).com
   
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Expert Template Script $inScript"
                  return 1
                 }
 
              set outUnit [openScript $parmInfo(tmpScript)_expert]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d_%03d.fid
              set parmInfo(outName) $parmInfo(outPrefix)_xy.ft2

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"

              catch "file delete $parmInfo(xyScript)_expert.com"
              catch "exec $parmInfo(tmpScript)_expert.com > $parmInfo(xyScript)_expert.com"

              if {![file exists $parmInfo(xyScript)_expert.com]} \
                 {
                  showMsg "VJ2Pipe Error Generating Expert Script $parmInfo(xyScript)_expert.com"
                  return 1
                 }

              catch "exec chmod a+rx $parmInfo(xyScript)_expert.com"
              catch "file delete $parmInfo(tmpScript)_expert.com" 
             }
          
          if {[strcasecmp $parmInfo(xzScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/com/$parmInfo(xzScript).com

              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Template Script $inScript"
                  return 1
                 }

              set outUnit [openScript $parmInfo(xzScript)_user]
              if {![strcasecmp $outUnit None]} {return 1}
   
              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d_%03d.fid
              set parmInfo(outName) $parmInfo(outPrefix)_xz.ft2
              set parmInfo(auxName) $parmInfo(outPrefix)_xz.fid

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"
             }

          if {$vjExpertFlag && [strcasecmp $parmInfo(xzScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/expert/$parmInfo(xzScript).com
   
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Expert Template Script $inScript"
                  return 1
                 }
 
              set outUnit [openScript $parmInfo(tmpScript)_expert]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d_%03d.fid
              set parmInfo(outName) $parmInfo(outPrefix)_xz.ft2
              set parmInfo(auxName) $parmInfo(outPrefix)_xz.fid

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"

              catch "file delete $parmInfo(xzScript)_expert.com"
              catch "exec $parmInfo(tmpScript)_expert.com > $parmInfo(xzScript)_expert.com"

              if {![file exists $parmInfo(xzScript)_expert.com]} \
                 {
                  showMsg "VJ2Pipe Error Generating Expert Script $parmInfo(xzScript)_expert.com"
                  return 1
                 }

              catch "exec chmod a+rx $parmInfo(xzScript)_expert.com"
              catch "file delete $parmInfo(tmpScript)_expert.com" 
             }
          
          if {[strcasecmp $parmInfo(xaScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/com/$parmInfo(xaScript).com

              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Template Script $inScript"
                  return 1
                 }

              set outUnit [openScript $parmInfo(xaScript)_user]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d_%03d.fid
              set parmInfo(outName) $parmInfo(outPrefix)_xa.ft2
              set parmInfo(auxName) $parmInfo(outPrefix)_xa.fid

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"
             }

          if {$vjExpertFlag && [strcasecmp $parmInfo(xaScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/expert/$parmInfo(xaScript).com
   
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Expert Template Script $inScript"
                  return 1
                 }
 
              set outUnit [openScript $parmInfo(tmpScript)_expert]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d_%03d.fid
              set parmInfo(outName) $parmInfo(outPrefix)_xa.ft2
              set parmInfo(auxName) $parmInfo(outPrefix)_xa.fid

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"

              catch "file delete $parmInfo(xaScript)_expert.com"
              catch "exec $parmInfo(tmpScript)_expert.com > $parmInfo(xaScript)_expert.com"

              if {![file exists $parmInfo(xaScript)_expert.com]} \
                 {
                  showMsg "VJ2Pipe Error Generating Expert Script $parmInfo(xaScript)_expert.com"
                  return 1
                 }

              catch "exec chmod a+rx $parmInfo(xaScript)_expert.com"
              catch "file delete $parmInfo(tmpScript)_expert.com" 
             }
          
          if {[strcasecmp $parmInfo(xyzaScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/com/$parmInfo(xyzaScript).com
   
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Template Script $inScript"
                  return 1
                 }

              set outUnit [openScript $parmInfo(xyzaScript)_user]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d_%03d.fid
              set parmInfo(outName) ft/$parmInfo(outPrefix)%03d_%03d.ft4
              set parmInfo(auxName) ft/$parmInfo(outPrefix)
 
              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"
             }

          if {$vjExpertFlag && [strcasecmp $parmInfo(xyzaScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/expert/$parmInfo(xyzaScript).com
   
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Expert Template Script $inScript"
                  return 1
                 }
 
              set outUnit [openScript $parmInfo(tmpScript)_expert]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  data/$parmInfo(outPrefix)%03d_%03d.fid
              set parmInfo(outName) ft/$parmInfo(outPrefix)%03d_%03d.ft4
              set parmInfo(auxName) ft/$parmInfo(outPrefix)
 
              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"

              catch "file delete $parmInfo(xyzaScript)_expert.com"
              catch "exec $parmInfo(tmpScript)_expert.com > $parmInfo(xyzaScript)_expert.com"

              if {![file exists $parmInfo(xyzaScript)_expert.com]} \
                 {
                  showMsg "VJ2Pipe Error Generating Expert Script $parmInfo(xyzaScript)_expert.com"
                  return 1
                 }

              catch "exec chmod a+rx $parmInfo(xyzaScript)_expert.com"
              catch "file delete $parmInfo(tmpScript)_expert.com" 
             }
          
          if {[strcasecmp $parmInfo(scrollScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/com/$parmInfo(scrollScript).com
       
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Template Script $inScript"
                  return 1
                 }

              set outUnit [openScript $parmInfo(scrollScript)_user]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  ft/$parmInfo(outPrefix)%03d_%03d.ft4

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"
             }

          if {$vjExpertFlag && [strcasecmp $parmInfo(scrollScript) None]} \
             {
              set inScript $parmInfo(vjAuxDir)/expert/$parmInfo(scrollScript).com
   
              if {![file exists $inScript]} \
                 {
                  showMsg "VJ2Pipe Error Finding Expert Template Script $inScript"
                  return 1
                 }
 
              set outUnit [openScript $parmInfo(tmpScript)_expert]
              if {![strcasecmp $outUnit None]} {return 1}

              set parmInfo(inName)  ft/$parmInfo(outPrefix)%03d_%03d.ft4

              set error [subsScript $outUnit $inScript]
              catch "close $outUnit"

              catch "file delete $parmInfo(scrollScript)_expert.com"
              catch "exec $parmInfo(tmpScript)_expert.com > $parmInfo(scrollScript)_expert.com"

              if {![file exists $parmInfo(scrollScript)_expert.com]} \
                 {
                  showMsg "VJ2Pipe Error Generating Expert Script $parmInfo(scrollScript)_expert.com"
                  return 1
                 }

              catch "exec chmod a+rx $parmInfo(scrollScript)_expert.com"
              catch "file delete $parmInfo(tmpScript)_expert.com" 
             }
          
          addExecCommand "$parmInfo(xyScript)_user.com   -$gArg"
          addExecCommand "$parmInfo(xzScript)_user.com   -$gArg"
          addExecCommand "$parmInfo(xaScript)_user.com   -$gArg"
          addExecCommand "$parmInfo(xyzaScript)_user.com -$gArg"

          if {$graphFlag} \
             {
              addExecCommand "sleep 10"
              addExecCommand $parmInfo(scrollScript)_user.com
             }

          subsUserScripts
         }
      } 

   return 0
}

proc subsUserScripts {} \
{
   global subsList parmInfo

   foreach inScript $subsList \
      {
       if {![strcasecmp $inScript None]} continue

       if {![file exists $inScript]} \
          {
           showMsg "VJ2Pipe Error Finding User Template Script $inScript"
           return 1
          }

       showMsg "VJ2Pipe Processing Script $inScript ..."

       set outScript [file root [file tail $inScript]]_user
       set outUnit   [openScript $outScript]
       if {![strcasecmp $outUnit None]} {return 1}

       set error [subsScript $outUnit $inScript]
       catch "close $outUnit"
      }

   return 0
}

proc execScripts {} \
{
   global execFlag execList

   if {!$execFlag} return
   return 0
}

#
# Substitute _VAR_ style values in a template script.

proc subsScript { outUnit inScript } \
{
   global parmInfo dimCount skipFlag

   set error [catch {set inUnit [open $inScript r]}]

   if {$error} \
      {
       showMsg "VJ2Pipe Error Opening Template Script $inScript"
       return 1 
      }

   set vList [lsort -command sortStrLen [concat $parmInfo(dimVars,names) $parmInfo(dimVars,userNames)]]
   set vdList ""

   foreach v $vList \
      {
       if {[lsearch $parmInfo(hideList) $v] < 0} {lappend vdList $v} 
      }

   set vList [lsort -command sortStrLen [concat $parmInfo(listVars,names) \
                                                $parmInfo(scalarVars,names) \
                                                $parmInfo(listVars,userNames) \
                                                $parmInfo(scalarVars,userNames)]]

   set vaList ""

   foreach v $vList \
      {
       if {[lsearch $parmInfo(hideList) $v] < 0} {lappend vaList $v} 
      }

   while {[gets $inUnit line] >= 0} \
      {
       if {[string first "_" $line] < 0} \
          {
           puts $outUnit $line
           continue
          }

       foreach v $vdList \
          {
           if {[string first "_" $line] < 0} break

           for {set dim 1} {$dim <= $dimCount} {incr dim} \
              {
               if {[string first "_" $line] < 0} break
#              if {$skipFlag && $parmInfo(N,$dim) == 1} continue

               set a [lindex $parmInfo(axisPrefix) [expr $dim - 1]]

               set vStr "_$a${v}_"
               set loc1 [string first $vStr $line]
               if {$loc1 < 0} continue

               set locN [expr $loc1 + [string length $vStr] - 1]

               if {![info exists parmInfo($v,$dim)]} {set parmInfo($v,$dim) ""}

               if {[strcasecmp $parmInfo($v,$dim) _default_]} \
                  {
                   set line [string replace $line $loc1 $locN [format " %-10s " $parmInfo($v,$dim)]]
                  }
              }
          }

       foreach v $vaList \
          {
           if {[string first "_" $line] < 0} break

           set vStr "_${v}_"
           set loc1 [string first $vStr $line]
           if {$loc1 < 0} continue

           set locN [expr $loc1 + [string length $vStr] - 1]

           if {![info exists parmInfo($v)]} {set parmInfo($v) ""}

           if {[strcasecmp $parmInfo($v) _default_]} \
              {
               set line [string replace $line $loc1 $locN [format " %-10s " $parmInfo($v)]]
              }
          }

       puts $outUnit $line
      }

   catch "close $inUnit"
}

#
# Test for sort by string length.

proc sortStrLen { s1 s2 } \
{
   set n1 [string length $s1]
   set n2 [string length $s2]

   if {$n1 > $n2} {return -1}
   if {$n1 < $n2} {return  1}

   return 0
}

#
# Open an executable script file for writing.

proc openScript { thisName } \
{
   if {![strcasecmp $thisName None]} {return None}

   set ext [file extension $thisName]
   if {![string length $ext]} {append thisName ".com"}
 
   set error [catch {set scriptUnit [open $thisName w 0744]}]

   if {$error} \
      { 
       showMsg "VJ2Pipe Error Opening Script File $thisName"
       return None
      }

   return $scriptUnit
}

#
# Add quotes as needed to protect special characters from shell.

proc addQuotes { s } \
{
   set qList { "(" ")" "{" "}" "\[" "\]" "*" "&" "#" "@" "!" "^" "|" ":" ";"}

   foreach q $qList \
      {
       if {[string first $q $s] >= 0} {return \"$s\"}
      }

   return $s
}

#
#-----------------------------------------------------------------------------

set error       0
set msgName     stderr
set msgUnit     stderr
set execList    ""
set subsList    ""

set vjShowFlag   0
set vjApodFlag   0
set vjExpertFlag 1
set vjHdrFlag    1
set partialFlag  1
set elemCount    0
set sampleCount  0
set skipFlag     1
set maxDim       4
set infoFlag     0
set graphFlag    1
set execFlag     1
set wrapperFlag  0
set dimCount     0
set parmCount    0
set verbose      1

if {[flagLoc $ARGV -verb]}       {set verbose 1}
if {[flagLoc $ARGV -noverb]}     {set verbose 0}

if {$verbose} {puts $ARGV}

#
# For arguments handled by wrapper script such as vj2pipe.com:

if {[flagLoc $ARGV -wrapper]}   {set wrapperFlag 1}
if {[flagLoc $ARGV -nowrapper]} {set wrapperFlag 0}

if {[flagLoc $ARGV -help]} \
   {
    showHelp
    exit 0
   }

getListArgD $ARGV -subs subsList

getArgD $ARGV -msgName msgName
getArgD $ARGV -maxDim  maxDim

if {[flagLoc $ARGV -vjshow]}     {set vjShowFlag 1}
if {[flagLoc $ARGV -novjshow]}   {set vjShowFlag 0}

if {[flagLoc $ARGV -vjapod]}     {set vjApodFlag 1}
if {[flagLoc $ARGV -novjapod]}   {set vjApodFlag 0}

if {[flagLoc $ARGV -vjhdr]}      {set vjHdrFlag  1}
if {[flagLoc $ARGV -novjhdr]}    {set vjHdrFlag  0}

if {[flagLoc $ARGV -vjexpert]}   {set vjExpertFlag 1}
if {[flagLoc $ARGV -novjexpert]} {set vjExpertFlag 0}

if {[flagLoc $ARGV -partial]}    {set partialFlag 1}
if {[flagLoc $ARGV -nopartial]}  {set partialFlag 0}

if {[flagLoc $ARGV -skip]}       {set skipFlag 1}
if {[flagLoc $ARGV -noskip]}     {set skipFlag 0}

if {[flagLoc $ARGV -graph]}      {set graphFlag 1}
if {[flagLoc $ARGV -nograph]}    {set graphFlag 0}

if {[flagLoc $ARGV -exec]}       {set execFlag 1}
if {[flagLoc $ARGV -noexec]}     {set execFlag 0}

if {[flagLoc $ARGV -sinfo]}      {set infoFlag 2}
if {[flagLoc $ARGV -info]}       {set infoFlag 1}
if {[flagLoc $ARGV -noinfo]}     {set infoFlag 0}

setParmNames
setParmDefaults

getScalarArgs cd vjfid procpar

set error [getVJInput]
if {$error > 0} {quitproc 1}

set error [setDefDir]
if {$error} {quitProc 1}

initMsgFile

if {$infoFlag == 2} {showMsg "-vj2pipeStatusStart"}
   
readProcpar
readVHdr

updateProcpar

if {$parmCount < 4} \
   {
    showMsg "VJ2NMRPipe Error reading procpar."
    quitProc 2
   }

getAxisList
getDimCount

if {$dimCount > $maxDim} \
   {
    showMsg "VJ2NMRPipe Error: $maxDim Dimensions maximum are allowed."
    quitProc 1
   }

getExpName
getArrayExp
getAQOrd
getTSize
getQuadSize

getTemperature
getSW
getNUC
getGAMMA
getOFF
getOBS
getCAR
getLAB

getLShift
getPhase
getApod
getEXT
getBASE

getNUSInfo

updatePartialExp
updateArrayExp

getSpecial

getListArgD $ARGV -sVar parmInfo(scalarVars,userNames)
getListArgD $ARGV -dVar parmInfo(dimVars,userNames)
getListArgD $ARGV -lVar parmInfo(listVars,userNames)

getListArgs   -noreset $parmInfo(listVars,names) 
getScalarArgs -noreset $parmInfo(scalarVars,names) 
getAxisArgs   -noreset $parmInfo(dimVars,names)

getListArgs   -noreset $parmInfo(listVars,userNames) 
getScalarArgs -noreset $parmInfo(scalarVars,userNames) 
getAxisArgs   -noreset $parmInfo(dimVars,userNames)

fixNullParms

if {$infoFlag == 1} \
   {
    showParms -outUnit $msgUnit -skip $skipFlag       \
              -listVars   $parmInfo(listVars,names)   \
              -scalarVars $parmInfo(scalarVars,names) \
              -axisVars   $parmInfo(dimVars,names)  

    quitProc 0
   } \
elseif {$infoFlag == 2} \
   {
    showMsg "-vj2pipeStatusEnd"

    showParmsS -outUnit $msgUnit -skip $skipFlag       \
               -listVars   $parmInfo(listVars,names)   \
               -scalarVars $parmInfo(scalarVars,names) \
               -axisVars   $parmInfo(dimVars,names)  

    quitProc 0
   }

set error [createConvScript]
if {$error} {quitProc 3}

set error [createProcScripts]
if {$error} {quitProc 4}

set error [execScripts]

quitProc 0
