#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@" -notk

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Prepares XPLOR DC Input Scripts for MFR Fragment Refinement."
    puts " -da       daList"
    puts " -dr       drList"
    puts "Force Constant Scaling Factors:"
    puts " -kNHN     [1.0]  N-HN" 
    puts " -kCHN     [9.0]  HN-CO"
    puts " -kCN      [64.0] N-CO" 
    puts " -kCAHA    [0.25] CA-HA"
    puts " -kCCA     [25.0] CA-CO"
    puts " -kCACB    [25.0] CA-CB"
    exit 1 
   }

set daList ""
set drList ""

set typeList   {NHN CHN CN CAHA CCA CACB}
set outName    xdipSetup.inp
set fcName     xdipForce.inp
set statName   xdipStat.inp

set k(NHN)   1.00
set k(CAHA)  0.25
set k(CCA)  25.00
set k(CN)   64.00
set k(CHN)   9.00
set k(CACB) 25.00

set maxDIPO 10000

getListArgD $ARGV -da daList
getListArgD $ARGV -dr drList

foreach type $typeList {getArgD $ARGV -k$type k($type)}

#
#----------------------------------------------------------------------

set di(NHN)  -21585.20
set di(CAHA)  44539.47
set di(CCA)   4284.77
set di(CN)   -2609.05
set di(CHN)   6666.07
set di(CACB)  4284.77

set class(NHN)  DNH
set class(CAHA) DCH
set class(CCA)  DCC
set class(CN)   DNC
set class(CHN)  DHC
set class(CACB) DAB

if {[llength $daList] != [llength $drList]} \
   {
    puts stderr "Da/Dr count mismatch"
    exit 1
   }

set nTensor [llength $daList]

#
# Xplor code for XDIP setup

set outUnit [open $outName w]

puts $outUnit "! parameters for XDIPolar energy term"
puts $outUnit ""
puts $outUnit "! relative weights"
puts $outUnit ""

foreach type $typeList \
   {
    puts $outUnit [format "evaluate (\$kdipo_%s_fac  = %s)" $type $k($type)]
   }

puts $outUnit ""

for {set i 0} {$i < $nTensor} {incr i} \
    {
     set thisDa [lindex $daList $i]
     set thisDr [lindex $drList $i]
     set r      [expr double($thisDr)/double($thisDa)]

     puts $outUnit "! alignment tensor magnitudes"
     puts $outUnit ""

     foreach type $typeList \
        {
         set da($type) [expr double($thisDa)*abs($di($type))]
         set fmt       "evaluate (\$da_%s%d  = %6.2f)"

         puts $outUnit [format $fmt $type $i $da($type)]
        }
 
     puts $outUnit "evaluate (\$r${i} = [format %.4f $r])"
     puts $outUnit "evaluate (\$r${i}_factor =  1 + 0.75 * \$r${i} * \$r${i})"
     puts $outUnit ""

     puts $outUnit "! relative scaling factors according to tensor magnitude"
     puts $outUnit ""

     foreach type $typeList \
        {
         set    fmt "evaluate (\$kdipo_%s%d_fac  = \$kdipo_%s_fac * "
         append fmt "((\$da_%s0 * \$r0_factor)^2/(\$da_%s%d * \$r%d_factor)^2))"

         puts $outUnit [format $fmt $type $i $type $type $type $i $i]
        }

     puts $outUnit ""

     puts $outUnit "! evaluate the individual force constants"
     puts $outUnit ""

     foreach type $typeList \
        {
         set fmt "evaluate (\$kdipo_%s%d  = \$kdipo * \$kdipo_%s%d_fac)"
         puts $outUnit [format $fmt $type $i $type $i]
        }

     puts $outUnit ""
    }

puts $outUnit "! opening DC tables"
puts $outUnit ""
puts $outUnit "set echo off message off end"
puts $outUnit ""
puts $outUnit "xdipo"
puts $outUnit "   reset"
puts $outUnit "   nres=$maxDIPO"

for {set i 0} {$i < $nTensor} {incr i} \
    {
     set tResID [expr $i + 500]

     foreach type $typeList \
        {
         puts $outUnit ""
         puts $outUnit "   class $class($type)$i"
         puts $outUnit "   force \$kdipo_$type$i"
         puts $outUnit "   potential harmonic"
         puts $outUnit "   coeff 0.0 \$da_$type$i \$r${i}"

         set thisName "dc-${tResID}-$type.xdip"

         if {[file exists $thisName]} \
            {
             puts $outUnit "   @$thisName"
            }
        }
    }

puts $outUnit ""
puts $outUnit "end"
puts $outUnit ""
puts $outUnit "set echo on message on end"

close $outUnit

#
# Xplor code for XDIP force constats

set outUnit [open $fcName w]

puts $outUnit "! force constants for XDIPolar energy term"
puts $outUnit ""

for {set i 0} {$i < $nTensor} {incr i} \
    {
     puts $outUnit "! evaluate the individual force constants"
     puts $outUnit ""

     foreach type $typeList \
        {
         set fmt "evaluate (\$kdipo_%s%d = \$kdipo * \$kdipo_%s%d_fac)"

         puts $outUnit [format $fmt $type $i $type $i]
        }

     puts $outUnit ""
    }

puts $outUnit "! set the XDIP force constants"
puts $outUnit ""
puts $outUnit "xdipo"

for {set i 0} {$i < $nTensor} {incr i} \
    {
     foreach type $typeList \
        {
         puts $outUnit "class $class($type)$i force \$kdipo_$type$i"
        }
    }

puts $outUnit "end"
puts $outUnit ""

close $outUnit

#
# Xplor code for analysis

set outUnit [open $statName w]

puts $outUnit "! statistics of violations of dipolar restraints"
puts $outUnit ""

for {set i 0} {$i < $nTensor} {incr i} \
    {
     foreach type $typeList \
        {
         puts $outUnit "xdip print threshold=0.0 class $class($type)$i end"
         puts $outUnit "evaluate (\$rms_$type$i=\$result)"
         puts $outUnit "evaluate (\$violations_$type$i=\$violations)"
        }

     puts $outUnit ""
    }

close $outUnit

exit

