/******************************************************************************/
/*                                                                            */
/*                   ---- NIH NMR Software System ----                        */
/*                        Copyright 1992 and 1993                             */
/*                             Frank Delaglio                                 */
/*                   NIH Laboratory of Chemical Physics                       */
/*                                                                            */
/*               This software is not for distribution without                */
/*                  the written permission of the author.                     */
/*                                                                            */
/******************************************************************************/

/***/
/* bin2pipe: conversions from binary floating format to nmrPipe stream;
/*           used primarily to add a correct nmrPipe header to binary
/*           data.
/***/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "version.h"
#include "cmndargs.h"
#include "testsize.h"
#include "dataio.h"
#include "minmax.h"
#include "memory.h"
#include "fdatap.h"
#include "prec.h"
#include "vutil.h"
#include "bswap.h"

#define RI_INTERLEAVED  1
#define RI_SEPERATED    2

#define SRC_UNKNOWN     0
#define SRC_GE          1
#define SRC_FTNEW       2
#define SRC_FTOLD       3
#define SRC_CHEM        4
#define SRC_GSX         5
#define SRC_EX          6
#define SRC_ALPHA       7
#define SRC_LAMBDA      8
#define SRC_VARIAN      9

#define LAM_SIZE       21

#define SWAP4( B ) { c=B[0];B[0]=B[3];B[3]=c; c=B[1];B[1]=B[2];B[2]=c; } c=c

#define NMRTXT     "NMRTXT"
#define FPR        (void)fprintf
#define SPR        (void)sprintf

static  int        wordSize = 4;

int main( argc, argv )

   int  argc;
   char **argv;
{
    int   xSize, ySize, zSize, aSize, xMid, yMid, zMid, aMid, byteOffset,
          planeCount, xFreqSize, yFreqSize, zFreqSize, aFreqSize, binSize,
          matSize, auxSize, plane, dimCount, dim, quadType, checkFlag, xyzFlag,
          ovFlag, thisSize, thisQuad, xQuadState, yQuadState, zQuadState,
          riOrder, verbose, bytesIn, bytesOut, ptsOut, lastAdj, firstPlane,
          lastPlane,  bytePad, padCount, swapFlag, i2rFlag, noInputFlag,
          noOutputFlag, negFlag, padFlag, ilFlag, dirFlag, srcType, 
          origByteOffset, totalByteOffset, yQuadFlag, zQuadFlag, aQuadFlag, 
          macCount, xTD, yTD, zTD, aTD, xZF, yZF, zZF, aZF,
          status, badCount, error;

    int   lamBuff[LAM_SIZE];

    char  *binMat, *auxMat, inName[NAMELEN+1], auxName[NAMELEN+1], 
          outName[NAMELEN+1], outFmt[NAMELEN+1], macName[NAMELEN+1],
          cmndName[NAMELEN+1], serverListName[NAMELEN+1], ctemp[NAMELEN+1];

    float *dataMat, xSW, ySW, zSW, aSW, xObs, yObs, zObs, aObs, 
          xCar, yCar, zCar, aCar, xOrig, yOrig, zOrig, aOrig,
          maxVal, minVal, value, badThresh, fdata[FDATASIZE];

    UNIT  inUnit, auxUnit, outUnit;

/***/
/* Initialize:
/***/

    error     = 0;
    matSize   = 0;
    binSize   = 0;
    auxSize   = 0;
    badCount  = 0;

    wordSize  = sizeof(float);

    minVal    =  LARGENUM;
    maxVal    = -LARGENUM;
    
    inUnit    = UNIT_NULL;
    auxUnit   = UNIT_NULL;
    outUnit   = UNIT_NULL;
 
    dataMat   = (float *) NULL;
    binMat    = (char *) NULL;
    auxMat    = (char *) NULL;

    *macName  = '\0';
    *cmndName = '\0';

    macCount  = 0;

    (void) initArgs( argc, argv, CHECK_ENV );
    (void) setHdrNull( fdata );

/***/
/* Extract command parameters:
/***/

    error = getparams( argc, argv,
                       inName, auxName, outFmt, serverListName,
                       fdata, &firstPlane, &lastPlane, &byteOffset, &bytePad,
                       &xyzFlag, &riOrder, &checkFlag, &ovFlag, &noInputFlag, 
                       &noOutputFlag, &swapFlag, &negFlag, &i2rFlag, &padFlag,
                       &ilFlag, &dirFlag, &srcType, &badThresh, &verbose );

    if (error)
       {
        FPR( stderr, "Bin2Pipe Error extracting arguments.\n" );
        goto shutdown;
       }

    (void) fixfdata( fdata );

    dimCount = getParm( fdata, FDDIMCOUNT, 0 );

/***/
/* Handle special indirect acquisition modes:
/***/

    yQuadFlag = getParm( fdata, NDQUADFLAG, CUR_YDIM );
    zQuadFlag = getParm( fdata, NDQUADFLAG, CUR_ZDIM );
    aQuadFlag = getParm( fdata, NDQUADFLAG, CUR_ADIM );

    switch( yQuadFlag )
       {
        case FD_SE:
           SPR( macName, "%s/%s -all", getenv( NMRTXT ), "bin_seY.M" );
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_YDIM );
           macCount++;
           break;
        case FD_GRAD:
           SPR( macName, "%s/%s", getenv( NMRTXT ), "bin_ranceY.M" );
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_YDIM );
           macCount++;
           break;
        default:
           break;
       }

    switch( zQuadFlag )
       {
        case FD_SE:
           SPR( macName, "%s/%s -all", getenv( NMRTXT ), "bin_seZ.M" );
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_ZDIM );
           macCount++;
           break;
        case FD_GRAD:
           SPR( macName, "%s/%s", getenv( NMRTXT ), "bin_ranceZ.M" );
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_ZDIM );
           macCount++;
           break;
        default:
           break;
       }

    switch( aQuadFlag )
       {
        case FD_SE:
           SPR( macName, "%s/%s -all", getenv( NMRTXT ), "bin_seA.M" );
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_ADIM );
           macCount++;
           break;
        case FD_GRAD:
           SPR( macName, "%s/%s", getenv( NMRTXT ), "bin_ranceA.M" );
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_ADIM );
           macCount++;
           break;
        default:
           break;
       }

    if (noOutputFlag)
       {
        *macName = '\0';
        macCount = 0;
       }

    if (macCount > 1)
       {
        FPR( stderr, 
             "Bin2Pipe Warning: %d special acquisition modes;\n",
             macCount );

        FPR( stderr, " Only one special mode is allowed.\n" );
       }

    if (*macName)
       {
        SPR( cmndName, "nmrPipe -fn MAC -macro %s -noRd -noWr", macName );

        xyzFlag = 0;

        if (*outFmt)
           {
            if (dimCount < 3)
               {
                (void) sprintf( ctemp, " -out %s", outFmt );
                (void) strcat( cmndName, ctemp );
               }
            else
               {
                (void) sprintf( ctemp, " | pipe2xyz -out %s -to 0", outFmt );
                (void) strcat( cmndName, ctemp );
               }

            if (ovFlag) (void) strcat( cmndName, " -ov" );

            *outFmt = '\0';
           }
       }
/***/
/* Set redundant parameters if they have not been explicitly set:
/***/

    if (flagLoc( argc, argv, "-xMODE" ) && !flagLoc( argc, argv, "-aqMODE" ))
       {
        value = getParm( fdata, NDQUADFLAG, CUR_XDIM );
        (void) setParm( fdata, FDQUADFLAG, value, 0 );
       }

    if (flagLoc( argc, argv, "-xT" ) && !flagLoc( argc, argv, "-aqN" ))
       {
        value = getParm( fdata, NDAPOD, CUR_XDIM );
        (void) setParm( fdata, FDREALSIZE, value, 0 );
       }

    if (flagLoc( argc, argv, "-yMODE" ) && !flagLoc( argc, argv, "-aq2D" ))
       {
        yQuadState = dimCount < 2 ? 1 : getQuad( fdata, NDQUADFLAG, CUR_YDIM );

        if (yQuadState == 1)
           (void) setParm( fdata, FD2DPHASE, (float)FD_TPPI,   NULL_DIM );
        else
           (void) setParm( fdata, FD2DPHASE, (float)FD_STATES, NULL_DIM );
       }

/***/
/* Extract sizes and quad states; adjust sizes and states for output.
/*   XSIZE, YSIZE, etc begin as count of real+imag points for each axis.
/*   XSIZE is reduced to number of complex points.
/***/

    dimCount = getParm( fdata, FDDIMCOUNT, 0 );
    quadType = getParm( fdata, FDQUADFLAG, 0 );

    xSize    = dimCount < 1 ? 1 : getParm( fdata, NDSIZE, CUR_XDIM );
    ySize    = dimCount < 2 ? 1 : getParm( fdata, NDSIZE, CUR_YDIM );
    zSize    = dimCount < 3 ? 1 : getParm( fdata, NDSIZE, CUR_ZDIM ); 
    aSize    = dimCount < 4 ? 1 : getParm( fdata, NDSIZE, CUR_ADIM ); 

    switch( quadType )
       {
        case 0:
           xQuadState  = 2;
           xSize      /= xQuadState;
           xMid        = 1 + xSize/2;
           xFreqSize   = xSize;
           (void) setParm( fdata, NDSIZE, (float)xSize, CUR_XDIM );
           break;
        case 1:
           xQuadState = 1;
           xMid       = 1 + xSize/4;
           xFreqSize  = xSize/2; 
           break;
        case 2:
           (void) setParm( fdata, NDQUADFLAG, (float)1.0, CUR_XDIM );
           (void) setParm( fdata, FDQUADFLAG, (float)1.0, 0 );
           xQuadState = 1;
           xMid       = 1 + xSize/4;
           xFreqSize  = xSize/2;
           break;
        default:
           FPR( stderr, "Bin2Pipe Error:" );
           FPR( stderr, " bad quad type %d.\n", quadType );
           error = 1;
           goto shutdown;
           break;
       }

    planeCount = zSize*aSize;

    if (dimCount <= 2 || xyzFlag)
       {
        (void) setParm( fdata, FDFILECOUNT, (float)planeCount, 0 );
        (void) setParm( fdata, FDPIPEFLAG,  (float)NULL_DIM,   0 );
       }
    else
       {
        (void) setParm( fdata, FDFILECOUNT, (float)1, 0 );
        (void) setParm( fdata, FDPIPEFLAG,  (float)1, 0 );
       }

    yQuadState = getQuad( fdata, NDQUADFLAG, CUR_YDIM );
    zQuadState = getQuad( fdata, NDQUADFLAG, CUR_ZDIM );

/***/
/* Center (Zero Freq) Locations are set according to complex data sizes.
/* Axis origins in hertz are set according to carrier at point freqSize/2 + 1.
/* Don't replace explicit definitions of Final Freq-Domain Size or Center:
/***/

    yFreqSize = ySize/2 ? ySize/2 : 1;
    zFreqSize = zSize/2 ? zSize/2 : 1;
    aFreqSize = aSize/2 ? aSize/2 : 1;

    xZF = 0;
    yZF = 0;
    zZF = 0;
    aZF = 0;

    if (flagLoc( argc, argv, "-xFD" ))
       {
        xFreqSize = intArg( argc, argv, "-xFD" );
        xMid      = xFreqSize/2 + 1;
        xZF       = -xFreqSize;
       }

    if (flagLoc( argc, argv, "-yFD" ))
       {
        yFreqSize = intArg( argc, argv, "-yFD" );
        yZF       = -yFreqSize;
       }

    if (flagLoc( argc, argv, "-zFD" ))
       {
        zFreqSize = intArg( argc, argv, "-zFD" );
        zZF       = -zFreqSize;
       }

    if (flagLoc( argc, argv, "-aFD" ))
       {
        aFreqSize = intArg( argc, argv, "-aFD" );
        aZF       = -aFreqSize;
       }

    yMid = 1 + yFreqSize/2;
    zMid = 1 + zFreqSize/2;
    aMid = 1 + aFreqSize/2; 

    if (flagLoc( argc, argv, "-xMID" )) xMid = intArg( argc, argv, "-xMID" );
    if (flagLoc( argc, argv, "-yMID" )) yMid = intArg( argc, argv, "-yMID" );
    if (flagLoc( argc, argv, "-zMID" )) zMid = intArg( argc, argv, "-zMID" );
    if (flagLoc( argc, argv, "-aMID" )) aMid = intArg( argc, argv, "-aMID" );

    (void) setParm( fdata, NDCENTER, (float)xMid, CUR_XDIM );
    (void) setParm( fdata, NDCENTER, (float)yMid, CUR_YDIM );
    (void) setParm( fdata, NDCENTER, (float)zMid, CUR_ZDIM );
    (void) setParm( fdata, NDCENTER, (float)aMid, CUR_ADIM );

    if (flagLoc( argc, argv, "-xZFP" )) xZF = intArg( argc, argv, "-xZFP" );
    if (flagLoc( argc, argv, "-yZFP" )) yZF = intArg( argc, argv, "-yZFP" );
    if (flagLoc( argc, argv, "-zZFP" )) zZF = intArg( argc, argv, "-zZFP" );
    if (flagLoc( argc, argv, "-aZFP" )) aZF = intArg( argc, argv, "-aZFP" );

    (void) setParm( fdata, NDZF, (float)xZF, CUR_XDIM );
    (void) setParm( fdata, NDZF, (float)yZF, CUR_YDIM );
    (void) setParm( fdata, NDZF, (float)zZF, CUR_ZDIM );
    (void) setParm( fdata, NDZF, (float)aZF, CUR_ADIM );

    xObs  = getParm( fdata, NDOBS, CUR_XDIM );
    yObs  = getParm( fdata, NDOBS, CUR_YDIM );
    zObs  = getParm( fdata, NDOBS, CUR_ZDIM );
    aObs  = getParm( fdata, NDOBS, CUR_ADIM );

    xSW   = getParm( fdata, NDSW,  CUR_XDIM );
    ySW   = getParm( fdata, NDSW,  CUR_YDIM );
    zSW   = getParm( fdata, NDSW,  CUR_ZDIM );
    aSW   = getParm( fdata, NDSW,  CUR_ADIM );

    xCar  = getParm( fdata, NDCAR, CUR_XDIM );
    yCar  = getParm( fdata, NDCAR, CUR_YDIM );
    zCar  = getParm( fdata, NDCAR, CUR_ZDIM );
    aCar  = getParm( fdata, NDCAR, CUR_ADIM );

/***/
/* Don't replace explicit definitions of Axis Origin:
/***/

    if (flagLoc( argc, argv, "-xORIG" ))
       xOrig = fltArg( argc, argv, "-xORIG" );
    else
       xOrig = xObs*xCar - xSW*(xFreqSize - xMid)/xFreqSize;

    if (flagLoc( argc, argv, "-yORIG" ))
       yOrig = fltArg( argc, argv, "-yORIG" );
    else
       yOrig = yObs*yCar - ySW*(yFreqSize - yMid)/yFreqSize;

    if (flagLoc( argc, argv, "-zORIG" ))
       zOrig = fltArg( argc, argv, "-zORIG" );
    else
       zOrig = zObs*zCar - zSW*(zFreqSize - zMid)/zFreqSize;

    if (flagLoc( argc, argv, "-aORIG" ))
       aOrig = fltArg( argc, argv, "-aORIG" );
    else
       aOrig = aObs*aCar - aSW*(aFreqSize - aMid)/aFreqSize;

    (void) setParm( fdata, NDORIG, xOrig, CUR_XDIM );
    (void) setParm( fdata, NDORIG, yOrig, CUR_YDIM );
    (void) setParm( fdata, NDORIG, zOrig, CUR_ZDIM );
    (void) setParm( fdata, NDORIG, aOrig, CUR_ADIM );

/***/
/* Don't replace explicit definitions of Original Time-Domain Size:
/***/

    if (flagLoc( argc, argv, "-xTD" ))
       xTD = intArg( argc, argv, "-xTD" );
    else
       xTD = getParm( fdata, NDAPOD, CUR_XDIM );

    if (flagLoc( argc, argv, "-yTD" ))
       yTD = intArg( argc, argv, "-yTD" );
    else
       yTD = getParm( fdata, NDAPOD, CUR_YDIM );

    if (flagLoc( argc, argv, "-zTD" ))
       zTD = intArg( argc, argv, "-zTD" );
    else
       zTD = getParm( fdata, NDAPOD, CUR_ZDIM );

    if (flagLoc( argc, argv, "-aTD" ))
       aTD = intArg( argc, argv, "-aTD" );
    else
       aTD = getParm( fdata, NDAPOD, CUR_ADIM );

    (void) setParm( fdata, NDTDSIZE, (float)xTD, CUR_XDIM );
    (void) setParm( fdata, NDTDSIZE, (float)yTD, CUR_YDIM );
    (void) setParm( fdata, NDTDSIZE, (float)zTD, CUR_ZDIM );
    (void) setParm( fdata, NDTDSIZE, (float)aTD, CUR_ADIM );

/***/
/* Extract explicit user adjustments to header data:
/***/

    if (error = parseHdr2( argc, argv, fdata ))
       {
        FPR( stderr, "Bin2Pipe Error setting User Header.\n" );
        goto shutdown;
       }

/***/
/* Check use of command-line options:
/***/

    (void) checkArgs( argc, argv );

/***/
/* Open a dummy output file for testing purposes.
/* Test for sufficient space as needed.
/* Delete the dummy file.
/* Open the binary input file.
/***/

    if (*outFmt && checkFlag && !noOutputFlag)
       {
        error = fidOpen( outFmt, outName, cmndName,
                         &outUnit, 1, fdata, ovFlag,
                         xyzFlag, dirFlag, verbose );

        if (error)
           {
            FPR( stderr, "Bin2Pipe Error Testing %s\n", outName );
            goto shutdown;
           }

        error = fidClose( outFmt, &outUnit, minVal, maxVal, fdata, xyzFlag );

        if (error)
           {
            FPR( stderr, "Bin2Pipe Error Updating %s\n", outName );
            goto shutdown;
           }

        switch( testNMRout( outName, fdata, xyzFlag, 1 ))
            {
             case FILE_SIZE_FAIL:
                FPR( stderr, "\n" );
                FPR( stderr, "Bin2Pipe Warning:" );
                FPR( stderr, " can't check size for file" );
                FPR( stderr, " %s\n", outName );
                break;
             case  FILE_SIZE_TOO_LARGE:
                FPR( stderr, "\n" );
                FPR( stderr, "Bin2Pipe Error:" );
                FPR( stderr, " insufficient space for file" );
                FPR( stderr, " %s\n", outName );
                error = 1;
                goto shutdown;
                break;
             default:
                break;
            }

        (void) delFile( outName );
       }

    if (!noInputFlag)
       {
        if (error = binOpen( inName, &inUnit ))
           {
            FPR( stderr, "Bin2Pipe Error Opening Input File %s.\n", inName );
            goto shutdown;
           }

        if (error = binOpen( auxName, &auxUnit ))
           {
            FPR( stderr, "Bin2Pipe Error Opening Input File %s.\n", auxName );
            goto shutdown;
           }
       }

/***/
/* Adjust and check sizes and allocate memory.
/* Minimum memory is header size to be on safe side.
/***/

    origByteOffset = byteOffset;
    byteOffset     = byteOffset < 0 ? 0 : byteOffset;
    padCount       = bytePad/wordSize;
    
    if (padCount*wordSize != bytePad) padCount++;

    binSize = byteOffset + bytePad + (xSize*xQuadState + padCount)*ySize;
    auxSize = byteOffset + bytePad + (xSize*xQuadState + padCount)*ySize/2;
    matSize = FDATASIZE  + xSize*xQuadState*ySize;

    bytesIn   = wordSize*xSize*xQuadState*ySize + bytePad*ySize;
    bytesOut  = wordSize*xSize*xQuadState*ySize;
    ptsOut    = xSize*xQuadState*ySize;

    if (ilFlag)
       {
        if (ySize < 2 || ySize % 2)
           {
            FPR( stderr, "Bin2Pipe Warning: interleaved 2D input requires\n" );
            FPR( stderr, "an even number of points for the output YSIZE.\n" );
           }

        bytesIn /= 2;
       }

    if (!(binMat = charAlloc( "bin", binSize*wordSize )))
       {
        FPR( stderr, "Bin2Pipe Error allocating input memory.\n" );
        error = 1;
        goto shutdown;
       }
 
    if (!(dataMat = fltAlloc( "bin", matSize )))
       {
        FPR( stderr, "Bin2Pipe Error allocating output memory.\n" );
        error = 1;
        goto shutdown;
       }

    if (ilFlag)
       {
        if (!(auxMat = charAlloc( "bin", auxSize*wordSize )))
           {
            FPR( stderr, "Bin2Pipe Error allocating aux memory.\n" );
            error = 1;
            goto shutdown;
           }
       }

/***/
/* Special Case:
/*  Determine byte-offset from input file header:
/***/

    totalByteOffset = byteOffset;

    if (srcType == SRC_LAMBDA && origByteOffset < 0)
       {
        if (*inName)
           error = dataReadC( inUnit, lamBuff, sizeof(int)*LAM_SIZE );
        else
           error = dataReadCB( inUnit, lamBuff, sizeof(int)*LAM_SIZE, -1, 0 );

        if (error)
           {
            FPR( stderr, "Bin2Pipe Error determining byte offset.\n" );
            error = 1;
            goto shutdown;
           }

        if (swapFlag) (void) iByteSwap( lamBuff, LAM_SIZE );

        totalByteOffset = lamBuff[LAM_SIZE-1];
        byteOffset      = totalByteOffset - sizeof(int)*LAM_SIZE;

        if (byteOffset < 0)
           {
            FPR( stderr,
                 "Bin2Pipe Error: bad byte offset %d.\n",
                 totalByteOffset );

            error = 1;
            goto shutdown;
           }
       }

/***/
/* Display initial status:
/***/

    if (verbose)
       {
        FPR( stderr, "\n" );
        FPR( stderr, "Binary --> NMRPipe Conversion.\n" );

        if (noInputFlag)
           {
            FPR( stderr, "Input File: NULL" );
           }
        else if (*inName)
           {
            if (ilFlag)
               FPR( stderr, "Input Files: %s and %s", inName, auxName );
            else
               FPR( stderr, "Input File: %s", inName );            
           }
        else
           { 
            FPR( stderr, "Input File: Standard Input." );
           }

        FPR( stderr, " Byte Offset: %d.\n", totalByteOffset );

        if (*outFmt)
           {
            if (xyzFlag)
               FPR( stderr, "Output Template: %s\n", outFmt );
            else
               FPR( stderr, "Output File: %s\n", outFmt );
           }
        else
           {
            FPR( stderr, "Output File: Standard Output.\n" );
           }

        if (noOutputFlag)
           {
            FPR( stderr, "Output Header Data Only.\n" );
           }
           
        FPR( stderr, "%dD Sizes: ", dimCount );

        for( dim = 1; dim <= dimCount; dim++ )
           {
            thisSize = getParm( fdata, NDSIZE, dim );
            thisQuad = getParm( fdata, NDQUADFLAG, dim );

            if (!thisQuad && dim == 1) thisSize *= 2;

            if (thisQuad)
               FPR( stderr, "(%d Real)", thisSize );
            else
               FPR( stderr, "(%d Real+Imag)", thisSize );
           }

        FPR( stderr, " Swap: %s\n\n", swapFlag ? "Yes" : "No" );
        (void) fflush( stderr );
       }

/*
    if (*outFmt && dimCount > 2 && !xyzFlag)
       {
        FPR( stderr, "Bin2Pipe Notice: %dD output name", dimCount );
        FPR( stderr, " is not a template.\n" );
        FPR( stderr, "\n\n" );
        (void) fflush( stderr );
       }
*/

/***/
/* For each 2D plane:
/*   Read past header if this is the first plane.
/*   Read the plane.
/*   Convert the slices.
/*   Write the slices as a single output plane.
/***/

    for( plane = 1; plane <= planeCount; plane++ )
       {
        lastAdj = plane == planeCount ? bytePad : 0;

        if (noInputFlag)
           {
            (void) cSet( binMat, (int)(binSize*wordSize), (char)0 );
           }
        else
           {
            if (padFlag)
               {
                (void) cSet( binMat, (int)(binSize*wordSize), (char)0 );
               }

            if (*inName)
               {
                if (plane == 1)
                   {
                    if (error = dataReadC( inUnit, binMat, byteOffset ))
                       {
                        if (!padFlag)
                           {
                            FPR( stderr, "Bin2Pipe Error Reading File.\n" );
                            goto shutdown;
                           }
                       }
                   }

                if (error = dataReadC( inUnit, binMat, bytesIn-lastAdj ))
                   {
                    if (!padFlag)
                       {
                        FPR( stderr, "Bin2Pipe Error Reading File.\n" );
                        goto shutdown; 
                       }
                   }

                if (swapFlag) (void) byteSwap4( binMat, bytesIn-lastAdj );
               }
            else
               {
                if (plane == 1)
                   {
                    if (error = dataReadCB( inUnit, binMat, byteOffset, -1, 0 ))
                       {
                        if (!padFlag)
                           {
                            FPR( stderr, "Bin2Pipe Error Reading Stream.\n" );
                            goto shutdown;
                           }
                       }
                   }

                if (error = dataReadCB(inUnit, binMat, bytesIn-lastAdj, -1, 0))
                   {
                    if (!padFlag)
                       {
                        FPR( stderr, "Bin2Pipe Error Reading Input Stream.\n" );
                        goto shutdown; 
                       }
                   }

                if (swapFlag) (void) byteSwap4( binMat, bytesIn-lastAdj );
               }
           }

        if (ilFlag)
           {
            if (padFlag)
               {
                (void) cSet( auxMat, (int)(binSize*wordSize), (char)0 );
               }

            if (*auxName)
               {
                if (plane == 1)
                   {
                    if (error = dataReadC( auxUnit, auxMat, byteOffset ))
                       {
                        if (!padFlag)
                           {
                            FPR( stderr, "Bin2Pipe Error Reading File.\n" );
                            goto shutdown;
                           }
                       }
                   }

                if (error = dataReadC( auxUnit, auxMat, bytesIn-lastAdj ))
                   {
                    if (!padFlag)
                       {
                        FPR( stderr, "Bin2Pipe Error Reading File.\n" );
                        goto shutdown; 
                       }
                   }

                if (swapFlag) (void) byteSwap4( auxMat, bytesIn-lastAdj );
               }
            else
               {
                if (plane == 1)
                   {
                    if (error = dataReadCB(auxUnit, auxMat, byteOffset, -1, 0))
                       {
                        if (!padFlag)
                           {
                            FPR( stderr, "Bin2Pipe Error Reading Stream.\n" );
                            goto shutdown;
                           }
                       }
                   }

                if (error = dataReadCB(auxUnit, auxMat, bytesIn-lastAdj, -1,0))
                   {
                    if (!padFlag)
                       {
                        FPR( stderr, "Bin2Pipe Error Reading Input Stream.\n" );
                        goto shutdown; 
                       }
                   }

                if (swapFlag) (void) byteSwap4( auxMat, bytesIn-lastAdj );
               }

            (void) bin2il( binMat, auxMat, 
                           (int)(wordSize*xSize*xQuadState + bytePad),
                           ySize/2 ); 
           }

        error = fidOpen( outFmt, outName, cmndName,
                         &outUnit, plane, fdata, ovFlag,
                         xyzFlag, dirFlag, verbose );

        if (error)
           {
            FPR( stderr, "Bin2Pipe Error Opening Output.\n" );
            goto shutdown;
           }

        (void) bin2fid( binMat, dataMat, xSize, ySize,
                        xQuadState, yQuadState, zQuadState,
                        bytePad, riOrder, negFlag, i2rFlag,
                        &minVal, &maxVal );

        if (badThresh != 0.0)
           {
            badCount += fixBadPts( dataMat, ptsOut, badThresh );
           }

        if (!noOutputFlag)
           { 
            if (dataWrite( outUnit, dataMat, bytesOut ))
               {
                FPR( stderr, "Bin2Pipe Writing Output.\n" );
                goto shutdown; 
               }
           }

        error = fidClose( outFmt, &outUnit, 
                          minVal, maxVal, fdata,
                          xyzFlag );

        if (error)
           {
            FPR( stderr, "Bin2Pipe Error Updating Output.\n" );
            goto shutdown;
           }

        if (verbose)
           {
            FPR( stderr, "Plane %d of %d\015", plane, planeCount );
            (void) fflush( stderr );
           }
       }

    if (verbose)
       {
        FPR( stderr, "\n" );

        if (badCount)
           {
            FPR( stderr, "Found %d bad points.\n", badCount );
           }

        (void) fflush( stderr );
       }

    if (*cmndName)
       {
        (void) wait( &status );
       }

/***/
/* Exit point:
/***/

shutdown:

    if (inUnit  != FB_STDIN)  (void) dataClose( inUnit );
    if (outUnit != FB_STDOUT) (void) dataClose( outUnit );

    (void) deAlloc( "bin", binMat,  wordSize*binSize );
    (void) deAlloc( "bin", auxMat,  wordSize*binSize );
    (void) deAlloc( "bin", dataMat, wordSize*matSize );

    return( error );
}

/***/
/* binOpen: open binary format file for input.
/***/

int binOpen( inName, inUnitPtr )

   char *inName;
   UNIT *inUnitPtr;
{
    if (*inName)
       {
        if (dataOpen( inName, inUnitPtr, FB_READ )) return( 1 );
       }
    else
       {
        *inUnitPtr = FB_STDIN;
       }

    return( 0 );
}

/***/
/* fidOpen: open a pipe format output file, write an initial header.
/***/

int fidOpen( outFmt, outName, cmndName, 
             outUnitPtr, plane, fdata,
             ovFlag, xyzFlag, dirFlag,
             verbose )

   UNIT  *outUnitPtr;
   float fdata[FDATASIZE];
   char  *outFmt, *outName, *cmndName;
   int   plane, ovFlag, xyzFlag, dirFlag, verbose;
{
    char   dirName[NAMELEN+1];
    static int dirCreated = 0;

    if (dirCreated) dirFlag = 0;

    (void) sprintf( outName, outFmt, plane );
 
    if (*outFmt)
       {
        if (dirFlag && plane == 1)
           {
            if (!dirExists( outName, dirName ))
               {
                if (verbose)
                   {
                    FPR( stderr,
                         "Bin2Pipe Notice: Creating directory %s\n",
                         dirName );
                   }

                if (dirCreate( dirName ))
                   {
                    FPR( stderr,
                         "Bin2Pipe Warning: Error creating directory %s\n",
                         dirName );
                   }

                dirCreated = 1;
               }
           }

        if (!ovFlag && fileExists( outName ))
           {
            FPR( stderr,
                 "Bin2Pipe Error: output file %s exists.\n",
                 outName );

            return( 1 );
           }

        if (xyzFlag || plane == 1)
           {
            if (dataOpen( outName, outUnitPtr, FB_CREATE )) return( 1 );
           }
       }
    else if (*cmndName)
       {
        if (plane == 1)
           {
            if (dataPOpen( cmndName, outUnitPtr, FB_WRITE )) return( 1 );
           }
       }
    else
       {
        *outUnitPtr = FB_STDOUT;
       }

/***/
/* Write a header if this is a seperate plane:
/***/

    if (xyzFlag || plane == 1)
       {
        if (dataWrite( *outUnitPtr, fdata, sizeof(float)*FDATASIZE ))
           {
            return( 1 );
           }
       }

    return( 0 );
}

/***/
/* fidClose: update header, then close pipe format output.
/***/

int fidClose( outFmt, outUnitPtr, minVal, maxVal, fdata, xyzFlag )

   int   xyzFlag;
   char  *outFmt;
   UNIT  *outUnitPtr;
   float minVal, maxVal, fdata[FDATASIZE];
{
    if (!*outFmt) return( 0 );
    if (!xyzFlag) return( 0 );

    (void) setParm( fdata, FDSCALEFLAG, (float)1.0, 0 );

    (void) setParm( fdata, FDMAX,     maxVal, 0 );
    (void) setParm( fdata, FDMIN,     minVal, 0 );
    (void) setParm( fdata, FDDISPMAX, maxVal, 0 );
    (void) setParm( fdata, FDDISPMIN, minVal, 0 );

    if (dataPos( *outUnitPtr, 0 )) return( 1 );
    if (dataWrite( *outUnitPtr, fdata, sizeof(float)*FDATASIZE )) return( 1 );

    (void) dataClose( *outUnitPtr );

    *outUnitPtr = UNIT_NULL;

    return( 0 );
}

/***/
/* fixBadPts: set points outside of thresh to zero.
/***/

int fixBadPts( vec, length, thresh )

   float *vec, thresh;
   int   length;
{
    int i, count;
    double tm, tp;

    if (thresh == 0.0) return( 0 );

    tm    = -thresh;
    tp    = thresh;
    count = 0;

    for( i = 0; i < length; i++ )
       {
        if (vec[i] < tm || vec[i] > tp)
           {
            vec[i] = 0.0;
            count++;
           }
       }

    return( count );
}
 
/***/
/* bin2il: convert data for two seperated 2D FIDs into one interleaved fid.
/***/

int bin2il( fidR, fidI, xSize, ySize )

   char  *fidR, *fidI;
   int   xSize, ySize;
{
    char  *srcR, *srcI, *dest;
    int   i;

    srcR  = fidR + xSize*ySize - 1;
    srcI  = fidI + xSize*ySize - 1;
    dest  = fidR + xSize*2*ySize - 1;

    while( ySize-- )
       {
        for( i = 0; i < xSize; i++ ) *dest-- = *srcI--;
        for( i = 0; i < xSize; i++ ) *dest-- = *srcR--;
       }

    return( 0 );
} 

/***/
/* bin2fid: convert binary format 2D plane to nmrPipe format plane.
/***/

int bin2fid( binMatC, dataMat, xSize, ySize,
             xQuadState, yQuadState, zQuadState,
             bytePad, riOrder, negFlag, i2rFlag,
             minVal, maxVal )

   int   xSize, ySize, xQuadState, yQuadState, zQuadState, bytePad,
         riOrder, negFlag, i2rFlag;

   float *dataMat, *minVal, *maxVal;
   char  *binMatC;
{
    if (i2rFlag)
       {
        (void) binI2fid( binMatC, 
                         dataMat,
                         xSize, ySize,
                         xQuadState, yQuadState, zQuadState,
                         bytePad, riOrder, negFlag, minVal, maxVal );
       }
    else
       {
        (void) binR2fid( binMatC, 
                         dataMat,
                         xSize, ySize,
                         xQuadState, yQuadState, zQuadState,
                         bytePad, riOrder, negFlag, minVal, maxVal );
       }

    return( 0 );
}

/***/
/* binI2fid: convert integer binary format 2D plane to nmrPipe format plane.
/***/

int binI2fid( binMatC, dataMat, xSize, ySize,
              xQuadState, yQuadState, zQuadState,
              bytePad, riOrder, negFlag, minVal, maxVal )

   int   xSize, ySize, xQuadState, yQuadState, zQuadState, bytePad,
         riOrder, negFlag;

   float *dataMat, *minVal, *maxVal;
   char  *binMatC;
{
    union b2i { int i; char c[4]; } *src;
    float *destR, *destI;
    int   ix, xJump;

    if (xQuadState == 2 && riOrder == RI_INTERLEAVED)
       {
        destR  = dataMat;
        destI  = destR + xSize;
        xJump  = 2*xSize*wordSize + bytePad;
 
        while( ySize-- )
           {
            src = (union b2i *) binMatC;

            for( ix = 0; ix < xSize; ix++ )
               {
                if (src->i > *maxVal) *maxVal = src->i;
                if (src->i < *minVal) *minVal = src->i;

                *destR++ = src->i;
                src++;

                if (negFlag) 
                   *destI++ = -src->i;
                else
                   *destI++ =  src->i;

                src++;
               }

            destR   += xSize; 
            destI   += xSize;
            binMatC += xJump; 
           }
       }
    else
       {
        if (xQuadState == 1)
           {
            xJump = xSize*wordSize + bytePad;

            while( ySize-- )
               {
                src = (union b2i *) binMatC;

                for( ix = 0; ix < xSize; ix++ )
                   {
                    if (src->i > *maxVal) *maxVal = src->i;
                    if (src->i < *minVal) *minVal = src->i;

                    *dataMat++ = src->i;
                    src++;
                   }

                binMatC += xJump;
               }
           }
        else
           {
            xJump = 2*xSize*wordSize + bytePad;

            while( ySize-- )
               {
                src = (union b2i *) binMatC;

                for( ix = 0; ix < xSize; ix++ )
                   {
                    if (src->i > *maxVal) *maxVal = src->i;
                    if (src->i < *minVal) *minVal = src->i;

                    *dataMat++ = src->i;
                    src++;
                   }
                
                for( ix = 0; ix < xSize; ix++ )
                   {
                    if (negFlag) 
                       *dataMat++ = -src->i;
                    else
                       *dataMat++ =  src->i;

                    src++;
                   }

                binMatC += xJump;
               }
           }
       }

    return( 0 );
}

/***/
/* binR2fid: convert float binary format 2D plane to nmrPipe format plane.
/***/

int binR2fid( binMatC, dataMat, xSize, ySize,
              xQuadState, yQuadState, zQuadState,
              bytePad, riOrder, negFlag, minVal, maxVal )

   int   xSize, ySize, xQuadState, yQuadState, zQuadState, bytePad,
         riOrder, negFlag;

   float *dataMat, *minVal, *maxVal;
   char  *binMatC;
{
    float *src, *destR, *destI;
    int   ix, xJump;

    if (xQuadState == 2 && riOrder == RI_INTERLEAVED)
       {
        destR  = dataMat;
        destI  = destR + xSize;
        xJump  = 2*xSize*wordSize + bytePad;
 
        while( ySize-- )
           {
            src = (float *)binMatC;

            for( ix = 0; ix < 2*xSize; ix += 2 )
               {
                if (src[ix] > *maxVal) *maxVal = src[ix];
                if (src[ix] < *minVal) *minVal = src[ix];

                *destR++ = src[ix];

                if (negFlag)
                   *destI++ = -src[ix+1];
                else
                   *destI++ = src[ix+1];
               }

            destR   += xSize; 
            destI   += xSize;
            binMatC += xJump; 
           }
       }
    else
       {
        if (xQuadState == 1)
           {
            xJump  = xSize*wordSize + bytePad;

            while( ySize-- )
               {
                src = (float *)binMatC;

                for( ix = 0; ix < xSize; ix++ )
                   {
                    if (src[ix] > *maxVal) *maxVal = src[ix];
                    if (src[ix] < *minVal) *minVal = src[ix];

                    *dataMat++ = src[ix];
                   }

                binMatC += xJump;
               }
           }
        else
           {
            xJump = 2*xSize*wordSize + bytePad;

            while( ySize-- )
               {
                src = (float *)binMatC;

                for( ix = 0; ix < xSize; ix++ )
                   {
                    if (src[ix] > *maxVal) *maxVal = src[ix];
                    if (src[ix] < *minVal) *minVal = src[ix];

                    *dataMat++ = src[ix];
                   }
                
                for( ix = xSize; ix < 2*xSize; ix++ )
                   {
                    if (negFlag) 
                       *dataMat++ = -src[ix];
                    else
                       *dataMat++ =  src[ix];
                   }

                binMatC += xJump;
               }
           }
       }

    return( 0 );
}

/***/
/* getparams: extract command arguments for bin2pipe.
/***/

int getparams( argc, argv,
               inName, auxName, outFmt, serverListName, fdata,
               firstPlane, lastPlane, byteOffset, bytePad, xyzFlag, riOrder,
               checkFlag, ovFlag, noInputFlag, noOutputFlag,
               swapFlag, negFlag, i2rFlag, padFlag, ilFlag, dirFlag,
               srcType, badThresh, verbose )

   int   argc, *checkFlag, *firstPlane, *lastPlane, *byteOffset, *bytePad,
         *xyzFlag, *ovFlag, *riOrder, *noInputFlag, *noOutputFlag, *swapFlag,
         *negFlag, *i2rFlag, *padFlag, *ilFlag, *dirFlag, *srcType, *verbose;

   char  **argv, *inName, *auxName, *outFmt, *serverListName;
   float *badThresh, fdata[FDATASIZE];
{
    int  loc1, loc2;

/***/
/* Initialize:
/***/

#ifdef SYS_BYTESWAP
    *swapFlag       = 1;
#else
    *swapFlag       = 0;
#endif

    *badThresh      = 0.0;
    *srcType        = SRC_UNKNOWN;
    *riOrder        = RI_SEPERATED;
    *negFlag        = 0;
    *i2rFlag        = 0;
    *padFlag        = 0;
    *ilFlag         = 0;

    *dirFlag        = 1;
    *checkFlag      = 1;
    *firstPlane     = 0;
    *lastPlane      = 0;
    *verbose        = 0;
    *xyzFlag        = 0;
    *ovFlag         = 0;
    *byteOffset     = 0;
    *bytePad        = 0;

    *noInputFlag    = 0;
    *noOutputFlag   = 0;

    *serverListName = '\0';
    *inName         = '\0';
    *auxName        = '\0';
    *outFmt         = '\0';

    if (argc == 1 || flagLoc( argc, argv, "-help" ))
       {
        FPR_VERSION;
        FPR( stderr, "Bin2Pipe: Four-Byte Binary to NMRPipe Conversion;\n" );
        FPR( stderr, " %s hdrFlags < binData > pipeFile\n", argv[0] );
        FPR( stderr, "General Flags:\n" );
        FPR( stderr, " -in  inName           Four-Byte Binary Input.\n" );
        FPR( stderr, " -in  inNameR inNameI  Four-Byte Binary Input Pair.\n" );
        FPR( stderr, " -out outTemplate      NMRPipe Output.\n" );
        FPR( stderr, " -ov                   Force Overwrite.\n" );
        FPR( stderr, " -nodir                Suppress Directory Creation.\n" );
        FPR( stderr, " -nofs                 Suppress File Size Check.\n" );
        FPR( stderr, " -verb                 Verbose Mode On.\n" );
        FPR( stderr, "General Format Flags:\n" );
        FPR( stderr, " -bo bytOff [0] Byte Offset at Start of Input.\n" );
        FPR( stderr, " -bp bytPad [0] Bytes Between 1D Slices in Input.\n" );
        FPR( stderr, " -pad           Pad missing data by zeros.\n" );
        FPR( stderr, " -nopad         Suppress padding by zeros (Default).\n" );
        FPR( stderr, " -il            Interleave Pairs of Input Planes;\n" );
        FPR( stderr, "                (Default if two inputs are given).\n" );
        FPR( stderr, " -noil          No Interleave of Input Planes;\n" );
        FPR( stderr, "                (Default if one input is given).\n" );
        FPR( stderr, " -i2r           Do Int-to-Real Conversion (Also -i2f).\n" );
        FPR( stderr, " -neg           Do Negation of Imaginary Data.\n" );
        FPR( stderr, " -ri            Do Separation of Real/Imag.\n" );
        FPR( stderr, " -bad bPt [0.0] Bad Point Threshold;\n" );
        FPR( stderr, "                (Use 0.0 for No Bad Point Adjust).\n" );
        FPR( stderr, "Byte-Swap Options:\n" );
#ifdef SYS_BYTESWAP
        FPR( stderr, " -swap             Do Four-Byte Swap (Default).\n" );
        FPR( stderr, " -noswap           Suppress Four-Byte Swap.\n");
        FPR( stderr, " -aswap            Suppress Four-Byte Swap.\n");
        FPR( stderr, "                   (Meaning is System-dependant)\n" );
        FPR( stderr, " -noaswap          Do Four-Byte Swap.\n" );
        FPR( stderr, "                   (Meaning is System-dependant)\n" );
#else
        FPR( stderr, " -swap             Do Four-Byte Swap.\n" );
        FPR( stderr, " -noswap           Suppress Four-Byte Swap (Default).\n");
        FPR( stderr, " -aswap            Do Four-Byte Swap.\n" );
        FPR( stderr, "                   (Meaning is System-dependant)\n" );
        FPR( stderr, " -noaswap          Suppress Four-Byte Swap.\n");
        FPR( stderr, "                   (Meaning is System-dependant)\n" );
#endif
        FPR( stderr, "Specific Format Flags:\n" );
        FPR( stderr, " -alpha    Same As: -bo 8704  -i2r -ri -neg -aswap\n" );
        FPR( stderr, " -alpha2   Same As: -bo 16384 -i2r -ri -neg -aswap\n" );
        FPR( stderr, " -lambda   Same As: -bo -1 -ri -neg -aswap\n" );
        FPR( stderr, " -gsx      Same As: -ri -i2r -aswap\n" );
        FPR( stderr, " -ex       Same As: -ri -i2r -aswap\n" );
        FPR( stderr, " -chem     Same As: -i2r -aswap\n" );
        FPR( stderr, " -ge       Same As: -ri -noaswap\n" );
        FPR( stderr, " -ftOLD    Same As: -bo 8 -bp 12 -ri -pad -noaswap\n" );
        FPR( stderr, "           (Required byte offset may vary)\n" );
        FPR( stderr, " -ftNEW    Same As: -bo 1036 -bp 4 -ri -pad -noaswap\n" );
        FPR( stderr, "           (Required byte offset may vary)\n" );
        FPR( stderr, "Null Data Flags:\n" );
        FPR( stderr, " -noIn      No Input Data; Use Zeros.\n" );
        FPR( stderr, " -noOut     No Output Data; Header Only.\n" );
        FPR( stderr, "Parallel Processing: (Not yet implemented)\n" );
        FPR( stderr, " -par   sListName  Server List File Name.\n" );
        FPR( stderr, " -first firstPlane First Plane to process.\n");
        FPR( stderr, " -last  lastPlane  Last Plane to process.\n");
        FPR( stderr, "Notes:\n" );
        FPR( stderr, " 1. JEOL GSX Phase-Sensitive input is specified\n" );
        FPR( stderr, "    as a pair of file names.\n" );
        FPR( stderr, " 2. JEOL LAMBDA mode (-lambda) will attempt to\n" );
        FPR( stderr, "    find the byte offset from the JEOL header.\n" );
        FPR( stderr, " 3. By default, byte-swap is ON for Pentium systems,\n");
        FPR( stderr, "    and OFF for other platforms.\n");
        FPR( stderr, "\n" );

        (void) parseHdr( argc, argv, fdata );
        return( 1 );
       }

/***/
/* Extract format-specific parameters:
/***/

    if (flagLoc( argc, argv, "-ge" ))
       {
        *srcType    = SRC_GE;
        *riOrder    = RI_INTERLEAVED;
#ifdef SYS_BYTESWAP
        *swapFlag   = 1;
#else
        *swapFlag   = 0;
#endif
       }

    if (flagLoc( argc, argv, "-alpha" ))
       {
        *srcType    = SRC_ALPHA;
        *riOrder    = RI_INTERLEAVED;
        *byteOffset = 8704;
        *bytePad    = 0;
        *i2rFlag    = 1;
        *negFlag    = 1;
#ifdef SYS_BYTESWAP
        *swapFlag   = 0;
#else
        *swapFlag   = 1;
#endif
       }

    if (flagLoc( argc, argv, "-alpha2" ))
       {
        *srcType    = SRC_ALPHA;
        *riOrder    = RI_INTERLEAVED;
        *byteOffset = 16384;
        *bytePad    = 0;
        *i2rFlag    = 1;
        *negFlag    = 1;
#ifdef SYS_BYTESWAP
        *swapFlag   = 0;
#else
        *swapFlag   = 1;
#endif
       }

    if (flagLoc( argc, argv, "-varian" ))
       {
        *srcType    = SRC_VARIAN;
        *riOrder    = RI_INTERLEAVED;
        *byteOffset = 32 + 28;
        *bytePad    = 28;
        *i2rFlag    = 1;
#ifdef SYS_BYTESWAP
        *swapFlag   = 1;
#else
        *swapFlag   = 0;
#endif
       }

    if (flagLoc( argc, argv, "-lambda" ))
       {
        *srcType    = SRC_LAMBDA;
        *riOrder    = RI_INTERLEAVED;
        *byteOffset = -1;
        *bytePad    = 0;
        *negFlag    = 1;
#ifdef SYS_BYTESWAP
        *swapFlag   = 0;
#else
        *swapFlag   = 1;
#endif
       }

    if (flagLoc( argc, argv, "-gsx" ))
       {
        *srcType    = SRC_GSX;
        *riOrder    = RI_INTERLEAVED;
        *i2rFlag    = 1;
#ifdef SYS_BYTESWAP
        *swapFlag   = 0;
#else
        *swapFlag   = 1;
#endif
       }

    if (flagLoc( argc, argv, "-ex" ))
       {
        *srcType    = SRC_EX;
        *riOrder    = RI_INTERLEAVED;
        *i2rFlag    = 1;
#ifdef SYS_BYTESWAP
        *swapFlag   = 0;
#else
        *swapFlag   = 1;
#endif
       }
 
    if (flagLoc( argc, argv, "-chem" ))
       {
        *srcType    = SRC_CHEM;
        *i2rFlag    = 1;
#ifdef SYS_BYTESWAP
        *swapFlag   = 0;
#else
        *swapFlag   = 1;
#endif
       }

    if (flagLoc( argc, argv, "-ftNEW" ))
       {
        *srcType    = SRC_FTNEW;
        *riOrder    = RI_INTERLEAVED;
        *byteOffset = 1036;
        *bytePad    = 4;
        *padFlag    = 1;
#ifdef SYS_BYTESWAP
        *swapFlag   = 1;
#else
        *swapFlag   = 0;
#endif
       }

    if (flagLoc( argc, argv, "-ftOLD" ))
       {
        *srcType    = SRC_FTOLD;
        *riOrder    = RI_INTERLEAVED;
        *byteOffset = 8;
        *bytePad    = 12;
        *padFlag    = 1;
#ifdef SYS_BYTESWAP
        *swapFlag   = 1;
#else
        *swapFlag   = 0;
#endif
       }

/***/
/* Extract input name or real/imag input pair:
/***/

    if (loc1 = flagLoc( argc, argv, "-in" ))
       {
        loc2 = nextFlag( argc, argv, loc1 );

        if (loc2 - loc1 > 2)
           {
            (void) strcpy( inName,  getNthArg( argc, argv, loc1+1 ));
            (void) strcpy( auxName, getNthArg( argc, argv, loc1+2 ));

            *ilFlag = 1;
           }
        else if (loc2 - loc1 > 1)
           {
            (void) strcpy( inName, getNthArg( argc, argv, loc1+1 ));
           } 
       }

/***/
/* Extract general flags:
/***/

    (void) strArgD( argc, argv, "-out",   outFmt );
    (void) strArgD( argc, argv, "-par",   serverListName );
    (void) intArgD( argc, argv, "-first", firstPlane );
    (void) intArgD( argc, argv, "-last",  lastPlane );
    (void) intArgD( argc, argv, "-bo",    byteOffset );
    (void) intArgD( argc, argv, "-bp",    bytePad );
    (void) fltArgD( argc, argv, "-bad",   badThresh );

    if (*badThresh < 0.0) *badThresh = -*badThresh;


    if (flagLoc( argc, argv, "-ri" ))      *riOrder   = RI_INTERLEAVED;

#ifdef SYS_BYTESWAP
    if (flagLoc( argc, argv, "-aswap" ))   *swapFlag     = 0;
    if (flagLoc( argc, argv, "-noaswap" )) *swapFlag     = 1;
#else
    if (flagLoc( argc, argv, "-aswap" ))   *swapFlag     = 1;
    if (flagLoc( argc, argv, "-noaswap" )) *swapFlag     = 0;
#endif

    if (flagLoc( argc, argv, "-swap" ))    *swapFlag     = 1;
    if (flagLoc( argc, argv, "-noswap" ))  *swapFlag     = 0;

    if (flagLoc( argc, argv, "-il" ))      *ilFlag       = 1;
    if (flagLoc( argc, argv, "-noil" ))    *ilFlag       = 0;
    if (flagLoc( argc, argv, "-pad" ))     *padFlag      = 1;
    if (flagLoc( argc, argv, "-nopad" ))   *padFlag      = 0;
    if (flagLoc( argc, argv, "-neg" ))     *negFlag      = 1;
    if (flagLoc( argc, argv, "-i2r" ))     *i2rFlag      = 1;
    if (flagLoc( argc, argv, "-i2f" ))     *i2rFlag      = 1;
    if (flagLoc( argc, argv, "-noIn" ))    *noInputFlag  = 1;
    if (flagLoc( argc, argv, "-noOut" ))   *noOutputFlag = 1;

    if (flagLoc( argc, argv, "-nodir" ))   *dirFlag      = 0;
    if (flagLoc( argc, argv, "-nofs" ))    *checkFlag    = 0;
    if (flagLoc( argc, argv, "-ov" ))      *ovFlag       = 1;
    if (flagLoc( argc, argv, "-verb" ))    *verbose      = 1;

    if (*noInputFlag) *ilFlag = 0;

    if (*i2rFlag)     wordSize = sizeof(int);

/***/
/* Test for required output format with xyz style output:
/***/

    if (isZFmt( outFmt ) || isAZFmt( outFmt )) *xyzFlag = 1;

    if (*xyzFlag && !*outFmt)
       {
        FPR( stderr, "Bin2Pipe Error: missing output format.\n" );
        FPR( stderr, " 2D Series Output requires -out template, e.g.:\n" );
        FPR( stderr, " %s -out test%%03d.fid < DATA\n", argv[0] );
        return( 1 );
       }

/***/
/* Set some default parameters:
/***/

    (void) txt2Flt( "X", fdata + FDF2LABEL, SIZE_F2LABEL );
    (void) txt2Flt( "Y", fdata + FDF1LABEL, SIZE_F1LABEL );
    (void) txt2Flt( "Z", fdata + FDF3LABEL, SIZE_F3LABEL );
    (void) txt2Flt( "A", fdata + FDF4LABEL, SIZE_F4LABEL );

/***/
/* Parse header parameters:
/***/

    if (parseHdr( argc, argv, fdata )) return( 1 );

    return( 0 );
}

/***/
/* Bottom.
/***/
