/******************************************************************************r
/*                                                                            */
/*                   ---- NIH NMR Software System ----                        */
/*                         Copyright 1992 - 1994                              */
/*                             Frank Delaglio                                 */
/*                   NIH Laboratory of Chemical Physics                       */
/*                                                                            */
/*               This software is not for distribution without                */
/*                  the written permission of the author.                     */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <string.h>

#include "cmndargs.h"
#include "testsize.h"
#include "memory.h"
#include "dataio.h"
#include "prec.h"
#include "bswap.h"

#define FPR (void) fprintf

#define CONV_NONE    0
#define CONV_UCHAR   1
#define CONV_CHAR    2
#define CONV_USHORT  3
#define CONV_SHORT   4
#define CONV_INT     5
#define CONV_DBL     6
#define CONV_FLT     7
#define CONV_FSHORT  8 

/***/
/* byteAdjust: adjust word size of data stream.
/***/

int main( argc, argv )

   int  argc;
   char **argv;
{
    double dFreeBytes;

    int    wordCount, byteCount, buffSize, wordsIn, bytesLeft, bytesDone, 
           wordSizeIn, wordSizeOut, byteOffset, bytePad, signBit, scale, 
           convertMode, loFlag, swapFlag, autoFlag, checkFlag, verbose,
           error;

    char   *inBuff, *outBuff, inName[NAMELEN+1], outName[NAMELEN+1];
    struct FileSize freeSizeInfo, fileSizeInfo;
    UNIT   inUnit, outUnit;

/***/
/* Initialize:
/***/

    error       = 0;
    inBuff      = (char *)NULL;
    outBuff     = (char *)NULL;
    inUnit      = UNIT_NULL;
    outUnit     = UNIT_NULL;
    autoFlag    = 0;

    inName[0]   = '\0';
    outName[0]  = '\0';

/***/
/* Extract command line.
/* Open files as needed.
/***/

    (void) initArgs( argc, argv, CHECK_ENV );

    error = getparams( argc, argv,
                       inName, outName,
                       &buffSize, &byteOffset, &bytePad, &wordCount,
                       &wordSizeIn, &wordSizeOut, &signBit, &scale,
                       &convertMode, &loFlag, &swapFlag, &checkFlag, &verbose );

    if (error)
       {
        FPR( stderr, "ByteAdjust Error extracting arguments.\n" );
        goto shutdown;
       }

    (void) checkArgs( argc, argv );

    if (*inName)
       {
        if (dataOpen( inName, &inUnit, FB_READ ))
           {
            FPR( stderr, "ByteAdjust Error opening input file %s\n", inName );
            goto shutdown;
           }
       }
    else
       {
        inUnit = FB_STDIN;
       }

    if (*outName)
       {
        if (dataOpen( outName, &outUnit, FB_UNKNOWN ))
           {
            FPR( stderr, "ByteAdjust Error opening output file %s\n", outName );
            goto shutdown;
           }
       }
    else
       {
        outUnit = FB_STDOUT;
       }

/***/
/* Allocate buffers.
/* Determine input file size if needed.
/* Check for sufficient output space if needed.
/***/

    if (!(inBuff = charAlloc( "adj", buffSize*wordSizeIn )))
       {
        FPR( stderr, "ByteAdjust Error allocating input buffer.\n" );
        goto shutdown;
       }

    if (!(outBuff = charAlloc( "adj", buffSize*wordSizeOut )))
       {
        FPR( stderr, "ByteAdjust Error allocating output buffer.\n" );
        goto shutdown;
       }

    if (!wordCount)
       {
        autoFlag  = 1;
        (void) getFileBytes( inName, &fileSizeInfo );

        if (fileSizeInfo.status == FILE_STATUS_FAIL)
           {
            FPR( stderr, "ByteAdjust Error determining input byte count.\n" );
            error = 1;
            goto shutdown;
           }

        if (fileSizeInfo.status == FILE_STATUS_INT_TOO_SMALL)
           {
            FPR( stderr, 
                 "ByteAdjust Error: file %s is too large: %.0lf bytes.\n", inName,
                 fileSizeInfo.dTotalBytes );

            error = 1;
            goto shutdown;
           }

        byteCount = fileSizeInfo.iTotalBytes;
        byteCount = byteOffset > byteCount ? 0 : byteCount - byteOffset;
        wordCount = byteCount/wordSizeIn;

        if (!bytePad && (wordCount*wordSizeIn != byteCount))
           {
            FPR( stderr, "ByteAdjust Warning: the input byte count is\n" );
            FPR( stderr, "not a multiple of the input word size.\n" );

            FPR( stderr,
                 "Byte Count: %d Word Size: %d\n",
                 byteCount, wordSizeIn );
           }
       }

    if (checkFlag && getFSCheck())
       {
        dFreeBytes = getFreeBytes( outName, &freeSizeInfo );

        if (dFreeBytes < 0.0)
           {
            FPR( stderr, "ByteAdjust Warning: can't determine available disk space.\n");
           }
        else if ((double)wordCount*wordSizeOut > dFreeBytes)
           {
            FPR( stderr, "ByteAdjust Error: insufficient disk space.\n" );
            error = 1;
            goto shutdown;
           }
       }

/***/
/* Skip past the byte offset as needed.
/*
/* Perform the conversion:
/* While words are left to convert:
/*  Determine number of words to read.
/*  Read the words.
/*  Convert the words.
/*  Write the words.
/***/

    if (*inName)
       {
        if (byteOffset)
           {
            if (verbose) FPR( stderr, "Skipping %d bytes.\n", byteOffset );

            if (error = dataPos( inUnit, byteOffset ))
               {
                FPR( stderr, "ByteAdjust Error positioning input file.\n" );
                goto shutdown;
               } 
           }
       }
    else
       {
        if (byteOffset)
           {
            if (verbose) FPR( stderr, "Skipping %d bytes.\n", byteOffset );

            error = dataPos2( inUnit, 
                              byteOffset,
                              (VOID *)inBuff,
                              buffSize*wordSizeIn );

            if (error)
               {
                FPR( stderr, "ByteAdjust Error positioning input stream.\n" );
                goto shutdown;
               }
           }
       }

    byteCount = wordSizeIn*wordCount;
    bytesLeft = byteCount;
    bytesDone = 0;

    while( bytesLeft > bytePad )
       {
        wordsIn = bytesLeft/wordSizeIn;
        wordsIn = wordsIn > buffSize ? buffSize : wordsIn;
       
        if (*inName)
           {
            if (error = dataReadC( inUnit, inBuff, wordsIn*wordSizeIn ))
               {
                FPR( stderr, "ByteAdjust Error reading input file.\n" );
                goto shutdown;
               } 
           }
        else
           {
            if (error = dataReadCB( inUnit, inBuff, wordsIn*wordSizeIn, -1, 0 ))
               {
                FPR( stderr, "ByteAdjust Error reading input stream.\n" );
                goto shutdown;
               }
           }

        (void) adjustWord( inBuff,  wordSizeIn,
                           outBuff, wordSizeOut,
                           wordsIn, signBit, scale,
                           convertMode, loFlag, swapFlag );

        if (error = dataWriteC( outUnit, outBuff, wordsIn*wordSizeOut ))
           {
            FPR( stderr, "ByteAdjust Error writing output.\n" );
            goto shutdown;
           }

        bytesDone += wordsIn*wordSizeIn;
        bytesLeft -= wordsIn*wordSizeIn;

        if (bytePad && (bytesLeft >= bytePad))
           {
            if (*inName)
               {
                if (error = dataReadC( inUnit, inBuff, bytePad ))
                   {
                    FPR( stderr, "ByteAdjust Error reading input file.\n" );
                    goto shutdown;
                   }
               }
            else
               {
                if (error = dataReadCB( inUnit, inBuff, bytePad, -1, 0 ))
                   {
                    FPR( stderr, "ByteAdjust Error reading input stream.\n" );
                    goto shutdown;
                   }
               }

            if (autoFlag)
               {
                bytesLeft -= bytePad;
                bytesDone += bytePad;
               }
           } 

        if (verbose)
           {
            FPR( stderr, "%d of %d Input Bytes\015", bytesDone, byteCount );
            (void) fflush( stderr );
           }
       }

    if (verbose)
       {
        FPR( stderr, "\n" );
        (void) fflush( stderr );
       }

/***/
/* Exit point:
/*   Deallocate buffers.
/*   Close files as needed.
/***/

shutdown:

    if (*inName)  (void) dataClose( inUnit );
    if (*outName) (void) dataClose( outUnit );

    (void) deAlloc( "adj", inBuff,  buffSize*wordSizeIn );
    (void) deAlloc( "adj", outBuff, buffSize*wordSizeOut );

    return( error );
}

/***/
/* adjustWord: adjust size of words in input buffer.
/***/

int adjustWord( inBuff,  wordSizeIn,
                outBuff, wordSizeOut,
                wordCount, signBit, scale,
                convertMode, loFlag, swapFlag )

   char *inBuff, *outBuff;
   int  wordSizeIn, wordSizeOut, wordCount, signBit, scale, convertMode, loFlag;
{
    int    i, j, jump;
    char   *src, *dest;

    unsigned char  *uPtr;
    char           *cPtr;
    unsigned short *qPtr;
    short          *tPtr;
    int            *iPtr;
    float          *fPtr;
    double         *dPtr;

/***/
/* Start with empty output buffer.
/* Swap input as needed.
/***/

    dest = outBuff;

    for( i = 0; i < wordCount*wordSizeOut; i++ ) *dest++ = 0;

    if (swapFlag) (void) byteSwapN( inBuff, wordCount*wordSizeIn, wordSizeIn );
 
/***/
/* If there is a particular conversion, perform it.
/***/

    if (scale == 0) scale = 1;

    switch( convertMode )
       {
        case CONV_UCHAR:
           fPtr = (float *)outBuff;
           uPtr = (unsigned char *)inBuff;

           for( i = 0; i < wordCount; i++ )
              {
               fPtr[i]  = uPtr[i];
               fPtr[i] /= scale;
              }

           return( 0 );
           break;

        case CONV_CHAR:
           fPtr = (float *)outBuff;
           cPtr = (char *)inBuff;

           for( i = 0; i < wordCount; i++ ) 
              {
               fPtr[i]  = cPtr[i];
               fPtr[i] /= scale;
              }

           return( 0 );
           break;

        case CONV_USHORT:
           fPtr = (float *)outBuff;
           qPtr = (unsigned short *)inBuff;

           for( i = 0; i < wordCount; i++ ) 
              {
               fPtr[i]  = qPtr[i];
               fPtr[i] /= scale;
              }

           return( 0 );
           break;

        case CONV_SHORT:
           fPtr = (float *)outBuff;
           tPtr = (short *)inBuff;

           for( i = 0; i < wordCount; i++ )
              {
               fPtr[i]  = tPtr[i];
               fPtr[i] /= scale;
              }

           return( 0 );
           break;

        case CONV_INT:
           fPtr = (float *)outBuff;
           iPtr = (int *)inBuff;

           for( i = 0; i < wordCount; i++ )
              {
               fPtr[i]  = iPtr[i];
               fPtr[i] /= scale;
              }

           return( 0 );
           break;

        case CONV_DBL: 
           fPtr = (float *)outBuff;
           dPtr = (double *)inBuff;

           for( i = 0; i < wordCount; i++ ) 
              {
               fPtr[i] = dPtr[i]/scale;
              }

           return( 0 );
           break;

        case CONV_FLT:
           fPtr = (float *)inBuff;
           iPtr = (int *)outBuff;

           for( i = 0; i < wordCount; i++ )
              {
               iPtr[i] = fPtr[i]/scale;
              }

           return( 0 );
           break;

        case CONV_FSHORT:
           fPtr = (float *)inBuff;
           tPtr = (short *)outBuff;

           for( i = 0; i < wordCount; i++ )
              {
               tPtr[i] = fPtr[i]/scale;
              }

           return( 0 );
           break;

        case CONV_NONE:
        default:
           break;
       }

/***/
/* If no word length adjustment is required:
/*  Copy input to output.
/*
/* If using low-order bytes:
/*  If compression is required use first bytes of each input word.
/*  If expansion is required, copy input word to first bytes of output.
/*
/* If using high-order bytes:
/*  If compression is required use last bytes of each input word.
/*  If expansion is required, copy input word to last bytes of output.
/***/

    src  = inBuff;
    dest = outBuff;

    if (wordSizeIn > wordSizeOut)
       jump = wordSizeIn - wordSizeOut;
    else
       jump = wordSizeOut - wordSizeIn;

    if (wordSizeIn == wordSizeOut)
       {
        for( j = 0; j < wordCount*wordSizeOut; j++ ) *dest++ = *src++; 
       }
    else if (loFlag)
       {
        if (wordSizeIn > wordSizeOut)
           {
            for( i = 0; i < wordCount; i++ )
               {
                for( j = 0; j < wordSizeOut; j++ ) *dest++ = *src++; 
                src += jump;
               }
           }
        else
           {
            for( i = 0; i < wordCount; i++ )
               {
                for( j = 0; j < wordSizeIn; j++ ) *dest++ = *src++;
                dest += jump;
               }
           }
       }
    else
       {
        if (wordSizeIn > wordSizeOut)
           {
            for( i = 0; i < wordCount; i++ )
               {
                src += jump;
                for( j = 0; j < wordSizeOut; j++ ) *dest++ = *src++;
               }
           }
        else
           {
            for( i = 0; i < wordCount; i++ )
               {
                dest += jump;
                for( j = 0; j < wordSizeIn; j++ ) *dest++ = *src++;
               }
           }
       }

/***/
/* If integer scaling is allowed, perform it:
/***/

    if (wordSizeOut == 4 && scale != 1)
       {
        iPtr = (int *)outBuff;

        for( i = 0; i < wordCount; i++ ) *iPtr++ /= scale;
       }

    return( 0 );
}

/***/
/* getparams: extract command-line arguments for byteAdjust.
/***/

int getparams( argc, argv,
               inName, outName,
               buffSize, byteOffset, bytePad, wordCount,
               wordSizeIn, wordSizeOut, signBit, scale,
               convertMode, loFlag, swapFlag, checkFlag, verbose )

   int  argc, *buffSize, *byteOffset, *bytePad, *wordCount, *wordSizeIn,
        *wordSizeOut, *signBit, *scale, *convertMode, *loFlag, *swapFlag, 
        *checkFlag, *verbose;

   char **argv, *inName, *outName;
{

/***/
/* Initialize:
/***/

    *inName      = '\0';
    *outName     = '\0';
    *buffSize    = 4096;
    *byteOffset  = 0;
    *bytePad     = 0;
    *wordCount   = 0;
    *wordSizeIn  = 2;
    *wordSizeOut = 4;
    *signBit     = -1;
    *scale       = 1;
    *loFlag      = 1;
    *swapFlag    = 0;
    *checkFlag   = 1;
    *verbose     = 0;
    *convertMode = CONV_NONE;

/***/
/* Print help text:
/***/

  if (flagLoc( argc, argv, "-help" ))
     {
      FPR( stderr, "ByteAdjust: adjust length of words in stream:\n" ); 
      FPR( stderr, " %s < inStream > outStream\n", argv[0] );
      FPR( stderr, "Options:\n" );
      FPR( stderr, " -in  inName  [stdin]  Input name of data to adjust.\n" );
      FPR( stderr, " -out outName [stdout] Output name for result.\n" ); 
      FPR( stderr, " -bs  bSize   [4096]   Buffer Size, Words.\n" );
      FPR( stderr, " -iws wSize   [2]      Input Word Size, Bytes.\n" );
      FPR( stderr, " -ows wSize   [4]      Output Word Size, Bytes.\n" );
      FPR( stderr, " -bo  bOffset [0]      Byte Offset for Adjustment.\n" );
      FPR( stderr, " -bp  bPad    [0]      Byte Pad Between Buffers.\n" );
      FPR( stderr, " -wc  wCount  [0]      Total Words to Adjust (Use 0 for Auto Mode).\n" );
      FPR( stderr, " -is  iScale  [1]      Integer Scale Factor (Use 0 for Auto with 4 Byte Data).\n" );
      FPR( stderr, " -last                 Pack Using Last Bytes of Word.\n" );
      FPR( stderr, " -nofs                 Suppress File Size Check.\n" );
      FPR( stderr, " -verb                 Verbose Mode On.\n" );
      FPR( stderr, "Special Float Conversions:\n" );
      FPR( stderr, " -u2f                  Unsigned Char to Float.\n" );
      FPR( stderr, " -c2f                  Char to Float.\n" );
      FPR( stderr, " -s2f                  Short to Float.\n" );
      FPR( stderr, " -t2f                  Unsigned Short to Float.\n" );
      FPR( stderr, " -i2f                  Int to Float.\n" );
      FPR( stderr, " -d2f                  Double to Float.\n" );
      FPR( stderr, " -f2i                  Float to Int.\n" );
      FPR( stderr, " -f2s                  Float to Short.\n" );
      FPR( stderr, "Byte-Swap Options:\n" );
#ifdef SYS_BYTESWAP
      FPR( stderr, " -swap             Do Four-Byte Swap.\n" );
      FPR( stderr, " -noswap           Suppress Four-Byte Swap.\n");
      FPR( stderr, " -aswap            Suppress Four-Byte Swap. (Meaning is System-dependant).\n" );
      FPR( stderr, " -noaswap          Do Four-Byte Swap. (Meaning is System-dependant).\n" );
#else
      FPR( stderr, " -swap             Do Four-Byte Swap.\n" );
      FPR( stderr, " -noswap           Suppress Four-Byte Swap.\n");
      FPR( stderr, " -aswap            Do Four-Byte Swap (Meaning is System-dependant).\n" );
      FPR( stderr, " -noaswap          Suppress Four-Byte Swap (Meaning is System-dependant).\n" );
#endif
      FPR( stderr, "Notes:\n" );
      FPR( stderr, " 1. In-place processing (same input and output) is allowed if options\n" );
      FPR( stderr, "    -in and -out are both used.\n" );
      FPR( stderr, " 2. Auto Byte-Count determination requires use of -in option.\n" );
      FPR( stderr, " 3. A scale factor of -is 65536 to may be required to convert 2-byte\n" );
      FPR( stderr, "    integers to 4-byte data.\n" );
      return( 1 );
     }

/***/
/* Extract arguments:
/***/

    (void) strArgD( argc, argv, "-in",  inName ); 
    (void) strArgD( argc, argv, "-out", outName ); 

    if (flagLoc( argc, argv, "-u2f" ))
       {
        *convertMode   = CONV_UCHAR;
        *wordSizeIn    = sizeof(unsigned char);
        *wordSizeOut   = sizeof(float);
       }

    if (flagLoc( argc, argv, "-c2f" ))
       {
        *convertMode   = CONV_CHAR;
        *wordSizeIn    = sizeof(char);
        *wordSizeOut   = sizeof(float);
       }

    if (flagLoc( argc, argv, "-s2f" ))
       {
        *convertMode   = CONV_SHORT;
        *wordSizeIn    = sizeof(short);
        *wordSizeOut   = sizeof(float);
       }

    if (flagLoc( argc, argv, "-t2f" ))
       {
        *convertMode   = CONV_USHORT;
        *wordSizeIn    = sizeof(unsigned short);
        *wordSizeOut   = sizeof(float);
       }

    if (flagLoc( argc, argv, "-i2f" ))
       {
        *convertMode   = CONV_INT;
        *wordSizeIn    = sizeof(int);
        *wordSizeOut   = sizeof(float);
       } 

    if (flagLoc( argc, argv, "-d2f" ))
       {
        *convertMode   = CONV_DBL;
        *wordSizeIn    = sizeof(double);
        *wordSizeOut   = sizeof(float);
       }

   if (flagLoc( argc, argv, "-f2i" ))
       {
        *convertMode   = CONV_FLT;
        *wordSizeIn    = sizeof(float);
        *wordSizeOut   = sizeof(int);
       }

   if (flagLoc( argc, argv, "-f2s" ))
       {
        *convertMode   = CONV_FSHORT;
        *wordSizeIn    = sizeof(float);
        *wordSizeOut   = sizeof(short);
       }

    (void) intArgD( argc, argv, "-bs",  buffSize ); 
    (void) intArgD( argc, argv, "-iws", wordSizeIn ); 
    (void) intArgD( argc, argv, "-ows", wordSizeOut );
    (void) intArgD( argc, argv, "-wc",  wordCount ); 
    (void) intArgD( argc, argv, "-bo",  byteOffset ); 
    (void) intArgD( argc, argv, "-bp",  bytePad ); 

    (void) intArgD( argc, argv, "-sb",  signBit ); 
    (void) intArgD( argc, argv, "-is",  scale ); 

#ifdef SYS_BYTESWAP
    if (flagLoc( argc, argv, "-aswap" ))   *swapFlag     = 0;
    if (flagLoc( argc, argv, "-noaswap" )) *swapFlag     = 1;

    if (*scale == 0) *scale = 1;
#else
    if (flagLoc( argc, argv, "-aswap" ))   *swapFlag     = 1;
    if (flagLoc( argc, argv, "-noaswap" )) *swapFlag     = 0;

    if (*scale == 0) *scale = 65536;
#endif

    if (flagLoc( argc, argv, "-swap" ))    *swapFlag     = 1;
    if (flagLoc( argc, argv, "-noswap" ))  *swapFlag     = 0;

    if (flagLoc( argc, argv, "-last" ))   *loFlag    = 0;
    if (flagLoc( argc, argv, "-nofs" ))   *checkFlag = 0;
    if (flagLoc( argc, argv, "-verb" ))   *verbose   = 1;

/***/
/* Adjust modes not consistant with stream input/output:
/***/

    if (!*outName)
       {
        *checkFlag = 0;
       }

    if (*inName && *outName && !strcmp( inName, outName ))
       {
        *checkFlag = 0;
       }

    if (!*wordCount && !*inName)
       {
        FPR( stderr, "ByteAdjust Error: auto word-count requires -in inName.\n" );
        return( 1 );
       }

    return( 0 );
}

/***/
/* Bottom.
/***/
