/******************************************************************************/
/*                                                                            */
/*                   ---- NIH NMR Software System ----                        */
/*                         Copyright 1992 - 1994                              */
/*                             Frank Delaglio                                 */
/*                   NIH Laboratory of Chemical Physics                       */
/*                                                                            */
/*               This software is not for distribution without                */
/*                  the written permission of the author.                     */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <string.h>

#include "cmndargs.h"
#include "testsize.h"
#include "memory.h"
#include "dataio.h"
#include "prec.h"
#include "bswap.h"

#define FPR (void) fprintf

/***/
/* byteSwap: perform N-byte swap on data stream.
/***/

int main( argc, argv )

   int  argc;
   char **argv;
{
    double dFreeBytes;

    int    byteCount, buffSize, bytesIn, bytesLeft, bytesDone, swapCount, 
           checkFlag, verbose, error;

    char   *buff, inName[NAMELEN+1], outName[NAMELEN+1];
    struct FileSize freeSizeInfo, fileSizeInfo;
    UNIT   inUnit, outUnit;

/***/
/* Initialize:
/***/

    error   = 0;
    buff    = (char *)NULL;
    inUnit  = UNIT_NULL;
    outUnit = UNIT_NULL;

/***/
/* Extract command line.
/* Open files as needed.
/***/

    (void) initArgs( argc, argv, CHECK_ENV );
    (void) initDataIO();

    (void) setAutoSwapFlag( 0 );
    (void) setByteSwapFlag( 0 );

    error = getparams( argc, argv,
                       inName, outName,
                       &buffSize, &byteCount, &swapCount,
                       &checkFlag, &verbose );

    if (error)
       {
        FPR( stderr, "ByteSwap Error extracting arguments.\n" );
        goto shutdown;
       }

    (void) checkArgs( argc, argv );

    if (*inName)
       {
        if (dataOpen( inName, &inUnit, FB_READ ))
           {
            FPR( stderr, "ByteSwap Error opening input file %s\n", inName );
            goto shutdown;
           }
       }
    else
       {
        inUnit = FB_STDIN;
       }

    if (*outName)
       {
        if (dataOpen( outName, &outUnit, FB_UNKNOWN ))
           {
            FPR( stderr, "ByteSwap Error opening output file %s\n", outName );
            goto shutdown;
           }
       }
    else
       {
        outUnit = FB_STDOUT;
       }

/***/
/* Allocate buffer.
/* Determine input file size if needed.
/* Check for sufficient output space if needed.
/***/

    if (!(buff = charAlloc( "swap", buffSize )))
       {
        FPR( stderr, "ByteSwap Error allocating buffer.\n" );
        goto shutdown;
       }

    if (!byteCount)
       {
        (void) getFileBytes( inName, &fileSizeInfo );

        if (fileSizeInfo.status == FILE_STATUS_FAIL)
           {
            FPR( stderr, "ByteSwap Error determining input byte count.\n" );
            error = 1;
            goto shutdown;
           }

        if (fileSizeInfo.status == FILE_STATUS_INT_TOO_SMALL)
           {
            FPR( stderr,
                 "ByteSwap Error: file %s is too large: %.0lf bytes.\n", inName,
                 fileSizeInfo.dTotalBytes );

            error = 1;
            goto shutdown;
           }

        byteCount = fileSizeInfo.iTotalBytes;
       }

    if (checkFlag && getFSCheck())
       {
        dFreeBytes = getFreeBytes( outName, &freeSizeInfo );

        if (dFreeBytes < 0)
           {
            FPR( stderr, "ByteSwap Warning: can't determine available disk space.\n" );
           }
        else if (byteCount > dFreeBytes)
           {
            FPR( stderr, "ByteSwap Error: insufficient disk space.\n" );
            error = 1;
            goto shutdown;
           }
       }

/***/
/* Perform the conversion:
/* While bytes are left to convert:
/*  Determine number of bytes to read.
/*  Read the bytes.
/*  Convert the bytes.
/*  Write the bytes.
/***/

    bytesDone = 0;

    for( bytesLeft = byteCount; bytesLeft > 0; bytesLeft -= bytesIn )
       {
        bytesIn    = bytesLeft <= buffSize ? bytesLeft : buffSize;
        bytesDone += bytesIn;
       
        if (verbose)
           {
            FPR( stderr, "%d of %d Bytes\015", bytesDone, byteCount );
            (void) fflush( stderr );
           }
 
        if (*inName)
           {
            if (error = dataRead( inUnit, buff, bytesIn ))
               {
                FPR( stderr, "ByteSwap Error reading input file.\n" );
                goto shutdown;
               } 
           }
        else
           {
            if (error = dataReadB( inUnit, buff, bytesIn, -1, 0 ))
               {
                FPR( stderr, "ByteSwap Error reading input stream.\n" );
                goto shutdown;
               }
           }

        if (swapCount == 3)
           (void) byteSwap3( buff, bytesIn );
        else if (swapCount == 4) 
           (void) byteSwap4( buff, bytesIn );
        else
           (void) byteSwapN( buff, bytesIn, swapCount );

        if (error = dataWrite( outUnit, buff, bytesIn ))
           {
            FPR( stderr, "ByteSwap Error writing output.\n" );
            goto shutdown;
           }
       }

    if (verbose)
       {
        FPR( stderr, "\n" );
        (void) fflush( stderr );
       }

/***/
/* Exit point:
/*   Deallocate buffer.
/*   Close files as needed.
/***/

shutdown:

    if (*inName)  (void) dataClose( inUnit );
    if (*outName) (void) dataClose( outUnit );

    (void) deAlloc( "swap", buff, buffSize );

    return( error );
}

/***/
/* getparams: extract command-line arguments for byteSwap.
/***/

int getparams( argc, argv,
               inName, outName,
               buffSize, byteCount, swapCount,
               checkFlag, verbose )

   int  argc, *buffSize, *byteCount, *swapCount, *checkFlag, *verbose;
   char **argv, *inName, *outName;
{

/***/
/* Initialize:
/***/

    *inName    = '\0';
    *outName   = '\0';
    *buffSize  = 4096;
    *byteCount = 0;
    *swapCount = 4;
    *checkFlag = 1;
    *verbose   = 0;

/***/
/* Print help text:
/***/

    if (flagLoc( argc, argv, "-help" ))
       {
        FPR( stderr, "ByteSwap: reverse byte-order of words in stream:\n" ); 
        FPR( stderr, " %s < inStream > outStream\n", argv[0] );
        FPR( stderr, "Options:\n" );
        FPR( stderr, " -in  inName    [stdin]  Input name of data to swap.\n" );
        FPR( stderr, " -out outName   [stdout] Output name for result.\n" ); 
        FPR( stderr, " -bs  buffSize  [4096]   Buffer Size, Words.\n" );
        FPR( stderr, " -ws  wordSize  [4]      Bytes Per Word to Swap.\n" );
        FPR( stderr, " -bc  byteCount [0]      Total Bytes to Swap (Use 0 for Auto Mode).\n" );
        FPR( stderr, " -nofs                   Suppress File Size Check.\n" );
        FPR( stderr, " -verb                   Verbose Mode On.\n" );
        FPR( stderr, "Notes:\n" );
        FPR( stderr, " 1. In-place processing (same input and output) is allowed if\n" );
        FPR( stderr, "    options -in and -out are both used.\n" );
        FPR( stderr, " 2. Auto Byte-Count determination requires use of -in option.\n" );
        FPR( stderr, " 3. Option -sc (swap-count) is supported as an alternative\n" );
        FPR( stderr, "    to -ws for backward-compatibility.\n" );
        return( 1 );
       }

/***/
/* Extract arguments:
/***/

    (void) strArgD( argc, argv, "-in",  inName ); 
    (void) strArgD( argc, argv, "-out", outName ); 

    (void) intArgD( argc, argv, "-bs",  buffSize ); 
    (void) intArgD( argc, argv, "-bc",  byteCount ); 
    (void) intArgD( argc, argv, "-sc",  swapCount ); 
    (void) intArgD( argc, argv, "-ws",  swapCount ); 

    if (flagLoc( argc, argv, "-nofs" )) *checkFlag = 0;
    if (flagLoc( argc, argv, "-verb" )) *verbose   = 1;

    *buffSize *= *swapCount;

/***/
/* Adjust modes not consistant with stream input/output:
/***/

    if (!*outName)
       {
        *checkFlag = 0;
       }

    if (*inName && *outName && !strcmp( inName, outName ))
       {
        *checkFlag = 0;
       }

    if (!*byteCount && !*inName)
       {
        FPR( stderr, "ByteSwap Error: auto byte-count requires -in inName.\n" );
        return( 1 );
       }

    return( 0 );
}

/***/
/* Bottom.
/***/
