/***/
/* NMRPipe utility for conversion of JEOL Delta Format NMR Data.
/* Based on their v 1.1 specifications.
/* Copyright 2006-2018 Frank Delaglio.
/*
/* Notes:
/*
/* 1. The JEOL format allows for 8 dimensions of 8-byte floats,
/*    and with arbitrarily spaced points. NMRPipe is currently 
/*    limited to 4 dimensions of 4-byte floats, with uniformly 
/*    spaced points. 
/*
/* 2. Since NMRPipe dimension sizes are encoded as 4-byte floats, 
/*    the maximum number of points in a given dimension is 16,777,216.
/*
/* 3. Like other modules in the NMRPipe system, this module is limited 
/*    to working with file sizes that can be expressed as a signed 4-byte 
/*    integer, 2147483646 bytes at most.
/* 
/* 4. As a simple way for this module to work with both files and
/*    streams, it manipulates the data entirely in-memory.  This
/*    may place a further limitation on the largest file sizes
/*    that can be handled.  If needed, future versions can include
/*    options for direct random I/O of the data files to avoid
/*    this limitation.
/***/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "version.h"
#include "cmndargs.h"
#include "testsize.h"
#include "fdatap.h"
#include "memory.h"
#include "dataio.h"
#include "prec.h"
#include "namelist.h"
#include "token.h"
#include "vutil.h"
#include "bswap.h"
#include "atof.h"

#include "delta.h"

#ifdef SYS_BYTESWAP
static int swapHdrFlag = 1;
#else
static int swapHdrFlag = 0;
#endif

static int swapDataFlag  = 0;

static int verbose       = 0;
static int showHdrFlag   = 0;
static int showParmFlag  = 0;
static int showRulerFlag = 0;
static int showTCLFlag   = 0;

static int infoFlag      = 0;
static int xyzFlag       = 0;
static int realOnlyFlag  = 0;
static int noInFlag      = 0;
static int noOutFlag     = 0;
static int checkFlag     = 1;
static int dirFlag       = 1;
static int ttyFlag       = 0;
static int hdrFlag       = 1;
static int ovFlag        = 0;

static int dimCount      = 0;
static int channelCount  = 0;
static int bytesRead     = 0;
 
static int totalInSize   = 0;
static int totalOutSize  = 0;

static int wordSizeIn    = 0;
static int wordSizeOut   = 0;

static int inSize[JMAXDIM];
static int outSize[JMAXDIM];
static int quadSize[JMAXDIM];
static int smxSize[JMAXDIM];

static int matX1[JMAXDIM];
static int matXN[JMAXDIM];
static int smxX1[JMAXDIM];
static int smxXN[JMAXDIM];

static int   dfOrders[JMAXDIM];
static int   dfFactors[JMAXDIM];

static int   dfFlag        =  0;
static int   dfSize        =  0;
static int   dfFinalSize   =  0;
static int   skipSize      = -1;
static int   zfFlag        =  1;
static int   dfCorrectFlag =  0;

static float trVal         =  0.0;
static float dfVal         =  0.0;

static char inName[NAMELEN+1];
static char outFmt[NAMELEN+1];
static char cmndName[NAMELEN+1];

static char *nmrPipeInfoStr = (char *)NULL;
static char **infoArgv      = (char **)NULL;
static char infoArgc        = 0;

static float pSW[JMAXDIM], pObs[JMAXDIM], pCar[JMAXDIM], pOrig[JMAXDIM];
static float pTemperature;

static UNIT inUnit;
static UNIT outUnit;
static FILE *vUnit;

static unsigned char *workBuff = (unsigned char *)NULL;
static int workSize = 0;

static char dimListU[] = "XYZABCDEF";
static char dimListL[] = "xyzabcdef";

#define ICLIP(M,LO,HI)  if (M < LO) M = LO; if (M > HI) M = HI

#define MIN(A,B) ((A) < (B) ? (A) : (B))
#define MAX(A,B) ((A) > (B) ? (A) : (B))

#define FPR (void)fprintf
#define SPR (void)sprintf

#define GetKeyword(N,V) getNameStrByVal(N,(float)(V))

int main( argc, argv )

   int  argc;
   char **argv;
{
    int error;

    int  hdrBuffSize,  histBuffSize, listBuffSize, parmBuffSize, dataBuffSize, 
         contBuffSize, noteBuffSize;

    char *hdrBuff,    *histBuff,     *listBuff,    *parmBuff,    *dataBuff,
         *contBuff,    *noteBuff;

    int   i, loc, n, nn, nc, listStart, listStop, listLength, plane, planeCount,
          iz, ia, stdHdrBuffSize, status;

    char  outName[NAMELEN+1], *buff1, *buff2, *sPtr;
    float minVal, maxVal, fdata[FDATASIZE];

    struct DeltaHdrItem *hPtr;
 
/***/
/* Null-initialize:
/***/

    error  = 0;
    status = 0;

    buff1 = (char *)NULL;
    buff2 = (char *)NULL;

    *outName    = '\0';

    listStart   = 0;
    listLength  = 0;
    planeCount  = 0;
    plane       = 0;
    iz          = 0;
    ia          = 0;

    minVal   =  LARGENUM;
    maxVal   = -LARGENUM;

    hdrBuff   = (char *)NULL; hdrBuffSize   = 0;
    histBuff  = (char *)NULL; histBuffSize  = 0;
    listBuff  = (char *)NULL; listBuffSize  = 0;
    parmBuff  = (char *)NULL; parmBuffSize  = 0;
    dataBuff  = (char *)NULL; dataBuffSize  = 0;
    contBuff  = (char *)NULL; contBuffSize  = 0;
    noteBuff  = (char *)NULL; noteBuffSize  = 0;

    stdHdrBuffSize = 1360;

    (void) initDeltaVars();
    (void) initArgs( argc, argv, CHECK_ENV );
    (void) initFDATA( fdata );
    (void) deltaDFNull();

    if (getDeltaArgs( argc, argv, fdata )) goto shutdown;

/***/
/* Null-initialize delta header structure:
/***/ 

    n = 0;
 
    for( hPtr = deltaHdrList1_1; hPtr->dest; hPtr++ )
       {
        (void) setDeltaHdrItemNull( hPtr );
	n += hPtr->byteCount;
       }

    if (n != stdHdrBuffSize)
       {
        FPR( stderr, "Delta Warning: Header Byte Tally Error:\n" );
	FPR( stderr, "Expected Bytes: %d Tally: %d\n", stdHdrBuffSize, n );
       }	    

    for( hPtr = deltaParmHdrList1_1; hPtr->dest; hPtr++ )
       {
        (void) setDeltaHdrItemNull( hPtr );
        n += hPtr->byteCount;
       }

    for( hPtr = deltaParmValList1_1; hPtr->dest; hPtr++ )
       {
        (void) setDeltaHdrItemNull( hPtr );
        n += hPtr->byteCount;
       }

/***/
/* Open input stream as needed:
/***/

    if (*inName)
       {
        if (error = dataOpen( inName, &inUnit, FB_READ ))
           {
            FPR( stderr, "Delta Error Opening Delta Input: %s\n", inName );
            goto shutdown;
           }
       }
    else
       {
        inUnit = FB_STDIN;
       }

    if (!ttyFlag && isTTY( inUnit ))
       {
        FPR( stderr, "Delta Error: input from a terminal:\n" );
	FPR( stderr, " Check for missing input argument.\n" );
	FPR( stderr, " Check for trailing  spaces  after\n" );
	FPR( stderr, " any \"\\\" line-continuations.\n" );
        error = 1;
        goto shutdown;
       }

/***/
/* Allocate and read header.
/* Extract header items according to internal header definition list.
/***/

    nc = stdHdrBuffSize; loc = 0;
    error = getDeltaBuff( &hdrBuff, &hdrBuffSize, "Header", nc, loc, 0 );
    if (error) goto shutdown;

    for( hPtr = deltaHdrList1_1; hPtr->dest; hPtr++ )
       {
        (void) getDeltaHdrItem( hPtr, hdrBuff + hPtr->byteOffset, swapHdrFlag );
       }

/***/
/* Interpret data type and size items in header.
/***/

#ifdef SYS_BYTESWAP 
    swapDataFlag = deltaHdr.endianMode == JEOL_LITTLE_ENDIAN ? 0 : 1;
#else
    swapDataFlag = deltaHdr.endianMode == JEOL_LITTLE_ENDIAN ? 1 : 0;
#endif

    dimCount     = deltaHdr.dimCount;

    channelCount = 1;
    totalInSize  = 1;
    totalOutSize = 1;

    for( i = 0; i < dimCount; i++ )
       {
        inSize[i]  = deltaHdr.sizeList[i];
        outSize[i] = 1 + deltaHdr.offsetStop[i] - deltaHdr.offsetStart[i];

        matX1[i]   = 1;
        matXN[i]   = outSize[i];
        smxX1[i]   = 1 + deltaHdr.offsetStart[i];
        smxXN[i]   = 1 + deltaHdr.offsetStop[i];

        totalInSize  *= inSize[i];
        totalOutSize *= outSize[i];

        if (isDeltaQuad( i ))
           {
            quadSize[i]   = 2;
	    channelCount *= 2;
           }
       }

    (void) getDeltaWordSize();
    (void) getSMXSize();

    dfFinalSize = outSize[0];
    dfSize      = outSize[0];

    n = intArgDN( argc, argv, "-smx", smxSize, JMAXDIM );

    if (n > 0 && n != dimCount)
       {
        FPR( stderr, "Delta Error Extracting -smx Arguments.\n" );
        error = 1;
        goto shutdown;
       }

    if (showHdrFlag)
       {
        for( hPtr = deltaHdrList1_1; hPtr->dest; hPtr++ )
           {
            (void) showDeltaHdrItem( hPtr );
           }
       }

/***/
/* Allocate and read the history buffer; currently unused.
/***/ 

    nc = MAX(deltaHdr.historyLength,deltaHdr.historyUsed); loc = bytesRead;
    error = getDeltaBuff( &histBuff, &histBuffSize, "History", nc, loc, 0 );
    if (error) goto shutdown;

/***/
/* Read the parameter buffer.
/* Report the parameter values.
/***/

    nc = deltaHdr.paramLength; loc = deltaHdr.paramStart;
    error = getDeltaBuff( &parmBuff, &parmBuffSize, "Parameter", nc, loc, 0 );
    if (error) goto shutdown;

    n = 0;

    for( hPtr = deltaParmHdrList1_1; hPtr->dest; hPtr++ )
       {
        (void) getDeltaHdrItem( hPtr, parmBuff+hPtr->byteOffset, swapDataFlag );
        if (showParmFlag) (void) showDeltaHdrItem( hPtr );
        n += hPtr->byteCount;
       }

    sPtr = parmBuff + n; 
 
    for( i = deltaParmHdr.loID; i < deltaParmHdr.hiID; i++ )
       {
        if (showParmFlag) FPR( vUnit, "\n" );

        for( hPtr = deltaParmValList1_1; hPtr->dest; hPtr++ )
           {
            (void) getDeltaHdrItem( hPtr, sPtr+hPtr->byteOffset, swapDataFlag );
            if (showParmFlag) showDeltaHdrItem( hPtr );
           }

	(void) storeDeltaParm( &deltaParmVal );

        sPtr += deltaParmHdr.parmSize;
       }

    if (showParmFlag) FPR( vUnit, "\n" );
 
/***/
/* Find the total limits of the various list buffers:
/***/

    listStart = bytesRead;
    listStop  = bytesRead;
    n = 0;

    for( i = 0; i < JMAXDIM; i++ )
       {
        if (!deltaHdr.listStart[i]) continue;

        n = deltaHdr.listStart[i] + deltaHdr.listLength[i] - 1;

	listStart = MIN( listStart, deltaHdr.listStart[i] );
	listStop  = MAX( n, listStop );
       }

    listLength = listStop - listStart;

/***/
/* Read the list buffers.
/* Report their ruler values.
/***/

    nc = listLength; loc = listStart;
    error = getDeltaBuff( &listBuff, &listBuffSize, "List", nc, loc, 0 );
    if (error) goto shutdown;

    if (showRulerFlag)
       {
        for( i = 0; i < JMAXDIM; i++ )
           {
            if (!deltaHdr.listLength[i]) continue;

	    n  = deltaHdr.listStart[i] - listStart;
            nc = deltaHdr.listLength[i]/sizeof(double);

	    (void) showDeltaRuler( vUnit, listBuff + n, nc, i + 1, 1 ); 
           }           
       }

/***/
/* Use data buffer 2x larger than result with first and second half
/* 1024-aligned for shuffling space.  Since the result will generally
/* be half the size of the input (4 byte words vs 8 byte) the size of
/* the buffer will be roughly the size of the input.
/*
/* Report data info.
/***/

    n   = 2*totalInSize*channelCount*wordSizeOut;
    nc  = deltaHdr.dataLength;
    loc = deltaHdr.dataStart;

    if (nc > n) n = nc;
    n += 1024;

    while( n % 2048 ) n++;

    if (showHdrFlag)
       {
        FPR( vUnit, "WordSizeIn 1:             %d\n", wordSizeIn );
        FPR( vUnit, "WordSizeOut 1:            %d\n", wordSizeOut );
        FPR( vUnit, "ChannelSizeIn 1:          %d\n", totalInSize );
        FPR( vUnit, "ChannelSizeOut 1:         %d\n", totalOutSize );
        FPR( vUnit, "Channels 1:               %d\n", channelCount );
        FPR( vUnit, "DataBufferSize 1:         %d\n", n );

        FPR( vUnit, "InSize %d:                ", JMAXDIM );
        for( i = 0; i < JMAXDIM; i++ ) FPR( vUnit, " %d", inSize[i] );
        FPR( vUnit, "\n" );

        FPR( vUnit, "OutSize %d:               ", JMAXDIM );
        for( i = 0; i < JMAXDIM; i++ ) FPR( vUnit, " %d", outSize[i] );
        FPR( vUnit, "\n" );

        FPR( vUnit, "QuadSize %d:              ", JMAXDIM );
        for( i = 0; i < JMAXDIM; i++ ) FPR( vUnit, " %d", quadSize[i] );
        FPR( vUnit, "\n" );

        FPR( vUnit, "SMXSize %d:               ", JMAXDIM );
        for( i = 0; i < JMAXDIM; i++ ) FPR( vUnit, " %d", smxSize[i] );
        FPR( vUnit, "\n" );
       }

    dfVal = getDeltaDF( dfOrders, dfFactors );

    if (!isDeltaQuad( 0 ) || !isDeltaTimeDomain( 0 )) dfCorrectFlag = 0;

    if (dfVal == 0.0)
       {
        dfCorrectFlag = 0;
        dfFlag = 0;
       }
    else
       {
        if (showHdrFlag)
	   {  	
            FPR( stderr, "\n" );
  	    FPR( stderr, "DF Adjustment:           %7.4f\n",  dfVal );
            FPR( stderr, "DF Transition:           %11.4e\n", trVal );
            FPR( stderr, "\n" );
           }
       }

/***/
/* Open a dummy output file for testing purposes.
/* Test for sufficient space as needed.
/* Delete the dummy file.
/***/

    if (noInFlag) dfCorrectFlag = 0;

    dfFinalSize = getDeltaDFSize( outSize[0], dfVal );

    if (dfCorrectFlag)
       {
        error = deltaDFInit( outSize[0], skipSize, zfFlag, dfVal, &dfSize );
        if (error) goto shutdown;
       }

    (void) deltaSetFDATA( argc, argv, fdata );

    if (showTCLFlag) (void) deltaShowTCL( infoFlag ? stdout : vUnit, fdata );

    if (infoFlag) goto shutdown;

    plane = 1;
    iz    = 1;
    ia    = 1;

    if (!*outFmt)
       {
/*
        if (!ttyFlag && isTTY( outUnit ))
           {
            FPR( stderr, "Delta Error: output to a terminal:\n" );
            FPR( stderr, " Check for missing output argument.\n" );
            FPR( stderr, " Check for trailing  spaces  after\n" );
            FPR( stderr, " any \"\\\" line-continuations.\n" );
            error = 1;
            goto shutdown;
           }
*/
       }

    if (*outFmt && checkFlag && !noOutFlag)
       {
        if (error = openNMR( outName, plane, iz, ia, fdata ))
           {
            FPR( stderr, "Delta Error Testing %s\n", outName );
            goto shutdown;
           }

        if (error = closeNMR())
           {
            FPR( stderr, "Delta Error Closing %s\n", outName );
            goto shutdown;
           }

       switch( testNMRout( outName, fdata, xyzFlag, 1 ))
            {
             case FILE_SIZE_FAIL:
                FPR( stderr, "\n" );
                FPR( stderr, "Delta Warning:" );
                FPR( stderr, " can't check size for file" );
                FPR( stderr, " %s\n", outName );
                break;
             case  FILE_SIZE_TOO_LARGE:
                FPR( stderr, "\n" );
                FPR( stderr, "Delta Error:" );
                FPR( stderr, " insufficient space for file" );
                FPR( stderr, " %s\n", outName );
                error = 1;
                goto shutdown;
                break;
             default:
                break;
            }

        (void) delFile( outName );
       }

/***/
/* Allocate and read the data buffer.
/***/

    if (noInFlag || noOutFlag)
       {
        minVal = 0.0;
        maxVal = 0.0;
       }
    else
       {
        if (verbose) FPR( stderr, "Reading Data: %d bytes.\n", nc );

        error = getDeltaBuff( &dataBuff, &dataBuffSize, "Data", nc, loc, n-nc );
        if (error) goto shutdown;

        if (swapDataFlag) (void)byteSwapN( dataBuff, dataBuffSize, wordSizeIn );

        if (wordSizeIn == sizeof(double))
           {
            (void) vDbl2Flt( dataBuff, totalInSize*channelCount );
           }

        minVal = vMin( (float *)dataBuff, totalInSize*channelCount );
        maxVal = vMax( (float *)dataBuff, totalInSize*channelCount );
       }

    if (showHdrFlag)
       {
        FPR( vUnit, "MinVal 1:                % e\n", minVal );
        FPR( vUnit, "MaxVal 1:                % e\n", maxVal );
        FPR( vUnit, "\n" );
       }

/***/
/* Perform submatrix conversions.
/* Perform R/I interleaving.
/***/

    if (noInFlag || noOutFlag)
       {
        buff1 = (char *)NULL;
        buff2 = (char *)NULL;
       }
    else
       {
        buff1 = dataBuff;
        buff2 = dataBuff + dataBuffSize/2;

        error = deltaConvertSMX( buff1, buff2, wordSizeOut );
        error = deltaInterleaveRI( buff1, buff2, wordSizeOut );
       }

/***/
/* Allocate and read context and history buffers.
/* This will avoid broken pipe problems when reading from a stream.
/***/

    nc = deltaHdr.contextLength; loc = deltaHdr.contextStart;
    if (noInFlag || noOutFlag) nc = 0;

    error = getDeltaBuff( &contBuff, &contBuffSize, "Context", nc, loc, 0 );
    if (error) goto shutdown;

    nc = deltaHdr.annoteLength; loc = deltaHdr.annoteStart;
    if (noInFlag || noOutFlag) nc = 0;

    error = getDeltaBuff( &noteBuff, &noteBuffSize, "Annotation", nc, loc, 0 );
    if (error) goto shutdown;

/***/
/* Write the result:
/***/

    if (noOutFlag) goto shutdown;
    if (noInFlag)  dfCorrectFlag = 0;

    if (realOnlyFlag)
       {
        for( i = 0; i < dimCount; i++ ) quadSize[i] = 1;
       }

    planeCount = outSize[2]*quadSize[2]*outSize[3]*quadSize[3];

    n     = wordSizeOut*outSize[0]*quadSize[0]*outSize[1]*quadSize[1];
    nn    = wordSizeOut*dfSize*quadSize[0]*outSize[1]*quadSize[1];
    sPtr  = dataBuff;
    plane = 1;

    for( ia = 1; ia <= outSize[3]*quadSize[3]; ia++ )
       {
        for( iz = 1; iz <= outSize[2]*quadSize[2]; iz++ )
           {
            if (verbose)
	       {
	        FPR( stderr, "Plane %d of %d\015", plane, planeCount );
	        (void) fflush( stderr );
	       }

	    if (dfCorrectFlag) (void) deltaConvertDF( sPtr, buff2 );

            if (error = openNMR( outName, plane, iz, ia, fdata )) goto shutdown;
	    if (error = dataWrite( outUnit, sPtr, nn )) goto shutdown;

            (void) closeNMR();

	    sPtr += n;
	    plane++;
           }
       }

    if (verbose)
       {
        FPR( stderr, "\n" );
        (void) fflush( stderr );
       }

    if (*cmndName) wait( &status );

/***/
/* Exit point:
/***/

shutdown:

    (void) checkArgs( argc, argv );

    (void) deAlloc( "delta", hdrBuff,   hdrBuffSize );
    (void) deAlloc( "delta", histBuff,  histBuffSize );
    (void) deAlloc( "delta", listBuff,  listBuffSize );
    (void) deAlloc( "delta", parmBuff,  parmBuffSize );
    (void) deAlloc( "delta", dataBuff,  dataBuffSize );
    (void) deAlloc( "delta", contBuff,  contBuffSize );
    (void) deAlloc( "delta", noteBuff,  noteBuffSize );
    (void) deAlloc( "delta", workBuff,  workSize );

    if (*inName) (void) dataClose( inUnit );

    (void) deltaDFFree();

    if (infoArgc)       (void) str2argvFree( infoArgc, infoArgv );
    if (nmrPipeInfoStr) (void) strFree( nmrPipeInfoStr );

    return( error );
}

/***/
/* Apply digital filter adjustments to vectors in the current 2D plane buffer.
/***/

int deltaConvertDF( buff, work )

   char *buff, *work;
{
    float *srcR, *srcI, *rPtrR, *rPtrI, *dest;
    int   i, j, vCount;

    dest   = (float *)work;
    srcR   = (float *)buff;
    srcI   = srcR + outSize[0];
    vCount = outSize[1]*quadSize[1];

    for( i = 0; i < vCount; i++ )
       {
        (void) deltaDF( srcR, srcI );

	rPtrR = srcR;
	rPtrI = srcI;

	for( j = 0; j < dfSize; j++ ) *dest++ = *rPtrR++; 
	for( j = 0; j < dfSize; j++ ) *dest++ = *rPtrI++; 

	srcR += outSize[0]*2;
        srcI  = srcR + outSize[0];
       }

    (void) memcpy( (void *)buff, (void *)work, (size_t)sizeof(float)*dfSize*2*vCount );
   
    return( 0 );
}

/***/
/* Convert submatrix data to sequential format.
/***/

int deltaConvertSMX( buff, work, wordSize )

   char *buff, *work;
   int  wordSize;
{
    char *src, *dest;
    int  i, n, error;

    src  = buff;
    dest = work;

    for( i = 0; i < channelCount; i++ )
       {
        if (verbose)
           {
            FPR( stderr, 
		 "Converting Submatrix %d of %d\015", 
		 i+1, channelCount );

	    (void) fflush( stderr );
           }


        error = smx2matrix( src, dest, 
                            outSize, matX1, matXN,
                            inSize, smxX1, smxXN, smxSize, 
                            wordSize, dimCount );

        src  += wordSize*totalInSize;
        dest += wordSize*totalOutSize;
       }

    (void) memcpy( (void *)buff, 
		   (void *)work, 
		   (size_t)wordSize*totalOutSize*channelCount );
   
   if (verbose)
      {
       FPR( stderr, "\n" );
       (void) fflush( stderr );
      }

    return( 0 );
}

/***/
/* Interleave real and imaginary data, adjusting sign as needed.
/***/

int deltaInterleaveRI( buff, work, wordSize )

   char  *buff, *work;
   int   wordSize;
{
    int   i, j, k, dim, pairCount, vSize, vCount, revFlag, quadSizeN, outSizeN;
    float *srcR, *srcI, *dest;

    if (channelCount == 1) return( 0 );
    if (realOnlyFlag) return( 0 );

    if (deltaHdr.axisType[0] != JEOL_AXISTYPE_ENVELOPE)
       {
        for( i = 0; i < dimCount; i++ )
           {
            deltaHdr.reversed[i] = !deltaHdr.reversed[i];
           }
       }

    pairCount = channelCount/2;

    if (dimCount >= 1 && quadSize[0] == 2)
       {
        dest    = (float *)work;
	srcR    = (float *)buff;
	srcI    = srcR + totalOutSize;
	vSize   = outSize[0];
        vCount  = totalOutSize/outSize[0];
        revFlag = deltaHdr.reversed[0];

	for( i = 0; i < pairCount; i++ )
           {
            deltaInterleavePair( dest, srcR, srcI, vSize, vCount, revFlag );

	    dest += totalOutSize*2;
	    srcR += totalOutSize*2;
	    srcI  = srcR + totalOutSize;
           }

        (void) memcpy( (void *)buff, 
                       (void *)work, 
	               (size_t)wordSize*totalOutSize*channelCount );

	pairCount /= 2;
       }

    for( dim = 1; dim < dimCount; dim++ )
       {
        if (quadSize[dim] != 2) continue;

        quadSizeN = 1;
	outSizeN  = 1;

	for( i = 0; i < dim; i++ ) quadSizeN *= quadSize[i];
	for( i = 0; i < dim; i++ ) outSizeN  *= outSize[i];

        dest    = (float *)work;
	srcR    = (float *)buff;
	srcI    = srcR + totalOutSize*quadSizeN;
	vSize   = outSizeN*quadSizeN;
        vCount  = totalOutSize/outSizeN;
        revFlag = deltaHdr.reversed[dim];

	for( i = 0; i < pairCount; i++ )
           {
            deltaInterleavePair( dest, srcR, srcI, vSize, vCount, revFlag );

	    dest += totalOutSize*quadSizeN*2;
	    srcR += totalOutSize*quadSizeN*2;
	    srcI  = srcR + totalOutSize*quadSizeN;
           }

        (void) memcpy( (void *)buff, 
                       (void *)work, 
	               (size_t)wordSize*totalOutSize*channelCount );

	pairCount /= 2;
       }

    return( 0 );
}

/***/
/* Interleave a particular real/imag block pair of vectors.
/***/

int deltaInterleavePair( dest, srcR, srcI, vSize, vCount, revFlag )

   float *dest, *srcR, *srcI;
   int   vSize, vCount, revFlag;
{
    int j, k;

    if (revFlag)
       {
        for( j = 0; j < vCount; j++ )
           {
            for( k = 0; k < vSize; k++ ) *dest++ = *srcR++;
            for( k = 0; k < vSize; k++ ) *dest++ = -(*srcI++);
           }
       }
    else
       {
        for( j = 0; j < vCount; j++ )
           {
            for( k = 0; k < vSize; k++ ) *dest++ = *srcR++;
            for( k = 0; k < vSize; k++ ) *dest++ = *srcI++;
           }
       }

   return( 0 );
}

/***/
/* Display values in the given Delta Ruler.
/***/

int showDeltaRuler( outUnit, rulerBuff, wordCount, rulerID, vFlag )

   FILE *outUnit;
   char *rulerBuff;
   int  wordCount, rulerID;
{
     unsigned char *sPtr;
     double   d;
     int      i;

     sPtr = (unsigned char *)rulerBuff;

     if (vFlag)
        {
         FPR( outUnit, "Ruler%d %d:\n", rulerID, wordCount );

         for( i = 0; i < wordCount; i++ )
            {
             d     = uc2flt8( sPtr, swapHdrFlag );
	     sPtr += sizeof(double);

             FPR( outUnit, " %5d % e", i+1, d );
            }
        }
     else
        {
         for( i = 0; i < wordCount; i++ )
            {
             d     = uc2flt8( sPtr, swapHdrFlag );
             sPtr += sizeof(double);

             FPR( outUnit, " % e", d );
            }
        }

     return( 0 );
}

/***/
/* Record some particular parameters of interest as they are converted.
/***/

int storeDeltaParm( pvPtr )

   struct DeltaParmVal *pvPtr;
{
    char   ctemp[NAMELEN+1];
    int    i, n;

    struct jVal *vvPtr;

    vvPtr = &deltaValVal;

    if (!strcasecmp( pvPtr->name, "NMRPipe_Info" ))
       {
        if (verbose) FPR( stderr, "Extracting NMRPipe_Info: %s\n", vvPtr->s );

        (void) str2argvC( "delta2pipe", vvPtr->s, &infoArgc, &infoArgv );

        nmrPipeInfoStr = strDup( vvPtr->s );

	return( 1 );
       }

    if (!strcasecmp( pvPtr->name, "TEMP_GET" ))
       {
        pTemperature = vvPtr->d;

	if (pvPtr->units[0].unitType == JEOL_SIUNIT_CELSIUS)
           {
            pTemperature += 273.0;
           }

	return( 1 );
       }

    if (!strcasecmp( pvPtr->name, "TRANSITION_RATIO" ))
       {
        trVal = vvPtr->d;
	return( 1 );
       }

    if (!strcasecmp( pvPtr->name, "DIGITAL_FILTER" ))
       {
        if (vvPtr->s[0] == 'T') dfFlag = 1;
        if (vvPtr->s[0] == 't') dfFlag = 1;
        if (vvPtr->s[0] == '1') dfFlag = 1;

	return( 1 );
       }

    if (!strcasecmp( pvPtr->name, "ORDERS" ))
       {
        n = cntToken( vvPtr->s );
        n = MIN( n, JMAXDIM );

	for( i = 0; i < n; i++ ) dfOrders[i] = getTokenI( i+1, vvPtr->s );

	return( 1 );
       }

    if (!strcasecmp( pvPtr->name, "FACTORS" ))
       {
        n = cntToken( vvPtr->s );
        n = MIN( n, JMAXDIM );

	for( i = 0; i < n; i++ ) dfFactors[i] = getTokenI( i+1, vvPtr->s );

	return( 1 );
       }

    n = strlen(dimListU);
    n = MIN( n, JMAXDIM );

    for( i = 0; i < n; i++ )
       {
        SPR( ctemp, "%c_OFFSET", dimListU[i] );

	if (!strcasecmp( pvPtr->name, ctemp ))
           {
            pCar[i] = vvPtr->d;
	    return( 1 );
           }

        SPR( ctemp, "%c_SWEEP", dimListU[i] );

	if (!strcasecmp( pvPtr->name, ctemp ))
           {
            pSW[i] = getDeltaUnitVal( vvPtr->d, &pvPtr->units[0] );


	    if (pvPtr->unitScale != 0) 
	       {
		pSW[i] *= pow( 10.0, (double)pvPtr->unitScale );
               }

	    return( 1 );
           }

        SPR( ctemp, "%c_FREQ", dimListU[i] );

	if (!strcasecmp( pvPtr->name, ctemp ))
           {
            pObs[i] = 1.0e-6*getDeltaUnitVal( vvPtr->d, &pvPtr->units[0] );

	    if (pvPtr->unitScale != 0) 
	       {
		pObs[i] *= pow( 10.0, (double)pvPtr->unitScale );
               }

	    return( 1 );
           }
       }

    return( 0 );
}

/***/
/* Create an NMRPipe format file header for the delta output.
/***/

int deltaSetFDATA( argc, argv, fdata )

   int  argc;
   char **argv;
   float fdata[FDATASIZE];
{
    int   i, j, n, sizeN, sizeT, sizeF, ftFlag, quadFlag, aq2DMode, planeCount;
    char *labPtr, *sPtr, ctemp[NAMELEN+1], lab[NAMELEN+1], macName[NAMELEN+1];
    float s;
    
    float sw[JMAXDIM], obs[JMAXDIM], car[JMAXDIM], orig[JMAXDIM];
    int   mid[JMAXDIM];

    for( i = 0; i < JMAXDIM; i++ )
       {
        sw[i]    = 1.0;
        obs[i]   = 1.0;
        car[i]   = 0.0;
        mid[i]   = 0;

        pOrig[i] = 0.0;
       }

    (void) fixfdata( fdata );

/***/
/* Detection modes and digital filter info:
/***/

    aq2DMode = getDeltaAQ2D();

    if (realOnlyFlag)
       {
        quadFlag = FD_REAL;
	dfVal    = 0.0;
       }
    else
       {
        quadFlag = quadSize[0] == 2 ? FD_COMPLEX : FD_REAL;
        if (isDeltaQuad( 0 )) dfVal = getDeltaDF( dfOrders, dfFactors );
       }

    (void) strcpy( ctemp, "Auto" );
    (void) strArgD( argc, argv, "-dfVal", ctemp );
    if (strcasecmp( ctemp, "Auto" )) dfVal = ATOF( ctemp ); 

    (void) strcpy( ctemp, "Auto" );
    (void) strArgD( argc, argv, "-trVal", ctemp );
    if (strcasecmp( ctemp, "Auto" )) trVal = ATOF( ctemp ); 
       
    if (dfVal == 0.0)
       {
        dfCorrectFlag = 0;
        dfFlag = 0;
       }

    (void) setParm( fdata, FDDIMCOUNT, (float)dimCount,     NULL_DIM );
    (void) setParm( fdata, FDQUADFLAG, (float)quadFlag,     NULL_DIM );
    (void) setParm( fdata, FD2DPHASE,  (float)aq2DMode,     NULL_DIM );

    if (!dfCorrectFlag)
       {
        (void) setParm( fdata, FDDMXVAL,   (float)dfVal,    NULL_DIM );
        (void) setParm( fdata, FDDELTATR,  (float)trVal,    NULL_DIM );
       }

/***/
/* Pipeline parameters:
/***/

    if (realOnlyFlag)
       planeCount = outSize[2]*outSize[3];
    else
       planeCount = outSize[2]*quadSize[2]*outSize[3]*quadSize[3];

    if (dimCount <= 2 || xyzFlag)
       {
        (void) setParm( fdata, FDFILECOUNT, (float)planeCount, 0 );
        (void) setParm( fdata, FDPIPEFLAG,  (float)0,          0 );
       }
    else
       {
        (void) setParm( fdata, FDFILECOUNT, (float)1, 0 );
        (void) setParm( fdata, FDPIPEFLAG,  (float)1, 0 );
       }

/***/
/* Axis labels, adjusted for redundant values and detection 1H -> HN:
/***/

    labPtr = deltaHdr.axisTitles;

    for( i = 0; i < dimCount; i++ )
       {
        (void) space2us( lab, labPtr ); labPtr += NAMELEN+1;

	if (!i && (!strcasecmp( lab, "1H" ) || !strcasecmp( lab, "H1" )))
           {
	    sPtr = deltaHdr.axisTitles;

	    for( j = 0; j < dimCount; j++ )
               {
                (void) space2us( ctemp, sPtr ); sPtr += NAMELEN+1;
                if (i == j) continue;

		if (!strcasecmp( ctemp, "15N" ) || !strcasecmp( ctemp, "N15" ))
                   {
		    (void) strcpy( lab, "HN" );
		    break;
		   }
               }
	   }

	sPtr = deltaHdr.axisTitles;

	for( j = 0; j < dimCount; j++ )
           {
            (void) space2us( ctemp, sPtr ); sPtr += NAMELEN+1;
            if (i == j) continue;

  	    if (!strcasecmp( lab, ctemp ))
               {
                SPR( ctemp, "%c", dimListL[i] );
	        (void) strcat( lab, ctemp );
	        break;
               }
           }

        (void) setParmStr( fdata, NDLABEL, lab, i+1 );
       }

/***/
/* Size settings and Calibration:
/***/

    for( i = 0; i < dimCount; i++ )
       {
        if (realOnlyFlag || quadSize[i] == 1)
           {
            quadFlag = FD_REAL;
            sizeN    = outSize[i]; 
            sizeT    = outSize[i]; 
           }
        else
           {
            quadFlag = FD_COMPLEX;
            sizeN    = i == 0 ? dfSize : 2*outSize[i]; 
            sizeT    = i == 0 ? dfSize : outSize[i]; 
           }

	if (isDeltaTimeDomain( i ))
           {
            ftFlag = 0;
	    sizeF  = nextPower2(2*sizeT);
	   }
	else
           {
            ftFlag = 1;
	    sizeF  = sizeT;
	    sizeT /= 2;
           }

        mid[i]   = sizeF/2 + 1; 
        sw[i]    = getDeltaSW( i );
        obs[i]   = getDeltaObs( i );
        car[i]   = getDeltaCar( i );

/***/
/* Here, pOrig is the PPM calibration based on time-domain info.
/* For freq-domain axis, it will be replaced by Delta header values. 
/***/

        pOrig[i] = obs[i]*car[i] - sw[i]*(sizeF - mid[i])/sizeF;	
        orig[i]  = getDeltaOrig( i );

	if (i == 0)
           {
            (void) setParm( fdata, FDREALSIZE, (float)sizeT, 0 );
           }

        (void) setParm( fdata, NDSIZE,     (float)sizeN,    i+1 );
        (void) setParm( fdata, NDAPOD,     (float)sizeT,    i+1 );
        (void) setParm( fdata, NDTDSIZE,   (float)sizeT,    i+1 );
        (void) setParm( fdata, NDFTFLAG,   (float)ftFlag,   i+1 );
        (void) setParm( fdata, NDQUADFLAG, (float)quadFlag, i+1 );

        (void) setParm( fdata, NDSW,       (float)sw[i],    i+1 );
        (void) setParm( fdata, NDOBS,      (float)obs[i],   i+1 );
        (void) setParm( fdata, NDCENTER,   (float)mid[i],   i+1 );
        (void) setParm( fdata, NDCAR,      (float)car[i],   i+1 );
        (void) setParm( fdata, NDORIG,     (float)orig[i],  i+1 );
       }

    if (infoArgc) (void) parseHdr( infoArgc, infoArgv, fdata );

    if (hdrFlag) (void) parseHdr( argc, argv, fdata );

/***/
/* Compensate for adjustments made by parsehdr:
/***/

    for( i = 0; i < dimCount; i++ )
       {
        SPR( ctemp, "-%cORIG", dimListL[i] );
        if (flagLoc( argc, argv, ctemp )) continue;

        (void) setParm( fdata, NDORIG, (float)orig[i],  i+1 );
       }

    if (quadSize[0] == 2)
       {
        if (flagLoc( argc, argv, "-xN" ))
           {
            sizeN  = getParm( fdata, NDSIZE, CUR_XDIM );
	    sizeN  = (sizeN/2) - (outSize[0] - dfSize);

            (void) setParm( fdata, NDSIZE, (float)sizeN, CUR_XDIM );
           }

        if (flagLoc( argc, argv, "-xT" ))
           {
            sizeT  = getParm( fdata, NDAPOD, CUR_XDIM );
	    sizeT  = sizeT - (outSize[0] - dfSize);

            (void) setParm( fdata, NDAPOD, (float)sizeT, CUR_XDIM );
           }
       }

/***/
/* Gradient handling:
/***/

    n = 0;

    for( i = 1; i < dimCount; i++ )
       {
        quadFlag = getParm( fdata, NDQUADFLAG, i+1 );

	if (quadFlag == FD_GRAD)
           {
            if (*cmndName) strcat( cmndName, " | " );

	    SPR( macName, "%s/delta_rance%c.M", getenv( "NMRTXT" ), dimListU[i] ); 
	    SPR( ctemp, "nmrPipe -fn MAC -macro %s -noRd -noWr", macName );

            if (verbose) FPR( stderr, "Output Macro: %s\n", macName );

	    (void) strcat( cmndName, ctemp );

            (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, i+1 );

	    n++;
           }
       }

    if (n)
       {
        if (*outFmt)
           {
            if (isZFmt( outFmt ) || isAZFmt( outFmt ))
               {
                SPR( ctemp, " | pipe2xyz -out %s", outFmt );
                (void) strcat( cmndName, ctemp );
               }
	    else
               {
                SPR( ctemp, " -out %s", outFmt );
                (void) strcat( cmndName, ctemp );
               }

	    if (ovFlag) strcat( cmndName, " -ov" );
           }
       }

    return( 0 );
}

/***/
/* Display delta parameters as TCL statements, used directly by graphical
/* conversion interface to set default values.
/***/

int deltaShowTCL( oUnit, fdata )

   FILE  *oUnit;
   float fdata[FDATASIZE];
{
    char ctemp[NAMELEN+1];
    int  i;

    FPR( oUnit, "\n" );
    FPR( oUnit, "#\n" );
    FPR( oUnit, "# Begin TCL\n" );

    FPR( oUnit, "\n" );
    FPR( oUnit, "set dimCount    %d\n",   dimCount );

    i = getDeltaAQ2D();

    if (i == FD_MAGNITUDE)
       FPR( oUnit, "set aq2D        %s\n", "Magnitude" ); 
    else if (i == FD_TPPI)
       FPR( oUnit, "set aq2D        %s\n", "TPPI" ); 
    else if (i == FD_REAL)
       FPR( oUnit, "set aq2D        %s\n", "Real" ); 
    else if (i == FD_IMAGE)
       FPR( oUnit, "set aq2D        %s\n", "Image" ); 
    else if (i == FD_COMPLEX || i == FD_STATES)
       FPR( oUnit, "set aq2D        %s\n", "States" ); 
    else if (dimCount > 1 && quadSize[0] != 1)
       FPR( oUnit, "set aq2D        %s\n", "Complex" ); 
    else
       FPR( oUnit, "set aq2D        %s\n", "Real" ); 

    FPR( oUnit, "set temperature %.3f\n", pTemperature );
    FPR( oUnit, "\n" );

    for( i = 0; i < dimCount; i++ )
       {
        (void) strcpy( ctemp, getParmStr( fdata, NDLABEL, i+1 ));

        FPR( oUnit, "set N%c     %d\n",   dimListL[i], outSize[i]*quadSize[i] );
        FPR( oUnit, "set T%c     %d\n",   dimListL[i], outSize[i] );
        FPR( oUnit, "set SW%c    %.3f\n", dimListL[i], getDeltaSW( i ));
        FPR( oUnit, "set ORIG%c  %.3f\n", dimListL[i], getDeltaOrig( i ));
        FPR( oUnit, "set OBS%c   %.3f\n", dimListL[i], getDeltaObs( i ));
        FPR( oUnit, "set CAR%c   %.3f\n", dimListL[i], getDeltaCar( i ));
        FPR( oUnit, "set LAB%c   %s\n",   dimListL[i], ctemp );

        FPR( oUnit,
             "set MODE%c  %s\n", dimListL[i], 
             quadSize[i] == 1 ? "Real" : "Complex" );

        FPR( oUnit,
             "set FT%c    %s\n", dimListL[i], 
             isDeltaTimeDomain( i ) ? "Time" : "Freq" );
       }

    FPR( oUnit, "\n" );
    FPR( oUnit, "set dfSize %d\n",   dfFinalSize );
    FPR( oUnit, "set dfVal  %.6f\n", dfVal );
    FPR( oUnit, "set trVal  %e\n",   trVal );
    FPR( oUnit, "\n" );

    if (nmrPipeInfoStr)
       {
        FPR( oUnit, "set NMRPipe_Info \"%s\"\n", '"', nmrPipeInfoStr );
        FPR( oUnit, "\n" );
       }

    FPR( oUnit, "# END TCL\n" );
    FPR( oUnit, "#\n" );
    FPR( oUnit, "\n" );

    return( 0 );
}

/***/
/* Allocate and Read a section from the delta input.
/***/

int getDeltaBuff( buff, finalSize, buffName, size, startLoc, extra )

   char **buff, *buffName;
   int  *finalSize, size, startLoc, extra;
{
    int n, error;

    if (size < 1) return( 0 );
    if (!buff) return( 0 );

/***/
/* Read forward if buffer bound was not yet reached:
/***/

    if (bytesRead < startLoc)
       {
        n = startLoc - bytesRead;

	if (error = workAlloc( n ))
           {
            FPR( stderr, "Delta Error Allocating %s Alignment Buffer.\n", buffName );
	    return( error );
           }

        if (*inName)
           error = dataReadC( inUnit, workBuff, n );
        else
           error = dataReadCB( inUnit, workBuff, n, -1, 0 );

        if (error)
           {
            FPR( stderr, "Delta Error Reading %s Alignment Buffer.\n", buffName );
            return( error );
           }

        bytesRead += n;
       }

    if (bytesRead != startLoc)
       {
        FPR( stderr, "Delta Warning: Misaligned %s Buffer:\n", buffName );
        FPR( stderr, " Current Byte: %d Expected: %d\n", bytesRead, startLoc );
       }

    if (!(*buff = charAlloc( "delta", size+extra )))
       {
        FPR( stderr, "Delta Error Allocating Delta %s Buffer.\n", buffName );
        return( 1 );
       }

    *finalSize = size + extra;

    if (*inName)
       error = dataReadC( inUnit, *buff, size );
    else
       error = dataReadCB( inUnit, *buff, size, -1, 0 );

    if (error)
       {
        FPR( stderr, "Delta Error Reading Delta %s Buffer.\n", buffName );
        return( error );
       }

    bytesRead += size;

    return( 0 );
}

/***/
/* Null-initialize contents of a header item.
/***/

int setDeltaHdrItemNull( hPtr )

   struct DeltaHdrItem *hPtr;
{
    struct jTime *jtPtr;
    struct jUnit *juPtr;
    struct jVal  *jvPtr;
    struct jExtendedUnit *jePtr;

    double *dvalPtr;
    float  *fvalPtr;
    int    *ivalPtr;
    char   *sPtr;

    int i, j;

    switch( hPtr->itemType )
       {
        case JEOL_UNDEF: 
        case JEOL_TEXT: 
           sPtr = hPtr->dest;

           for( j = 0; j < hPtr->itemCount; j++ ) 
              {
               for( i = 0; i <= NAMELEN; i++ ) sPtr[i] = '\0';
               sPtr += (NAMELEN+1);
              }
           break;

        case JEOL_1BIT:
        case JEOL_2BIT:
        case JEOL_4BIT:
        case JEOL_6BIT:
        case JEOL_UINT1:
        case JEOL_UINT2:
        case JEOL_UINT4:
        case JEOL_INT2:
        case JEOL_INT4:
        case JEOL_UINT8:
           ivalPtr = (int *)hPtr->dest; 
           for( i = 0; i < hPtr->itemCount; i++ ) ivalPtr[i] = 0;
           break;

        case JEOL_FLOAT:
           fvalPtr = (float *)hPtr->dest; 
           for( i = 0; i < hPtr->itemCount; i++ ) fvalPtr[i] = 0.0;
           break;

        case JEOL_DOUBLE:
           dvalPtr = (double *)hPtr->dest; 
           for( i = 0; i < hPtr->itemCount; i++ ) dvalPtr[i] = 0.0;
           break;

        case JEOL_JTIME:
           jtPtr = (struct jTime *)hPtr->dest;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               jtPtr->year    = 0;
               jtPtr->month   = 0;
               jtPtr->day     = 0;
               jtPtr->dayFrac = 0;
               jtPtr->hour    = 0;
               jtPtr->min     = 0;
               jtPtr->sec     = 0;
               jtPtr++;
              }
           break;

        case JEOL_JUNIT:
           juPtr = (struct jUnit *)hPtr->dest;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               juPtr->unitType  = JEOL_SIUNIT_NONE;
               juPtr->scaleType = JEOL_SCALE_NONE;
               juPtr->unitExp   = 0;
               juPtr++;
              }
           break;

       case JEOL_JVAL:
           jvPtr = (struct jVal *)hPtr->dest;
           jvPtr = &deltaValVal;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               for( j = 0; j <= NAMELEN; j++ ) jvPtr->s[j] = '\0';

               jvPtr->i    = 0;
               jvPtr->inf  = 0;
               jvPtr->d    = 0.0;
               jvPtr->z.re = 0.0;
               jvPtr->z.im = 0.0;
               jvPtr++;
              }
           break;

        case JEOL_EUNIT:
          jePtr = (struct jExtendedUnit *)hPtr->dest;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               juPtr = jePtr->jUnitInfo;

               for( j = 0; j < JEOL_NUM_EXTUNITS; j++ )
                  {
                   juPtr->unitType  = JEOL_SIUNIT_NONE;
                   juPtr->scaleType = JEOL_SCALE_NONE;
                   juPtr->unitExp   = 0;
                   juPtr++;
                  }

               jePtr->unitExp = 0;
               jePtr++;
              }
           break;

        default:
           break;
       }

   return( 0 );
}

/***/
/* Extract contents of a header item from delta input buffer.
/***/

int getDeltaHdrItem( hPtr, csrc, swapFlag )

   struct DeltaHdrItem *hPtr;
   int    swapFlag;
   char   *csrc;
{
    struct jTime *jtPtr;
    struct jUnit *juPtr;
    struct jVal  *jvPtr;
    struct jExtendedUnit *jePtr;

    unsigned char *src;

    double *dvalPtr;
    float  *fvalPtr;
    int    *ivalPtr;
    char   *sPtr;

    char ctemp[NAMELEN+1];
    int  i, j, k, n;

    src = (unsigned char *)csrc;

    i = 0;
    j = 0;
    k = 0;
    n = 0;

    switch( hPtr->itemType )
       {
        case JEOL_UNDEF: 
        case JEOL_TEXT: 
           n    = hPtr->itemCount > 0 ? hPtr->byteCount/hPtr->itemCount : 0;
           sPtr = hPtr->dest;

           for( j = 0; j < hPtr->itemCount; j++ )
              {
               for( i = 0; i <= n; i++ ) ctemp[i] = '\0';
               for( i = 0; i <  n; i++ ) ctemp[i] = src[k++];

               (void) trimString( sPtr, ctemp );

               sPtr += (NAMELEN+1);
              }
           break;

        case JEOL_1BIT:
           ivalPtr = (int *)hPtr->dest; 
           k       = hPtr->bitOffset;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               ivalPtr[i] = (src[j] >> (7-k)) & 1;

               k++;

               if (k == 8)
                  {
                   k = 0;
                   j++;
                  }
              }
           break;

        case JEOL_2BIT:
           ivalPtr = (int *)hPtr->dest;
           k       = hPtr->bitOffset;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               ivalPtr[i] = (src[j] >> (6-k)) & 3;

               k++;

               if (k == 8)
                  {
                   k = 0;
                   j++;
                  }
              }
           break;

        case JEOL_4BIT:
           ivalPtr = (int *)hPtr->dest; 

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               if (!(i % 2))
                  ivalPtr[i] = (src[j] >> 4) & 15;
               else
                  ivalPtr[i] = src[j++] & 15;
              }
           break;

        case JEOL_6BIT:
           ivalPtr = (int *)hPtr->dest;
           k       = hPtr->bitOffset;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               ivalPtr[i] = (src[j] >> (2-k)) & 31;

               k++;

               if (k == 8)
                  {
                   k = 0;
                   j++;
                  }
              }

           break;

        case JEOL_UINT1:
           ivalPtr = (int *)hPtr->dest;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               ivalPtr[i] = src[i];
              }
           break;

        case JEOL_UINT2:
           ivalPtr = (int *)hPtr->dest;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               ivalPtr[i] = uc2uint2( src, swapFlag );
               src += 2;
              }
           break;

        case JEOL_UINT4:
           ivalPtr = (int *)hPtr->dest;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               ivalPtr[i] = uc2uint4( src, swapFlag );
               src += 4;
              }
           break;

       case JEOL_INT2:
           ivalPtr = (int *)hPtr->dest;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               ivalPtr[i] = uc2int2( src, swapFlag );
               src += 2;
              }
           break;

        case JEOL_INT4:
           ivalPtr = (int *)hPtr->dest;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               ivalPtr[i] = uc2int4( src, swapFlag );
               src += 4;
              }
           break;

        case JEOL_UINT8:
           ivalPtr = (int *)hPtr->dest;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               ivalPtr[i] = uc2uint8( src, swapFlag );
               src += 8;
              }
           break;

        case JEOL_FLOAT:
           fvalPtr = (float *)hPtr->dest;

          for( i = 0; i < hPtr->itemCount; i++ )
              {
               fvalPtr[i] = uc2flt4( src, swapFlag );
               src += 4;
              }
           break;

        case JEOL_DOUBLE:
          dvalPtr = (double *)hPtr->dest;

          for( i = 0; i < hPtr->itemCount; i++ )
              {
               dvalPtr[i] = uc2flt8( src, swapFlag );
               src += 8;
              }
           break;

        case JEOL_JTIME:
           jtPtr = (struct jTime *)hPtr->dest;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
#ifdef SYS_BYTESWAP
               n = uc2uint2( src + j, 1 );
               j += 2;

               jtPtr->dayFrac = uc2uint2( src + j, 1 );;
               j += 2;
#else
               n = uc2uint2( src + j, 0 );
               j += 2;

               jtPtr->dayFrac = uc2uint2( src + j, 0 );;
               j += 2;
#endif

               jtPtr->day     = n & 31;
               jtPtr->month   = (n >> 5) & 15;
               jtPtr->year    = 1990 + ((n >> 9) & 127);

               n = jtPtr->dayFrac*JEOL_DAYFRAC;
              
               jtPtr->hour = n/(60*60); 
               ICLIP( jtPtr->hour, 0, 23 ); 
               n -= jtPtr->hour*60*60;

               jtPtr->min = n/60;      
               ICLIP( jtPtr->min, 0, 59 ); 
               n -= jtPtr->min*60;

               jtPtr->sec = n;         
               ICLIP( jtPtr->sec, 0, 59 );

               jtPtr++;
              }
           break;

       case JEOL_JVAL:
           jvPtr = (struct jVal *)hPtr->dest;
           jvPtr = &deltaValVal;

           switch( deltaParmVal.valType )
              {
               case JEOL_PARMVAL_STR:
                  for( i = 0; i <= JEOL_JVAL_STRLEN; i++ ) ctemp[i] = '\0';
                  for( i = 0; i <  JEOL_JVAL_STRLEN; i++ ) ctemp[i] = src[k++];

                  (void) trimString( jvPtr->s, ctemp );
                  break;
               case JEOL_PARMVAL_INT:
                  jvPtr->i = uc2int4( src, swapFlag );
                  break;
               case JEOL_PARMVAL_FLT:
                  jvPtr->d = uc2flt8( src, swapFlag );
                  break;
               case JEOL_PARMVAL_Z:
                  jvPtr->z.re = uc2flt8( src, swapFlag );
                  src += 8;
                  jvPtr->z.im = uc2flt8( src, swapFlag );
                  break;
               case JEOL_PARMVAL_INF:
                  jvPtr->inf = uc2int4( src, swapFlag );
                  break;
               default:
                  break;
              }
           break;

        case JEOL_JUNIT:
           juPtr = (struct jUnit *)hPtr->dest;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               juPtr->unitExp = src[j] & 15;
               if (juPtr->unitExp > 7) juPtr->unitExp -= 16;

               juPtr->scaleType = (src[j++] >> 4) & 15;

               if (juPtr->scaleType > 7) juPtr->scaleType -= 16;

               juPtr->unitType  = src[j++];
               juPtr++;
              }
           break;

        case JEOL_EUNIT:
          jePtr = (struct jExtendedUnit *)hPtr->dest;

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               jePtr->unitExp = uc2uint2( src + j, swapFlag );
               if (jePtr->unitExp > 32767) juPtr->unitExp -= 65536;
               juPtr = jePtr->jUnitInfo;
               j += 2;

               for( j = 0; j < JEOL_NUM_EXTUNITS; j++ )
                  {
                   juPtr->scaleType = (src[j] >> 4) & 15;
                   if (juPtr->scaleType > 7) juPtr->scaleType -= 16;

                   juPtr->unitExp = src[j++] & 15;
                   if (juPtr->unitExp > 7) juPtr->unitExp -= 16;

                   juPtr->unitType  = src[j++];
                   juPtr++;
                  }

               jePtr++;
              }
           break;

        default:
           break;
       }

   return( 0 );
}

/***/
/* Display header item from delta input buffer.
/***/

int showDeltaHdrItem( hPtr )

   struct DeltaHdrItem *hPtr;
{
    struct jTime *jtPtr;
    struct jUnit *juPtr;
    struct jVal  *jvPtr;
    struct jExtendedUnit *jePtr;

    double *dvalPtr;
    float  *fvalPtr;
    int    *ivalPtr;
    char   *sPtr;

    int    i, j, n, itemCount;
    char   ctemp[NAMELEN+1];

    SPR( ctemp, "%s %d:", hPtr->itemName, hPtr->itemCount );
    FPR( vUnit, "%-24s ", ctemp );

    itemCount = hPtr->itemCount == JMAXDIM ? dimCount : hPtr->itemCount;

    i = 0;
    j = 0;
    n = 0;

    switch( hPtr->itemType )
       {
        case JEOL_UNDEF: 
           sPtr = hPtr->dest;
           n    = hPtr->byteCount/hPtr->itemCount;

           ICLIP( n, 0, NAMELEN );

           for( j = 0; j < hPtr->itemCount; j++ )
              {
               FPR( vUnit, " " );

	       if (n)
		  { 
                   for( i = 0; i < n; i++ ) 
                      {
                       FPR( vUnit, "%d", (unsigned int)sPtr[i] );
                       if (i < n - 1) FPR( vUnit, "," );
		      }
                  }
	       else
		  {
                   FPR( vUnit, "0" );
                  }

               sPtr += (NAMELEN+1);
              }

           FPR( vUnit, "\n" );
           break;

        case JEOL_TEXT: 
           sPtr = hPtr->dest;

           for( j = 0; j < itemCount; j++ )
              {
               FPR( vUnit, " \"%s\"", sPtr );
               sPtr += (NAMELEN+1);
              }

           FPR( vUnit, "\n" );
           break;

        case JEOL_1BIT:
        case JEOL_2BIT:
        case JEOL_4BIT:
        case JEOL_6BIT:
        case JEOL_UINT1:
        case JEOL_UINT2:
        case JEOL_UINT4:
        case JEOL_INT2:
        case JEOL_INT4:
        case JEOL_UINT8:
           ivalPtr = (int *)hPtr->dest; 

           for( j = 0; j < hPtr->itemCount; j++ )
              {
               FPR( vUnit, " %d", ivalPtr[j] );
              }

           if (hPtr->nameList)
              {
               FPR( vUnit, " (" );

               for( j = 0; j < itemCount; j++ ) /* hPtr->itemCount */
                  {
                   if (j < itemCount)
		      {
                       FPR( vUnit, "%s", 
                            GetKeyword( hPtr->nameList, ivalPtr[j] ) );
		      }
		   else 
                      {
                       FPR( vUnit, "." );
                      }

                   if (j < itemCount - 1) FPR( vUnit, " " );
                  }

               FPR( vUnit, ")" );
              }

           FPR( vUnit, "\n" );
           break;

        case JEOL_FLOAT:
           fvalPtr = (float *)hPtr->dest; 

           for( j = 0; j < dimCount; j++ )
              {
               FPR( vUnit, " %.7e", fvalPtr[j] );
              }

           FPR( vUnit, "\n" );
           break;

        case JEOL_DOUBLE:
           dvalPtr = (double *)hPtr->dest; 

           for( j = 0; j < dimCount; j++ )
              {
               FPR( vUnit, " %.11e", dvalPtr[j] );
              }

           FPR( vUnit, "\n" );
           break;

        case JEOL_JTIME:
           jtPtr = (struct jTime *)hPtr->dest;

           for( j = 0; j < hPtr->itemCount; j++ )
              {
               FPR( vUnit, 
                    " %s/%d/%d %d:%d %d\n", 
                    GetKeyword(monthList,jtPtr->month), jtPtr->day, jtPtr->year,
                    jtPtr->hour, jtPtr->min, jtPtr->sec );

               jtPtr++;
              }
           break;

/* Single value, based on type in current jVal structure: */

        case JEOL_JVAL:
           jvPtr = (struct jVal *)hPtr->dest;
           jvPtr = &deltaValVal;

           switch( deltaParmVal.valType )
              {
               case JEOL_PARMVAL_STR:
                  FPR( vUnit, " %s\n", jvPtr->s );
                  break;
               case JEOL_PARMVAL_INT:
                  FPR( vUnit, " %d\n", jvPtr->i );
                  break;
               case JEOL_PARMVAL_FLT:
                  FPR( vUnit, " %.11e\n", jvPtr->d );
                  break;
               case JEOL_PARMVAL_Z:
                  FPR( vUnit, " %.11e %.11e\n", jvPtr->z.re, jvPtr->z.im );
                  break;
               case JEOL_PARMVAL_INF:
                  FPR( vUnit, 
                       " %d (%s)\n", 
                       jvPtr->inf, GetKeyword( jeolInfList, jvPtr->inf ));
                  break;
               default:
                  FPR( vUnit, " (UnknownType)\n" );
                  break;
              }
           break;

        case JEOL_JUNIT:
           juPtr = (struct jUnit *)hPtr->dest;

           for( j = 0; j < hPtr->itemCount; j++ )
              {
               FPR( vUnit, " %d", juPtr[j].unitType );
              }

           FPR( vUnit, " (" );

          for( j = 0; j < itemCount; j++ )
              {
               FPR( vUnit, "%s^%d", 
                    GetKeyword( jeolUnitTypeList, juPtr[j].unitType ), 
                    juPtr->unitExp );

               if (j < itemCount - 1) FPR( vUnit, " " );
              }

           FPR( vUnit, ")\n" );

           SPR( ctemp, "%s %d:", "DataScale", hPtr->itemCount );
           FPR( vUnit, "%-24s ", ctemp );

           for( j = 0; j < hPtr->itemCount; j++ )
              {
               FPR( vUnit, " %d", juPtr[j].scaleType );
              }

           FPR( vUnit, " (" );

           for( j = 0; j < itemCount; j++ )
              {
               FPR( vUnit, "%s", 
                    GetKeyword( jeolUnitScaleList, juPtr[j].scaleType ) );

               if (j < itemCount - 1) FPR( vUnit, " " );
              }

           FPR( vUnit, ")\n" );
           break;

/* Initial header only: */

        case JEOL_EUNIT:
	   for( i = 0; i < JMAXDIM; i++ )
	      {
	       if (deltaHdr.unitLoc[i]) n++;
	      }

	   if (!n) 
              {
               for( j = 0; j < hPtr->itemCount; j++ ) FPR( vUnit, " 0" );
               FPR( vUnit, "\n" );
               break;
              }

           jePtr = (struct jExtendedUnit *)hPtr->dest;

           FPR( vUnit, "EExp: %d\n", jePtr->unitExp );

           for( i = 0; i < hPtr->itemCount; i++ )
              {
               juPtr = jePtr->jUnitInfo;

               for( j = 0; j < JEOL_NUM_EXTUNITS; j++ )
                  {
                   FPR( vUnit,
                        " EUnit %d: Unit %2d (%s) Scale % 2d (%s) Exp %d\n",
                        j+1,
                        juPtr->unitType,  
                        GetKeyword( jeolUnitTypeList, juPtr->unitType ),
                        juPtr->scaleType, 
                        GetKeyword( jeolUnitScaleList, juPtr->scaleType ),
                        juPtr->unitExp );

                   juPtr++;
                  }

               jePtr++;
              }
           break;

        default:
           FPR( vUnit, "\n" );
           break;
       }

   return( 0 );
}

/***/
/* Type conversions, with byte swapping.
/***/

static double uc2flt8( s, swapFlag )

   unsigned char *s;
{
   union {double d; char c[8];} u;

   if (swapFlag)
      {
       u.c[7] = s[0];
       u.c[6] = s[1];
       u.c[5] = s[2];
       u.c[4] = s[3];
       u.c[3] = s[4];
       u.c[2] = s[5];
       u.c[1] = s[6];
       u.c[0] = s[7];
      }
   else
      {
       u.c[7] = s[7];
       u.c[6] = s[6];
       u.c[5] = s[5];
       u.c[4] = s[4];
       u.c[3] = s[3];
       u.c[2] = s[2];
       u.c[1] = s[1];
       u.c[0] = s[0];
      }

   return( u.d );
}

static float uc2flt4( s, swapFlag )

   unsigned char *s;
{
   union {float f; char c[4];} u;

   if (swapFlag)
      {
       u.c[3] = s[0];
       u.c[2] = s[1];
       u.c[1] = s[2];
       u.c[0] = s[3];
      }
   else
      {
       u.c[3] = s[3];
       u.c[2] = s[2];
       u.c[1] = s[1];
       u.c[0] = s[0];
      }

   return( u.f );
}

static int uc2uint8( s, swapFlag )

   unsigned char *s;
{
   union {unsigned long long i; char c[8];} u;
   int   i;

   if (swapFlag)
      {
       u.c[7] = s[0];
       u.c[6] = s[1];
       u.c[5] = s[2];
       u.c[4] = s[3];
       u.c[3] = s[4];
       u.c[2] = s[5];
       u.c[1] = s[6];
       u.c[0] = s[7];
      }
   else
      {
       u.c[7] = s[7];
       u.c[6] = s[6];
       u.c[5] = s[5];
       u.c[4] = s[4];
       u.c[3] = s[3];
       u.c[2] = s[2];
       u.c[1] = s[1];
       u.c[0] = s[0];
      }

   i = (int)u.i;

   return( i );
}

static int uc2uint4( s, swapFlag )

   unsigned char *s;
{
   union {unsigned int i; char c[4];} u;
   int   i;

   if (swapFlag)
      {
       u.c[3] = s[0];
       u.c[2] = s[1];
       u.c[1] = s[2];
       u.c[0] = s[3];
      }
   else
      {
       u.c[3] = s[3];
       u.c[2] = s[2];
       u.c[1] = s[1];
       u.c[0] = s[0];
      }

   i = (int)u.i;

   return( i );
}

static int uc2int4( s, swapFlag )

   unsigned char *s;
{
   union {int i; char c[4];} u;
   int   i;

   if (swapFlag)
      {
       u.c[3] = s[0];
       u.c[2] = s[1];
       u.c[1] = s[2];
       u.c[0] = s[3];
      }
   else
      {
       u.c[3] = s[3];
       u.c[2] = s[2];
       u.c[1] = s[1];
       u.c[0] = s[0];
      }

   i = (int)u.i;

   return( i );
}

static int uc2uint2( s, swapFlag )

   unsigned char *s;
{
   union {unsigned short i; char c[2];} u;
   int   i;

   if (swapFlag)
      {
       u.c[1] = s[0];
       u.c[0] = s[1];
      }
   else
      {
       u.c[1] = s[1];
       u.c[0] = s[0];
      }

   i = (int)u.i;

   return( i );
}

static int uc2int2( s, swapFlag )

   unsigned char *s;
{
   union {short i; char c[2];} u;
   int   i;

   if (swapFlag)
      {
       u.c[1] = s[0];
       u.c[0] = s[1];
      }
   else
      {
       u.c[1] = s[1];
       u.c[0] = s[0];
      }

   i = (int)u.i;

   return( i );
}

/***/
/* Remove leading/trailing space from string.
/***/

static int trimString( dest, src )

   char *dest, *src;
{
    char *sPtr, *lastPtr;

    if (!dest) return( 0 );

    if (!src)
       {
        *dest = '\0';
        return( 0 );
       }

    lastPtr = src;

    while( *src && isspace( *src )) src++;

    for( sPtr = src; *sPtr; sPtr++ )
       {
        if (!isspace( *sPtr )) lastPtr = sPtr + 1;
       }

    while( src < lastPtr ) *dest++ = *src++;

    *dest++ = '\0';

    return( 0 );
}

/***/
/* Allocate or reallocate global work buffer of a mininum requested size.
/***/

static int workAlloc( size )

   int size;
{
    if (size < workSize) return( 0 );

    (void) workFree();

    if (!(workBuff = (unsigned char *)charAlloc( "delta", size ))) return( 1 );

    workSize = size;

    return( 0 );
}

/***/
/* Deallocate global work buffer.
/***/

static int workFree()
{
    (void) deAlloc( "delta", workBuff, workSize );

    workBuff = (unsigned char *)NULL;
    workSize = 0;

    return( 0 );
}

/***/
/* Interpret delta word size.
/***/

static int getDeltaWordSize()
{
    if (!wordSizeIn)
       {
        if (deltaHdr.dataType == JEOL_DATATYPE_DOUBLE) 
  	   wordSizeIn = sizeof(double);
	else if (deltaHdr.dataType == JEOL_DATATYPE_FLOAT)
  	   wordSizeIn = sizeof(float);
       }

    if (!wordSizeIn)
       {
        wordSizeIn  = totalInSize ? deltaHdr.dataLength/totalInSize : 0;
        wordSizeIn /= channelCount;

        if (wordSizeIn >= 3 && wordSizeIn <= 5) wordSizeIn = 4;
        if (wordSizeIn >= 7 && wordSizeIn <= 9) wordSizeIn = 8;
       }

    if (!wordSizeOut) wordSizeOut = sizeof(float);

    return( 0 );
}

/***/
/* Interpret delta submatrix size.
/***/

static int getSMXSize()
{
    int i;

    for( i = 0; i < JMAXDIM; i++ ) smxSize[i] = 1;

    switch( deltaHdr.dataFormat )
       {
        case JEOL_FORMAT_1D:
           for( i = 0; i < dimCount; i++ ) smxSize[i] = 8;
           break;

        case JEOL_FORMAT_2D:
           for( i = 0; i < dimCount; i++ ) smxSize[i] = 32;
           break;

        case JEOL_FORMAT_3D:
        case JEOL_FORMAT_4D:
           for( i = 0; i < dimCount; i++ ) smxSize[i] = 8;
           break;

        case JEOL_FORMAT_5D:
        case JEOL_FORMAT_6D:
           for( i = 0; i < dimCount; i++ ) smxSize[i] = 4;
           break;

        case JEOL_FORMAT_7D:
        case JEOL_FORMAT_8D:
           for( i = 0; i < dimCount; i++ ) smxSize[i] = 2;
           break;

        case JEOL_FORMAT_SMALL2D:
        case JEOL_FORMAT_SMALL3D:
        case JEOL_FORMAT_SMALL4D:
           for( i = 0; i < dimCount; i++ ) smxSize[i] = 4;
           break;

        case JEOL_FORMAT_9:
        case JEOL_FORMAT_10:
        case JEOL_FORMAT_11:
        default:
           break;
       }

    return( 0 );
}

/***/
/* space2us: copy a version of src with spaces converted to underscores.
/***/

static char *space2us( dest, src )

   char *dest, *src;
{
    if (!src) return( src );

    do
       {
        *dest++ = (*src && isspace( *src )) ? '_' : *src; 
       }
    while( *src++ );

    return( dest );
}

/***/
/* openNMR: open NMRPipe format output file, write an initial header.
/***/

static int openNMR( outName, plane, iz, ia, fdata )

   float fdata[FDATASIZE];
   char  *outName;
   int   plane, iz, ia;
{
    char dirName[NAMELEN+1];

    if (dimCount == 4)
       {
        if (isAZFmt( outFmt )) 
 	   SPR( outName, outFmt, ia, iz );
	else
 	   SPR( outName, outFmt, plane );
       }
    else
       {
 	SPR( outName, outFmt, plane );
       }

    if (*cmndName)
       {
        if (plane == 1)
           {
            if (verbose) FPR( stderr, "Delta Pipeline Command: %s\n", cmndName );

            if (dataPOpen( cmndName, &outUnit, FB_WRITE )) return( 1 );
            if (dataWrite( outUnit, fdata, sizeof(float)*FDATASIZE )) return( 1 );
           }

        return( 0 );
       }

    if (*outFmt)
       {
        if (dirFlag)
           {
            if (!dirExists( outName, dirName ))
               {
                if (verbose)
                   {
                    FPR( stderr, "Delta Notice: Creating directory %s\n", dirName );
                   }

               if (dirCreate( dirName ))
                   {
                    FPR( stderr, "Delta Warning: Error creating directory %s\n", dirName );
                   }

                dirFlag = 0;
               }
           }

        if (!ovFlag && fileExists( outName ))
           {
            FPR( stderr, "Delta Error: output file %s exists.\n", outName );
            return( 1 );
           }

        if (xyzFlag || plane == 1)
           {
            if (dataOpen( outName, &outUnit, FB_CREATE )) return( 1 );
           }
       }
    else
       {
        outUnit = FB_STDOUT;
       }

/***/
/* Write a header if this is a seperate plane:
/***/

    if (xyzFlag || plane == 1)
       {
        if (dataWrite( outUnit, fdata, sizeof(float)*FDATASIZE ))
           {
            return( 1 );
           }
       }

    return( 0 );
}

/***/
/* closeNMR: close explicit file output.
/***/

static int closeNMR()
{
    if (!*outFmt) return( 0 );
    if (!xyzFlag) return( 0 );

    (void) dataClose( outUnit );

    outUnit = UNIT_NULL;

    return( 0 );
}

/***/
/* Interpret JEOL SI units scale.
/***/

static double getDeltaUnitVal( val, juPtr )

   struct jUnit *juPtr;
   double val;
{
    switch( juPtr->scaleType ) 
       {
        case JEOL_SCALE_YOTTA:
           val *= 1.0e+24;
	   break;
        case JEOL_SCALE_ZETTA:
           val *= 1.0e+21;
	   break;
        case JEOL_SCALE_EXA:
           val *= 1.0e+18;
	   break;
        case JEOL_SCALE_PETA:
           val *= 1.0e+15;
	   break;
        case JEOL_SCALE_TERA:
           val *= 1.0e+12;
	   break;
        case JEOL_SCALE_GIGA:
           val *= 1.0e+9;
	   break;
        case JEOL_SCALE_MEGA:
           val *= 1.0e+6;
	   break;
        case JEOL_SCALE_KILO:
           val *= 1.0e+3;
	   break;
        case JEOL_SCALE_NONE:
           val *= 1.0;
	   break;
        case JEOL_SCALE_MILLI:
           val *= 1.0e-3;
	   break;
        case JEOL_SCALE_MICRO:
           val *= 1.0e-6;
	   break;
        case JEOL_SCALE_NANO:
           val *= 1.0e-9;
	   break;
        case JEOL_SCALE_PICO:
           val *= 1.0e-12;
	   break;
        case JEOL_SCALE_FEMTO:
           val *= 1.0e-15;
	   break;
        case JEOL_SCALE_ATTO:
           val *= 1.0e-18;
	   break;
        case JEOL_SCALE_ZEPTO:
           val *= 1.0e-21;
	   break;
	default:
	   break;
       }

    if (juPtr->unitExp != 0 && juPtr->unitExp != 1)
       { 
        val = pow( val, (double)juPtr->unitExp );
       }

    return( val );
}

/***/
/* Interpret data type.
/***/

static int isDeltaQuad( dim )

   int dim;
{
   if (deltaHdr.axisType[dim] == JEOL_AXISTYPE_COMPLEX ||
       deltaHdr.axisType[dim] == JEOL_AXISTYPE_ENVELOPE) return( 1 );

   if (dim == 0 && deltaHdr.axisType[dim] == JEOL_AXISTYPE_REAL_COMPLEX) return( 1 );

   return( 0 );
}

/***/
/* Interpret data domain and domain units.
/***/

static int isDeltaTimeDomain( dim )

   int dim;
{
    if (deltaHdr.unitList[dim].unitType == JEOL_SIUNIT_SECONDS)
       {
        if (deltaHdr.unitList[dim].unitExp == 0) return( 1 );
        if (deltaHdr.unitList[dim].unitExp == 1) return( 1 );
       }

    if (deltaHdr.unitList[dim].unitType == JEOL_SIUNIT_HZ)
       {
        if (deltaHdr.unitList[dim].unitExp == -1) return( 1 );
       }

    if (deltaHdr.unitList[dim].unitType == JEOL_SIUNIT_PPM)
       {
        if (deltaHdr.unitList[dim].unitExp == -1) return( 1 );
       }

    return( 0 );
}

static int isDeltaPPM( dim )

   int dim;
{
    if (deltaHdr.unitList[dim].unitType == JEOL_SIUNIT_PPM)
       {
        if (deltaHdr.unitList[dim].unitExp == 0) return( 1 );
        if (deltaHdr.unitList[dim].unitExp == 1) return( 1 );
       }

    return( 0 );
}

static int isDeltaHz( dim )

   int dim;
{
    if (deltaHdr.unitList[dim].unitType == JEOL_SIUNIT_HZ)
       {
        if (deltaHdr.unitList[dim].unitExp == 0) return( 1 );
        if (deltaHdr.unitList[dim].unitExp == 1) return( 1 );
       }

    if (deltaHdr.unitList[dim].unitType == JEOL_SIUNIT_SECONDS)
       {
        if (deltaHdr.unitList[dim].unitExp == -1) return( 1 );
       }

    return( 0 );
}

/***/
/* Interpret data 2D mode.
/***/

static int getDeltaAQ2D()
{
    int aq2DMode;

    if (dimCount < 2) return( 0 );

    aq2DMode = FD_REAL;

    if (quadSize[0] == 2 && quadSize[1] == 2)
       aq2DMode = FD_STATES;
    else if (deltaHdr.axisType[1] == JEOL_AXISTYPE_TPPI)
       aq2DMode = FD_TPPI;
    else if (deltaHdr.axisType[0] == JEOL_AXISTYPE_REAL_COMPLEX)
       aq2DMode = FD_MAGNITUDE;

    return( aq2DMode );
}

/***/
/* Interpret various calibration parameters.
/***/

static float getDeltaSW( dim )

   int dim;
{
    float t, s1, s2, sw;
    int   n;

    n  = deltaHdr.offsetStop[dim] - deltaHdr.offsetStart[dim];
    sw = 0.0;

    s1 = getDeltaUnitVal( deltaHdr.axisStart[dim], &deltaHdr.unitList[dim] );
    s2 = getDeltaUnitVal( deltaHdr.axisStop[dim],  &deltaHdr.unitList[dim] );
    t  = fabs( s1 - s2 );

    if (isDeltaTimeDomain( dim ))
       {
        if (pSW[dim] != 0.0)
           sw = pSW[dim];
        else
           sw = t == 0.0 ? n : n/t;
       }
    else if (isDeltaPPM( dim ))
       {
        sw = t*getDeltaObs( dim );
       }
    else if (isDeltaHz( dim ))
       {
        sw = t;
       }

    if (sw == 0.0) sw = pSW[dim]; 
    if (sw == 0.0) sw = n;
    if (sw == 0.0) sw = 1.0;

    return( sw );
}

static float getDeltaOrig( dim )

   int dim;
{
    float orig, s, sw;
    int   n;

    orig = 0.0;

    s  = getDeltaUnitVal( deltaHdr.axisStop[dim],  &deltaHdr.unitList[dim] );
    sw = getDeltaSW( dim );
    n  = outSize[dim] == 0.0 ? 1 : outSize[dim];

    if (isDeltaTimeDomain( dim ))
       {
        orig = pOrig[dim];
       }
    else if (isDeltaPPM( dim ))
       {
        orig  = s*getDeltaObs( dim );
	orig += 0.5*sw/n;
       }
    else if (isDeltaHz( dim ))
       {
        orig  = s;
	orig += 0.5*sw/n;
       }

    return( orig );
}

static float getDeltaObs( dim )

   int dim;
{
    float obs;

    obs = deltaHdr.baseFreq[dim];

    if (obs == 0.0) obs = pObs[dim];
    if (obs == 0.0) obs = 1.0;

    return( obs );
}

static float getDeltaCar( dim )

   int dim;
{
   float sw, obs, z, car;

    if (isDeltaTimeDomain( dim ))
       {
        sw  = getDeltaSW( dim );
        obs = getDeltaObs( dim );
        z   = deltaHdr.zeroPoint[dim];
        car = sw*z/obs;
       }
    else
       {
        car = pCar[dim];
       }

    return( car );
}

static float getDeltaTemperature()
{
   float t;
   t = pTemperature;
   return( t );
}

static int getDeltaDFSize( thisXSize, thisDFVal )

   int   thisXSize;
   float thisDFVal;
{
    int iShift, thisDFSize;

    thisDFSize = thisXSize;
    iShift     = (int)thisDFVal;
    thisDFSize = thisXSize - (iShift + 2);

    return( thisDFSize );
}

static float getDeltaDF( orders, factors )

   int orders[JMAXDIM], factors[JMAXDIM];
{
   int   i, j, stages;
   float dp, p, s;

   stages = orders[0];
   s = 0.0;

   for( i = 0; i < stages; i ++ )
      {
       p = 1.0;

       for ( j = i; j < stages - 1; j++ ) p *= factors[j];
       if (p == 0.0) p = 1.0;

       s += (orders[i+1] - 1)/p;
      }

    if (factors[stages-1] == 0.0)
       dp = 0.5*s;
    else
       dp = 0.5*s/factors[stages-1];

    return( dp );
}

/***/
/* Primary command-line argument extraction;
/* note that arguments may be extracted elsewhere as well.
/***/

static int getDeltaArgs( argc, argv, fdata )

   int  argc;
   char **argv;
   float fdata[FDATASIZE];
{
 if (flagLoc( argc, argv, "-help" ))
 {
 FPR_VERSION;
 FPR( stderr, "delta2pipe: JEOL Delta to NMRPipe Automated Conversion Version  2018.204.14.00.\n" );
 FPR( stderr, " -in  inName       Delta jdf Input.\n" );
 FPR( stderr, " -out outTemplate  NMRPipe Output File or Template;\n" );
 FPR( stderr, "                   Template Required for 3D and 4D.\n" );
 FPR( stderr, " -ov               Force Overwrite.\n" );
 FPR( stderr, " -nodir            Suppress Directory Creation.\n" );
 FPR( stderr, " -nofs             Suppress File Size Check.\n" );
 FPR( stderr, "Special Digital Filter Parameters:\n" );
 FPR( stderr, " -df               Perform DF Adjustment During Conversion.\n" );
 FPR( stderr, " -nodf             Perform DF Adjustment During Processing.\n" );
 FPR( stderr, " -realOnly         All-Real Output Only, No DF Adjustment.\n" );
 FPR( stderr, " -dfVal  df [Auto] DF Adjustment Value, Points.\n" );
 FPR( stderr, " -trVal  tr [Auto] TR Adjustment Value, Fraction.\n" );
 FPR( stderr, " -skip   sV [-1]   Points to Skip in Head Correction.\n");
 FPR( stderr, "                    (Use -1 for No Correction)\n" );
 FPR( stderr, " -zf               Temporary Zero Fill for Speed.\n" );
 FPR( stderr, "                    (Default)\n" );
 FPR( stderr, " -nozf             No Temporary Zero Fill.\n" );
 FPR( stderr, "Informational Output:\n" );
 FPR( stderr, " -showHdr          Show Delta Header Info.\n" );
 FPR( stderr, " -showParm         Show Delta Parameter Info.\n" );
 FPR( stderr, " -showRuler        Show Delta Ruler Info.\n" );
 FPR( stderr, " -verb             Show Conversion Status.\n" );
 FPR( stderr, " -all              Show All Info Above.\n" );
 FPR( stderr, " -info             Show Info Only, No Conversion.\n" );
 FPR( stderr, " -showTCL          Show TCL Conversion Text.\n" );
 FPR( stderr, "General Conversion Arguments:\n" );
 FPR( stderr, " -nohdr            Ignore General Header Arguments:\n" );

 (void) parseHdr( argc, argv, fdata );
 return( 1 );
}

    if (flagLoc( argc, argv, "-all" ))
       {
	verbose       = 1;
	showHdrFlag   = 1;
	showParmFlag  = 1;
	showRulerFlag = 1;
       }

    if (flagLoc( argc, argv, "-verb" ))        verbose  = 1;
    if (flagLoc( argc, argv, "-noverb" ))      verbose  = 0;

    if (flagLoc( argc, argv, "-showTCL" ))     showTCLFlag   = 1;
    if (flagLoc( argc, argv, "-noshowTCL" ))   showTCLFlag   = 0;

    if (flagLoc( argc, argv, "-showHdr" ))     showHdrFlag   = 1;
    if (flagLoc( argc, argv, "-noshowHdr" ))   showHdrFlag   = 0;

    if (flagLoc( argc, argv, "-showParm" ))    showParmFlag  = 1;
    if (flagLoc( argc, argv, "-noshowParm" ))  showParmFlag  = 0;

    if (flagLoc( argc, argv, "-showRuler" ))   showRulerFlag = 1;
    if (flagLoc( argc, argv, "-noshowRuler" )) showRulerFlag = 0;

    if (flagLoc( argc, argv, "-df" ))     dfCorrectFlag  = 1;
    if (flagLoc( argc, argv, "-nodf" ))   dfCorrectFlag  = 0;

    if (flagLoc( argc, argv, "-zf" ))     zfFlag  = 1;
    if (flagLoc( argc, argv, "-nozf" ))   zfFlag  = 0;

    if (flagLoc( argc, argv, "-ov" ))     ovFlag  = 1;
    if (flagLoc( argc, argv, "-noov" ))   ovFlag  = 0;

    if (flagLoc( argc, argv, "-tty" ))    ttyFlag  = 1;
    if (flagLoc( argc, argv, "-notty" ))  ttyFlag  = 0;

    if (flagLoc( argc, argv, "-hdr" ))    hdrFlag  = 1;
    if (flagLoc( argc, argv, "-nohdr" ))  hdrFlag  = 0;

    if (flagLoc( argc, argv, "-info" ))   infoFlag = 1;
    if (flagLoc( argc, argv, "-noinfo" )) infoFlag = 0;

    if (flagLoc( argc, argv, "-noIn" ))   noInFlag  = 1;
    if (flagLoc( argc, argv, "-noOut" ))  noOutFlag = 0;

    if (flagLoc( argc, argv, "-nofs" ))   checkFlag = 0;
    if (flagLoc( argc, argv, "-nodir" ))  dirFlag   = 0;

    if (flagLoc( argc, argv, "-realOnly" ))    realOnlyFlag = 1;
    if (flagLoc( argc, argv, "-norealOnly" ))  realOnlyFlag = 0;

    if (realOnlyFlag) dfCorrectFlag = 0;

    (void) strArgD( argc, argv, "-in",  inName );
    (void) strArgD( argc, argv, "-out", outFmt );

    (void) intArgD( argc, argv, "-inws",  &wordSizeIn );
    (void) intArgD( argc, argv, "-outws", &wordSizeOut );
    (void) intArgD( argc, argv, "-skip",  &skipSize );

/***/
/* Test for required output format with xyz style output:
/***/

    if (isZFmt( outFmt ) || isAZFmt( outFmt )) xyzFlag = 1;

    if (xyzFlag && !*outFmt)
       {
        FPR( stderr, "Delta Error: missing output format.\n" );
        FPR( stderr, " 3D and 4D Output requires -out template, e.g.:\n" );
        FPR( stderr, " %s -out fid/test%%03d.fid\n", argv[0] );
        return( 1 );
       }

    return( 0 );
}

/***/
/* Initialize options and variables used fro conversion.
/***/

static int initDeltaVars()
{
    int i;

    verbose       = 0;
    infoFlag      = 0;
    xyzFlag       = 0;
    realOnlyFlag  = 0;
    noInFlag      = 0;
    noOutFlag     = 0;
    checkFlag     = 1;
    dirFlag       = 1;
    ttyFlag       = 0;
    hdrFlag       = 1;
    ovFlag        = 0;
    swapDataFlag  = 0;

    showHdrFlag   = 0;
    showParmFlag  = 0;
    showRulerFlag = 0;
    showTCLFlag   = 0;

    dimCount      = 0;
    channelCount  = 0;
    bytesRead     = 0;

    totalInSize   = 0;
    totalOutSize  = 0;

    wordSizeIn    = 0;
    wordSizeOut   = 0;

    vUnit       = stderr;
    inUnit      = UNIT_NULL;
    outUnit     = UNIT_NULL;

    *inName     = '\0';
    *outFmt     = '\0';
    *cmndName   = '\0';

    pTemperature = 0.0;

    dfFlag        =  0;
    dfSize        =  0;
    skipSize      = -1;
    zfFlag        =  1;
    dfCorrectFlag =  0;
    trVal         =  0.0;
    dfVal         =  0.0;

    for( i = 0; i < JMAXDIM; i++ )
       {
        inSize[i]   = 1;
        outSize[i]  = 1;
        quadSize[i] = 1;
        smxSize[i]  = 1;

        matX1[i]    = 0;
        matXN[i]    = 0;
        smxX1[i]    = 0;
        smxXN[i]    = 0;

        pSW[i]      = 0.0; 
	pObs[i]     = 0.0; 
	pCar[i]     = 0.0;
	pOrig[i]    = 0.0;

	dfOrders[i]  = 0;
	dfFactors[i] = 0;
       }

    return( 0 );
}

/***/
/* Dummy modules for compiling without an FFT library, as used in
/* digital filter adjustment.
/***/

#ifdef NO_DMX_SRC
int deltaDFNull()
{
	 return( 0 );
}
int deltaDFInit()
{
	 return( 0 );
}
int deltaDFFree()
{
	 return( 0 );
}
int deltaDF()
{
	 return( 0 );
}
#endif

/***/
/* Bottom.
/***/
