
/***/
/* NMRPipe Copyright 1992 - 2024 Frank Delaglio
/* Utilities for reading NMRPipe-format tables.
/***/

/***/
/* rdTable: read a text-table of character and numerical data,
/*          interpreted as a numerical matrix.  Related data
/*          structures are described in file "table.h".
/*
/* Data Table Format:
/*
/* The text-table must include the VARS header line, which describes
/* the variables in each column of the file, plus at least one line
/* of data.
/*
/* Variable type and output format can be described by the optional
/* FORMAT line, using C-formats of type %f,%g,%e %d and %s for float,
/* integer, and string (text) variables.  If a FORMAT line is not
/* included, all data will be interpreted as float.
/*
/* Missing character or numerical values can be indicated by use of
/* NULLSTRING and NULLVALUE lines.  NULLVALUE values should be integers.
/*
/* Tolerances in variable values can be indicted by TOLERANCE lines.
/*
/* Blank lines, and comments which begin with REMARK or # are permitted.
/*
/* The following general format rules apply:
/*    Keyword lines such as FORMAT should have no trailing spaces.
/*    Variable names and formats should not contain spaces.
/*    The file must contain one or more spaces between fields.
/*    All fields must be present in all lines.
/*
/* Example:
/*   
/*   REMARK sample data file.
/*   FORMAT %3s %3d %4.1f
/*   VARS RESNAME RESID HN(I)
/*   TOLERANCE 0 0 0.05
/*   NULLVALUE -1.0
/*   NULLSTRING *
/*   
/*   GLY 1  8.5
/*   CYS 2  9.2
/*   *   3  7.9
/*   LEU 4 -1.0
/*   CYS 5  9.3
/*
/* NOTA BENE: the data structures allocated by this procedure should be
/*            deallocated using the freeTab procedure below.
/***/ 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "stralloc.h"
#include "memory.h"
#include "table.h"
#include "token.h"
#include "prec.h"

#define FPR (void) fprintf

#define TXTFREE( P ) \
   if (P) (void)deAlloc( "strDup", P, sizeof(char)*strlen(P)+1 )

#define TXTFREE2( W, P ) \
   if (P) (void)deAlloc( W, P, sizeof(char)*strlen(P)+1 )

static char *us2space();

int rdTable( inName,            /* Data table file to read.               */
             matrixPtr,         /* On return, the data matrix.            */
             varCountPtr,       /* On return, variables per entry.        */
             entryCountPtr,     /* On return, entry count in matrix.      */
             varInfoPtr,        /* On return, describes variables.        */
             defNullValue )     /* Value to use for "missing" variables.  */

   struct tabVar **varInfoPtr;
   int           *varCountPtr, *entryCountPtr, defNullValue;
   float         **matrixPtr;
   char          *inName;
{
    int error;

    error = rdTable2( inName,
                      matrixPtr,
                      varCountPtr,
                      entryCountPtr,
                      varInfoPtr,
                      (struct hdrList **)NULL,
                      defNullValue,
                      EMPTY_TAB_BAD,
                      READ_DATA_YES );

    return( error );
}

/***/
/* rdTable2: the extended interface to table-reading; returns miscellaneous
/*           contents of text file (REMARKS and DATA) as well as the data
/*           table itself.
/***/
 
int rdTable2( inName, 		/* Data table file to read.                */ 
              matrixPtr,   	/* On return, the data matrix.             */ 
              varCountPtr,	/* On return, variables per entry.         */ 
              entryCountPtr,	/* On return, entry count in matrix.       */
              varInfoPtr,   	/* On return, describes variables.         */
              hdrBasePtr,       /* On return, misc info (if ptr not null). */
              defNullValue,     /* Value to use for "missing" variables.   */
              nullEntryFlag,    /* If empty table is OK or not.            */
              readMode )        /* Read all data or only header.           */

   struct tabVar  **varInfoPtr; 
   struct hdrList **hdrBasePtr;

   int   *varCountPtr, *entryCountPtr, defNullValue, nullEntryFlag, readMode;
   float **matrixPtr;
   char  *inName;
{
   FILE  *inUnit;

   struct tabVar *varInfo, *varPtr;
   float         *matrix, *rPtr;

   char *cPtr, *formats, *names, *tols, *line, cBuff[NAMELEN+1], 
        inBuff[NAMELEN+1], ctemp[NAMELEN+1], nullStr[NAMELEN+1];

   int  useNullString, useNullValue, nullValue, entryCount, varCount, 
        fmtCount, tolCount, var, i, inLen, cBuffLen, maxLen, dataFlag,
        error;

/***/
/* Initialize:
/***/

   error  = 0;

   inUnit        = (FILE *) NULL;
   matrix        = (float *) NULL;
   varInfo       = (struct tabVar *) NULL;

   useNullString = 0;
   useNullValue  = 0;
   entryCount    = 0;
   varCount      = 0;
   nullValue     = -99999;

   formats       = (char *)NULL;
   names         = (char *)NULL;
   tols          = (char *)NULL;

   line          = cBuff;
   inLen         = NAMELEN+1;
   cBuffLen      = NAMELEN+1;
   maxLen        = NAMELEN+1;

   cBuff[0]      = '\0';
   inBuff[0]     = '\0';

/***/
/* Open the text table file:
/***/

   if (!(inUnit = fopen( inName, "r")))
      {
       FPR( stderr, "RdTable File open error: %s.\n", inName );
       error = 1;
       goto shutdown;
      }

/***/
/* Count valid data entries (not blank, FORMAT, REMARK, etc):
/* Save FORMAT VARS TOLERANCE and NULL statements along the way.
/***/

    while( fgetsAlloc( &line, &maxLen, inUnit, inBuff, inLen, cBuff, cBuffLen ))
      {
       if (EOF != sscanf( line, "%s", ctemp ))
          {
           if (!strcmp( "FORMAT", ctemp ))
              {
               if (!formats) formats = strDup( line );
              }

           else if (!strcmp( "VARS", ctemp ))
              {
               if (!names) names = strDup( line );
              }

           else if (!strcmp( "TOLERANCE", ctemp ))
              {
               if (!tols) tols = strDup( line );
              }

           else if (!strcmp( "NULLSTRING", ctemp ))
              {
               (void) strcpy( nullStr, getToken( 2, line ));
               useNullString = 1;
              }

           else if (!strcmp( "NULLVALUE", ctemp ))
              {
               nullValue = getTokenI( 2, line );
               useNullValue = 1;
              }

           else if (isDataLine( ctemp ))
              {
               entryCount++;
              }
          }
      }

/***/
/* Stop for empty data:
/***/

    if (!entryCount)
       {
        if (nullEntryFlag == EMPTY_TAB_BAD)
           {
            FPR( stderr, "RdTable Error: no entries in %s\n", inName );
            error = 1;
           }

        goto shutdown;
       }

/***/
/* Interpret the VARS FORMAT TOLERANCE lines, part I:
/*   Count the number of variables and properties; abort if inconsistant. 
/*   Allocate space for the variable info.
/***/

    varCount = cntToken( names ) - 1;
    fmtCount = cntToken( formats ) - 1;
    tolCount = cntToken( tols ) - 1;

    if (varCount < 1)
       {
        FPR( stderr, "RdTable Error: no variables found.\n" );
        error = 1;
        goto shutdown;
       }

    if ((fmtCount > 0) && (fmtCount != varCount))
       {
        FPR( stderr, "RdTable Error: VARS/FORMAT mismatch:\n" );
        FPR( stderr, "%s\n%s\n", names, formats );
        error = 1;
        goto shutdown;
       }

    if ((tolCount > 0) && (tolCount != varCount))
       {
        FPR( stderr, "RdTable Error: VARS/TOLERANCE mismatch:\n" );
        FPR( stderr, "%s\n%s\n", names, tols );
        error = 1;
        goto shutdown;
       }

    varInfo = (struct tabVar *) 
       voidAlloc( "table", sizeof(struct tabVar)*varCount );

    if (!varInfo)
       {
        FPR( stderr, "RdTable Error allocating variable info.\n" );
        error = 1;
        goto shutdown;
       }

/***/
/* Interpret the VARS and FORMAT lines, part II:
/*   For each variable:
/*     Extract and allocate the variable name.
/*     Extract and allocate the variable format.
/*     Set the variable type.
/*     Set the variable tolerance.
/*     Initialize the string equivalent pointer.
/***/

    varPtr = varInfo;
 
    for( var = 0; var < varCount; var++ )
       {
        cPtr         = getToken( var + 2, names );
        varPtr->name = strDup( cPtr );

        if (!varPtr->name)
           {
            FPR( stderr, "RdTable Error allocating var name.\n" );
            FPR( stderr, "Name: %s\n", cPtr );
            error = 1;
            goto shutdown;
           }
 
        if (fmtCount > 0)
           cPtr = getToken( var + 2, formats );
        else
           cPtr = "%f";

        varPtr->fmt = strDup( cPtr );

        if (!varPtr->fmt)
           {
            FPR( stderr, "RdTable Error allocating var format.\n" );
            FPR( stderr, "Format: %s\n", cPtr );
            error = 1;
            goto shutdown;
           }

        (void) us2space( varPtr->fmt );

        cPtr += strlen( cPtr ) - 1;

        if (*cPtr == 's')
           varPtr->type = VAR_TEXT; 
        else if (*cPtr == 'd')
           varPtr->type = VAR_INT; 
        else if (*cPtr == 'f' || *cPtr == 'g' || *cPtr == 'e')
           varPtr->type = VAR_FLT; 
        else
           {
            FPR( stderr, "RdTable Error: bad format %s\n", formats );
            error = 1;
            goto shutdown;
           }


        if (tolCount > 0)
           varPtr->tol = getTokenR( var + 2, tols );
        else
           varPtr->tol = 0.0;

        varPtr->equivCount = 0;
        varPtr->equivList  = (char **) NULL;

        varPtr++;
       }

/***/
/* Allocate memory for the data matrix.
/* Rewind the text file, read the data:
/***/

    if (readMode == READ_DATA_YES)
       {
        if (!(matrix = fltAlloc( "table", varCount*entryCount )))
           {
            FPR( stderr, "RdTable error allocating data matrix.\n" );
            error = 1;
            goto shutdown;
           }
       }

    (void) rewind( inUnit );

    rPtr = matrix;

    while( fgetsAlloc( &line, &maxLen, inUnit, inBuff, inLen, cBuff, cBuffLen ))
       {
        dataFlag = isDataLine( line );

        if (dataFlag && (readMode == READ_DATA_YES))
           {
            varPtr = varInfo;

            for( var = 0; var < varCount; var++ )
               {
                switch( varPtr->type )
                  {
                   case VAR_INT:
                      *rPtr = getTokenI( var + 1, line );
                 
                      if (badToken) 
                         {
                          FPR( stderr, "RdTable Warning: Bad I-Item" );
                          FPR( stderr, " %d:\n%s\n", var + 1, line );
                         }

                      if (useNullValue)
                         {
                          if (*rPtr == (float) nullValue) *rPtr = defNullValue;
                         }

                      break;

                   case VAR_FLT:
                      *rPtr = getTokenR( var + 1, line );
                 
                      if (badToken) 
                         {
                          FPR( stderr, "RdTable Warning: Bad F-Item" );
                          FPR( stderr, " %d:\n%s\n", var + 1, line );
                         }

                      if (useNullValue)
                         {
                          if (*rPtr == (float) nullValue) *rPtr = defNullValue;
                         }

                      break;

                   case VAR_TEXT:
                      (void) strcpy( ctemp, getToken( var + 1, line ));

                      if (badToken) 
                         {
                          FPR( stderr, "RdTable Warning: Bad T-Item" );
                          FPR( stderr, " %d:\n%s\n", var + 1, line );
                         }
             
                      if (useNullString && !strcmp( ctemp, nullStr ))
                         {
                          *rPtr = nullValue;
                         }
                      else
                         {
                          *rPtr = mapText( ctemp, 
                                           &(varPtr->equivList),
                                           &(varPtr->equivCount),
                                           &error );

                          if (error)
                             {
                              FPR( stderr, "RdTable Error mapping text.\n" );
                              goto shutdown;
                             }
                         }
 
                      break;

                   default:
                      FPR( stderr, "RdTable Error: bad vartype.\n" );
                      error = 1;
                      goto shutdown;
                      break;

                  } 

                rPtr++;
                varPtr++;
               }
           }
        else if (!dataFlag && hdrBasePtr)
           {
            (void) strcpy( ctemp, getToken( 1, line ));

            if (*ctemp == '#' ||
                 !strcasecmp( ctemp, "DATA" ) || 
                   !strcasecmp( ctemp, "REMARK" ))
               {
                if (i = strlen( line ))
                   {
                    if (line[i-1] == '\n') line[i-1] = '\0';
                    (void) addHdrLine( line, *hdrBasePtr );
                   }
               }
           }
       }

/***/
/* Exit point:
/*   Close text file.
/*   Deallocate memory.
/*   Set up return values.
/***/

shutdown:

    if (inUnit) (void) fclose( inUnit );

    (void) strAllocFree( &line, cBuff, cBuffLen, &maxLen );

    TXTFREE( formats );
    TXTFREE( names );
    TXTFREE( tols );

    (void) freeToken();
 
    *matrixPtr     = matrix;
    *varInfoPtr    = varInfo;
    *varCountPtr   = varCount;
    *entryCountPtr = entryCount;
    
    return( error );
} 

/***/
/* freeTab: free memory as allocated by rdTable.
/*
/*          Use NULL matrix and 0 entryCount to free varInfo only.
/*          Use NULL varInfo and 0 varCount to free matrix only.
/***/

void freeTab( matrix, varInfo, varCount, entryCount )

   int    varCount, entryCount;
   struct tabVar *varInfo;
   float  *matrix;
{
    struct tabVar *varPtr;
    char **listPtr;
    int  var;

/***/
/* Free the data matrix itself:
/* For each variable, free the format and name text.
/* If the variable has an text equiv list, free its contents then itself.
/***/

    if (matrix)
       {
        (void) deAlloc( "table", matrix, sizeof(float)*varCount*entryCount );
       }

    if (varInfo)
       {
        varPtr = varInfo;

        for( var = 0; var < varCount; var++ )
           {
            if (varPtr) 
               {
                TXTFREE( varPtr->fmt );
                TXTFREE( varPtr->name );
    
                if (varPtr->equivList)
                   {
                    listPtr = varPtr->equivList;
    
                    while( *listPtr )
                       {
                        TXTFREE2( "mapText", *listPtr ); 
                        listPtr++;
                       }
    
                    (void) deAlloc( "mapText",
                                    varPtr->equivList,
                                    sizeof(char *)*(varPtr->equivCount + 1) );
                   }
               } 

            varPtr++;
           }
       }

/***/
/* Free the list of variable info structures:
/***/

    (void) deAlloc( "table", varInfo, sizeof(struct tabVar)*varCount );
}

/***/
/* freeHdr: free the header line linked-list.
/***/

void freeHdr( hdrBase )

   struct hdrList *hdrBase;
{
   struct hdrList *hPtr;

   while( hdrBase )
      {
       hPtr = hdrBase->next;

       TXTFREE( hdrBase->text );

       (void) deAlloc( "table", hdrBase, sizeof(struct hdrList) );

       hdrBase = hPtr;
      } 
}

/***/
/* getTabVar: return index number of existing table variable name;
/*            returns -1 if variable isn't found; not case sensitive.
/***/

int getTabVar( varInfo, varCount, varName )

   struct tabVar *varInfo;
   int    varCount;
   char   *varName;
{
    char ctemp1[NAMELEN+1], ctemp2[NAMELEN+1];

    (void) strcpy( ctemp1, varName );
    (void) raiseStr( ctemp1 );

    while( varCount-- )
       {
        (void) strcpy( ctemp2, (varInfo + varCount)->name );
        (void) raiseStr( ctemp2 );

        if (!strcmp( ctemp1, ctemp2 )) return( varCount );
       }

    return( varCount );
}

/***/
/* getTabVal: extracts the value of variable in a table entry as text.
/***/

int getTabVal( entry, varInfo, varLoc, text )

   struct tabVar *varInfo;
   int    varLoc;
   float  *entry;
   char   *text;
{
    int  id, error;

    error    = 0;
    varInfo += varLoc;

    switch( varInfo->type )
       {
        case VAR_INT:
           (void)sprintf( text, varInfo->fmt, (int)entry[varLoc]  );
           break;

        case VAR_FLT:
           (void)sprintf( text, varInfo->fmt, entry[varLoc]  );
           break;

        case VAR_TEXT:
           id = (int)entry[varLoc];

           if (id < 0 || id > varInfo->equivCount - 1)
              (void)sprintf( text, varInfo->fmt, "???"  );
           else
              (void)sprintf( text, varInfo->fmt, varInfo->equivList[id]  );
           break;

        default:
           (void) strcpy( text, "???" );
           break;
       } 

    return( 0 );
}

/***/
/* setTabVal: sets a variable in a table entry from a text specification.
/***/

int setTabVal( entry, varInfo, varLoc, text )

   struct tabVar *varInfo;
   int    varLoc;
   float  *entry;
   char   *text; 
{
    int error;

    error    = 0;
    varInfo += varLoc;

    switch( varInfo->type )
       {
        case VAR_INT:
           entry[varLoc] = getTokenI( 1, text );
           if (badToken) error = 1;
           break;

        case VAR_FLT:
           entry[varLoc] = getTokenR( 1, text );
           if (badToken) error = 1;
           break;

        default:
           entry[varLoc] = mapText( text,
                                    &varInfo->equivList,
                                    &varInfo->equivCount,
                                    &error );

           break;
       }

    return( error );
}

/***/
/* getTabData: return pointer to the value of first matching DATA statement.
/***/

char *getTabData( hdrBase, varName )

   struct hdrList *hdrBase;
   char   *varName;
{
    struct hdrList *hPtr;
    char   *sPtr;

    if (!hdrBase) return( (char *)NULL );

    for( hPtr = hdrBase; hPtr->next; hPtr = hPtr->next )
       {
        sPtr = getToken( 1, hPtr->text );

        if (!strcasecmp( sPtr, "DATA" ))
           {
            sPtr = getToken( 2, hPtr->text );

            if (!strcasecmp( sPtr, varName ))
               {
                sPtr = rightToken( 3, hPtr->text );
                return( sPtr );
               }
           }
       }

    return( (char *)NULL );
}

/***/
/* getTabDataR: return real value of first matching DATA statement.
/***/

float getTabDataR( hdrBase, varName )

   struct hdrList *hdrBase;
   char   *varName;
{
    struct hdrList *hPtr;
    char   *sPtr;
    float  v;

    v = 0.0;

    if (sPtr = getTabData( hdrBase, varName ))
       {
        (void) sscanf( sPtr, "%f", &v );
       }

    return( v );
}    

/***/
/* getTabDataI: return int value of first matching DATA statement.
/***/

int getTabDataI( hdrBase, varName )

   struct hdrList *hdrBase;
   char   *varName;
{
    struct hdrList *hPtr;
    char   *sPtr;
    int    n;

    n = 0;

    if (sPtr = getTabData( hdrBase, varName ))
       {
        (void) sscanf( sPtr, "%d", &n );    
       }

    return( n );
}   

/***/
/* us2space: return a version of str with underscores converted to spaces.
/***/

static char *us2space( str )

   char *str;
{
    char *s;

    if (!str) return( str );

    for( s = str; *s; s++ )
       {
        if (*s == '_') *s = ' ';
       }

    return( str );
}

/***/
/* Bottom.
/***/
