/***/
/* NMRPipe Copyright 1992 - 2024 Frank Delaglio
/* Utilities for writing NMRPipe-format tables.
/***/

/***/
/* wrTable: write a text-table of character and numerical data,
/*          stored as a numerical matrix.  Related data
/*          format and structures are described in files
/*          "table.h" and "rdtable.c".
/***/ 

#define STRCPY_ALLOC(DEST,SRC) strcpyAlloc(&DEST,SRC,cBuff,NAMELEN+1,&cBuffLen)
#define STRCAT_ALLOC(DEST,SRC) strcatAlloc(&DEST,SRC,cBuff,NAMELEN+1,&cBuffLen)

#define FPR        (void)fprintf
#define _NULLTXT   "*"

#include <stdio.h>
#include <string.h>

#include "stralloc.h"
#include "memory.h"
#include "table.h"
#include "prec.h"

static char *space2us();

int wrTable( outName,      /* Name of data table to write.         */
             matrix,       /* Data table matrix.                   */
             varCount,     /* Variables per entry in table.        */
             entryCount,   /* Numberof entries in table.           */
             varInfo,      /* Descibes each variable in the table. */
             nullValue,    /* Value of "missing" variables.        */
             hdrBase )     /* Begining of optional header list.    */

   struct tabVar *varInfo; 
   struct hdrList *hdrBase; 
   int    varCount, entryCount, nullValue;
   float  *matrix;
   char   *outName;
{
    int error;

    error = wrTable2( outName,
                      matrix, 
                      varCount,
                      entryCount,
                      varInfo,
                      nullValue,
                      hdrBase,
                      NULL, NULL );

    return( error );
}

int wrTable2( outName,      /* Name of data table to write.               */
              matrix,       /* Data table matrix.                         */
              varCount,     /* Variables per entry in table.              */
              entryCount,   /* Numberof entries in table.                 */
              varInfo,      /* Descibes each variable in the table.       */
              nullValue,    /* Value of "missing" variables.              */
              hdrBase,      /* Begining of optional header list.          */
              delFlag,      /* If true, delete certain entries on output. */
              delVarLoc )   /* Variable indicating entry to delete.       */

   int    varCount, entryCount, nullValue, delFlag, delVarLoc;
   struct tabVar *varInfo;
   struct hdrList *hdrBase;
   char   *outName;
   float  *matrix;
{
   char   *ctemp, cBuff[NAMELEN+1], fmt[NAMELEN+1], number[NAMELEN+1];
   int    cBuffLen, wrFlag, zeroCount, id, entry, var, error;
   struct tabVar *varPtr;
   float  *rPtr;

   FILE   *outUnit;

/***/
/*-----------------------------------------------------------------------
/***/

/***/
/* Initialize:
/***/

   error    = 0;
   outUnit  = (FILE *) NULL;
   cBuffLen = NAMELEN + 1;
   ctemp    = cBuff;
   cBuff[0] = '\0';

   (void) strAllocInit( &ctemp, cBuff, NAMELEN+1, &cBuffLen );

/***/
/* Open the text table file:
/***/

   if (!(strcmp( outName, "-" )))
      {
       outUnit = stdout;
      }
   else if (!(outUnit = fopen( outName, "w")))
      {
       FPR( stderr, "File open error: %s\n", outName );
       error = 1;
       goto shutdown;
      }

/***/
/* Write optional header if any:
/***/

    if (hdrBase)
       {
        while( hdrBase->next )
           {
            FPR( outUnit, "%s\n", hdrBase->text );
            hdrBase = hdrBase->next;
           }
       }

/***/
/* Build and write the VARS header line.
/* Build and write the FORMAT header line.
/* Build and write the TOLERANCE header line if needed.
/* Write the NULLVALUE and NULLSTRING header lines.
/***/

    (void) STRCPY_ALLOC( ctemp, "VARS   " );

    varPtr = varInfo; 

    for( var = 0; var < varCount; var++ )
       {
        (void) STRCAT_ALLOC( ctemp, varPtr->name ); 
        if (var != varCount - 1) (void) STRCAT_ALLOC( ctemp, " " );
        varPtr++;
       }

    FPR( outUnit, "%s\n", ctemp );

    (void) STRCPY_ALLOC( ctemp, "FORMAT " );

    varPtr = varInfo;

    for( var = 0; var < varCount; var++ )
       {
        (void) strcpy( fmt, varPtr->fmt );
        (void) STRCAT_ALLOC( ctemp, space2us( fmt )); 
        if (var != varCount - 1) (void) STRCAT_ALLOC( ctemp, " " );
        varPtr++;
       }

    FPR( outUnit, "%s\n", ctemp );

    (void) STRCPY_ALLOC( ctemp, "TOLERANCE " );

    varPtr    = varInfo;
    zeroCount = 0;

    for( var = 0; var < varCount; var++ )
       {
        (void) sprintf( number, "%5.2f", varPtr->tol );
        (void) STRCAT_ALLOC( ctemp, number ); 

        if (var != varCount - 1) (void) STRCAT_ALLOC( ctemp, " " );

        if (varPtr->tol == 0.0) zeroCount++;

        varPtr++;
       }

    if (zeroCount != varCount) FPR( outUnit, "%s\n", ctemp );

    FPR( outUnit, "\n" );
    FPR( outUnit, "NULLVALUE %d\n", nullValue );
    FPR( outUnit, "NULLSTRING %s\n", _NULLTXT );
    FPR( outUnit, "\n" );

/***/
/* For every entry:
/*  Print each variable according to type and format.
/***/

    rPtr = matrix;

    for( entry = 0; entry < entryCount; entry++ )
       {
        varPtr = varInfo;
        *ctemp = '\0';

        if (delFlag && rPtr[delVarLoc] < 0.0)
           wrFlag = 0;
        else
           wrFlag = 1;

        for( var = 0; var < varCount; var++ )
           {
            switch( wrFlag ? varPtr->type : VAR_NULL )
               {
                case VAR_NULL:
                   break;

                case VAR_FLT: 
                   (void) sprintf( number, varPtr->fmt, *rPtr );
                   (void) STRCAT_ALLOC( ctemp, number );
                   break;

                case VAR_INT:
                   (void) sprintf( number, varPtr->fmt, (int) *rPtr );
                   (void) STRCAT_ALLOC( ctemp, number );
                   break;

                case VAR_TEXT:
                   if (*rPtr == (float) nullValue)
                      {
                       (void) sprintf( number, varPtr->fmt, "_NULLTXT" );
                       (void) STRCAT_ALLOC( ctemp, number );
                      }
                   else
                      {
                       id = (int) *rPtr;

                       if (id < 0 || id > varPtr->equivCount - 1)
                          {
                           FPR( stderr, "WrTable Error: bad text.\n" );
                           FPR( stderr, "Entry %d, ID %d\n", entry, id );
                           error = 1;
                          }
                       else
                          {
                           (void) sprintf( number, 
                                           varPtr->fmt,
                                           varPtr->equivList[id] );

                           (void) STRCAT_ALLOC( ctemp, number );
                          }
                      }

                   break;

                default:
                   FPR( stderr, "WrTable Error: bad type.\n" );
                   FPR( stderr, "Entry %d, %d\n", entry, varPtr->type );
                   error = 1;
                   goto shutdown;
                   break;
               }

            if (wrFlag && var != varCount - 1)
               {
                (void) STRCAT_ALLOC( ctemp, " " );
               }

            varPtr++;
            rPtr++;
           }

        if (wrFlag) FPR( outUnit, "%s\n", ctemp );
       }

/***/
/* Exit point:
/***/

shutdown:

    if (outUnit) (void) fclose( outUnit );

    (void) strAllocFree( &ctemp, cBuff, NAMELEN+1, &cBuffLen );

    return( error );
}

/***/
/* wrTableN: write a text-table of numerical data, with 
/*           a series of related variables.
/*
/*           Works by building "varInfo" structure and
/*           using general wrTable procedure.
/***/ 

int wrTableN( outName,      /* Name of data table to write.            */
              matrix,       /* Data table matrix.                      */
              varCount,     /* Variables per entry in table.           */
              entryCount,   /* Number of entries in table.             */
              varRoot,      /* Common root name of variables in table. */
              varFmt,       /* Common format of variables in table.    */
              tol,          /* Common tolerance of variables in table. */
              nullValue,    /* Value of "missing" variables.           */
              hdrBase )     /* Optional header list.                   */

   struct hdrList *hdrBase;
   int    varCount, entryCount, nullValue;
   char   *outName, *varRoot, *varFmt;
   float  *matrix, tol;

{
    struct tabVar *varInfo, *varPtr;
    char   ctemp[NAMELEN+1];
    int    var, error;

/***/
/* Initialize:
/***/

    error   = 0;
    varInfo = (struct tabVar *) NULL;

/***/
/* Allocate and fill the var info array:
/***/

    varInfo = (struct tabVar *) 
       voidAlloc( "table", sizeof(struct tabVar)*varCount );

    if (!varInfo)
       {
        FPR( stderr, "WrTableN Error allocating variable info.\n" );
        error = 1;
        goto shutdown;
       }

    varPtr = varInfo;

    for( var = 0; var < varCount; var++ )
       {
        (void) sprintf( ctemp, "%s%d", varRoot, var + 1 );

        if (!(varPtr->name = strDup( ctemp )))
           {
            FPR( stderr, "WrTableN Error allocating variable name.\n" );
            error = 1;
            goto shutdown;
           } 
  
        if (!(varPtr->fmt = strDup( varFmt )))
           {
            FPR( stderr, "WrTableN Error allocating variable format.\n" );
            error = 1;
            goto shutdown;
           }

        varPtr->type       = VAR_FLT;
        varPtr->equivList  = (char **) NULL;
        varPtr->equivCount = 0;
        varPtr->tol        = tol; 

        varPtr++;
       }

/***/
/* Write the table:
/***/

    error = wrTable( outName, 
                     matrix,
                     varCount,
                     entryCount,
                     varInfo,
                     nullValue,
                     hdrBase );

/***/
/* Exit point:
/***/

shutdown:

   (void) freeTab( (float *)NULL, varInfo, varCount, 0 );

   return( error );
}

/***/
/* addMatVar: add variables to an existing table matrix.
/***/

int addMatVar( matrix, entryCount, varCount, addCount )

   int   entryCount, varCount, addCount;
   float **matrix;
{
    int   i, j, newCount;
    float *src, *dest;

    if (addCount < 1) return( 0 );

    newCount = varCount + addCount;

    *matrix = fltReAlloc( "table",
                          *matrix,
                          entryCount*newCount,
                          entryCount*varCount );

    if (!*matrix)
       {
        return( 1 );
       }
   
    src  = *matrix + entryCount*varCount - 1;
    dest = *matrix + entryCount*newCount - 1;
 
    for( i = 0; i < entryCount; i++ )
       {
        for( j = 0; j < addCount; j++ ) *dest-- = 0.0;
        for( j = 0; j < varCount; j++ ) *dest-- = *src--;
       }
 
    return( 0 );
}

/***/
/* addTabVar: add a numerical table variable to existing tabVar structure.
/***/

int addTabVar( varInfo, varCount, name, fmt, tol, type )

   struct tabVar **varInfo;
   int    type,  *varCount;
   char   *name, *fmt;
   float  tol;
{
    struct tabVar *varPtr;
    char   *nPtr, *fPtr;

/***/
/* Reallocate existing varInfo list.
/* Adjust last variable to reflect desription in arguments.
/***/

    varPtr = (struct tabVar *) 
       voidReAlloc( "table", 
                    (void *)*varInfo,
                    sizeof(struct tabVar)*(1 + *varCount),
                    sizeof(struct tabVar)*(*varCount) );

    if (!varPtr)
       {
        FPR( stderr, "Error reallocating table variables.\n" );
        return( 1 );
       }

    if (!(nPtr = strDup( name )))
       {
        FPR( stderr, "AddTabVar Error reallocating variable name.\n" );
        return( 1 );
       }

    if (!(fPtr = strDup( fmt )))
       {
        FPR( stderr, "AddTabVar Error reallocating variable format.\n" );
        return( 1 );
       }

    (varPtr + *varCount)->equivList  = (char **) NULL;
    (varPtr + *varCount)->equivCount = 0;

    (varPtr + *varCount)->name = nPtr;
    (varPtr + *varCount)->fmt  = fPtr;
    (varPtr + *varCount)->tol  = tol;
    (varPtr + *varCount)->type = type;

    *varInfo = varPtr;
    (*varCount)++;

    return( 0 );
}

/***/
/* chTabVar: change characteristics of a numerical table variable.
/***/

int chTabVar( varPtr, name, fmt, tol, type )

   struct tabVar *varPtr;
   char   *name, *fmt;
   int    type;
   float  tol;
{

/***/
/* Free the existing name and format.
/* Allocate and assign the new name and format.
/* Set the tolerance and type.
/***/

    (void) deAlloc( "strDup", varPtr->name, strlen(varPtr->name) + 1 );
    (void) deAlloc( "strDup", varPtr->fmt,  strlen(varPtr->name) + 1 );

    if (!(varPtr->name = strDup( name )))
       {
        FPR( stderr, "ChTabVar Error allocating NAME.\n" );
        return( 1 );
       }

    if (!(varPtr->fmt = strDup( fmt )))
       {
        FPR( stderr, "ChTabVar Error allocating FMT.\n" );
        return( 1 );
       }

    varPtr->type = type;
    varPtr->tol  = tol;

    return( 0 );
}
 
/***/
/* addHdrLine: add a header line to the optional header linked-list.
/***/

int addHdrLine( line, hPtr )

   struct hdrList *hPtr;
   char   *line;
{

/***/
/* Find the end of the list.
/* Allocate space for the line to add.
/* Copy the line.
/* Allocate space for the next structure.
/* Terminate the next structure.
/***/

   while( hPtr->next ) hPtr = hPtr->next;

   if (!(hPtr->text = strDup( line )))
      {
       FPR( stderr, "AddHdrLine Error allocating header line.\n" );
       return( 1 );
      }

   if (!(hPtr->next = hdrInit()))
      {
       FPR( stderr, "AddHdrLine Error allocating header item.\n" );
       return( 1 );
      }

   return( 0 );
}

/***/
/* hdrInit: allocate and initialize another link in the header-line 
/*          linked-list.
/***/

struct hdrList *hdrInit()
{
   struct hdrList *hPtr;

   hPtr = (struct hdrList *) voidAlloc( "table", sizeof(struct hdrList ));

   if (hPtr)
      {
       hPtr->text = (char *) NULL;
       hPtr->next = (struct hdrList *) NULL;
      }

    return( hPtr );
}

/***/
/* space2us: return a version of str with spaces converted to underscores.
/***/

static char *space2us( str )

   char *str;
{
    char *s;

    if (!str) return( str );

    for( s = str; *s; s++ )
       {
        if (*s == ' ') *s = '_';
       }

    return( str );
}

int fmtTabVar( varPtr, rPtr, number )

   float *rPtr;
   char  *number;
   struct tabVar *varPtr;
{
    int id;

    (void) strcpy( number, "(Null)" );

    switch( varPtr->type )
       {
        case VAR_FLT: 
           (void) sprintf( number, varPtr->fmt, *rPtr );
           break;
        case VAR_INT:
           (void) sprintf( number, varPtr->fmt, (int) *rPtr );
           break;
        case VAR_TEXT:
           id = (int) *rPtr;

           if (id >= 0 && id < varPtr->equivCount)
              {
               (void) sprintf( number, varPtr->fmt, varPtr->equivList[id] );
              }

            break;

         default:
            break;
        }

   return( 0 );
}

/***/
/* Bottom.
/***/
